unit BasicBrowse;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DBGrids, ExtCtrls, ToolWin, ComCtrls, ImgList, Menus, ActnList,
  Db, StdCtrls, ovTypes, ovIObjects,
  ovComponents, ovVirtualDataSet, ovDataSet;

type
  TBasicBrowseForm = class(TForm)
    ActionImages: TImageList;
    ActionList: TActionList;
    BrowseGrid: TDBGrid;
    BrowseGridPanel: TPanel;
    DeleteAction: TAction;
    DeleteButton: TToolButton;
    DeleteItem: TMenuItem;
    EditAction: TAction;
    EditButton: TToolButton;
    EditItem: TMenuItem;
    GridMenu: TPopupMenu;
    NewAction: TAction;
    NewButton: TToolButton;
    NewItem: TMenuItem;
    SearchEdit: TEdit;
    SelectAction: TAction;
    SelectButton: TToolButton;
    SelectItem: TMenuItem;
    StatusBar: TStatusBar;
    ToolBar: TToolBar;
    ToolSep1: TToolButton;
    ObjectExposer: TovExposer;
    ObjectSource: TDataSource;
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure NewActionExecute(Sender: TObject);
    procedure EditActionExecute(Sender: TObject);
    procedure DeleteActionExecute(Sender: TObject);
    procedure BrowseGridDblClick(Sender: TObject);
    procedure SelectActionExecute(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
  private
    FLookupMode: Boolean;
    function GetSelected: IovObject;
    procedure SetLookupMode(Value: Boolean);
    procedure SetSelected(Value: IovObject);
  protected
    function ConfirmDelete: Boolean; virtual;
    function Find(const Text: string): Boolean; virtual;
    procedure Search;
    procedure Select; virtual;
    procedure UpdateControls; virtual;
  public
    function Execute: Boolean;
    property LookupMode: Boolean read FLookupMode write SetLookupMode;
    property Selected: IovObject read GetSelected write SetSelected;
  end;

implementation

uses BasicEdit, Utility, Model;

{$R *.DFM}

{ TBasicBrowseForm }

procedure TBasicBrowseForm.ActionListUpdate(Action: TBasicAction;
  var Handled: Boolean);
var
  HasItem: Boolean;
begin
  HasItem := Not ObjectExposer.IsEmpty;
  NewAction.Enabled := not LookupMode;
  EditAction.Enabled := HasItem;
  DeleteAction.Enabled := HasItem;
  SelectAction.Enabled := HasItem;
end;

procedure TBasicBrowseForm.BrowseGridDblClick(Sender: TObject);
begin
  with SelectAction do
    if Visible then Execute;
end;

function TBasicBrowseForm.ConfirmDelete: Boolean;
begin
  Result := Confirm(Format(' %s?', [ObjectExposer['']]));
end;

procedure TBasicBrowseForm.DeleteActionExecute(Sender: TObject);
begin
  if ConfirmDelete then
    ObjectExposer.DeleteCurrentObject;
end;

procedure TBasicBrowseForm.EditActionExecute(Sender: TObject);
begin
  if dgEditing in BrowseGrid.Options then
    ObjectExposer.Edit
  else
    EditObject(ObjectExposer.CurrentObject);
end;

function TBasicBrowseForm.Execute: Boolean;
begin
  if ObjectExposer.Active then ObjectExposer.First;
  Result := ShowModal = mrOk;
end;

function TBasicBrowseForm.Find(const Text: string): Boolean;
begin
  Result := False;
end;

procedure TBasicBrowseForm.FormCreate(Sender: TObject);
begin
  UpdateControls;
end;

procedure TBasicBrowseForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  case Ord(Key) of
    VK_ESCAPE:
      if LookupMode then
        Close
      else
        if ObjectExposer.State in [dsEdit, dsInsert]
          then ObjectExposer.Cancel
          else Close;

    VK_RETURN:
      if LookupMode then
      begin
        if SearchEdit.Focused then
          Search
        else
          Select;
      end else
        ObjectExposer.CheckBrowseMode
  else
    Exit;
  end;
  Key := #0;
end;

function TBasicBrowseForm.GetSelected: IovObject;
begin
  Result := ObjectExposer.CurrentObject;
end;

procedure TBasicBrowseForm.NewActionExecute(Sender: TObject);
var
  NewObject: TOID;
begin
  if dgEditing in BrowseGrid.Options then
    ObjectExposer.Insert
  else begin          (*
    NewObject := ObjectExposer.CreateObject;
    if Not EditObject(ObjectExposer.Objects[NewObject]) {and Exposer.Active} then
      ObjectExposer.Objects[NewObject].Delete; *)
  end;
end;

procedure TBasicBrowseForm.Search;
begin
  BeginBusy;
  try
    if Find(SearchEdit.Text) then
      ActiveControl := BrowseGrid
    else
      ActiveControl := SearchEdit;
  finally
    EndBusy;
  end;
end;

procedure TBasicBrowseForm.Select;
begin
  if Assigned(Selected) then
    ModalResult := mrOk;
end;

procedure TBasicBrowseForm.SelectActionExecute(Sender: TObject);
begin
  Select;
end;

procedure TBasicBrowseForm.SetLookupMode(Value: Boolean);
begin
  if Value <> LookupMode then
  begin
    FLookupMode := Value;
    UpdateControls;
  end;
end;

procedure TBasicBrowseForm.SetSelected(Value: IovObject);
begin
  if (Value <> nil) and Not ObjectExposer.FindObject(Value.ObjectId) then
    raise Exception.Create('  ')
end;

procedure TBasicBrowseForm.UpdateControls;
begin
  SearchEdit.Visible := LookupMode;
  ToolSep1.Visible := LookupMode;
  SelectAction.Visible := LookupMode;
  with BrowseGrid do
    if LookupMode then
      Options := Options + [dgRowSelect]
    else
      Options := Options - [dgRowSelect];
  ToolBar.Repaint;
end;


end.
