unit BasicEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Db, ComCtrls, ovIObjects,
  ovComponents;

type
  TBasicEditFormClass = class of TBasicEditForm;

  TBasicEditForm = class(TForm)
    ButtonPanel: TPanel;
    CancelButton: TButton;
    ClientPanel: TPanel;
    OkButton: TButton;
    PageControl: TPageControl;
    DetailsSheet: TTabSheet;
    procedure OkButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FSubject: IovObject;
  protected
    function GetIsValid: Boolean; virtual;
    procedure UpdateActions; override;
    procedure SetSubject(const Value: IovObject); virtual;
  public
    class function ObjectClass: TClass; virtual; abstract;
    class procedure RegisterClass(ObjectClassName: String);
    property IsValid: Boolean read GetIsValid;
    property Subject: IovObject read FSubject write SetSubject;
  end;

function EditObject(AObject: IovObject): Boolean;
function FindEditFormClass(ObjectClassName: String): TBasicEditFormClass;
function GetEditFormClass(ObjectClassName: String): TBasicEditFormClass;
procedure RegisterEditFormClass(ObjectClassName: String; EditFormClass: 
  TBasicEditFormClass);

implementation

{$R *.DFM}

var
  EditFormClasses: TStringList;

function EditObject(AObject: IovObject): Boolean;
var
  EditFormClass: TBasicEditFormClass;
begin
  EditFormClass := GetEditFormClass(AObject.ClassName);
  with EditFormClass.Create(nil) do
  try
    Subject := AObject;
    Result := ShowModal = mrOk;
  finally
    Free;
  end;
end;

function FindEditFormClass(ObjectClassName: String): TBasicEditFormClass;
var
  I: Integer;
begin
  I := EditFormClasses.IndexOf(UpperCase(ObjectClassName));
  if I >= 0 then
    Result := TBasicEditFormClass(EditFormClasses.Objects[I])
  else
    Result := nil;
end;

function GetEditFormClass(ObjectClassName: String): TBasicEditFormClass;
begin
  Result := FindEditFormClass(ObjectClassName);
  if not Assigned(Result) then
    raise Exception.Create('No editor registered for: ' + ObjectClassName);
end;

procedure RegisterEditFormClass(ObjectClassName: String; EditFormClass:
  TBasicEditFormClass);
begin
  EditFormClasses.AddObject(UpperCase(ObjectClassName), TObject(EditFormClass));
end;

{ TBasicEditForm }

function TBasicEditForm.GetIsValid: Boolean;
begin
  Result := True;
end;

procedure TBasicEditForm.OkButtonClick(Sender: TObject);
begin
  try
//    ovFormController.Objects[ovFormController.Items[0].Name].Post;
  except
    ModalResult := mrNone;
    raise;
  end;
end;

procedure TBasicEditForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
//  if ModalResult <> mrOk then ovFormController.Objects[ovFormController.Items[0].Name].Cancel;
end;

class procedure TBasicEditForm.RegisterClass(ObjectClassName: String);
begin
  RegisterEditFormClass(ObjectClassName, Self);
end;

procedure TBasicEditForm.SetSubject(const Value: IovObject);
begin
  FSubject := Value
end;

procedure TBasicEditForm.UpdateActions;
begin
  inherited;
  OkButton.Enabled := IsValid;
end;

initialization
  EditFormClasses := TStringList.Create;
finalization
  EditFormClasses.Free;
end.
