unit ContactView;

interface

{$IFDEF VER150}
{$WARN UNSAFE_TYPE OFF}
{$WARN UNSAFE_CAST OFF}
{$WARN UNSAFE_CODE OFF}
{$ENDIF}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Grids, DBGrids, ExtCtrls, Db, Buttons,
  StdCtrls, ActnList, ToolWin, ImgList, Menus, BasicView,
  ovComponents, ovIObjects, ovVirtualDataSet, ovDataSet, ovXMLImportExport,
  ovObjectInspector, ovXMLSchema;

type
  TContactViewForm = class(TBasicViewForm)          
    ActionList: TActionList;
    ActionImages: TImageList;
    ClientPanel: TPanel;
    ContactGridMenu: TPopupMenu;
    ContactGridPanel: TPanel;
    ContactSource: TDataSource;
    BrowseCountryAction: TAction;
    CountriesButton: TToolButton;
    DeleteAction: TAction;
    DeleteButton: TToolButton;
    DeleteItem: TMenuItem;
    EditAction: TAction;
    EditButton: TToolButton;
    EditItem: TMenuItem;
    ExplorerAction: TAction;
    ExportAction: TAction;
    ExportItem: TMenuItem;
    FilterAction: TAction;
    FilterButton: TToolButton;
    FindAction: TAction;
    FindButton: TBitBtn;
    FindEdit: TComboBox;
    FindLabel: TLabel;
    FindPanel: TPanel;
    GridPanel: TPanel;
    HeaderBevel: TBevel;
    HeaderPanel: TPanel;
    IndexTabControl: TTabControl;
    MenuSep1: TMenuItem;
    MenuSep2: TMenuItem;
    NewCompanyAction: TAction;
    NewCompanyButton: TToolButton;
    NewCompanyItem: TMenuItem;
    NewPersonAction: TAction;
    NewPersonButton: TToolButton;
    NewPersonItem: TMenuItem;
    SortAction: TAction;
    ToolBar: TToolBar;
    ToolSep1: TToolButton;
    ToolSep4: TToolButton;
    ToolSep2: TToolButton;
    ExplorerSplitter: TSplitter;
    ExplorerPanel: TPanel;
    ExplorerImages: TImageList;
    CategoriesButton: TToolButton;
    ToolButton2: TToolButton;
    BrowseCategoryAction: TAction;
    ContactExposer: TovExposer;
    ContactGrid: TDBGrid;
    ContactExposerDSDesigner: TStringField;
    ContactExposerDSDesigner2: TStringField;
    ContactExposerDSDesigner3: TStringField;
    ContactExposerDSDesigner4: TStringField;
    PhoneField: TStringField;
    ContactClassId: TIntegerField;
    ToolButton1: TToolButton;
    XMLExport: TovXMLExport;
    XMLImport: TovXMLImport;
    ToolButton3: TToolButton;
    ObjectInspector: TovObjectInspector;
    XMLSchemaGenerator: TovXMLSchemaGenerator;
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure BrowseCountryActionExecute(Sender: TObject);
    procedure DeleteActionExecute(Sender: TObject);
    procedure EditActionExecute(Sender: TObject);
    procedure ExplorerActionExecute(Sender: TObject);
    procedure ExportActionExecute(Sender: TObject);
    procedure FilterActionExecute(Sender: TObject);
    procedure FindActionExecute(Sender: TObject);
    procedure FindEditClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure IndexTabControlChange(Sender: TObject);
    procedure NewCompanyActionExecute(Sender: TObject);
    procedure NewPersonActionExecute(Sender: TObject);
    procedure BrowseCategoryActionExecute(Sender: TObject);
    procedure ContactSelectorBeforeClose(DataSet: TDataSet);
    procedure FormShow(Sender: TObject);
    procedure ContactListColumnClick(Sender: TObject; Column: TListColumn);
    procedure ContactExposerCalcFields(DataSet: TDataSet);
    procedure ContactExposerAfterScroll(DataSet: TDataSet);
    procedure ContactGridDblClick(Sender: TObject);
    procedure ContactGridDrawColumnCell(Sender: TObject; const Rect: TRect;
      DataCol: Integer; Column: TColumn; State: TGridDrawState);
    procedure ContactGridKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure XMLExportRelationExport(Sender: TObject; AObject: IovObject;
      ARelation: String; var Skip: Boolean;
      var ExportOptions: TObjectExportOptions);
    procedure XMLExportImportProgress(Sender: TObject; Stage: TProgressStage;
      Done, Max: Integer; var Cancel: Boolean);
  private
    FContactFilter: String;
    FSortColumn: Integer;
    FIsFiltered: Boolean;
    FCompanyClassId: Integer;
    FEmployeeClassId: Integer;
    function GetExplorerVisible: Boolean;
    procedure SetExplorerVisible(const Value: Boolean);
    procedure SetIsFiltered(const Value: Boolean);
    function GetSelectedContact: IovObject;
    procedure SetContactFilter(const Value: String);
    procedure UpdateStatus;
  protected
    procedure PerformFindQuery;
    procedure PerformQuery(const Where: string);
    procedure PerformTabQuery;
    property ContactFilter: String read FContactFilter write SetContactFilter;
    property ExplorerVisible: Boolean read GetExplorerVisible write SetExplorerVisible;
    property IsFiltered: Boolean read FIsFiltered write SetIsFiltered;
  public
    procedure Connect; override;
    procedure Disconnect; override;
    procedure Reset; override;
    procedure ExportObjects(const FileName: string);
    procedure ImportObjects(const FileName: string);
    property SelectedContact: IovObject read GetSelectedContact;
  end;

implementation

uses
  Utility, BasicEdit, ShellApi, CountryBrowse, CategoryBrowse, Model,
  ovTypes;

{$R *.DFM}

{ TContactViewForm }

procedure TContactViewForm.FormCreate(Sender: TObject);
begin
  FSortColumn := 1; //        
  ExplorerPanel.Width := Screen.Width div 3;
  ExplorerVisible := False;

  ContactExposer.Model.LoadModel;
  FCompanyClassId := ContactExposer.Model.DomainModel.GetObject('').Id;
  FEmployeeClassId := ContactExposer.Model.DomainModel.GetObject('').Id;
end;

procedure TContactViewForm.FormShow(Sender: TObject);
begin
  inherited;
  Connect
end;

procedure TContactViewForm.ActionListUpdate(Action: TBasicAction;
  var Handled: Boolean);
var
  IsSelected: Boolean;
begin    exit;
  IsSelected := SelectedContact <> nil;

  NewCompanyAction.Enabled := IsConnected;
  NewPersonAction.Enabled := IsConnected;
  DeleteAction.Enabled := IsSelected;
  EditAction.Enabled := IsSelected;
  ExportAction.Enabled := IsSelected;
  FilterAction.Enabled := IsConnected;
  SortAction.Enabled := IsConnected;
  BrowseCountryAction.Enabled := IsConnected;
  BrowseCategoryAction.Enabled := IsConnected;
  FindAction.Enabled := FindEdit.Text <> '';
  FindButton.Default := FindEdit.Focused;
  IndexTabControl.Enabled := IsConnected;
  FindPanel.Enabled := IsConnected;
end;

procedure TContactViewForm.BrowseCategoryActionExecute(Sender: TObject);
begin
  with TCategoryBrowseForm.Create(nil) do
  try
    Execute;
  finally
    Free;
  end;
end;

procedure TContactViewForm.BrowseCountryActionExecute(Sender: TObject);
begin
  with TCountryBrowseForm.Create(nil) do
  try
    Execute;
  finally
    Free;
  end;
end;

procedure TContactViewForm.Connect;
begin
  inherited;
  PerformTabQuery;
end;

procedure TContactViewForm.ContactSelectorBeforeClose(DataSet: TDataSet);
begin
//  Explorer.Clear;
end;

procedure TContactViewForm.DeleteActionExecute(Sender: TObject);
var
  ContactName: String;
begin
  ContactName := SelectedContact[''].AsString;
  if Confirm(Format(' "%s"?', [ContactName])) then
  begin
    ContactExposer.CurrentObject.Delete;
    ContactExposer.Refresh;
    UpdateStatus;
  end
end;

procedure TContactViewForm.Disconnect;
begin
//  Explorer.Clear;
//  UpdateExplorer;
  inherited;
end;

procedure TContactViewForm.EditActionExecute(Sender: TObject);
begin
  if EditObject(SelectedContact) then ContactExposer.Refresh
end;

procedure TContactViewForm.ExplorerActionExecute(Sender: TObject);
begin
  ExplorerVisible := not ExplorerVisible;
  ContactExposerAfterScroll(ContactExposer);
end;

procedure TContactViewForm.FilterActionExecute(Sender: TObject);
var
  FilterObject: IovObject;
begin
  //  '' - ,      
  if ObjectSpace.Classes[''].Objects.Count = 0
    then FilterObject := CreateObject('')
    else FilterObject := ObjectSpace.Classes[''].Objects[0];

  IsFiltered := not IsFiltered and EditObject(FilterObject);
  FilterButton.Down := IsFiltered;
end;

procedure TContactViewForm.SetIsFiltered(const Value: Boolean);
var
  Cond: String;
  FilterObject: IovObject;

  procedure AddCondition(NewCond: String);
  begin
    if Cond <> ''
    then Cond := Cond + ' AND ' + NewCond
    else Cond := NewCond
  end;

begin
  if Not Value then ContactFilter := ''
  else begin
    FilterObject := ObjectSpace.Classes[''].Objects[0];
    with FilterObject do
    begin
      Cond := '';

      if Properties[''].AsString <> ''
        then AddCondition(Format('[] = "%s"', [Properties[''].AsString]));
      if Properties['.'].AsString <> ''
        then AddCondition(Format('[.] = "%s"', [Properties['.'].AsString]));
      if Properties['.'].AsString <> ''
        then AddCondition(Format('[.] = "%s"', [Properties['.'].AsString]));
      if Properties['.'].AsString <> ''
        then AddCondition(Format('[.] = "%s"', [Properties['.'].AsString]));
 {     if Properties['.'].AsString <> ''
        then AddCondition(Format('[.] = "%s"', [Properties['.'].AsString]));}
      if Properties['..'].AsString <> ''
        then AddCondition(Format('[..] = "%s"', [Properties['..'].AsString]));
      if Properties['.'].AsString <> ''
        then AddCondition(Format('[.] = "%s"', [Properties['.'].AsString]));

      ContactFilter := Cond;
    end;
  end;
  FIsFiltered := Value;
end;

procedure TContactViewForm.SetContactFilter(const Value: String);
begin
  if FContactFilter <> Value then
  begin
    FContactFilter := Value;
    PerformTabQuery;
  end;
end;

procedure TContactViewForm.FindActionExecute(Sender: TObject);
begin
  PerformFindQuery;
end;

procedure TContactViewForm.FindEditClick(Sender: TObject);
begin
  PerformFindQuery;
end;

function TContactViewForm.GetExplorerVisible: Boolean;
begin
  Result := ExplorerPanel.Visible;
end;

procedure TContactViewForm.IndexTabControlChange(Sender: TObject);
begin
  PerformTabQuery;
end;

procedure TContactViewForm.NewCompanyActionExecute(Sender: TObject);
var
  Company: IovObject;
begin
  Company := CreateObject('');
  if EditObject(Company)
    then PerformTabQuery
    else Company.Delete;
end;

procedure TContactViewForm.NewPersonActionExecute(Sender: TObject);
var
  Person: IovObject;
begin
  Person := CreateObject('');
  if EditObject(Person)
    then PerformTabQuery
    else Person.Delete;
end;

procedure TContactViewForm.PerformFindQuery;
begin
  PerformQuery(Format('[] LIKE ''%%%s%%''', [FindEdit.Text]));
  if Not ContactExposer.IsEmpty then
  begin
    with FindEdit do
      if Items.IndexOf(Text) = -1 then
        Items.Add(Text);
    if ContactGrid.CanFocus then
      ContactGrid.SetFocus;
  end else if FindEdit.CanFocus then
    FindEdit.SetFocus;
  with IndexTabControl do
    TabIndex := Pred(Tabs.Count);
end;

procedure TContactViewForm.PerformQuery(const Where: string);
begin
  BeginBusy;
  try
    with ContactExposer do
    begin
      Close;
      if Where <> ''
        then ObjectFilter := Where
        else ObjectFilter := '';
      if ContactFilter <> '' then
        if ObjectFilter <> ''
          then ObjectFilter := '('+ContactFilter+') AND ('+ObjectFilter+')'
          else ObjectFilter := ContactFilter;
      Open;
    end
  finally
    EndBusy;
  end;
end;

procedure TContactViewForm.PerformTabQuery;

  function WhereClause: string;
  var
    I: Integer;
    OrStr, Letters: string;
  begin
    Result := '';
    with IndexTabControl do
    begin
      Letters := Tabs[TabIndex];
      if Letters = 'all' then
        Exit;
    end;
    OrStr := '';
    for I := 1 to Length(Letters) do
    begin
      Result := Result + OrStr + Format('([] LIKE ''%s%%'')',
        [UpperCase(Letters[I])]);
      OrStr := ' OR ';
    end;
  end;

begin
  with IndexTabControl do
    if TabIndex = Pred(Tabs.Count) then
      PerformFindQuery
    else
      PerformQuery(WhereClause);
end;

procedure TContactViewForm.Reset;
begin
  inherited;
  PerformTabQuery;
end;

procedure TContactViewForm.SetExplorerVisible(const Value: Boolean);
begin
  ExplorerPanel.Visible := Value;
  ExplorerSplitter.Visible := Value;
//  ExplorerButton.Down := Value;
end;

procedure TContactViewForm.ContactListColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  inherited;
  if Column.Index > 0 then begin
    if Abs(FSortColumn) = Column.Index
      then FSortColumn := - FSortColumn //   
      else FSortColumn := Column.Index;
  end
end;

function TContactViewForm.GetSelectedContact: IovObject;
begin
  if ContactExposer.Active and Not ContactExposer.IsEmpty
    then Result := ContactExposer.CurrentObject
    else Result := nil
end;

procedure TContactViewForm.ContactExposerCalcFields(DataSet: TDataSet);
var
  AObject: IovObject;
begin
  inherited;
  AObject := ContactExposer.CurrentObject;
  if Assigned(AObject) then with AObject.Properties[''] do
  begin
    try
      PhoneField.Value := Objects[0][''].AsString;
    except
      on E: EListError do ;
    end;
    ContactClassId.Value := ContactExposer.CurrentClassId;
  end
end;

procedure TContactViewForm.ContactExposerAfterScroll(DataSet: TDataSet);
begin
  inherited;
  UpdateStatus;
  if ExplorerPanel.Visible then
  begin
    if ContactClassId.AsInteger = FEmployeeClassId then
      ObjectInspector.ObjectImageIndex := 4
    else
      ObjectInspector.ObjectImageIndex := 3;
    ObjectInspector.ShowObject(ContactExposer.CurrentObject);
  end
end;

procedure TContactViewForm.ContactGridDblClick(Sender: TObject);
begin
  inherited;
  if EditAction.Enabled then EditAction.Execute;
end;

procedure TContactViewForm.ContactGridDrawColumnCell(Sender: TObject;
  const Rect: TRect; DataCol: Integer; Column: TColumn;
  State: TGridDrawState);
var
  ImageIndex: Integer;
begin
  inherited;
  if gdSelected in State then
    with ContactGrid do
      if not Focused then
      begin
        Canvas.Brush.Color := clBtnFace;
        Canvas.Font.Color := clBlack;
      end;
  ContactGrid.DefaultDrawColumnCell(Rect, DataCol, Column, State);
  if (Column.FieldName = '') and ContactExposer.Active and Not ContactExposer.IsEmpty then
  begin
    if ContactClassId.AsInteger = FEmployeeClassId then ImageIndex := 0
    else
    if ContactClassId.AsInteger = FCompanyClassId then ImageIndex := 1
    else
      ImageIndex := -1;
    with ContactGrid do
      ActionImages.Draw(Canvas, Rect.Left + 1, Rect.Top + 1, ImageIndex);
  end;
end;

procedure TContactViewForm.ContactGridKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  inherited;
  if (Shift = [ssCtrl]) and (Chr(Key) = 'F') then
    FindEdit.SetFocus;
end;

procedure TContactViewForm.UpdateStatus;
begin
//  with ContactExposer do
 //   ShowStatus(Format('%d/%d', [RecNo, RecordCount]));
end;

{ ----------------------------- /  ---------------------- }

{{
         .
}
procedure TContactViewForm.ExportActionExecute(Sender: TObject);
var
  FileName: string;
begin
  FileName := ExtractFilePath(Application.ExeName) + 'Export.xml';
  XMLExport.OutputFileName := FileName;
  XMLExport.Options := [eoEmbedReferencedObjects];
  XMLExport.Objects.Add(ContactExposer.CurrentObject);
  XMLExport.Execute;

  XMLSchemaGenerator.GenerateSchema(
    ExtractFilePath(Application.ExeName) + 'Export.xsd');

  ShellExecute(Application.Handle, nil, PChar(FileName), nil, nil, SW_SHOW);
  ShowStatus('');
end;

{{
      
}
procedure TContactViewForm.XMLExportRelationExport(Sender: TObject;
  AObject: IovObject; ARelation: String; var Skip: Boolean;
  var ExportOptions: TObjectExportOptions);
begin
  inherited;
  if (eoEmbedReferencedObjects in XMLExport.Options) and
     ((ARelation = ' ') or (ARelation = ''))
    then ExportOptions := [eoObjectId, eoAttributes]
end;

{{
     .
}
procedure TContactViewForm.ExportObjects(const FileName: string);
begin
  with XMLExport do
  begin
    Options := Options - [eoEmbedReferencedObjects];
    with ContactExposer do begin
      First;
      while Not EOF do begin
        XMLExport.Objects.Add(CurrentObject);
        Next;
      end
    end;
    OutputFileName := FileName;
    Execute;
    ShowStatus('');
  end;
end;

procedure TContactViewForm.XMLExportImportProgress(Sender: TObject;
  Stage: TProgressStage; Done, Max: Integer; var Cancel: Boolean);
begin
  inherited;
  if Stage = psMain then
    ShowStatus(Format(' : %d  %d', [Done, Max]));
end;

procedure TContactViewForm.ImportObjects(const FileName: string);
begin
  with XMLImport do
  begin
    InputFileName := FileName;
    Execute;
    ShowStatus('');
  end;
end;

end.
