unit Main;

interface

{$IFDEF VER150}
{$WARN UNSAFE_TYPE OFF}
{$WARN UNSAFE_CAST OFF}
{$WARN UNSAFE_CODE OFF}
{$ENDIF}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, ComCtrls, ToolWin, Grids, DBGrids, ImgList, ActnList,
  StdCtrls, BasicView, Stopwatch;

type
  TMainForm = class(TForm)
    ActionImages: TImageList;
    ActionList: TActionList;
    ConnectionManagerAction: TAction;
    ConnectionManagerButton: TToolButton;
    ConnectionManagerItem: TMenuItem;
    ExitItem: TMenuItem;
    FileMenu: TMenuItem;
    MainMenu: TMainMenu;
    N1: TMenuItem;
    RandomDataAction: TAction;
    RandomDataButton: TToolButton;
    RandomDataItem: TMenuItem;
    SideBar: TListView;
    SideBarImages: TImageList;
    SideBarPanel: TPanel;
    SideBarSplitter: TSplitter;
    SideBarTopSpacer: TBevel;
    StatusBar: TStatusBar;
    ToolBar: TToolBar;
    WorkClientPanel: TPanel;
    WorkPanel: TPanel;
    WorkTitleLabel: TLabel;
    WorkTitlePanel: TPanel;
    WorkTitleSpacer: TBevel;
    N2: TMenuItem;
    ExportItem: TMenuItem;
    ImportItem: TMenuItem;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ReportManagerAction: TAction;
    N3: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure SideBarSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure RandomDataActionExecute(Sender: TObject);
    procedure StopwatchStep(Sender: TObject);
    procedure StopwatchStop(Sender: TObject);
    procedure SubViewShowStatus(Sender: TObject; Text: string);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ExportItemClick(Sender: TObject);
    procedure ImportItemClick(Sender: TObject);
    procedure ToolButton2Click(Sender: TObject);
    procedure ConnectionManagerActionExecute(Sender: TObject);
    procedure ReportManagerActionExecute(Sender: TObject);
  private
    FActiveSubView: TBasicViewForm;
    FStopwatch: TStopwatch;
    FSubViewList: TList;
    procedure CreateRandomContacts(Count: Integer);
    procedure CreateSubViews;
    function GetStatusText: string;
    function GetStopwatch: TStopwatch;
    function GetSubViewList: TList;
    function GetSubViewCount: Integer;
    function GetSubViews(Index: Integer): TBasicViewForm;
    procedure InitSideBar;
    procedure SetActiveSubView(const Value: TBasicViewForm);
    procedure SetStatusText(const Value: string);
    property SubViewList: TList read GetSubViewList;
  protected
    procedure UpdateActions; override;
    procedure UpdateControls;
    procedure UpdateStatus;
    property StatusText: string read GetStatusText write SetStatusText;
    property Stopwatch: TStopwatch read GetStopwatch;
    property SubViewCount: Integer read GetSubViewCount;
    property SubViews[Index: Integer]: TBasicViewForm read GetSubViews;
  public
    destructor Destroy; override;
    procedure Connect;
    procedure Disconnect;
    procedure Reset;
    function IsShortCut(var Message: TWMKey): Boolean; override;
    property ActiveSubView: TBasicViewForm read FActiveSubView write SetActiveSubView;
  end;

var
  MainForm: TMainForm;

implementation

uses
  ShellAPI, Contnrs, Welcome, RandomData, DemoData, Utility, ContactView,
  PerformanceView, DemoDataRequest, HelpView, QueryView, ovIObjects, ovComponents,
  Model
{$IFDEF Use_Reports}
  ,ReportEdit
{$ENDIF}
  , ovConnectionManagerDlg;

{$R *.DFM}

const
  XMLFilter = 'XML files (*.xml)|*.xml';
  XMLExt = 'xml';

{ TMainForm }

procedure TMainForm.Connect;
var
  I: Integer;
begin   (*
  if not Assigned(FConnector) then
    Exit;
  BeginBusy;
  try
    FConnector.Connect;
    for I := 0 to Pred(SubViewCount) do
      SubViews[I].Connect;
 //   MainDataModule.Connect;
    UpdateControls;
    UpdateStatus;
  finally
    EndBusy;
  end;   *)
end;

procedure TMainForm.CreateRandomContacts(Count: Integer);
var
  Companies: TInterfaceList;

  function CreateRandomContact: IovObject;
  var
    Company: IovObject;
  begin
    if Random(2) = 0 then
    begin
      if (Random(2) = 0) and (Companies.Count > 10) then
        Company := IovObject(Companies[Random(Companies.Count)])
      else
        Company := nil;
      Result := CreateRandomPerson(Company)
    end else
    begin
      Result := CreateRandomCompany;
      if Random(2) = 0 then
      begin
        if Companies.Count > 50 then
          Companies.Delete(0);
        Companies.Add(Result);
      end;
    end;
  end;

var
  I, CommitCount: Integer;
begin
  ObjectSpace.StartTransaction;
  try
    ObjectSpace.BeginUpdate;
    try
      CreateCategories;
      CreateCountries;
    finally
      ObjectSpace.EndUpdate;
    end;
    ObjectSpace.CommitTransaction;
  except
    ObjectSpace.RollbackTransaction;
    raise;
  end;

  CommitCount := 100;
  GetAsyncKeyState(VK_ESCAPE);
  Randomize;
  Companies := TInterfaceList.Create;
  Stopwatch.Start(Count);
  try
    ObjectSpace.StartTransaction;
    try
      for I := 0 to Pred(Count) do
      begin

        CreateRandomContact;

        Stopwatch.Step;
        if GetAsyncKeyState(VK_ESCAPE) <> 0 then
        begin
          Application.ProcessMessages;
          if Confirm(' ?') then
            Break
          else
            Application.ProcessMessages;
        end;
        if (Succ(I) mod CommitCount) = 0 then
        begin
          ObjectSpace.CommitTransaction;
          ObjectSpace.StartTransaction;
        end;
      end;
      ObjectSpace.CommitTransaction;
    except
      ObjectSpace.RollbackTransaction;
      raise;
    end;
  finally
    Stopwatch.Stop;
    Companies.Free;
  end;
end;

procedure TMainForm.CreateSubViews;

  procedure CreateSubView(FormClass: TBasicViewFormClass);
  var
    Form: TBasicViewForm;
  begin
    Form := FormClass.Create(Self);
    Form.OnShowStatus := SubViewShowStatus;
    SubViewList.Add(Form);
  end;

begin
  CreateSubView(THelpViewForm);
  CreateSubView(TContactViewForm);
  CreateSubView(TQueryViewForm);
  InitSideBar;
end;

destructor TMainForm.Destroy;
begin
  inherited;
  FSubViewList.Free;
  FStopwatch.Free;
end;

procedure TMainForm.Disconnect;
var
  I: Integer;
begin
(*  BeginBusy;
  try
    for I := 0 to Pred(SubViewCount) do
      SubViews[I].Disconnect;
 //   MainDataModule.Disconnect;
    if Assigned(FConnector) then
    begin
      FConnector.Disconnect;
      FreeAndNil(FConnector);
    end;
    FConnectionDef := nil;
    UpdateControls;
    UpdateStatus;
  finally
    EndBusy;
  end;     *)
end;

procedure TMainForm.ExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.ExportItemClick(Sender: TObject);
begin
  with TSaveDialog.Create(nil) do
  try
    Options := Options + [ofOverwritePrompt];
    Filter := XMLFilter;
    DefaultExt := XMLExt;
    if Execute then
    begin
      (SubViews[1] as TContactViewForm).ExportObjects(FileName);
      ShellExecute(Application.Handle, nil, PChar(FileName), nil, nil, SW_SHOW);
    end
  finally
    Free;
  end;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Disconnect;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  CreateSubViews;
  UpdateStatus;
{$IFNDEF Use_Reports}
  ReportManagerAction.Visible := False;
{$ENDIF}
end;

function TMainForm.GetStatusText: string;
begin
  Result := StatusBar.Panels[0].Text;
end;

function TMainForm.GetStopwatch: TStopwatch;
begin
  if not Assigned(FStopwatch) then
  begin
    FStopwatch := TStopwatch.Create;
    FStopwatch.OnStep := StopwatchStep;
    FStopwatch.OnStop := StopwatchStop;
  end;
  Result := FStopwatch;
end;

function TMainForm.GetSubViewCount: Integer;
begin
  Result := SubViewList.Count;
end;

function TMainForm.GetSubViewList: TList;
begin
  if not Assigned(FSubViewList) then
    FSubViewList := TList.Create;
  Result := FSubViewList;
end;

function TMainForm.GetSubViews(Index: Integer): TBasicViewForm;
begin
  Result := SubViewList[Index];
end;

procedure TMainForm.ImportItemClick(Sender: TObject);
begin
  with TOpenDialog.Create(nil) do
  try
    Filter := XMLFilter;
    if Execute then
    begin
      (SubViews[1] as TContactViewForm).ImportObjects(FileName);
      Reset;
    end;
  finally
    Free;
  end;
end;

procedure TMainForm.InitSideBar;

  procedure AddShortcut(Form: TForm; Index: Integer);
  begin
    with SideBar.Items.Add do
    begin
      Caption := Form.Caption;
      ImageIndex := Index;
      Data := Form;
    end;
  end;

var
  I: Integer;
begin
  SideBar.Items.Clear;
  for I := 0 to Pred(SubViewCount) do
    AddShortcut(SubViews[I], I);
  SideBar.Selected := SideBar.Items[0];
end;

procedure TMainForm.RandomDataActionExecute(Sender: TObject);
begin
  with TDemoDataRequestForm.Create(nil) do
  try
    Count := 100;
    if ShowModal = mrOk then
    begin
      CreateRandomContacts(Count);

      while ObjectSpace.InTransaction do ObjectSpace.CommitTransaction;
      
      Reset;
    end;
  finally
    Free;
  end;
end;

procedure TMainForm.Reset;
var
  I: Integer;
begin
  BeginBusy;
  try
    Update;
    for I := 0 to Pred(SubViewCount) do
      SubViews[I].Reset;
  finally
    EndBusy;
  end;
end;

procedure TMainForm.SetActiveSubView(const Value: TBasicViewForm);
begin
  if Value <> FActiveSubView then
  begin
    if Assigned(FActiveSubView) then
      FActiveSubView.Hide;
    FActiveSubView := Value;
    if Assigned(FActiveSubView) then
    begin
      WorkTitleLabel.Caption := FActiveSubView.Caption;
      with FActiveSubView do
      begin
        BorderStyle := bsNone;
        Align := alClient;
        Parent := WorkClientPanel;
        Height := Parent.ClientHeight;
        Width := Parent.ClientWidth;
        Show;
        if CanFocus then
          SetFocus;
      end;
    end;
    StatusText := '';
    UpdateControls;
  end;
end;

procedure TMainForm.SetStatusText(const Value: string);
begin
  StatusBar.Panels[0].Text := Value;
  StatusBar.Update;
end;

procedure TMainForm.SideBarSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  if Assigned(SideBar.Selected) then
    ActiveSubView := SideBar.Selected.Data;
end;

procedure TMainForm.StopwatchStep(Sender: TObject);
begin
  with Stopwatch do
    StatusText := IntToStr(StepPercent) + '%';
end;

procedure TMainForm.StopwatchStop(Sender: TObject);
begin
  with Stopwatch do
    ShowMessage(Format('%d objects processed in %d ms. (%d o/s)',
      [StepIndex, TotalTime, Round(StepIndex / (TotalTime / 1000))]));
  StatusText := '';
end;

procedure TMainForm.SubViewShowStatus(Sender: TObject; Text: string);
begin
  StatusText := Text;
end;

procedure TMainForm.UpdateActions;
{$J+}
const
  FirstTime: Boolean = True;
{$J-}
begin
  inherited;
  (*
  if FirstTime then
  begin
    FirstTime := False;
    with TWelcomeForm.Create(nil) do
    try
      ShowModal;
    finally
      Free;
    end;
  end;
  ImportItem.Enabled := IsConnected;
  ExportItem.Enabled := IsConnected;  *)
end;

procedure TMainForm.UpdateControls;
var
  I: Integer;
begin
//  for I := 0 to Pred(SubViewCount) do
//    SubViews[I].UpdateControls;
end;

procedure TMainForm.UpdateStatus;
const
  Pad = '      ';
begin
  with StatusBar do
  begin
//    Panels[1].Text := 'Connection: ' + ConnectionName + Pad;
    Update;
  end;
end;

procedure TMainForm.ToolButton2Click(Sender: TObject);
begin
  Close
end;

//       "
// "   .    , 
//    ActionList    .
function TMainForm.IsShortCut(var Message: TWMKey): Boolean;
begin
  if ActiveSubView <> nil
    then Result := ActiveSubView.IsShortCut(Message)
    else Result := False;
  if Not Result then Result := inherited IsShortCut(Message);
end;

procedure TMainForm.ConnectionManagerActionExecute(Sender: TObject);
begin
  if dmModel.ovModel.ShowConnectionManager and (ActiveSubView <> nil)
    then ActiveSubView.Reset
end;

procedure TMainForm.ReportManagerActionExecute(Sender: TObject);
begin
{$IFDEF Use_Reports}
  with TReportEditForm.Create(Application) do
  try
    ShowModal
  finally
    Free
  end;
{$ENDIF}
end;

end.
