unit ReportEdit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ovDataLinks, ovComponents, ovDataObjects, ActnList, StdCtrls,
  Grids, DBGrids, DBActns, ovActions, ovDBControlLinks, ImgList, ComCtrls,
  ToolWin, ovIObjects, DB, ovVirtualDataSet, ovDataSet, ovFReportDesigner;

type
  TReportEditForm = class(TForm)
    DBGrid: TDBGrid;
    ActionList: TActionList;
    NewReportAction: TovDataObjectsAction;
    DeleteReportAction: TovDataObjectsAction;
    EditReportAction: TovDataObjectsAction;
    PreviewAction: TAction;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    DisabledImages: TImageList;
    CmdImages: TImageList;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ReportExposer: TovExposer;
    ReportSource: TDataSource;
    ExportReportAction: TAction;
    ImportReportAction: TAction;
    ReportDesigner: TovFrxReportDesigner;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    ToolButton7: TToolButton;
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure DeleteReportActionExecute(Sender: TObject);
    procedure EditReportActionExecute(Sender: TObject);
    procedure ExportReportActionExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ImportReportActionExecute(Sender: TObject);
    procedure NewReportActionExecute(Sender: TObject);
    procedure PreviewActionExecute(Sender: TObject);
    procedure ReportDesignerLoadReport(Sender: TObject);
    procedure ReportDesignerSaveReport(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure LoadReport;
    procedure SaveReport;
    { Public declarations }
  end;

var
  ReportEditForm: TReportEditForm;

implementation

{$R *.dfm}

procedure TReportEditForm.NewReportActionExecute(Sender: TObject);
var
  ReportName: String;
begin
  if InputQuery('  ', ' ', ReportName) then
  with ReportSource.DataSet do
  begin
    Append;
    FieldByName('').AsString := ReportName;
    Post;
    EditReportAction.Execute;
  end
end;

procedure TReportEditForm.DeleteReportActionExecute(Sender: TObject);
begin
  if Application.MessageBox('   ?',
    '', MB_ICONQUESTION or MB_YESNO) = IDYES
  then ReportExposer.Delete
end;

procedure TReportEditForm.EditReportActionExecute(Sender: TObject);
begin
  ReportDesigner.ShowDesigner;
end;

procedure TReportEditForm.PreviewActionExecute(Sender: TObject);
begin
  ReportDesigner.PrintReport(True);
end;

procedure TReportEditForm.ActionListUpdate(Action: TBasicAction; var Handled:
    Boolean);
begin
  with ReportSource do
  begin
    NewReportAction.Enabled := Assigned(DataSet) and DataSet.Active;
    DeleteReportAction.Enabled := NewReportAction.Enabled and Not DataSet.IsEmpty;
    EditReportAction.Enabled := DeleteReportAction.Enabled;
    PreviewAction.Enabled := EditReportAction.Enabled;
    ExportReportAction.Enabled := EditReportAction.Enabled;
    ImportReportAction.Enabled := NewReportAction.Enabled;
  end
end;

procedure TReportEditForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ReportExposer.CheckBrowseMode;
end;

procedure TReportEditForm.ReportDesignerLoadReport(Sender: TObject);
begin
  LoadReport;
end;

procedure TReportEditForm.ReportDesignerSaveReport(Sender: TObject);
begin
  if (Application.MessageBox('    ?',
    '', MB_ICONQUESTION or MB_YESNO) = IDYES)
  then SaveReport;
end;

procedure TReportEditForm.LoadReport;
begin
  with ReportSource.DataSet do
  begin
    ReportDesigner.NewReport;
    try
      ReportDesigner.LoadFromDB(dkDataObjects, FieldByName(''));
      ReportDesigner.OpenData;
    except
      on E: Exception do
        Application.MessageBox(PChar(E.Message), '  ', MB_ICONERROR or MB_OK);
    end;
    ReportDesigner.LoadFromDB(dkReport, FieldByName(''));
  end;
end;

procedure TReportEditForm.SaveReport;
begin
  with ReportSource.DataSet do
  begin
    Edit;
    ReportDesigner.SaveToDB(dkDataObjects, FieldByName(''));
    ReportDesigner.SaveToDB(dkReport, FieldByName(''));
    Post
  end;
end;

procedure TReportEditForm.ExportReportActionExecute(Sender: TObject);
var
  FileName: String;
begin
  SaveDialog.FileName := ReportExposer.FieldByName('').AsString;
  if SaveDialog.Execute then
  begin
    LoadReport;
    FileName := ChangeFileExt(SaveDialog.FileName, '.frd');
    ReportDesigner.SaveToFile(dkDataObjects, FileName);
    FileName := ChangeFileExt(SaveDialog.FileName, '.fr3');
    ReportDesigner.SaveToFile(dkReport, FileName);
  end
end;

procedure TReportEditForm.ImportReportActionExecute(Sender: TObject);
var
  FileName: String;
begin
  if OpenDialog.Execute then
  begin
    FileName := ChangeFileExt(OpenDialog.FileName, '.frd');
    if FileExists(FileName) then
      ReportDesigner.LoadFromFile(dkDataObjects, FileName);
    FileName := ChangeFileExt(OpenDialog.FileName, '.fr3');
    if FileExists(FileName) then
      ReportDesigner.LoadFromFile(dkReport, FileName);
    with ReportSource.DataSet do
    begin
      Append;
      FieldByName('').AsString := ChangeFileExt(ExtractFileName(OpenDialog.FileName), '');
      Post;
    end;
    SaveReport;
  end
end;

end.
