unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ovComponents, ovFReportDesigner, ActnList,
  DB, Grids, DBGrids;

type
  TfmReportManager = class(TForm)
    ReportDesigner: TovFrxReportDesigner;
    DesignerBtn: TBitBtn;
    PreviewBtn: TButton;
    Grid: TDBGrid;
    DataSource: TDataSource;
    ActionList: TActionList;
    NewAction: TAction;
    DeleteAction: TAction;
    EditAction: TAction;
    PreviewAction: TAction;
    Button1: TButton;
    Button2: TButton;
    Label1: TLabel;
    Label2: TLabel;
    procedure NewActionExecute(Sender: TObject);
    procedure DeleteActionExecute(Sender: TObject);
    procedure EditActionExecute(Sender: TObject);
    procedure PreviewActionExecute(Sender: TObject);
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure FormCreate(Sender: TObject);
  private
    procedure CreateReportTable;
    { Private declarations }
  protected
    function OpenReportDataSet: TDataSet;
  public
    { Public declarations }
    procedure LoadReport;
    procedure SaveReport;
  end;

var
  fmReportManager: TfmReportManager;

implementation

uses ovIObjects, ovTypes;

{$R *.dfm}

procedure TfmReportManager.FormCreate(Sender: TObject);
begin
  DataSource.DataSet := OpenReportDataSet;
end;

function TfmReportManager.OpenReportDataSet: TDataSet;
begin
  try
    Result := IDatabaseAdapter(DefaultModel.DatabaseAdapter).OpenTable('Reports');
  except
    CreateReportTable;
    Result := IDatabaseAdapter(DefaultModel.DatabaseAdapter).OpenTable('Reports');
  end;
end;

procedure TfmReportManager.CreateReportTable;
begin
  with TTableDefs.Create do
  try
    with FieldDefs do
    begin
      Add('Id', ftAutoInc);
      Add('Name', ftString, 100);
      Add('ReportDef', ftBlob);
      Add('DataObjectsDef', ftBlob);
    end;
    IDatabaseAdapter(DefaultModel.DatabaseAdapter).CreateTable(
      'Reports', FieldDefs, IndexDefs);
  finally
    Free
  end;
end;

procedure TfmReportManager.NewActionExecute(Sender: TObject);
var
  ReportName: String;
begin
  if InputQuery('  ', ' ', ReportName) then
  with DataSource.DataSet do
  begin
    Append;
    FieldByName('Name').AsString := ReportName;
    Post;
    EditAction.Execute;
  end
end;

procedure TfmReportManager.DeleteActionExecute(Sender: TObject);
begin
  if Application.MessageBox(' ?',
    '', MB_ICONQUESTION or MB_YESNO) = IDYES
  then DataSource.DataSet.Delete
end;

procedure TfmReportManager.EditActionExecute(Sender: TObject);
begin
  LoadReport;
  ReportDesigner.ShowDesigner;
  SaveReport;
end;

procedure TfmReportManager.PreviewActionExecute(Sender: TObject);
begin
  LoadReport;
  ReportDesigner.ShowReport;
end;

procedure TfmReportManager.ActionListUpdate(Action: TBasicAction;
  var Handled: Boolean);
begin
  with DataSource do
  begin
    NewAction.Enabled := Assigned(DataSet) and DataSet.Active;
    DeleteAction.Enabled := NewAction.Enabled and Not DataSet.IsEmpty;
    EditAction.Enabled := DeleteAction.Enabled;
    PreviewAction.Enabled := DeleteAction.Enabled;
  end
end;

procedure TfmReportManager.LoadReport;
begin
  with DataSource.DataSet do
  begin
    ReportDesigner.NewReport;
    ReportDesigner.LoadFromDB(dkDataObjects, FieldByName('DataObjectsDef'));
    ReportDesigner.LoadFromDB(dkReport, FieldByName('ReportDef'));
  end;
end;

procedure TfmReportManager.SaveReport;
begin
  with DataSource.DataSet do
  begin
    Edit;
    ReportDesigner.SaveToDB(dkDataObjects, FieldByName('DataObjectsDef'));
    ReportDesigner.SaveToDB(dkReport, FieldByName('ReportDef'));
    Post
  end;
end;

end.
