unit ovADOAdapter;
interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, ADODB, ovTypes, ovModel, ovComponents;

type
  {{
        ADO  .
  }
  TovADOAdapter = class(TovCustomDatabaseAdapter)
  private
    FConnection: TADOConnection;
    procedure DatabaseDisconnectHandler(Sender: TObject);
    procedure SetConnection(Value: TADOConnection);
  protected
    procedure GetCreateTableQuery(ATableName: String; AFieldsDefs: TFieldDefs;
        AnIndexDefs: TIndexDefs; AQuery: TStrings); virtual; abstract;
    procedure CheckConnection;
    procedure CloseDatabase; override;
    procedure CommitTransaction; override;
    procedure ExecQuery(SQLText: String); override;
    procedure OpenDatabase; override;
    function OpenQuery(QueryText: String; MasterDataSource: TDataSource = nil;
      LiveDataSet: Boolean = False): TDataSet; override;
    function OpenTable(ATableName: String; AReadOnly: Boolean = False; AFilter:
      String = ''): TDataSet; override;
    procedure RollbackTransaction; override;
    procedure StartTransaction; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function InTransaction: Boolean; override;
    procedure CreateTable(ATableName: String; AFieldsDefs: TFieldDefs; AnIndexDefs:
      TIndexDefs); override;
    function RestructureTable(ATableName: String; AFieldsDefs: TFieldDefs;
        AnIndexDefs: TIndexDefs): Boolean; virtual; abstract;
    function GetFieldList(ATableName: String; SkipBlobFields: Boolean): String;
    function TableExists(ATableName: String): Boolean; override;
  public
    procedure SetupConnection(ConnectionParams: TStrings); override;
    function EditConnection(ConnectionParams: TStrings): Boolean; override;
  published
    property Connection: TADOConnection read FConnection write SetConnection;
  end;

function GetConnectionStringParam(ConnectionString, ParamName: String): String;
function SetConnectionStringParam(ConnectionString, ParamName, ParamValue:
  String): String;

implementation

uses ActiveX, TypInfo, ovConsts, ovObjects, AdoConEd, Variants;

{{
   ParamName    ConnectionString.
}
function GetConnectionStringParam(ConnectionString, ParamName: String): String;
var
  I, P: Integer;
  S: String;
begin
  Result := '';
  P := 1;
  while P <= Length(ConnectionString) do
  begin
    S := ExtractFieldName(ConnectionString, P);
    I := Pos('=', S);
    if Copy(S, 1, I-1) = ParamName then
    begin
      Result := Copy(S, I+1, Length(S));
      Exit;
    end
  end;
end;

{{
   ParamName    ConnectionString 
ParamValue     .
}
function SetConnectionStringParam(ConnectionString, ParamName, ParamValue:
  String): String;
var
  I, P: Integer;
  S: String;
begin
  Result := '';
  P := 1;
  while P <= Length(ConnectionString) do
  begin
    S := ExtractFieldName(ConnectionString, P);
    I := Pos('=', S);
    if Copy(S, 1, I-1) = ParamName then
      Result := Result + ';' + ParamName + '=' + ParamValue
    else
      Result := Result + ';' + S;
  end;
  Delete(Result, 1, 1);
end;

{
******************************* TovADOAdapter ***********************
}

var
  DatabaseCount: Integer = 0;

procedure TovADOAdapter.SetConnection(Value: TADOConnection);
begin
  if FConnection <> Value then
  begin
    if Assigned(FConnection) then FConnection.AfterDisconnect := nil;
    FConnection := Value;
    if Value <> nil then
    begin
      Value.FreeNotification(Self);
      Value.AfterDisconnect := DatabaseDisconnectHandler;
    end;
  end;
end;

procedure TovADOAdapter.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = Connection) and (Operation = opRemove)
    then Connection := nil;
end;

{{
       
}
procedure TovADOAdapter.DatabaseDisconnectHandler(Sender: TObject);
begin
  //       
  CloseDatabaseNotification
end;

{
*******************************     ************************
}

procedure TovADOAdapter.OpenDatabase;
begin
  CheckConnection;
  Connection.Open;
end;

procedure TovADOAdapter.CloseDatabase;
begin
  if Assigned(Connection) then Connection.Close;
end;

procedure TovADOAdapter.StartTransaction;
begin
  CheckConnection;
  Connection.BeginTrans
end;

procedure TovADOAdapter.RollbackTransaction;
begin
  CheckConnection;
  Connection.RollbackTrans
end;

procedure TovADOAdapter.CommitTransaction;
begin
  CheckConnection;
  Connection.CommitTrans;
end;

function TovADOAdapter.InTransaction: Boolean;
begin
  CheckConnection;
  Result := Connection.InTransaction
end;

{
*******************************   ****************************
}

procedure TovADOAdapter.ExecQuery(SQLText: String);
begin
  CheckConnection;

  with TADOQuery.Create(nil) do
  try
    Connection := Self.Connection;
    ParamCheck := False;
    SQL.Text := SQLText;
    ExecSQL;
  finally
    Free
  end;
end;

function TovADOAdapter.OpenQuery(QueryText: String;
  MasterDataSource: TDataSource = nil; LiveDataSet: Boolean = False): TDataSet;
begin
  CheckConnection;

  Result := TADOQuery.Create(nil);
  try
    with TADOQuery(Result) do
    begin
      Connection := Self.Connection;
      SQL.Text := QueryText;
      DataSource := MasterDataSource;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

{
*******************************    *****************************
}

function TovADOAdapter.OpenTable(ATableName: String; AReadOnly:
  Boolean = False; AFilter: String = ''): TDataSet;
begin
  CheckConnection;

  Result := nil;
  try
    if AFilter <> '' then
    begin
      Result := TADOQuery.Create(nil);
      with TADOQuery(Result) do
      begin
        Connection := Self.Connection;
        SQL.Text := 'SELECT * FROM ' +
          QuerySyntaxParams.LeftTableDelimiter + ATableName +
          QuerySyntaxParams.RightTableDelimiter + ' WHERE ' + AFilter;
      end;
    end else
    begin
      Result := TADOTable.Create(nil);
      with TADOTable(Result) do
      begin
        Connection := Self.Connection;
        TableName := ATableName;
        ReadOnly := AReadOnly;
      end;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

procedure TovADOAdapter.CreateTable(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs);
var
  I: Integer;
  Query: TStrings;
begin
  CheckConnection;

  if Not RestructureTable(ATableName, AFieldsDefs, AnIndexDefs) then
  begin
    Query := TStringList.Create;
    try
      GetCreateTableQuery(ATableName, AFieldsDefs, AnIndexDefs, Query);
      for I := 0 to Query.Count - 1 do
        ExecQuery(Query[I]);
    finally
      Query.Free
    end;
  end
end;

function TovADOAdapter.TableExists(ATableName: String): Boolean;
var
  Filter: OleVariant;
  ADataSet: TADODataSet;
begin
  Filter := VarArrayCreate([0, 3], varVariant);
  Filter[2] := ATableName;
  ADataSet := TADODataSet.Create(nil);
  try
    Connection.OpenSchema(siTables, Filter, EmptyParam, ADataSet);
    Result := ADataSet.RecordCount > 0;
  finally
    ADataSet.Free
  end;
end;

{{
        , 
.  SkipBlobFields     blob-.
}
function TovADOAdapter.GetFieldList(ATableName: String; SkipBlobFields:
  Boolean): String;
var
  I: Integer;
  Table: TADOTable;
begin
  Table := TADOTable.Create(nil);
  with Table do
  try
    Connection := Self.Connection;
    TableName := ATableName;
    FieldDefs.Update;
    Result := '';
    for I := 0 to FieldDefs.Count - 1 do
      if Not SkipBlobFields or (FieldDefs[I].DataType <> ftBlob) then
        Result := Result + ', ' +
          QuerySyntaxParams.LeftFieldDelimiter + FieldDefs[I].Name +
          QuerySyntaxParams.RightFieldDelimiter;
    System.Delete(Result, 1, 2);
  finally
    Free;
  end;
end;

{
*******************************   ***************************
}

procedure TovADOAdapter.CheckConnection;
begin
  if Not Assigned(Connection) then
    raise Exception.Create('   Connection  ' + Name);
end;

function TovADOAdapter.EditConnection(ConnectionParams: TStrings):
    Boolean;
var
  ADOConnection: TADOConnection;
begin
  ADOConnection := TADOConnection.Create(Self);
  try
    ADOConnection.ConnectionString := ConnectionParams.Values['ConnectionString'];
    Result := EditConnectionString(ADOConnection);
    if Result then
      ConnectionParams.Values['ConnectionString'] := ADOConnection.ConnectionString;
  finally
    ADOConnection.Free
  end;
end;

procedure TovADOAdapter.SetupConnection(ConnectionParams: TStrings);
begin
  Connection := TADOConnection.Create(Self);
  with ConnectionParams do
  begin
    Connection.ConnectionString := Values['ConnectionString'];
    Connection.LoginPrompt := StrToBoolDef(Values['LoginPrompt'], False);
  end
end;

initialization
  CoInitialize(nil);
end.
