unit ovAccuracerDBParams;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, ComCtrls, Mask;

const
  SCryptoAlgorithm = 'CryptoAlgorithm';
  SCompressionLevel = 'CompressionLevel';
  SCompressionAlgorithm = 'CompressionAlgorithm';

type
  TovAbsoluteDBParams = class(TForm)
    OkBtn: TButton;
    CancelBtn: TButton;
    OpenDialog: TOpenDialog;
    GroupBox1: TGroupBox;
    Label3: TLabel;
    Label5: TLabel;
    cbCryptoAlgorithm: TComboBox;
    edPassword: TEdit;
    cbEncrypted: TCheckBox;
    edDatabase: TEdit;
    OpenBtn: TSpeedButton;
    GroupBox2: TGroupBox;
    cbCompressionAlgorithm: TComboBox;
    Label1: TLabel;
    edCompressionLevel: TMaskEdit;
    UpDown: TUpDown;
    Label2: TLabel;
    cbBlobCompression: TCheckBox;
    procedure cbEncryptedClick(Sender: TObject);
    procedure OpenBtnClick(Sender: TObject);
    procedure cbBlobCompressionClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    procedure SetParams(Params: TStrings);
    procedure GetParams(Params: TStrings);
    function ValidateParams: Boolean;
    { Private declarations }
  public
    { Public declarations }
  end;

function ShowAdapterParams(Params: TStrings): Boolean;

implementation


{$R *.DFM}

function ShowAdapterParams(Params: TStrings): Boolean;
begin
  with TovAbsoluteDBParams.Create(Application) do
  try
    SetParams(Params);
    Result := ShowModal = mrOk;
    if Result then GetParams(Params);
  finally
    Free
  end;
end;

{ TovAbsoluteDBParams }

procedure TovAbsoluteDBParams.SetParams(Params: TStrings);
begin
  edDatabase.Text := Params.Values['DatabaseFileName'];
  edPassword.Text := Params.Values['Password'];
  cbEncrypted.Checked := Params.Values['Password'] <> '';
  if Params.Values[SCryptoAlgorithm] <> ''
    then cbCryptoAlgorithm.ItemIndex := StrToInt(Params.Values[SCryptoAlgorithm])
    else cbCryptoAlgorithm.ItemIndex := -1;
end;

procedure TovAbsoluteDBParams.GetParams(Params: TStrings);
begin
  Params.Values['DatabaseFileName'] := edDatabase.Text;
  if cbEncrypted.Checked then begin
    Params.Values['Password'] := edPassword.Text;
    Params.Values[SCryptoAlgorithm] := IntToStr(cbCryptoAlgorithm.ItemIndex);
  end else begin
    Params.Values['Password'] := '';
    Params.Values[SCryptoAlgorithm] := '';
  end;
  if cbBlobCompression.Checked then begin
    Params.Values[SCompressionLevel] := edCompressionLevel.Text;
    Params.Values[SCompressionAlgorithm] := IntToStr(cbCompressionAlgorithm.ItemIndex + 1);
  end else begin
    Params.Values[SCompressionLevel] := '';
    Params.Values[SCompressionAlgorithm] := '';
  end;
end;

procedure TovAbsoluteDBParams.cbEncryptedClick(Sender: TObject);
begin
  edPassword.Enabled := cbEncrypted.Checked;
  cbCryptoAlgorithm.Enabled := cbEncrypted.Checked;
  if Not cbEncrypted.Checked then begin
    edPassword.Text := '';
    cbCryptoAlgorithm.ItemIndex := -1;
  end
end;

procedure TovAbsoluteDBParams.cbBlobCompressionClick(Sender: TObject);
begin
  edCompressionLevel.Enabled := cbBlobCompression.Checked;
  cbCompressionAlgorithm.Enabled := cbBlobCompression.Checked;
  if Not cbBlobCompression.Checked then begin
    edCompressionLevel.Text := '0';
    cbCompressionAlgorithm.ItemIndex := -1;
  end
end;

procedure TovAbsoluteDBParams.OpenBtnClick(Sender: TObject);
begin
  OpenDialog.FileName := edDatabase.Text;
  if OpenDialog.Execute
    then edDatabase.Text := OpenDialog.FileName
end;

procedure TovAbsoluteDBParams.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := (ModalResult <> mrOk) or ValidateParams
end;

function TovAbsoluteDBParams.ValidateParams: Boolean;
begin
  Result := False;
  if edDatabase.Text = '' then begin
    MessageDlg('     ', mtError, [mbOk], 0);
    edDatabase.SetFocus;
    Exit;
  end;

  if cbEncrypted.Checked then
    if cbCryptoAlgorithm.Text = '' then begin
      MessageDlg('   ', mtError, [mbOk], 0);
      cbCryptoAlgorithm.SetFocus;
      Exit;
    end else
    if edPassword.Text = '' then begin
      MessageDlg('   ', mtError, [mbOk], 0);
      edPassword.SetFocus;
      Exit;
    end;

  if cbBlobCompression.Checked then
    if cbCompressionAlgorithm.Text = '' then begin
      MessageDlg('   ', mtError, [mbOk], 0);
      cbCompressionAlgorithm.SetFocus;
      Exit;
    end else
    if edCompressionLevel.Text = '' then begin
      MessageDlg('   ', mtError, [mbOk], 0);
      edCompressionLevel.SetFocus;
      Exit;
    end;
  Result := True;
end;

end.
