{$I ..\Core\OView.inc}

unit ovEAAdapter;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  {$IFDEF DELPHI6_OR_HIGHER}
  Variants,
  {$ENDIF}
  Forms, Dialogs, ovModel, ovComponents, ovModelSync, EA_TLB;

type
  TovEAAdapter = class(TovUpdatableModelStorage)
  private
    FEAApplication: TApp;
    FEARepository: OLEVariant;
    FPackageId: Integer;
    FImportExternalClasses: Boolean;
    function GetEARepository: OLEVariant;
    function GetEAApplication: TApp;
    procedure LoadObjects(Model: TDomainModel);
    function LoadRelation(Model: TDomainModel; EAObject, EAConnector: OleVariant): 
      TModelRelation;
    function GetCurrentPackage: OLEVariant;
    function LoadObject(Model: TDomainModel; EAObject: OleVariant): TModelObject;
    property CurrentPackage: OLEVariant read GetCurrentPackage;
    procedure LoadAttributes(ModelObject: TModelObject; EAObject: OleVariant);
    procedure LoadRelations(Model: TDomainModel);
    function LoadGeneralization(Model: TDomainModel; EAObject, EAConnector:
      OleVariant): TModelRelation;
    function FindClass(APackage: OleVariant; AClassName: String; var AClass: 
      OleVariant): Integer;
    function GetClass(APackage: OleVariant; AClassName: String): OLEVariant;
      overload;
    function FindAttribute(AClass: OleVariant; AttributeName: String; var
      Attribute: OleVariant): Integer;
    function GetAttribute(AClass: OleVariant; AttributeName: String): OLEVariant;
      overload;
    function FindGeneralization(AClass: OleVariant; AParentClassName: String; var
      Generalization: OleVariant): Integer;

    procedure AddAttributeInternal(SourceAttribute: TModelAttribute; TargetObject:
      OleVariant);
    procedure AddRelationInternal(SourceRelation: TModelRelation; TargetObject:
      OleVariant);
    procedure AddGeneralizationInternal(SourceRelation: TModelRelation;
      TargetObject: OleVariant);
  protected
  public
    //   TovCustomModelStorage
    procedure LoadModel(Model: TDomainModel); override;

    //   TovUpdatableModelStorage
    procedure AddClass(SourceModel: TDomainModel; ClassName: String); override;
    procedure AddClassRelations(SourceModel: TDomainModel; ClassName: String);
      override;
    procedure AddAttribute(SourceModel: TDomainModel; ClassName, AttributeName:
      String); override;
    procedure AddRelation(SourceModel: TDomainModel; ClassName, RelationName:
      String); override;
    procedure AddGeneralization(SourceModel: TDomainModel; ClassName,
      ParentClassName: String); override;

    procedure UpdateAttribute(SourceModel: TDomainModel; ClassName, AttributeName:
      String); override;
    procedure UpdateRelation(SourceModel: TDomainModel; ClassName, RelationName:
      String); override;

    procedure DeleteClass(ClassName: String); override;
    procedure DeleteAttribute(ClassName, AttributeName: String); override;
    procedure DeleteRelation(ClassName, RelationName: String); override;
    procedure DeleteGeneralization(ClassName, ParentClassName: String); override;
    procedure SaveModel(Model: TDomainModel; SaveAllObjects: Boolean = False); 
      override;
    //  
    property EARepository: OLEVariant read GetEARepository;
    property EAApplication: TApp read GetEAApplication;
  published
    property PackageId: Integer read FPackageId write FPackageId;
    property ImportExternalClasses: Boolean read FImportExternalClasses write 
      FImportExternalClasses default False;
  end;

procedure ParseDomainType(TypeDef: String; var TypeName: String; var TypeSize:
  Integer);

implementation

uses ComObj, ovUtils, ovEAPackages;

{$R *.dcr}

procedure ParseCardinality(Cardinality: String; var Min, Max: Integer);

  function ConvertPart(S: String): Integer;
  begin
    if S = '' then Result := 0
    else
    if S = '*' then Result := Unlimited
    else Result := StrToInt(S)
  end;

  procedure GetParts(Cardinality: String; var MinStr, MaxStr: String);
  var P: Integer;
  begin
    P := Pos('..', Cardinality);
    if P = 0 then begin
      MinStr := Cardinality;
      MaxStr := Cardinality;
    end else begin
      MinStr := Trim(Copy(Cardinality, 1, P-1));
      MaxStr := Trim(Copy(Cardinality, P+2, Length(Cardinality)));
    end;
  end;

var
  MinStr, MaxStr: String;
begin
  GetParts(Cardinality, MinStr, MaxStr);
  if MinStr = '*' then MinStr := '0';

  Min := ConvertPart(MinStr);
  Max := ConvertPart(MaxStr);
end;

function MakeCardinality(Min, Max: Integer): String;

  function ConvertPart(Cardinality: Integer): String;
  begin
    if Cardinality = Unlimited then Result := '*'
    else Result := IntToStr(Cardinality)
  end;

begin
  Result := ConvertPart(Min) + '..' + ConvertPart(Max);
end;

function GetTaggedValue(EAEntity: OleVariant; TaggedValueName: String; Value:
  Variant): Boolean;
var
  I: Integer;
  TaggedValues: OLEVariant;
begin
  TaggedValues := EAEntity.TaggedValues;
  Result := False;
  for I := 0 to TaggedValues.Count - 1 do
    if TaggedValues.GetAt(I).Name = TaggedValueName then
    begin
      Result := True;
      Value := TaggedValues.GetAt(I).Value;
    end
end;

{{
   :   ()
}
procedure ParseDomainType(TypeDef: String; var TypeName: String; var TypeSize:
  Integer);
var
  PL, PR: Integer;
begin
  PL := Pos('(', TypeDef);
  if PL > 0 then begin
    PR := Pos(')', Copy(TypeDef, PL+1, Length(TypeDef)));
    if PR > 0 then begin
      TypeName := Trim(Copy(TypeDef, 1, PL-1));
      TypeSize := StrToInt(Trim(Copy(TypeDef, PL+1, PR-1)));
    end else
      raise Exception.Create('    : ' + TypeDef);
  end else begin
    TypeName := TypeDef;
    TypeSize := 0;
  end;
end;

{
***************************** TovEAAdapter ******************************
}
function TovEAAdapter.GetEAApplication: TApp;
begin
  if Not Assigned(FEAApplication) then
  try
    FEAApplication := TApp.Create(Self);
    FEAApplication.Connect;
  except
    FreeAndNil(FEAApplication);
    MessageDlg('Enterprise Architect  ', mtWarning, [mbOK], 0); Abort;
  end;

  Result := FEAApplication;
end;

function TovEAAdapter.GetEARepository: OLEVariant;
begin
  if VarIsEmpty(FEARepository) then
    FEARepository := EAApplication.Repository;
  Result := FEARepository;
end;

function TovEAAdapter.GetCurrentPackage: OLEVariant;
begin
  if FPackageId = 0 then FPackageId := SelectPackage(Self);
  if FPackageId = 0 then
    raise Exception.Create('     ');
  Result := EARepository.GetPackageById(FPackageId);
end;

function TovEAAdapter.FindClass(APackage: OleVariant; AClassName: String; var
  AClass: OleVariant): Integer;
var
  Element: OLEVariant;
  I: Integer;
begin
  for I := 0 to APackage.Elements.Count-1 do
  begin
    Element := APackage.Elements.GetAt(I);
    if (Element.Type = 'Class') and
       (Element.Name = AClassName) then
    begin
      AClass := Element;
      Result := I;
      Exit;
    end;
  end;
  Result := -1;
end;

function TovEAAdapter.GetClass(APackage: OleVariant; AClassName: String):
  OLEVariant;
var
  Index: Integer;
begin
  Index := FindClass(APackage, AClassName, Result);
  if Index < 0 then
    raise Exception.CreateFmt(' %s  ', [AClassName]);
end;

function TovEAAdapter.FindAttribute(AClass: OleVariant; AttributeName: String;
  var Attribute: OleVariant): Integer;
var
  I: Integer;
begin
  for I := 0 to AClass.Attributes.Count-1 do
  begin
    Attribute := AClass.Attributes.GetAt(I);
    if Attribute.Name = AttributeName then
    begin
      Result := I;
      Exit;
    end;
  end;
  Result := -1;
end;

function TovEAAdapter.GetAttribute(AClass: OleVariant; AttributeName: String):
  OLEVariant;
var
  Index: Integer;
begin
  Index := FindAttribute(AClass, AttributeName, Result);
  if Index < 0 then
    raise Exception.CreateFmt(' %s  ', [AttributeName]);
end;

function TovEAAdapter.FindGeneralization(AClass: OleVariant; AParentClassName: 
  String; var Generalization: OleVariant): Integer;
var
  Connector: OLEVariant;
  I: Integer;
begin
  for I := 0 to AClass.Connectors.Count-1 do
  begin
    Connector := AClass.Connectors.GetAt(I);
    if (Connector.Type = 'Generalization') and
       (EARepository.GetElementById(Connector.SupplierId).Name = AParentClassName)
    then begin
      Generalization := Connector;
      Result := I;
      Exit;
    end;
  end;
  Result := -1;
end;

//*********************************   ****************************

procedure TovEAAdapter.LoadModel(Model: TDomainModel);
begin
  if VarIsEmpty(CurrentPackage) then
    raise Exception.Create('      ');
  LoadObjects(Model);
  LoadRelations(Model);
end;

procedure TovEAAdapter.LoadObjects(Model: TDomainModel);
var
  Elements: OLEVariant;
  I: Integer;
begin
  Elements := CurrentPackage.Elements;
  for I := 0 to Elements.Count-1 do
    if Elements.GetAt(I).Type = 'Class' then
    try
      LoadObject(Model, Elements.GetAt(I));
    except
      on E: Exception do
        Application.MessageBox(PChar(E.Message), '  ', MB_ICONERROR or MB_OK);
    end;
end;

function TovEAAdapter.LoadObject(Model: TDomainModel; EAObject: OleVariant):
  TModelObject;
var
  Value: Variant;
begin
  Result := Model.AddObject(EAObject.Name, NameSpace);
  if GetTaggedValue(EAObject, 'TableName', Value)
    then Result.TableName := Value;
  if GetTaggedValue(EAObject, 'IsAbstract', Value)
    then Result.IsAbstract := Value='True';
  Result.Description := EAObject.Notes;
  LoadAttributes(Result, EAObject);
end;

procedure TovEAAdapter.LoadAttributes(ModelObject: TModelObject; EAObject:
  OleVariant);
var
  TypeName: String;
  Value: Variant;
  Attribute: TModelAttribute;
  EAAttribute: OLEVariant;
  I, TypeSize: Integer;
begin
  for I := 0 to EAObject.Attributes.Count - 1 do
  begin
    EAAttribute := EAObject.Attributes.GetAt(I);
    ParseDomainType(EAAttribute.Type, TypeName, TypeSize);
    if (TypeSize = 0) and (EAAttribute.Length <> '')
      then TypeSize := StrToInt(EAAttribute.Length);
    Attribute := ModelObject.AddAttribute(EAAttribute.Name, TypeName, TypeSize);
    if GetTaggedValue(EAAttribute, 'FieldName', Value)
      then Attribute.FieldName := Value;
    Attribute.Description := EAAttribute.Notes;
  end
end;

procedure TovEAAdapter.LoadRelations(Model: TDomainModel);
var
  EAObject: OLEVariant;
  I, J: Integer;
  Connector: OleVariant;
begin
  for I := 0 to CurrentPackage.Elements.Count-1 do
  begin
    EAObject := CurrentPackage.Elements.GetAt(I);
    if EAObject.Type = 'Class' then
      for J := 0 to EAObject.Connectors.Count - 1 do
      try
        Connector := EAObject.Connectors.GetAt(J);
        if ((Connector.Type = 'Aggregation') or (Connector.Type = 'Association'))
          then LoadRelation(Model, EAObject, Connector) else
        if (Connector.Type = 'Generalization') and
           (Connector.SupplierId {Target} <> EAObject.ElementId)
          then LoadGeneralization(Model, EAObject, Connector);
      except
        on E: Exception do
          Application.MessageBox(PChar(E.Message), '  ', MB_ICONERROR or MB_OK);
      end;
  end
end;

{{
   ,   Connector 
SourceObject    EA
}
function TovEAAdapter.LoadRelation(Model: TDomainModel; EAObject, EAConnector:
  OleVariant): TModelRelation;
//   EA: Client -   ,  Supplier -
//  ,         ,
//   .
var
  TargetId: Integer;
  SourceObjectName, TargetObjectName: String;
  ModelObject: TModelObject;
  MinCount, MaxCount: Integer;
  RelationType: TRelationType;
  SourceEnd, TargetEnd, TargetObject: OLEVariant;
  RelationName: String;
begin
  Result := nil;
  if EAConnector.ClientId = EAObject.ElementId then begin
    SourceEnd := EAConnector.ClientEnd;
    TargetEnd := EAConnector.SupplierEnd;
    TargetId := EAConnector.SupplierId;
  end else begin
    SourceEnd := EAConnector.SupplierEnd;
    TargetEnd := EAConnector.ClientEnd;
    TargetId := EAConnector.ClientId;
  end;
  // ,        EAObject
  TargetObject := EARepository.GetElementById(TargetId);

  if ((TargetEnd.Aggregation = 1) or (TargetEnd.Aggregation = 2)) and
     // shared or composite aggregation
     (TargetId <> EAObject.ElementId)
  //  EAObject     
  then Exit;

  if Not (((SourceEnd.Aggregation = 1) or (SourceEnd.Aggregation = 2))) then
    //   ,    
    // (..     )
    if ((EAConnector.Direction = 'Source -> Destination') and
        (EAConnector.ClientId <> EAObject.ElementId) or
        (EAConnector.Direction = 'Destination -> Source') and
        (EAConnector.SupplierId <> EAObject.ElementId))
    //          
    then Exit;

  if SourceEnd.Aggregation = 2 then
    RelationType := rtComposition
  else
    RelationType := rtAggregation;
  RelationName := EAConnector.Name;
  if TargetEnd.Role <> '' then RelationName := TargetEnd.Role;
  if RelationName = '' then RelationName := TargetObject.Name;

  SourceObjectName := EAObject.Name;
  ModelObject := Model.GetObject(SourceObjectName);

  TargetObjectName := TargetObject.Name;
  if Model.FindObject(TargetObjectName) = nil then
    //      (    )
    // -      
    if ImportExternalClasses then
      LoadObject(Model, TargetObject)
    else
      Exit;

  Result := ModelObject.AddRelation(RelationName, TargetObjectName, RelationType);

  if TargetEnd.Cardinality = '' then begin
    MinCount := 0;
    MaxCount := 1;
  end else
    ParseCardinality(TargetEnd.Cardinality, MinCount, MaxCount);
  Result.TargetMinCount := MinCount;
  Result.TargetMaxCount := MaxCount;
  Result.Description := EAConnector.Notes;
end;

function TovEAAdapter.LoadGeneralization(Model: TDomainModel; EAObject, 
  EAConnector: OleVariant): TModelRelation;
var
  TargetObject: OLEVariant;
  SourceObjectName, TargetObjectName: String;
  ModelObject: TModelObject;
begin
  Result := nil;

  // ,        EAObject
  TargetObject := EARepository.GetElementById(EAConnector.SupplierId);
  SourceObjectName := EAObject.Name;
  ModelObject := Model.GetObject(SourceObjectName);
  TargetObjectName := TargetObject.Name;
  if Model.FindObject(TargetObjectName) = nil then
    //      (    )
    // -      
    if ImportExternalClasses then
      LoadObject(Model, TargetObject)
    else
      Exit;

  Result := ModelObject.AddGeneralization(TargetObjectName);
end;

//*******************************   ****************************

procedure TovEAAdapter.AddClass(SourceModel: TDomainModel; ClassName: String);
var
  I: Integer;
  TargetObject: OLEVariant;
begin
  with SourceModel.GetObject(ClassName) do
  begin
    TargetObject := CurrentPackage.Elements.AddNew(Name, 'Class');
    for I := 0 to AttributeCount - 1 do
      with Attributes[I] do
        if Not IsInherited then
          AddAttributeInternal(Attributes[I], TargetObject);
    TargetObject.Attributes.Refresh;
  end;
end;

procedure TovEAAdapter.AddClassRelations(SourceModel: TDomainModel; ClassName:
  String);
var
  SourceObject: TModelObject;
  TargetObject: OLEVariant;
  I: Integer;
begin
  SourceObject := SourceModel.GetObject(ClassName);
  TargetObject := GetClass(CurrentPackage, ClassName);

  //  
  with SourceObject do
  for I := 0 to RelationCount - 1 do
    if Not Relations[I].IsInherited then
      AddRelationInternal(Relations[I], TargetObject);

  //   
  with SourceObject do
  for I := 0 to ParentCount - 1 do
    AddGeneralizationInternal(SourceObject.ParentRelations[I], TargetObject);
end;

procedure TovEAAdapter.AddAttribute(SourceModel: TDomainModel; ClassName,
  AttributeName: String);
var
  TargetObject: OLEVariant;
  SourceObject: TModelObject;
begin
  SourceObject := SourceModel.GetObject(ClassName);
  TargetObject := GetClass(CurrentPackage, ClassName);
  AddAttributeInternal(SourceObject.GetAttribute(AttributeName), TargetObject);
  TargetObject.Attributes.Refresh;
end;

procedure TovEAAdapter.AddRelation(SourceModel: TDomainModel; ClassName,
  RelationName: String);
var
  TargetObject: OLEVariant;
  SourceObject: TModelObject;
begin
  SourceObject := SourceModel.GetObject(ClassName);
  TargetObject := GetClass(CurrentPackage, ClassName);
  AddRelationInternal(SourceObject.GetRelation(RelationName), TargetObject);
end;

procedure TovEAAdapter.AddGeneralization(SourceModel: TDomainModel; ClassName,
  ParentClassName: String);
var
  TargetObject: OLEVariant;
  SourceObject: TModelObject;
begin
  SourceObject := SourceModel.GetObject(ClassName);
  TargetObject := GetClass(CurrentPackage, ClassName);
  AddGeneralizationInternal(SourceObject.GetParentRelation(ParentClassName),
    TargetObject);
end;

procedure TovEAAdapter.DeleteClass(ClassName: String);
var
  AClass: OLEVariant;
  Index: Integer;
begin
  Index := FindClass(CurrentPackage, ClassName, AClass);
  if Index >= 0 then CurrentPackage.Elements.Delete(Index)
end;

procedure TovEAAdapter.DeleteAttribute(ClassName, AttributeName: String);
var
  AClass, Attribute: OLEVariant;
  Index: Integer;
begin
  AClass := GetClass(CurrentPackage, ClassName);
  Index := FindAttribute(AClass, AttributeName, Attribute);
  if Index >= 0 then AClass.Attributes.Delete(Index)
end;

{{
 , ..  ,      
}
procedure TovEAAdapter.DeleteRelation(ClassName, RelationName: String);
begin
end;

procedure TovEAAdapter.DeleteGeneralization(ClassName, ParentClassName: String);
var
  AClass, Generalization: OLEVariant;
  Index: Integer;
begin
  AClass := GetClass(CurrentPackage, ClassName);
  Index := FindGeneralization(AClass, ParentClassName, Generalization);
  if Index >= 0 then AClass.Connectors.Delete(Index);
end;

procedure TovEAAdapter.UpdateAttribute(SourceModel: TDomainModel; ClassName,
  AttributeName: String);
var
  SourceAttribute: TModelAttribute;
  TargetObject, TargetAttribute: OLEVariant;
  SourceObject: TModelObject;
begin
  SourceObject := SourceModel.GetObject(ClassName);
  SourceAttribute := SourceObject.GetAttribute(AttributeName);
  TargetObject := GetClass(CurrentPackage, ClassName);
  TargetAttribute := GetAttribute(TargetObject, AttributeName);
  TargetAttribute.Type := SourceAttribute.TypeName;
  if SourceAttribute.Size > 0 then
    TargetAttribute.Length := IntToStr(SourceAttribute.Size)
  else
    TargetAttribute.Length := '';
end;

{{
 , ..  ,      
}
procedure TovEAAdapter.UpdateRelation(SourceModel: TDomainModel; ClassName,
  RelationName: String);
begin
end;

{{
   , ..     
TovUpdatableModelStorage.
}
procedure TovEAAdapter.SaveModel(Model: TDomainModel; SaveAllObjects: Boolean = 
  False);
begin
end;

{{
   TargetObject   SourceAttribute
}
procedure TovEAAdapter.AddAttributeInternal(SourceAttribute: TModelAttribute; 
  TargetObject: OleVariant);
var
  TargetAttribute: OLEVariant;
begin
  with SourceAttribute do
  begin
    TargetAttribute := TargetObject.Attributes.AddNew(Name, TypeName);
    if Size > 0 then TargetAttribute.Length := IntToStr(Size);
    TargetAttribute.Update;
  end
end;

{{
   TargetObject   SourceRelation
}
procedure TovEAAdapter.AddRelationInternal(SourceRelation: TModelRelation;
  TargetObject: OleVariant);
var
  Connector: OLEVariant;
  RelationTarget: OLEVariant;
begin
  //  ,     
  if FindClass(CurrentPackage, SourceRelation.TargetObjectName, RelationTarget) >= 0 then
  begin
    Connector := TargetObject.Connectors.AddNew('', 'Association');
    Connector.SupplierId := RelationTarget.ElementId;
    Connector.SupplierEnd.Role := SourceRelation.Name;
    Connector.SupplierEnd.Cardinality :=
      MakeCardinality(SourceRelation.TargetMinCount, SourceRelation.TargetMaxCount);
    if SourceRelation.IsComposition then
      Connector.ClientEnd.Aggregation := 2;
    Connector.Direction := 'Source -> Destination';
    Connector.Update;
    TargetObject.Connectors.Refresh;
  end;
end;

{{
   TargetObject    SourceRelation
}
procedure TovEAAdapter.AddGeneralizationInternal(SourceRelation: TModelRelation;
  TargetObject: OleVariant);
var
  Connector: OLEVariant;
  RelationTarget: OLEVariant;
begin
  //  ,     
  if FindClass(CurrentPackage, SourceRelation.TargetObjectName, RelationTarget) >= 0 then
  begin
    Connector := TargetObject.Connectors.AddNew('', 'Generalization');
    Connector.SupplierId := RelationTarget.ElementId;
    Connector.Direction := 'Source -> Destination';
    Connector.Update;
    TargetObject.Connectors.Refresh;
  end;
end;


end.
