unit ovEAPackages;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, ovEAAdapter, ImgList;

type
  TfmSelectPackage = class(TForm)
    PackageTree: TTreeView;
    Panel: TPanel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    ImageList: TImageList;
    procedure PackageTreeDblClick(Sender: TObject);
  private
    function AddTreeNode(ParentNode: TTreeNode; NodeText: String; NodeId: Integer): 
      TTreeNode;
    { Private declarations }
  protected
    procedure PopulatePackageTree(EARepository: OleVariant);
  public
    function SelectedPackage: Integer;
    { Public declarations }
  end;

function SelectPackage(EAAdapter: TovEAAdapter): Integer;

implementation

uses Math;

{$R *.dfm}

function SelectPackage(EAAdapter: TovEAAdapter): Integer;
begin
  with TfmSelectPackage.Create(Application) do
  try
    PopulatePackageTree(EAAdapter.EARepository);
    if ShowModal = mrOk then
      Result := SelectedPackage
    else
      Result := 0
  finally
    Free
  end;
end;

{ TfmSelectPackage }

procedure TfmSelectPackage.PopulatePackageTree(EARepository: OleVariant);

  procedure PopulatePackages(ParentNode: TTreeNode; ParentPackage: OleVariant);
  var
    Node: TTreeNode;
    I: Integer;
  begin
    Node := AddTreeNode(ParentNode, ParentPackage.Name, ParentPackage.PackageId);
    for I := 0 to ParentPackage.Packages.Count-1 do
      PopulatePackages(Node, ParentPackage.Packages.GetAt(I));
  end;

var
  I: Integer;
begin
  for I := 0 to EARepository.Models.Count-1 do
    PopulatePackages(nil, EARepository.Models.GetAt(I));
  PackageTree.FullExpand;
end;

function TfmSelectPackage.AddTreeNode(ParentNode: TTreeNode; NodeText: String; 
  NodeId: Integer): TTreeNode;
var
  ImageIndex: Integer;
begin
  if ParentNode = nil then
    Result := PackageTree.Items.Add(nil, NodeText)
  else
    Result := PackageTree.Items.AddChild(ParentNode, NodeText);

  ImageIndex := Min(Result.Level, 2);
  Result.ImageIndex := ImageIndex;
  Result.SelectedIndex := ImageIndex;
  Result.Data := Pointer(NodeId);
end;

function TfmSelectPackage.SelectedPackage: Integer;
begin
  if PackageTree.Selected <> nil then
    Result := Integer(PackageTree.Selected.Data)
  else
    Result := 0;
end;

procedure TfmSelectPackage.PackageTreeDblClick(Sender: TObject);
begin
  ModalResult := mrOk
end;

end.
