unit ovEasyDBAdapter;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, EasyTable, ovTypes, ovModel, ovComponents;

type
  TovEasyDBAdapter = class(TovCustomDatabaseAdapter)
  private
    FDatabase: TEasyDatabase;
    FTransactionCount: Integer;
    procedure RestructureTable(Table: TEasyTable; NewFieldsDefs: TFieldDefs;
      NewIndexDefs: TIndexDefs);
    procedure SetDatabase(Value: TEasyDatabase);
    procedure CheckDatabase;
    procedure GenerateDatabaseName(Database: TEasyDatabase);
  protected
    procedure CloseDatabase; override;
    procedure CommitTransaction; override;
    procedure CreateTable(ATableName: String; AFieldsDefs: TFieldDefs;
      AnIndexDefs: TIndexDefs); override;
    procedure ExecQuery(SQLText: String); override;
    procedure OpenDatabase; override;
    function OpenQuery(QueryText: String; MasterDataSource: TDataSource; 
      LiveDataSet: Boolean): TDataSet; override;
    function OpenTable(ATableName: String; AReadOnly: Boolean; AFilter: String):
      TDataSet; override;
    procedure RollbackTransaction; override;
    procedure StartTransaction; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function TableExists(ATableName: String): Boolean; override;
    function InTransaction: Boolean; override;
  public
    procedure SetupConnection(ConnectionParams: TStrings); override;
    constructor Create(AOwner: TComponent); override;
  published
    property Database: TEasyDatabase read FDatabase write SetDatabase;
  end;

procedure Register;

implementation

uses ovConsts, ovObjects;

//{$R *.dcr}

procedure Register;
begin
  RegisterComponents(cpObjectViewAddOn, [TovEasyDBAdapter]);
end;

var DatabaseCount: Integer = 0;

{
**************************** TovAbsoluteDBAdapter ******************************
}
{<<Category>>: Undefined}
constructor TovEasyDBAdapter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  with QuerySyntaxParams do begin
    LeftTableDelimiter := '[';
    RightTableDelimiter := ']';
    LeftFieldDelimiter := '[';
    RightFieldDelimiter := ']';
  end;
end;

procedure TovEasyDBAdapter.CloseDatabase;
begin
  if Assigned(Database) then Database.Close;
end;

procedure TovEasyDBAdapter.CommitTransaction;
begin
  Dec(FTransactionCount)
end;

function TovEasyDBAdapter.InTransaction: Boolean;
begin
  Result := FTransactionCount > 0
end;

procedure TovEasyDBAdapter.CreateTable(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs);
var
  Table: TEasyTable;
begin
  CheckDatabase;

  Table := TEasyTable.Create(nil);
  with Table do
  try
    DatabaseName := Self.Database.DatabaseName;
    TableName := ATableName;
    FieldDefs.Assign(AFieldsDefs);
    if AnIndexDefs <> nil
      then IndexDefs.Assign(AnIndexDefs);
    if Not Exists
      then CreateTable
      else Self.RestructureTable(Table, AFieldsDefs, AnIndexDefs);
  finally
    Free
  end;
end;

procedure TovEasyDBAdapter.ExecQuery(SQLText: String);
begin
  CheckDatabase;

  with TEasyQuery.Create(nil) do
  try
    DatabaseName := Self.Database.DatabaseName;
    SQL.Text := SQLText;
    ExecSQL;
  finally
    Free
  end;
end;

procedure TovEasyDBAdapter.OpenDatabase;
begin
  CheckDatabase;
  Database.Open;
end;

function TovEasyDBAdapter.OpenQuery(QueryText: String; MasterDataSource: 
  TDataSource; LiveDataSet: Boolean): TDataSet;
begin
  CheckDatabase;

  Result := TEasyQuery.Create(nil);
  try
    with TEasyQuery(Result) do begin
      DatabaseName := Self.Database.DatabaseName;
      SQL.Text := QueryText;
      if MasterDataSource <> nil then
        DataSource := MasterDataSource;
      RequestLive := LiveDataSet;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

function TovEasyDBAdapter.OpenTable(ATableName: String; AReadOnly: Boolean;
  AFilter: String): TDataSet;
begin
  CheckDatabase;

  Result := TEasyTable.Create(nil);
  try
    with TEasyTable(Result) do begin
      DatabaseName := Self.Database.DatabaseName;
      TableName := ATableName;
      ReadOnly := AReadOnly;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

procedure TovEasyDBAdapter.RestructureTable(Table: TEasyTable; NewFieldsDefs: 
  TFieldDefs; NewIndexDefs: TIndexDefs);
var
  I, J, N: Integer;
begin (*
  with Table do begin

    //    
    N := RestructureFieldDefs.Count;
    for I := 0 to NewFieldsDefs.Count - 1 do
    with NewFieldsDefs[I] do begin
      J := RestructureFieldDefs.IndexOf(Name);
      if J < 0 then begin //   
        Inc(N);
        RestructureFieldDefs.Add(N, Name, DataType, Size, Required);
      end else begin      //    
        RestructureFieldDefs[J].DataType := DataType;
        RestructureFieldDefs[J].Size := Size;
        RestructureFieldDefs[J].Required := Required;
      end
    end;

    //   
    RestructureIndexDefs.Clear;
    for I := 0 to NewIndexDefs.Count - 1 do
      with NewIndexDefs[I] do
        RestructureIndexDefs.Add(Name, Fields, Options);

    //   
    RestructureTable(Self.LanguageID);
  end   *)
end;

procedure TovEasyDBAdapter.RollbackTransaction;
begin
  Dec(FTransactionCount)
end;

procedure TovEasyDBAdapter.SetDatabase(Value: TEasyDatabase);
begin
  if FDatabase <> Value then
  begin
    FDatabase := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovEasyDBAdapter.StartTransaction;
begin
  Inc(FTransactionCount)
end;

procedure TovEasyDBAdapter.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = Database) and (Operation = opRemove)
    then Database := nil;
end;

procedure TovEasyDBAdapter.CheckDatabase;
begin
  if Not Assigned(Database) then
    raise Exception.Create('   Database : ' + Name);
  if Database.DatabaseName = '' then
    raise Exception.Create('      : ' + Name);
end;

function TovEasyDBAdapter.TableExists(ATableName: String): Boolean;
var
  Table: TEasyTable;
begin
  Table := TEasyTable.Create(nil);
  with Table do
  try
    DatabaseName := Self.Database.DatabaseName;
    TableName := ATableName;
    Result := Exists;
  finally
    Free
  end;
end;

procedure TovEasyDBAdapter.GenerateDatabaseName(Database: TEasyDatabase);
var
  DatabaseCount: Integer;
begin
  DatabaseCount := 1;
  while True do
  try
    Inc(DatabaseCount);
    Database.DatabaseName := 'Database_' + IntToStr(DatabaseCount);
    Break;
  except
    //     DatabaseName   
  end;
end;

procedure TovEasyDBAdapter.SetupConnection(ConnectionParams: TStrings);
begin
{    ,    - .    DBISAM'e
      SessionName  Table  Query.

  Session := TABSSession.Create(Self);
  if ConnectionParams.Values['SessionName'] = ''
    then Session.AutoSessionName := True
    else Session.SessionName := ConnectionParams.Values['SessionName'];
}
  Database := TEasyDatabase.Create(Self);
//  Database.SessionName := Session.SessionName;
  if ConnectionParams.Values['DatabaseName'] = ''
    then GenerateDatabaseName(Database)
    else Database.DatabaseName := ConnectionParams.Values['DatabaseName'];

  Database.DatabaseFileName := ConnectionParams.Values['DatabaseFileName'];
end;

initialization
  //      ,      TovDatabaseConnection
  RegisterClass(TovEasyDBAdapter);
finalization
end.
