 
{
  Interbase/Firebird  ADO.
}
{$INCLUDE oView.inc}
unit ovIBADOAdapter;
interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, ADODB, ovTypes, ovModel, ovADOAdapter;

type
  TovInterbaseADOAdapter = class(TovADOAdapter)
  private
    FAlwaysRebuildTable: Boolean;
    FCreateSimpleTable: Boolean;
    procedure ExecAlterTableQuery(TableName: String; NewFields, ChangedFields:
      TStrings);
    procedure RestructureTableInternal(ATableName: String; ATable: TDataSet;
        AFieldsDefs: TFieldDefs; AnIndexDefs: TIndexDefs);
  protected
    procedure CloneTable(SourceTable: TDataSet; TableName: String; FieldsDefs:
      TFieldDefs; IndexDefs: TIndexDefs); override;
    procedure CreateTable(ATableName: String; AFieldsDefs: TFieldDefs; AnIndexDefs:
      TIndexDefs; ASimpleTable, AlwaysRebuild: Boolean);
    procedure GetCreateTableQuery(ATableName: String; AFieldsDefs: TFieldDefs;
        AnIndexDefs: TIndexDefs; AQuery: TStrings); override;
    function RestructureTable(ATableName: String; AFieldsDefs: TFieldDefs;
        AnIndexDefs: TIndexDefs): Boolean; override;
    function TableExists(ATableName: String): Boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

procedure Register;

implementation

//{$R *.dcr}

uses TypInfo, ovConsts, ovObjects, AdoConEd, ovUtils;

procedure Register;
begin
  RegisterComponents(cpObjectViewAddOn, [TovInterbaseADOAdapter]);
end;

function GetSQLFieldType(FieldType: TFieldType): string;
const
{$IFNDEF Delphi2006}
  SQLFieldTypes: array [TFieldType] of String =
   ({ftUnknown}'Unknown', {ftString}'VARCHAR', {ftSmallint}'SMALLINT',
    {ftInteger}'INTEGER', {ftWord}'INTEGER', {ftBoolean}'BOOLEAN',
    {ftFloat}'DOUBLE PRECISION', {ftCurrency}'Currency', {ftBCD}'NUMERIC',
    {ftDate}'TIMESTAMP', {ftTime}'TIME', {ftDateTime}'TIMESTAMP',
    {ftBytes}'Bytes', {ftVarBytes}'VarBytes', {ftAutoInc}'AutoInc',
    {ftBlob}'BLOB SUB_TYPE 0', {ftMemo}'BLOB SUB_TYPE 0',
    {ftGraphic}'BLOB SUB_TYPE 0', {ftFmtMemo}'BLOB SUB_TYPE 0',
    {ftParadoxOle}'ParadoxOle', {ftDBaseOle}'DBaseOle',
    {ftTypedBinary}'TypedBinary',
    {ftCursor}'Cursor', {ftFixedChar}'CHAR', {ftWideString}'VARCHAR',
    {ftLargeint}'INTEGER', {ftADT}'ADT', {ftArray}'Array',
    {ftReference}'Reference', {ftDataSet}'DataSet', {ftOraBlob}'OraBlob',
    {ftOraClob}'OraBlob', {ftVariant}'Variant', {ftInterface}'Interface',
    {ftIDispatch}'IDispatch', {ftGuid}'Guid', {ftTimeStamp}'TIMESTAMP',
    {ftFMTBcd}'NUMERIC');
{$ELSE}
   ...to-do
{$ENDIF}
begin
  Result := SQLFieldTypes[FieldType];
  if Result = '' then
    raise Exception.Create(' :' +
      GetEnumName(TypeInfo(TFieldType), Integer(FieldType)));
end;

{{
     SQL-.
}
function GetSQLFieldDef(AFieldName: String; AFieldType: TDataType; AFieldSize:
  Integer): String;
begin
  Result := Format('[%s] %s', [AFieldName, GetSQLFieldType(AFieldType)]);
  if AFieldType in [ftString] then
    Result := Result + Format('(%d)', [AFieldSize]);
end;

{{
   ,   
.
}
function GetPrimaryKeyFieldDef(FieldDefs: TFieldDefs): TFieldDef;
var
  I: Integer;
begin
  for I := 0 to FieldDefs.Count - 1 do
    if FieldDefs[I].DataType = ftAutoInc then
    begin
      Result := FieldDefs[I];
      Exit;
    end;
  raise Exception.Create('    ');
end;

{
******************************* TovInterbaseADOAdapter ***********************
}

resourcestring
  SCreateIndexSQL =
    'CREATE INDEX %s ON %s (%s);';
  SCreateTableSQL =
    'CREATE TABLE [%s] ('+
    '[%s] int IDENTITY(1,1) NOT NULL, '+
    '%s, '+
    'CONSTRAINT [%s] PRIMARY KEY ([%s])'+
    ');';
   SCreateSimpleTableSQL =
    'CREATE TABLE [%s] ('+
    '[%s] int NOT NULL, '+
    '%s, '+
    ');';
   SAlterTableSQL = 'ALTER TABLE [%s] ';
   SDeleteTableSQL = 'DROP TABLE [%s]';
   SCopyTableSQL =
    'SET IDENTITY_INSERT [%0:s] ON '#13+
    'INSERT INTO [%0:s] (%1:s) '#13+
    'SELECT %1:s From [%2:s] '#13+
    'SET IDENTITY_INSERT [%0:s] OFF ';

constructor TovInterbaseADOAdapter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  with QuerySyntaxParams do begin
    LeftTableDelimiter := '';
    RightTableDelimiter := '';
    LeftFieldDelimiter := '';
    RightFieldDelimiter := '';
  end;
end;

function TovInterbaseADOAdapter.TableExists(ATableName: String): Boolean;
var
  Query: TDataSet;
begin
  try
    //   ,   
    Query := OpenQuery(Format('SELECT TOP 1 * FROM [%s]', [ATableName]));
    Query.Free;
    Result := True;
  except
    Result := False
  end;
end;

{
*******************************   ********************************
}

{{
  .    ASimpleTable,   
    IDENTITY .    AlwaysRebuild  
  ,     ,    .
}
procedure TovInterbaseADOAdapter.CreateTable(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs; ASimpleTable, AlwaysRebuild: Boolean);
begin
  FCreateSimpleTable := ASimpleTable;
  FAlwaysRebuildTable := AlwaysRebuild;
  try
    inherited CreateTable(ATableName, AFieldsDefs, AnIndexDefs);
  finally
    FCreateSimpleTable := False;
    FAlwaysRebuildTable := False;
  end;
end;

function TovInterbaseADOAdapter.RestructureTable(ATableName: String;
    AFieldsDefs: TFieldDefs; AnIndexDefs: TIndexDefs): Boolean;
var
  Table: TDataSet;
begin
  try
    Table := OpenTable(ATableName, True);
    if FAlwaysRebuildTable then begin
      ExecQuery(Format(SDeleteTableSQL, [ATableName]));
      Result := False;
      Exit;
    end;

    try
      //   -  
      RestructureTableInternal(ATableName, Table, AFieldsDefs, AnIndexDefs)
    finally
      Table.Free
    end;
    //  
    Result := True;
  except
    //    -   
    Result := False;
    Exit;
  end;
end;

procedure TovInterbaseADOAdapter.RestructureTableInternal(ATableName: String;
    ATable: TDataSet; AFieldsDefs: TFieldDefs; AnIndexDefs: TIndexDefs);
var
  I: Integer;
  NewFields, ChangedFields: TStringList;
  Field: TField;
begin
  NewFields := TStringList.Create;
  ChangedFields := TStringList.Create;
  try
    //      
    for I := 0 to AFieldsDefs.Count - 1 do
    begin
      Field := ATable.FindField(AFieldsDefs[I].Name);
      if Not Assigned(Field) then
        with AFieldsDefs[I] do
        NewFields.Add(GetSQLFieldDef(Name, DataType, Size))
      else
      if (GetSQLFieldType(Field.DataType) <> GetSQLFieldType(AFieldsDefs[I].DataType)) or
        (Field.Size <> AFieldsDefs[I].Size)
      then
        with AFieldsDefs[I] do
        ChangedFields.Add(GetSQLFieldDef(Name, DataType, Size))
    end;
    ExecAlterTableQuery(ATableName, NewFields, ChangedFields);
  finally
    ChangedFields.Free;
    NewFields.Free;
  end;
end;

procedure TovInterbaseADOAdapter.GetCreateTableQuery(ATableName: String;
    AFieldsDefs: TFieldDefs; AnIndexDefs: TIndexDefs; AQuery: TStrings);

  {{
         (
   !).
  }
  function GetFieldListSQL: String;
  var
    I: Integer;
  begin
    Result := '';
    for I := 1 to AFieldsDefs.Count - 1 do
    begin
      with AFieldsDefs[I] do
        Result := Result + GetSQLFieldDef(Name, DataType, Size);
      if I < AFieldsDefs.Count - 1 then
        Result := Result + ', ';
    end;
  end;

  {{
     .
  }
  function GetCreateIndexQuery: String;
  var
    FieldList, IndexName: String;
  begin
    // ,     AnIndexDefs   
    IndexName := 'IDX_' + ATableName;
    //  
    FieldList := StringReplace(AnIndexDefs[1].Fields, ';', ', ', [rfReplaceAll]);
    Result := Format(SCreateIndexSQL, [IndexName, ATableName, FieldList])
  end;

var
  FieldList, PrimaryKeyField: String;
begin
  if Not FCreateSimpleTable then
  begin
    FieldList := GetFieldListSQL;
    // ,     AnIndexDefs   , 
    //      -   
    PrimaryKeyField := AFieldsDefs[0].Name;
    AQuery.Add(Format(SCreateTableSQL, [ATableName, PrimaryKeyField, FieldList,
      ATableName +'_'+ AnIndexDefs[0].Name, PrimaryKeyField]));
    AQuery.Add(GetCreateIndexQuery);
  end else
  begin
    //    FCreateSimpleTable,     IDENTITY
    //     ,     
    FieldList := GetFieldListSQL;
    PrimaryKeyField := AFieldsDefs[0].Name;
    AQuery.Add(Format(SCreateSimpleTableSQL, [ATableName, PrimaryKeyField, FieldList]));
  end
end;

procedure TovInterbaseADOAdapter.ExecAlterTableQuery(TableName: String;
  NewFields, ChangedFields: TStrings);
var
  I: Integer;
  Query: String;
begin
  for I := 0 to NewFields.Count - 1 do
  begin
    Query := Format(SAlterTableSQL, [TableName]) + 'ADD ' + NewFields[I];
    ExecQuery(Query);
  end;
  for I := 0 to ChangedFields.Count - 1 do
  begin
    Query := Format(SAlterTableSQL, [TableName]) + 'ALTER COLUMN ' + ChangedFields[I];
    ExecQuery(Query);
  end;
end;

{{
    TableName,        
  SourceTable,      .
}
procedure TovInterbaseADOAdapter.CloneTable(SourceTable: TDataSet; TableName:
  String; FieldsDefs: TFieldDefs; IndexDefs: TIndexDefs);

  {{
       SourceTable   DestTable, 
   ,    Identity   DestTable.
  }
  procedure CopyDataToIdentityColumn(SourceTable, DestTable: String);
  var
    FieldList: String;
    I: Integer;
  begin
    FieldList := '';
    for I := 0 to FieldsDefs.Count - 1 do
      FieldList := FieldList + ', [' + FieldsDefs[I].Name + ']';
    Delete(FieldList, 1, 2);
    ExecQuery(Format(SCopyTableSQL, [DestTable, FieldList, SourceTable]));
  end;

var
  DestTable: TDataSet;
  WorkTableName: String;
begin
  //      ,   
  //   IDENTITY 
  WorkTableName := '#' + TableName;
  CreateTable(WorkTableName, FieldsDefs, IndexDefs, True, True);
  try
    CreateTable(TableName, FieldsDefs, IndexDefs, False, True);

    //         
    DestTable := OpenTable(WorkTableName);
    try
      CopyData(SourceTable, DestTable);
    finally
      DestTable.Free;
    end;

    //        
    CopyDataToIdentityColumn(WorkTableName, TableName);

  finally
    ExecQuery(Format(SDeleteTableSQL, [WorkTableName]));
  end;
end;

initialization
  TovInterbaseADOAdapter.RegisterAdapter(TovInterbaseADOAdapter);
finalization
  TovInterbaseADOAdapter.UnregisterAdapter(TovInterbaseADOAdapter);
end.
