{
  Interbase/Firebird    BDE.

   .     
  .
}
unit ovIBBDEAdapter;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBTables, ovTypes, ovModel, ovComponents;

type
  TovIBBDEAdapter = class(TovCustomDatabaseAdapter)
  private
    FDatabase: TDatabase;
    procedure SetDatabase(Value: TDatabase);
    procedure CheckDatabase;
    procedure GenerateDatabaseName(Database: TDatabase);
    function GetCreateTableSQL(ATableName: String; AFieldsDefs: TFieldDefs; 
      AnIndexDefs: TIndexDefs): TStrings;
  protected
    procedure CloseDatabase; override;
    procedure CommitTransaction; override;
    procedure CreateTable(ATableName: String; AFieldsDefs: TFieldDefs;
      AnIndexDefs: TIndexDefs); override;
    procedure ExecQuery(SQLText: String); override;
    procedure OpenDatabase; override;
    function OpenQuery(QueryText: String; MasterDataSource: TDataSource; 
      LiveDataSet: Boolean): TDataSet; override;
    function OpenTable(ATableName: String; AReadOnly: Boolean): TDataSet; override;
    procedure RollbackTransaction; override;
    procedure StartTransaction; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function TableExists(ATableName: String): Boolean; override;
    function InTransaction: Boolean; override;
  public
    procedure SetupConnection(ConnectionParams: TStrings); override;
    function EditConnection(ConnectionParams: TStrings): Boolean; override;
    constructor Create(AOwner: TComponent); override;
  published
    property Database: TDatabase read FDatabase write SetDatabase;
  end;

procedure Register;

implementation

{$R ovBDEAdapter.dcr}

uses ovConsts, ovObjects, ovBDEParams;

procedure Register;
begin
  RegisterComponents(cpObjectViewAddOn, [TovIBBDEAdapter]);
end;

var DatabaseCount: Integer = 0;

{
******************************* TovBDEAdapter *******************************
}
constructor TovIBBDEAdapter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  with QuerySyntaxParams do begin
    LeftTableDelimiter := '';
    RightTableDelimiter := '';
    LeftFieldDelimiter := '';
    RightFieldDelimiter := '';
  end;
end;

procedure TovIBBDEAdapter.CloseDatabase;
begin
  if Assigned(Database) then Database.Close;
end;

procedure TovIBBDEAdapter.StartTransaction;
begin
  CheckDatabase;
  //      TransIsolation  tiDirtyRead
  if Not Database.IsSQLBased
    then Database.TransIsolation := tiDirtyRead;
  Database.StartTransaction
end;

procedure TovIBBDEAdapter.RollbackTransaction;
begin
  CheckDatabase;
  Database.Rollback
end;

procedure TovIBBDEAdapter.CommitTransaction;
begin
  CheckDatabase;
  Database.Commit;
end;

function TovIBBDEAdapter.InTransaction: Boolean;
begin
  CheckDatabase;
  Result := Database.InTransaction
end;

procedure TovIBBDEAdapter.CreateTable(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs);
var
  I: Integer;
  SQL: TStrings;
begin
  SQL := GetCreateTableSQL(ATableName, AFieldsDefs, AnIndexDefs);
  try
    for I := 0 to SQL.Count - 1 do
      ExecQuery(SQL[I]);
  finally
    SQL.Free
  end;
end;

function TovIBBDEAdapter.GetCreateTableSQL(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs): TStrings;

const
  SQLTemplate =
    'CREATE TABLE %s (                        '+
    '%s INTEGER NOT NULL,                     '+
    '%s,                                      '+
    'PRIMARY KEY (%s)                         '+
    ');                                       '#13+

    'CREATE GENERATOR %s;                     '#13+

    'CREATE TRIGGER %s FOR %s                 '+
    'ACTIVE BEFORE INSERT POSITION 0 AS       '+
    'BEGIN NEW.%s = GEN_ID(%s, 1);            '+
    'END; ';

  SQLFieldTypes: array [TFieldType] of String =
   ({ftUnknown}'Unknown', {ftString}'VARCHAR', {ftSmallint}'SMALLINT',
    {ftInteger}'INTEGER', {ftWord}'INTEGER', {ftBoolean}'BOOLEAN',
    {ftFloat}'DOUBLE PRECISION', {ftCurrency}'Currency', {ftBCD}'NUMERIC',
    {ftDate}'TIMESTAMP', {ftTime}'TIME', {ftDateTime}'TIMESTAMP',
    {ftBytes}'Bytes', {ftVarBytes}'VarBytes', {ftAutoInc}'AutoInc',
    {ftBlob}'BLOB SUB_TYPE 0', {ftMemo}'BLOB SUB_TYPE 0',
    {ftGraphic}'BLOB SUB_TYPE 0', {ftFmtMemo}'BLOB SUB_TYPE 0',
    {ftParadoxOle}'ParadoxOle', {ftDBaseOle}'DBaseOle',
    {ftTypedBinary}'TypedBinary',
    {ftCursor}'Cursor', {ftFixedChar}'CHAR', {ftWideString}'VARCHAR',
    {ftLargeint}'INTEGER', {ftADT}'ADT', {ftArray}'Array',
    {ftReference}'Reference', {ftDataSet}'DataSet', {ftOraBlob}'OraBlob',
    {ftOraClob}'OraBlob', {ftVariant}'Variant', {ftInterface}'Interface',
    {ftIDispatch}'IDispatch', {ftGuid}'Guid', {ftTimeStamp}'TIMESTAMP',
    {ftFMTBcd}'NUMERIC');

  function GetFieldListSQL: String;
  var
    I: Integer;
  begin
    Result := '';
    for I := 1 to AFieldsDefs.Count - 1 do
    begin
      Result := Result + Format('%s %s', [
        AFieldsDefs[I].Name, SQLFieldTypes[AFieldsDefs[I].DataType]]);
      if AFieldsDefs[I].DataType in [ftString] then
        Result := Result + Format('(%d)', [AFieldsDefs[I].Size]);
      if I < AFieldsDefs.Count - 1 then
        Result := Result + ',';
    end;
  end;

  {{
     .
  }
  function GetCreateIndexSQL: String;
  var
    FieldList, IndexName: String;
  const
    SQLTemplate = 'CREATE INDEX %s ON %s(%s);';
  begin
    IndexName := 'IDX_' + ATableName;
    //  
    FieldList := StringReplace(AnIndexDefs[1].Fields, ';', ', ', [rfReplaceAll]);
    Result := Format(SQLTemplate, [IndexName, ATableName, FieldList])
  end;

var
  FieldList, PrimaryKey, GeneratorName, TriggerName, SQL: String;
begin
  FieldList := GetFieldListSQL;
  PrimaryKey := AFieldsDefs[0].Name;
  GeneratorName := 'GEN_' + ATableName;
  TriggerName := 'TI_' + ATableName;
  SQL := Format(SQLTemplate, [ATableName, PrimaryKey, FieldList, PrimaryKey,
    GeneratorName, TriggerName, ATableName, PrimaryKey, GeneratorName]);
  SQL := SQL + #13 + GetCreateIndexSQL;

  Result := TStringList.Create;
  Result.Text := SQL;
end;

procedure TovIBBDEAdapter.ExecQuery(SQLText: String);
begin
  CheckDatabase;

  with TQuery.Create(nil) do
  try
    DatabaseName := Self.Database.DatabaseName;
    SQL.Text := SQLText;
    ExecSQL;
  finally
    Free
  end;
end;

procedure TovIBBDEAdapter.OpenDatabase;
begin
  CheckDatabase;
  Database.Open;
end;

function TovIBBDEAdapter.OpenQuery(QueryText: String; MasterDataSource: 
  TDataSource; LiveDataSet: Boolean): TDataSet;
begin
  CheckDatabase;

  Result := TQuery.Create(nil);
  try
    with TQuery(Result) do begin
      DatabaseName := Self.Database.DatabaseName;
      SQL.Text := QueryText;
      if MasterDataSource <> nil then
        DataSource := MasterDataSource;
      RequestLive := LiveDataSet;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

function TovIBBDEAdapter.OpenTable(ATableName: String; AReadOnly: Boolean):
  TDataSet;
begin
  CheckDatabase;

  Result := TTable.Create(nil);
  try
    with TTable(Result) do begin
      DatabaseName := Self.Database.DatabaseName;
      TableName := ATableName;
      ReadOnly := AReadOnly;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

procedure TovIBBDEAdapter.SetDatabase(Value: TDatabase);
begin
  if FDatabase <> Value then
  begin
    FDatabase := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovIBBDEAdapter.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = Database) and (Operation = opRemove)
    then Database := nil;
end;

procedure TovIBBDEAdapter.CheckDatabase;
begin
  if Not Assigned(Database) then
    raise Exception.Create('   Database  ' + Name);
end;

function TovIBBDEAdapter.TableExists(ATableName: String): Boolean;
var
  Table: TTable;
begin
  Table := TTable.Create(nil);
  with Table do
  try
    DatabaseName := Self.Database.DatabaseName;
    TableName := ATableName;
    Result := Exists;
  finally
    Free
  end;
end;

procedure TovIBBDEAdapter.GenerateDatabaseName(Database: TDatabase);
var
  DatabaseCount: Integer;
begin
  DatabaseCount := 1;
  while True do
  try
    Inc(DatabaseCount);
    Database.DatabaseName := 'Database_' + IntToStr(DatabaseCount);
    Break;
  except
    //     DatabaseName   
  end;
end;

procedure TovIBBDEAdapter.SetupConnection(ConnectionParams: TStrings);
begin
  Database := TDatabase.Create(Self);
  if ConnectionParams.Values['DatabaseName'] = ''
    then GenerateDatabaseName(Database)
    else Database.DatabaseName := ConnectionParams.Values['DatabaseName'];

  with ConnectionParams do
  begin
    if Values['AliasName'] <> '' then
      Database.AliasName := Values['AliasName'];
    if Values['DriverName'] <> '' then
      Database.DriverName := Values['DriverName'];

    Database.Params := ConnectionParams;
    Database.Params.Values['DatabaseName'] := '';
    Database.Params.Values['AliasName'] := '';
    Database.Params.Values['DriverName'] := '';
    if Values['PATH'] <> '' then ForceDirectories(Values['PATH']);
  end
end;

function TovIBBDEAdapter.EditConnection(ConnectionParams: TStrings): Boolean;
begin
  Result := ShowAdapterParams(ConnectionParams)
end;


initialization
  TovIBBDEAdapter.RegisterAdapter(TovIBBDEAdapter);
finalization
  TovIBBDEAdapter.UnregisterAdapter(TovIBBDEAdapter);
end.
