{
  IBObjects 4.5

   ,      
BDE  Interbase.    .
,       IBObjects
   ,   BDE.
}
unit ovIBOAdapter;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, ovTypes, ovModel, ovComponents,
  IB_Session, IBODataSet;

type
  TovIBObjectsAdapter = class(TovCustomDatabaseAdapter)
  private
    FDatabase: TIBODatabase;
    procedure SetDatabase(Value: TIBODatabase);
    procedure CheckDatabase;
  protected
    procedure CloseDatabase; override;
    procedure CommitTransaction; override;
    procedure CreateTable(ATableName: String; AFieldsDefs: TFieldDefs;
      AnIndexDefs: TIndexDefs); override;
    procedure ExecQuery(SQLText: String); override;
    procedure OpenDatabase; override;
    function OpenQuery(QueryText: String; MasterDataSource: TDataSource;
      LiveDataSet: Boolean): TDataSet; override;
    function OpenTable(ATableName: String; AReadOnly: Boolean): TDataSet; override;
    procedure RollbackTransaction; override;
    procedure StartTransaction; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function TableExists(ATableName: String): Boolean; override;
    function InTransaction: Boolean; override;
  public
    procedure SetupConnection(ConnectionParams: TStrings); override;
    function EditConnection(ConnectionParams: TStrings): Boolean; override;
    constructor Create(AOwner: TComponent); override;
  published
    property Database: TIBODatabase read FDatabase write SetDatabase;
  end;

procedure Register;

implementation

{$R ovBDEAdapter.dcr}

uses ovConsts, ovObjects;

procedure Register;
begin
  RegisterComponents(cpObjectViewAddOn, [TovIBObjectsAdapter]);
end;

var DatabaseCount: Integer = 0;

{
******************************* TovBDEAdapter *******************************
}
constructor TovIBObjectsAdapter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  with QuerySyntaxParams do begin
    LeftTableDelimiter := '';
    RightTableDelimiter := '';
    LeftFieldDelimiter := '';
    RightFieldDelimiter := '';
  end;
end;

procedure TovIBObjectsAdapter.CloseDatabase;
begin
  if Assigned(Database) then Database.Close;
end;

procedure TovIBObjectsAdapter.StartTransaction;
begin
  CheckDatabase;
  Database.StartTransaction
end;

procedure TovIBObjectsAdapter.RollbackTransaction;
begin
  CheckDatabase;
  Database.Rollback
end;

procedure TovIBObjectsAdapter.CommitTransaction;
begin
  CheckDatabase;
  Database.Commit;
end;

function TovIBObjectsAdapter.InTransaction: Boolean;
begin
  CheckDatabase;
  Result := Database.InTransaction
end;

procedure TovIBObjectsAdapter.CreateTable(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs);
begin
end;

procedure TovIBObjectsAdapter.ExecQuery(SQLText: String);
begin
  CheckDatabase;

  with TIBOQuery.Create(nil) do
  try
    DatabaseName := Self.Database.DatabaseName;
    SQL.Text := SQLText;
    ExecSQL;
  finally
    Free
  end;
end;

procedure TovIBObjectsAdapter.OpenDatabase;
begin
  CheckDatabase;
  Database.Open;
end;

function TovIBObjectsAdapter.OpenQuery(QueryText: String; MasterDataSource:
  TDataSource; LiveDataSet: Boolean): TDataSet;
begin
  CheckDatabase;

  Result := TIBOQuery.Create(nil);
  try
    with TIBOQuery(Result) do begin
      DatabaseName := Self.Database.DatabaseName;
      SQL.Text := QueryText;
      if MasterDataSource <> nil then
        DataSource := MasterDataSource;
      RequestLive := LiveDataSet;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

function TovIBObjectsAdapter.OpenTable(ATableName: String; AReadOnly: Boolean):
  TDataSet;
begin
  CheckDatabase;

  Result := TIBOTable.Create(nil);
  try
    with TIBOTable(Result) do begin
      DatabaseName := Self.Database.DatabaseName;
      TableName := ATableName;
      ReadOnly := AReadOnly;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

procedure TovIBObjectsAdapter.SetDatabase(Value: TIBODatabase);
begin
  if FDatabase <> Value then
  begin
    FDatabase := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovIBObjectsAdapter.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = Database) and (Operation = opRemove)
    then Database := nil;
end;

procedure TovIBObjectsAdapter.CheckDatabase;
begin
  if Not Assigned(Database) then
    raise Exception.Create('   Database  ' + Name);
end;

function TovIBObjectsAdapter.TableExists(ATableName: String): Boolean;
var
  Table: TIBOTable;
begin
  Table := TIBOTable.Create(nil);
  with Table do
  try
    DatabaseName := Self.Database.DatabaseName;
    TableName := ATableName;
    Result := Exists;
  finally
    Free
  end;
end;

procedure TovIBObjectsAdapter.SetupConnection(ConnectionParams: TStrings);
begin
  Database := TIBODatabase.Create(Self);
  with ConnectionParams do
  begin
    Database.DatabaseName := 'IBODatabase';
    Database.Protocol := cpLocal;
    Database.Path := Values['Path'];
    Database.LoginPrompt := True;
  end
end;

function TovIBObjectsAdapter.EditConnection(ConnectionParams: TStrings): Boolean;
begin
  Result := False
end;

initialization
  TovIBObjectsAdapter.RegisterAdapter(TovIBObjectsAdapter);
finalization
  TovIBObjectsAdapter.UnregisterAdapter(TovIBObjectsAdapter);
end.
