{
  Interbase/Firebird  IBX.
}
unit ovIBXAdapter;
interface

uses
  SysUtils, Windows, Classes, Forms, DB, Contnrs, ovTypes, ovModel, ovComponents,
  IBDatabase;

type
  TovIBXAdapter = class(TovCustomDatabaseAdapter)
  private
    FDatabase: TIBDatabase;
    FDefaultTransaction: TIBTransaction;
    FInTransaction: Boolean;
    procedure CheckOpened;
    procedure SetDatabase(Value: TIBDatabase);
    procedure DatabaseDisconnectHandler(Sender: TObject);
    function GetCreateTableSQL(ATableName: String; AFieldsDefs: TFieldDefs;
      AnIndexDefs: TIndexDefs): String;
    procedure RestructureTable(ATableName: String; AFieldsDefs: TFieldDefs;
      AnIndexDefs: TIndexDefs);
    procedure ExecAlterTableQuery(TableName: String; NewFields, ChangedFields:
      TObjectList);
    procedure DeleteTable(ATableName: String);
    procedure SafeExecQuery(SQLText: String);
    procedure CreateTableInternal(ATableName: String;
      AFieldsDefs: TFieldDefs; AnIndexDefs: TIndexDefs);
    procedure CreateDatabaseInternal(ADatabaseName, AUserName, APassword: String);
    function GetCreateIndexSQL(ATableName: String; AnIndexDefs: TIndexDefs): String;
    procedure SafeExecQueryList(QueryList: String);
  protected
    function TableExists(ATableName: String): Boolean; override;
    procedure CommitTransaction; override;
    procedure CreateTable(ATableName: String; AFieldsDefs: TFieldDefs; AnIndexDefs:
      TIndexDefs); override;
    procedure ExecQuery(SQLText: String); override;
    function InTransaction: Boolean; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function OpenQuery(QueryText: String; MasterDataSource: TDataSource;
      LiveDataSet: Boolean): TDataSet; override;
    function OpenTable(ATableName: String; AReadOnly: Boolean; AFilter: String =
      ''): TDataSet; override;
    procedure RollbackTransaction; override;
    procedure StartTransaction; override;
    function ActiveTransaction: TIBTransaction;
    procedure CloneTable(SourceTable: TDataSet; SourceTableFieldNames: TStrings;
      TableName: String; FieldsDefs: TFieldDefs; IndexDefs: TIndexDefs); override;
    function GetNewRecordId(ATableName: String): Integer; override;
    procedure SetNewRecordId(ATableName: String; AValue: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    function EditConnection(ConnectionParams: TStrings): Boolean; override;
    procedure SetupConnection(ConnectionParams: TStrings); override;
    procedure CloseDatabase; override;
    procedure OpenDatabase; override;
    procedure CreateDatabase(DatabaseName: String; ConnectionParams: TStrings);
      override;
  published
    property Database: TIBDatabase read FDatabase write SetDatabase;
    property DefaultTransaction: TIBTransaction read FDefaultTransaction write FDefaultTransaction;
  end;

procedure Register;

implementation

//{$R *.dcr}

uses TypInfo, ovConsts, ovObjects, ovUtils, IBTable, IBQuery,
  IBCustomDataSet, ovIBXAdapterParams;

procedure Register;
begin
  RegisterComponents(cpObjectViewAddOn, [TovIBXAdapter]);
end;

resourcestring
   SAlterTableSQL = 'ALTER TABLE %s';
   SDeleteTableSQL = 'DROP TABLE %s';
   SAlterTriggerOn = 'ALTER TRIGGER TI_%s ACTIVE';
   SAlterTriggerOff = 'ALTER TRIGGER TI_%s INACTIVE';
   SDropTrigger = 'DROP TRIGGER TI_%s';
   SDropGenerator = 'DROP GENERATOR GEN_%s';
   SSetGenerator = 'SET GENERATOR GEN_%s TO %d';

function GetSQLFieldType(FieldType: TFieldType): string;
const
{$IFNDEF Delphi2006}
  SQLFieldTypes: array [TFieldType] of String =
   ({ftUnknown}'Unknown', {ftString}'VARCHAR', {ftSmallint}'SMALLINT',
    {ftInteger}'INTEGER', {ftWord}'INTEGER', {ftBoolean}'SMALLINT',
    {ftFloat}'DOUBLE PRECISION', {ftCurrency}'Currency', {ftBCD}'NUMERIC',
    {ftDate}'DATE', {ftTime}'TIME', {ftDateTime}'TIMESTAMP',
    {ftBytes}'Bytes', {ftVarBytes}'VarBytes', {ftAutoInc}'INTEGER',
    {ftBlob}'BLOB SUB_TYPE 0', {ftMemo}'BLOB SUB_TYPE 0',
    {ftGraphic}'BLOB SUB_TYPE 0', {ftFmtMemo}'BLOB SUB_TYPE 0',
    {ftParadoxOle}'ParadoxOle', {ftDBaseOle}'DBaseOle',
    {ftTypedBinary}'TypedBinary',
    {ftCursor}'Cursor', {ftFixedChar}'CHAR', {ftWideString}'VARCHAR',
    {ftLargeint}'INTEGER', {ftADT}'ADT', {ftArray}'Array',
    {ftReference}'Reference', {ftDataSet}'DataSet', {ftOraBlob}'OraBlob',
    {ftOraClob}'OraBlob', {ftVariant}'Variant', {ftInterface}'Interface',
    {ftIDispatch}'IDispatch', {ftGuid}'Guid', {ftTimeStamp}'TIMESTAMP',
    {ftFMTBcd}'NUMERIC');
{$ELSE}
   ...to-do
{$ENDIF}
begin
  Result := SQLFieldTypes[FieldType];
  if Result = '' then
    raise Exception.Create(' :' +
      GetEnumName(TypeInfo(TFieldType), Integer(FieldType)));
end;

type
  TSQLFieldDef = class
  private
    FFieldName: String;
    FFieldType: String;
  public
    property FieldName: String read FFieldName;
    property FieldType: String read FFieldType;
  end;

{{
     SQL-.
}
function GetSQLFieldDef(AFieldName: String; AFieldType: TFieldType; AFieldSize:
  Integer): TSQLFieldDef;
begin
  Result := TSQLFieldDef.Create;
  try
    Result.FFieldName := AFieldName;
    Result.FFieldType := GetSQLFieldType(AFieldType);
    if AFieldType in [ftString, ftWideString] then
      Result.FFieldType := Result.FFieldType + Format('(%d)', [AFieldSize]);
  except
    Result.Free; raise
  end;
end;

{{
   ,   
.
}
function GetPrimaryKeyFieldDef(FieldDefs: TFieldDefs): TFieldDef;
var
  I: Integer;
begin
  for I := 0 to FieldDefs.Count - 1 do
    if FieldDefs[I].DataType = ftAutoInc then
    begin
      Result := FieldDefs[I];
      Exit;
    end;
  raise Exception.Create('    ');
end;

type
  {         RecordCount
     , ..     
       (  1,     
     RecordCount ),      . }

  TovIBQuery = class(TIBQuery)
  protected
    function GetRecordCount: Integer; override;
  end;

  TovIBTable = class(TIBTable)
  protected
    function GetRecordCount: Integer; override;
  end;

{ TovIBQuery }

function TovIBQuery.GetRecordCount: Longint;
begin
  FetchAll;
  Result := inherited GetRecordCount
end;

{ TovIBTable }

function TovIBTable.GetRecordCount: Longint;
begin
  FetchAll;
  Result := inherited GetRecordCount
end;

{ TovIBXAdapter }

constructor TovIBXAdapter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  with QuerySyntaxParams do begin
    LeftTableDelimiter := '';
    RightTableDelimiter := '';
    LeftFieldDelimiter := '';
    RightFieldDelimiter := '';
    FilterLeftFieldDelimiter := '';
    FilterRightFieldDelimiter := '';
  end;
  FSupportsBooleanType := False;
  FSupportsLocalisedNames := False;
  FFormatSettings.DecimalSeparator := '.';
end;

procedure TovIBXAdapter.SetDatabase(Value: TIBDatabase);
begin
  if FDatabase <> Value then
  begin
    if Assigned(FDatabase) then FDatabase.AfterDisconnect := nil;
    FDatabase := Value;
    if Value <> nil then
    begin
      Value.FreeNotification(Self);
      Value.AfterDisconnect := DatabaseDisconnectHandler;
    end;
  end;
end;

{{
       
}
procedure TovIBXAdapter.DatabaseDisconnectHandler(Sender: TObject);
begin
  //       
  CloseDatabaseNotification
end;

function TovIBXAdapter.ActiveTransaction: TIBTransaction;
begin
  Result := DefaultTransaction
end;

procedure TovIBXAdapter.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = Database) and (Operation = opRemove)
    then Database := nil else
  if (AComponent = DefaultTransaction) and (Operation = opRemove)
    then DefaultTransaction := nil
end;

{ **************************************************************************** }
{                                                                   }
{ **************************************************************************** }

procedure TovIBXAdapter.CheckOpened;
begin
  if Not Assigned(Database) then
    raise Exception.Create('   Database  ' + Name);
end;

procedure TovIBXAdapter.OpenDatabase;
begin
  CheckOpened;
  Database.Open;
end;

procedure TovIBXAdapter.CloseDatabase;
begin
  if Assigned(Database) then Database.Close;
end;

procedure TovIBXAdapter.StartTransaction;
begin
  CheckOpened;
  if DefaultTransaction.Active then
    DefaultTransaction.CommitRetaining
  else
    DefaultTransaction.StartTransaction;
  FInTransaction := True;
end;

procedure TovIBXAdapter.CommitTransaction;
begin
  CheckOpened;
  DefaultTransaction.CommitRetaining;
  FInTransaction := False;
end;

procedure TovIBXAdapter.RollbackTransaction;
begin
  CheckOpened;
  DefaultTransaction.RollbackRetaining;
  FInTransaction := False;
end;

function TovIBXAdapter.InTransaction: Boolean;
begin
  CheckOpened;
  Result := FInTransaction;
end;

{ **************************************************************************** }
{                                                            }
{ **************************************************************************** }

function TovIBXAdapter.TableExists(ATableName: String): Boolean;
begin
  CheckOpened;
  with TIBTable.Create(nil) do
  try
    Database := Self.Database;
    Transaction := ActiveTransaction;
    TableName := ATableName;
    Result := Exists;
  finally
    Free;
  end;
end;

function TovIBXAdapter.OpenTable(ATableName: String; AReadOnly: Boolean;
  AFilter: String = ''): TDataSet;
begin
  CheckOpened;
  Result := TovIBTable.Create(nil);
  try
    with TovIBTable(Result) do begin
      Database := Self.Database;
      Transaction := ActiveTransaction;
      TableName := ATableName;
      ReadOnly := AReadOnly;
      Filter := AFilter;
      if AFilter <> '' then Filtered := True;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

procedure TovIBXAdapter.CreateTable(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs);
begin
  if TableExists(ATableName) then
    RestructureTable(ATableName, AFieldsDefs, AnIndexDefs)
  else
    CreateTableInternal(ATableName, AFieldsDefs, AnIndexDefs);
end;

procedure TovIBXAdapter.CreateTableInternal(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs);
begin
  SafeExecQueryList(GetCreateTableSQL(ATableName, AFieldsDefs, AnIndexDefs));
end;

{ **************************************************************************** }
{                      /                         }
{ **************************************************************************** }

{{
  .    
 ,     .    .
}
procedure TovIBXAdapter.RestructureTable(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs);
var
  I: Integer;
  NewFields, ChangedFields: TObjectList;
  FieldDef: TFieldDef;
  ATable: TIBTable;
begin
  ATable := TIBTable.Create(nil);
  try
    ATable.Database := Self.Database;
    ATable.Transaction := ActiveTransaction;
    ATable.TableName := ATableName;
    ATable.FieldDefs.Update;

    NewFields := TObjectList.Create;
    ChangedFields := TObjectList.Create;
    try
      //      
      for I := 0 to AFieldsDefs.Count - 1 do
      begin
        try
          FieldDef := ATable.FieldDefs.Find(AFieldsDefs[I].Name);
        except
          //    ,   EDatabaseError
          on EDatabaseError: Exception do FieldDef := nil;
        end;

        if Not Assigned(FieldDef) then
          with AFieldsDefs[I] do
          NewFields.Add(GetSQLFieldDef(Name, DataType, Size))
        else
        if (GetSQLFieldType(FieldDef.DataType) <> GetSQLFieldType(AFieldsDefs[I].DataType)) or
          (FieldDef.Size <> AFieldsDefs[I].Size)
        then
          with AFieldsDefs[I] do
          ChangedFields.Add(GetSQLFieldDef(Name, DataType, Size))
      end;
      ExecAlterTableQuery(ATableName, NewFields, ChangedFields);
    finally
      ChangedFields.Free;
      NewFields.Free;
    end;

  finally
    ATable.Free
  end;

  //   (   ,   )
  SafeExecQueryList(GetCreateIndexSQL(ATableName, AnIndexDefs));
end;

procedure TovIBXAdapter.ExecAlterTableQuery(TableName: String; NewFields,
  ChangedFields: TObjectList);

  procedure RecreateField(FieldDef: TSQLFieldDef);
  var
    Query: String;
  begin
    with FieldDef do
    begin
      Query := Format(SAlterTableSQL + ' DROP %s', [TableName, FieldName]);
      ExecQuery(Query);
      Query := Format(SAlterTableSQL + ' ADD %s %s', [TableName, FieldName, FieldType]);
      ExecQuery(Query);
    end
  end;

var
  I: Integer;
  Query: String;
begin
  for I := 0 to NewFields.Count - 1 do
  with TSQLFieldDef(NewFields[I]) do
  begin
    Query := Format(SAlterTableSQL + ' ADD %s %s', [TableName, FieldName, FieldType]);
    ExecQuery(Query);
  end;
  for I := 0 to ChangedFields.Count - 1 do
  with TSQLFieldDef(ChangedFields[I]) do
  begin
    Query := Format(SAlterTableSQL + ' ALTER COLUMN %s TYPE %s', [TableName, FieldName, FieldType]);
    try
      ExecQuery(Query);
    except on E: Exception do
      if Application.MessageBox(
        PChar(Format('    :'+
        #13+#10+'%s'+#13+#10+'       ?', [E.Message])),
        PChar('    ' + FieldName), MB_ICONWARNING or MB_YESNO) = IDNO
        then raise
        else RecreateField(TSQLFieldDef(ChangedFields[I]))
    end;
  end;
end;

{{
   .
}
function TovIBXAdapter.GetCreateIndexSQL(ATableName: String; AnIndexDefs:
  TIndexDefs): String;
var
  FieldList, IndexName, S, FieldName: String;
  I: Integer;
  UniqueStr, DescendingStr: String;
  ATable: TIBTable;
  IndexDef: TIndexDef;
const
  SQLTemplate = 'CREATE %s %s INDEX %s ON %s (%s);';
begin
  ATable := TIBTable.Create(nil);
  try
    ATable.Database := Self.Database;
    ATable.Transaction := ActiveTransaction;
    ATable.TableName := ATableName;
    ATable.IndexDefs.Update;

    Result := '';

    //    -   -  
    for I := 1 to AnIndexDefs.Count - 1 do
    begin
      //        ,  
      //    ,   AnIndexDefs[I].Name;
      IndexName := ATableName+'_IDX' + IntToStr(I);

      //         ( 
      //     -  :  
      //        ,    
      //       -  )
      try
        IndexDef := ATable.IndexDefs.Find(IndexName);
      except
        //    ,   EDatabaseError
        on EDatabaseError: Exception do IndexDef := nil;
      end;

      if IndexDef = nil then
      begin
        //  ;  ,   []
        S := AnIndexDefs[I].Fields;
        FieldList := '';
        while S <> '' do
        begin
          FieldName := ExtractSubStr(S, ';');
          FieldList := FieldList + ',' + FieldName;
        end;
        Delete(FieldList, 1, 1);
  
        if ixUnique in AnIndexDefs[I].Options
          then UniqueStr := 'UNIQUE' else UniqueStr := '';
        if ixDescending in AnIndexDefs[I].Options
          then DescendingStr := 'DESC' else DescendingStr := '';
  
        Result := Result + #13 + Format(SQLTemplate, [UniqueStr, DescendingStr,
          IndexName, ATableName, FieldList]);
      end;

    end;
    Delete(Result, 1, 1);

  finally
    ATable.Free;
  end
end;

function TovIBXAdapter.GetCreateTableSQL(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs): String;

const
  SQLTemplate =
    'CREATE TABLE %s (                        '+
    '%s INTEGER NOT NULL,                     '+
    '%s,                                      '+
    'PRIMARY KEY (%s)                         '+
    ');                                       '#13+

    'CREATE GENERATOR %s;                     '#13+

    'CREATE TRIGGER %s FOR %s                 '+
    'ACTIVE BEFORE INSERT POSITION 0 AS       '+
    'BEGIN NEW.%s = GEN_ID(%s, 1);            '+
    'END; ';

  function GetFieldListSQL: String;
  var
    I: Integer;
  begin
    Result := '';
    for I := 1 to AFieldsDefs.Count - 1 do
    begin
      Result := Result + Format('%s %s', [
        AFieldsDefs[I].Name, GetSQLFieldType(AFieldsDefs[I].DataType)]);
      if AFieldsDefs[I].DataType in [ftString] then
        Result := Result + Format('(%d)', [AFieldsDefs[I].Size]);
      if I < AFieldsDefs.Count - 1 then
        Result := Result + ',';
    end;
  end;

var
  FieldList, PrimaryKey, GeneratorName, TriggerName: String;
begin
  FieldList := GetFieldListSQL;
  PrimaryKey := AFieldsDefs[0].Name;
  GeneratorName := 'GEN_' + ATableName;
  TriggerName := 'TI_' + ATableName;
  Result := Format(SQLTemplate, [ATableName, PrimaryKey, FieldList, PrimaryKey,
    GeneratorName, TriggerName, ATableName, PrimaryKey, GeneratorName]) + #13 +
    GetCreateIndexSQL(ATableName, AnIndexDefs);
end;

{{
    TableName,        
  SourceTable,      .

:

-  ,     (   )
-      
-  
-     
-       
}
procedure TovIBXAdapter.CloneTable(SourceTable: TDataSet;
  SourceTableFieldNames: TStrings; TableName: String; FieldsDefs: TFieldDefs;
  IndexDefs: TIndexDefs);
var
  DestTable: TDataSet;
  NewGeneratorValue: Integer;
begin
  CheckOpened;
  DeleteTable(TableName);
  CreateTableInternal(TableName, FieldsDefs, IndexDefs);
  SafeExecQuery(Format(SAlterTriggerOff, [TableName]));
  //         

  DefaultTransaction.CommitRetaining; //  OpenTable    

  DestTable := OpenTable(TableName, False);
  try
    NewGeneratorValue := CopyData(SourceTable, SourceTableFieldNames, TableName,
      DestTable, SourceTable.FieldByName(ObjectIdField));
  finally
    DestTable.Free;
  end;
  SafeExecQuery(Format(SAlterTriggerOn, [TableName]));
  SafeExecQuery(Format(SSetGenerator, [TableName, NewGeneratorValue]));
end;

procedure TovIBXAdapter.DeleteTable(ATableName: String);
begin
  SafeExecQuery(Format(SDropTrigger, [ATableName]));
  SafeExecQuery(Format(SDropGenerator, [ATableName]));
  SafeExecQuery(Format(SDeleteTableSQL, [ATableName]));
end;

{ **************************************************************************** }
{                                    }
{ **************************************************************************** }

{{
      .
}
function TovIBXAdapter.GetNewRecordId(ATableName: String): Integer;
begin
  with OpenQuery(Format('SELECT DISTINCT GEN_ID(GEN_%s, 0) FROM RDB$DATABASE', {    }
    [ATableName]), nil, False) do
  try
    Open;
    Result := Fields[0].AsInteger;
  finally
    Free
  end;
end;

{{
      .
}
procedure TovIBXAdapter.SetNewRecordId(ATableName: String; AValue: Integer);
begin
  ExecQuery(Format(SSetGenerator, [ATableName, AValue]));
end;

{ **************************************************************************** }
{                                                            }
{ **************************************************************************** }

procedure TovIBXAdapter.ExecQuery(SQLText: String);
begin
  CheckOpened;
  with TIBQuery.Create(nil) do
  try
    Database := Self.Database;
    Transaction := ActiveTransaction;
    SQL.Text := SQLText;
    ExecSQL;
  finally
    Free
  end;
end;

function TovIBXAdapter.OpenQuery(QueryText: String; MasterDataSource:
  TDataSource; LiveDataSet: Boolean): TDataSet;
begin
  CheckOpened;
  Result := TovIBQuery.Create(nil);
  try
    with TovIBQuery(Result) do begin
      Database := Self.Database;
      Transaction := ActiveTransaction;
      SQL.Text := QueryText;
      DataSource := MasterDataSource;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

procedure TovIBXAdapter.SafeExecQuery(SQLText: String);
begin
  try
    ExecQuery(SQLText);
  except
  end;
end;

procedure TovIBXAdapter.SafeExecQueryList(QueryList: String);
var
  I: Integer;
  SQL: TStrings;
begin
  SQL := TStringList.Create;
  try
    SQL.Text := QueryList;
    for I := 0 to SQL.Count - 1 do
      SafeExecQuery(SQL[I]);
  finally
    SQL.Free
  end;
end;

{ **************************************************************************** }
{                                                              }
{ **************************************************************************** }

function TovIBXAdapter.EditConnection(ConnectionParams: TStrings):
  Boolean;
begin
  Result := EditAdapterParams(ConnectionParams)
end;

procedure TovIBXAdapter.SetupConnection(ConnectionParams: TStrings);
begin
  Database := TIBDatabase.Create(Self);
  Database.DatabaseName := ConnectionParams.Values['DatabaseFileName'];
  if ConnectionParams.Values['ServerName'] <> '' then
    Database.DatabaseName := ConnectionParams.Values['ServerName'] + ':' + Database.DatabaseName;

  if (ConnectionParams.Values['ServerName'] = '') and (Database.DatabaseName <> '') and
     Not FileExists(Database.DatabaseName)
  then
  try
    CreateDatabaseInternal(Database.DatabaseName,
           ConnectionParams.Values['user_name'], ConnectionParams.Values['password']);
  except
    {      -      
      ,           }
  end;

  Database.LoginPrompt := ConnectionParams.Values['LoginPrompt'] = BoolToStr(True);
  Database.Params.Assign(ConnectionParams);
  { ..    ,      }
  Database.Params.Values['DatabaseFileName'] := '';
  Database.Params.Values['ServerName'] := '';
  Database.Params.Values['LoginPrompt'] := '';

  DefaultTransaction := TIBTransaction.Create(Self);
  DefaultTransaction.DefaultDatabase := Database;
end;

{{
     .   ConnectionParams 
   .

:
-     
-      
-   
}
procedure TovIBXAdapter.CreateDatabase(DatabaseName: String; ConnectionParams:
  TStrings);
var
  SourceDBName: String;
  NewDBName: String;
begin
  //     
  SourceDBName := ChangeFileExt(Database.DatabaseName, ''); //    
  NewDBName := SourceDBName + ' ' + DatabaseName + ExtractFileExt(Database.DatabaseName);

  //   
  try
    CreateDatabaseInternal(NewDBName,
      Database.Params.Values['user_name'], Database.Params.Values['password']);
  except
    //    ,     -   
  end;

  //   
  ConnectionParams.Assign(Database.Params);
  ConnectionParams.Values['DatabaseFileName'] := NewDBName;
end;

procedure TovIBXAdapter.CreateDatabaseInternal(ADatabaseName, AUserName,
  APassword: String);
begin
  with TIBDatabase.Create(Self) do
  try
    DatabaseName := ADatabaseName;
    Params.Add(Format('USER ''%s'' PASSWORD ''%s''', [AUserName, APassword]));
    CreateDatabase;
  finally
    Free
  end;
end;

initialization
  TovIBXAdapter.RegisterAdapter(TovIBXAdapter);
finalization
  TovIBXAdapter.UnregisterAdapter(TovIBXAdapter);
end.
