unit ovIBXAdapterParams;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons;

type
  TfrmIBXAdapterParams = class(TForm)
    GroupBox2: TGroupBox;
    Label7: TLabel;
    Label8: TLabel;
    LocalRbtn: TRadioButton;
    RemoteRbtn: TRadioButton;
    ServerName: TEdit;
    Protocol: TComboBox;
    Panel1: TPanel;
    OKBtn: TButton;
    CancelBtn: TButton;
    GroupBox1: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    Label5: TLabel;
    UserName: TEdit;
    Password: TEdit;
    DBParams: TMemo;
    LoginPrompt: TCheckBox;
    Label1: TLabel;
    DatabaseFile: TEdit;
    OpenBtn: TSpeedButton;
    OpenDialog: TOpenDialog;
    procedure LocalRbtnClick(Sender: TObject);
    procedure OpenBtnClick(Sender: TObject);
    procedure RemoteRbtnClick(Sender: TObject);
    procedure UserNameChange(Sender: TObject);
    procedure PasswordChange(Sender: TObject);
  private
    { Private declarations }
    function GetParam(Name: string): string;
    procedure AddParam(Name, Value: string);
    procedure DeleteParam(Name: string);
    procedure GetParams(Params: TStrings);
    procedure SetParams(Params: TStrings);
  public
  end;

function EditAdapterParams(Params: TStrings): Boolean;

implementation

uses TypInfo;

{$R *.dfm}

function EditAdapterParams(Params: TStrings): Boolean;
begin
  with TfrmIBXAdapterParams.Create(Application) do
  try
    SetParams(Params);
    Result := ShowModal = mrOk;
    if Result then GetParams(Params);
  finally
    Free
  end;
end;

{ TfrmServiceEditor }

procedure TfrmIBXAdapterParams.AddParam(Name, Value: string);
var
  i: Integer;
  found: boolean;
begin
  found := False;
  if Trim(Value) <> '' then  {do not localize}
  begin
    for i := 0 to DBParams.Lines.Count - 1 do
    begin
      if (Pos(Name, LowerCase(DBParams.Lines.Names[i])) = 1) then {mbcs ok}
      begin
        DBParams.Lines.Values[DBParams.Lines.Names[i]] := Value;
        found := True;
        break;
      end;
    end;
    if not found then
      DBParams.Lines.Add(Name + '=' + Value);  {do not localize}
  end
  else
    DeleteParam(Name);
end;

procedure TfrmIBXAdapterParams.DeleteParam(Name: string);
var
  i: Integer;
begin
    for i := 0 to DBParams.Lines.Count - 1 do
    begin
      if (Pos(Name, LowerCase(DBParams.Lines.Names[i])) = 1) then {mbcs ok}
      begin
        DBParams.Lines.Delete(i);
        break;
      end;
    end;
end;

function TfrmIBXAdapterParams.GetParam(Name: string): string;
var
  i: Integer;
begin
  Result := '';  {do not localize}
  for i := 0 to DBParams.Lines.Count - 1 do
  begin
    if (Pos(Name, LowerCase(DBParams.Lines.Names[i])) = 1) then {mbcs ok}
    begin
      Result := DBParams.Lines.Values[DBParams.Lines.Names[i]];
      break;
    end;
  end;
end;

procedure TfrmIBXAdapterParams.LocalRbtnClick(Sender: TObject);
begin
  Label7.Enabled := False;
  Label8.Enabled := False;
  ServerName.Enabled := False;
  Protocol.Enabled := False;
  ServerName.Text := '';
  Protocol.ItemIndex := -1;
end;

procedure TfrmIBXAdapterParams.OpenBtnClick(Sender: TObject);
begin
  if OpenDialog.Execute then DatabaseFile.Text := OpenDialog.FileName
end;

procedure TfrmIBXAdapterParams.RemoteRbtnClick(Sender: TObject);
begin
  Label7.Enabled := True;
  Label8.Enabled := True;
  Protocol.Enabled := True;
  ServerName.Enabled := True;
end;

procedure TfrmIBXAdapterParams.UserNameChange(Sender: TObject);
begin
  AddParam('user_name', UserName.Text);  {do not localize}
end;

procedure TfrmIBXAdapterParams.PasswordChange(Sender: TObject);
begin
  AddParam('password', Password.Text);  {do not localize}
end;

procedure TfrmIBXAdapterParams.GetParams(Params: TStrings);
begin
  Params.Assign(DBParams.Lines);
  Params.Values['DatabaseFileName'] := DatabaseFile.Text;
  Params.Values['ServerName'] := ServerName.Text;
  Params.Values['LoginPrompt'] := BoolToStr(LoginPrompt.Checked);
end;

procedure TfrmIBXAdapterParams.SetParams(Params: TStrings);
begin
  DatabaseFile.Text := Params.Values['DatabaseFileName'];
  ServerName.Text := Params.Values['ServerName'];
  if ServerName.Text <> '' then RemoteRbtn.Checked := True
  else LocalRbtn.Checked := True;
  LoginPrompt.Checked := StrToBoolDef(Params.Values['LoginPrompt'], False);
  DBParams.Lines.Assign(Params);
  DBParams.Lines.Values['DatabaseFileName'] := '';
  DBParams.Lines.Values['ServerName'] := '';
  DBParams.Lines.Values['LoginPrompt'] := '';

  UserName.Text := Params.Values['user_name'];  {do not localize}
  Password.Text := Params.Values['password'];   {do not localize}
end;

end.
