{
  MSSQL  BDE.

      MSSQL   TTable  ftAutoInc
    ,      
  .
}
unit ovMSSQLBDEAdapter;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBTables, ovTypes, ovModel, ovComponents;

type
  TovMSSQLAdapter = class(TovCustomDatabaseAdapter)
  private
    FDatabase: TDatabase;
    procedure SetDatabase(Value: TDatabase);
    procedure CheckDatabase;
    procedure GenerateDatabaseName(Database: TDatabase);
  protected
    procedure CloseDatabase; override;
    procedure CommitTransaction; override;
    procedure CreateTable(ATableName: String; AFieldsDefs: TFieldDefs; 
      AnIndexDefs: TIndexDefs); override;
    procedure ExecQuery(SQLText: String); override;
    procedure OpenDatabase; override;
    function OpenQuery(QueryText: String; MasterDataSource: TDataSource; 
      LiveDataSet: Boolean): TDataSet; override;
    function OpenTable(ATableName: String; AReadOnly: Boolean; AFilter: String):
      TDataSet; override;
    procedure RollbackTransaction; override;
    procedure StartTransaction; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function TableExists(ATableName: String): Boolean; override;
    function InTransaction: Boolean; override;
  public
    procedure SetupConnection(ConnectionParams: TStrings); override;
    constructor Create(AOwner: TComponent); override;
    function EditConnection(ConnectionParams: TStrings): Boolean; override;
  published
    property Database: TDatabase read FDatabase write SetDatabase;
  end;

procedure Register;

implementation

uses ovConsts, ovObjects, ovBDEParams;

procedure Register;
begin
  RegisterComponents(cpObjectViewAddOn, [TovMSSQLAdapter]);
end;

var DatabaseCount: Integer = 0;

{
******************************* TovBDEAdapter *******************************
}
{<<Category>>: Undefined}
constructor TovMSSQLAdapter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  with QuerySyntaxParams do begin
    LeftTableDelimiter := '[';
    RightTableDelimiter := ']';
    LeftFieldDelimiter := '[';
    RightFieldDelimiter := ']';
  end;
end;

procedure TovMSSQLAdapter.CloseDatabase;
begin
  if Assigned(Database) then Database.Close;
end;

procedure TovMSSQLAdapter.StartTransaction;
begin
  CheckDatabase;
  //      TransIsolation  tiDirtyRead
  if Not Database.IsSQLBased
    then Database.TransIsolation := tiDirtyRead;
  Database.StartTransaction
end;

procedure TovMSSQLAdapter.RollbackTransaction;
begin
  CheckDatabase;
  Database.Rollback
end;

procedure TovMSSQLAdapter.CommitTransaction;
begin
  CheckDatabase;
  Database.Commit;
end;

function TovMSSQLAdapter.InTransaction: Boolean;
begin
  CheckDatabase;
  Result := Database.InTransaction
end;

procedure TovMSSQLAdapter.CreateTable(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs);
var
  Query: String;
  I: Integer;
  Table: TTable;
begin
  CheckDatabase;

  Table := TTable.Create(nil);
  with Table do
  try
    DatabaseName := Self.Database.DatabaseName;
    TableName := ATableName;
    FieldDefs.Assign(AFieldsDefs);
    if AnIndexDefs <> nil
      then IndexDefs.Assign(AnIndexDefs);
    if Not Exists
      then CreateTable
      else Exit;

    //   .         ,
    //    .
    Query := '';
    for I := 0 to AFieldsDefs.Count - 1 do
      if AFieldsDefs[I].DataType = ftAutoInc then
        Query := Query + #13 + Format(
          'DROP INDEX [%0:s].[Primary]'#13+
          'ALTER TABLE [%0:s] DROP COLUMN [%1:s]'#13+
          'ALTER TABLE [%0:s] ADD [%1:s] int IDENTITY (1, 1)'#13+
          'CREATE  INDEX [Primary] ON [dbo].[%0:s]([%1:s]) ON [PRIMARY]',
          [ATableName, AFieldsDefs[I].Name]);
    if Query <> '' then ExecQuery(Query);

  finally
    Free
  end;
end;

procedure TovMSSQLAdapter.ExecQuery(SQLText: String);
begin
  CheckDatabase;

  with TQuery.Create(nil) do
  try
    DatabaseName := Self.Database.DatabaseName;
    SQL.Text := SQLText;
    ExecSQL;
  finally
    Free
  end;
end;

procedure TovMSSQLAdapter.OpenDatabase;
begin
  CheckDatabase;
  Database.Open;
end;

function TovMSSQLAdapter.OpenQuery(QueryText: String; MasterDataSource: 
  TDataSource; LiveDataSet: Boolean): TDataSet;
begin
  CheckDatabase;

  Result := TQuery.Create(nil);
  try
    with TQuery(Result) do begin
      DatabaseName := Self.Database.DatabaseName;
      SQL.Text := QueryText;
      if MasterDataSource <> nil then
        DataSource := MasterDataSource;
      RequestLive := LiveDataSet;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

function TovMSSQLAdapter.OpenTable(ATableName: String; AReadOnly: Boolean;
  AFilter: String): TDataSet;
begin
  CheckDatabase;

  Result := TTable.Create(nil);
  try
    with TTable(Result) do begin
      DatabaseName := Self.Database.DatabaseName;
      TableName := ATableName;
      ReadOnly := AReadOnly;
      Filter := AFilter;
      if AFilter <> '' then Filtered := True;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

procedure TovMSSQLAdapter.SetDatabase(Value: TDatabase);
begin
  if FDatabase <> Value then
  begin
    FDatabase := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovMSSQLAdapter.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = Database) and (Operation = opRemove)
    then Database := nil;
end;

procedure TovMSSQLAdapter.CheckDatabase;
begin
  if Not Assigned(Database) then
    raise Exception.Create('   Database  ' + Name);
end;

function TovMSSQLAdapter.TableExists(ATableName: String): Boolean;
var
  Table: TTable;
begin
  Table := TTable.Create(nil);
  with Table do
  try
    DatabaseName := Self.Database.DatabaseName;
    TableName := ATableName;
    Result := Exists;
  finally
    Free
  end;
end;

procedure TovMSSQLAdapter.GenerateDatabaseName(Database: TDatabase);
var
  DatabaseCount: Integer;
begin
  DatabaseCount := 1;
  while True do
  try
    Inc(DatabaseCount);
    Database.DatabaseName := 'Database_' + IntToStr(DatabaseCount);
    Break;
  except
    //     DatabaseName   
  end;
end;

procedure TovMSSQLAdapter.SetupConnection(ConnectionParams: TStrings);
begin
  Database := TDatabase.Create(Self);
  if ConnectionParams.Values['DatabaseName'] = ''
    then GenerateDatabaseName(Database)
    else Database.DatabaseName := ConnectionParams.Values['DatabaseName'];

  with ConnectionParams do
  begin
    if Values['AliasName'] <> '' then
      Database.AliasName := Values['AliasName'];
    if Values['DriverName'] <> '' then
      Database.DriverName := Values['DriverName'];

    Values['DatabaseName'] := '';
    Values['AliasName'] := '';
    Values['DriverName'] := '';
    Database.Params := ConnectionParams;
  end
end;

function TovMSSQLAdapter.EditConnection(ConnectionParams: TStrings): Boolean;
begin
  Result := ShowAdapterParams(ConnectionParams)
end;

initialization
  TovMSSQLAdapter.RegisterAdapter(TovMSSQLAdapter);
end.
