{
  MS SQL Server Data Access 3.55

   ,      
BDE  SQL Server.    .
,       SDAC   2  ,
  BDE.
}
unit ovSDACAdapter;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, ovTypes, ovModel, ovComponents, MSAccess;

type
  TovSDACAdapter = class(TovCustomDatabaseAdapter)
  private
    FConnection: TMSConnection;
    procedure SetConnection(Value: TMSConnection);
    procedure CheckDatabase;
  protected
    procedure CloseDatabase; override;
    procedure CommitTransaction; override;
    procedure CreateTable(ATableName: String; AFieldsDefs: TFieldDefs; 
      AnIndexDefs: TIndexDefs); override;
    procedure ExecQuery(SQLText: String); override;
    procedure OpenDatabase; override;
    function OpenQuery(QueryText: String; MasterDataSource: TDataSource;
      LiveDataSet: Boolean): TDataSet; override;
    function OpenTable(ATableName: String; AReadOnly: Boolean): TDataSet; override;
    procedure RollbackTransaction; override;
    procedure StartTransaction; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function TableExists(ATableName: String): Boolean; override;
    function InTransaction: Boolean; override;
  public
    procedure SetupConnection(ConnectionParams: TStrings); override;
    function EditConnection(ConnectionParams: TStrings): Boolean; override;
    constructor Create(AOwner: TComponent); override;
  published
    property Connection: TMSConnection read FConnection write SetConnection;
  end;

procedure Register;

implementation

//{$R *.dcr}

uses ovConsts, ovObjects, DBAccess;

procedure Register;
begin
  RegisterComponents(cpObjectViewAddOn, [TovSDACAdapter]);
end;

var DatabaseCount: Integer = 0;

{
******************************* TovSDACAdapter *******************************
}
constructor TovSDACAdapter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  with QuerySyntaxParams do begin
    LeftTableDelimiter := '';
    RightTableDelimiter := '';
    LeftFieldDelimiter := '';
    RightFieldDelimiter := '';
  end;
end;

procedure TovSDACAdapter.CloseDatabase;
begin
  if Assigned(Connection) then Connection.Close;
end;

procedure TovSDACAdapter.StartTransaction;
begin
  CheckDatabase;
  Connection.StartTransaction
end;

procedure TovSDACAdapter.RollbackTransaction;
begin
  CheckDatabase;
  Connection.Rollback
end;

procedure TovSDACAdapter.CommitTransaction;
begin
  CheckDatabase;
  Connection.Commit;
end;

function TovSDACAdapter.InTransaction: Boolean;
begin
  CheckDatabase;
  Result := Connection.InTransaction
end;

procedure TovSDACAdapter.CreateTable(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs);
begin
end;

procedure TovSDACAdapter.ExecQuery(SQLText: String);
begin
  CheckDatabase;

  with TMSQuery.Create(nil) do
  try
    Connection := Self.Connection;
    SQL.Text := SQLText;
    Execute;
  finally
    Free
  end;
end;

procedure TovSDACAdapter.OpenDatabase;
begin
  CheckDatabase;
  Connection.Open;
end;

function TovSDACAdapter.OpenQuery(QueryText: String; MasterDataSource:
  TDataSource; LiveDataSet: Boolean): TDataSet;
begin
  CheckDatabase;

  Result := TMSQuery.Create(nil);
  try
    with TMSQuery(Result) do begin
      Connection := Self.Connection;
      SQL.Text := QueryText;
      if MasterDataSource <> nil then
        MasterSource := MasterDataSource;
      ReadOnly := Not LiveDataSet;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

function TovSDACAdapter.OpenTable(ATableName: String; AReadOnly: Boolean): 
  TDataSet;
begin
  CheckDatabase;

  Result := TMSTable.Create(nil);
  try
    with TMSTable(Result) do begin
      Connection := Self.Connection;
      TableName := ATableName;
      ReadOnly := AReadOnly;
    end;
    Result.Open;
  except
    Result.Free; raise;
  end;
end;

procedure TovSDACAdapter.SetConnection(Value: TMSConnection);
begin
  if FConnection <> Value then
  begin
    FConnection := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovSDACAdapter.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = Connection) and (Operation = opRemove)
    then Connection := nil;
end;

procedure TovSDACAdapter.CheckDatabase;
begin
  if Not Assigned(Connection) then
    raise Exception.Create('   Connection  ' + Name);
end;

function TovSDACAdapter.TableExists(ATableName: String): Boolean;
var
  Table: TMSTable;
begin
  Table := TMSTable.Create(nil);
  with Table do
  try
    Connection := Self.Connection;
    TableName := ATableName;
    Result := True ///Exists;
  finally
    Free
  end;
end;

procedure TovSDACAdapter.SetupConnection(ConnectionParams: TStrings);
begin
  Connection := TMSConnection.Create(Self);

  with ConnectionParams do
  begin
    Connection.ConnectString := Values['ConnectionString']
  end
end;

function TovSDACAdapter.EditConnection(ConnectionParams: TStrings): Boolean;
begin
  Result := False
end;

initialization
  TovSDACAdapter.RegisterAdapter(TovSDACAdapter);
finalization
  TovSDACAdapter.UnregisterAdapter(TovSDACAdapter);
end.
