{
  MSSQL  ADO.
}
unit ovSQLServerADOAdapter;
interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, ADODB, ovTypes, ovModel, ovADOAdapter;

type
  TovMSSQLServerADOAdapter = class(TovADOAdapter)
  private
    FAlwaysRebuildTable: Boolean;
    FCreateSimpleTable: Boolean;
    FDateFormat: String;
    FTimeFormat: String;
    procedure ExecAlterTableQuery(TableName: String; NewFields, ChangedFields:
      TStrings);
    procedure RestructureTableInternal(ATableName: String; AFieldsDefs: TFieldDefs;
      AnIndexDefs: TIndexDefs);
  protected
    procedure CloneTable(SourceTable: TDataSet; SourceTableFieldNames: TStrings;
      TableName: String; FieldsDefs: TFieldDefs; IndexDefs: TIndexDefs); override;
    procedure CreateTable(ATableName: String; AFieldsDefs: TFieldDefs; AnIndexDefs:
      TIndexDefs; ASimpleTable, AlwaysRebuild: Boolean);
    procedure GetCreateTableQuery(ATableName: String; AFieldsDefs: TFieldDefs;
        AnIndexDefs: TIndexDefs; AQuery: TStrings); override;
    function RestructureTable(ATableName: String; AFieldsDefs: TFieldDefs;
        AnIndexDefs: TIndexDefs): Boolean; override;
    function ValueToSQLString(const Value: Variant; FieldType: TFieldType;
      FieldSize: Integer): String; override;
    function DatabaseExists(ADatabaseName: String): Boolean;
    procedure OpenDatabase; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure CreateDatabase(DatabaseName: String; ConnectionParams: TStrings);
      override;
  end;

procedure Register;

implementation

{$R *.dcr}

uses TypInfo, ovConsts, ovObjects, AdoConEd, ovUtils, ovIObjects, Variants,
  ovComponents;

procedure Register;
begin
  RegisterComponents(cpObjectViewAddOn, [TovMSSQLServerADOAdapter]);
end;

var
  FormatSettings: TFormatSettings;

function GetSQLFieldType(FieldType: TFieldType): string;
const
{$IFNDEF Delphi2006}
  SQLFieldTypes: array [TFieldType] of String =
   ({ftUnknown}'', {ftString}'varchar', {ftSmallint}'Smallint',
    {ftInteger}'int', {ftWord}'int', {ftBoolean}'Smallint',
    {ftFloat}'Float', {ftCurrency}'Money', {ftBCD}'Money',
    {ftDate}'DateTime', {ftTime}'DateTime', {ftDateTime}'DateTime',
    {ftBytes}'Binary', {ftVarBytes}'VarBinary', {ftAutoInc}'int',
    {ftBlob}'VarBinary(MAX)', {ftMemo}'Text',
    {ftGraphic}'Image', {ftFmtMemo}'Text',
    {ftParadoxOle}'', {ftDBaseOle}'',
    {ftTypedBinary}'Binary',
    {ftCursor}'Cursor', {ftFixedChar}'Char', {ftWideString}'NVarChar',
    {ftLargeint}'BigInt', {ftADT}'', {ftArray}'',
    {ftReference}'', {ftDataSet}'', {ftOraBlob}'',
    {ftOraClob}'', {ftVariant}'SQL_variant', {ftInterface}'',
    {ftIDispatch}'', {ftGuid}'Uniqueidentifier', {ftTimeStamp}'TimeStamp',
    {ftFMTBcd}'Decimal');
{$ELSE}
  SQLFieldTypes: array [TFieldType] of String =
   ({ftUnknown}'', {ftString}'varchar', {ftSmallint}'Smallint',
    {ftInteger}'int', {ftWord}'int', {ftBoolean}'Smallint',
    {ftFloat}'Float', {ftCurrency}'Money', {ftBCD}'Money',
    {ftDate}'DateTime', {ftTime}'DateTime', {ftDateTime}'DateTime',
    {ftBytes}'Binary', {ftVarBytes}'VarBinary', {ftAutoInc}'int',
    {ftBlob}'VarBinary(MAX)', {ftMemo}'Text',
    {ftGraphic}'Image', {ftFmtMemo}'Text',
    {ftParadoxOle}'', {ftDBaseOle}'',
    {ftTypedBinary}'Binary',
    {ftCursor}'Cursor', {ftFixedChar}'Char', {ftWideString}'NVarChar',
    {ftLargeint}'BigInt', {ftADT}'', {ftArray}'',
    {ftReference}'', {ftDataSet}'', {ftOraBlob}'',
    {ftOraClob}'', {ftVariant}'SQL_variant', {ftInterface}'',
    {ftIDispatch}'', {ftGuid}'Uniqueidentifier', {ftTimeStamp}'TimeStamp',
    {ftFMTBcd}'Decimal', '', '', '', '');
{$ENDIF}
begin
  Result := SQLFieldTypes[FieldType];
  if Result = '' then
    raise Exception.Create(' :' +
      GetEnumName(TypeInfo(TFieldType), Integer(FieldType)));
end;

{{
     SQL-.
}
function GetSQLFieldDef(AFieldName: String; AFieldType: TDataType; AFieldSize:
  Integer): String;
begin
  Result := Format('[%s] %s', [AFieldName, GetSQLFieldType(AFieldType)]);
  if AFieldType in [ftString] then
    Result := Result + Format('(%d)', [AFieldSize]);
end;

{{
   ,   
.
}
function GetPrimaryKeyFieldDef(FieldDefs: TFieldDefs): TFieldDef;
var
  I: Integer;
begin
  for I := 0 to FieldDefs.Count - 1 do
    if FieldDefs[I].DataType = ftAutoInc then
    begin
      Result := FieldDefs[I];
      Exit;
    end;
  raise Exception.Create('    ');  
end;

{
******************************* TovMSSQLServerADOAdapter ***********************
}

resourcestring
  SCreateDatabase = 'CREATE DATABASE [%s]';

  SCreateIndexSQL =
    'CREATE INDEX [%s] ON [%s](%s);';
  SCreateTableSQL =
    'CREATE TABLE [%s] ('+
    '[%s] int IDENTITY(1,1) NOT NULL, '+
    '%s, '+
    'CONSTRAINT [%s] PRIMARY KEY ([%s])'+
    ');';
   SCreateSimpleTableSQL =
    'CREATE TABLE [%s] ('+
    '[%s] int NOT NULL, '+
    '%s, '+
    ');';
   SAlterTableSQL = 'ALTER TABLE [%s] ';
   SDeleteTableSQL = 'DROP TABLE [%s]';
   SCopyTableSQL =
    'SET IDENTITY_INSERT [%0:s] ON '#13+
    'INSERT INTO [%0:s] (%1:s) '#13+
    'SELECT %1:s From [%2:s] '#13+
    'SET IDENTITY_INSERT [%0:s] OFF ';

constructor TovMSSQLServerADOAdapter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSupportsBooleanType := False;
  with QuerySyntaxParams do begin
    LeftTableDelimiter := '[';
    RightTableDelimiter := ']';
    LeftFieldDelimiter := '[';
    RightFieldDelimiter := ']';
    FilterLeftFieldDelimiter := '[';
    FilterRightFieldDelimiter := ']';
  end;
  FDateFormat := 'dd"/"mm"/"yyyy';
  FTimeFormat := 'hh":"nn":"ss';
end;

{
*******************************   ********************************
}

{{
  .    ASimpleTable,   
    IDENTITY .    AlwaysRebuild  
  ,     ,    .
}
procedure TovMSSQLServerADOAdapter.CreateTable(ATableName: String; AFieldsDefs:
  TFieldDefs; AnIndexDefs: TIndexDefs; ASimpleTable, AlwaysRebuild: Boolean);
begin
  FCreateSimpleTable := ASimpleTable;
  FAlwaysRebuildTable := AlwaysRebuild;
  try
    inherited CreateTable(ATableName, AFieldsDefs, AnIndexDefs);
  finally
    FCreateSimpleTable := False;
    FAlwaysRebuildTable := False;
  end;
end;

function TovMSSQLServerADOAdapter.RestructureTable(ATableName: String;
    AFieldsDefs: TFieldDefs; AnIndexDefs: TIndexDefs): Boolean;
var
  IsExists: Boolean;
begin
  IsExists := TableExists(ATableName);
  if IsExists and FAlwaysRebuildTable then begin
    ExecQuery(Format(SDeleteTableSQL, [ATableName]));
    Result := False; //   -    
  end else
  if Not IsExists then
    //    -   
    Result := False
  else
  begin
    //   -  
    RestructureTableInternal(ATableName, AFieldsDefs, AnIndexDefs);
    //  
    Result := True;
  end;
end;

{
TODO :
      
 , ..     .

PRE:   .
}
procedure TovMSSQLServerADOAdapter.RestructureTableInternal(ATableName: String;
  AFieldsDefs: TFieldDefs; AnIndexDefs: TIndexDefs);
var
  I: Integer;
  NewFields, ChangedFields: TStringList;
  FieldDef: TFieldDef;
  ATable: TADOTable;
begin
  ATable := TADOTable.Create(nil);
  try
    ATable.Connection := Self.Connection;
    ATable.TableName := ATableName;
    ATable.FieldDefs.Update;

    NewFields := TStringList.Create;
    ChangedFields := TStringList.Create;
    try
      //      
      for I := 0 to AFieldsDefs.Count - 1 do
      begin
        try
          FieldDef := ATable.FieldDefs.Find(AFieldsDefs[I].Name);
        except
          //    ,   EDatabaseError
          on EDatabaseError: Exception do FieldDef := nil;
        end;

        if Not Assigned(FieldDef) then
          with AFieldsDefs[I] do
          NewFields.Add(GetSQLFieldDef(Name, DataType, Size))
        else
        if (GetSQLFieldType(FieldDef.DataType) <> GetSQLFieldType(AFieldsDefs[I].DataType)) or
          (FieldDef.Size <> AFieldsDefs[I].Size)
        then
          with AFieldsDefs[I] do
          ChangedFields.Add(GetSQLFieldDef(Name, DataType, Size))
      end;
      ExecAlterTableQuery(ATableName, NewFields, ChangedFields);
    finally
      ChangedFields.Free;
      NewFields.Free;
    end;

  finally
    ATable.Free
  end;
end;

procedure TovMSSQLServerADOAdapter.GetCreateTableQuery(ATableName: String;
    AFieldsDefs: TFieldDefs; AnIndexDefs: TIndexDefs; AQuery: TStrings);

  {{
         (
   !).
  }
  function GetFieldListSQL: String;
  var
    I: Integer;
  begin
    Result := '';
    for I := 1 to AFieldsDefs.Count - 1 do
    begin
      with AFieldsDefs[I] do
        Result := Result + GetSQLFieldDef(Name, DataType, Size);
      if I < AFieldsDefs.Count - 1 then
        Result := Result + ', ';
    end;
  end;

  {{
     .
  }
  procedure AddIndexQuery;
  var
    FieldList, IndexName, S: String;
    I: Integer;
    FieldName: String;
  begin
    //    -   -  
    for I := 1 to AnIndexDefs.Count - 1 do
    begin
      IndexName := AnIndexDefs[I].Name;

      //  ;  ,   []
      S := AnIndexDefs[I].Fields;
      FieldList := '';
      while S <> '' do
      begin
        FieldName := ExtractSubStr(S, ';');
        FieldList := FieldList + ',[' + FieldName + ']';
      end;
      Delete(FieldList, 1, 1);

      S := Format(SCreateIndexSQL, [IndexName, ATableName, FieldList]);
      AQuery.Add(S);
    end;
  end;

var
  FieldList, PrimaryKeyField: String;
begin
  if Not FCreateSimpleTable then
  begin
    FieldList := GetFieldListSQL;
    // ,     AnIndexDefs   , 
    //      -   
    PrimaryKeyField := AFieldsDefs[0].Name;
    AQuery.Add(Format(SCreateTableSQL, [ATableName, PrimaryKeyField, FieldList,
      ATableName +'_'+ AnIndexDefs[0].Name, PrimaryKeyField]));
    AddIndexQuery;
  end else
  begin
    //    FCreateSimpleTable,     IDENTITY
    //     ,     
    FieldList := GetFieldListSQL;
    PrimaryKeyField := AFieldsDefs[0].Name;
    AQuery.Add(Format(SCreateSimpleTableSQL, [ATableName, PrimaryKeyField, FieldList]));
  end
end;

procedure TovMSSQLServerADOAdapter.ExecAlterTableQuery(TableName: String;
  NewFields, ChangedFields: TStrings);
var
  I: Integer;
  Query: String;
begin
  for I := 0 to NewFields.Count - 1 do
  begin
    Query := Format(SAlterTableSQL, [TableName]) + 'ADD ' + NewFields[I];
    ExecQuery(Query);
  end;
  for I := 0 to ChangedFields.Count - 1 do
  begin
    Query := Format(SAlterTableSQL, [TableName]) + 'ALTER COLUMN ' + ChangedFields[I];
    ExecQuery(Query);
  end;
end;

{{
    TableName,        
  SourceTable,      .
}
procedure TovMSSQLServerADOAdapter.CloneTable(SourceTable: TDataSet;
  SourceTableFieldNames: TStrings; TableName: String; FieldsDefs: TFieldDefs;
  IndexDefs: TIndexDefs);

  {{
       SourceTable   DestTable, 
   ,    Identity   DestTable.
  }
  procedure CopyDataToIdentityColumn(SourceTable, DestTable: String);
  var
    FieldList: String;
    I: Integer;
  begin
    FieldList := '';
    for I := 0 to FieldsDefs.Count - 1 do
      FieldList := FieldList + ', [' + FieldsDefs[I].Name + ']';
    Delete(FieldList, 1, 2);
    ExecQuery(Format(SCopyTableSQL, [DestTable, FieldList, SourceTable]));
  end;

var
  DestTable: TDataSet;
  WorkTableName: String;
begin
  //      ,   
  //   IDENTITY 
  WorkTableName := '#' + TableName;
  CreateTable(WorkTableName, FieldsDefs, IndexDefs, True, True);
  try
    CreateTable(TableName, FieldsDefs, IndexDefs, False, True);

    //         
    DestTable := OpenTable(WorkTableName);
    try
      CopyData(SourceTable, SourceTableFieldNames, TableName, DestTable);
    finally
      DestTable.Free;
    end;

    //        
    CopyDataToIdentityColumn(WorkTableName, TableName);

  finally
    ExecQuery(Format(SDeleteTableSQL, [WorkTableName]));
  end;
end;

function TovMSSQLServerADOAdapter.ValueToSQLString(const Value: Variant;
  FieldType: TFieldType; FieldSize: Integer): String;
begin
  case FieldType of
    ftFloat:
      Result := FloatToStr(Value, FormatSettings);
    ftDate:
      Result := '''' + FormatDateTime(FDateFormat, Value)+ '''';
    ftTime:
      Result := '''' + FormatDateTime(FTimeFormat, Value)+ '''';
    ftDateTime:
      Result := '''' + FormatDateTime(FDateFormat+' '+ FTimeFormat, Value)+ '''';
    else Result := inherited ValueToSQLString(Value, FieldType, FieldSize)
  end
end;

procedure TovMSSQLServerADOAdapter.OpenDatabase;
begin
  inherited;
  ExecQuery('SET DATEFORMAT dmy');
end;

{{
     .   ConnectionParams 
   .

:
-     
-      
-   ConnectionString,        
}
procedure TovMSSQLServerADOAdapter.CreateDatabase(DatabaseName: String;
  ConnectionParams: TStrings);
var
  SourceDBName: String;
  NewDBName: String;
begin
  SourceDBName := GetConnectionStringParam(Connection.ConnectionString, 'Initial Catalog');
  NewDBName := SourceDBName + ' ' + DatabaseName;
  while DatabaseExists(NewDBName) do NewDBName := NewDBName + '0';
  ExecQuery(Format(SCreateDatabase, [NewDBName]));
  ConnectionParams.Values['ConnectionString'] :=
    SetConnectionStringParam(Connection.ConnectionString, 'Initial Catalog', NewDBName);
end;

function TovMSSQLServerADOAdapter.DatabaseExists(ADatabaseName: String):
  Boolean;
var
  Filter: OleVariant;
  ADataSet: TADODataSet;
begin
  Filter := VarArrayCreate([0, 0], varVariant);
  Filter[0] := ADatabaseName;
  ADataSet := TADODataSet.Create(nil);
  try
    Connection.OpenSchema(siCatalogs, Filter, EmptyParam, ADataSet);
    Result := ADataSet.RecordCount > 0;
  finally
    ADataSet.Free
  end;
end;

initialization
  GetLocaleFormatSettings(LOCALE_USER_DEFAULT, FormatSettings);
  FormatSettings.DecimalSeparator := '.';

  TovMSSQLServerADOAdapter.RegisterAdapter(TovMSSQLServerADOAdapter);
finalization
  TovMSSQLServerADOAdapter.UnregisterAdapter(TovMSSQLServerADOAdapter);
end.
