{$I ..\Core\OView.inc}

unit ovVUMLAdapter;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  {$IFDEF DELPHI6_OR_HIGHER}
  Variants,
  {$ENDIF}
  Forms, Dialogs, ovComponents, ovModelSync;

type
  TovVisualUMLAdapter = class (TovCustomModelAdapter)
  private
    FUMLApplication: Variant;
    function GetUMLModel: Variant;
    function GetUMLApplication: Variant;
    function IsAssociatedBy(ClassName: String; Association: Variant): Boolean;
  protected
    procedure AddAttribute(ClassName, AttributeName: String; const 
      AttributeParams: TAttributeParams); override;
    procedure AddClass(ClassName: String); override;
    procedure AddGeneralization(ClassName, TargetClassName: String); override;
    procedure AddRelation(ClassName, RelationName: String; const 
      RelationParams: TRelationParams); override;
    procedure DeleteAttribute(ClassName, AttributeName: String); override;
    procedure DeleteClass(ClassName: String); override;
    procedure DeleteGeneralization(ClassName, TargetClassName: String); 
      override;
    procedure DeleteRelation(ClassName, RelationName: String); override;
    procedure GetAttributeParams(ClassName, AttributeName: String; var 
      AttributeParams: TAttributeParams); override;
    procedure GetAttributes(ClassName: String; Attributes: TStrings); override;
    procedure GetClasses(Classes: TStrings); override;
    procedure GetGeneralizations(ClassName: String; Generalizations: TStrings); 
      override;
    procedure GetRelationParams(ClassName, RelationName: String; var 
      RelationParams: TRelationParams); override;
    procedure GetRelations(ClassName: String; Relations: TStrings); override;
    procedure UpdateAttribute(ClassName, AttributeName: String; const 
      AttributeParams: TAttributeParams); override;
    procedure UpdateRelation(ClassName, RelationName: String; const 
      RelationParams: TRelationParams); override;
  public
    procedure GetNameSpaces(ANameSpaces: TStrings); override;
    property UMLModel: Variant read GetUMLModel;
    property UMLApplication: Variant read GetUMLApplication;
  end;
  

implementation

uses ComObj, ovModel;

{$R *.dcr}

procedure ParseMultiplicity(Multiplicity: String; var Min, Max: Integer);

  function ConvertPart(S: String): Integer;
  begin
    if S = '' then Result := 0
    else
    if S = '*' then Result := Unlimited
    else Result := StrToInt(S)
  end;

  procedure GetParts(Multiplicity: String; var MinStr, MaxStr: String);
  var P: Integer;
  begin
    P := Pos('..', Multiplicity);
    if P = 0 then begin
      MinStr := Multiplicity;
      MaxStr := Multiplicity;
    end else begin
      MinStr := Trim(Copy(Multiplicity, 1, P-1));
      MaxStr := Trim(Copy(Multiplicity, P+2, Length(Multiplicity)));
    end;
  end;

var
  MinStr, MaxStr: String;
begin
  GetParts(Multiplicity, MinStr, MaxStr);
  if MinStr = '*' then MinStr := '0';

  Min := ConvertPart(MinStr);
  Max := ConvertPart(MaxStr);
end;

{
***************************** TovVisualUMLAdapter ******************************
}
procedure TovVisualUMLAdapter.AddAttribute(ClassName, AttributeName: String; 
  const AttributeParams: TAttributeParams);
begin
end;

procedure TovVisualUMLAdapter.AddClass(ClassName: String);
begin
end;

procedure TovVisualUMLAdapter.AddGeneralization(ClassName, TargetClassName: 
  String);
begin
end;

procedure TovVisualUMLAdapter.AddRelation(ClassName, RelationName: String; 
  const RelationParams: TRelationParams);
begin
end;

procedure TovVisualUMLAdapter.DeleteAttribute(ClassName, AttributeName: String);
begin
end;

procedure TovVisualUMLAdapter.DeleteClass(ClassName: String);
begin
end;

procedure TovVisualUMLAdapter.DeleteGeneralization(ClassName, TargetClassName: 
  String);
begin
end;

procedure TovVisualUMLAdapter.DeleteRelation(ClassName, RelationName: String);
begin
end;

procedure TovVisualUMLAdapter.GetAttributeParams(ClassName, AttributeName: 
  String; var AttributeParams: TAttributeParams);
var
  ModelObj, Attribute: Variant;
begin
  ModelObj := UMLModel.Classes[ClassName];
  Attribute := ModelObj.Attributes[AttributeName];
  
  with AttributeParams do
  begin
    AttrType := Attribute.AttrType;

    if Attribute.AttrLength <> ''
      then AttrSize := StrToInt(Attribute.AttrLength)
      else AttrSize := 0;
  
    if Attribute.TaggedValues.Exists('FieldName')
      then AttrField := Attribute.TaggedValues['FieldName'].TagValue
      else AttrField := ''
  end;
end;

procedure TovVisualUMLAdapter.GetAttributes(ClassName: String; Attributes: 
  TStrings);
var
  I: Integer;
  ModelObj: Variant;
begin
  Attributes.Clear;
  ModelObj := UMLModel.Classes[ClassName];
  for I := 1 to ModelObj.Attributes.Count do
    Attributes.Add(ModelObj.Attributes[I].AttrName);
end;

procedure TovVisualUMLAdapter.GetClasses(Classes: TStrings);
var
  I: Integer;
begin
  Classes.Clear;
  for I := 1 to UMLModel.Classes.Count do
    Classes.Add(UMLModel.Classes[I].ObjectName)
end;

procedure TovVisualUMLAdapter.GetGeneralizations(ClassName: String;
  Generalizations: TStrings);
var
  I: Integer;
begin
  Generalizations.Clear;
  //     
  for I := 1 to UMLModel.Generalizations.Count do
    if UMLModel.Generalizations[I].StartObjectName = ClassName
      then Generalizations.Add(UMLModel.Generalizations[I].EndObjectName);
end;

function TovVisualUMLAdapter.GetUMLModel: Variant;
begin
  Result := UMLApplication.Model;
end;

procedure TovVisualUMLAdapter.GetRelationParams(ClassName, RelationName: String;
  var RelationParams: TRelationParams);
var
  Multiplicity: String;
  I: Integer;
  Association: Variant;
begin
  for I := 1 to UMLModel.Associations.Count do
    if IsAssociatedBy(ClassName, UMLModel.Associations[I]) and
       (UMLModel.Associations[I].Name = RelationName)
    then Association := UMLModel.Associations[I];
  if VarIsEmpty(Association) then
    raise Exception.CreateFmt(' %s   %s  ', [RelationName, ClassName]);

  with RelationParams do
  begin
    if Association.StartObjectName = ClassName then begin

      RelationTarget := Association.EndObjectName;
      if Association.StartRole.Composition
        then RelationType := rtComposition
        else RelationType := rtAggregation;
      Multiplicity := Association.EndRole.Multiplicity;

    end else begin

      RelationTarget := Association.StartObjectName;
      if Association.EndRole.Composition
        then RelationType := rtComposition
        else RelationType := rtAggregation;
      Multiplicity := Association.StartRole.Multiplicity;

    end;

    ParseMultiplicity(Multiplicity, RelationMinCount, RelationMaxCount);

    if Association.TaggedValues.Exists('FieldName')
      then RelationField := Association.TaggedValues['FieldName'].TagValue
      else RelationField := ''
  end;
end;

procedure TovVisualUMLAdapter.GetRelations(ClassName: String; Relations:
  TStrings);
var
  I: Integer;
begin
  Relations.Clear;
  for I := 1 to UMLModel.Associations.Count do
    if IsAssociatedBy(ClassName, UMLModel.Associations[I])
      then Relations.Add(UMLModel.Associations[I].Name);
end;

{{  True,        .
     ,    ForwardDirection
 True,     ForwardDirection  False.
}
function TovVisualUMLAdapter.IsAssociatedBy(ClassName: String; Association: 
  Variant): Boolean;
var
  N1, N2: Boolean;
begin
  if (Association.StartObjectName <> ClassName) and (Association.EndObjectName <> ClassName)
  then begin
    //      
    Result := False;
    Exit;
  end;

  N1 := Association.StartRole.Navigable;
  N2 := Association.EndRole.Navigable;

  if Not N1 and Not N2 then begin
    Result := False;
  end else
  if N1 and N2 then begin
    Result := False;
  end else
    Result := (Association.StartObjectName = ClassName) and N2 or
              (Association.EndObjectName = ClassName) and N1;
end;

function TovVisualUMLAdapter.GetUMLApplication: Variant;
begin
  if VarIsEmpty(FUMLApplication) then
  try
    FUMLApplication := GetActiveOLEObject('VisualUML.CApplication');
  except
    MessageDlg('VisualUML  ', mtWarning, [mbOK], 0); Abort;
  end;

  Result := FUMLApplication;
end;

procedure TovVisualUMLAdapter.UpdateAttribute(ClassName, AttributeName: String;
  const AttributeParams: TAttributeParams);
begin
end;

procedure TovVisualUMLAdapter.UpdateRelation(ClassName, RelationName: String;
  const RelationParams: TRelationParams);
begin
end;

procedure TovVisualUMLAdapter.GetNameSpaces(ANameSpaces: TStrings);
begin
  ANameSpaces.Clear;
end;

end.
