unit ovCxControlLinks;

interface

uses Classes, SysUtils, ovDataSet, ovDataLinks, cxDBEdit, cxDBLookupComboBox;

type
  TcxDBControlDataLinkController = class(TovDataLinkController)
  private
    FLookupAttributes: String;
    FLookupDataObject: String;
    procedure SetupLookupProperties(Properties: TcxLookupComboBoxProperties; 
      Active: Boolean);
    procedure SetupDataBinding(DataBinding: TcxDBEditDataBinding; Active: Boolean);
  protected
    procedure SetActive(Value: Boolean); override;
  published
    property LookupAttributes: String read FLookupAttributes write
      FLookupAttributes;
    property LookupDataObject: String read FLookupDataObject write
      FLookupDataObject;
  end;

function GetDataBinding(AComponent: TComponent; APropName: String): 
  TcxDBEditDataBinding;

function GetLookupProperties(AComponent: TComponent; APropName: String): 
  TcxLookupComboBoxProperties;

implementation

uses TypInfo;

function GetDataBinding(AComponent: TComponent; APropName: String): 
  TcxDBEditDataBinding;
var
  AObject: TObject;
  PropInfo: PPropInfo;
begin
  Result := nil;
  PropInfo := GetPropInfo(AComponent, APropName);
  if PropInfo <> nil then
  begin
    AObject := GetObjectProp(AComponent, PropInfo);
    if AObject is TcxDBEditDataBinding then Result := TcxDBEditDataBinding(AObject)
  end
end;

function GetLookupProperties(AComponent: TComponent; APropName: String):
  TcxLookupComboBoxProperties;
var
  AObject: TObject;
  PropInfo: PPropInfo;
begin
  Result := nil;
  PropInfo := GetPropInfo(AComponent, APropName);
  if PropInfo <> nil then
  begin
    AObject := GetObjectProp(AComponent, PropInfo);
    if AObject is TcxLookupComboBoxProperties then Result := TcxLookupComboBoxProperties(AObject)
  end
end;

{
************************ TcxDBControlDataLinkController ************************
}

procedure TcxDBControlDataLinkController.SetActive(Value: Boolean);
var
  DataBinding: TcxDBEditDataBinding;
  Properties: TcxLookupComboBoxProperties;
begin
  with Link do
  begin
    if Component = nil then
      raise Exception.Create('     ');

    DataBinding := GetDataBinding(Component, 'DataBinding');
    if DataBinding <> nil then SetupDataBinding(DataBinding, Value);
    Properties := GetLookupProperties(Component, 'Properties');
    if Properties <> nil then
      SetupLookupProperties(Properties, Value)
    else
      if LookupDataObject <> '' then
        raise Exception.Create(' Lookup-    ' + Component.Name)
  end
end;

procedure TcxDBControlDataLinkController.SetupDataBinding(DataBinding: 
  TcxDBEditDataBinding; Active: Boolean);
begin
  if Not Active then
    DataBinding.DataSource := nil
  else begin
    if Link.DataObject = '' then
      raise Exception.Create('   DataObject  ');
    DataBinding.DataField := '';
    DataBinding.DataSource := Link.Owner.DataObjects.DataSources[Link.DataObject];
    if Link.Attribute <> '' then DataBinding.DataField := Link.Attribute;
  end
end;

procedure TcxDBControlDataLinkController.SetupLookupProperties(Properties:
  TcxLookupComboBoxProperties; Active: Boolean);
var
  LookupDataSet: TovDataSet;
begin
  with Link do
  if LookupDataObject <> ''then
  begin
    LookupDataSet := Owner.DataObjects[LookupDataObject];
    with TcxLookupComboBoxProperties(Properties) do begin
      KeyFieldNames := LookupDataSet.IdFieldName;
      if ListColumns.Count = 0 then //      ListColumns
        ListFieldNames := LookupAttributes;
      ListSource := Link.Owner.DataObjects.DataSources[LookupDataObject];
    end
  end
end;

initialization
  RegisterDataLinkController(TcxDBControlDataLinkController);
finalization
  UnregisterDataLinkController(TcxDBControlDataLinkController);
end.
