unit ovCxControlsDataLinksExplorer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  StdCtrls, Dialogs, AppEvnts, ToolWin, ImgList, ActnList, ComCtrls,
  ovDataLinks, ovDataLinksExplorer;

type
  TfrmCxControlsDataLinksExplorer = class(TfrmDataLinksExplorer)
    AddLabelsCmd: TAction;
    ToolButton3: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure AddLabelsCmdExecute(Sender: TObject);
  private
    procedure AddComponentLabels;
    procedure AddComponentLabel(AComponent: TWinControl);
    { Private declarations }
  protected
    function CanLink(AComponent: TComponent): Boolean; override;
    function GetObjectInfo(AObject: TPersistent; var ATitle, ADataObject, 
      AnAttribute, ALookupDataObject, ALookupAttributes: String): Boolean; override;
  public
    procedure AddLookupLink(ALookupDataObjectName, ALookupAttributes: String; 
      ObjectIndex: Integer); override;
    { Public declarations }
  end;

var
  frmCxControlsDataLinksExplorer: TfrmCxControlsDataLinksExplorer;

implementation

uses TypInfo, ovUtils, ovCxControlLinks, ovDataLinksEditor, cxDBEdit;

{$R *.dfm}

procedure TfrmCxControlsDataLinksExplorer.FormCreate(Sender: TObject);
begin
  inherited;
  DataLinkControllerClass := TcxDBControlDataLinkController;
end;

{{
  ,   DataBinding: TcxDBEditDataBinding
 Properties: TcxLookupComboBoxProperties
}
function TfrmCxControlsDataLinksExplorer.CanLink(AComponent: TComponent): Boolean;
begin
  Result := Assigned(GetDataBinding(AComponent, 'DataBinding')) or
    Assigned(GetLookupProperties(AComponent, 'Properties'))
end;

{{
   ,    ObjectIndex 
.       ,    ,
    .   - TDBGrid,   
  ,       .
}
procedure TfrmCxControlsDataLinksExplorer.AddLookupLink(ALookupDataObjectName, 
  ALookupAttributes: String; ObjectIndex: Integer);
var
  AObject: TObject;
  Link: TovDataLink;
begin
  Link := GetLinkAt(ObjectIndex);
  if Link = nil then begin
    AObject := GetObjectAt(ObjectIndex);
    if AObject is TComponent then
      Link := DataLinks.AddLink(DataLinkControllerClass, TComponent(AObject), '', '')
    else
      Exit;
  end;
  if Link.Controller is TcxDBControlDataLinkController then
  with TcxDBControlDataLinkController(Link.Controller) do
  begin
    LookupDataObject := ALookupDataObjectName;
    LookupAttributes := ALookupAttributes;
  end;
  ShowComponentList;
end;

function TfrmCxControlsDataLinksExplorer.GetObjectInfo(AObject: TPersistent; 
  var ATitle, ADataObject, AnAttribute, ALookupDataObject, ALookupAttributes: 
  String): Boolean;
var
  Link: TovDataLink;
begin
  Result := inherited GetObjectInfo(AObject,
    ATitle, ADataObject, AnAttribute, ALookupDataObject, ALookupAttributes);

  Link := FindLink(AObject);
  if (Link <> nil) and (Link.Controller is TcxDBControlDataLinkController) then begin
    with TcxDBControlDataLinkController(Link.Controller) do
    begin
      ALookupDataObject := LookupDataObject;
      ALookupAttributes := LookupAttributes;
    end;
  end
end;

procedure TfrmCxControlsDataLinksExplorer.AddLabelsCmdExecute(Sender: TObject);
begin
  inherited;
  AddComponentLabels
end;

procedure TfrmCxControlsDataLinksExplorer.AddComponentLabels;
var
  I: Integer;
begin
  with ComponentList do
    for I := 0 to Items.Count - 1 do
      if Items[I].Selected then
        AddComponentLabel(GetObjectAt(I) as TWinControl);
end;

procedure TfrmCxControlsDataLinksExplorer.AddComponentLabel(AComponent:
  TWinControl);
var
  Link: TovDataLink;
  ALabel: TLabel;
begin
  Link := DataLinks.FindLink(AComponent);
  if Link <> nil then
  if AComponent is TcxDBCheckBox then
    TcxDBCheckBox(AComponent).Caption := Link.Attribute
  else begin
    ALabel := TLabel.Create(AComponent.Owner);
    GenerateComponentName(ALabel);
    with ALabel do begin
      Parent := AComponent.Parent;
      Left := AComponent.Left;
      Top := AComponent.Top - Height;
      FocusControl := AComponent;
      Caption := Link.Attribute;
    end
  end
end;

initialization
  RegisterDataLinkExplorer('  cx-', TfrmCxControlsDataLinksExplorer);
finalization
  UnregisterDataLinkExplorer(TfrmCxControlsDataLinksExplorer);
end.
