unit ovCxDBTreeListPopupMenu;

interface

uses Classes, Menus, cxDBTL, ovCxDBTreeListCustomization;

type
  TovCxDBTreeListPopupMenu = class(TPopupMenu)
  private
    FCustomization: TovCxDBTreeListCustomization;
    FAllowColumnSetup: Boolean;
    procedure SetupItemsClick(Sender: TObject);
    procedure SaveGridSettingsClick(Sender: TObject);
    procedure LoadGridSettingsClick(Sender: TObject);
    procedure SetCustomization(Value: TovCxDBTreeListCustomization);
    procedure RenameColumnClick(Sender: TObject);
    procedure RenameHitColumn;
    function GetTreeList: TcxDBTreeList;
  protected
    procedure Loaded; override;
    procedure SetupPopupMenus; dynamic;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    property TreeList: TcxDBTreeList read GetTreeList;
  published
    property Customization: TovCxDBTreeListCustomization read FCustomization write
      SetCustomization;
    property AllowColumnSetup: Boolean read FAllowColumnSetup write
      FAllowColumnSetup default True;
  end;

implementation

uses Dialogs;

constructor TovCxDBTreeListPopupMenu.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAllowColumnSetup := True;
end;

procedure TovCxDBTreeListPopupMenu.SetCustomization(Value: 
  TovCxDBTreeListCustomization);
begin
  if FCustomization <> Value then
  begin
    FCustomization := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovCxDBTreeListPopupMenu.Loaded;
begin
  inherited;
  if Not (csDesigning in ComponentState) then SetupPopupMenus;
end;

procedure TovCxDBTreeListPopupMenu.SetupPopupMenus;
var
  PopupMenu: TPopupMenu;
  MenuItem: TMenuItem;
begin
  PopupMenu := Self;

  if AllowColumnSetup then
  begin
    MenuItem := TMenuItem.Create(Self);
    MenuItem.Caption := ' ..';
    MenuItem.OnClick := SetupItemsClick;
    PopupMenu.Items.Add(MenuItem);
  end;

  MenuItem := TMenuItem.Create(Self);
  MenuItem.Caption := ' ..';
  MenuItem.OnClick := RenameColumnClick;
  PopupMenu.Items.Add(MenuItem);

  MenuItem := TMenuItem.Create(Self);
  MenuItem.Caption := '-';
  PopupMenu.Items.Add(MenuItem);

  MenuItem := TMenuItem.Create(Self);
  MenuItem.Caption := '-';
  PopupMenu.Items.Add(MenuItem);

  MenuItem := TMenuItem.Create(Self);
  MenuItem.Caption := ' ';
  MenuItem.OnClick := SaveGridSettingsClick;
  PopupMenu.Items.Add(MenuItem);

  MenuItem := TMenuItem.Create(Self);
  MenuItem.Caption := ' ';
  MenuItem.OnClick := LoadGridSettingsClick;
  PopupMenu.Items.Add(MenuItem);
end;

procedure TovCxDBTreeListPopupMenu.SetupItemsClick(Sender: TObject);
begin
  if Assigned(Customization) then Customization.SelectColumns;
end;

procedure TovCxDBTreeListPopupMenu.RenameColumnClick(Sender: TObject);
begin
  if Assigned(Customization) then RenameHitColumn
end;

procedure TovCxDBTreeListPopupMenu.SaveGridSettingsClick(Sender: TObject);
begin
  if Assigned(Customization) then Customization.SaveSettings;
end;

procedure TovCxDBTreeListPopupMenu.LoadGridSettingsClick(Sender: TObject);
begin
  if Assigned(Customization) then Customization.LoadSettings;
end;

procedure TovCxDBTreeListPopupMenu.RenameHitColumn;
var
  NewCaption: String;
begin
  TreeList.HitTest.HitPoint := TreeList.ScreenToClient(PopupPoint);
  if TreeList.HitTest.HitAtColumn or TreeList.HitTest.HitAtColumnHeader then
  begin
    NewCaption := TreeList.HitTest.HitColumn.Caption.Text;
    if InputQuery(' ', ' ', NewCaption) then
      TreeList.HitTest.HitColumn.Caption.Text := NewCaption;
  end;
end;

function TovCxDBTreeListPopupMenu.GetTreeList: TcxDBTreeList;
begin
  Result := Customization.TreeList
end;

procedure TovCxDBTreeListPopupMenu.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if AComponent = Customization then Customization := nil
end;

end.
