unit ovCxGridCustomization;

interface

uses Classes, DB, SysUtils, ovDataSet, ovDataLinks, ovCustomization,
  cxGrid, cxGridCustomView, cxGridCustomTableView, cxGridTableView,
  cxGridDBTableView, cxGridDBBandedTableView, cxGridDBCardView, Contnrs;

type
  TovCxGridCustomization = class;

  {{
    ,       GridView.
  }
  TovCxGridViewCustomization = class (TComponent)
  private
    FGridView: TcxCustomGridView;
    FObjectExposer: TovExposer;
    FCustomization: TovCxGridCustomization;
    FSelectedAttributes: String;
    procedure GridViewItemsChanged;
    procedure DoCreateGridViewItems(Attributes, ColumnNames: String);
    function GetRequiredFields: String;
    procedure SetGridView(const Value: TcxCustomGridView);
  protected
    procedure SaveSettings(RegistryKey: String);
    procedure LoadSettings(RegistryKey: String; DataLinks: TovDataLinks);
    procedure EraseSettings(RegistryKey: String);
    procedure SelectGridViewItems;
    procedure ClearGridViewItems; virtual; abstract;
    procedure SetupGridViewItems; virtual; abstract;
    procedure SetupObjectExposer; virtual; abstract;
    procedure CreateGridViewItems(Attributes, ColumnNames: TStrings); virtual;
      abstract;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function GetItemNames: String; virtual; abstract;
  public
    constructor Create(ACustomization: TovCxGridCustomization; AGridView:
      TcxCustomGridView);
    property ObjectExposer: TovExposer read FObjectExposer;
    property Customization: TovCxGridCustomization read FCustomization;
    property GridView: TcxCustomGridView read FGridView write SetGridView;
  end;

  TGetRequiredFieldsEvent = procedure (Sender:TovCxGridViewCustomization;
    RequiredFields: TStrings) of object;

  TovCxGridCustomization = class(TovCustomization)
  private
    FDataLinks: TovDataLinks;
    FRestoreSettingsMode: Boolean;
    FOnCreateGridViewItems: TNotifyEvent;
    FOnGetRequiredFields: TGetRequiredFieldsEvent;
    FGridViewCustomizations: TComponentList;
    function GetGrid: TcxGrid;
    procedure SetGrid(Value: TcxGrid);
    procedure SetDataLinks(const Value: TovDataLinks);
    function GetGridViewCustomizations(Index: Integer): TovCxGridViewCustomization;
    function FindGridViewCustomization(GridView: TcxCustomGridView):
      TovCxGridViewCustomization;
  protected
    function GetGridViewCustomization(GridView: TcxCustomGridView):
      TovCxGridViewCustomization;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    procedure SaveSettings; override;
    procedure LoadSettings; override;
    procedure SelectGridViewItems(AGridView: TcxCustomGridView);
    procedure RestoreDefaultSettings;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property GridViewCustomizations[Index: Integer]: TovCxGridViewCustomization
      read GetGridViewCustomizations;
  published
    property Grid: TcxGrid read GetGrid write SetGrid;
    property DataLinks: TovDataLinks read FDataLinks write SetDataLinks;
    property OnGetRequiredFields: TGetRequiredFieldsEvent read FOnGetRequiredFields
      write FOnGetRequiredFields;
    {{
             
      /         .
           ,  
        ,       .
    }
    property OnCreateGridViewItems: TNotifyEvent read FOnCreateGridViewItems write
      FOnCreateGridViewItems;
  end;

  TovCxGridTableViewCustomization = class(TovCxGridViewCustomization)
  private
    function GetGridView: TcxGridDBTableView;
    procedure GridTableItemGetStoredProperties(Sender: TcxCustomGridTableItem;
      AProperties: TStrings);
    procedure GridTableItemGetStoredPropertyValue(
      Sender: TcxCustomGridTableItem; const AName: string;
      var AValue: Variant);
    procedure GridTableItemSetStoredPropertyValue(Sender: TcxCustomGridTableItem;
      const AName: string; const AValue: Variant);
    procedure GridViewInitStoredObject(Sender: TcxCustomGridView; AObject: TObject);
    procedure CreateGridViewItem(FieldName, ItemName: String; Hidden: Boolean);
  protected
    procedure ClearGridViewItems; override;
    procedure SetupGridViewItems; override;
    procedure CreateGridViewItems(Attributes, ColumnNames: TStrings); override;
    procedure SetupObjectExposer; override;
    function GetItemNames: String; override;
  public
    destructor Destroy; override;
    property GridView: TcxGridDBTableView read GetGridView;
  end;

  TovCxGridBandedTableViewCustomization = class(TovCxGridViewCustomization)
  private
    function GetGridView: TcxGridDBBandedTableView;
    procedure GridTableItemGetStoredProperties(
      Sender: TcxCustomGridTableItem; AProperties: TStrings);
    procedure GridTableItemGetStoredPropertyValue(
      Sender: TcxCustomGridTableItem; const AName: string;
      var AValue: Variant);
    procedure GridTableItemSetStoredPropertyValue(
      Sender: TcxCustomGridTableItem; const AName: string;
      const AValue: Variant);
    procedure GridViewInitStoredObject(Sender: TcxCustomGridView;
      AObject: TObject);
    procedure CreateGridViewItem(FieldName, ItemName: String; Hidden: Boolean);
  protected
    procedure ClearGridViewItems; override;
    procedure SetupGridViewItems; override;
    procedure CreateGridViewItems(Attributes, ColumnNames: TStrings); override;
    procedure SetupObjectExposer; override;
    function GetItemNames: String; override;
  public
    property GridView: TcxGridDBBandedTableView read GetGridView;
    destructor Destroy; override;
  end;

  TovCxGridCardViewCustomization = class(TovCxGridViewCustomization)
  private
    function GetGridView: TcxGridDBCardView;
    procedure GridTableItemGetStoredProperties(Sender: TcxCustomGridTableItem;
      AProperties: TStrings);
    procedure GridTableItemGetStoredPropertyValue(Sender: TcxCustomGridTableItem;
      const AName: string; var AValue: Variant);
    procedure GridTableItemSetStoredPropertyValue(Sender: TcxCustomGridTableItem;
      const AName: string; const AValue: Variant);
    procedure GridViewInitStoredObject(Sender: TcxCustomGridView; AObject: TObject);
    procedure CreateGridViewItem(FieldName, ItemName: String; Hidden: Boolean);
  protected
    procedure ClearGridViewItems; override;
    procedure SetupGridViewItems; override;
    procedure CreateGridViewItems(Attributes, ColumnNames: TStrings); override;
    procedure SetupObjectExposer; override;
    function GetItemNames: String; override;
  public
    destructor Destroy; override;
    property GridView: TcxGridDBCardView read GetGridView;
  end;

implementation

uses ovUtils, ovAttributeSelectionDlg, cxCheckBox, ovDataObjects,
  cxGridCardView, Registry;

resourcestring
  SSelectedAttributesRegKeyName = 'SelectedAttributes';
  SColumnNamesRegKeyName = 'ColumnNames';

{ TovCxGridCustomization }

constructor TovCxGridCustomization.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FGridViewCustomizations := TComponentList.Create(False);
end;

destructor TovCxGridCustomization.Destroy;
begin
  FreeAndNil(FGridViewCustomizations);
  inherited Destroy;
end;

function TovCxGridCustomization.GetGrid: TcxGrid;
begin
  Result := TcxGrid(SubjectComponent);
end;

procedure TovCxGridCustomization.SetGrid(Value: TcxGrid);
begin
  SubjectComponent := Value
end;

procedure TovCxGridCustomization.SetDataLinks(const Value: TovDataLinks);
begin
  if FDataLinks <> Value then
  begin
    FDataLinks := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovCxGridCustomization.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = DataLinks) then DataLinks := nil
end;

function TovCxGridCustomization.GetGridViewCustomizations(Index: Integer):
  TovCxGridViewCustomization;
begin
  Result := TovCxGridViewCustomization(FGridViewCustomizations[Index]);
end;

function TovCxGridCustomization.FindGridViewCustomization(GridView:
  TcxCustomGridView): TovCxGridViewCustomization;
var
  I: Integer;
begin
  for I := 0 to FGridViewCustomizations.Count - 1 do
    if GridViewCustomizations[I].GridView = GridView then
    begin
      Result := GridViewCustomizations[I];
      Exit;
    end;
  Result := nil;
end;

function TovCxGridCustomization.GetGridViewCustomization(GridView:
  TcxCustomGridView): TovCxGridViewCustomization;
begin
  Result := FindGridViewCustomization(GridView);
  if Result = nil then
  begin
    if GridView is TcxGridDBTableView then
      Result := TovCxGridTableViewCustomization.Create(Self, TcxGridDBTableView(GridView)) else
    if GridView is TcxGridDBBandedTableView then
      Result := TovCxGridBandedTableViewCustomization.Create(Self, TcxGridDBBandedTableView(GridView)) else
    if GridView is TcxGridDBCardView then
      Result := TovCxGridCardViewCustomization.Create(Self, TcxGridDBCardView(GridView)) else
    raise Exception.Create('   ');
    FGridViewCustomizations.Add(Result);
  end;
  Result.SetupObjectExposer;
end;

{***************************************************************************** }
{                                                                }
{ **************************************************************************** }

procedure TovCxGridCustomization.SaveSettings;
var
  I: Integer;
begin
  if Not FRestoreSettingsMode then
  begin
    inherited;
    if Assigned(Grid) then
    for I := 0 to Grid.ViewCount - 1 do
      with GetGridViewCustomization(Grid.Views[I]) do
        SaveSettings(GetFullRegistryKey);
  end;
end;

procedure TovCxGridCustomization.LoadSettings;
var
  I: Integer;
begin
  try
    if Assigned(Grid) then
    for I := 0 to Grid.ViewCount - 1 do
      with GetGridViewCustomization(Grid.Views[I]) do
        LoadSettings(GetFullRegistryKey, DataLinks);
  except
    {      -       }
    RestoreDefaultSettings;
    raise;
  end;

  inherited;
end;

procedure TovCxGridCustomization.RestoreDefaultSettings;
var
  I: Integer;
begin
  if Assigned(Grid) then
  for I := 0 to Grid.ViewCount - 1 do
    with GetGridViewCustomization(Grid.Views[I]) do
    begin
      EraseSettings(GetFullRegistryKey);
      FRestoreSettingsMode := True;
    end;
end;

{{
       .
}
procedure TovCxGridCustomization.SelectGridViewItems(AGridView:
  TcxCustomGridView);
begin
  with GetGridViewCustomization(AGridView) do
    SelectGridViewItems
end;

{ TovCxGridViewCustomization }

constructor TovCxGridViewCustomization.Create(ACustomization:
  TovCxGridCustomization; AGridView: TcxCustomGridView);
begin
  inherited Create(ACustomization);
  GridView := AGridView;
  FCustomization := ACustomization;
end;

procedure TovCxGridViewCustomization.SetGridView(const Value:
  TcxCustomGridView);
begin
  if FGridView <> Value then
  begin
    FGridView := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovCxGridViewCustomization.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = GridView) then GridView := nil
end;

{{
       , 
        (/) 
  .
}
procedure TovCxGridViewCustomization.SelectGridViewItems;
var
  Selection: String;
begin
  with ObjectExposer do
  begin
    if FSelectedAttributes = '' then
      Selection := ObjectAttributes
    else
      Selection := FSelectedAttributes;
    if SelectObjectAttributes(Model,
      Model.DomainModel.GetObject(ObjectClassName), Selection,
        ' ', [soExpandRelations])
    then begin
      {  FSelectedAttributes  ,   }
      FSelectedAttributes := Selection;
      {  ObjectAttributes       }
      ObjectAttributes := CombineSubStr(FSelectedAttributes, ',', GetRequiredFields);
      { TODO :    }
      Open;
      {      ()   
               }
      DoCreateGridViewItems(FSelectedAttributes, '');
      GridViewItemsChanged;
    end;
  end;
end;

{{
       .
}
procedure TovCxGridViewCustomization.DoCreateGridViewItems(Attributes,
  ColumnNames: String);
var
  AttributeList, ColumnList: TStrings;
begin
  AttributeList := TStringList.Create;
  ColumnList := TStringList.Create;
  try
    AttributesToList(Attributes, AttributeList);
    ColumnList.CommaText := ColumnNames;
    CreateGridViewItems(AttributeList, ColumnList);
  finally
    ColumnList.Free;
    AttributeList.Free;
  end;
end;

procedure TovCxGridViewCustomization.SaveSettings(RegistryKey: String);
var
  RegPath: String;
begin
  if ObjectExposer <> nil then
  begin
    SetupGridViewItems;
    RegPath := RegistryKey+'\'+GridView.Name;
    GridView.StoreToRegistry(RegPath, True, [gsoUseFilter, gsoUseSummary]);
    if FSelectedAttributes <> '' then
      SaveStringToRegistry(RegPath, SSelectedAttributesRegKeyName, FSelectedAttributes)
    else
      {        -      }
      SaveStringToRegistry(RegPath, SSelectedAttributesRegKeyName, ObjectExposer.ObjectAttributes);
    {      }
    SaveStringToRegistry(RegPath, SColumnNamesRegKeyName, GetItemNames);
  end;
end;

procedure TovCxGridViewCustomization.EraseSettings(RegistryKey: String);
begin
  with TRegistry.Create do
  try
    DeleteKey(RegistryKey+'\'+GridView.Name)
  finally
    Free
  end;
end;

procedure TovCxGridViewCustomization.LoadSettings(RegistryKey: String;
  DataLinks: TovDataLinks);
var
  DataLink: TovDataLink;
  Attributes: String;
  DataObject: TovDataObject;
  ColumnNames: String;
  RegPath: String;
begin
  RegPath := RegistryKey+'\'+GridView.Name;
  FSelectedAttributes := LoadStringFromRegistry(RegPath, SSelectedAttributesRegKeyName);
  if FSelectedAttributes = '' then Exit; //     
  Attributes := CombineSubStr(FSelectedAttributes, ',', GetRequiredFields);

  if Assigned(DataLinks) then
  begin
    DataLink := DataLinks.FindLink(GridView);
    if Assigned(DataLink) then
    begin
      DataObject := DataLinks.DataObjects.FindObjectByName(DataLink.DataObject);
      if Assigned(DataObject) then
      begin
        { TODO :  ,     -   , 
           Exposer'     ObjectAttributes }
        if DataObject.DataSet <> nil then
          (DataObject.DataSet as TovExposer).ObjectAttributes := Attributes;

        DataObject.ObjectAttributes := Attributes;
        DataObject.Open;
        FObjectExposer := DataObject.DataSet as TovExposer;
      end;
    end;
  end else
  begin
    ObjectExposer.ObjectAttributes := Attributes;
    ObjectExposer.Open;
  end;

  {   ,      ; 
         RestoreFromRegistry,  
       ,   ,  
             }
  ColumnNames := LoadStringFromRegistry(RegPath, SColumnNamesRegKeyName);

  ClearGridViewItems;
  {         }
  DoCreateGridViewItems(FSelectedAttributes, ColumnNames);
  SetupGridViewItems;
  {     2-  3-       }
  GridView.RestoreFromRegistry(RegPath, False, False, [gsoUseFilter, gsoUseSummary]);
  GridViewItemsChanged;
end;

procedure TovCxGridViewCustomization.GridViewItemsChanged;
begin
  if Assigned(Customization.FOnCreateGridViewItems) then Customization.FOnCreateGridViewItems(Self);
end;

{{
     (),   
,   Customization.OnGetRequiredFields.
}
function TovCxGridViewCustomization.GetRequiredFields: String;
var
  RequiredFields: TStringList;
begin
  Result := '';
  if Assigned(Customization.FOnGetRequiredFields) then
  begin
    RequiredFields := TStringList.Create;
    try
      Customization.FOnGetRequiredFields(Self, RequiredFields);
      Result := AttributesFromList(RequiredFields);
    finally
      RequiredFields.Free;
    end;
  end;
end;

{***************************************************************************** }
{                         TcxGridDBTableView                         }
{ **************************************************************************** }

destructor TovCxGridTableViewCustomization.Destroy;
var
  I: Integer;
begin
  if Assigned(GridView) then
  begin
    {         ,
       }
    GridView.OnInitStoredObject := nil;
    for I := 0 to GridView.ColumnCount - 1 do
    with GridView.Columns[I] do
    begin
      OnGetStoredProperties := nil;
      OnGetStoredPropertyValue := nil;
      OnSetStoredPropertyValue := nil;
    end;
  end;

  inherited Destroy;
end;

procedure TovCxGridTableViewCustomization.SetupObjectExposer;
begin
  FObjectExposer := GridView.DataController.DataSet as TovExposer
end;

function TovCxGridTableViewCustomization.GetGridView: TcxGridDBTableView;
begin
  Result := inherited GridView as TcxGridDBTableView
end;

procedure TovCxGridTableViewCustomization.ClearGridViewItems;
begin
  GridView.ClearItems;
end;

procedure TovCxGridTableViewCustomization.CreateGridViewItems(Attributes,
  ColumnNames: TStrings);
var
  I: Integer;
  Column: TcxGridDBColumn;
begin
  {    }
  for I := GridView.ColumnCount - 1 downto 0 do
    if Attributes.IndexOf(GridView.Columns[I].DataBinding.FieldName) < 0 then
      GridView.Columns[I].Free;

  {    }
  for I := 0 to Attributes.Count - 1 do
  begin
    Column := GridView.GetColumnByFieldName(Attributes[I]);
    if Column = nil then
      if ColumnNames.Count = 0 then
        CreateGridViewItem(Attributes[I], '', False)
      else
        CreateGridViewItem(Attributes[I], ColumnNames[I], False)
    else
      Column.Visible := True
  end;

  if dsoExposeIdField in ObjectExposer.Options then
    if GridView.GetColumnByFieldName(ObjectExposer.IdFieldName) = nil then
      CreateGridViewItem(ObjectExposer.IdFieldName, '', True);
end;

{{
       (   
 ).
}
procedure TovCxGridTableViewCustomization.CreateGridViewItem(FieldName,
  ItemName: String; Hidden: Boolean);
var
  AColumn: TcxGridDBColumn;
begin
  AColumn := GridView.CreateColumn;
  if ItemName = '' then
    GenerateComponentName(AColumn, GridView.Name + 'Column')
  else
    AColumn.Name := ItemName;
  AColumn.DataBinding.FieldName := FieldName;

  case ObjectExposer.FieldByName(FieldName).DataType of
    ftBoolean:
    begin
      AColumn.PropertiesClass := TcxCheckBoxProperties;
      (AColumn.Properties as TcxCheckBoxProperties).NullStyle := nssUnchecked;
    end;
  end;
  AColumn.HeaderAlignmentHorz := taCenter;
  AColumn.Hidden := Hidden;
  AColumn.Visible := Not Hidden;
  GridView.ApplyBestFit(AColumn);
end;

{{
        .
}
procedure TovCxGridTableViewCustomization.GridViewInitStoredObject(Sender:
  TcxCustomGridView; AObject: TObject);
begin
  with AObject as TcxGridDBColumn do
  begin
    OnGetStoredProperties := GridTableItemGetStoredProperties;
    OnGetStoredPropertyValue := GridTableItemGetStoredPropertyValue;
    OnSetStoredPropertyValue := GridTableItemSetStoredPropertyValue;
    HeaderAlignmentHorz := taCenter;
  end;
end;

{{
     /   .
}
procedure TovCxGridTableViewCustomization.SetupGridViewItems;
var
  I: Integer;
begin
  GridView.OnInitStoredObject := GridViewInitStoredObject;
  for I := 0 to GridView.ColumnCount - 1 do
  with GridView.Columns[I] do
  begin
    OnGetStoredProperties := GridTableItemGetStoredProperties;
    OnGetStoredPropertyValue := GridTableItemGetStoredPropertyValue;
    OnSetStoredPropertyValue := GridTableItemSetStoredPropertyValue;
  end;
end;

{{
        
     ,    .
}
procedure TovCxGridTableViewCustomization.GridTableItemGetStoredProperties(
  Sender: TcxCustomGridTableItem; AProperties: TStrings);
begin
  AProperties.Add('FieldName');
  AProperties.Add('Caption');
end;

procedure TovCxGridTableViewCustomization.GridTableItemGetStoredPropertyValue(
  Sender: TcxCustomGridTableItem; const AName: string; var AValue: Variant);
begin
  if AName = 'FieldName' then AValue := TcxGridDBColumn(Sender).DataBinding.FieldName else
  if AName = 'Caption' then AValue := TcxGridDBColumn(Sender).Caption
end;

procedure TovCxGridTableViewCustomization.GridTableItemSetStoredPropertyValue(
  Sender: TcxCustomGridTableItem; const AName: string; const AValue: Variant);
var
  AColumn: TcxGridDBColumn;
begin
  AColumn := TcxGridDBColumn(Sender);
  if AName = 'FieldName' then
  begin
    AColumn.DataBinding.FieldName := AValue;
    case ObjectExposer.FieldByName(AValue).DataType of
      ftBoolean:
      begin
        AColumn.PropertiesClass := TcxCheckBoxProperties;
        (AColumn.Properties as TcxCheckBoxProperties).NullStyle := nssUnchecked;
      end else
        AColumn.PropertiesClass := nil;
    end;
  end
  else
  if AName = 'Caption' then AColumn.Caption := AValue
end;

function TovCxGridTableViewCustomization.GetItemNames: String;
var
  ColumnList: TStrings;
  I: Integer;
begin
  ColumnList := TStringList.Create;
  try
    for I := 0 to GridView.ColumnCount - 1 do
      ColumnList.Add(GridView.Columns[I].Name);
    Result := ColumnList.CommaText
  finally
    ColumnList.Free;
  end;
end;

{***************************************************************************** }
{                      TcxGridDBBandedTableView                      }
{ **************************************************************************** }

destructor TovCxGridBandedTableViewCustomization.Destroy;
var
  I: Integer;
begin
  if Assigned(GridView) then
  begin
    {         ,
       }
    GridView.OnInitStoredObject := nil;
    for I := 0 to GridView.ColumnCount - 1 do
    with GridView.Columns[I] do
    begin
      OnGetStoredProperties := nil;
      OnGetStoredPropertyValue := nil;
      OnSetStoredPropertyValue := nil;
    end;
  end;

  inherited Destroy;
end;

procedure TovCxGridBandedTableViewCustomization.SetupObjectExposer;
begin
  FObjectExposer := GridView.DataController.DataSet as TovExposer
end;

function TovCxGridBandedTableViewCustomization.GetGridView: TcxGridDBBandedTableView;
begin
  Result := inherited GridView as TcxGridDBBandedTableView
end;

procedure TovCxGridBandedTableViewCustomization.ClearGridViewItems;
begin
  GridView.ClearItems;
end;

{{
   BandedTableView.  ,   
     CreateGridViewItems  TableView,
  GridView       
    GridView.
}
procedure TovCxGridBandedTableViewCustomization.CreateGridViewItems(Attributes,
  ColumnNames: TStrings);
var
  I: Integer;
  Column: TcxGridDBBandedColumn;
begin
  {    }
  for I := GridView.ColumnCount - 1 downto 0 do
    if Attributes.IndexOf(GridView.Columns[I].DataBinding.FieldName) < 0 then
      GridView.Columns[I].Free;

  {    }
  for I := 0 to Attributes.Count - 1 do
  begin
    Column := GridView.GetColumnByFieldName(Attributes[I]);
    if Column = nil then
      if ColumnNames.Count = 0 then
        CreateGridViewItem(Attributes[I], '', False)
      else
        CreateGridViewItem(Attributes[I], ColumnNames[I], False)
    else
      Column.Visible := True
  end;

  if dsoExposeIdField in ObjectExposer.Options then
    if GridView.GetColumnByFieldName(ObjectExposer.IdFieldName) = nil then
      CreateGridViewItem(ObjectExposer.IdFieldName, '', True);
end;

{{
       (   
 ).
}
procedure TovCxGridBandedTableViewCustomization.CreateGridViewItem(FieldName,
  ItemName: String; Hidden: Boolean);
var
  AColumn: TcxGridDBBandedColumn;
begin
  AColumn := GridView.CreateColumn;
  if ItemName = '' then
    GenerateComponentName(AColumn, GridView.Name + 'Column')
  else
    AColumn.Name := ItemName;
  AColumn.DataBinding.FieldName := FieldName;

  case ObjectExposer.FieldByName(FieldName).DataType of
    ftBoolean:
    begin
      AColumn.PropertiesClass := TcxCheckBoxProperties;
      (AColumn.Properties as TcxCheckBoxProperties).NullStyle := nssUnchecked;
    end;
  end;
  AColumn.HeaderAlignmentHorz := taCenter;
  AColumn.Hidden := Hidden;
  AColumn.Visible := Not Hidden;
  GridView.ApplyBestFit(AColumn);
end;

{{
        .
}
procedure TovCxGridBandedTableViewCustomization.GridViewInitStoredObject(Sender:
  TcxCustomGridView; AObject: TObject);
begin
  with AObject as TcxGridDBBandedColumn do
  begin
    OnGetStoredProperties := GridTableItemGetStoredProperties;
    OnGetStoredPropertyValue := GridTableItemGetStoredPropertyValue;
    OnSetStoredPropertyValue := GridTableItemSetStoredPropertyValue;
    HeaderAlignmentHorz := taCenter;
  end;
end;

{{
     /   .
}
procedure TovCxGridBandedTableViewCustomization.SetupGridViewItems;
var
  I: Integer;
begin
  GridView.OnInitStoredObject := GridViewInitStoredObject;
  for I := 0 to GridView.ColumnCount - 1 do
  with GridView.Columns[I] do
  begin
    OnGetStoredProperties := GridTableItemGetStoredProperties;
    OnGetStoredPropertyValue := GridTableItemGetStoredPropertyValue;
    OnSetStoredPropertyValue := GridTableItemSetStoredPropertyValue;
  end;
end;

{{
        
     ,    .
}
procedure TovCxGridBandedTableViewCustomization.GridTableItemGetStoredProperties(
  Sender: TcxCustomGridTableItem; AProperties: TStrings);
begin
  AProperties.Add('FieldName');
  AProperties.Add('Caption');
end;

procedure TovCxGridBandedTableViewCustomization.GridTableItemGetStoredPropertyValue(
  Sender: TcxCustomGridTableItem; const AName: string; var AValue: Variant);
begin
  if AName = 'FieldName' then AValue := TcxGridDBBandedColumn(Sender).DataBinding.FieldName else
  if AName = 'Caption' then AValue := TcxGridDBBandedColumn(Sender).Caption
end;

procedure TovCxGridBandedTableViewCustomization.GridTableItemSetStoredPropertyValue(
  Sender: TcxCustomGridTableItem; const AName: string; const AValue: Variant);
var
  AColumn: TcxGridDBBandedColumn;
begin
  AColumn := TcxGridDBBandedColumn(Sender);
  if AName = 'FieldName' then
  begin
    AColumn.DataBinding.FieldName := AValue;
    case ObjectExposer.FieldByName(AValue).DataType of
      ftBoolean:
      begin
        AColumn.PropertiesClass := TcxCheckBoxProperties;
        (AColumn.Properties as TcxCheckBoxProperties).NullStyle := nssUnchecked;
      end else
        AColumn.PropertiesClass := nil;
    end;
  end
  else
  if AName = 'Caption' then AColumn.Caption := AValue
end;

function TovCxGridBandedTableViewCustomization.GetItemNames: String;
var
  ColumnList: TStrings;
  I: Integer;
begin
  ColumnList := TStringList.Create;
  try
    for I := 0 to GridView.ColumnCount - 1 do
      ColumnList.Add(GridView.Columns[I].Name);
    Result := ColumnList.CommaText
  finally
    ColumnList.Free;
  end;
end;

{***************************************************************************** }
{                      TcxGridDBCardView                             }
{ **************************************************************************** }

destructor TovCxGridCardViewCustomization.Destroy;
var
  I: Integer;
begin
  if Assigned(GridView) then
  begin
    {         ,
       }
    GridView.OnInitStoredObject := nil;
    for I := 0 to GridView.RowCount - 1 do
    with GridView.Rows[I] do
    begin
      OnGetStoredProperties := nil;
      OnGetStoredPropertyValue := nil;
      OnSetStoredPropertyValue := nil;
    end;
  end;

  inherited Destroy;
end;

procedure TovCxGridCardViewCustomization.SetupObjectExposer;
begin
  FObjectExposer := GridView.DataController.DataSet as TovExposer
end;

function TovCxGridCardViewCustomization.GetGridView: TcxGridDBCardView;
begin
  Result := inherited GridView as TcxGridDBCardView
end;

procedure TovCxGridCardViewCustomization.ClearGridViewItems;
begin
  GridView.ClearItems;
end;

procedure TovCxGridCardViewCustomization.CreateGridViewItems(Attributes,
  ColumnNames: TStrings);
var
  I: Integer;
  Column: TcxGridDBCardViewRow;
begin
  {    }
  for I := GridView.RowCount - 1 downto 0 do
    if Attributes.IndexOf(GridView.Rows[I].DataBinding.FieldName) < 0 then
      GridView.Rows[I].Free;

  {    }
  for I := 0 to Attributes.Count - 1 do
  begin
    Column := GridView.GetRowByFieldName(Attributes[I]);
    if Column = nil then
      if ColumnNames.Count = 0 then
        CreateGridViewItem(Attributes[I], '', False)
      else
        CreateGridViewItem(Attributes[I], ColumnNames[I], False)
    else
      Column.Visible := True
  end;

  if dsoExposeIdField in ObjectExposer.Options then
    if GridView.GetRowByFieldName(ObjectExposer.IdFieldName) = nil then
      CreateGridViewItem(ObjectExposer.IdFieldName, '', True);
end;

procedure TovCxGridCardViewCustomization.CreateGridViewItem(FieldName,
  ItemName: String; Hidden: Boolean);
var
  ARow: TcxGridDBCardViewRow;
begin
  ARow := GridView.CreateRow;
  if ItemName = '' then
    GenerateComponentName(ARow, GridView.Name + 'Row')
  else
    ARow.Name := ItemName;
  ARow.DataBinding.FieldName := FieldName;

  case ObjectExposer.FieldByName(FieldName).DataType of
    ftBoolean:
    begin
      ARow.PropertiesClass := TcxCheckBoxProperties;
      (ARow.Properties as TcxCheckBoxProperties).NullStyle := nssUnchecked;
    end;
  end;
  ARow.Hidden := Hidden;
  ARow.Visible := Not Hidden;
end;

{{
        
     ,    .
}
procedure TovCxGridCardViewCustomization.GridTableItemGetStoredProperties(
  Sender: TcxCustomGridTableItem; AProperties: TStrings);
begin
  AProperties.Add('FieldName');
  AProperties.Add('Caption');
end;

procedure TovCxGridCardViewCustomization.GridTableItemGetStoredPropertyValue(
  Sender: TcxCustomGridTableItem; const AName: string; var AValue: Variant);
begin
  if AName = 'FieldName' then AValue := TcxGridDBCardViewRow(Sender).DataBinding.FieldName else
  if AName = 'Caption' then AValue := TcxGridDBCardViewRow(Sender).Caption
end;

procedure TovCxGridCardViewCustomization.GridTableItemSetStoredPropertyValue(
  Sender: TcxCustomGridTableItem; const AName: string; const AValue: Variant);
var
  ARow: TcxGridDBCardViewRow;
begin
  ARow := TcxGridDBCardViewRow(Sender);
  if AName = 'FieldName' then
  begin
    ARow.DataBinding.FieldName := AValue;
    case ObjectExposer.FieldByName(AValue).DataType of
      ftBoolean:
      begin
        ARow.PropertiesClass := TcxCheckBoxProperties;
        (ARow.Properties as TcxCheckBoxProperties).NullStyle := nssUnchecked;
      end else
        ARow.PropertiesClass := nil;
    end;
  end
  else
  if AName = 'Caption' then ARow.Caption := AValue
end;

{{
        .
}
procedure TovCxGridCardViewCustomization.GridViewInitStoredObject(Sender:
  TcxCustomGridView; AObject: TObject);
begin
  with AObject as TcxGridDBCardViewRow do
  begin
    OnGetStoredProperties := GridTableItemGetStoredProperties;
    OnGetStoredPropertyValue := GridTableItemGetStoredPropertyValue;
    OnSetStoredPropertyValue := GridTableItemSetStoredPropertyValue;
  end;
end;

{{
     /   .
}
procedure TovCxGridCardViewCustomization.SetupGridViewItems;
var
  I: Integer;
begin
  GridView.OnInitStoredObject := GridViewInitStoredObject;
  for I := 0 to GridView.RowCount - 1 do
  with GridView.Rows[I] do
  begin
    OnGetStoredProperties := GridTableItemGetStoredProperties;
    OnGetStoredPropertyValue := GridTableItemGetStoredPropertyValue;
    OnSetStoredPropertyValue := GridTableItemSetStoredPropertyValue;
  end;
end;

function TovCxGridCardViewCustomization.GetItemNames: String;
var
  ColumnList: TStrings;
  I: Integer;
begin
  ColumnList := TStringList.Create;
  try
    for I := 0 to GridView.RowCount - 1 do
      ColumnList.Add(GridView.Rows[I].Name);
    Result := ColumnList.CommaText
  finally
    ColumnList.Free;
  end;
end;

end.
