unit ovCxGridDataLinksExplorer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ImgList, AppEvnts, ActnList, ComCtrls,
  ToolWin, ExtCtrls, cxGridCustomView, cxGridLevel, cxGrid,
  ovDataLinks, ovDataLinksExplorer;

type
  TfrmCxGridDataLinksExplorer = class(TfrmDataLinksExplorer)
    AlignHeaderCenterCmd: TAction;
    AlignHeaderLeftCmd: TAction;
    AlignHeaderRightCmd: TAction;
    VSplitter2: TSplitter;
    GridObjectsTree: TTreeView;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    DeleteComponentExCmd: TAction;
    procedure AlignHeaderCenterCmdExecute(Sender: TObject);
    procedure AlignHeaderLeftCmdExecute(Sender: TObject);
    procedure AlignHeaderRightCmdExecute(Sender: TObject);
    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    procedure GridObjectsTreeKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure RefreshComponentListCmdExecute(Sender: TObject);
    procedure GridObjectsTreeChange(Sender: TObject; Node: TTreeNode);
    procedure GridObjectsTreeClick(Sender: TObject);
    procedure GridObjectsTreeDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure GridObjectsTreeDragDrop(Sender, Source: TObject; X,
      Y: Integer);
    procedure ComponentListDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure ComponentListDragDrop(Sender, Source: TObject; X,
      Y: Integer);
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure DeleteComponentExCmdExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FNeedRefreshTree: Boolean;
    function CreateViewItem(View: TcxCustomGridView; ADataObjectName,
      AttributeName: String): TComponent;
    function GetSelectedGridObject: TPersistent;
    procedure SetColumnHeaderAlignment(Alignment: TAlignment);
    procedure ShowGridObjects;
    procedure DeleteSelectedGridObject;
    { Private declarations }
  protected
    function CanLink(AComponent: TComponent): Boolean; override;
    function GetChildObject(AObject: TPersistent; Index: Integer): TPersistent;
      override;
    function GetChildObjectCount(AObject: TPersistent): Integer; override;
    function GetObjectInfo(AObject: TPersistent; var ATitle, ADataObject, 
      AnAttribute, ALookupDataObject, ALookupAttributes: String): Boolean; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function FindLink(AObject: TPersistent): TovDataLink; override;
  public
    function FindGridObject(AComponent: TComponent): Integer;
    procedure AddAttributeLink(DataObjectName: String; Attributes: TStrings; 
      ObjectIndex: Integer); override;
    procedure AddDataObjectLink(DataObjectName: String; ObjectIndex: Integer); 
      override;
    procedure AddLookupLink(ALookupDataObjectName, ALookupAttributes: String; 
      ObjectIndex: Integer); override;
    property SelectedGridObject: TPersistent read GetSelectedGridObject;
    { Public declarations }
  end;

var
  frmCxGridDataLinksExplorer: TfrmCxGridDataLinksExplorer;

implementation

uses TypInfo,
  cxGridDBTableView, cxGridTableView, cxGridDBCardView,
  cxGridCardView, cxGridCustomTableView, cxGridDBBandedTableView,
  ovUtils,
  ovDataObjects, ovDataLinksEditor, ovDataObjectsEditor, ovCxGridLinks,
  ovCxNewGridView;

{$R *.dfm}

type
  TcxCustomGridTableItemFriend = class (TcxCustomGridTableItem)
  end;

procedure TfrmCxGridDataLinksExplorer.FormCreate(Sender: TObject);
begin
  inherited;
  //    -  
  ToolButton1.Action := DeleteComponentExCmd;
end;

procedure TfrmCxGridDataLinksExplorer.AddAttributeLink(DataObjectName: String; 
  Attributes: TStrings; ObjectIndex: Integer);
var
  I: Integer;
  AObject: TPersistent;
begin
  AObject := GetObjectAt(ObjectIndex); //  
  if AObject is TcxCustomGridView then
  begin
    //   /  
    for I := 0 to Attributes.Count - 1 do
      CreateViewItem(TcxCustomGridView(AObject), DataObjectName, Attributes[I]);

    if FindLink(AObject) = nil then
      DataLinks.AddLink(TcxGridViewDataLinkController,
        TcxCustomGridView(AObject), DataObjectName);

    ShowComponentList
  end else
  begin
    DataLinkControllerClass := TcxGridItemDataLinkController;
    inherited
  end
end;

procedure TfrmCxGridDataLinksExplorer.AddDataObjectLink(DataObjectName: String;
  ObjectIndex: Integer);
begin
  DataLinkControllerClass := TcxGridViewDataLinkController;
  inherited
end;

{{
    Grid'a    
}
function TfrmCxGridDataLinksExplorer.CanLink(AComponent: TComponent): Boolean;
begin
  Result :=
    (AComponent is TcxCustomGridView) and
    ((TcxCustomGridView(AComponent).Level = SelectedGridObject) or
     ((TcxCustomGridView(AComponent).Level <> nil) and
     ((TcxCustomGridView(AComponent).Level as TcxGridLevel).Control = SelectedGridObject)))
end;

function TfrmCxGridDataLinksExplorer.CreateViewItem(View: TcxCustomGridView;
  ADataObjectName, AttributeName: String): TComponent;
var
  Row: TcxGridDBCardViewRow;
  Column: TcxGridDBColumn;
  BandedColumn: TcxGridDBBandedColumn;
begin
  if View is TcxGridDBTableView then begin
    Column := TcxGridDBTableView(View).CreateColumn;
    Column.Caption := AttributeName;
    DataLinks.AddLink(TcxGridItemDataLinkController, Column, ADataObjectName, AttributeName);
    Result := Column;
  end else
  if View is TcxGridDBBandedTableView then begin
    BandedColumn := TcxGridDBBandedTableView(View).CreateColumn;
    BandedColumn.Caption := AttributeName;
    DataLinks.AddLink(TcxGridItemDataLinkController, BandedColumn, ADataObjectName, AttributeName);
    Result := BandedColumn;
  end else
  if View is TcxGridDBCardView then begin
    Row := TcxGridDBCardView(View).CreateRow;
    Row.Caption := AttributeName;
    DataLinks.AddLink(TcxGridItemDataLinkController, Row, ADataObjectName, AttributeName);
    Result := Row;
  end else
    Result := nil;

  if Result <> nil then GenerateComponentName(Result);
end;

{{
    ,   AComponent,
 -1,    .
}
function TfrmCxGridDataLinksExplorer.FindGridObject(AComponent: TComponent):
  Integer;
var
  I: Integer;
begin
  for I := 0 to GridObjectsTree.Items.Count - 1 do
    if GridObjectsTree.Items[I].Data = AComponent then begin
      Result := I;
      Exit;
    end;
  Result := -1;
end;

function TfrmCxGridDataLinksExplorer.GetChildObject(AObject: TPersistent; Index:
  Integer): TPersistent;
begin
  if AObject is TcxGridDBTableView then
    Result := TcxGridDBTableView(AObject).Columns[Index]
  else
  if AObject is TcxGridDBBandedTableView then
    Result := TcxGridDBBandedTableView(AObject).Columns[Index]
  else
  if AObject is TcxGridDBCardView then
    Result := TcxGridDBCardView(AObject).Rows[Index]
  else
    Result := nil
end;

function TfrmCxGridDataLinksExplorer.GetChildObjectCount(AObject: TPersistent):
  Integer;
begin
  if AObject is TcxGridDBTableView then
    Result := TcxGridDBTableView(AObject).ColumnCount
  else
  if AObject is TcxGridDBBandedTableView then
    Result := TcxGridDBBandedTableView(AObject).ColumnCount
  else
  if AObject is TcxGridDBCardView then
    Result := TcxGridDBCardView(AObject).RowCount
  else
    Result := 0
end;

function TfrmCxGridDataLinksExplorer.FindLink(AObject: TPersistent):
  TovDataLink;
begin
  if (AObject is TcxGridDBTableView) or (AObject is TcxGridDBCardView) or
     (AObject is TcxGridDBBandedTableView) then
    Result := DataLinks.FindLink(TComponent(AObject), TcxGridViewDataLinkController)
  else
    Result := DataLinks.FindLink(TComponent(AObject), TcxGridItemDataLinkController)
end;

function TfrmCxGridDataLinksExplorer.GetObjectInfo(AObject: TPersistent; var
  ATitle, ADataObject, AnAttribute, ALookupDataObject, ALookupAttributes:
  String): Boolean;

  function GetCaptionOrName(Caption, Name: String): String;
  begin
    if Caption <> ''
    then Result := Caption
    else Result := Name
  end;

var
  Link: TovDataLink;
begin
  Result := inherited GetObjectInfo(AObject,
    ATitle, ADataObject, AnAttribute, ALookupDataObject, ALookupAttributes);

  if AObject is TcxGridDBColumn
    then ATitle := GetCaptionOrName(TcxGridDBColumn(AObject).Caption, TcxGridDBColumn(AObject).Name) else
  if AObject is TcxGridDBBandedColumn
    then ATitle := GetCaptionOrName(TcxGridDBBandedColumn(AObject).Caption, TcxGridDBBandedColumn(AObject).Name) else
  if AObject is TcxGridDBCardViewRow
    then ATitle := GetCaptionOrName(TcxGridDBCardViewRow(AObject).Caption, TcxGridDBCardViewRow(AObject).Name);

  Link := FindLink(AObject);
  if (Link <> nil) and (Link.Controller is TcxGridItemDataLinkController) then begin
    with TcxGridItemDataLinkController(Link.Controller) do
    begin
      ALookupDataObject := LookupDataObject;
      ALookupAttributes := LookupAttributes;
    end;
  end
end;

function TfrmCxGridDataLinksExplorer.GetSelectedGridObject: TPersistent;
begin
  if GridObjectsTree.Selected <> nil
    then Result := TPersistent(GridObjectsTree.Selected.Data)
    else Result := nil
end;

procedure TfrmCxGridDataLinksExplorer.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and Not (csDestroying in ComponentState) and
     (FindGridObject(AComponent) >= 0) then FNeedRefreshTree := True;
end;

procedure TfrmCxGridDataLinksExplorer.SetColumnHeaderAlignment(Alignment:
  TAlignment);
var
  I: Integer;
begin
  with ComponentList do
    for I := 0 to Items.Count - 1 do
      if (Items[I].Selected) and (GetObjectAt(I) is TcxCustomGridTableItem) then
        TcxCustomGridTableItemFriend(GetObjectAt(I)).HeaderAlignmentHorz := Alignment
end;

procedure TfrmCxGridDataLinksExplorer.ShowGridObjects;

  procedure AddLevels(ParentNode: TTreeNode; Level: TcxGridLevel);
  var
    Node: TTreeNode;
    I: Integer;
  begin
    Node := GridObjectsTree.Items.AddChild(ParentNode, Level.Name);
    Node.Data := Level;
    Level.FreeNotification(Self);
    if Level.Caption <> '' then Node.Text := Node.Text + Format(' (%s)', [Level.Caption]);
    for I := 0 to Level.Count - 1 do
      AddLevels(Node, Level.Items[I])
  end;

  procedure AddGrid(Grid: TcxGrid);
  var
    Node: TTreeNode;
    I: Integer;
  begin
    Node := GridObjectsTree.Items.AddChild(nil, Grid.Name);
    Node.Data := Grid;
    Grid.FreeNotification(Self);
    for I := 0 to Grid.Levels.Count - 1 do
      AddLevels(Node, Grid.Levels[I]);
  end;

var
  I: Integer;
begin
  with GridObjectsTree do
  begin
    Items.BeginUpdate;
    try
      Items.Clear;
      if Assigned(DataLinks.Owner) then
        with DataLinks.Owner do
          for I := 0 to ComponentCount - 1 do
            if Components[I] is TcxGrid
              then AddGrid(TcxGrid(Components[I]));
      FullExpand;
    finally
      Items.EndUpdate;
    end;
  end;
end;

procedure TfrmCxGridDataLinksExplorer.AlignHeaderCenterCmdExecute(
  Sender: TObject);
begin
  inherited;
  SetColumnHeaderAlignment(taCenter)
end;

procedure TfrmCxGridDataLinksExplorer.AlignHeaderLeftCmdExecute(
  Sender: TObject);
begin
  inherited;
  SetColumnHeaderAlignment(taLeftJustify)
end;

procedure TfrmCxGridDataLinksExplorer.AlignHeaderRightCmdExecute(
  Sender: TObject);
begin
  inherited;
  SetColumnHeaderAlignment(taRightJustify)
end;

procedure TfrmCxGridDataLinksExplorer.ApplicationEventsIdle(Sender: TObject;
  var Done: Boolean);
begin
  inherited;
  if FNeedRefreshTree then
  begin
    ShowGridObjects;
    FNeedRefreshTree := False;
  end;
end;

procedure TfrmCxGridDataLinksExplorer.ActionListUpdate(Action: TBasicAction;
  var Handled: Boolean);
begin
  inherited;
  AlignHeaderLeftCmd.Enabled := Assigned(ComponentList.Selected);
  AlignHeaderCenterCmd.Enabled := AlignHeaderLeftCmd.Enabled;
  AlignHeaderRightCmd.Enabled := AlignHeaderLeftCmd.Enabled;
  DeleteComponentExCmd.Enabled := DeleteComponentCmd.Enabled or Assigned(SelectedGridObject);
end;

procedure TfrmCxGridDataLinksExplorer.GridObjectsTreeKeyDown(
  Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  inherited;
  if Key = VK_DELETE then begin
    DeleteSelectedGridObject;
    Key := 0;
  end;
end;

procedure TfrmCxGridDataLinksExplorer.DeleteComponentExCmdExecute(
  Sender: TObject);
begin
  inherited;
  if Assigned(SelectedGridObject) then
    DeleteSelectedGridObject
  else
    DeleteComponentCmd.Execute
end;

procedure TfrmCxGridDataLinksExplorer.DeleteSelectedGridObject;
begin
  if Assigned(SelectedGridObject) then
  begin
    SelectedGridObject.Free;
    ShowGridObjects;
    ShowComponentList;
  end;
end;

procedure TfrmCxGridDataLinksExplorer.FormShow(Sender: TObject);
begin
  inherited;
  ShowGridObjects
end;

procedure TfrmCxGridDataLinksExplorer.RefreshComponentListCmdExecute(
  Sender: TObject);
begin
  inherited;
  ShowGridObjects;
end;

procedure TfrmCxGridDataLinksExplorer.GridObjectsTreeChange(Sender: TObject;
  Node: TTreeNode);
begin
  inherited;
  ShowComponentList;
end;

procedure TfrmCxGridDataLinksExplorer.GridObjectsTreeClick(Sender: TObject);
begin
  inherited;
  if Designer <> nil then Designer.SelectComponent(SelectedGridObject);
end;

procedure TfrmCxGridDataLinksExplorer.GridObjectsTreeDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  inherited;
  Accept := Assigned(DataLinksEditor) and
            (Source = (DataLinksEditor as TfrmDataLinksEditor).DataObjectsTree)
            or (Source = GridObjectsTree)
end;

procedure TfrmCxGridDataLinksExplorer.GridObjectsTreeDragDrop(Sender,
  Source: TObject; X, Y: Integer);

  {{
        /     
  ParentLevel.
  }
  procedure CreateDataItemLevels(DataItemNode: TTreeNode; Grid: TcxGrid;
    ParentLevel: TcxGridLevel);
  var
    Level: TcxGridLevel;
    I: Integer;
    ViewType: TViewType;
  begin
    if SelectViewType('    ' + DataItemNode.Text, ViewType) then
    begin

      if ParentLevel = nil then begin
//        if Grid.Levels.Count > 0 then Grid.RootLevelOptions.DetailTabsPosition := dtpTop;
        Level := Grid.Levels.Add;
      end else begin
//        if ParentLevel.Count > 0 then ParentLevel.Options.DetailTabsPosition := dtpTop;
        Level := ParentLevel.Add;
      end;
      Level.Caption := DataItemNode.Text;
      GenerateComponentName(Level);

      case ViewType of
        vtTableView:
          Level.GridView := Grid.CreateView(TcxGridDBTableView);
        vtBandedTableView:
          Level.GridView := Grid.CreateView(TcxGridDBBandedTableView);
        vtCardView:
          Level.GridView := Grid.CreateView(TcxGridDBCardView);
      end;

      GenerateComponentName(Level.GridView);
      DataLinks.AddLink(TcxGridViewDataLinkController, Level.GridView, DataItemNode.Text);

      for I := 0 to DataItemNode.Count - 1 do
        if DataItemNode.Item[I].ImageIndex = 0 then
          CreateDataItemLevels(DataItemNode.Item[I], Grid, Level)
        else
          CreateViewItem(Level.GridView, DataItemNode.Text, DataItemNode.Item[I].Text);
    end
  end;

  procedure CreateDataItemGrid(DataItemNode: TTreeNode);
  var
    Grid: TcxGrid;
  begin
    Grid := TcxGrid.Create(DataLinks.Owner);
    Grid.Parent := DataLinks.Owner as TWinControl;
    //            ,
    //   ,    Grid.
    if Grid.Levels.Count > 0 then begin
      Grid.Levels[0].GridView.Free;
      Grid.Levels[0].Free;
    end;
    GenerateComponentName(Grid);
    CreateDataItemLevels(DataItemNode, Grid, nil);
  end;

  {{
    /     GridObjectsTree
  }
  procedure CreateGridObjects;
  var
    SourceNode, TargetNode: TTreeNode;
    Grid: TcxGrid;
    Level: TcxGridLevel;
  begin
    with (DataLinksEditor as TfrmDataLinksEditor) do
      SourceNode := DataObjectsTree.Selected;
    if SourceNode = nil then Exit;
    TargetNode := GridObjectsTree.GetNodeAt(X, Y);
    if (SourceNode.Parent = nil) and (TargetNode = nil) then
    begin
      //          
      CreateDataItemGrid(SourceNode);
      ShowGridObjects;
    end else
    if (SourceNode.Parent = nil) and (TargetNode <> nil) then
    begin
      //          
      if TObject(TargetNode.Data) is TcxGrid then
      begin
        Level := nil;
        Grid := TcxGrid(TargetNode.Data);
      end else begin
        Level := TObject(TargetNode.Data) as TcxGridLevel;
        Grid := Level.Control as TcxGrid;
      end;

      CreateDataItemLevels(SourceNode, Grid, Level);
      ShowGridObjects;
    end else
    if (SourceNode.ImageIndex = 0) {    } and (TargetNode = nil) then
    begin
      //          
      CreateDataItemGrid(SourceNode);
      ShowGridObjects;
    end else
    if (SourceNode.ImageIndex = 0) {    } and (TargetNode <> nil) then
    begin
      //          
      if TObject(TargetNode.Data) is TcxGrid then
      begin
        Level := nil;
        Grid := TcxGrid(TargetNode.Data);
      end else begin
        Level := TObject(TargetNode.Data) as TcxGridLevel;
        Grid := Level.Control as TcxGrid;
      end;

      CreateDataItemLevels(SourceNode, Grid, Level);
      ShowGridObjects;
    end else
      MessageDlg('     ', mtWarning, [mbOK], 0)
  end;

  {{
   TcxGridLevel   Grid'a
  }
  procedure MoveGridObjects;
  var
    SourceNode, TargetNode: TTreeNode;
    Source, Target: TObject;
  begin
    SourceNode := GridObjectsTree.Selected;
    TargetNode := GridObjectsTree.GetNodeAt(X, Y);
    if (SourceNode <> nil) and (TargetNode <> nil) then
    begin
      Source := TObject(SourceNode.Data);
      Target := TObject(TargetNode.Data);

      if (Source is TcxGridLevel) and (Target is TcxGridLevel) and
         // Grid    
         (TcxGridLevel(Source).Control = TcxGridLevel(Target).Control) and
         //     
         (TcxGridLevel(Source).Level = TcxGridLevel(Target).Level)
      then begin
        TcxGridLevel(Source).Index := TcxGridLevel(Target).Index;
        ShowGridObjects;
      end
    end;
  end;

begin
  inherited;
  if Assigned(DataLinksEditor) and
     (Source = (DataLinksEditor as TfrmDataLinksEditor).DataObjectsTree)
  then
    CreateGridObjects
  else
  if Source = GridObjectsTree
    then MoveGridObjects
end;

procedure TfrmCxGridDataLinksExplorer.ComponentListDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  inherited;
  Accept := Accept or (Source = ComponentList)
end;

procedure TfrmCxGridDataLinksExplorer.ComponentListDragDrop(Sender,
  Source: TObject; X, Y: Integer);

  {{
   /   
  }
  procedure MoveColumns;
  var
    SourceItem, TargetItem: TListItem;
    Source, Target: TObject;
  begin
    SourceItem := ComponentList.Selected;
    TargetItem := ComponentList.GetItemAt(X, Y);
    if (SourceItem <> nil) and (TargetItem <> nil) then
    begin
      Source := GetObjectAt(SourceItem.Index);
      Target := GetObjectAt(TargetItem.Index);

      if (Source is TcxCustomGridTableItem) and (Target is TcxCustomGridTableItem)
      then begin
        TcxCustomGridTableItem(Source).Index := TcxCustomGridTableItem(Target).Index;
        ShowComponentList;
      end
    end;
  end;

begin
  inherited;
  if Source = ComponentList then MoveColumns
end;

procedure TfrmCxGridDataLinksExplorer.AddLookupLink(ALookupDataObjectName, 
  ALookupAttributes: String; ObjectIndex: Integer);
var
  AObject: TObject;
  Link: TovDataLink;
begin
  Link := GetLinkAt(ObjectIndex);
  if Link = nil then begin
    AObject := GetObjectAt(ObjectIndex);
    if AObject is TComponent then
      Link := DataLinks.AddLink(TcxGridItemDataLinkController, TComponent(AObject), '', '')
    else
      Exit;
  end;
  if Link.Controller is TcxGridItemDataLinkController then
  with TcxGridItemDataLinkController(Link.Controller) do
  begin
    LookupDataObject := ALookupDataObjectName;
    LookupAttributes := ALookupAttributes;
  end;
  ShowComponentList;
end;

initialization
  RegisterDataLinkExplorer('  cxGrid', TfrmCxGridDataLinksExplorer);
finalization
  UnregisterDataLinkExplorer(TfrmCxGridDataLinksExplorer);
end.
