unit ovCxGridLinks;

interface

uses Windows, Classes, SysUtils, Forms, DB,
     cxEdit, cxDBData, cxGridDBDataDefinitions, cxGridLevel,
     cxGridCustomView, cxGridCustomTableView,
     cxGridDBTableView, cxGridDBBandedTableView, cxGridDBCardView,
     ovModel, ovDataSet, ovDataObjects, ovDataLinks;

type

  TcxGridViewDataLinkController = class(TovDataLinkController)
  private
    FDataSet: TovDataSet;
    procedure SetupTableView(View: TcxGridDBTableView);
    procedure SetupCardView(View: TcxGridDBCardView);
    procedure DataModeControllerDetailFirst(Sender: TcxDBDataModeController;
      ADataSet: TDataSet; const AMasterDetailKeyFieldNames: String; const 
      AMasterDetailKeyValues: Variant; var AReopened: Boolean);
    function DataModeControllerDetailIsCurrentQuery(Sender: TcxDBDataModeController;
      ADataSet: TDataSet; const AMasterDetailKeyFieldNames: String; const
      AMasterDetailKeyValues: Variant): Boolean;
    procedure SetupDataController(DataController: TcxDBDataController);
    procedure DataSetCustomCalcFields(ADataSet: TDataSet);
    procedure SetupBandedTableView(View: TcxGridDBBandedTableView);
  protected
    procedure SetActive(Value: Boolean); override;
  published
  end;

  TcxGridItemDataLinkController = class(TovDataLinkController)
  private
    FLookupAttributes: String;
    FLookupDataObject: String;
    procedure SetupTableViewColumn(Column: TcxGridDBColumn);
    procedure SetupCardViewRow(Row: TcxGridDBCardViewRow);
    procedure SetupDataBinding(DataBinding: TcxGridItemDBDataBinding; Properties:
      TcxCustomEditProperties);
    procedure SetupTableViewBandedColumn(Column: TcxGridDBBandedColumn);
  protected
    procedure SetActive(Value: Boolean); override;
  published
    property LookupAttributes: String read FLookupAttributes write
      FLookupAttributes;
    property LookupDataObject: String read FLookupDataObject write
      FLookupDataObject;
  end;

implementation

uses cxCustomData, cxDBLookupComboBox, ovVirtualDataSet;

const
  ParentIdFieldName = 'ParentId'; //      

{
************************ TcxGridViewDataLinkController *************************
}

procedure TcxGridViewDataLinkController.SetActive(Value: Boolean);
begin
  with Link do
  begin
    if Component = nil then
      raise Exception.Create('     ');

    if Value then
    begin
      with Owner.DataObjects.ObjectByName[DataObject] do
      begin
        if Not Assigned(DataSet) then CreateDataSet;
        FDataSet := DataSet;
      end;
      if Component is TcxGridDBTableView then SetupTableView(TcxGridDBTableView(Component)) else
      if Component is TcxGridDBBandedTableView then SetupBandedTableView(TcxGridDBBandedTableView(Component)) else
      if Component is TcxGridDBCardView then SetupCardView(TcxGridDBCardView(Component))
      else
        raise Exception.Create('     ');
    end
  end
end;

procedure TcxGridViewDataLinkController.SetupTableView(View: TcxGridDBTableView);
var
  I: Integer;
begin
  //        
  for I := 0 to View.ItemCount - 1 do
    Link.Owner.ActivateLink(View.Items[I]);

  //      
  SetupDataController(View.DataController);

  //    ,   
  if Not Assigned(View.Level) then raise Exception.Create('   Level  ' + View.Name);

  with View.Level as TcxGridLevel do
    if Count > 0 then Link.Owner.ActivateLink(Items[0].GridView);
end;

procedure TcxGridViewDataLinkController.SetupBandedTableView(View:
  TcxGridDBBandedTableView);
var
  I: Integer;
begin
  //        
  for I := 0 to View.ItemCount - 1 do
    Link.Owner.ActivateLink(View.Items[I]);

  //      
  SetupDataController(View.DataController);

  //    ,   
  if Not Assigned(View.Level) then raise Exception.Create('   Level  ' + View.Name);

  with View.Level as TcxGridLevel do
    if Count > 0 then Link.Owner.ActivateLink(Items[0].GridView);
end;

procedure TcxGridViewDataLinkController.SetupCardView(View: TcxGridDBCardView);
var
  I: Integer;
begin
  //        
  for I := 0 to View.ItemCount - 1 do
    Link.Owner.ActivateLink(View.Items[I]);

  //      
  SetupDataController(View.DataController);
end;

procedure TcxGridViewDataLinkController.SetupDataController(DataController:
  TcxDBDataController);
var
  ParentDataObject: TovDataObject;
begin
  with FDataSet do
  begin
    Options := Options + [dsoExposeIdField]; // Id-    
                                             //  
    if DataController.IsDetailMode then
    begin
      Close;
      AddCustomCalcField(ParentIdFieldName, ftInteger, 0);
      OnCustomCalcFields := DataSetCustomCalcFields;
      Open;
    end;
  end;

  with DataController do
  begin
    KeyFieldNames := Self.FDataSet.IdFieldName;

    if IsDetailMode then
    begin
      ParentDataObject := Link.Owner.DataObjects.ObjectByName[Link.DataObject].Parent;
      if Not Assigned(ParentDataObject) then
        raise Exception.Create('       ');

      DataModeController.OnDetailFirst := DataModeControllerDetailFirst;
      DataModeController.OnDetailIsCurrentQuery := DataModeControllerDetailIsCurrentQuery;
      DataModeController.DetailInSQLMode := True;
      Options := Options - [dcoAssignMasterDetailKeys];

      MasterKeyFieldNames := Link.Owner.DataObjects[ParentDataObject.Name].IdFieldName;
      DetailKeyFieldNames := ParentIdFieldName;
    end;
    DataSource := Link.Owner.DataObjects.DataSources[Link.DataObject];
  end;

end;

procedure TcxGridViewDataLinkController.DataModeControllerDetailFirst(Sender:
  TcxDBDataModeController; ADataSet: TDataSet; const
  AMasterDetailKeyFieldNames: String; const AMasterDetailKeyValues: Variant;
  var AReopened: Boolean);
begin
  with ADataSet as TovDataSet do
  begin
    //    
    if MasterObjectId = AMasterDetailKeyValues then
    begin
      First;
      Exit;
    end;
    DisableControls;
    try
      Close;
      MasterObjectId := AMasterDetailKeyValues;
      Open;
    finally
      EnableControls;
    end;
    AReopened := True;
  end;
end;

function TcxGridViewDataLinkController.DataModeControllerDetailIsCurrentQuery(Sender:
  TcxDBDataModeController; ADataSet: TDataSet; const
  AMasterDetailKeyFieldNames: String; const AMasterDetailKeyValues: Variant):
  Boolean;
begin
  Result := (ADataSet as TovDataSet).MasterObjectId = AMasterDetailKeyValues
end;

procedure TcxGridViewDataLinkController.DataSetCustomCalcFields(ADataSet: TDataSet);
begin
  ADataSet.FieldByName(ParentIdFieldName).Value :=
    (ADataSet as TovDataSet).MasterObjectId
end;

{
************************ TcxGridItemDataLinkController *************************
}

procedure TcxGridItemDataLinkController.SetActive(Value: Boolean);
begin
  if Value then
  with Link do
  begin
    if Component = nil then
      raise Exception.Create('     ');
    if Component is TcxGridDBColumn then
      SetupTableViewColumn(TcxGridDBColumn(Component)) else
    if Component is TcxGridDBBandedColumn then
      SetupTableViewBandedColumn(TcxGridDBBandedColumn(Component)) else
    if Component is TcxGridDBCardViewRow then
      SetupCardViewRow(TcxGridDBCardViewRow(Component))
    else
      raise Exception.Create('     ')
  end
end;

procedure TcxGridItemDataLinkController.SetupCardViewRow(Row:
  TcxGridDBCardViewRow);
begin
  SetupDataBinding(Row.DataBinding, Row.Properties);
end;

procedure TcxGridItemDataLinkController.SetupTableViewColumn(Column:
  TcxGridDBColumn);
begin
  SetupDataBinding(Column.DataBinding, Column.Properties);
end;

procedure TcxGridItemDataLinkController.SetupTableViewBandedColumn(Column:
  TcxGridDBBandedColumn);
begin
  SetupDataBinding(Column.DataBinding, Column.Properties);
end;

procedure TcxGridItemDataLinkController.SetupDataBinding(DataBinding:
  TcxGridItemDBDataBinding; Properties: TcxCustomEditProperties);
var
  LookupDataSet: TovDataSet;
begin
  with Link do
  begin
    if Owner.DataObjects[DataObject].FindField(Attribute) = nil then
      raise Exception.CreateFmt('  "%s"    "%s"', [DataObject, Attribute]);

    DataBinding.FieldName := Attribute;
    if (LookupDataObject <> '') and (Properties is TcxLookupComboBoxProperties) then
    begin
      LookupDataSet := Owner.DataObjects[LookupDataObject];
      with TcxLookupComboBoxProperties(Properties) do begin
        KeyFieldNames := LookupDataSet.IdFieldName;
        ListFieldNames := LookupAttributes;
        ListSource := Link.Owner.DataObjects.DataSources[LookupDataObject];
      end
    end
  end;
end;

initialization
  RegisterDataLinkController(TcxGridViewDataLinkController);
  RegisterDataLinkController(TcxGridItemDataLinkController);
finalization
  UnregisterDataLinkController(TcxGridViewDataLinkController);
  UnregisterDataLinkController(TcxGridItemDataLinkController);
end.
