unit ovCxGridPopupMenu;

interface

uses Classes, Menus, cxGridPopupMenu, cxGridCustomPopupMenu,
  ovCxGridCustomization;

type
  TovCxGridPopupMenu = class(TcxGridPopupMenu)
  private
    FGridCustomization: TovCxGridCustomization;
    FAllowColumnSetup: Boolean;
    procedure SetGridCustomization(Value: TovCxGridCustomization);
    procedure ExportDataClick(Sender: TObject);
    procedure SaveGridSettingsClick(Sender: TObject);
    procedure LoadGridSettingsClick(Sender: TObject);
    procedure SetupItemsClick(Sender: TObject);
    procedure RenameGridItemsClick(Sender: TObject);
    procedure ExportDataToFile(FileName: String; FormatIndex: Integer);
    procedure RenameHitGridItem;
    procedure DefaultSettingsClick(Sender: TObject);
  protected
    procedure Loaded; override;
    procedure SetupPopupMenus; dynamic;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure InsertMenuItems(PopupMenu: TPopupMenu; IsColumnMenu: Boolean);
  public
    constructor Create(AOwner: TComponent); override;
  published
    property GridCustomization: TovCxGridCustomization read FGridCustomization
      write SetGridCustomization;
    property AllowColumnSetup: Boolean read FAllowColumnSetup write
      FAllowColumnSetup default True;
  end;

implementation

uses cxGridCustomView, cxGridDBTableView, cxGridDBBandedTableView,
  cxGridDBCardView, Dialogs, cxExportGrid4Link, cxGridTableView, Windows;

constructor TovCxGridPopupMenu.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAllowColumnSetup := True;
end;

procedure TovCxGridPopupMenu.SetGridCustomization(Value:
  TovCxGridCustomization);
begin
  if FGridCustomization <> Value then
  begin
    FGridCustomization := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovCxGridPopupMenu.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if AComponent = GridCustomization then GridCustomization := nil
end;

procedure TovCxGridPopupMenu.Loaded;
begin
  inherited;
  if Not (csDesigning in ComponentState) then SetupPopupMenus;
end;

{{
    ,    , 
   , ,    .
}
procedure TovCxGridPopupMenu.SetupPopupMenus;
var
  I: Integer;
  PopupMenu: TPopupMenu;
  ADoPopupProc: TcxGridOnPopupProc;
begin
  inherited;
  //  ,    
  PopupMenu := nil;
  with BuiltInPopupMenus do
  for I := 0 to Count - 1 do
    if gvhtColumnHeader in MenuInfos[I].HitTypes then
      PopupMenu := MenuInfos[I].PopupMenu as TPopupMenu;
  if Assigned(PopupMenu) then
  begin
    {     "  ", " " 
      }
    PopupMenu.Items.Delete(9);
    PopupMenu.Items.Delete(9);
    PopupMenu.Items.Delete(9);
    InsertMenuItems(PopupMenu, True);
  end;

  //  , ,       
  //   
  PopupMenu := TPopupMenu.Create(Self);
  InsertMenuItems(PopupMenu, False);
  ADoPopupProc := nil;
  RegisterPopupMenu(PopupMenu, [gvhtNone], ADoPopupProc, TcxGridDBTableView);
  RegisterPopupMenu(PopupMenu, [gvhtNone], ADoPopupProc, TcxGridDBBandedTableView);
  RegisterPopupMenu(PopupMenu, [gvhtNone], ADoPopupProc, TcxGridDBCardView);
end;

procedure TovCxGridPopupMenu.InsertMenuItems(PopupMenu: TPopupMenu;
  IsColumnMenu: Boolean);
var
  MenuItem: TMenuItem;
begin
  MenuItem := TMenuItem.Create(Self);
  MenuItem.Caption := '-';
  PopupMenu.Items.Add(MenuItem);

  if Assigned(GridCustomization) then
  begin
    if AllowColumnSetup then
    begin
      MenuItem := TMenuItem.Create(Self);
      MenuItem.Caption := ' ..';
      MenuItem.OnClick := SetupItemsClick;
      PopupMenu.Items.Add(MenuItem);
    end;

    if IsColumnMenu then
    begin
      MenuItem := TMenuItem.Create(Self);
      MenuItem.Caption := ' ..';
      MenuItem.OnClick := RenameGridItemsClick;
      PopupMenu.Items.Add(MenuItem);
    end;

    MenuItem := TMenuItem.Create(Self);
    MenuItem.Caption := '-';
    PopupMenu.Items.Add(MenuItem);

    MenuItem := TMenuItem.Create(Self);
    MenuItem.Caption := ' ';
    MenuItem.OnClick := SaveGridSettingsClick;
    PopupMenu.Items.Add(MenuItem);

    MenuItem := TMenuItem.Create(Self);
    MenuItem.Caption := ' ';
    MenuItem.OnClick := LoadGridSettingsClick;
    PopupMenu.Items.Add(MenuItem);

    MenuItem := TMenuItem.Create(Self);
    MenuItem.Caption := '   ';
    MenuItem.OnClick := DefaultSettingsClick;
    PopupMenu.Items.Add(MenuItem);

    MenuItem := TMenuItem.Create(Self);
    MenuItem.Caption := '-';
    PopupMenu.Items.Add(MenuItem);

    MenuItem := TMenuItem.Create(Self);
    MenuItem.Caption := '   ..';
    MenuItem.OnClick := ExportDataClick;
    PopupMenu.Items.Add(MenuItem);
  end
end;

procedure TovCxGridPopupMenu.SetupItemsClick(Sender: TObject);
begin
  if Assigned(GridCustomization) then
    if HitGridView.IsDetail then
      GridCustomization.SelectGridViewItems(HitGridView.PatternGridView)
    else
      GridCustomization.SelectGridViewItems(HitGridView);
end;

procedure TovCxGridPopupMenu.RenameGridItemsClick(Sender: TObject);
begin
  if Assigned(GridCustomization) then RenameHitGridItem
end;

procedure TovCxGridPopupMenu.RenameHitGridItem;
var
  NewCaption: String;
begin
  if HitTest.ViewInfo is TcxGridColumnHeaderViewInfo then
  begin
    NewCaption := TcxGridColumnHeaderViewInfo(HitTest.ViewInfo).Column.Caption;
    if InputQuery(' ', ' ', NewCaption) then
      TcxGridColumnHeaderViewInfo(HitTest.ViewInfo).Column.Caption := NewCaption;
  end;
end;

procedure TovCxGridPopupMenu.SaveGridSettingsClick(Sender: TObject);
begin
  if Assigned(GridCustomization) then GridCustomization.SaveSettings;
end;

procedure TovCxGridPopupMenu.LoadGridSettingsClick(Sender: TObject);
begin
  if Assigned(GridCustomization) then GridCustomization.LoadSettings;
end;

procedure TovCxGridPopupMenu.DefaultSettingsClick(Sender: TObject);
begin
  if Assigned(GridCustomization) then
  begin
    GridCustomization.RestoreDefaultSettings;
    MessageBox(0, '     '+#13+#10+
    '     ', ' ', MB_ICONINFORMATION or MB_OK);
  end;
end;

procedure TovCxGridPopupMenu.ExportDataClick(Sender: TObject);
begin
  if Assigned(GridCustomization) then
  with TSaveDialog.Create(nil) do
  try
    Filter := 'Excel-|*.xls|HTML-|*.html';
    Options := Options + [ofOverwritePrompt];
    Title := '   ';
    if Execute then ExportDataToFile(FileName, FilterIndex)
  finally
    Free
  end;
end;

procedure TovCxGridPopupMenu.ExportDataToFile(FileName: String; FormatIndex:
  Integer);
begin
  case FormatIndex of
    1: ExportGrid4ToExcel(FileName, Grid);
    2: ExportGrid4ToHTML(FileName, Grid);
  end
end;

end.
