unit ovFReportDesigner;

interface

uses
  SysUtils, Classes, ComCtrls, DB, Contnrs,
  ovComponents, frxClass, frxDesgn, ovDataObjects, Forms, Dialogs;

type
  TDataKind = (dkReport, dkDataObjects);

  TovFrxReportDesigner = class(TovModelClient)
  private
    { Private declarations }
    FDesigner: TfrxDesigner;
    FReport: TfrxReport;
    FDataObjects: TovDataObjects;
    FDataSets: TObjectList;
    FModified: Boolean;
    FOnLoadReport: TNotifyEvent;
    FOnSaveReport: TNotifyEvent;
    procedure DesignerShow(Sender: TObject);
    procedure DataManagerClick(Sender: TObject);
    procedure SetupReportDataSets;
    procedure LoadReportComponent(Reader: TReader);
    procedure StoreReportComponent(Writer: TWriter);
    procedure LoadDataObjectsComponent(Reader: TReader);
    procedure StoreDataObjectsComponent(Writer: TWriter);
    function ReportUserFunction(const MethodName: String;
      var Params: Variant): Variant;
    function GetOnPrintReport: TNotifyEvent;
    procedure SetOnPrintReport(const Value: TNotifyEvent);
  protected
    { Protected declarations }
    procedure SetupDesignerIDE; dynamic;
    procedure DefineProperties(Filer: TFiler); override;
    procedure LoadReport; dynamic;
    procedure UnloadReport; dynamic;
    procedure SaveReport; dynamic;
  public
    { Public declarations }
    procedure ShowDesigner;
    procedure SaveToStream(DataKind: TDataKind; Stream: TStream);
    procedure SaveToFile(DataKind: TDataKind; FileName: String);
    procedure LoadFromFile(DataKind: TDataKind; FileName: String);
    procedure LoadFromStream(DataKind: TDataKind; Stream: TStream);
    constructor Create(AOwner: TComponent); override;
    procedure CloseData; dynamic;
    procedure OpenData; dynamic;
    destructor Destroy; override;
    procedure PrintReport(OnScreen: Boolean = False; NumberOfCopies: Integer = 1;
      JobTitle: String = '');
    procedure LoadFromDB(DataKind: TDataKind; BlobField: TField);
    procedure SaveToDB(DataKind: TDataKind; BlobField: TField);
    procedure NewReport;
    procedure GenerateReportToStream(const Stream: TStream);
    procedure ShowReportFromStream(const Stream: TStream; OnScreen: Boolean);
    procedure GenerateReport2;
    property Designer: TfrxDesigner read FDesigner;
    property Report: TfrxReport read FReport;
    property DataObjects: TovDataObjects read FDataObjects;
    property Modified: Boolean read FModified;
  published
    { Published declarations }
    property OnLoadReport: TNotifyEvent read FOnLoadReport write FOnLoadReport;
    property OnSaveReport: TNotifyEvent read FOnSaveReport write FOnSaveReport;
    property OnPrintReport: TNotifyEvent read GetOnPrintReport write
      SetOnPrintReport;
  end;

implementation

uses ovDataObjectsEditor, Menus, frxDBSet, ovUtils, Controls, Windows,
  ovDataSet, fs_iinterpreter;

constructor TovFrxReportDesigner.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataSets := TObjectList.Create;

  FDataObjects := TovDataObjects.Create(Self);
  GenerateComponentName(FDataObjects);
  FDesigner := TfrxDesigner.Create(Self);
  FDesigner.CloseQuery := False;
  GenerateComponentName(FDesigner);
  FReport := TfrxReport.Create(Self);
  GenerateComponentName(FReport);

  Report.AddFunction('function GetObjectDataSet(DataSetName: String):TovExposer',
    'ctOther', '      ');
  Report.AddFunction('function GetQueryDataSet(DataSetName: String):TovQuery',
    'ctOther', '        ');
  Report.OnUserFunction := ReportUserFunction;
end;

destructor TovFrxReportDesigner.Destroy;
begin
  FreeAndNil(FDataSets);
  inherited Destroy;
end;

procedure TovFrxReportDesigner.PrintReport(OnScreen: Boolean = False;
  NumberOfCopies: Integer = 1; JobTitle: String = '');
begin
  LoadReport;
  try
    if OnScreen then
      Report.ShowReport
    else
    begin
      if JobTitle <> '' then Report.ReportOptions.Name := JobTitle;
      Report.PrintOptions.Copies := NumberOfCopies;
      Report.PrintOptions.ShowDialog := False;
      Report.PrepareReport;
      Report.Print;
    end;
  finally
    UnloadReport
  end;
end;

procedure TovFrxReportDesigner.GenerateReportToStream(const Stream: TStream);
begin
  LoadReport;
  try
    Report.PrepareReport;
    Report.PreviewPages.SaveToStream(Stream);
  finally
    UnloadReport
  end;
end;

procedure TovFrxReportDesigner.GenerateReport2;
begin
  LoadReport;
  try
    Report.PrepareReport;
    if Assigned(Self.OnPrintReport) then Self.OnPrintReport(Report);
  finally
    UnloadReport
  end;
end;

procedure TovFrxReportDesigner.ShowReportFromStream(const Stream: TStream;
    OnScreen: Boolean);
begin
  Report.PreviewPages.LoadFromStream(Stream);
  if OnScreen then
    Report.ShowPreparedReport
  else
    Report.Print
end;

procedure TovFrxReportDesigner.NewReport;
begin
  Report.Clear;
  FDataSets.Clear;
  DataObjects.DeleteObjects;
end;

function TovFrxReportDesigner.GetOnPrintReport: TNotifyEvent;
begin
  Result := Report.OnAfterPrintReport;
end;

procedure TovFrxReportDesigner.SetOnPrintReport(const Value: TNotifyEvent);
begin
  Report.OnAfterPrintReport := Value;
end;

{ **************************************************************************** }
{                                                                                          }
{ **************************************************************************** }

procedure TovFrxReportDesigner.ShowDesigner;
begin
  LoadReport;
  try
    DataObjects.Model := Model;

    {      TfrxDesigner, 
          ,      ,
        frxDesignerComp: TfrxDesigner   frxDesign 
         ,    ,   
      ShowDesigner      OnShow 
         SetupDesignerIDE  .

           :
    }
    frxDesignerComp := Designer;

    Designer.OnShow := DesignerShow;
    FModified := False;
    Report.Modified := False;
    Report.DesignReport;
    if FModified or Report.Modified then
    begin
      FModified := True;
      SaveReport;
    end
  finally
    UnloadReport
  end;
end;

procedure TovFrxReportDesigner.DesignerShow(Sender: TObject);
begin
  SetupDesignerIDE
end;

procedure TovFrxReportDesigner.SetupDesignerIDE;
var
  Item: TMenuItem;
  Toolbar: TToolbar;
begin
  //   
  Item := TMenuItem.Create(Self);
  Item.Caption := ' ..';
  Item.ImageIndex := Report.Designer.Menu.Items[2].Items[0].ImageIndex;
  Item.ShortCut := TextToShortCut('F4');
  Item.OnClick := DataManagerClick;
  Report.Designer.Menu.Items[2].Insert(0, Item);

  //     
{$IFDEF FR3}
  Toolbar := (Report.Designer as TfrxDesignerForm).ExtraToolsTB1;
{$ELSE} // FR4
  Toolbar := (Report.Designer as TfrxDesignerForm).ExtraToolsTB;
{$ENDIF}

  with TToolButton.Create(Toolbar) do
  begin
    Parent := ToolBar;
    Style := tbsButton;
    ImageIndex := Item.ImageIndex;
    Hint := '  ';
    OnClick := DataManagerClick;
  end;
  (Report.Designer as TfrxDesignerForm).ExtraToolsTB.AutoSize := True;

  with Report.Designer as TfrxDesignerForm do
  begin
    PreviewMI.ShortCut := TextToShortCut('F3');
    DataTreeMI.ShortCut := TextToShortCut('F12');
  end;
end;

procedure TovFrxReportDesigner.DataManagerClick(Sender: TObject);
begin
  if ShowDataObjectsEditor(DataObjects) then
  begin
    FModified := True;
    OpenData;
    Report.Designer.UpdateDataTree;
  end;
end;

{ **************************************************************************** }
{                  /                                                              }
{ **************************************************************************** }

{{
   TfrxDBDataSet  ,    
 DataObjects.
}
procedure TovFrxReportDesigner.SetupReportDataSets;

  function FindDBDataSet(ADataObject: TovDataObject): TfrxDBDataset;
  var
    I: Integer;
  begin
    for I := 0 to FDataSets.Count - 1 do
      if TfrxDBDataset(FDataSets[I]).UserName = ADataObject.Name then
      begin
        Result := TfrxDBDataset(FDataSets[I]);
        Exit;
      end;
    Result := nil
  end;

  {{
          ,
    ADataObject.
  }
  procedure DoSetupDataLink(ADataObject: TovDataObject);
  var
    DBDataSet: TfrxDBDataset;
  begin
    DBDataSet := FindDBDataSet(ADataObject);
    if Not Assigned(DBDataSet) then
    begin
      DBDataSet := TfrxDBDataset.Create(Self);
      FDataSets.Add(DBDataSet);

      DBDataSet.UserName := ADataObject.Name;
      GenerateComponentName(DBDataSet);
      DBDataSet.DataSet := DataObjects[ADataObject.Name];
      Report.DataSets.Add(DBDataSet)
    end else
      DBDataSet.DataSet := DataObjects[ADataObject.Name];
  end;

  procedure SetupDataLink(ADataObject: TovDataObject);
  var
    I: Integer;
  begin
    DoSetupDataLink(ADataObject);
    for I := 0 to ADataObject.Count - 1 do
      SetupDataLink(ADataObject.Objects[I])
  end;

var
  I: Integer;
  DS: TfrxDBDataset;
begin
  for I := 0 to DataObjects.RootObjectCount - 1 do
    SetupDataLink(DataObjects.RootObjects[I]);

   { TODO :        }
(*
  for I := Report.DataSets.Count - 1 downto 0 do
    if Not Assigned(Report.DataSets[I].DataSet) or
       Not Assigned((Report.DataSets[I].DataSet as TfrxDBDataset).DataSet) or
       Not (Report.DataSets[I].DataSet as TfrxDBDataset).DataSet.Active
    then
    begin
      DS := Report.DataSets[I].DataSet as TfrxDBDataset;
      Report.DataSets.Delete(I);
//      FDataSets.Remove(DS);
    end
*)    
end;

procedure TovFrxReportDesigner.CloseData;
begin
  DataObjects.Close;
end;

procedure TovFrxReportDesigner.OpenData;
begin
  try
    SetupReportDataSets;
  except
    on E: Exception do
      Application.MessageBox(PChar(E.Message), '  ', MB_ICONERROR or MB_OK);
  end;
end;

{ **************************************************************************** }
{                  /               }
{ **************************************************************************** }

procedure TovFrxReportDesigner.LoadReport;
begin
  if Assigned(FOnLoadReport) then FOnLoadReport(Self);
end;

procedure TovFrxReportDesigner.SaveReport;
begin
  if Assigned(FOnSaveReport) then FOnSaveReport(Self);
end;

procedure TovFrxReportDesigner.UnloadReport;
begin
  NewReport;
end;

procedure TovFrxReportDesigner.LoadDataObjectsComponent(Reader: TReader);
begin
  Reader.ReadComponent(DataObjects);
end;

procedure TovFrxReportDesigner.StoreDataObjectsComponent(Writer: TWriter);
begin
  Writer.WriteComponent(DataObjects);
end;

procedure TovFrxReportDesigner.LoadReportComponent(Reader: TReader);
begin
  FReport := Reader.ReadComponent(nil) as TfrxReport;
end;

procedure TovFrxReportDesigner.StoreReportComponent(Writer: TWriter);
begin
  Writer.WriteComponent(Report);
end;

procedure TovFrxReportDesigner.DefineProperties(Filer: TFiler);
begin
  inherited; { allow base classes to define properties }
//  Filer.DefineProperty('DataObjects', LoadDataObjectsComponent, StoreDataObjectsComponent, True);
//  Filer.DefineProperty('Report', LoadReportComponent, StoreReportComponent, True);
end;

procedure TovFrxReportDesigner.SaveToStream(DataKind: TDataKind; Stream:
  TStream);
var
  BinaryStream: TStream;
begin
  if DataKind = dkReport then
    Report.SaveToStream(Stream)
  else begin
    BinaryStream := TMemoryStream.Create;
    try
      BinaryStream.WriteComponent(DataObjects);
      BinaryStream.Position := 0;
      ObjectBinaryToText(BinaryStream, Stream);
    finally
      BinaryStream.Free;
    end;
  end
end;

procedure TovFrxReportDesigner.LoadFromStream(DataKind: TDataKind; Stream:
  TStream);
var
  BinaryStream: TMemoryStream;
begin
  if Stream.Size > 0 then
    try
      if DataKind = dkReport then
          Report.LoadFromStream(Stream)
      else begin
        BinaryStream := TMemoryStream.Create;
        try
          ObjectTextToBinary(Stream, BinaryStream);
          BinaryStream.Position := 0;
          DataObjects.DeleteObjects;
          BinaryStream.ReadComponent(DataObjects);
        finally
          BinaryStream.Free;
        end;
      end
    except
      on E: Exception do
        Application.MessageBox(PChar(E.Message), '  ', MB_ICONERROR or MB_OK);
    end
end;

procedure TovFrxReportDesigner.SaveToFile(DataKind: TDataKind; FileName:
  String);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(DataKind, Stream);
  finally
    Stream.Free;
  end;
end;

procedure TovFrxReportDesigner.LoadFromFile(DataKind: TDataKind; FileName:
  String);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(DataKind, Stream);
  finally
    Stream.Free;
  end;
end;

procedure TovFrxReportDesigner.LoadFromDB(DataKind: TDataKind; BlobField:
  TField);
var
  Stream: TStream;
begin
  Stream := BlobField.DataSet.CreateBlobStream(BlobField, bmRead);
  try
    LoadFromStream(DataKind, Stream);
  finally
    Stream.Free
  end;
end;

procedure TovFrxReportDesigner.SaveToDB(DataKind: TDataKind; BlobField: TField);
var
  Stream: TStream;
begin
  Stream := BlobField.DataSet.CreateBlobStream(BlobField, bmWrite);
  try
    SaveToStream(DataKind, Stream);
  finally
    Stream.Free;
  end;
end;

{ **************************************************************************** }
{       GetObjectDataSet  GetQueryDataSet     }
{ **************************************************************************** }

function TovFrxReportDesigner.ReportUserFunction(const MethodName: String;
  var Params: Variant): Variant;

  function GetDataSet: TDataSet;
  var
    DBDataSet: TfrxDBDataSet;
  begin
    DBDataSet := Report.GetDataSet(Params[0]) as TfrxDBDataSet;
    if Not Assigned(DBDataSet) then raise
      Exception.CreateFmt('      "%s"', [String(Params[0])]);
    Result := DBDataSet.DataSet;
  end;

begin
  if MethodName = 'GETOBJECTDATASET' then
    Result := Integer(GetDataSet as TovExposer) else
  if MethodName = 'GETQUERYDATASET' then
    Result := Integer(GetDataSet as TovQuery)
end;

type
  TFunctions = class(TfsRTTIModule)
  private
  public
    constructor Create(AScript: TfsScript); override;
  end;

constructor TFunctions.Create(AScript: TfsScript);
begin
  inherited Create(AScript);
  with AScript do
  begin
    AddClass(TovExposer, 'TDataSet');
    AddClass(TovQuery, 'TDataSet');
  end;
end;

initialization
  fsRTTIModules.Add(TFunctions);
end.
