unit ovSSForms;

interface

uses SysUtils, Classes, ssForms, ovMemObjects,


db, ovTypes, ovMemObjectManager;

type
  TovFormField = class(TssFormField)
  private
    FAttributeName: String;
    FDataObjectName: String;
    FIsLookup: Boolean;
    FLookupDisplayAttribute: String;
    FLookupListFilter: String;
    FLookupListOrder: String;
    FLookupRelation: String;
    {{
          .   
         LoadFieldData (  LoadLookupList).
    }
    FLookupList: TStringList;
    procedure LoadLookupList(ADataObject: TovMemoryObject; const ARelationName:
      String);
    function GetLookupKey: TOID;
    procedure SetLookupKey(const Value: TOID);
  protected
    function GetDataRef(var DataObject: TMemoryObject; var Attribute: String):
      Boolean;
  public
    procedure LoadFieldData;
    procedure SaveFieldData;
    procedure Assign(Source: TPersistent); override;
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    property LookupKey: TOID read GetLookupKey write SetLookupKey;
  published
    {{
       ,   DataObjectName, 
       .
    }
    property AttributeName: String read FAttributeName write FAttributeName;
    {{
      ,      .
    }
    property DataObjectName: String read FDataObjectName write FDataObjectName;

    {{
       ,     .
    }
    property IsLookup: Boolean read FIsLookup write FIsLookup default False;
    property LookupDisplayAttribute: String read FLookupDisplayAttribute write
      FLookupDisplayAttribute;
    property LookupListFilter: String read FLookupListFilter write
      FLookupListFilter;
    property LookupListOrder: String read FLookupListOrder write FLookupListOrder;
    property LookupRelation: String read FLookupRelation write FLookupRelation;
  end;

  TovForm = class(TssForm)
  private
    function GetFieldCount: Integer;
    function GetFields(Index: Integer): TovFormField;
  public
    procedure LoadFieldData;
    procedure SaveFieldData;
    function FindFieldByAttributeName(AttributeName: String): TovFormField;
    function FieldByName(FieldName: String): TovFormField;
    property FieldCount: Integer read GetFieldCount;
    property Fields[Index: Integer]: TovFormField read GetFields;
  end;

  TGetDataObjectEvent = procedure (Sender: TObject; Field: TovFormField;
    var DataObject: TMemoryObject) of object;

  TovSpreadSheetForms = class(TSpreadSheetForms)
  private
    FOnGetDataObject: TGetDataObjectEvent;
    FDataObjects: TStringList;
    FLoadLookupLists: Boolean;
    function GetForms(Index: Integer): TovForm;
    function GetFormCount: Integer;
  protected
    function GetObjectClass(ObjectClassType: TObjectClassType): TClass; override;
    function GetDataObject(AField: TovFormField; ADataObjectName: String):
      TMemoryObject; virtual;
  public
    procedure LoadFieldData;
    procedure SaveFieldData;
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddDataObject(ADataObjectName: String; ADataObject: TMemoryObject);
    property Forms[Index: Integer]: TovForm read GetForms;
    property FormCount: Integer read GetFormCount;
  published
    {{
         .   
      .
    }
    property LoadLookupLists: Boolean read FLoadLookupLists write FLoadLookupLists
      default True;
    property OnGetDataObject: TGetDataObjectEvent read FOnGetDataObject write
      FOnGetDataObject;
  end;

implementation

uses Variants, ovIObjects;

{{
   Expression     "."
     FirstPart.      "."
  RestPart.   "."   ,  FirstPart = ''.
}
procedure ParseReference(const Expression: String; var FirstPart, RestPart:
  String);
var
  P: Integer;
begin
  P := Pos('.', Expression);
  if P > 0 then
  begin
    FirstPart := Copy(Expression, 1, P-1);
    RestPart := Copy(Expression, P+1, Length(Expression));
  end
  else
  begin
    FirstPart := '';
    RestPart := Expression;
  end;
end;

type
  TPath = array of String;

{{
    ,     .
 }
function ConvertPath(const APath: String): TPath;
var
  FirstPart, RestPart: String;
  Count: Integer;
  VA: Variant;
begin
  VA := VarArrayCreate([0, 15], varOleStr);
  Count := 0;
  repeat
    ParseReference(APath, FirstPart, RestPart);
    if FirstPart <> '' then
    begin
      VA[Count] := FirstPart;
      Inc(Count);
    end;
  until FirstPart = '';
  VA[Count] := RestPart;
  Inc(Count);
  VarArrayRedim(VA, Count-1);
  Result := VA;
end;

function GetObjectInternal(ThisObject: TovMemoryObject; const APath: array of String; AIndex: Integer):
  TMemoryObject;
var
  AObject: TovMemoryObject;
  ARelation: TMemoryObjectRelation;
  ARelationObjectClassName: String;
begin
  ARelation := TMemoryObjectRelation(ThisObject.FindRelationByName(APath[AIndex]));
  if ARelation.ObjectCount = 0 then
  begin
    ARelationObjectClassName := ARelation.DefaultObjectClassName;
    Result := ThisObject.ObjectManager.CreateObjectInstance(ARelationObjectClassName,
      ThisObject.ObjectManager.GenerateObjectID(ARelationObjectClassName), False, ThisObject);
    ARelation.AddObject(Result);
  end
  else begin
    AObject := ARelation.Objects[0] as TovMemoryObject;
    if AIndex >= High(APath) then Result := AObject
    else Result := GetObjectInternal(AObject, APath, AIndex + 1)
  end;
end;

{{
  ,      ,
   APath.      
   .

:
-   R = APath[0]    O = R.Objects[0]
-    ,    O,   
   O.FindObject   ,    APath[1]
}
function GetObject(ThisObject: TovMemoryObject; const APath: array of String): TMemoryObject;
begin
  Result := GetObjectInternal(ThisObject, APath, Low(APath))
end;

{ TovSpreadSheetForms }

constructor TovSpreadSheetForms.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FDataObjects := TStringList.Create;
  FDataObjects.Sorted := True;
  FDataObjects.Duplicates := dupError;

  LoadLookupLists := True;
end;

destructor TovSpreadSheetForms.Destroy;
begin
  FreeAndNil(FDataObjects);
  inherited Destroy;
end;

function TovSpreadSheetForms.GetObjectClass(ObjectClassType: TObjectClassType):
  TClass;
begin
  case ObjectClassType of
    octForm  : Result := TovForm;
    octField : Result := TovFormField;
    else Result := inherited GetObjectClass(ObjectClassType)
  end;
end;

function TovSpreadSheetForms.GetForms(Index: Integer): TovForm;
begin
  Result := inherited Forms[Index] as TovForm;
end;

function TovSpreadSheetForms.GetFormCount: Integer;
begin
  Result := inherited Forms.Count;
end;

procedure TovSpreadSheetForms.LoadFieldData;
var
  I: Integer;
begin
  for I := 0 to FormCount - 1 do
    Forms[I].LoadFieldData;
end;

procedure TovSpreadSheetForms.SaveFieldData;
var
  I: Integer;
begin
  for I := 0 to FormCount - 1 do
    Forms[I].SaveFieldData;
end;

{{
    .       , 
  .
}
procedure TovSpreadSheetForms.AddDataObject(ADataObjectName: String;
  ADataObject: TMemoryObject);
var
  I: Integer;
begin
  I := FDataObjects.IndexOf(ADataObjectName);
  if I < 0 then
    FDataObjects.AddObject(ADataObjectName, ADataObject)
  else
    FDataObjects.Objects[I] := ADataObject
end;

{{
         .
}
function TovSpreadSheetForms.GetDataObject(AField: TovFormField;
  ADataObjectName: String): TMemoryObject;
var
  I: Integer;
begin
  I := FDataObjects.IndexOf(ADataObjectName);
  if I >= 0 then
    Result := TMemoryObject(FDataObjects.Objects[I])
  else
    Result := nil;
  if Assigned(FOnGetDataObject) then FOnGetDataObject(Self, AField, Result);
  if Not Assigned(Result) then raise Exception.Create('     :'+ADataObjectName);
end;

{ TovForm }

constructor TovFormField.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FLookupList := TStringList.Create;
end;

destructor TovFormField.Destroy;
begin
  FreeAndNil(FLookupList);
  inherited Destroy;
end;

function TovForm.GetFieldCount: Integer;
begin
  Result := inherited Fields.Count;
end;

function TovForm.GetFields(Index: Integer): TovFormField;
begin
  Result := inherited Fields[Index] as TovFormField;
end;

procedure TovForm.LoadFieldData;
var
  I: Integer;
begin
  for I := 0 to FieldCount - 1 do Fields[I].LoadFieldData;
end;

procedure TovForm.SaveFieldData;
var
  I: Integer;
begin
  for I := 0 to FieldCount - 1 do
    Fields[I].SaveFieldData;
end;

function TovForm.FindFieldByAttributeName(AttributeName: String): TovFormField;
var
  I: Integer;
begin
  for I := 0 to FieldCount - 1 do
    if Fields[I].AttributeName = AttributeName then
    begin
      Result := Fields[I];
      Exit;
    end;
  Result := nil;
end;

function TovForm.FieldByName(FieldName: String): TovFormField;
var
  I: Integer;
begin
  for I := 0 to FieldCount - 1 do
    if Fields[I].Name = FieldName then
    begin
      Result := Fields[I];
      Exit;
    end;
  raise Exception.CreateFmt(' %s   ', [FieldName]);
end;

{ TovFormField }

procedure TovFormField.Assign(Source: TPersistent);
begin
  inherited;
  if Source is TovFormField then with TovFormField(Source) do
  begin
    Self.DataObjectName := DataObjectName;
    Self.AttributeName := AttributeName;

    Self.IsLookup := IsLookup;
    Self.LookupDisplayAttribute := LookupDisplayAttribute;
    Self.LookupRelation := LookupRelation;
    Self.LookupListFilter := LookupListFilter;
    Self.LookupListOrder := LookupListOrder;
  end
end;

{{
  ,         (
IsLookup = False).        
   (    ).
}
function TovFormField.GetDataRef(var DataObject: TMemoryObject; var Attribute:
  String): Boolean;

  function GetDataObject: TMemoryObject;
  var
    FirstPart, RestPart: String;
  begin
    if DataObjectName <> '' then
    begin
      ParseReference(DataObjectName, FirstPart, RestPart);
      if FirstPart = '' then
        {     }
        Result := (Form.SpreadSheet as TovSpreadSheetForms).GetDataObject(Self, DataObjectName)
      else
      begin
        {       }
        Result := (Form.SpreadSheet as TovSpreadSheetForms).GetDataObject(Self, FirstPart);
        Result := GetObject(Result as TovMemoryObject, ConvertPath(RestPart));
      end;
    end else
      Result := nil
  end;

var
  NameInBrackets: String;
  DataRef: String;
  FirstPart, RestPart: String;
begin
  if (DataObjectName <> '') and ((AttributeName <> '') or IsLookup) then
  begin
    {         }
    DataObject := GetDataObject;
    Attribute := AttributeName;
    Result := Assigned(DataObject);  {     ,
           (  ).
              }
  end else
  begin
    {          }
    NameInBrackets := Trim(Cell.AsString);
    DataRef := Copy(NameInBrackets, 2, Length(NameInBrackets)-2);
    ParseReference(DataRef, FirstPart, RestPart);
    Result := DataRef <> '';
    if Result then
    begin
      DataObject := (Form.SpreadSheet as TovSpreadSheetForms).GetDataObject(Self, FirstPart);
      AttributeName := RestPart;
    end;
  end;

  { TODO :        
  (  ), .. < >.<> }
  if IsLookup then Attribute := LookupRelation;
end;

function TovFormField.GetLookupKey: TOID;
var
  ItemIndex: Integer;
begin
  ItemIndex := Cell.Value;
  Result := TOID(FLookupList.Objects[ItemIndex]);
end;

procedure TovFormField.SetLookupKey(const Value: TOID);
begin
  Cell.Value := FLookupList.IndexOfObject(Pointer(Value));
end;

{***************************************************************************** }
{                                                                }
{ **************************************************************************** }

{{
   ,      .
    ,     
    .
}
procedure TovFormField.LoadFieldData;
var
  DataObject: TMemoryObject;
  Attribute: String;
  Relation: String;
  AValue: Variant;
  AObjectId: TOID;
begin
  if Not IsLookup then
  begin
    if GetDataRef(DataObject, Attribute) then
      AValue := DataObject[Attribute].Value
    else
      AValue := Null;
    if VarIsNull(AValue) then Clear
    else Value := AValue;
  end else
  if GetDataRef(DataObject, Relation) then
    if DataObject.RelationByName[Relation].ObjectCount > 0 then
    begin
      AObjectId := DataObject.RelationByName[Relation].ObjectIDs[0];
      if TovSpreadSheetForms(Form.SpreadSheet).LoadLookupLists then
        LoadLookupList(DataObject as TovMemoryObject, Relation); {   Lookup-
            TovMemoryObject (    TMemoryObject) }
      LookupKey := AObjectId;
    end else
    begin
      if TovSpreadSheetForms(Form.SpreadSheet).LoadLookupLists then
        LoadLookupList(DataObject as TovMemoryObject, Relation);
      Clear;
    end
  else Clear
end;

{{
          
  FLookupList.
}
procedure TovFormField.LoadLookupList(ADataObject: TovMemoryObject; const
  ARelationName: String);
var
  ALookupItemsClassName: String;
begin
  ALookupItemsClassName := ADataObject.ModelObject.GetRelation(ARelationName).TargetObjectName;
  {    ,    
    LookupRelation    }
  with ADataObject.ObjectManager.Model.ObjectSpace.Classes[ALookupItemsClassName].Objects do
    Select(FLookupList, LookupDisplayAttribute, LookupListFilter, LookupListOrder);
  Cell.SetComboBoxItems(FLookupList);
end;

{***************************************************************************** }
{                                                              }
{ **************************************************************************** }

{{
        ,  .
    ,     
       .

         LoadFieldData
 LoadLookupList.
}
procedure TovFormField.SaveFieldData;
var
  DataObject: TMemoryObject;
  Attribute: String;
  Relation: String;
begin
  if Not IsLookup then
  begin
    if GetDataRef(DataObject, Attribute) then
      DataObject[Attribute].Value  := Value
  end else
  if GetDataRef(DataObject, Relation) then
  begin
    if Value = '' then
      DataObject.RelationByName[Relation].AsObjectID := 0
    else
      DataObject.RelationByName[Relation].AsObjectID := LookupKey;
  end;
end;

end.
