unit ovSSReportsDataLinksExplorer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ImgList, AppEvnts, ActnList, ComCtrls, ToolWin,
  ovDataLinks, ovSSReportsLinks, ovDataLinksExplorer, ssReports;

type
  TfrmSSReportsDataLinksExplorer = class(TfrmDataLinksExplorer)
  private
    function FindDataSourceLink(ADataSource: TssReportDataSource): TovDataLink;
    { Private declarations }
  protected
    function CanLink(AComponent: TComponent): Boolean; override;
    function GetChildObject(AObject: TPersistent; Index: Integer): TPersistent; 
      override;
    function GetChildObjectCount(AObject: TPersistent): Integer; override;
    function GetObjectInfo(AObject: TPersistent; var ATitle, ADataObject, 
      AnAttribute, ALookupDataObject, ALookupAttributes: String): Boolean; override;
    function FindLink(AObject: TPersistent): TovDataLink; override;
  public
    procedure AddDataObjectLink(DataObjectName: String; ObjectIndex: Integer); 
      override;
    procedure AddAttributeLink(DataObjectName: String; Attributes: TStrings; 
      ObjectIndex: Integer); override;
    procedure AddLookupLink(ALookupDataObjectName, ALookupAttributes: String; 
      ObjectIndex: Integer); override;
    { Public declarations }
  end;

var
  frmSSReportsDataLinksExplorer: TfrmSSReportsDataLinksExplorer;

implementation

{$R *.dfm}

function TfrmSSReportsDataLinksExplorer.FindDataSourceLink(ADataSource:
  TssReportDataSource): TovDataLink;
var
  I: Integer;
begin
  with DataLinks do
  for I := 0 to Links.Count - 1 do
    if (Links[I].Controller is TovSSReportsDataLinkController) and
       (TovSSReportsDataLinkController(Links[I].Controller).DataSource = ADataSource) then
    begin
      Result :=	Links[I];
      Exit;
    end;
  Result := nil;
end;

function TfrmSSReportsDataLinksExplorer.GetChildObjectCount(AObject:
  TPersistent): Integer;
begin
  if AObject is TSpreadSheetReports then
    Result := TSpreadSheetReports(AObject).DataSourceCount
  else
    Result := 0;
end;

function TfrmSSReportsDataLinksExplorer.GetChildObject(AObject: TPersistent;
  Index: Integer): TPersistent;
begin
  if AObject is TSpreadSheetReports then
    Result := TSpreadSheetReports(AObject).DataSources[Index]
  else
    Result := nil;
end;

function TfrmSSReportsDataLinksExplorer.GetObjectInfo(AObject: TPersistent; var
  ATitle, ADataObject, AnAttribute, ALookupDataObject, ALookupAttributes:
  String): Boolean;
begin
  Result := inherited GetObjectInfo(AObject,
    ATitle, ADataObject, AnAttribute, ALookupDataObject, ALookupAttributes);

  if AObject is TssReportDataSource then
    ATitle := TssReportDataSource(AObject).Name
end;

function TfrmSSReportsDataLinksExplorer.CanLink(AComponent: TComponent): Boolean;
begin
  Result := AComponent is TSpreadSheetReports
end;

function TfrmSSReportsDataLinksExplorer.FindLink(AObject: TPersistent):
  TovDataLink;
begin
  if AObject is TSpreadSheetReports then
    Result := nil
  else
  if AObject is TssReportDataSource then
    Result := FindDataSourceLink(TssReportDataSource(AObject))
  else
    Result := inherited FindLink(AObject)
end;

procedure TfrmSSReportsDataLinksExplorer.AddDataObjectLink(DataObjectName: 
  String; ObjectIndex: Integer);
var
  AObject: TObject;
  Link: TovDataLink;
begin
  AObject := GetObjectAt(ObjectIndex);

  if AObject is TSpreadSheetReports then begin
    //           
    TSpreadSheetReports(AObject).AddDataSource(DataObjectName, nil);
    (DataLinks.AddLink(TovSSReportsDataLinkController,
      TComponent(AObject), DataObjectName,
      '').Controller as TovSSReportsDataLinkController).DataSourceName := DataObjectName;
    ShowComponentList;
  end else
  if AObject is TssReportDataSource then
  begin
    ObjectIndex := GetParentObject(ObjectIndex);
    if GetObjectAt(ObjectIndex) is TSpreadSheetReports then
    begin
      Link := GetLinkAt(ObjectIndex);
      if Link <> nil then
      begin
        Link.DataObject := DataObjectName
      end else
        (DataLinks.AddLink(TovSSReportsDataLinkController,
          TComponent(GetObjectAt(ObjectIndex)), DataObjectName,
          '').Controller as TovSSReportsDataLinkController).DataSourceName := DataObjectName;
      ShowComponentList;
    end
  end
end;

{{
    
}
procedure TfrmSSReportsDataLinksExplorer.AddAttributeLink(DataObjectName:
  String; Attributes: TStrings; ObjectIndex: Integer);
begin
end;

{{
    
}
procedure TfrmSSReportsDataLinksExplorer.AddLookupLink(ALookupDataObjectName,
  ALookupAttributes: String; ObjectIndex: Integer);
begin
end;


initialization
  RegisterDataLinkExplorer('  ', TfrmSSReportsDataLinksExplorer);
finalization
  UnregisterDataLinkExplorer(TfrmSSReportsDataLinksExplorer);
end.
