unit sdDBFormDesigner;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, sdBaseProps, ssForms, sdFormDesigner, cxGraphics, Menus,
  ssComponents, dxDockControl, dxBar, ActnList, dxBarExtItems, ExtDlgs,
  AppEvnts, ImgList, StdCtrls, dxDockPanel, cxControls, cxContainer,
  cxEdit, cxTextEdit, cxMaskEdit, cxDropDownEdit, cxColorComboBox, ComCtrls,
  sdDesigner;

type
  TDropFieldMode = (dfmDataField, dfmLookupKeyField, dfmLookupDisplayField);

  TfrmDBFormDesigner = class(TfrmSSFormDesigner)
    DropFieldPopup: TPopupMenu;
    DropDataFieldItem: TMenuItem;
    DropKeyFieldItem: TMenuItem;
    DropDisplayFieldItem: TMenuItem;
    procedure dpDataNavigatorVisibleChanging(Sender: TdxCustomDockControl);
    procedure DropDataFieldItemClick(Sender: TObject);
    procedure DropDisplayFieldItemClick(Sender: TObject);
    procedure DropKeyFieldItemClick(Sender: TObject);
  private
    FDropFieldCol: Integer;
    FDropFieldRow: Integer;
    FDropFieldName: String;
    FDropFieldDataSourceName: String;
    procedure DragDropDataField(ACol, ARow: Integer; ADataSourceName, AFieldName:
      String; ADropMode: TDropFieldMode);
    procedure SpreadSheetDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure SpreadSheetDragOver(Sender, Source: TObject; X, Y: Integer; State:
      TDragState; var Accept: Boolean);
    { Private declarations }
  protected
    function CreateObjectProperties(ASpreadSheet: TSpreadSheetForms):
        TfrmBaseProperties; override;
    function CreateSpreadSheet: TSpreadSheet; override;
    procedure InitSpreadSheet; override;
  public
    { Public declarations }
  end;

var
  frmDBFormDesigner: TfrmDBFormDesigner;

implementation

uses ssObjects, ssDBForms, sdUtility, sdDBFormProps, sdDataObjectNavigator, ovSSDBForms;

{$R *.dfm}

{ TfrmDBFormDesigner }

{***************************************************************************** }
{                                              }
{ **************************************************************************** }

function TfrmDBFormDesigner.CreateSpreadSheet: TSpreadSheet;
begin
  Result := TovDBSpreadSheetForms.Create(Self);
end;

function TfrmDBFormDesigner.CreateObjectProperties(ASpreadSheet:
    TSpreadSheetForms): TfrmBaseProperties;
begin
  Result := TfrmDBFormProperties.Create(Self);
  TfrmDBFormProperties(Result).SpreadSheet := ASpreadSheet;
end;

procedure TfrmDBFormDesigner.InitSpreadSheet;
begin
  inherited;
  SpreadSheet.OnDragDrop := SpreadSheetDragDrop;
  SpreadSheet.OnDragOver := SpreadSheetDragOver;
end;

{***************************************************************************** }
{                                              }
{ **************************************************************************** }

procedure TfrmDBFormDesigner.dpDataNavigatorVisibleChanging(Sender:
  TdxCustomDockControl);
begin
  inherited;
  if Not Assigned(fmDataObjectNavigator) then
  begin
    fmDataObjectNavigator := TfmDataObjectNavigator.Create(Self);
    DockForm(fmDataObjectNavigator, dpDataNavigator);
    fmDataObjectNavigator.RefreshDataTree((SpreadSheet as TovDBSpreadSheetForms).DataObjects);
  end;
end;

procedure TfrmDBFormDesigner.SpreadSheetDragOver(Sender, Source: TObject; X, Y:
  Integer; State: TDragState; var Accept: Boolean);
begin
  inherited;
  Accept := Assigned(fmDataObjectNavigator) and (Source = fmDataObjectNavigator.DataTree) and
    Assigned(fmDataObjectNavigator.DataTree.Selected) and
    Assigned(fmDataObjectNavigator.DataTree.Selected.Parent)
end;

procedure TfrmDBFormDesigner.SpreadSheetDragDrop(Sender, Source: TObject; X, Y:
  Integer);
var
  ACol, ARow: Integer;
  P: TPoint;
begin
  inherited;
  SpreadSheet.ActiveSheet.GetCellAt(X, Y, ACol, ARow);
  with fmDataObjectNavigator.DataTree do
  if Not IsCtrlPressed then
    DragDropDataField(ACol, ARow, Selected.Parent.Text, Selected.Text, dfmDataField)
  else begin
    FDropFieldCol := ACol;
    FDropFieldRow := ARow;
    FDropFieldName := Selected.Text;
    FDropFieldDataSourceName := Selected.Parent.Text;
    P := (Sender as TControl).ClientToScreen(Point(X, Y));
    DropFieldPopup.Popup(P.X, P.Y);
  end
end;

procedure TfrmDBFormDesigner.DropDataFieldItemClick(Sender: TObject);
begin
  inherited;
  DragDropDataField(FDropFieldCol, FDropFieldRow, FDropFieldDataSourceName, FDropFieldName, dfmDataField)
end;

procedure TfrmDBFormDesigner.DropDisplayFieldItemClick(Sender: TObject);
begin
  inherited;
  DragDropDataField(FDropFieldCol, FDropFieldRow, FDropFieldDataSourceName, FDropFieldName, dfmLookupDisplayField)
end;

procedure TfrmDBFormDesigner.DropKeyFieldItemClick(Sender: TObject);
begin
  inherited;
  DragDropDataField(FDropFieldCol, FDropFieldRow, FDropFieldDataSourceName, FDropFieldName, dfmLookupKeyField)
end;

{{
   AFieldName   ADataSourceName   
  (      )  
 ACol  ARow.
}
procedure TfrmDBFormDesigner.DragDropDataField(ACol, ARow: Integer;
  ADataSourceName, AFieldName: String; ADropMode: TDropFieldMode);
var
  Form: TssForm;
  Field: TssDBFormField;
begin
  with SpreadSheet as TovDBSpreadsheetForms do
  begin
    Form := ActiveForm;
    if Form <> nil then
    begin
      Field := Form.FieldByCell(ACol, ARow) as TssDBFormField;
      if Field = nil then
      begin
        Field := Form.Fields.Add(ACol, ARow) as TssDBFormField;
      end;

      case ADropMode of
        dfmDataField:
        begin
          Field.DataSourceName := ADataSourceName;
          Field.DataField := AFieldName;
        end;
        dfmLookupDisplayField:
        begin
          Field.IsLookup := True;
          Field.LookupDataSourceName := ADataSourceName;
          Field.LookupDisplayField := AFieldName;
        end;
        dfmLookupKeyField:
        begin
          Field.IsLookup := True;
          Field.LookupDataSourceName := ADataSourceName;
          Field.LookupKeyField := AFieldName;
        end;
      end;

      UpdateNavigator(True);
      UpdateObjectProperties;
    end;
  end
end;



end.
