{***************************************************************************** }
{                                                       }
{ **************************************************************************** }

unit ovDataGen;

interface

uses Classes, ovModel, ovIObjects;

{{
  ANumberOfInstance    AModelObject.
       
ANumberOfChildInstance  .  IgnoreRelations 
  ,       ,
,   .     
  <  >.< >
}
procedure GenerateTestObjects(AObjectSpace: IovObjectSpace; AModelObject:
  TModelObject; ANumberOfInstance, ANumberOfChildInstance: Integer;
  IgnoreRelations: TStrings);

implementation

uses SysUtils, DB, ovProgress, Math, TypInfo;

function GenerateAttributeValue(AFieldType: TFieldType; AFieldSize: Integer):
  Variant;
begin
  case AFieldType of
    ftInteger, ftSmallint: Result := Trunc(Random * 1000);
    ftFloat : Result := Random * 100;
    ftString: Result := Copy(' ' + IntToStr(Trunc(Random * 1000)), 1, AFieldSize);
    ftBoolean: Result := Random > 0.5;
    ftDate, ftDateTime, ftTime: Result := Now;
    ftMemo, ftBlob: Result := ' 1'#13' 2'#13' 3';
    else
      raise Exception.Create('      ' +
        GetEnumName(TypeInfo(TFieldType), Ord(AFieldType)));
  end
end;

{{
 ObjectName      
    .
 IgnoreRelations      ,
 .. .
}
procedure GenerateTestData(AObject: IovObject; AModelObject: TModelObject;
  ANumberOfChildInstance: Integer; IgnoreRelations: TStrings; ObjectName:
  String);

  function IsRelationIgnored(Relation: TModelRelation): Boolean;
  begin
    Result := IgnoreRelations.IndexOf(AModelObject.FullName + '.' + Relation.Name) >= 0
  end;

var
  I, J: Integer;
  AFieldType: TFieldType;
  AFieldSize: Integer;
  AChildObject: IovObject;
begin
  with AModelObject do
  begin
    for I := 0 to AttributeCount - 1 do
      with Attributes[I] do
      if Not Calculated then
      begin
        GetFieldDef(AFieldType, AFieldSize);
        AObject[Name].Value := GenerateAttributeValue(AFieldType, AFieldSize);
      end;

    for I := 0 to RelationCount - 1 do
    if Not IsRelationIgnored(Relations[I]) then
    with Relations[I] do
    begin
      {     }
      for J := 0 to AObject[Name].AsObjects.Count - 1 do
      begin
        AChildObject := AObject[Name].AsObjects[J];
        GenerateTestData(AChildObject, TargetObject, ANumberOfChildInstance, IgnoreRelations, Relations[I].Name);
      end;

      {     }
      for J := 1 to Min(ANumberOfChildInstance, TargetMaxCount) do
      begin
        AChildObject := AObject[Name].Add;
        GenerateTestData(AChildObject, TargetObject, ANumberOfChildInstance, IgnoreRelations, Relations[I].Name);
      end;
    end;
  end;
end;

procedure GenerateTestObject(AObjectSpace: IovObjectSpace; AModelObject:
  TModelObject; ANumberOfChildInstance: Integer; IgnoreRelations: TStrings);
var
  AObject: IovObject;
begin
  AObjectSpace.StartTransaction;
  try
    AObject := AObjectSpace.CreateObject(AModelObject.FullName);
    GenerateTestData(AObject, AModelObject, ANumberOfChildInstance, IgnoreRelations, AModelObject.FullName);
    AObjectSpace.CommitTransaction;
  except
    AObjectSpace.RollbackTransaction;
    raise;
  end;
end;

procedure GenerateTestObjects(AObjectSpace: IovObjectSpace; AModelObject:
  TModelObject; ANumberOfInstance, ANumberOfChildInstance: Integer;
  IgnoreRelations: TStrings);
var
  I: Integer;
begin
  ShowProgress(ANumberOfInstance, '   ' + AModelObject.FullName);
  try
    for I := 1 to ANumberOfInstance do
    begin
      GenerateTestObject(AObjectSpace, AModelObject, ANumberOfChildInstance, IgnoreRelations);
      if Not ProgressStep then Break;
    end;
  finally
    HideProgress
  end;
end;

initialization
  Randomize;
end.
