unit ovAttributeSelectionDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, ImgList,
  ovComponents, ovModelTreeViewBuilder, ovModel, ovCheckTreeView, AppEvnts;

type
  { ,       }
  TSelectOption = (
    //  ,        1
    soShowMultiRelations,
    //  ,     
    soAllowSelectRelations,
    //  ,    , .. 
    //     
    soExpandRelations,
    soChangeAttributeOrder);
  TSelectOptions = set of TSelectOption;

  TfmAttributeSelectionDialog = class(TForm)
    PageControl: TPageControl;
    TabSheet1: TTabSheet;
    TreeView: TovCheckTreeView;
    DescriptionMemo: TMemo;
    tsAttributeOrder: TTabSheet;
    SelectionList: TListBox;
    CancelBtn: TBitBtn;
    OkBtn: TBitBtn;
    Memo1: TMemo;
    SelectAllBtn: TBitBtn;
    UnselectAllBtn: TBitBtn;
    ImageList: TImageList;
    TreeViewBuilder: TovModelTreeViewBuilder;
    Memo2: TMemo;
    MoveDownBtn: TBitBtn;
    MoveUpBtn: TBitBtn;
    ApplicationEvents: TApplicationEvents;
    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    procedure FormShow(Sender: TObject);
    procedure MoveDownBtnClick(Sender: TObject);
    procedure MoveUpBtnClick(Sender: TObject);
    procedure SelectAllBtnClick(Sender: TObject);
    procedure TreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure TreeViewStateChange(Sender: TObject; Node: TTreeNode; NewState:
      TCheckState);
  private
    FSelectOptions: TSelectOptions;
    { Private declarations }
    procedure SelectAll(SetChecked: Boolean);
    procedure GetSelection(ParentNode: TTreeNode; Selection: TStrings; Path:
      String); overload;
    procedure SelectAttribute(AttributePath: String);
    procedure SwapItems(ItemIndex1, ItemIndex2: Integer);
    function GetNodeSelectionText(Node: TTreeNode): String; overload;
    property SelectOptions: TSelectOptions read FSelectOptions;
  public
    { Public declarations }
    procedure BuildTreeView; dynamic;
    procedure GetSelection(Selection: TStrings); overload;
    procedure SetSelection(Selection: TStrings);
    { Public declarations }
  end;

function SelectObjectAttributes(AModel: TovModel; AModelObject: TModelObject;
  ASelection: TStrings; ACaption: String; AOptions: TSelectOptions): Boolean;
  overload;

function SelectObjectAttributes(AModel: TovModel; AModelObject: TModelObject;
  var ASelection: String; ACaption: String; AOptions: TSelectOptions): Boolean;
  overload;

implementation

uses ovUtils;

{$R *.dfm}

function SelectObjectAttributes(AModel: TovModel; AModelObject: TModelObject;
  ASelection: TStrings; ACaption: String; AOptions: TSelectOptions): Boolean;
var
  RootObject: TRootObject;
begin
  with TfmAttributeSelectionDialog.Create(Application) do
  try
    FSelectOptions := AOptions;
    Caption := ACaption;
    TreeViewBuilder.Model := AModel;
    RootObject.RootObjectType := rtObject;
    RootObject.RootObject := AModelObject;
    TreeViewBuilder.RootObject := RootObject;
    SelectionList.Items.Assign(ASelection);
    Result := ShowModal = mrOk;
    if Result then ASelection.Assign(SelectionList.Items);
  finally
    Free
  end;
end;

function SelectObjectAttributes(AModel: TovModel; AModelObject: TModelObject;
  var ASelection: String; ACaption: String; AOptions: TSelectOptions): Boolean;
var
  Selection: TStringList;
begin
  Selection := TStringList.Create;
  try
    (*
      , ..   ,   
      / 
    Selection.Sorted := True;
    Selection.Duplicates := dupIgnore;
    *)
    AttributesToList(ASelection, Selection);
    Result := SelectObjectAttributes(AModel, AModelObject, Selection, ACaption, AOptions);
    if Result then
      ASelection := AttributesFromList(Selection);
  finally
    Selection.Free;
  end;
end;

{{
    NodeText,      ParentNode.
}
function FindNodeByText(ParentNode: TTreeNode; NodeText: String; var ChildNode:
  TTreeNode): Boolean;
var
  I: Integer;
begin
  for I := 0 to ParentNode.Count - 1 do
    if ParentNode.Item[I].Text = NodeText then
    begin
      Result := True;
      ChildNode := ParentNode.Item[I];
      Exit;
    end;
  Result := False;
  ChildNode := nil;
end;

{ TfmAttributeSelectionDialog }

procedure TfmAttributeSelectionDialog.FormShow(Sender: TObject);
begin
  if soShowMultiRelations in SelectOptions then
    TreeViewBuilder.Options := TreeViewBuilder.Options - [tvbShowSingleRelationsOnly]
  else
    TreeViewBuilder.Options := TreeViewBuilder.Options + [tvbShowSingleRelationsOnly];
  if soExpandRelations in SelectOptions then
    TreeViewBuilder.Options := TreeViewBuilder.Options + [tvbExpandRelations]
  else
    TreeViewBuilder.Options := TreeViewBuilder.Options - [tvbExpandRelations];

  BuildTreeView;
  SetSelection(SelectionList.Items);
  TreeView.OnStateChange := TreeViewStateChange;

  tsAttributeOrder.TabVisible := soChangeAttributeOrder in FSelectOptions;
end;

procedure TfmAttributeSelectionDialog.BuildTreeView;
begin
  TreeViewBuilder.BuildTreeView;
  if TreeView.Items.Count > 0 then TreeView.Items[0].Selected := True;
end;

procedure TfmAttributeSelectionDialog.TreeViewChange(Sender: TObject; Node:
  TTreeNode);
begin
  if TreeViewBuilder.NodeTypes[Node] = ntAttribute then
    DescriptionMemo.Lines.Text := TModelAttribute(TreeViewBuilder.NodeObjects[Node]).Description
  else
  if TreeViewBuilder.NodeTypes[Node] = ntRelation then
    DescriptionMemo.Lines.Text := TModelRelation(TreeViewBuilder.NodeObjects[Node]).Description
  else
    DescriptionMemo.Lines.Text := ''
end;

procedure TfmAttributeSelectionDialog.SelectAllBtnClick(Sender: TObject);
begin
  SelectAll(Sender = SelectAllBtn);
end;

procedure TfmAttributeSelectionDialog.TreeViewStateChange(Sender: TObject;
  Node: TTreeNode; NewState: TCheckState);
var
  S: String;
  I: Integer;
begin
  S := GetNodeSelectionText(Node);
  I := SelectionList.Items.IndexOf(S);

  if (NewState = csUnchecked) and (I >= 0) then
    SelectionList.Items.Delete(I);

  if (S <> '') and
     ( (TreeViewBuilder.NodeTypes[Node] <> ntRelation) and (TreeViewBuilder.NodeTypes[Node] <> ntRelationNodes) or
       (soAllowSelectRelations in SelectOptions)
     )
  then begin
    if (NewState <> csUnchecked) and (I < 0) then
      SelectionList.Items.Add(S)
  end;
end;

procedure TfmAttributeSelectionDialog.MoveDownBtnClick(Sender: TObject);
begin
  SwapItems(SelectionList.ItemIndex, SelectionList.ItemIndex+1);
end;

procedure TfmAttributeSelectionDialog.MoveUpBtnClick(Sender: TObject);
begin
  SwapItems(SelectionList.ItemIndex, SelectionList.ItemIndex-1);
end;

procedure TfmAttributeSelectionDialog.ApplicationEventsIdle(Sender: TObject;
  var Done: Boolean);
begin
  MoveDownBtn.Enabled := (SelectionList.ItemIndex >= 0) and
    (SelectionList.ItemIndex < SelectionList.Items.Count-1);
  MoveUpBtn.Enabled := (SelectionList.ItemIndex > 0);
end;

{***************************************************************************** }
{                                                         }
{ **************************************************************************** }

{{
    ,    
}
procedure TfmAttributeSelectionDialog.SelectAll(SetChecked: Boolean);

  procedure SetRootState(Checked: Boolean);
  var
    Node: TTreeNode;
  begin
    with TreeView do begin
      Node := Items.GetFirstNode;
      if Assigned(Node) then
      begin
        if Checked
          then ItemState[Node.AbsoluteIndex] := csChecked
          else ItemState[Node.AbsoluteIndex] := csUnChecked;
      end;
    end
  end;

begin
  with TreeView do begin
    Items.BeginUpdate;
    try
      {     ,   - 
         ,   
        , ..    
          }

      {       , 
              }
      CascadeChecks := False;
      SetRootState(Not SetChecked);
      CascadeChecks := True;
      SetRootState(SetChecked);
    finally
      Items.EndUpdate;
    end;
  end;
end;

procedure TfmAttributeSelectionDialog.GetSelection(Selection: TStrings);
begin
  Selection.Clear;
  if TreeView.GetNodeState(TreeView.Items.GetFirstNode) <> csUnChecked then
    GetSelection(TreeView.Items.GetFirstNode, Selection, '');
end;

{{
         ParentNode.
}
procedure TfmAttributeSelectionDialog.GetSelection(ParentNode: TTreeNode;
  Selection: TStrings; Path: String);
var
  ChildNode: TTreeNode;
begin
  ChildNode := ParentNode.GetFirstChild;
  with TreeView do
  while Assigned(ChildNode) do
  begin
    if GetNodeState(ChildNode) <> csUnChecked then
    begin
      if TreeViewBuilder.NodeTypes[ChildNode] = ntAttribute then
        Selection.Add(CombineSubStr(Path, '.', ChildNode.Text));
      GetSelection(ChildNode, Selection, CombineSubStr(Path, '.', ChildNode.Text));
    end;
    ChildNode := ChildNode.GetNextSibling;
  end
end;

procedure TfmAttributeSelectionDialog.SetSelection(Selection: TStrings);
var
  I: Integer;
begin
  SelectAll(False);
  for I := 0 to Selection.Count - 1 do
    SelectAttribute(Selection[I])
end;

procedure TfmAttributeSelectionDialog.SelectAttribute(AttributePath: String);
var
  CurrName: String;
  CurrParent: TTreeNode;
  ChildNode: TTreeNode;
begin
  CurrParent := TreeView.Items.GetFirstNode;
  while ParseAttribute(CurrName, AttributePath) do
  begin
    CurrParent.Expand(False);
    if Not FindNodeByText(CurrParent, CurrName, ChildNode) then Exit;
    CurrParent := ChildNode;
  end;
  CurrParent.Expand(False);
  if FindNodeByText(CurrParent, CurrName, ChildNode) then
    TreeView.ItemState[ChildNode.AbsoluteIndex] := csChecked;
end;

procedure TfmAttributeSelectionDialog.SwapItems(ItemIndex1, ItemIndex2:
  Integer);
begin
  SelectionList.Items.Move(ItemIndex1, ItemIndex2);
  SelectionList.ItemIndex := ItemIndex2;
end;

function TfmAttributeSelectionDialog.GetNodeSelectionText(Node: TTreeNode):
  String;
begin
  Result := '';
  while Assigned(Node.Parent) do
  begin
    Result := CombineSubStr(Node.Text, '.', Result);
    Node := Node.Parent;
  end
end;

end.
