unit ovComponents;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Contnrs,
  Forms, Dialogs, DB, ovTypes, ovModel, ovObjects, ovIObjects,
  ovDatabase;

type
  //       ovOperations
  TovModel = class;
  TBeforeGenerateDatabaseEvent = procedure (Sender: TovModel; Max: Integer) of object;
  TAfterGenerateDatabaseEvent = procedure(Sender: TovModel; Exception: Exception)
    of object;
  TGenerateDatabaseProgressEvent = procedure (Sender: TovModel; Step: Integer) of object;
  TModelEvent = procedure (Sender: TovModel) of object;
  TLoadModelEvent = procedure(Sender: TovModel; DomainModel: TDomainModel) of
    object;

  TOnCalcAttributeEvent = procedure (Sender: TovModel;
    AModelObject: TModelObject; AObjectId: TOID;
    AModelAttribute: TModelAttribute; var Value: Variant) of object;

  //       ovDatabase
  TDeleteObjectEvent = procedure (Sender: TovModel;
    AModelObject: TModelObject; AObjectId: TOID) of object;

  TValidateObjectEvent = procedure (Sender: TovModel;
    AModelObject: TModelObject; AObjectId: TOID) of object;

  TovCustomDatabaseAdapter = class;
  TovCustomDatabaseAdapterClass = class of TovCustomDatabaseAdapter;

  TovCustomDatabaseAdapter = class(TComponent, IDatabaseAdapter)
    function GetOnCloseDatabase: TNotifyEvent;
    procedure SetOnCloseDatabase(const Value: TNotifyEvent);
    {{
       True,      
     ,   ..,   .
    }
    function SupportsLocalisedNames: Boolean;
  private
    FOnCloseDatabase: TNotifyEvent;
    FQuerySyntaxParams: TQuerySyntaxParams;
    function GetQuerySyntaxParams: TQuerySyntaxParams;
  protected
    FSupportsBooleanType: Boolean;
    FSupportsLocalisedNames: Boolean;
    FFormatSettings: TFormatSettings;
    procedure CloneTable(SourceTable: TDataSet; SourceTableFieldNames: TStrings;
      TableName: String; FieldsDefs: TFieldDefs; IndexDefs: TIndexDefs); virtual;
      abstract;
    procedure CloseDatabaseNotification;
    function ValueToSQLString(const Value: Variant; FieldType: TFieldType;
      FieldSize: Integer): String; virtual;
    function StringValueToSQLString(const Value: String; FieldType: TFieldType;
      FieldSize: Integer): String; virtual;
    property QuerySyntaxParams: TQuerySyntaxParams read FQuerySyntaxParams write
      FQuerySyntaxParams;
  public
    constructor Create(AOwner: TComponent); override;
    procedure CreateDatabase(DatabaseName: String; ConnectionParams: TStrings);
      virtual; abstract;
    {{
            
    ConnectionParams.        
    (..  ).
    }
    procedure SetupConnection(ConnectionParams: TStrings); virtual; abstract;
    function EditConnection(ConnectionParams: TStrings): Boolean; virtual;
    class procedure RegisterAdapter(AdapterClass: TovCustomDatabaseAdapterClass);
    class procedure UnRegisterAdapter(AdapterClass: TovCustomDatabaseAdapterClass);
    class function GetAdapter(AdapterClassName: String):
      TovCustomDatabaseAdapterClass;
    class procedure GetAdaptersFromModule(AModule: HModule; AdapterClassNames:
      TStrings);
    procedure OpenDatabase; virtual; abstract;
    procedure CloseDatabase; virtual; abstract;
    function SupportsBooleanType: Boolean;

    procedure CommitTransaction; virtual; abstract;
    procedure RollbackTransaction; virtual; abstract;
    procedure StartTransaction; virtual; abstract;
    function InTransaction: Boolean; virtual; abstract;

    procedure ExecQuery(SQLText: String); virtual; abstract;
    function OpenQuery(QueryText: String; MasterDataSource: TDataSource = nil;
      LiveDataSet: Boolean = False): TDataSet; virtual; abstract;
    procedure CreateTable(ATableName: String; AFieldsDefs: TFieldDefs;
      AnIndexDefs: TIndexDefs); virtual; abstract;
    function OpenTable(TableName: String; ReadOnly: Boolean = False; Filter: String
      = ''): TDataSet; virtual; abstract;
    function TableExists(TableName: String): Boolean; virtual; abstract;

    function GetLastKeyValue(TableName, IdFieldName: String): Integer; virtual;
    {{
           , 
          .
    }
    function GetNewRecordId(ATableName: String): Integer; virtual; abstract;
    procedure SetNewRecordId(ATableName: String; AValue: Integer); virtual;
      abstract;

    property OnCloseDatabase: TNotifyEvent read GetOnCloseDatabase write
        SetOnCloseDatabase;
  end;

  TovModelClient = class(TComponent)
  private
    FModel: TovModel;
    FModelName: String;
    procedure SetModel(Value: TovModel);
    procedure SetModelName(const Value: String);
    function IsModelStored: Boolean;
    function GetDomainModel: TDomainModel;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure ModelChanged; virtual;
    procedure ModelChanging; virtual;
  public
    procedure ModelNeeded;
    constructor Create(AOwner: TComponent); override;
    property DomainModel: TDomainModel read GetDomainModel;
  published
    property Model: TovModel read FModel write SetModel stored IsModelStored;
    property ModelName: String read FModelName write SetModelName;
  end;

  //       ovModelStorage
  TovCustomModelStorage = class(TovModelClient)
  private
    FNamespace: String;
    FOnAfterOpen: TModelEvent;
    procedure SaveObjects(Model: TDomainModel; SaveAllObjects: Boolean);
    procedure SaveRelations(Model: TDomainModel);
    procedure SetNamespace(const Value: String);
    procedure SaveDataTypes(Model: TDomainModel; SaveAllDataTypes: Boolean);
  protected
    procedure WriteObject(AObject: TModelObject); virtual; abstract;
    procedure WriteAttribute(AModelObject: TModelObject; Attribute:
      TModelAttribute); virtual; abstract;
    procedure WriteIndex(AModelObject: TModelObject; Index: TModelObjectIndex);
      virtual; abstract;
    procedure WriteRelation(Relation: TModelRelation); virtual; abstract;
    procedure ReadAttribute(AObject: TModelObject; ObjectIndex, AttributeIndex:
      Integer); virtual; abstract;
    function ReadObject(Model: TDomainModel; Index: Integer): TModelObject; virtual;
      abstract;
    function GetObjectCount: Integer; virtual; abstract;
    function GetAttributeCount(Index: Integer): Integer; virtual; abstract;
    function GetRelationCount: Integer; virtual; abstract;
    procedure ReadRelation(Model: TDomainModel; Index: Integer); virtual; abstract;
    procedure ModelChanged; override;
    procedure ModelChanging; override;
    procedure LoadObjects(Model: TDomainModel); dynamic;
    procedure LoadRelations(Model: TDomainModel); dynamic;
    procedure WriteDataType(AModelDataType: TModelDataType); virtual; abstract;
    procedure LoadDataTypes(Model: TDomainModel); dynamic;
    function GetDataTypeCount: Integer; virtual; abstract;
    function ReadDataType(Model: TDomainModel; Index: Integer): TModelDataType;
      virtual; abstract;
    function GetIndexCount(Index: Integer): Integer; virtual; abstract;
    procedure ReadIndex(AObject: TModelObject; ObjectIndex, Index: Integer);
      virtual; abstract;
  public
    procedure SaveModel(Model: TDomainModel; SaveAll: Boolean = False); dynamic;
    procedure LoadModel(Model: TDomainModel); dynamic;
    destructor Destroy; override;
  published
    property Namespace: String read FNamespace write SetNamespace;
    property OnAfterOpen: TModelEvent read FOnAfterOpen write FOnAfterOpen;
  end;

  {{
        ovDBConnections
  }
  TovCustomDatabaseConnections = class(TComponent)
  protected
    function OpenDefaultConnection(AModel: TovModel): TovCustomDatabaseAdapter;
      virtual; abstract;
    procedure CloseActiveConnection; virtual; abstract;
  end;

  TovModelOption = (moAutoOpen, moAutoGenerateDatabase, moRebuildExistingTables,
    moShowGenerateDatabaseProgress, moAutoShowConnectionManager);
  TovModelOptions = set of TovModelOption;

  TovModel = class(TComponent)
  private
    FDatabase: TovDatabase;
    FDatabaseAdapter: TovCustomDatabaseAdapter;
    FDomainModel: TDomainModel;
    FModelLoaded: Boolean;
    FStorages: TComponentList;
    FClients: TComponentList;
    FEvents: TComponentList;
    FModelName: String;
    FVersion: String;
    FDatabaseEvents: TovDatabaseEvents;
    FOnAfterGenerateDatabase: TAfterGenerateDatabaseEvent;
    FOnBeforeGenerateDatabase: TBeforeGenerateDatabaseEvent;
    FOnGenerateDatabaseProgress: TGenerateDatabaseProgressEvent;
    FOnBeforeOpen: TModelEvent;
    FOnAfterOpen: TModelEvent;
    FDatabaseConnection: TovCustomDatabaseConnections;
    FOptions: TovModelOptions;
    FActive: Boolean;
    FAutoOpenedDataSets: TComponentList;
    FOnLoadModel: TLoadModelEvent;
    FOnConnectToDatabase: TNotifyEvent;
    FOnAfterClose: TModelEvent;
    FOnBeforeClose: TModelEvent;
    procedure SetDatabaseAdapter(Value: TovCustomDatabaseAdapter);
    procedure SetModelName(const Value: String);
    function GetDomainModel: TDomainModel;
    function GetStorages(Index: Integer): TovCustomModelStorage;
    function GetStorageCount: Integer;
    function GetAutoGenerateNames: Boolean;
    procedure SetAutoGenerateNames(Value: Boolean);
    procedure LinkClients;
    procedure SetVersion(const Value: String);
    function GetObjectSpace: IovObjectSpace;
    procedure DoAfterGenerateDatabase(Exception: Exception);
    procedure DoBeforeGenerateDatabase(Max: Integer);
    procedure DoCloseDatabase(Sender: TObject);
    procedure DoGenerateDatabaseProgress(Step: Integer);
    procedure DoCreateObject(AModelObject: TModelObject; AObjectId: TOID);
    procedure DoDeleteObject(AModelObject: TModelObject; AObjectId: TOID);
    procedure DoDeleteObjectError(AModelObject: TModelObject; AObjectId: TOID);
    procedure DoValidateObject(AModelObject: TModelObject; AObjectId: TOID);
    procedure DoCalculateAttriute(AObjectId: TOID; const AModelAttribute:
      TModelAttribute; var Value: Variant);
    procedure SetDatabaseConnection(Value: TovCustomDatabaseConnections);
    procedure CloseActiveDataSets;
    procedure ConnectToDatabase;
    procedure DisconnectFromDatabase;
    procedure SetActive(const Value: Boolean);
    procedure DoAfterOpen;
    procedure ModelStoragesChanged;
    procedure AddClient(AModelClient: TovModelClient);
    procedure RemoveClient(AModelClient: TovModelClient);
    procedure ModelClientsChanged;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation);
      override;
    procedure Loaded; override;
    procedure AutoOpenDataSets;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Close;
    procedure NewModel;
    {{
       ,    .  
    RebuildExistingTables = False,      ,
       .
    }
    procedure GenerateDatabase(ADatabaseAdapter: IDatabaseAdapter = nil;
      RebuildExistingTables: Boolean = True);
    procedure CopyDatabase(SourceDatabase, TargetDatabase: IDatabaseAdapter);
    procedure LoadModel;
    procedure Open;
    procedure SaveModel;
    procedure AddStorage(AModelStorage: TovCustomModelStorage);
    procedure RemoveStorage(AModelStorage: TovCustomModelStorage);
    procedure UnloadModel;
    procedure RegisterClass(AClassName: String; AClass: TovObjectClass);
    procedure UnRegisterClass(AClassName: String; AClass: TovObjectClass);
    function ShowConnectionManager: Boolean;
    procedure CopyDatabaseTo(ATargetModel: TovModel; ASourceAdapter,
      ATargetAdapter: TovCustomDatabaseAdapter);
    procedure LoadModelTo(AModel: TovModel);
    function OpenSQLQuery(Query: String; MasterDataSource: TDataSource = nil;
      LiveDataSet: Boolean = False): TDataSet;
    procedure ExecSQLQuery(AQuery: String);
    property DomainModel: TDomainModel read GetDomainModel;
    property Database: TovDatabase read FDatabase;
    property Storages[Index: Integer]: TovCustomModelStorage read GetStorages;
    property StorageCount: Integer read GetStorageCount;
    property ObjectSpace: IovObjectSpace read GetObjectSpace;
  published
    property DatabaseAdapter: TovCustomDatabaseAdapter read FDatabaseAdapter
      write SetDatabaseAdapter;
    property ModelName: String read FModelName write SetModelName;
    property AutoGenerateNames: Boolean read GetAutoGenerateNames write
        SetAutoGenerateNames default False;
    property Version: String read FVersion write SetVersion stored False;
    property DatabaseConnection: TovCustomDatabaseConnections read
      FDatabaseConnection write SetDatabaseConnection;
    property Options: TovModelOptions read FOptions write FOptions
      default [moShowGenerateDatabaseProgress];
    property Active: Boolean read FActive write SetActive default False;
    property OnAfterGenerateDatabase: TAfterGenerateDatabaseEvent read
      FOnAfterGenerateDatabase write FOnAfterGenerateDatabase;
    property OnBeforeGenerateDatabase: TBeforeGenerateDatabaseEvent read
      FOnBeforeGenerateDatabase write FOnBeforeGenerateDatabase;
    property OnGenerateDatabaseProgress: TGenerateDatabaseProgressEvent read
      FOnGenerateDatabaseProgress write FOnGenerateDatabaseProgress;
    property OnBeforeOpen: TModelEvent read FOnBeforeOpen write FOnBeforeOpen;
    property OnAfterOpen: TModelEvent read FOnAfterOpen write FOnAfterOpen;
    property OnAfterClose: TModelEvent read FOnAfterClose write FOnAfterClose;
    property OnBeforeClose: TModelEvent read FOnBeforeClose write FOnBeforeClose;
    property OnConnectToDatabase: TNotifyEvent read FOnConnectToDatabase write
      FOnConnectToDatabase;
    property OnLoadModel: TLoadModelEvent read FOnLoadModel write FOnLoadModel;
  end;

procedure RegisterModel(ModelComponent: TovModel);
procedure UnregisterModel(ModelComponent: TovModel);
procedure GetRegisteredModels(ModelNames: TStrings);
function FindModel(ModelName: String): TovModel;
function GetModel(ModelName: String): TovModel;

{{
   .   -    , 
      SetDefaultModel.
}
function DefaultModel: TovModel;

{{
    .    AModel 
 nil,       .
}
procedure SetDefaultModel(AModel: TovModel);

function ObjectSpace: IovObjectSpace; overload;
function ObjectSpace(ModelName: String): IovObjectSpace; overload;

type
  TShowConnectionManagerProc = function (Model: TovModel;
    DBConnections: TovCustomDatabaseConnections): Boolean;
var
  ShowConnectionManagerProc: TShowConnectionManagerProc = nil;

implementation

uses ovConsts, ovDataSet,
     ovObjectDeleteError, ovProgress, ovModelEvents,
     Variants;            //      ,
                          //       uses,
                          //    initializition

var RegisteredModels: TStrings;

procedure RegisterModel(ModelComponent: TovModel);
begin
  if RegisteredModels.IndexOf(UpperCase(ModelComponent.ModelName)) < 0 then
    RegisteredModels.AddObject(UpperCase(ModelComponent.ModelName), ModelComponent)
end;

procedure UnregisterModel(ModelComponent: TovModel);
var
  I: Integer;
begin
  if Assigned(RegisteredModels) then begin //      
    I := RegisteredModels.IndexOfObject(ModelComponent);
    if I >= 0 then RegisteredModels.Delete(I)
  end
end;

function FindModel(ModelName: String): TovModel;
var
  I: Integer;
begin
  I := RegisteredModels.IndexOf(UpperCase(ModelName));
  if I >= 0 then Result := TovModel(RegisteredModels.Objects[I])
  else Result := nil;
end;

function GetModel(ModelName: String): TovModel;
begin
  Result := FindModel(ModelName);
  if Result = nil then raise EModelNotFound.CreateFmt(' %s  ', [ModelName])
end;

procedure GetRegisteredModels(ModelNames: TStrings);
begin
  ModelNames.Assign(RegisteredModels);
end;

var
  _DefaultModel: TovModel = nil;

function DefaultModel: TovModel;
begin
  Result := _DefaultModel;
end;

procedure SetDefaultModel(AModel: TovModel);
begin
  _DefaultModel := AModel;
end;

function ObjectSpace: IovObjectSpace;
begin
  if Assigned(_DefaultModel) then
    Result := _DefaultModel.ObjectSpace
  else
    raise Exception.Create('    ');
end;

function ObjectSpace(ModelName: String): IovObjectSpace;
begin
  Result := GetModel(ModelName).ObjectSpace
end;

{
*********************************** TovModel ***********************************
}
var
  ModelClients: TComponentList;

{<<Category>>: No category}
constructor TovModel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FVersion := '1.72';
  FOptions := [moShowGenerateDatabaseProgress];
  FAutoOpenedDataSets := TComponentList.Create(False);
  FStorages := TComponentList.Create(False);
  FClients := TComponentList.Create(False);
  FEvents := TComponentList.Create(False);
  FDomainModel := TDomainModel.Create;

  FDatabaseEvents.OnBeforeGenerateDatabase := DoBeforeGenerateDatabase;
  FDatabaseEvents.OnAfterGenerateDatabase := DoAfterGenerateDatabase;
  FDatabaseEvents.OnGenerateDatabaseProgress := DoGenerateDatabaseProgress;
  FDatabaseEvents.OnCalcAttribute := DoCalculateAttriute;
  FDatabaseEvents.OnCreateObject := DoCreateObject;
  FDatabaseEvents.OnDeleteObject := DoDeleteObject;
  FDatabaseEvents.OnDeleteObjectError := DoDeleteObjectError;
  FDatabaseEvents.OnValidateObject := DoValidateObject;

  FDatabase := TovDatabase.Create(FDomainModel, FDatabaseEvents);
  IUnknown(FDatabase)._AddRef;

  if Not Assigned(_DefaultModel) then _DefaultModel := Self;

end;

destructor TovModel.Destroy;
begin
  Close;
  UnregisterModel(Self);

  if _DefaultModel = Self then _DefaultModel := nil;

  IUnknown(FDatabase)._Release;
  FDatabase := nil;

  FreeAndNil(FDomainModel);
  FreeAndNil(FEvents);
  FreeAndNil(FClients);
  FreeAndNil(FStorages);
  FreeAndNil(FAutoOpenedDataSets);

  inherited Destroy;
end;

procedure TovModel.Loaded;
begin
  inherited;
  LinkClients;
  if (moAutoOpen in Options) and Not (csDesigning in ComponentState) then Open;
end;

procedure TovModel.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if AComponent = DatabaseAdapter then DatabaseAdapter := nil else
    if AComponent = DatabaseConnection then DatabaseConnection := nil
end;

{ ***************************   ***************************** }

procedure TovModel.SetVersion(const Value: String);
begin
end;

function TovModel.GetObjectSpace: IovObjectSpace;
begin
  Result := Database;
end;

procedure TovModel.SetDatabaseAdapter(Value: TovCustomDatabaseAdapter);
begin
  if FDatabaseAdapter <> Value then
  begin
    Close;
    FDatabaseAdapter := Value;
    if Value <> nil then Value.FreeNotification(Self);
    // DatabaseAdapter  DatabaseConnection     
    FDatabaseConnection := nil;
  end;
end;

procedure TovModel.SetDatabaseConnection(Value: TovCustomDatabaseConnections);
begin
  if FDatabaseConnection <> Value then
  begin
    Close;
    FDatabaseConnection := Value;
    if Value <> nil then Value.FreeNotification(Self);
    // DatabaseAdapter  DatabaseConnection     
    FDatabaseAdapter := nil;
  end;
end;

procedure TovModel.SetModelName(const Value: String);
begin
  if Value <> ModelName then begin
    UnregisterModel(Self);
    FModelName := Value;
    if ModelName <> '' then RegisterModel(Self);
  end
end;

function TovModel.GetDomainModel: TDomainModel;
begin
//  if Not FModelLoaded then LoadModel;
  Result := FDomainModel;
end;

function TovModel.GetAutoGenerateNames: Boolean;
begin
  Result := FDomainModel.AutoGenerateNames;
end;

procedure TovModel.SetAutoGenerateNames(Value: Boolean);
begin
  FDomainModel.AutoGenerateNames := Value;
end;

{***************************************************************************** }
{                                                              }
{ **************************************************************************** }

{ TODO :     OnDeleteObjectError }
procedure TovModel.DoDeleteObjectError(AModelObject: TModelObject; AObjectId:
  TOID);
begin
  ShowDeleteObjectErrorHandler(Self, AModelObject, AObjectId);
end;

{{
       
}
procedure TovModel.DoCloseDatabase(Sender: TObject);
begin
  Close
end;

{{
      TovModelEvents,  
 .
}
procedure TovModel.DoCreateObject(AModelObject: TModelObject; AObjectId: TOID);
var
  I: Integer;
begin
  for I := 0 to FEvents.Count - 1 do
    TovModelEvents(FEvents[I]).HandleCreateObjectEvent(AModelObject, AObjectId);
end;

{{
      TovModelEvents,  
 .
}
procedure TovModel.DoDeleteObject(AModelObject: TModelObject; AObjectId: TOID);
var
  I: Integer;
begin
  for I := 0 to FEvents.Count - 1 do
    TovModelEvents(FEvents[I]).HandleDeleteObjectEvent(AModelObject, AObjectId);
end;

{{
      TovModelEvents,  
 .
}
procedure TovModel.DoValidateObject(AModelObject: TModelObject; AObjectId: TOID);
var
  I: Integer;
begin
  for I := 0 to FEvents.Count - 1 do
    TovModelEvents(FEvents[I]).HandleValidateObjectEvent(AModelObject, AObjectId);
end;

{{
      TovModelEvents,  
 .
}
procedure TovModel.DoCalculateAttriute(AObjectId: TOID; const AModelAttribute:
  TModelAttribute; var Value: Variant);
var
  I: Integer;
begin
  for I := 0 to FEvents.Count - 1 do
    TovModelEvents(FEvents[I]).HandleCalculateAttributeEvent(
      AModelAttribute.Owner, AObjectId, AModelAttribute, Value);
end;

procedure TovModel.DoAfterOpen;
var
  I: Integer;
begin
  if Assigned(FOnAfterOpen) then FOnAfterOpen(Self);
  for I := 0 to StorageCount - 1 do
    with Storages[I] do
      if Assigned(FOnAfterOpen) then FOnAfterOpen(Self);
end;

{***************************************************************************** }
{                                                                 }
{ **************************************************************************** }

procedure TovModel.AddClient(AModelClient: TovModelClient);
begin
  if FClients.IndexOf(AModelClient) < 0 then
  begin
    FClients.Add(AModelClient);
    ModelClientsChanged;
  end;
end;

procedure TovModel.RemoveClient(AModelClient: TovModelClient);
begin
  if FClients.IndexOf(AModelClient) >= 0 then
  begin
    FClients.Extract(AModelClient);
    ModelClientsChanged;
  end;
end;

procedure TovModel.ModelClientsChanged;
var
  I: Integer;
begin
  FEvents.Clear;
  for I := 0 to FClients.Count - 1 do
    if FClients[I] is TovModelEvents then FEvents.Add(FClients[I])
end;

{{
  Model,    TModelClient,  
    .
}
procedure TovModel.LinkClients;
var
  I: Integer;
begin
  if ModelName <> '' then
  for I := 0 to ModelClients.Count - 1 do
    if TovModelClient(ModelClients[I]).ModelName = ModelName
      then TovModelClient(ModelClients[I]).Model := Self;
end;

{***************************************************************************** }
{                           /                          }
{ **************************************************************************** }

procedure TovModel.LoadModel;
begin
  if Not FModelLoaded then LoadModelTo(Self);
end;

{{
        ADomainModel.
    OnLoadModel   DomainModel 
ADomainModel.
}
procedure TovModel.LoadModelTo(AModel: TovModel);
var
  I: Integer;
  ADomainModel: TDomainModel;
begin
  ADomainModel := AModel.DomainModel;
  ADomainModel.Clear;

  //   
  for I := 0 to StorageCount - 1 do
    ADomainModel.NameSpaces.Add(Storages[I].NameSpace);

  //   
  for I := 0 to StorageCount - 1 do Storages[I].LoadDataTypes(ADomainModel);

  //      
  for I := 0 to StorageCount - 1 do Storages[I].LoadObjects(ADomainModel);

  //       (   
  //   )
  for I := 0 to StorageCount - 1 do Storages[I].LoadRelations(ADomainModel);

  if Assigned(FOnLoadModel) then FOnLoadModel(Self, ADomainModel);

  AModel.FModelLoaded := True;
end;

procedure TovModel.UnloadModel;
begin
  FDomainModel.Clear;
  FModelLoaded := False;
end;

procedure TovModel.SaveModel;
var
  I: Integer;
begin
  for I := 0 to StorageCount - 1 do Storages[I].SaveModel(FDomainModel)
end;

procedure TovModel.NewModel;
begin
  DomainModel.Clear;
  FModelLoaded := False;
end;

{***************************************************************************** }
{                /                 }
{ **************************************************************************** }

procedure TovModel.SetActive(const Value: Boolean);
begin
  if FActive <> Value then
    if Value then Open else Close
end;

procedure TovModel.Open;
begin
  if Not FActive then
  begin
    if Assigned(FOnBeforeOpen) then FOnBeforeOpen(Self);
    if (moAutoShowConnectionManager in Options) and Not ShowConnectionManager then
      raise Exception.Create('      ');

    LoadModel;
    ConnectToDatabase;
    FActive := True;

    if moAutoGenerateDatabase in Options then
      GenerateDatabase(nil, moRebuildExistingTables in Options);
    DoAfterOpen;
    AutoOpenDataSets;
  end
end;

procedure TovModel.Close;
begin
  if FActive then
  begin
    if Assigned(FOnBeforeClose) then FOnBeforeClose(Self);
    CloseActiveDataSets;
    DisconnectFromDatabase;
    if Assigned(FOnAfterClose) then FOnAfterClose(Self);
    FActive := False;
  end;
  {     DomainModel   ,       }
  UnloadModel;
end;

procedure TovModel.ConnectToDatabase;
begin
  if Database.Adapter = nil then //       GenerateDatabase
  begin
    if DatabaseConnection <> nil then
      FDatabaseAdapter := DatabaseConnection.OpenDefaultConnection(Self);
    Database.Adapter := DatabaseAdapter;

    if Not Assigned(Database.Adapter) then
      raise Exception.Create('      ' + Name);

    Database.Open;
    Database.Adapter.OnCloseDatabase := DoCloseDatabase;

    if Assigned(FOnConnectToDatabase) then FOnConnectToDatabase(Self);
  end;
end;

procedure TovModel.DisconnectFromDatabase;
begin
  if Database.Adapter <> nil then
  begin
    Database.Adapter.OnCloseDatabase := nil;
    Database.Close;
    if DatabaseConnection <> nil then
     begin
       DatabaseConnection.CloseActiveConnection;
       FDatabaseAdapter := nil;
     end;
    Database.Adapter := nil;
  end;
end;

{{
      .
}
procedure TovModel.CloseActiveDataSets;
var
  I: Integer;
begin
  for I := 0 to TovDataSet.DataSetCount - 1 do
    with TovDataSet.GetDataSet(I) do
      if (Model = Self) and Active then
      begin
        Close;
        FAutoOpenedDataSets.Add(TovDataSet.GetDataSet(I));
      end;
end;

{{
   ,      
 .
}
procedure TovModel.AutoOpenDataSets;
var
  I: Integer;
begin
  for I := 0 to FAutoOpenedDataSets.Count - 1 do
    TovDataSet(FAutoOpenedDataSets[I]).Open;
  FAutoOpenedDataSets.Clear;
end;

{***************************************************************************** }
{                                                  }
{ **************************************************************************** }

{{
   ,      Database.
  Database = nil,       
.   RebuildExistingTables = False,   
 ,      .
}
procedure TovModel.GenerateDatabase(ADatabaseAdapter: IDatabaseAdapter = nil;
  RebuildExistingTables: Boolean = True);
var
  WasOpened: Boolean;
begin
  LoadModel;
  if ADatabaseAdapter <> nil then
    Database.GenerateDatabase(ADatabaseAdapter, RebuildExistingTables)
  else
  begin
    WasOpened := Database.Adapter <> nil;
    ConnectToDatabase;
    try
      Database.GenerateDatabase(DatabaseAdapter, RebuildExistingTables)
    finally
      if Not WasOpened then DisconnectFromDatabase;
    end;
  end;
end;

procedure TovModel.DoBeforeGenerateDatabase(Max: Integer);
begin
  if Assigned(FOnBeforeGenerateDatabase) then FOnBeforeGenerateDatabase(Self, Max);

  if moShowGenerateDatabaseProgress in Options then
  begin
    ShowProgress(Max, ' ');
  end
end;

procedure TovModel.DoGenerateDatabaseProgress(Step: Integer);
begin
  if moShowGenerateDatabaseProgress in Options then
  begin
    if Not ProgressStep then Abort;
  end;

  if Assigned(FOnGenerateDatabaseProgress) then FOnGenerateDatabaseProgress(Self, Step);
end;

procedure TovModel.DoAfterGenerateDatabase(Exception: Exception);
begin
  if moShowGenerateDatabaseProgress in Options then
  begin
    HideProgress;
    if Exception is EAbort then
      Application.MessageBox('  ', '', MB_ICONWARNING or MB_OK);
  end;

  if Assigned(FOnAfterGenerateDatabase) then FOnAfterGenerateDatabase(Self, Exception);
end;

{***************************************************************************** }
{                                                                  }
{ **************************************************************************** }

{{
       
SourceDatabase  TargetDatabase.
}
procedure TovModel.CopyDatabase(SourceDatabase, TargetDatabase:
  IDatabaseAdapter);
begin
  LoadModel;
  Database.CopyDatabase(SourceDatabase, DomainModel, TargetDatabase, DomainModel);
end;

{{
           
   ATargetModel.       
    (!), ..       .
}
procedure TovModel.CopyDatabaseTo(ATargetModel: TovModel; ASourceAdapter,
  ATargetAdapter: TovCustomDatabaseAdapter);
begin
  Database.CopyDatabase(ASourceAdapter, DomainModel, ATargetAdapter, ATargetModel.DomainModel);
end;

procedure TovModel.RegisterClass(AClassName: String; AClass: TovObjectClass);
begin
  LoadModel;
  FDatabase.RegisterClassImplementor(AClass, AClassName);
end;

procedure TovModel.UnRegisterClass(AClassName: String; AClass: TovObjectClass);
begin
  FDatabase.UnregisterClassImplementor(AClass, AClassName);
end;

function TovModel.ShowConnectionManager: Boolean;
begin
  LoadModel;

  if Not Assigned(DatabaseConnection) then
    raise Exception.Create('   DatabaseConnection  ' + Name);

  if Not Assigned(ShowConnectionManagerProc) then
    raise Exception.Create('   ');

  Result := ShowConnectionManagerProc(Self, DatabaseConnection)
end;

{{
   SQL- (    )
}
function TovModel.OpenSQLQuery(Query: String; MasterDataSource: TDataSource =
  nil; LiveDataSet: Boolean = False): TDataSet;
begin
  Result := Database.OpenSQLQuery(Query, MasterDataSource, LiveDataSet);
end;

{{
    SQL- (    )
}
procedure TovModel.ExecSQLQuery(AQuery: String);
begin
  Database.ExecSQLQuery(AQuery);
end;

{***************************************************************************** }
{                                                       }
{ **************************************************************************** }

function TovModel.GetStorageCount: Integer;
begin
  Result := FStorages.Count;
end;

function TovModel.GetStorages(Index: Integer): TovCustomModelStorage;
begin
  Result := TovCustomModelStorage(FStorages[Index]);
end;

procedure TovModel.AddStorage(AModelStorage: TovCustomModelStorage);
begin
  if FStorages.IndexOf(AModelStorage) < 0 then
  begin
    FStorages.Add(AModelStorage);
    ModelStoragesChanged;
  end;
end;

procedure TovModel.RemoveStorage(AModelStorage: TovCustomModelStorage);
begin
  if FStorages.IndexOf(AModelStorage) >= 0 then
  begin
    FStorages.Extract(AModelStorage);
    ModelStoragesChanged;
  end;
end;

procedure TovModel.ModelStoragesChanged;
var
  I: Integer;
begin
  if Assigned(FDomainModel) then
  begin
    FDomainModel.NameSpaces.Clear;
    for I := 0 to StorageCount - 1 do
    begin
      FDomainModel.NameSpaces.Add(Storages[I].NameSpace);
    end;
  end
end;

{
*****************************   ****************************
}

type
  TRegisteredAdapter = class
    AdapterClass: TovCustomDatabaseAdapterClass;
    Module: HModule;
  end;

  TRegisteredAdapters = class(TObjectList)
  public
    constructor Create;
    procedure AddAdapter(AdapterClass: TovCustomDatabaseAdapterClass);
    function FindAdapter(AdapterClassName: String): Integer;
  end;

var
  RegisteredAdapters: TRegisteredAdapters;

constructor TRegisteredAdapters.Create;
begin
  inherited Create(True);
end;

procedure TRegisteredAdapters.AddAdapter(AdapterClass:
  TovCustomDatabaseAdapterClass);
var
  RegisteredAdapter: TRegisteredAdapter;
begin
  RegisteredAdapter := TRegisteredAdapter.Create;
  RegisteredAdapter.AdapterClass := AdapterClass;
  RegisteredAdapter.Module := FindClassHInstance(AdapterClass);
  inherited Add(RegisteredAdapter);
end;

function TRegisteredAdapters.FindAdapter(AdapterClassName: String): Integer;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    with TRegisteredAdapter(Items[I]) do
      if AdapterClass.ClassName = AdapterClassName then
      begin
        Result := I;
        Exit;
      end;
  Result := -1;
end;

{
************************** TovCustomDatabaseAdapter ****************************
}

constructor TovCustomDatabaseAdapter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSupportsBooleanType := True;
  FSupportsLocalisedNames := True;
  GetLocaleFormatSettings(LOCALE_USER_DEFAULT, FFormatSettings);
end;

class procedure TovCustomDatabaseAdapter.RegisterAdapter(AdapterClass:
  TovCustomDatabaseAdapterClass);
begin
  if RegisteredAdapters.FindAdapter(AdapterClass.ClassName) >= 0 then
    raise Exception.CreateFmt(' %s  ', [AdapterClass.ClassName]);
  RegisteredAdapters.AddAdapter(AdapterClass);
end;

class procedure TovCustomDatabaseAdapter.UnRegisterAdapter(AdapterClass:
  TovCustomDatabaseAdapterClass);
var
  I: Integer;
begin
  I := RegisteredAdapters.FindAdapter(AdapterClass.ClassName);
  if I < 0 then
    raise Exception.CreateFmt(' %s   ', [AdapterClass.ClassName]);
  RegisteredAdapters.Delete(I);
end;

class function TovCustomDatabaseAdapter.GetAdapter(AdapterClassName: String):
  TovCustomDatabaseAdapterClass;
var
  I: Integer;
begin
  I := RegisteredAdapters.FindAdapter(AdapterClassName);
  if I < 0 then
    raise Exception.CreateFmt(' %s   ', [AdapterClassName]);
  Result := TRegisteredAdapter(RegisteredAdapters[I]).AdapterClass;
end;

{{
     ,  
   () AModule.
}
class procedure TovCustomDatabaseAdapter.GetAdaptersFromModule(AModule: HModule;
  AdapterClassNames: TStrings);
var
  I: Integer;
begin
  AdapterClassNames.Clear;
  for I := 0 to RegisteredAdapters.Count - 1 do
    with TRegisteredAdapter(RegisteredAdapters[I]) do
      if Module = AModule then AdapterClassNames.Add(AdapterClass.ClassName);
end;

function TovCustomDatabaseAdapter.GetQuerySyntaxParams: TQuerySyntaxParams;
begin
  Result := FQuerySyntaxParams
end;

function TovCustomDatabaseAdapter.SupportsBooleanType: Boolean;
begin
  Result := FSupportsBooleanType;
end;

function TovCustomDatabaseAdapter.SupportsLocalisedNames: Boolean;
begin
  Result := FSupportsLocalisedNames;
end;

function TovCustomDatabaseAdapter.ValueToSQLString(const Value: Variant;
  FieldType: TFieldType; FieldSize: Integer): String;
begin
  case VarType(Value) of
    varNull:
      Result := 'Null';
    varInt64, varByte, varWord, varSmallint, varShortInt, varInteger:
    begin
      Result := IntToStr(Value);
      if FieldType = ftString then
        Result := '''' + Result + ''''
    end;
    varOleStr, varString:
      if (FieldType = ftBoolean) and Not SupportsBooleanType
      then
        if StrToBool(Value) then Result := TrueValueReplacementStr else Result := FalseValueReplacementStr
      else
        Result := StringValueToSQLString(Value, FieldType, FieldSize);
    varDouble:
      Result := FloatToStr(Value, FFormatSettings);
    varDate:
      if FieldType = ftTime then
        Result := '''' + TimeToStr(Value)+ ''''
      else
      if FieldType = ftDate then
        Result := '''' + DateToStr(Value)+ ''''
      else
        Result := '''' + DateTimeToStr(Value)+ '''';
    varBoolean:
      if (FieldType = ftBoolean) and Not SupportsBooleanType
      then
        if Value then Result := TrueValueReplacementStr else Result := FalseValueReplacementStr
      else
        Result := BoolToStr(Value);

    else raise Exception.Create('    ValueToSQLString')
  end
end;

function TovCustomDatabaseAdapter.StringValueToSQLString(const Value: String;
  FieldType: TFieldType; FieldSize: Integer): String;
var
  S: String;
begin
  if FieldType = ftString then
    S := Copy(Value, 1, FieldSize)
    {         Nexus, .. 
             , 
        }
  else
    S := Value;
  Result := QuotedStr(S);
end;

function TovCustomDatabaseAdapter.GetLastKeyValue(TableName, IdFieldName:
  String): Integer;
var
  Query: String;
begin
  Query := Format('SELECT Max(%s%s%s) FROM %s%s%s',
    [QuerySyntaxParams.LeftFieldDelimiter, IdFieldName, QuerySyntaxParams.RightFieldDelimiter,
    QuerySyntaxParams.LeftTableDelimiter, TableName, QuerySyntaxParams.RightTableDelimiter]);
  with OpenQuery(Query, nil, False) do
  try
    Result := Fields[0].AsInteger;
  finally
    Free
  end;
end;

function TovCustomDatabaseAdapter.EditConnection(ConnectionParams: TStrings):
  Boolean;
begin
  raise Exception.Create('      : ' + ClassName)
end;

function TovCustomDatabaseAdapter.GetOnCloseDatabase: TNotifyEvent;
begin
  Result := FOnCloseDatabase;
end;

procedure TovCustomDatabaseAdapter.SetOnCloseDatabase(const Value:
    TNotifyEvent);
begin
  FOnCloseDatabase := Value;
end;

procedure TovCustomDatabaseAdapter.CloseDatabaseNotification;
begin
  if Assigned(FOnCloseDatabase) then FOnCloseDatabase(Self)
end;

{
******************************* TovModelClient *********************************
}

constructor TovModelClient.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ModelClients.Add(Self);
end;

procedure TovModelClient.SetModelName(const Value: String);
begin
  if FModelName <> Value then
  begin
    FModelName := Value;
    Model := FindModel(FModelName);
  end
end;

procedure TovModelClient.SetModel(Value: TovModel);
begin
  if FModel <> Value then
  begin
    ModelChanging;
    FModel := Value;
    if Value <> nil then begin
      Value.FreeNotification(Self);
      FModelName := Value.ModelName;
    end;
    ModelChanged;
  end
end;

function TovModelClient.IsModelStored: Boolean;
begin
  Result := ModelName = '';
end;

procedure TovModelClient.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = Model) and (Operation = opRemove) then Model := nil
end;

procedure TovModelClient.ModelNeeded;
begin
  if Not Assigned(FModel) and (ModelName <> '') then Model := FindModel(FModelName);
  if Not Assigned(FModel) then raise Exception.Create('    ' + Name);
end;

function TovModelClient.GetDomainModel: TDomainModel;
begin
  if Not Assigned(Model) then raise Exception.Create('    ' + Name);
  Result := Model.DomainModel;
end;

procedure TovModelClient.ModelChanged;
begin
  if Assigned(Model) then Model.AddClient(Self);
end;

procedure TovModelClient.ModelChanging;
begin
  if Assigned(Model) then Model.RemoveClient(Self);
end;

{
*************************** TovCustomModelStorage ******************************
}

destructor TovCustomModelStorage.Destroy;
begin
  if Assigned(Model) then Model.RemoveStorage(Self);
  inherited Destroy;
end;

procedure TovCustomModelStorage.ModelChanging;
begin
  inherited;
  if Assigned(Model) then Model.RemoveStorage(Self)
end;

procedure TovCustomModelStorage.ModelChanged;
begin
  inherited;
  if Assigned(Model) then Model.AddStorage(Self);
end;

procedure TovCustomModelStorage.SetNamespace(const Value: String);
begin
  if FNamespace <> Value then
  begin
    // ,      
    if Assigned(Model) then Model.RemoveStorage(Self);
    FNamespace := Value;
    if Assigned(Model) then Model.AddStorage(Self);
  end;
end;

{***************************************************************************** }
{                                                              }
{ **************************************************************************** }

{{
  SaveAll  True,        , 
          Namespace.
}
procedure TovCustomModelStorage.SaveModel(Model: TDomainModel; SaveAll: Boolean
  = False);
begin
  SaveDataTypes(Model, SaveAll);
  SaveObjects(Model, SaveAll);
  SaveRelations(Model);
end;

procedure TovCustomModelStorage.SaveDataTypes(Model: TDomainModel;
  SaveAllDataTypes: Boolean);
var
  I: Integer;
begin
  with Model do
    for I := 0 to DataTypeCount - 1 do
      if SaveAllDataTypes or (DataTypes[I].Namespace = Namespace) then
        WriteDataType(DataTypes[I]);
end;

procedure TovCustomModelStorage.SaveObjects(Model: TDomainModel;
  SaveAllObjects: Boolean);
var
  I, J: Integer;
begin
  with Model do
  for I := 0 to ObjectCount - 1 do
  if SaveAllObjects or (Objects[I].Namespace = Namespace) then
  //   ,      
  begin
    WriteObject(Objects[I]);
    with Objects[I] do
    begin
      for J := 0 to AttributeCount - 1 do
        if Not Attributes[J].IsInherited then
          WriteAttribute(Objects[I], Attributes[J]);
      for J := 0 to IndexCount - 1 do
        WriteIndex(Objects[I], Indexes[J]);
    end;
  end;
end;

procedure TovCustomModelStorage.SaveRelations(Model: TDomainModel);
var
  I: Integer;
begin
  with Model do
  for I := 0 to RelationCount - 1 do
    if Not Relations[I].IsInherited and
      (Relations[I].SourceObjectNameSpace = Namespace)
    then
      WriteRelation(Relations[I]);
end;

{***************************************************************************** }
{                                                                }
{ **************************************************************************** }

procedure TovCustomModelStorage.LoadModel(Model: TDomainModel);
begin
  LoadDataTypes(Model);
  LoadObjects(Model);
  LoadRelations(Model);
end;

procedure TovCustomModelStorage.LoadDataTypes(Model: TDomainModel);
var
  I: Integer;
begin
  for I := 0 to GetDataTypeCount - 1 do
    ReadDataType(Model, I);
end;

procedure TovCustomModelStorage.LoadObjects(Model: TDomainModel);
var
  I, J: Integer;
  O: TModelObject;
begin
  for I := 0 to GetObjectCount - 1 do
  begin
    O := ReadObject(Model, I);
    for J := 0 to GetAttributeCount(I) - 1 do ReadAttribute(O, I, J);
    for J := 0 to GetIndexCount(I) - 1 do ReadIndex(O, I, J);
  end
end;

procedure TovCustomModelStorage.LoadRelations(Model: TDomainModel);
var
  I: Integer;
begin
  for I := 0 to GetRelationCount - 1 do ReadRelation(Model, I)
end;

initialization
  RegisteredModels := TStringList.Create;
  RegisteredAdapters := TRegisteredAdapters.Create;
  ModelClients := TComponentList.Create;
finalization
  FreeAndNil(ModelClients);
  FreeAndNil(RegisteredAdapters);
  FreeAndNil(RegisteredModels);
end.
