unit ovConnectionManagerDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ImgList, Menus, ActnList, Contnrs,
  ovComponents, ovDBConnections, ExtCtrls, ToolWin, ovDBBackup;

type
  TfmConnectionManager = class(TForm)
    ActionList: TActionList;
    BuildAction: TAction;
    ConnectAction: TAction;
    ConnectionImages: TImageList;
    ConnectionView: TListView;
    DeleteAction: TAction;
    DisconnectAction: TAction;
    EditAction: TAction;
    RenameAction: TAction;
    NewAction: TAction;
    OpenDialog: TOpenDialog;
    ToolBar: TToolBar;
    ToolButton1: TToolButton;
    tbTargetConnection: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    BackupDatabaseAction: TAction;
    RestoreDatabaseAction: TAction;
    ToolbarImages: TImageList;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    ToolButton10: TToolButton;
    ToolButton11: TToolButton;
    ModelDatabaseBackup: TovModelDatabaseBackup;
    TargetDatabasePopupMenu: TPopupMenu;
    ss1: TMenuItem;
    LargeImages: TImageList;
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure BackupDatabaseActionExecute(Sender: TObject);
    procedure ConnectionViewEdited(Sender: TObject; Item: TListItem;
      var S: String);
    procedure ConnectActionExecute(Sender: TObject);
    procedure ConnectionViewDblClick(Sender: TObject);
    procedure DeleteActionExecute(Sender: TObject);
    procedure EditActionExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure RenameActionExecute(Sender: TObject);
    procedure NewActionExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure DisconnectActionExecute(Sender: TObject);
    procedure BuildActionExecute(Sender: TObject);
    procedure ConnectionViewCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure Connect(Connection: TovDatabaseConnection);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure RestoreDatabaseActionExecute(Sender: TObject);
    procedure TargetDatabasePopupMenuPopup(Sender: TObject);
  private
    FConnections: TovDatabaseConnections;
    FModel: TovModel;
    procedure CopyDatabaseItemClick(Sender: TObject);
    function ConfirmDlg(const Text: string): Boolean;
    procedure Disconnect;
    function GetCurrentConnection: TovDatabaseConnection;
    procedure SetCurrentConnection(const Value: TovDatabaseConnection);
    procedure CopyDatabase(SourceConnection, TargetConnection:
      TovDatabaseConnection);
  protected
    function Edit(Connection: TovDatabaseConnection): Boolean;
    procedure GenerateDatabase(TargetConnection: TovDatabaseConnection);
    procedure ShowConnections;
  public
//    property ConnectionFromPackage[FileName: String]: TovConnectionDef read
//      GetConnectionFromPackage;
    property CurrentConnection: TovDatabaseConnection read GetCurrentConnection
      write SetCurrentConnection;
    property Model: TovModel read FModel;
  end;

function ShowConnectionManager(AModel: TovModel; AConnections:
  TovCustomDatabaseConnections): Boolean;

implementation

uses ovIObjects, ovProgress;

{$R *.DFM}

var
  Connecting: Boolean = False;

function ShowConnectionManager(AModel: TovModel; AConnections:
  TovCustomDatabaseConnections): Boolean;
var
  SaveOptions: TovModelOptions;
begin
  if Connecting then Result := True
  else
  with TfmConnectionManager.Create(Application) do
  try
    FModel := AModel;
    FConnections := AConnections as TovDatabaseConnections;
    ModelDatabaseBackup.BackupInfoFileName :=
      ChangeFileExt(FConnections.ConnectionFileName, '.ovb');

    SaveOptions := FModel.Options;
    FModel.Options := FModel.Options -
      [moAutoGenerateDatabase] + [moShowGenerateDatabaseProgress];
    try
      Result := ShowModal = mrOk;
    finally
      FModel.Options := SaveOptions;
    end;

  finally
    Free
  end;
end;

procedure AddMenuItem(APopupMenu: TPopupMenu; const ACaption: String; ATag:
  Integer; OnClickHandler: TNotifyEvent);
var
  MenuItem: TMenuItem;
begin
  MenuItem := TMenuItem.Create(APopupMenu);
  MenuItem.Caption := ACaption;
  MenuItem.Tag := ATag;
  MenuItem.OnClick := OnClickHandler;
  APopupMenu.Items.Add(MenuItem);
end;

{ TfmConnectionManager }

{{
         .
}
procedure TfmConnectionManager.FormShow(Sender: TObject);
var
  Item: TListItem;
begin
  FConnections.LoadAllConnections;
  ShowConnections;

  with ConnectionView do
    if (Items.Count > 0) and not Assigned(ItemFocused) then
    begin
      Item := FindData(0, FConnections.DefaultConnection, True, False);
      if Item <> nil then
      begin
        Selected := Item;
        ItemFocused := Selected;
      end;
    end;
end;

procedure TfmConnectionManager.ShowConnections;
var
  I: Integer;
begin
  with ConnectionView.Items do
  begin
    BeginUpdate;
    try
      Clear;
      with FConnections do
      for I := 0 to ConnectionCount - 1 do
      begin
        with Add do
        begin
          if Connections[I].IsDefault then
            ImageIndex := 2
          else
            ImageIndex := 1;
          Caption := Connections[I].ConnectionName;
          Data := Connections[I];
          SubItems.Add(Connections[I].AdapterClassName);
        end;
      end;
    finally
      EndUpdate;
    end;
  end;
end;

procedure TfmConnectionManager.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  try
    FConnections.SaveAllConnections;
  except
    on E: Exception do
      MessageDlg('  : ' + E.Message, mtError, [mbOK], 0);
  end;
end;

procedure TfmConnectionManager.FormKeyPress(Sender: TObject;
  var Key: Char);
begin
  if (Key = #27) then ModalResult := mrCancel else
  if (Key = #13) and Not ConnectionView.IsEditing then ConnectionViewDblClick(nil)
end;

procedure TfmConnectionManager.ConnectionViewDblClick(Sender: TObject);
begin
  if ConnectAction.Enabled then
  begin
    Connect(CurrentConnection);
    ModalResult := mrOk;
  end;
end;

function TfmConnectionManager.ConfirmDlg(
  const Text: string): Boolean;
begin
  Result := MessageDlg(Text, mtConfirmation, [mbYes, mbNo], 0) = mrYes;
end;

procedure TfmConnectionManager.ConnectionViewCustomDrawItem(
  Sender: TCustomListView; Item: TListItem; State: TCustomDrawState;
  var DefaultDraw: Boolean);
begin
  if TovDatabaseConnection(Item.Data).IsDefault then
    with ConnectionView.Canvas.Font do Style := Style + [fsBold]
end;

procedure TfmConnectionManager.ActionListUpdate(Action: TBasicAction;
  var Handled: Boolean);
var
  ConnectionIsSelected, Connected: Boolean;
begin
  ConnectionIsSelected := Assigned(CurrentConnection);
  Connected := FModel.Active;

  NewAction.Enabled := Not ConnectionView.IsEditing;
  EditAction.Enabled := ConnectionIsSelected;
  RenameAction.Enabled := ConnectionIsSelected;
  DeleteAction.Enabled := ConnectionIsSelected and
    Not ConnectionView.IsEditing; //     Delete   
  BuildAction.Enabled := ConnectionIsSelected;
  ConnectAction.Enabled := (FModel.Active or ConnectionIsSelected) and Not ConnectionView.IsEditing;
  ConnectAction.Checked := FModel.Active;                              

  DisconnectAction.Enabled := Connected;

  BackupDatabaseAction.Enabled := ConnectionIsSelected;
  RestoreDatabaseAction.Enabled := ConnectionIsSelected;

  tbTargetConnection.Enabled := ConnectionIsSelected;
end;

function TfmConnectionManager.GetCurrentConnection: TovDatabaseConnection;
begin
  with ConnectionView do
    if Assigned(Selected) then
      Result := Selected.Data
    else
      Result := nil;
end;

procedure TfmConnectionManager.SetCurrentConnection(const Value:
  TovDatabaseConnection);
var
  Item: TListItem;
begin
  Item := ConnectionView.FindData(0, Value, True, True);
  if Assigned(Item) then
  begin
    Item.Focused := True;
    Item.Selected := True;
  end;
end;

procedure TfmConnectionManager.ConnectActionExecute(Sender: TObject);
begin
  if ConnectAction.Checked then
  begin
    Connect(CurrentConnection);
    ModalResult := mrOk;
  end else
    Disconnect
end;

procedure TfmConnectionManager.Connect(Connection: TovDatabaseConnection);
begin
  Connection.IsDefault := True;
  FConnections.SaveAllConnections;
  FModel.Close;
  Connecting := True;
  try
    FModel.Open;
  finally
    Connecting := False;
  end;
end;

procedure TfmConnectionManager.DisconnectActionExecute(Sender: TObject);
begin
  Disconnect;
end;

procedure TfmConnectionManager.Disconnect;
var
  I: Integer;
begin
  FModel.Close;

  with ConnectionView do
  for I := 0 to Items.Count - 1 do Items[I].ImageIndex := 1;
end;

{ **************************************************************************** }
{                                                }
{ **************************************************************************** }

procedure TfmConnectionManager.NewActionExecute(Sender: TObject);
var
  Connection: TovDatabaseConnection;
  Item: TListItem;
begin
  if OpenDialog.Execute then
  begin
    Connection := FConnections.AddConnectionFromPackage(
      ' ', OpenDialog.FileName);
    ShowConnections;
    Item := ConnectionView.FindData(0, Connection, True, True);
    if Assigned(Item) then Item.EditCaption;
  end;
end;

procedure TfmConnectionManager.DeleteActionExecute(
  Sender: TObject);
begin
  if Assigned(CurrentConnection) and
    ConfirmDlg(Format('  "%s"?', [CurrentConnection.ConnectionName])) then
  begin
    FConnections.DeleteConnection(CurrentConnection.ConnectionName);
    ShowConnections;
  end;
end;

procedure TfmConnectionManager.RenameActionExecute(Sender: TObject);
begin
  with ConnectionView do
    if Assigned(Selected) then Selected.EditCaption;
end;

procedure TfmConnectionManager.ConnectionViewEdited(Sender: TObject;
  Item: TListItem; var S: String);
var
  Connection: TovDatabaseConnection;
begin
  Connection := Item.Data;
  Connection.ConnectionName := S;
end;

procedure TfmConnectionManager.EditActionExecute(Sender: TObject);
begin
  if Edit(CurrentConnection) then ShowConnections;
end;

function TfmConnectionManager.Edit(Connection: TovDatabaseConnection): Boolean;
begin
  with FConnections.OpenConnection(Connection.ConnectionName, Model, nil, False) do
  try
    Result := EditConnection(Connection.ConnectionParams);
  finally
    Free
  end;
end;

{ **************************************************************************** }
{                                                          }
{ **************************************************************************** }

procedure TfmConnectionManager.BuildActionExecute(Sender: TObject);
begin
  if Application.MessageBox('   ?', '',
    MB_ICONQUESTION or MB_YESNO) = IDYES then
  begin
    Disconnect;
    GenerateDatabase(CurrentConnection);
  end
end;

{{
     TargetConnection.
}
procedure TfmConnectionManager.GenerateDatabase(TargetConnection:
  TovDatabaseConnection);
var
  TargetDatabaseAdapter: TovCustomDatabaseAdapter;
begin
  TargetDatabaseAdapter := nil;
  try
    TargetDatabaseAdapter := FConnections.OpenConnection(TargetConnection.ConnectionName, Model);
    FModel.GenerateDatabase(TargetDatabaseAdapter);
  finally
    TargetDatabaseAdapter.Free;
  end;
end;

procedure TfmConnectionManager.TargetDatabasePopupMenuPopup(Sender: TObject);
var
  I: Integer;
begin
  TargetDatabasePopupMenu.Items.Clear;
  //      ,     
  with ConnectionView do
  for I := 0 to Items.Count - 1 do
  if Items[I].Caption <> CurrentConnection.ConnectionName
  then
    AddMenuItem(TargetDatabasePopupMenu, Items[I].Caption,
      Integer(Items[I].Data), CopyDatabaseItemClick);
end;

procedure TfmConnectionManager.CopyDatabaseItemClick(Sender: TObject);
var
  TargetConnection: TovDatabaseConnection;
begin
  TargetConnection := TObject((Sender as TMenuItem).Tag) as TovDatabaseConnection;
  if Application.MessageBox(PChar(
    Format('   "%s"  "%s" ?',
    [CurrentConnection.ConnectionName, TargetConnection.ConnectionName])),
    '', MB_ICONQUESTION or MB_YESNO) = IDYES then
  begin
    Disconnect;
    CopyDatabase(CurrentConnection, TargetConnection);
  end
end;

{{
     SourceConnection   
TargetConnection.
}
procedure TfmConnectionManager.CopyDatabase(SourceConnection, TargetConnection:
  TovDatabaseConnection);
var
  SourceModel, TargetModel: TovModel;
  SourceDatabaseAdapter: TovCustomDatabaseAdapter;
  TargetDatabaseAdapter: TovCustomDatabaseAdapter;
begin
  SourceModel := TovModel.Create(Self);
  TargetModel := TovModel.Create(Self);
  try
    FModel.LoadModelTo(SourceModel);

    SourceDatabaseAdapter := nil;
    TargetDatabaseAdapter := nil;

    try
      //       ,  
      //   OnOpenConnection,    
      //          
      // 
      SourceDatabaseAdapter := FConnections.OpenConnection(
        SourceConnection.ConnectionName, SourceModel);
  
      FModel.LoadModelTo(TargetModel);
      //       ,  
      //   OnOpenConnection,    
      //          
      // 
      TargetDatabaseAdapter := FConnections.OpenConnection(
        TargetConnection.ConnectionName, TargetModel);
  
      SourceModel.CopyDatabaseTo(TargetModel, SourceDatabaseAdapter, TargetDatabaseAdapter);

    finally
      TargetDatabaseAdapter.Free;
      SourceDatabaseAdapter.Free;
    end;

  finally
    TargetModel.Free;
    SourceModel.Free;
  end;
end;

{***************************************************************************** }
{                         /                           }
{ **************************************************************************** }

procedure TfmConnectionManager.BackupDatabaseActionExecute(Sender: TObject);
begin
  Disconnect;
  with ModelDatabaseBackup do
  begin
    Model := Self.Model;
    ShowBackupDialog(CurrentConnection.ConnectionName);
  end
end;

procedure TfmConnectionManager.RestoreDatabaseActionExecute(Sender: TObject);
begin
  Disconnect;
  with ModelDatabaseBackup do
  begin
    Model := Self.Model;
    ShowRestoreDialog(CurrentConnection.ConnectionName);
  end
end;

initialization
  //     
  ShowConnectionManagerProc := ShowConnectionManager;
end.
