unit ovDBControlLinks;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBGrids, ovDataSet, ovDataLinks;

type
  TCustomDBControlDataLinkController = class(TovDataLinkController)
  protected
    procedure SetActive(Value: Boolean); override;
  end;

  TDBControlDataLinkController = class(TCustomDBControlDataLinkController)
  private
    FLookupAttributes: String;
    FLookupDataObject: String;
  protected
    procedure SetActive(Value: Boolean); override;
  published
    property LookupAttributes: String read FLookupAttributes write
      FLookupAttributes;
    property LookupDataObject: String read FLookupDataObject write
      FLookupDataObject;
  end;

  TDBGridDataLinkController = class(TCustomDBControlDataLinkController)
  protected
    procedure SetActive(Value: Boolean); override;
  end;

  TColumnDataLinkController = class(TovDataLinkController)
  private
    FColumn: TColumn;
    FColumnIndex: Integer;
    procedure ReadColumn(Reader: TReader);
    procedure WriteColumn(Writer: TWriter);
    procedure SetColumn(Value: TColumn);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure SetActive(Value: Boolean); override;
    procedure Loaded; override;
  public
    constructor Create;
    property Column: TColumn read FColumn write SetColumn;
  end;

implementation

uses TypInfo;

procedure SetStringProp(Target: TComponent; PropName, PropValue: String);
var
  PropInfo: PPropInfo;
begin
  PropInfo := GetPropInfo(Target, PropName);
  if (PropInfo <> nil) and (PropInfo.PropType^.Kind = tkLString) then
    SetStrProp(Target, PropName, PropValue);
end;

procedure SetDataSourceProp(Target: TComponent; PropName: String; PropValue:
  TDataSource);
var
  PropInfo: PPropInfo;
begin
  PropInfo := GetPropInfo(Target, PropName);
  if (PropInfo <> nil) and (PropInfo.PropType^.Name = 'TDataSource') then
    SetObjectProp(Target, PropName, PropValue);
end;

{
********************** TCustomDBControlDataLinkController **********************
}

procedure TCustomDBControlDataLinkController.SetActive(Value: Boolean);
var
  DataSource: TDataSource;
begin
  with Link do
  begin
    if Value and Not Assigned(Component) then
      raise Exception.Create('     ');

    if Assigned(Component) then
      if Not Value then
        //   DataSource (  )
        SetDataSourceProp(Component, 'DataSource', nil)
      else begin

        if DataObject = '' then
          raise Exception.Create('   DataObject  ');

        DataSource := Owner.DataObjects.DataSources[DataObject];

        //    DataField (  )
        SetStringProp(Component, 'DataField', '');

        //   DataSource (  )
        SetDataSourceProp(Component, 'DataSource', DataSource);

        //   DataField
        if Attribute <> '' then
          SetStringProp(Component, 'DataField', Attribute);
      end
  end
end;

{
************************** TColumnDataLinkController ***************************
}

constructor TColumnDataLinkController.Create;
begin
  inherited;
  FColumnIndex := -1;
end;

procedure TColumnDataLinkController.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('Column', ReadColumn, WriteColumn,
    (FColumnIndex >= 0) or Assigned(Column));
end;

procedure TColumnDataLinkController.WriteColumn(Writer: TWriter);
begin
  if FColumnIndex >= 0 then
    Writer.WriteInteger(FColumnIndex)
  else
    Writer.WriteInteger(Column.Index)
end;

procedure TColumnDataLinkController.ReadColumn(Reader: TReader);
begin
  FColumnIndex := Reader.ReadInteger
end;

procedure TColumnDataLinkController.Loaded;
begin
  inherited;
  with Link do
    if (Component is TDBGrid) and (FColumnIndex >= 0) and
       (FColumnIndex < TDBGrid(Component).Columns.Count)
      then FColumn := TDBGrid(Component).Columns[FColumnIndex];
end;

procedure TColumnDataLinkController.SetActive(Value: Boolean);
begin
  with Link do
  begin
    if Value and Not Assigned(Component) then
      raise Exception.Create('     ');

    if Value and Assigned(Column) then Column.FieldName := Attribute;
  end
end;

procedure TColumnDataLinkController.SetColumn(Value: TColumn);
begin
  FColumn := Value;
  FColumnIndex := -1;
end;

{
************************ TDBGridDataLinkController *****************************
}

{{
     .  ,  
 -,    .
}
procedure TDBGridDataLinkController.SetActive(Value: Boolean);
var
  I: Integer;
begin
  inherited;
  if Value then
  with Link.Owner do
    for I := 0 to Links.Count - 1 do
      if (Links[I].Controller is TColumnDataLinkController) and
         (Links[I].Component = Link.Component)
      then
        Links[I].Active := Value
end;

{
********************** TDBControlDataLinkController ****************************
}

procedure TDBControlDataLinkController.SetActive(Value: Boolean);
var
  LookupSource: TDataSource;
  LookupDataSet: TovDataSet;
begin
  inherited;
  with Link do
  if (LookupDataObject <> '') and Assigned(Component) then
  begin
    //    ListSource
    SetDataSourceProp(Component, 'ListSource', nil);
    LookupSource := Owner.DataObjects.DataSources[LookupDataObject];
    LookupDataSet := Owner.DataObjects.DataSets[LookupDataObject];
    //   KeyField
    SetStringProp(Component, 'KeyField', LookupDataSet.IdFieldName);
    //   ListField
    SetStringProp(Component, 'ListField', LookupAttributes);
    //   ListSource
    SetDataSourceProp(Component, 'ListSource', LookupSource);
  end
end;

initialization
  RegisterDataLinkController(TCustomDBControlDataLinkController);
  RegisterDataLinkController(TDBControlDataLinkController);
  RegisterDataLinkController(TDBGridDataLinkController);
  RegisterDataLinkController(TColumnDataLinkController);
finalization
  UnregisterDataLinkController(TCustomDBControlDataLinkController);
  UnregisterDataLinkController(TDBControlDataLinkController);
  UnregisterDataLinkController(TDBGridDataLinkController);
  UnregisterDataLinkController(TColumnDataLinkController);
end.
