unit ovDBSynchronization2;

interface

uses Classes, DB, ovTypes, ovXMLObjectStorage, ovIObjects;

type
  {{
         .
      TovObjectStorage.  
   :
        -      XML-
        -   XML-  
             ( DatabaseId)
        -        
                  ;
           :
              ,     ,
              ,     
        -   XML-    , 
              ,     
              (..     
            )    
  }
  TovDBSynchronization = class(TovXMLObjectStorage)
  private
    FDatabaseId: String;
    FLookupTableName: String;
    FAutoGenerateLookupTable: Boolean;
    procedure CheckDatabaseId(ReadMode: Boolean);
    procedure CheckGenerateLookupTable;
  protected
    function GetLocalObjectId(ADatabaseId, AObjectClassName: String;
      AExternalObjectId: TOID): TOID;
    procedure UpdateExternalObjectId(ADatabaseId, AObjectClassName: String;
      ALocalObjectId, AExternalObjectId: TOID);
    function DoFindObject(AObjectClassName: String; AObjectId: TOID): IovObject;
      override;
    procedure DoMatchObjects(AObjectClassName: String; ExternalObjectId,
      InternalObjectId: TOID); override;
    function GetExternalObjectId(ADatabaseId, AObjectClassName: String;
      ALocalObjectId: TOID): TOID;
  public
    procedure LoadFromStream(AStream: TStream); override;
    procedure SaveToStream(AStream: TStream); override;
    procedure GenerateLookupTable;
    constructor Create(AOwner: TComponent); override;
  published
    property DatabaseId: String read FDatabaseId write FDatabaseId;
    property LookupTableName: String read FLookupTableName write FLookupTableName;
    property AutoGenerateLookupTable: Boolean read FAutoGenerateLookupTable write
      FAutoGenerateLookupTable default True;
  end;

resourcestring
  {      }
  DefaultLookupTableName = 'SYNCDB_TABLE';

  {     }
  ltRecordIdFieldName = 'RecID';
  ltDBIdFieldName = 'DatabaseID';
  ltClassNameFieldName = 'ClassName';
  ltLocalObjectIdFieldName = 'LocalOID';
  ltExternalObjectIdFieldName = 'ExternalOID';

implementation

uses SysUtils, ovComponents;

{ TovDBSynchronization }

constructor TovDBSynchronization.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  LookupTableName := DefaultLookupTableName;
  AutoGenerateLookupTable := True;
end;

{***************************************************************************** }
{                                                         }
{ **************************************************************************** }

procedure TovDBSynchronization.CheckDatabaseId(ReadMode: Boolean);
begin
  if DatabaseId = '' then
    if ReadMode then
      raise Exception.Create('      ')
    else
      raise Exception.Create('   DatabaseId  ' + Name)
end;

procedure TovDBSynchronization.SaveToStream(AStream: TStream);
begin
  CheckDatabaseId(False);
  CheckGenerateLookupTable;
  XMLDocument.DocumentElement.Attributes['DatabaseId'] := DatabaseId;
  inherited;
end;

procedure TovDBSynchronization.LoadFromStream(AStream: TStream);
begin
  inherited;
  CheckGenerateLookupTable;
  DatabaseId := XMLDocument.DocumentElement.Attributes['DatabaseId'];
  CheckDatabaseId(True);
end;

{***************************************************************************** }
{                                                   }
{ **************************************************************************** }

procedure TovDBSynchronization.CheckGenerateLookupTable;
begin
  if AutoGenerateLookupTable and Not Model.DatabaseAdapter.TableExists(LookupTableName) then
    GenerateLookupTable;
end;

procedure TovDBSynchronization.GenerateLookupTable;
var
  TableDefs: TTableDefs;
begin
  TableDefs := TTableDefs.Create(LookupTableName);
  try
    with TableDefs do
    begin
      AddField(ltRecordIdFieldName, ftInteger, 0);
      AddField(ltDBIdFieldName, ftString, 30);
      AddField(ltClassNameFieldName, ftString, 50);
      AddField(ltLocalObjectIdFieldName, ftInteger, 0);
      AddField(ltExternalObjectIdFieldName, ftInteger, 0);

      AddIndex('ByExternalOIDIdx', ltDBIdFieldName+';'+ltClassNameFieldName+';'+ltExternalObjectIdFieldName, [ixUnique]);

      Model.DatabaseAdapter.CreateTable(TableName, FieldDefs, IndexDefs);
    end;
  finally
    TableDefs.Free
  end;
end;

{{
         
AExternalObjectId   ADatabaseId,    AObjectClassName.
    ,    0.
}
function TovDBSynchronization.GetLocalObjectId(ADatabaseId, AObjectClassName:
  String; AExternalObjectId: TOID): TOID;
begin
  with Model.DatabaseAdapter.OpenQuery(
    Format('SELECT %s FROM %s WHERE %s = ''%s'' and %s = ''%s'' and %s = %d',
      [ltLocalObjectIdFieldName, LookupTableName,
      ltDBIdFieldName, ADatabaseId, ltClassNameFieldName, AObjectClassName,
      ltExternalObjectIdFieldName, AExternalObjectId])) do
  try
    if Not IsEmpty
      then Result := Fields[0].AsInteger
      else Result := 0;
  finally
    Free
  end;
end;

{{
     ADatabaseId   
 ALocalObjectId   ,    AObjectClassName.
    ,    0.
}
function TovDBSynchronization.GetExternalObjectId(ADatabaseId,
  AObjectClassName: String; ALocalObjectId: TOID): TOID;
begin
  with Model.DatabaseAdapter.OpenQuery(
    Format('SELECT %s FROM %s WHERE %s = ''%s'' and %s = ''%s'' and %s = %d',
      [ltExternalObjectIdFieldName, LookupTableName,
      ltDBIdFieldName, ADatabaseId, ltClassNameFieldName, AObjectClassName,
      ltLocalObjectIdFieldName, ALocalObjectId])) do
  try
    if Not IsEmpty
      then Result := Fields[0].AsInteger
      else Result := 0;
  finally
    Free
  end;
end;

{{
     RemoteDBId   ALocalObjectId,
   AObjectClassName,    AExternalObjectId.
}
procedure TovDBSynchronization.UpdateExternalObjectId(ADatabaseId,
  AObjectClassName: String; ALocalObjectId, AExternalObjectId: TOID);
begin
  if GetExternalObjectId(ADatabaseId, AObjectClassName, ALocalObjectId) <> 0 then
    Model.DatabaseAdapter.ExecQuery(
      Format('UPDATE %s SET %s = %d WHERE %s = ''%s'' and %s = ''%s'' and %s = %d',
        [LookupTableName, ltExternalObjectIdFieldName, AExternalObjectId,
          ltDBIdFieldName, ADatabaseId, ltClassNameFieldName, AObjectClassName,
          ltLocalObjectIdFieldName, ALocalObjectId]))
  else
    Model.DatabaseAdapter.ExecQuery(
      Format('INSERT INTO %s (%s, %s, %s, %s) VALUES (''%s'', ''%s'', %d, %d)',
        [LookupTableName,
        ltDBIdFieldName, ltClassNameFieldName, ltLocalObjectIdFieldName, ltExternalObjectIdFieldName,
        ADatabaseId, AObjectClassName, ALocalObjectId, AExternalObjectId]))
end;

{***************************************************************************** }
{                                           }
{ **************************************************************************** }

{{
     ,   AObjectId
  .    ,      nil.

   ,     , 
   ,     .
}
function TovDBSynchronization.DoFindObject(AObjectClassName: String; AObjectId:
  TOID): IovObject;
var
  LocalObjectId: TOID;
begin
  Result := inherited DoFindObject(AObjectClassName, AObjectId);
  if Result = nil then
  begin
    LocalObjectId := GetLocalObjectId(DatabaseId, AObjectClassName, AObjectId);
    if LocalObjectId <> 0 then
    begin
      Result := ObjectSpace.GetObject(AObjectClassName, LocalObjectId);
      {    ,       }
      if Not Result.IsExist then Result := nil
    end;
  end;
end;

{{
        
InternalObjectId   ExternalObjectId   .
}
procedure TovDBSynchronization.DoMatchObjects(AObjectClassName: String;
  ExternalObjectId, InternalObjectId: TOID);
begin
  inherited; {      }
  {      }
  UpdateExternalObjectId(DatabaseId, AObjectClassName, InternalObjectId, ExternalObjectId);
end;

end.
