unit ovDBSynchronizer;

interface

uses Classes, DB, ovTypes, ovObjectStorage, ovXMLObjectStorage, ovIObjects;

type
  {{
         .
      TovObjectStorage.  
   :
        -      XML-
        -   XML-  
             ( DatabaseId)
        -        
                  ;
           :
              ,     ,
              ,     
        -   XML-    , 
              ,     
              (..     
            )    
  }
  TovDBSynchronizer = class(TovXMLObjectStorage)
  private
    FCurrentDatabaseId: String;
    FLookupTableName: String;
    FAutoGenerateLookupTable: Boolean;
    FExternalDatabaseId: String;
    procedure CheckGenerateLookupTable;
  protected
    function GetLocalObjectId(ADatabaseId, AObjectClassName: String;
      AExternalObjectId: TOID): TOID;
    procedure UpdateExternalObjectId(ADatabaseId, AObjectClassName: String;
      ALocalObjectId, AExternalObjectId: TOID);
    function DoFindObject(AObjectClassName: String; AObjectId: TOID): IovObject;
      override;
    procedure DoMatchObjects(AObjectClassName: String; ExternalObjectId,
      InternalObjectId: TOID); override;
    function GetExternalObjectId(ADatabaseId, AObjectClassName: String;
      ALocalObjectId: TOID): TOID;
  public
    procedure LoadFromStream(AStream: TStream); override;
    procedure SaveToStream(AStream: TStream); override;
    procedure GenerateLookupTable;
    constructor Create(AOwner: TComponent); override;

    property XMLObjectReader;
    property XMLObjectWriter;
  published
    {{
       , ..      
      .     
    .
    }
    property CurrentDatabaseId: String read FCurrentDatabaseId write
      FCurrentDatabaseId;
    {{
            
    .
    }
    property ExternalDatabaseId: String read FExternalDatabaseId;
    property LookupTableName: String read FLookupTableName write FLookupTableName;
    property AutoGenerateLookupTable: Boolean read FAutoGenerateLookupTable write
      FAutoGenerateLookupTable default True;
  end;

resourcestring
  {      }
  DefaultLookupTableName = 'SYNCDB_TABLE';

  {     }
  ltRecordIdFieldName = 'RecID';
  ltDBIdFieldName = 'DatabaseID';
  ltClassNameFieldName = 'ClassName';
  ltLocalObjectIdFieldName = 'LocalOID';
  ltExternalObjectIdFieldName = 'ExternalOID';

implementation

uses SysUtils, ovComponents, XMLIntf;

{ TovDBSynchronizer }

constructor TovDBSynchronizer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  LookupTableName := DefaultLookupTableName;
  AutoGenerateLookupTable := True;
end;

{***************************************************************************** }
{                                                         }
{ **************************************************************************** }

procedure TovDBSynchronizer.SaveToStream(AStream: TStream);
begin
  if CurrentDatabaseId = '' then
    raise Exception.Create('      ' + Name);

  CheckGenerateLookupTable;
  XMLDocument.DocumentElement.Attributes['DatabaseId'] := CurrentDatabaseId;
  inherited;
end;

procedure TovDBSynchronizer.LoadFromStream(AStream: TStream);
begin
  inherited;
  CheckGenerateLookupTable;
  if CurrentDatabaseId = '' then  {      }
    raise Exception.Create('      ' + Name);

  FExternalDatabaseId := XMLDocument.DocumentElement.Attributes['DatabaseId'];
  if ExternalDatabaseId = '' then
    raise Exception.Create('      ')
end;

{***************************************************************************** }
{                                                   }
{ **************************************************************************** }

procedure TovDBSynchronizer.CheckGenerateLookupTable;
begin
  if AutoGenerateLookupTable and Not Model.DatabaseAdapter.TableExists(LookupTableName) then
    GenerateLookupTable;
end;

procedure TovDBSynchronizer.GenerateLookupTable;
var
  TableDefs: TTableDefs;
begin
  TableDefs := TTableDefs.Create(LookupTableName);
  try
    with TableDefs do
    begin
      AddField(ltRecordIdFieldName, ftAutoInc, 0);
      AddField(ltDBIdFieldName, ftString, 30);
      AddField(ltClassNameFieldName, ftString, 50);
      AddField(ltLocalObjectIdFieldName, ftInteger, 0);
      AddField(ltExternalObjectIdFieldName, ftInteger, 0);

      AddIndex('ByExternalOIDIdx', ltDBIdFieldName+';'+ltClassNameFieldName+';'+ltExternalObjectIdFieldName, [ixUnique]);

      Model.DatabaseAdapter.CreateTable(TableName, FieldDefs, IndexDefs);
    end;
  finally
    TableDefs.Free
  end;
end;

{{
         
AExternalObjectId   ADatabaseId,    AObjectClassName.
    ,    0.
}
function TovDBSynchronizer.GetLocalObjectId(ADatabaseId, AObjectClassName:
  String; AExternalObjectId: TOID): TOID;
begin
  with Model.DatabaseAdapter.OpenQuery(
    Format('SELECT %s FROM %s WHERE %s = ''%s'' and %s = ''%s'' and %s = %d',
      [ltLocalObjectIdFieldName, LookupTableName,
      ltDBIdFieldName, ADatabaseId, ltClassNameFieldName, AObjectClassName,
      ltExternalObjectIdFieldName, AExternalObjectId])) do
  try
    if Not IsEmpty
      then Result := Fields[0].AsInteger
      else Result := 0;
  finally
    Free
  end;
end;

{{
     ADatabaseId   
 ALocalObjectId   ,    AObjectClassName.
    ,    0.
}
function TovDBSynchronizer.GetExternalObjectId(ADatabaseId,
  AObjectClassName: String; ALocalObjectId: TOID): TOID;
begin
  with Model.DatabaseAdapter.OpenQuery(
    Format('SELECT %s FROM %s WHERE %s = ''%s'' and %s = ''%s'' and %s = %d',
      [ltExternalObjectIdFieldName, LookupTableName,
      ltDBIdFieldName, ADatabaseId, ltClassNameFieldName, AObjectClassName,
      ltLocalObjectIdFieldName, ALocalObjectId])) do
  try
    if Not IsEmpty
      then Result := Fields[0].AsInteger
      else Result := 0;
  finally
    Free
  end;
end;

{{
     RemoteDBId   ALocalObjectId,
   AObjectClassName,    AExternalObjectId.
}
procedure TovDBSynchronizer.UpdateExternalObjectId(ADatabaseId,
  AObjectClassName: String; ALocalObjectId, AExternalObjectId: TOID);
begin
  Model.DatabaseAdapter.ExecQuery(
    Format('DELETE FROM %s WHERE %s = ''%s'' and %s = ''%s'' and %s = %d',
      [LookupTableName,
       ltDBIdFieldName, ADatabaseId, ltClassNameFieldName, AObjectClassName,
         ltExternalObjectIdFieldName, AExternalObjectId]));

  Model.DatabaseAdapter.ExecQuery(
    Format('INSERT INTO %s (%s, %s, %s, %s) VALUES (''%s'', ''%s'', %d, %d)',
      [LookupTableName,
      ltDBIdFieldName, ltClassNameFieldName, ltLocalObjectIdFieldName, ltExternalObjectIdFieldName,
      ADatabaseId, AObjectClassName, ALocalObjectId, AExternalObjectId]))
end;

{***************************************************************************** }
{                                           }
{ **************************************************************************** }

{{
     ,   AObjectId
  .    ,      nil.

   ,     , 
   ,     .
}
function TovDBSynchronizer.DoFindObject(AObjectClassName: String; AObjectId:
  TOID): IovObject;
var
  LocalObjectId: TOID;
begin
  Result := inherited DoFindObject(AObjectClassName, AObjectId);
  if Result = nil then
  begin
    if ExternalDatabaseId <> CurrentDatabaseId then
      LocalObjectId := GetLocalObjectId(ExternalDatabaseId, AObjectClassName, AObjectId)
    else
      LocalObjectId := AObjectId;

    if LocalObjectId <> 0 then
    begin
      Result := ObjectSpace.GetObject(AObjectClassName, LocalObjectId);
      {    ,       }
      if Not Result.IsExist then Result := nil
    end;
  end;
end;

{{
        
InternalObjectId   ExternalObjectId   .
}
procedure TovDBSynchronizer.DoMatchObjects(AObjectClassName: String;
  ExternalObjectId, InternalObjectId: TOID);
begin
  inherited; {      }
  {      }
  UpdateExternalObjectId(ExternalDatabaseId, AObjectClassName, InternalObjectId, ExternalObjectId);
end;

end.
