unit ovDataLinks;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ovDataSet, ovDataObjects;

type
  TovDataLink = class;
  TovDataLinks = class;
  TovDataLinkController = class(TPersistent)
  private
    FLink: TovDataLink;
  protected
    procedure SetActive(Value: Boolean); virtual; abstract;
    procedure Loaded; dynamic;
  public
    property Link: TovDataLink read FLink;
  end;

  TovDataLinkCollection = class(TCollection)
  private
    FOwner: TovDataLinks;
    function GetLinks(Index: Integer): TovDataLink;
    procedure RemoveNotification(AComponent: TComponent);
  protected
    procedure Notify(Item: TCollectionItem; Action: TCollectionNotification); 
      override;
  public
    property Links[Index: Integer]: TovDataLink read GetLinks; default;
  end;

  TovDataLinkControllerClass = class of TovDataLinkController;

  TovDataLink = class(TCollectionItem)
  private
    FController: TovDataLinkController;
    FDataObject: String;
    FOwner: TovDataLinks;
    FEnabled: Boolean;
    FActive: Boolean;
    FAttribute: String;
    FComponent: TComponent;
    function GetControllerClassName: String;
    procedure SetController(Value: TovDataLinkController);
    procedure SetControllerClassName(const Value: String);
    procedure SetEnabled(Value: Boolean);
    procedure Loaded;
    procedure SetComponent(Value: TComponent);
  protected
    procedure SetActive(Value: Boolean); virtual;
    procedure AssignTo(Dest: TPersistent); override;
  public
    destructor Destroy; override;
    constructor Create(ACollection: TCollection); override;
    function GetDataSet: TovDataSet;
    property Owner: TovDataLinks read FOwner;
    property Active: Boolean read FActive write SetActive;
  published
    property ControllerClassName: String read GetControllerClassName write
      SetControllerClassName;
    property Controller: TovDataLinkController read FController write
      SetController;
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property DataObject: String read FDataObject write FDataObject;
    property Attribute: String read FAttribute write FAttribute;
    property Component: TComponent read FComponent write SetComponent;
  end;

  TovDataLinks = class(TComponent)
  private
    FDataObjects: TovDataObjects;
    FLinks: TovDataLinkCollection;
    FStreamedActive: Boolean;
    FAutoOpen: Boolean;
    procedure SetActive(Value: Boolean);
    procedure SetDataObjects(Value: TovDataObjects);
    procedure SetLinks(Value: TovDataLinkCollection);
    function GetActive: Boolean;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation);
      override;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AddLink(AControllerClass: TovDataLinkControllerClass; 
      ATargetComponent: TComponent; ADataObjectName: String; AttributeName: String 
      = ''): TovDataLink;
    function FindLink(ATargetComponent: TComponent; AControllerClass: 
      TovDataLinkControllerClass = nil): TovDataLink; virtual;
    procedure ActivateLink(AComponent: TComponent);
//    procedure ActivateLinks(DataObjectName: String);
  published
    property DataObjects: TovDataObjects read FDataObjects write SetDataObjects;
    property Links: TovDataLinkCollection read FLinks write SetLinks;
    property Active: Boolean read GetActive write SetActive;
    property AutoOpen: Boolean read FAutoOpen write FAutoOpen default False;
  end;

procedure RegisterDataLinkController(ControllerClass:
  TovDataLinkControllerClass);

procedure UnregisterDataLinkController(ControllerClass:
  TovDataLinkControllerClass);

function GetDataLinkController(ControllerClassName: String):
  TovDataLinkController;

procedure GetDataLinkControllers(ControllerClassNames: TStrings);

implementation

var
  RegisteredControllers: TList;

function FindDataLinkController(ControllerClassName: String): Integer;
var
  I: Integer;
begin
  for I := 0 to RegisteredControllers.Count - 1 do
    if TovDataLinkControllerClass(RegisteredControllers[I]).ClassName = ControllerClassName
      then begin
        Result := I;
        Exit;
      end;
  Result := -1;
end;

procedure RegisterDataLinkController(ControllerClass:
  TovDataLinkControllerClass);
begin
  RegisteredControllers.Add(ControllerClass);
end;

procedure UnregisterDataLinkController(ControllerClass:
  TovDataLinkControllerClass);
var
  I: Integer;
begin
  I := FindDataLinkController(ControllerClass.ClassName);
  if I >= 0 then RegisteredControllers.Delete(I);
end;

function GetDataLinkController(ControllerClassName: String):
  TovDataLinkController;
var
  I: Integer;
begin
  I := FindDataLinkController(ControllerClassName);
  if I >= 0 then Result := TovDataLinkControllerClass(RegisteredControllers[I]).Create
  else raise Exception.CreateFmt('Controller %s not found', [ControllerClassName]);
end;

procedure GetDataLinkControllers(ControllerClassNames: TStrings);
var
  I: Integer;
begin
  ControllerClassNames.Clear;
  for I := 0 to RegisteredControllers.Count - 1 do
    ControllerClassNames.Add(TovDataLinkControllerClass(RegisteredControllers[I]).ClassName)
end;

{
********************************* TovDataLink **********************************
}
constructor TovDataLink.Create(ACollection: TCollection);
begin
  inherited Create(ACollection);
  FEnabled := True;
end;

destructor TovDataLink.Destroy;
begin
  FreeAndNil(FController);
  inherited Destroy;
end;

function TovDataLink.GetControllerClassName: String;
begin
  if Assigned(Controller) then
    Result := Controller.ClassName
  else
    Result := ''
end;

procedure TovDataLink.SetController(Value: TovDataLinkController);
begin
  if FController <> Value then
  begin
    FreeAndNil(FController);
    FController := Value;
    if Value <> nil then
      FController.FLink := Self;
  end;
end;

procedure TovDataLink.SetControllerClassName(const Value: String);
begin
  if ControllerClassName <> Value then
    Controller := GetDataLinkController(Value);
end;

procedure TovDataLink.SetEnabled(Value: Boolean);
begin
  if FEnabled <> Value then
  begin
    if Not Value then Active := False;
    FEnabled := Value;
  end;
end;

procedure TovDataLink.SetActive(Value: Boolean);
begin
  if Active <> Value then
  begin
    if Assigned(Controller) then
    begin
      Controller.SetActive(Value);
      FActive := Value;
    end else
      raise Exception.Create('     ');
  end
end;

procedure TovDataLink.Loaded;
begin
  if Assigned(Controller) then Controller.Loaded
end;

procedure TovDataLink.SetComponent(Value: TComponent);
begin
  if FComponent <> Value then
  begin
    FComponent := Value;
    if Value <> nil then Value.FreeNotification(TovDataLinkCollection(Collection).FOwner)
  end;
end;

procedure TovDataLink.AssignTo(Dest: TPersistent);
begin
  if Dest is TovDataLink then
    with TovDataLink(Dest) do
    begin
      ControllerClassName := Self.ControllerClassName;
      DataObject := Self.DataObject;
      Attribute := Self.Attribute;
      Component := Self.Component;
    end
  else inherited AssignTo(Dest);
end;

function TovDataLink.GetDataSet: TovDataSet;
begin
  Result := Owner.DataObjects[DataObject];
end;

{
**************************** TovDataLinkController *****************************
}

procedure TovDataLinkController.Loaded;
begin
end;

{
***************************** TovDataLinkCollection ****************************
}
procedure TovDataLinkCollection.Notify(Item: TCollectionItem; Action: 
  TCollectionNotification);
begin
  if Action = cnAdded then
    (Item as TovDataLink).FOwner := FOwner
  else
    (Item as TovDataLink).FOwner := nil
end;

function TovDataLinkCollection.GetLinks(Index: Integer): TovDataLink;
begin
  Result := TovDataLink(Items[Index]);
end;

procedure TovDataLinkCollection.RemoveNotification(AComponent: TComponent);
var
  I: Integer;
begin
  for I := Count - 1 downto 0 do
    if Links[I].Component = AComponent then Delete(I)
end;

{
********************************* TovDataLinks *********************************
}
constructor TovDataLinks.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLinks := TovDataLinkCollection.Create(TovDataLink);
  FLinks.FOwner := Self;
end;

destructor TovDataLinks.Destroy;
begin
  FreeAndNil(FLinks);
  inherited Destroy;
end;

procedure TovDataLinks.Loaded;
var
  I: Integer;
begin
  inherited Loaded;
  for I := 0 to Links.Count - 1 do Links[I].Loaded;
  
  if FStreamedActive or FAutoOpen and Not (csDesigning in ComponentState)
    then Active := True;
end;

procedure TovDataLinks.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if AComponent = DataObjects then
      DataObjects := nil
    else
      if Assigned(Links) then
        Links.RemoveNotification(AComponent);
end;

procedure TovDataLinks.SetActive(Value: Boolean);
var
  I: Integer;
  ADataSet: TovDataSet;
begin
  if csReading in ComponentState then
    FStreamedActive := Value
  else begin
    if Value and Not Assigned(DataObjects)
      then raise Exception.Create('   DataObjects  ' + Name);

    for I := 0 to Links.Count - 1 do
      with Links[I] do
        if Enabled then Active := Value;

    // ,      
    // (        
    //    )
    if Value then
    for I := 0 to Links.Count - 1 do
      with Links[I] do
        if Enabled and (DataObject <> '') then
          ADataSet := DataObjects[DataObject];
  end
end;

function TovDataLinks.GetActive: Boolean;
var
  I: Integer;
begin
  Result := Assigned(DataObjects) and DataObjects.Active;
  //        
  if Result then
  begin
    for I := 0 to Links.Count - 1 do
      if Links[I].Active then Exit;
    Result := False;
  end;
end;

procedure TovDataLinks.SetDataObjects(Value: TovDataObjects);
begin
  if FDataObjects <> Value then
  begin
    Active := False;
    FDataObjects := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovDataLinks.SetLinks(Value: TovDataLinkCollection);
begin
  FLinks.Assign(Value);
end;

function TovDataLinks.AddLink(AControllerClass: TovDataLinkControllerClass;
  ATargetComponent: TComponent; ADataObjectName: String; AttributeName: String 
  = ''): TovDataLink;
begin
  Result := FLinks.Add as TovDataLink;
  Result.Controller := AControllerClass.Create;
  Result.DataObject := ADataObjectName;
  Result.Component := ATargetComponent;
  Result.Attribute := AttributeName;
end;

{{
  ,    , ..  
Component = ATargetComponent.   AControllerClass  , 
     ,  -    
   .
}
function TovDataLinks.FindLink(ATargetComponent: TComponent; AControllerClass:
  TovDataLinkControllerClass = nil): TovDataLink;
var
  I: Integer;
begin
  for I := 0 to Links.Count - 1 do
    if (Links[I].Component = ATargetComponent) and
       ((AControllerClass = nil) or (Links[I].Controller is AControllerClass))
    then begin
      Result :=	Links[I];
      Exit;
    end;
  Result := nil;
end;

procedure TovDataLinks.ActivateLink(AComponent: TComponent);
var
  Link: TovDataLink;
begin
  Link := FindLink(AComponent);
  if Assigned(Link) then Link.Active := True;
end;

//{{
//    ,     
//.
//}
//procedure TovDataLinks.ActivateLinks(DataObjectName: String);
//var
//  I: Integer;
//begin
//  for I := 0 to Links.Count - 1 do
//    if Links[I].DataObject = DataObjectName then
//      Links[I].Active := True
//end;

initialization
  RegisteredControllers := TList.Create;
finalization
  RegisteredControllers.Free;
end.
