unit ovDataObjectTreeViewBuilder;

interface

uses SysUtils, Classes, ComCtrls, ovDataObjects;

type
  TovDataObjectTreeViewBuilder = class(TComponent)
  private
    FTreeView: TCustomTreeView;
    FDataObjects: TovDataObjects;
    procedure SetTreeView(Value: TCustomTreeView);
    function GetTreeViewItems: TTreeNodes;
    procedure BuildTreeView;
    procedure SetDataObjects(const Value: TovDataObjects);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure BuildItems(RootNode: TTreeNode; RootObject: TovDataObject); virtual;
    property TreeViewItems: TTreeNodes read GetTreeViewItems;
  public
    procedure RefreshTreeView;
  published
    property TreeView: TCustomTreeView read FTreeView write SetTreeView;
    property DataObjects: TovDataObjects read FDataObjects write SetDataObjects;
  end;

implementation

uses ovDataSet;

const
  ntDataSet   = 0;
  ntAttribute = 1;
  ntQuery     = 2;

type
  TTreeViewFriend = class (TCustomTreeView)
  end;

{ TovDataObjectTreeViewBuilder }

procedure TovDataObjectTreeViewBuilder.SetTreeView(Value: TCustomTreeView);
begin
  if FTreeView <> Value then
  begin
    FTreeView := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovDataObjectTreeViewBuilder.SetDataObjects(const Value:
  TovDataObjects);
begin
  if FDataObjects <> Value then
  begin
    FDataObjects := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovDataObjectTreeViewBuilder.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = TreeView)then TreeView := nil else
  if (Operation = opRemove) and (AComponent = DataObjects)then DataObjects := nil;
end;

function TovDataObjectTreeViewBuilder.GetTreeViewItems: TTreeNodes;
begin
  Result := TTreeViewFriend(TreeView).Items;
end;

{***************************************************************************** }
{                                                            }
{ **************************************************************************** }

procedure TovDataObjectTreeViewBuilder.BuildItems(RootNode: TTreeNode;
  RootObject: TovDataObject);

  procedure BuildAttributes(ParentNode: TTreeNode; Item: TovDataObject);
  var
    I: Integer;
    S: TStrings;
    Node: TTreeNode;
  begin
    S := TStringList.Create;
    try
      Item.GetSelectedAttributes(S);
      for I := 0 to S.Count - 1 do begin
        Node := GetTreeViewItems.AddChild(ParentNode, S[I]);
        Node.ImageIndex := ntAttribute;
        Node.SelectedIndex := ntAttribute;
      end
    finally
      S.Free;
    end;
  end;

var
  Node: TTreeNode;
  I: Integer;
begin
  Node := GetTreeViewItems.AddChild(RootNode, RootObject.Name);
  if RootObject.DataSetClass = TovExposer then
  begin
    Node.ImageIndex := ntDataSet;
    Node.SelectedIndex := ntDataSet;
  end else
  begin
    Node.ImageIndex := ntQuery;
    Node.SelectedIndex := ntQuery;
  end;
  BuildAttributes(Node, RootObject);
  for I := 0 to RootObject.Count - 1 do
    BuildItems(Node, RootObject.Objects[I]);
end;

procedure TovDataObjectTreeViewBuilder.BuildTreeView;
var
  I: Integer;
begin
  with DataObjects do
  for I := 0 to RootObjectCount - 1 do
    BuildItems(nil, RootObjects[I]);
end;

{{
   .
}
procedure TovDataObjectTreeViewBuilder.RefreshTreeView;
begin
  if Not Assigned(TreeView) then raise Exception.Create('   TreeView  '+Name);
  if Not Assigned(DataObjects) then raise Exception.Create('   DataObjects  '+Name);

  with GetTreeViewItems do
  begin
    BeginUpdate;
    try
      Clear;
      BuildTreeView;
      TreeView.FullExpand;
    finally
      EndUpdate;
    end;
  end;
end;

end.
