unit ovDataObjects;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Contnrs,
  Forms, Dialogs, DB, ovTypes, ovIObjects, ovComponents, ovDataSet;

type
  TovCustomDataObjects = class;

  TovDataObject = class(TCollectionItem)
  private
    FObjects: TCollection;
    FMasterRelationName: String;
    FName: String;
    FObjectAttributes: String;
    FObjectClassName: String;
    FReadOnly: Boolean;
    FObjectFilter: String;
    FObjectOrder: String;
    FOwner: TovCustomDataObjects;
    FDataSet: TovDataSet;
    FDataSource: TDataSource;
    FParent: TovDataObject;
    FDataSetClass: TovDataSetClass;
    FQuery: TStrings;
    FAutoRefresh: Boolean;
    FOptions: TovDataSetOptions;
    FMasterFieldName: String;
    FAutoOpen: Boolean;
    FJoinExpression: String;
    function GetCount: Integer;
    function GetObjects(Index: Integer): TovDataObject;
    function GetIsChild: Boolean;
    procedure ReadObjects(Reader: TReader);
    procedure WriteObjects(Writer: TWriter);
    procedure SetName(const Value: String);
    procedure SetQuery(Value: TStrings);
    procedure Initialize(AOwner: TovCustomDataObjects; AParent: TovDataObject);
    function GetDataSetClassName: String;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure SetDataSetClassName(const Value: String); virtual;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Close;
    procedure Open;
    procedure GetSelectedAttributes(Attributes: TStrings);
    procedure CloseDataSet;
    procedure CreateDataSet;
    property Count: Integer read GetCount;
    property Objects[Index: Integer]: TovDataObject read GetObjects;
    property IsChild: Boolean read GetIsChild;
    property Owner: TovCustomDataObjects read FOwner;
    property DataSet: TovDataSet read FDataSet;
    property DataSource: TDataSource read FDataSource;
    property Parent: TovDataObject read FParent;
    property DataSetClass: TovDataSetClass read FDataSetClass write FDataSetClass;
  published
    property Name: String read FName write SetName;
    property Options: TovDataSetOptions read FOptions write FOptions default [];
    property ObjectClassName: String read FObjectClassName write
      FObjectClassName;
    property ObjectAttributes: String read FObjectAttributes write
      FObjectAttributes;
    property ObjectFilter: String read FObjectFilter write FObjectFilter;
    property ObjectOrder: String read FObjectOrder write FObjectOrder;
    property JoinExpression: String read FJoinExpression write FJoinExpression;
    property Query: TStrings read FQuery write SetQuery;
    property MasterRelationName: String read FMasterRelationName write
      FMasterRelationName;
    property MasterFieldName: String read FMasterFieldName write FMasterFieldName;
    property AutoRefresh: Boolean read FAutoRefresh write FAutoRefresh default
      False;
    property ReadOnly: Boolean read FReadOnly write FReadOnly default False;
    {{
            DataSetClass
    }
    property DataSetClassName: String read GetDataSetClassName write
      SetDataSetClassName;
    property AutoOpen: Boolean read FAutoOpen write FAutoOpen default True;
  end;

  TovDataObjectItemClass = class of TovDataObject;

  TVisitObjectProc = procedure (AObject: TovDataObject; Data: Pointer) of object;

  TovDataObjectsNotifyEvent = procedure (Sender: TovCustomDataObjects; ADataObjectName: String;
    DataSet: TovDataSet) of object;

  TovDataObjectsDataChangeEvent = procedure (Sender: TovCustomDataObjects; ADataObjectName: String;
    Field: TField) of object;

  TovDataObjectsCreateObjectEvent = procedure (Sender: TObject;
    ADataObjectName: String; AObject: IovObject; var Cancel: Boolean) of object;

  TovDataObjectsAddObjectEvent = procedure (Sender: TObject;
    ADataObjectName: String; var NewObject: TOID; var Cancel: Boolean) of object;

  TovDataObjectsDeleteObjectEvent = procedure (Sender: TObject;
    ADataObjectName: String; AObject: IovObject) of object;

  TovDataObjectsRemoveObjectEvent = procedure (Sender: TObject;
    ADataObjectName: String; AObject: IovObject; var ACancel, DeleteObject: Boolean) of object;

  TovCustomDataObjects = class(TovModelClient)
  private
    FRootObjects: TCollection;
    FObjects: TList;
    FOnCreateDataSet: TovDataObjectsNotifyEvent;
    FAutoOpen: Boolean;
    FLockOpen: Boolean;
    FStreamedActive: Boolean;
    FNameSpace: String;
    FParams: TStrings;
    function GetRootObjectCount: Integer;
    function GetRootObjects(Index: Integer): TovDataObject;
    function GetObjectByName(ObjectName: String): TovDataObject;
    procedure OpenObject(AObject: TovDataObject; Data: Pointer);
    procedure ReadObjects(Reader: TReader);
    procedure DoRefresh(AObject: TovDataObject; Data: Pointer); virtual;
    procedure WriteObjects(Writer: TWriter);
    procedure DoCheckBrowseMode(AObject: TovDataObject; Data: Pointer);
    function GetDataSets(ObjectName: String): TovDataSet;
    function GetDataSources(ObjectName: String): TDataSource;
    procedure CloseObject(AObject: TovDataObject; Data: Pointer);
    procedure DoCancel(AObject: TovDataObject; Data: Pointer);
    function GetObjectCount: Integer;
    function GetObjects(Index: Integer): TovDataObject;
    procedure SetActive(Value: Boolean);
    function GetParamByName(ParamName: String): String;
    procedure SetParamByName(ParamName: String; const Value: String);
    procedure SetParams(Value: TStrings);
    procedure ParamsChange(Sender: TObject);
    procedure UpdateObjectParams(AObject: TovDataObject; Data: Pointer);
    function GetActive: Boolean;
    procedure CheckObjectIsActive(AObject: TovDataObject; Data: Pointer);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure VisitObjects(VisitProc: TVisitObjectProc; Data: Pointer);
    procedure CheckOpen;
    procedure ModelChanging; override;
    procedure Loaded; override;
    function OpenDataSource(ADataObject: TovDataObject): TDataSource; virtual;
    function OpenDataSet(ADataObject: TovDataObject): TovDataSet;
    procedure AfterCreateDataSet(ADataObject: TovDataObject); dynamic;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AddObject(AObjectName: String; AParentObjectName: String = ''): 
      TovDataObject;
    procedure Assign(Source: TPersistent); override;
    procedure Close; dynamic;
    procedure DeleteObject(AObjectName: String); dynamic;
    function FindObjectByName(AObjectName: String): TovDataObject;
    {{
       
    }
    procedure Refresh;
    procedure DeleteObjects;
    procedure Post;
    procedure Open;
    procedure Cancel;
    procedure OpenDataSetForObject(ADataObjectName: String; AObject: IovObject);
    property RootObjectCount: Integer read GetRootObjectCount;
    property RootObjects[Index: Integer]: TovDataObject read GetRootObjects;
    property ObjectByName[ObjectName: String]: TovDataObject read GetObjectByName;
    property DataSets[ObjectName: String]: TovDataSet read GetDataSets; default;
    property DataSources[ObjectName: String]: TDataSource read GetDataSources;
    property ObjectCount: Integer read GetObjectCount;
    property Objects[Index: Integer]: TovDataObject read GetObjects;
    property OnCreateDataSet: TovDataObjectsNotifyEvent read FOnCreateDataSet write
      FOnCreateDataSet;
    property ParamByName[ParamName: String]: String read GetParamByName write 
      SetParamByName;
    property Params: TStrings read FParams write SetParams;
  published
    property Active: Boolean read GetActive write SetActive default False;
    property AutoOpen: Boolean read FAutoOpen write FAutoOpen default False;
    property NameSpace: String read FNameSpace write FNameSpace;
  end;

  TovDataObjects = class(TovCustomDataObjects)
  private
    FAfterCancel: TovDataObjectsNotifyEvent;
    FAfterDelete: TovDataObjectsNotifyEvent;
    FAfterEdit: TovDataObjectsNotifyEvent;
    FAfterInsert: TovDataObjectsNotifyEvent;
    FAfterPost: TovDataObjectsNotifyEvent;
    FAfterScroll: TovDataObjectsNotifyEvent;
    FBeforeCancel: TovDataObjectsNotifyEvent;
    FBeforeDelete: TovDataObjectsNotifyEvent;
    FBeforeEdit: TovDataObjectsNotifyEvent;
    FBeforeInsert: TovDataObjectsNotifyEvent;
    FBeforePost: TovDataObjectsNotifyEvent;
    FBeforeScroll: TovDataObjectsNotifyEvent;
    FOnNewRecord: TovDataObjectsNotifyEvent;
    FOnCreateObject: TovDataObjectsCreateObjectEvent;
    FOnAddObject: TovDataObjectsAddObjectEvent;
    FOnDeleteObject: TovDataObjectsDeleteObjectEvent;
    FOnRemoveObject: TovDataObjectsRemoveObjectEvent;
    FAfterRefresh: TovDataObjectsNotifyEvent;
    FBeforeRefresh: TovDataObjectsNotifyEvent;
    FOnDataChange: TovDataObjectsDataChangeEvent;
    FOnStateChange: TovDataObjectsNotifyEvent;
    FOnUpdateData: TovDataObjectsNotifyEvent;
    FOnCustomCalcFields: TovDataObjectsNotifyEvent;
    FBeforeOpen: TovDataObjectsNotifyEvent;
    FAfterOpen: TovDataObjectsNotifyEvent;
    FBeforeClose: TovDataObjectsNotifyEvent;
    procedure DoAfterCancel(DataSet: TDataSet);
    procedure DoAfterDelete(DataSet: TDataSet);
    procedure DoAfterEdit(DataSet: TDataSet);
    procedure DoAfterInsert(DataSet: TDataSet);
    procedure DoAfterPost(DataSet: TDataSet);
    procedure DoAfterScroll(DataSet: TDataSet);
    procedure DoBeforeCancel(DataSet: TDataSet);
    procedure DoBeforeDelete(DataSet: TDataSet);
    procedure DoBeforeEdit(DataSet: TDataSet);
    procedure DoBeforeInsert(DataSet: TDataSet);
    procedure DoBeforePost(DataSet: TDataSet);
    procedure DoBeforeScroll(DataSet: TDataSet);
    procedure DoNewRecord(DataSet: TDataSet);
    function GetObjectName(ADataSet: TDataSet): String;
    procedure DoOnAddObject(Sender: TObject; var NewObject: TOID; var Cancel:
      Boolean);
    procedure DoOnCreateObject(Sender: TObject; AObject: IovObject; var Cancel:
      Boolean);
    procedure DoOnDeleteObject(Sender: TObject; AObject: IovObject);
    procedure DoOnRemoveObject(Sender: TObject; AObject: IovObject; var ACancel, 
      ADeleteObject: Boolean);
    procedure DoAfterRefresh(DataSet: TDataSet);
    procedure DoBeforeRefresh(DataSet: TDataSet);
    procedure DoOnDataChange(Sender: TObject; Field: TField);
    procedure DoOnStateChange(Sender: TObject);
    procedure DoOnUpdateData(Sender: TObject);
    procedure DoOnCustomCalcFields(DataSet: TDataSet);
    procedure DoBeforeOpen(DataSet: TDataSet);
    procedure DoAfterOpen(DataSet: TDataSet);
    procedure DoBeforeClose(DataSet: TDataSet);
  protected
    procedure AfterCreateDataSet(ADataObject: TovDataObject); override;
    function OpenDataSource(ADataObject: TovDataObject): TDataSource; override;
  published
    //   TDataSet
    property AfterCancel: TovDataObjectsNotifyEvent read FAfterCancel write
      FAfterCancel;
    property AfterDelete: TovDataObjectsNotifyEvent read FAfterDelete write
      FAfterDelete;
    property AfterEdit: TovDataObjectsNotifyEvent read FAfterEdit write FAfterEdit;
    property AfterInsert: TovDataObjectsNotifyEvent read FAfterInsert write
      FAfterInsert;
    property AfterPost: TovDataObjectsNotifyEvent read FAfterPost write FAfterPost;
    property AfterScroll: TovDataObjectsNotifyEvent read FAfterScroll write
      FAfterScroll;
    property AfterOpen: TovDataObjectsNotifyEvent read FAfterOpen write FAfterOpen;
    property BeforeOpen: TovDataObjectsNotifyEvent read FBeforeOpen write
      FBeforeOpen;
    property BeforeCancel: TovDataObjectsNotifyEvent read FBeforeCancel write
      FBeforeCancel;
    property BeforeDelete: TovDataObjectsNotifyEvent read FBeforeDelete write
      FBeforeDelete;
    property BeforeEdit: TovDataObjectsNotifyEvent read FBeforeEdit write
      FBeforeEdit;
    property BeforeInsert: TovDataObjectsNotifyEvent read FBeforeInsert write
      FBeforeInsert;
    property BeforePost: TovDataObjectsNotifyEvent read FBeforePost write
      FBeforePost;
    property BeforeScroll: TovDataObjectsNotifyEvent read FBeforeScroll write
      FBeforeScroll;
    property OnNewRecord: TovDataObjectsNotifyEvent read FOnNewRecord write
      FOnNewRecord;
    property AfterRefresh: TovDataObjectsNotifyEvent read FAfterRefresh write
      FAfterRefresh;
    property BeforeRefresh: TovDataObjectsNotifyEvent read FBeforeRefresh write
      FBeforeRefresh;
    property OnCustomCalcFields: TovDataObjectsNotifyEvent read FOnCustomCalcFields
      write FOnCustomCalcFields;

    //   TDataSource
    property OnDataChange: TovDataObjectsDataChangeEvent read FOnDataChange write
      FOnDataChange;
    property OnStateChange: TovDataObjectsNotifyEvent read FOnStateChange write
      FOnStateChange;
    property OnUpdateData: TovDataObjectsNotifyEvent read FOnUpdateData write
      FOnUpdateData;

    //   TovDataSet
    property OnCreateObject: TovDataObjectsCreateObjectEvent read FOnCreateObject
      write FOnCreateObject;
    property OnDeleteObject: TovDataObjectsDeleteObjectEvent read FOnDeleteObject
      write FOnDeleteObject;
    property OnAddObject: TovDataObjectsAddObjectEvent read FOnAddObject write
      FOnAddObject;
    property OnRemoveObject: TovDataObjectsRemoveObjectEvent read FOnRemoveObject
      write FOnRemoveObject;

    //  
    property OnCreateDataSet;
    property BeforeClose: TovDataObjectsNotifyEvent read FBeforeClose write
      FBeforeClose;
  end;

implementation

uses ovModel, ovUtils;

{
******************************** TovObjectsItem ********************************
}
constructor TovDataObject.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FAutoOpen := True;
  FDataSetClass := TovExposer;
  //       ,    
  FObjects := TCollection.Create(TCollectionItemClass(ClassType));
  FQuery := TStringList.Create;
end;

destructor TovDataObject.Destroy;
begin
  Close;
  FQuery.Free;
  FObjects.Free;
  inherited Destroy;
end;

procedure TovDataObject.Close;
begin
  FreeAndNil(FDataSet);
  FreeAndNil(FDataSource);
end;

procedure TovDataObject.CloseDataSet;
begin
  if Assigned(FDataSet) then FDataSet.Close
end;

procedure TovDataObject.Open;
begin
  FOwner.DataSets[Name]
end;

procedure TovDataObject.Initialize(AOwner: TovCustomDataObjects; AParent:
  TovDataObject);
var
  I: Integer;
begin
  FOwner := AOwner;
  FParent := AParent;
  AOwner.FObjects.Add(Self);
  for I := 0 to Count - 1 do Objects[I].Initialize(AOwner, Self);
end;

procedure TovDataObject.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('Objects', ReadObjects, WriteObjects, FObjects.Count > 0);
end;

procedure TovDataObject.ReadObjects(Reader: TReader);
begin
  Reader.ReadValue;
  Reader.ReadCollection(FObjects)
end;

procedure TovDataObject.WriteObjects(Writer: TWriter);
begin
  Writer.WriteCollection(FObjects)
end;

function TovDataObject.GetCount: Integer;
begin
  Result := FObjects.Count
end;

function TovDataObject.GetObjects(Index: Integer): TovDataObject;
begin
  Result := TovDataObject(FObjects.Items[Index])
end;

function TovDataObject.GetIsChild: Boolean;
begin
  Result := Assigned(Parent);
end;

procedure TovDataObject.SetName(const Value: String);
begin
  if Assigned(Owner) and Assigned(Owner.FindObjectByName(Value)) then
    raise Exception.CreateFmt('    "%s"  ' , [Value]);
  FName := Value;
end;

procedure TovDataObject.SetQuery(Value: TStrings);
begin
  FQuery.Assign(Value);
end;

function TovDataObject.GetDataSetClassName: String;
begin
  Result := DataSetClass.ClassName;
end;

procedure TovDataObject.SetDataSetClassName(const Value: String);
begin
  if Value = 'TovExposer' then DataSetClass := TovExposer else
  if Value = 'TovQuery' then DataSetClass := TovQuery else
  raise Exception.CreateFmt('    "%s"', [Name])
end;

procedure TovDataObject.GetSelectedAttributes(Attributes: TStrings);
var
  S: String;
begin
  if DataSetClass = TovQuery then
    S := ExtractSelectFromQuery(Query)
  else
    S := ObjectAttributes;
  AttributesToList(S, ',', Attributes);
end;

procedure TovDataObject.CreateDataSet;
begin
  FreeAndNil(FDataSet);
  FDataSet := DataSetClass.Create(FOwner);

  //   TovExposer  TovQuery
  if FDataSet is TovQuery then
    with TovQuery(FDataSet) do
    begin
      Query := Self.Query;
      Options := Self.Options;
    end
  else
    with TovExposer(FDataSet) do
    begin
      if Not Self.IsChild then
        ObjectClassName := Self.ObjectClassName;
      ObjectAttributes := Self.ObjectAttributes;
      ObjectFilter := Self.ObjectFilter;
      ObjectOrder := Self.ObjectOrder;
      JoinExpression := Self.JoinExpression;
      Options := Self.Options;
    end;

  //   
  with FDataSet do
  begin
    if IsChild then
    begin
      MasterDataSet := FOwner.DataSets[Parent.Name];
      MasterRelationName := Self.MasterRelationName;
      MasterFieldName := Self.MasterFieldName;
    end;
    AutoRefresh := Self.AutoRefresh;
    ReadOnly := Self.ReadOnly;
    Model := FOwner.Model;
    NameSpace := FOwner.NameSpace;
    //         
    //        
    AutoOpen := True;
    Params := FOwner.Params;
  end;

  FOwner.AfterCreateDataSet(Self);
  if Assigned(FOwner.FOnCreateDataSet) then
    FOwner.FOnCreateDataSet(FOwner, Name, FDataSet);
end;

{
***************************** TovCustomDataObjects *****************************
}
constructor TovCustomDataObjects.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FRootObjects := TCollection.Create(TovDataObject);
  FObjects := TList.Create;

  FParams := TStringList.Create;
  TStringList(FParams).OnChange := ParamsChange;
end;

destructor TovCustomDataObjects.Destroy;
begin
  FParams.Free;
  FObjects.Free;
  FRootObjects.Free;
  inherited Destroy;
end;

procedure TovCustomDataObjects.DefineProperties(Filer: TFiler);
var
  I: Integer;
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('Objects', ReadObjects, WriteObjects, FObjects.Count > 0);

  if Filer is TReader then
    for I := 0 to RootObjectCount - 1 do RootObjects[I].Initialize(Self, nil);
end;

procedure TovCustomDataObjects.ReadObjects(Reader: TReader);
begin
  Reader.ReadValue;
  Reader.ReadCollection(FRootObjects)
end;

procedure TovCustomDataObjects.WriteObjects(Writer: TWriter);
begin
  Writer.WriteCollection(FRootObjects)
end;

{{
     Source.
}
procedure TovCustomDataObjects.Assign(Source: TPersistent);
var
  SaveName: String;
begin
  if Source is TovCustomDataObjects then
  begin
    Close;
    DeleteObjects;
    with TMemoryStream.Create do
    try
      SaveName := Name;
      WriteComponent(TovCustomDataObjects(Source));
      Position := 0;

      //      
      //   Loaded (  Source   AutoOpen)
      FLockOpen := True;
      try
        ReadComponent(Self);
      finally
        FLockOpen := False
      end;

      Name := SaveName;
    finally
      Free
    end;
  end else
    inherited Assign(Source);
end;

procedure TovCustomDataObjects.ModelChanging;
begin
  inherited;
  Close;
end;

{ ============================================================================ }
{ /                                               }
{ ============================================================================ }

procedure TovCustomDataObjects.CheckOpen;
begin
  ModelNeeded;
end;

function TovCustomDataObjects.GetDataSets(ObjectName: String): TovDataSet;
begin
  CheckOpen;
  Result := OpenDataSet(ObjectByName[ObjectName])
end;

function TovCustomDataObjects.GetDataSources(ObjectName: String): TDataSource;
var
  AObject: TovDataObject;
begin
  CheckOpen;
  AObject := ObjectByName[ObjectName];
  if Assigned(AObject.DataSource) then
    Result := AObject.DataSource
  else
    Result := OpenDataSource(AObject)
end;

function TovCustomDataObjects.OpenDataSource(ADataObject: TovDataObject):
  TDataSource;
begin
  OpenDataSet(ADataObject);
  Result := TDataSource.Create(Self);
  ADataObject.FDataSource := Result;
  Result.DataSet := ADataObject.DataSet;
end;

procedure TovCustomDataObjects.SetActive(Value: Boolean);
begin
  if csReading in ComponentState then
    FStreamedActive := Value
  else
    if Value then Open else Close
end;

{{
 True,       .
}
function TovCustomDataObjects.GetActive: Boolean;
begin
  Result := False;
  try
    VisitObjects(CheckObjectIsActive, nil);
  except
    on E: EAbort do Result := True
  end;
end;

procedure TovCustomDataObjects.CheckObjectIsActive(AObject: TovDataObject;
  Data: Pointer);
begin
  if Assigned(AObject.DataSet) and AObject.DataSet.Active
    then Abort //  ,    
end;

procedure TovCustomDataObjects.Loaded;
begin
  inherited Loaded;
  if Not FLockOpen and (AutoOpen or FStreamedActive) and Not (csDesigning in ComponentState)
    then Active := True;
end;

{{
    ,     AutoOpen.
}
procedure TovCustomDataObjects.Open;
begin
  CheckOpen;
  VisitObjects(OpenObject, nil);
end;

{{
    TovDataSet  TDataSource.
}
procedure TovCustomDataObjects.Close;
begin
  VisitObjects(CloseObject, nil);
end;

procedure TovCustomDataObjects.OpenObject(AObject: TovDataObject; Data: Pointer);
begin
  OpenDataSet(AObject)
end;

procedure TovCustomDataObjects.CloseObject(AObject: TovDataObject; Data: Pointer);
begin
  AObject.Close
end;

function TovCustomDataObjects.OpenDataSet(ADataObject: TovDataObject):
  TovDataSet;
begin
  if Not Assigned(ADataObject.DataSet) then ADataObject.CreateDataSet;
  try
    if ADataObject.AutoOpen then ADataObject.DataSet.Open;
  except
    on E: Exception do
      raise Exception.Create('    "' + ADataObject.Name + '": '#13 +
        E.Message);
  end;
  Result := ADataObject.DataSet;
end;

{{
     ADataObjectName,     , 
    AObject.  AObject   , 
  ADataObjectName.
}
procedure TovCustomDataObjects.OpenDataSetForObject(ADataObjectName: String;
  AObject: IovObject);
var
  ADataObject: TovDataObject;
begin
  ADataObject := ObjectByName[ADataObjectName];
  {
     Namespace
  if ADataObject.ObjectClassName <> AObject.ClassName then
    raise Exception.CreateFmt('  : "%s".  : "%s".',
      [AObject.ClassName, ADataObject.ObjectClassName]);
  }
  if Not Assigned(ADataObject.DataSet) then ADataObject.CreateDataSet;
  (ADataObject.DataSet as TovExposer).ExposeObject(AObject, ADataObject.ObjectAttributes);
end;

procedure TovCustomDataObjects.AfterCreateDataSet(ADataObject: TovDataObject);
begin
end;

{ ============================================================================ }
{                                              }
{ ============================================================================ }

{{
 Refresh     
}
procedure TovCustomDataObjects.Refresh;
begin
  VisitObjects(DoRefresh, nil);
end;

procedure TovCustomDataObjects.DoRefresh(AObject: TovDataObject; Data: Pointer);
begin
  if Assigned(AObject.DataSet) then AObject.DataSet.Refresh;
end;

{{
 CheckBrowseMode     
}
procedure TovCustomDataObjects.Post;
begin
  VisitObjects(DoCheckBrowseMode, nil);
end;

procedure TovCustomDataObjects.DoCheckBrowseMode(AObject: TovDataObject; Data:
  Pointer);
begin
  if Assigned(AObject.DataSet) and AObject.DataSet.Active then
    AObject.DataSet.CheckBrowseMode;
end;

{{
 Cancel     
}
procedure TovCustomDataObjects.Cancel;
begin
  VisitObjects(DoCancel, nil);
end;

procedure TovCustomDataObjects.DoCancel(AObject: TovDataObject; Data:
  Pointer);
begin
  if Assigned(AObject.DataSet) then AObject.DataSet.Cancel;
end;

{ ============================================================================ }
{                                                               }
{ ============================================================================ }

procedure TovCustomDataObjects.VisitObjects(VisitProc: TVisitObjectProc; Data:
  Pointer);
var
  I: Integer;
begin
  for I := 0 to ObjectCount - 1 do
    VisitProc(Objects[I], Data);
end;

function TovCustomDataObjects.FindObjectByName(AObjectName: String): TovDataObject;
var
  I: Integer;
begin
  for I := 0 to ObjectCount - 1 do
    if Objects[I].Name = AObjectName then
    begin
      Result := Objects[I];
      Exit;
    end;
  Result := nil
end;

function TovCustomDataObjects.GetObjectByName(ObjectName: String): TovDataObject;
begin
  Result := FindObjectByName(ObjectName);
  if Not Assigned(Result) then
    raise Exception.CreateFmt('  "%s"  ' , [ObjectName]);
end;

function TovCustomDataObjects.GetObjectCount: Integer;
begin
  Result := FObjects.Count;
end;

function TovCustomDataObjects.GetObjects(Index: Integer): TovDataObject;
begin
  Result := TovDataObject(FObjects.Items[Index])
end;

function TovCustomDataObjects.GetRootObjectCount: Integer;
begin
  Result := FRootObjects.Count;
end;

function TovCustomDataObjects.GetRootObjects(Index: Integer): TovDataObject;
begin
  Result := TovDataObject(FRootObjects.Items[Index])
end;

{ ============================================================================ }
{                                                   }
{ ============================================================================ }

function TovCustomDataObjects.AddObject(AObjectName: String; AParentObjectName:
  String = ''): TovDataObject;
var
  AParent: TovDataObject;
begin
  if FindObjectByName(AObjectName) <> nil then
    raise Exception.CreateFmt('  "%s"  ' , [AObjectName]);

  if AParentObjectName <> '' then
    AParent := ObjectByName[AParentObjectName]
  else
    AParent := nil;

  if Assigned(AParent) then 
    Result := TovDataObject(AParent.FObjects.Add)
  else
    Result := TovDataObject(FRootObjects.Add);
  Result.FOwner := Self;
  Result.FParent := AParent;
  Result.FName := AObjectName;
  FObjects.Add(Result);
end;

procedure TovCustomDataObjects.DeleteObject(AObjectName: String);
var
  AObject: TovDataObject;
begin
  AObject := ObjectByName[AObjectName];
  FObjects.Remove(AObject);
  AObject.Free;
end;

procedure TovCustomDataObjects.DeleteObjects;
begin
  FObjects.Clear;     //    
  FRootObjects.Clear; //   
end;

{ ============================================================================ }
{                                                                     }
{ ============================================================================ }

function TovCustomDataObjects.GetParamByName(ParamName: String): String;
begin
  Result := FParams.Values[ParamName];
end;

procedure TovCustomDataObjects.SetParamByName(ParamName: String; const Value:
  String);
begin
  FParams.Values[ParamName] := Value;
end;

procedure TovCustomDataObjects.SetParams(Value: TStrings);
begin
  FParams.Assign(Value);
end;

procedure TovCustomDataObjects.ParamsChange(Sender: TObject);
begin
  if Active then VisitObjects(UpdateObjectParams, nil);
end;

procedure TovCustomDataObjects.UpdateObjectParams(AObject: TovDataObject; Data:
  Pointer);
begin
  if Assigned(AObject.DataSet) then begin
    AObject.DataSet.Params := Params;
    AObject.DataSet.Open;
  end
end;

{
******************************** TovDataObjects ********************************
}

procedure TovDataObjects.AfterCreateDataSet(ADataObject: TovDataObject);
begin
  inherited;

  with ADataObject.DataSet do begin
    Tag := Integer(ADataObject); //   Tag    ,
                                 //      !!

    //    TDataSet
    if Assigned(FAfterPost)     then AfterPost    := DoAfterPost;
    if Assigned(FAfterCancel)   then AfterCancel  := DoAfterCancel;
    if Assigned(FAfterDelete)   then AfterDelete  := DoAfterDelete;
    if Assigned(FAfterEdit)     then AfterEdit    := DoAfterEdit;
    if Assigned(FAfterInsert)   then AfterInsert  := DoAfterInsert;
    if Assigned(FAfterScroll)   then AfterScroll  := DoAfterScroll;
    if Assigned(FAfterOpen)     then AfterOpen    := DoAfterOpen;
    if Assigned(FBeforeCancel)  then BeforeCancel := DoBeforeCancel;
    if Assigned(FBeforeDelete)  then BeforeDelete := DoBeforeDelete;
    if Assigned(FBeforeEdit)    then BeforeEdit   := DoBeforeEdit;
    if Assigned(FBeforeInsert)  then BeforeInsert := DoBeforeInsert;
    if Assigned(FBeforePost)    then BeforePost   := DoBeforePost;
    if Assigned(FBeforeScroll)  then BeforeScroll := DoBeforeScroll;
    if Assigned(FOnNewRecord)   then OnNewRecord  := DoNewRecord;
    if Assigned(FBeforeOpen)    then BeforeOpen   := DoBeforeOpen;
    if Assigned(FBeforeClose)   then BeforeClose   := DoBeforeClose;
    if Assigned(FBeforeRefresh) then BeforeRefresh := DoBeforeRefresh;
    if Assigned(FAfterRefresh)  then AfterRefresh  := DoAfterRefresh;

    //    TovDataSet
    if Assigned(FOnCreateObject) then OnCreateObject := DoOnCreateObject;
    if Assigned(FOnDeleteObject) then OnDeleteObject := DoOnDeleteObject;
    if Assigned(FOnAddObject)    then OnAddObject := DoOnAddObject;
    if Assigned(FOnRemoveObject) then OnRemoveObject := DoOnRemoveObject;
    // OnCalcFields   , ..   
    // TField      .   
    // ""   !
    if Assigned(FOnCustomCalcFields)  then OnCustomCalcFields := DoOnCustomCalcFields;
  end
end;

function TovDataObjects.OpenDataSource(ADataObject: TovDataObject): TDataSource;
begin
  Result := inherited OpenDataSource(ADataObject);
  if Assigned(FOnDataChange) then Result.OnDataChange:= DoOnDataChange;
  if Assigned(FOnStateChange) then Result.OnStateChange := DoOnStateChange;
  if Assigned(FOnUpdateData) then Result.OnUpdateData := DoOnUpdateData;
end;

{{
  Tag        
}
function TovDataObjects.GetObjectName(ADataSet: TDataSet): String;
begin
  Result := TovDataObject(ADataSet.Tag).Name;
end;

{ ============================================================================ }
{    TDataSet                                      }
{ ============================================================================ }

procedure TovDataObjects.DoAfterCancel(DataSet: TDataSet);
begin
  if Assigned(FAfterCancel) then
    FAfterCancel(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoAfterDelete(DataSet: TDataSet);
begin
  if Assigned(FAfterDelete) then
    FAfterDelete(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoAfterEdit(DataSet: TDataSet);
begin
  if Assigned(FAfterEdit) then
    FAfterEdit(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoAfterInsert(DataSet: TDataSet);
begin
  if Assigned(FAfterInsert) then
    FAfterInsert(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoAfterPost(DataSet: TDataSet);
begin
  if Assigned(FAfterPost) then
    FAfterPost(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoAfterScroll(DataSet: TDataSet);
begin
  if Assigned(FAfterScroll) then
    FAfterScroll(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoBeforeCancel(DataSet: TDataSet);
begin
  if Assigned(FBeforeCancel) then
    FBeforeCancel(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoBeforeDelete(DataSet: TDataSet);
begin
  if Assigned(FBeforeDelete) then
    FBeforeDelete(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoBeforeEdit(DataSet: TDataSet);
begin
  if Assigned(FBeforeEdit) then
    FBeforeEdit(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoBeforeInsert(DataSet: TDataSet);
begin
  if Assigned(FBeforeInsert) then
    FBeforeInsert(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoBeforePost(DataSet: TDataSet);
begin
  if Assigned(FBeforePost) then
    FBeforePost(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoBeforeScroll(DataSet: TDataSet);
begin
  if Assigned(FBeforeScroll) then
    FBeforeScroll(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoNewRecord(DataSet: TDataSet);
begin
  if Assigned(FOnNewRecord) then
    FOnNewRecord(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoAfterRefresh(DataSet: TDataSet);
begin
  if Assigned(FAfterRefresh) then
    FAfterRefresh(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoBeforeRefresh(DataSet: TDataSet);
begin
  if Assigned(FBeforeRefresh) then
    FBeforeRefresh(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoOnCustomCalcFields(DataSet: TDataSet);
begin
  if Assigned(FOnCustomCalcFields) then
    FOnCustomCalcFields(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoBeforeOpen(DataSet: TDataSet);
begin
  if Assigned(FBeforeOpen) then
    FBeforeOpen(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoBeforeClose(DataSet: TDataSet);
begin
  if Assigned(FBeforeClose) then
    FBeforeClose(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

procedure TovDataObjects.DoAfterOpen(DataSet: TDataSet);
begin
  if Assigned(FAfterOpen) then
    FAfterOpen(Self, GetObjectName(DataSet), TovDataSet(DataSet))
end;

{ ============================================================================ }
{    TDataSource                                   }
{ ============================================================================ }

procedure TovDataObjects.DoOnDataChange(Sender: TObject; Field: TField);
var
  DataSet: TDataSet;
begin
  DataSet := TDataSource(Sender).DataSet;
  if Assigned(FOnDataChange) and Assigned(Field) then
    FOnDataChange(Self, GetObjectName(DataSet), Field)
end;

procedure TovDataObjects.DoOnUpdateData(Sender: TObject);
var
  DataSet: TDataSet;
begin
  DataSet := TDataSource(Sender).DataSet;
  if Assigned(FOnUpdateData) then
    FOnUpdateData(Self, GetObjectName(DataSet), DataSet as TovDataSet)
end;

procedure TovDataObjects.DoOnStateChange(Sender: TObject);
var
  DataSet: TDataSet;
begin
  DataSet := TDataSource(Sender).DataSet;
  if Assigned(FOnStateChange) then
    FOnStateChange(Self, GetObjectName(DataSet), DataSet as TovDataSet)
end;

{ ============================================================================ }
{    TovDataSet                                    }
{ ============================================================================ }

procedure TovDataObjects.DoOnCreateObject(Sender: TObject; AObject: IovObject;
  var Cancel: Boolean);
begin
  if Assigned(FOnCreateObject) then
    FOnCreateObject(Self, GetObjectName(TDataSet(Sender)), AObject, Cancel)
end;

procedure TovDataObjects.DoOnAddObject(Sender: TObject; var NewObject: TOID;
  var Cancel: Boolean);
begin
  if Assigned(FOnAddObject) then
    FOnAddObject(Self, GetObjectName(TDataSet(Sender)), NewObject, Cancel)
end;

procedure TovDataObjects.DoOnDeleteObject(Sender: TObject; AObject: IovObject);
begin
  if Assigned(FOnDeleteObject) then
    FOnDeleteObject(Self, GetObjectName(TDataSet(Sender)), AObject)
end;

procedure TovDataObjects.DoOnRemoveObject(Sender: TObject; AObject: IovObject;
  var ACancel, ADeleteObject: Boolean);
begin
  if Assigned(FOnRemoveObject) then
    FOnRemoveObject(Self, GetObjectName(TDataSet(Sender)), AObject, ACancel, ADeleteObject)
end;




end.
