unit ovDataSet;

interface

uses SysUtils, Classes, Contnrs, DB, DBCtrls,
     ovConsts, ovTypes, ovVirtualDataSet, ovModel, ovDatabase, ovComponents,
     ovIObjects, ovQueryInfo;

type
  TovDataSetOption = (dsoExposeIdField, dsoExposeObjectIdField,
    dsoExposeObjectIdFields,
    dsoShowClassNames, dsoCreateChildObjects, dsoDeleteChildObjects);
  TovDataSetOptions = set of TovDataSetOption;

  TCreateObjectEvent = procedure (Sender: TObject; AObject: IovObject;
    var Cancel: Boolean) of object;
  TDeleteObjectEvent = procedure (Sender: TObject; AObject: IovObject) of object;
  TAddObjectEvent = procedure (Sender: TObject; var NewObject: TOID;
    var Cancel: Boolean) of object;
  TRemoveObjectEvent = procedure (Sender: TObject; AObject: IovObject;
    var ACancel, ADeleteObject: Boolean) of object;

  TDataSetFieldInfo = class
  private
    FField: TField;
    FFieldName: String;
    FObjectName: String;
    FCalculated: Boolean;
    FFieldInfo: TFieldInfo;
    FModelObject: TModelObject;
    FSourceFieldName: String;
    function FieldIsAttribute(
      var AModelAttribute: TModelAttribute): Boolean;
    function FieldIsReference(var AModelRelation: TModelRelation): Boolean;
  public
    constructor Create(const AFieldName: String; AFieldInfo: TFieldInfo;
      AModelObject: TModelObject; AObjectName: String; ACalculated: Boolean =
      False);
    constructor CreateSimple(const AFieldName, ASourceFieldName: String);
    property Field: TField read FField;
    property FieldName: String read FFieldName;
    property ObjectName: String read FObjectName;
    property Calculated: Boolean read FCalculated;
    property FieldInfo: TFieldInfo read FFieldInfo;
    property ModelObject: TModelObject read FModelObject;
    property SourceFieldName: String read FSourceFieldName;
  end;

  TovDataSet = class(TovCustomVirtualDataSet, ITableChangeListener)
  private
    FModel: TovModel;
    FModelName: String;
    FAutoOpen: Boolean;
    FMasterDataSource: TDataSource;
    FMasterDataSet: TovDataSet;
    FAttributes: TObjectList;
    FMasterObjectName: String;
    FMasterRelationName: String;
    FIsRefreshing: Boolean;
    FQueryInfo: TSelectQueryInfo;
    FAutoRefresh: Boolean;
    FUpdateLockCount: Integer;
    FInPost: Boolean;
    FOptions: TovDataSetOptions;
    FIdFieldName: String;
    FOnAddObject: TAddObjectEvent;
    FOnCreateObject: TCreateObjectEvent;
    FOnDeleteObject: TDeleteObjectEvent;
    FOnRemoveObject: TRemoveObjectEvent;
    FMasterObjectId: TOID;
    FNameSpace: String;
    FRecordIdFieldName: String;
    FObjectIdFieldName: String;
    FClassIdFieldName: String;
    FMasterLink: TMasterDataLink;
    FReadOnlyMode: Boolean;
    FRefreshPending: Boolean;
    FParams: TStrings;
    FMasterFieldName: String;
    FFieldInfoList: TObjectList;
    FDeleteMode: TDeleteMode;
    procedure TableChanged(ATableName: String);
    procedure SetModel(Value: TovModel);
    procedure SetModelName(const Value: String);
    function IsModelStored: Boolean;
    function GetDatabase: TovDatabase;
    procedure SetMasterDataSet(Value: TovDataSet);
    function GetCurrentRecordId: TOID;
    function GetObjectCount: Integer;
    function GetObjectNames(Index: Integer): String;
    function GetObjects(Index: Integer): IovObject;
    procedure SetMasterObjectName(const Value: String);
    procedure SetMasterRelationName(const Value: String);
    function GetMasterRelation: TModelRelation;
    procedure SetOptions(Value: TovDataSetOptions);
    procedure SetIdFieldName(const Value: String);
    function IsIdFieldNameStored: Boolean;
    function GetObjectClasses(Index: Integer): String;
    procedure SetMasterObjectId(Value: TOID);
    procedure SetNameSpace(Value: String);
    function GetCurrentClassId: TOID;
    function GetCurrentObject: IovObject;
    function GetCurrentObjectId: TOID;
    procedure CreateVirtualFields;
    function GetMasterObjectName: String;
    function GetMasterFieldName: String;
    procedure MasterChanged(Sender: TObject);
    procedure MasterDisabled(Sender: TObject);
    procedure InitFields;
    procedure KeyDataSetAfterOpen(ADataSet: TDataSet);
    function GetParamByName(ParamName: String): String;
    procedure SetParamByName(ParamName: String; const Value: String);
    procedure SetParams(Value: TStrings);
    procedure ParamsChange(Sender: TObject);
    procedure RefreshKeyDataSet;
    function GetKeyDataSetField(AFieldName: String): String;
    function GetFieldInfoCount: Integer;
    function GetFieldInfoListItem(Index: Integer): TDataSetFieldInfo;
    procedure BindInfoListFields;
    function GetFieldInfo(AFieldName: String): TDataSetFieldInfo;
    procedure AddFieldInfo(const AFieldName: String; AFieldInfo: TFieldInfo;
      AModelObject: TModelObject; AObjectName: String; ACalculated: Boolean =
      False);
    procedure AddSimpleFieldInfo(const AFieldName, ASourceFieldName: String);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function GetIsMasterDetail: Boolean;
    procedure Loaded; override;
    procedure InternalOpen; override;
    function OpenDataSet: TDataSet; virtual; abstract;
    procedure InternalRefresh; override;
    procedure DoAfterOpen; override;
    procedure InternalClose; override;
    function CurrentObjectClassName: String; dynamic;
    function GetKeyDataSetFieldLocateFor(AFieldName: String): String; override;
    function LocateRecord(const KeyFields: string; const KeyValues: Variant; 
      Options: TLocateOptions; SyncCursor: Boolean): Boolean; override;
    function GetCanModify: Boolean; override;
    procedure Idle; virtual;
    procedure DoAfterClose; override;
    function SetParamValues(const Expresssion: String): String;
    function SetMasterField(const Expresssion: String): String;
    procedure DoOnCalcFields; override;
    procedure CopyFieldData(const FieldName: String; var Buffer: PChar;
      SourceField: TField); overload; override;
    procedure GetFieldDef(FieldName: String; FieldDef: TFieldDef; var FieldType:
      TFieldType; var FieldSize: Integer; var Required: Boolean); override;
    procedure UpdateRecordData(Buffer: PChar); override;
    procedure InternalPost; override;
    procedure InsertRecordInternal; virtual; abstract;
    property Database: TovDatabase read GetDatabase;
    {{
         KeyDataSet,   
    }
    property RecordIdFieldName: String read FRecordIdFieldName;
    {{
         KeyDataSet,   
    }
    property ClassIdFieldName: String read FClassIdFieldName;
    {{
         KeyDataSet,   
    }
    property ObjectIdFieldName: String read FObjectIdFieldName;
    property FieldInfoCount: Integer read GetFieldInfoCount;
    property FieldInfo[Index: Integer]: TDataSetFieldInfo read GetFieldInfoListItem;
  public
    // class methods
    class function DataSetCount: Integer;
    class function GetDataSet(Index: Integer): TovDataSet;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DeleteAll;
    function FindObject(AObjectId: TOID): Boolean;
    procedure BeginUpdate;
    procedure EndUpdate;
    function IsUpdating: Boolean;
    function ObjectByName(ObjectName: String): IovObject;
    function FindRecord(ARecordId: Integer): Boolean;
    procedure LocateObject(AObject: IovObject);
    function GetObjectClass(ObjectName: String): String; virtual;
    function ObjectIdByName(ObjectName: String): TOID;
    function IsCurrentObjectAvailable: Boolean;
    property IsMasterDetail: Boolean read GetIsMasterDetail;
    property CurrentRecordId: TOID read GetCurrentRecordId;
    property ObjectCount: Integer read GetObjectCount;
    property ObjectNames[Index: Integer]: String read GetObjectNames;
    property Objects[Index: Integer]: IovObject read GetObjects;
    property MasterRelation: TModelRelation read GetMasterRelation;
    property ObjectClasses[Index: Integer]: String read GetObjectClasses;
    {{
           Master-Detail,
             
     MasterObjectId.   MasterDataSet     .
    MasterDataSet     -     
      .
    }
    property MasterObjectId: TOID read FMasterObjectId write SetMasterObjectId;
    {{
          TovQuery    
    MasterDataSet,     Master-Detail.  
            .
    }
    property MasterFieldName: String read FMasterFieldName write FMasterFieldName;
    property CurrentClassId: TOID read GetCurrentClassId;
    property CurrentObject: IovObject read GetCurrentObject;
    property CurrentObjectId: TOID read GetCurrentObjectId;
    property Params: TStrings read FParams write SetParams;
    property ParamByName[ParamName: String]: String read GetParamByName write 
      SetParamByName;
  published
    //    TDataSet
    property Active;
    property AutoCalcFields;
    property BeforeOpen;
    property AfterOpen;
    property BeforeClose;
    property AfterClose;
    property BeforeInsert;
    property AfterInsert;
    property BeforeEdit;
    property AfterEdit;
    property BeforePost;
    property AfterPost;
    property BeforeCancel;
    property AfterCancel;
    property BeforeDelete;
    property AfterDelete;
    property BeforeScroll;
    property AfterScroll;
    property BeforeRefresh;
    property AfterRefresh;
    property OnCalcFields;
    property OnDeleteError;
    property OnEditError;
    property OnNewRecord;
    property OnPostError;
    property OnFilterRecord;

    //    TovCustomVirtualDataSet
    property ReadOnly default False;
    property OnCustomCalcFields;

    //    
    property AutoRefresh: Boolean read FAutoRefresh write FAutoRefresh default True;
    property Model: TovModel read FModel write SetModel stored IsModelStored;
    property ModelName: String read FModelName write SetModelName;
    property AutoOpen: Boolean read FAutoOpen write FAutoOpen default False;
    property MasterDataSet: TovDataSet read FMasterDataSet write SetMasterDataSet;
    {{
        MasterDataSet,      
     Master-Detail ( ,   MasterDataSet 
     ).
    }
    property MasterObjectName: String read FMasterObjectName write
      SetMasterObjectName;
    {{
         MasterObjectName  MasterDataSet, 
         Master-Detail.
    }
    property MasterRelationName: String read FMasterRelationName write
      SetMasterRelationName;
    property Options: TovDataSetOptions read FOptions write SetOptions default [];
    property IdFieldName: String read FIdFieldName write SetIdFieldName stored 
      IsIdFieldNameStored;

    property OnAddObject: TAddObjectEvent read FOnAddObject write FOnAddObject;
    property OnCreateObject: TCreateObjectEvent read FOnCreateObject write 
      FOnCreateObject;
    property OnDeleteObject: TDeleteObjectEvent read FOnDeleteObject write 
      FOnDeleteObject;
    property OnRemoveObject: TRemoveObjectEvent read FOnRemoveObject write
      FOnRemoveObject;
    property NameSpace: String read FNameSpace write SetNameSpace;
    property DeleteMode: TDeleteMode read FDeleteMode write FDeleteMode default
      dmCheckRefs;
  end;

  TovDataSetClass = class of TovDataSet;

  TovExposer = class(TovDataSet)
  private
    FObjectFilter: String;
    FObjectAttributes: String;
    FObjectClassName: String;
    FObjectOrder: String;
    FNewObject: TOID;
    FJoinExpression: String;
    procedure SetObjectFilter(Value: String);
    procedure SetObjectAttributes(const Value: String);
    procedure SetObjectClassName(Value: String);
    procedure SetObjectOrder(Value: String);
    function GetModelObject: TModelObject;
    function GetMasterObject: IovObject;
    function GetObjectClassName: String; overload;
    procedure SetJoinExpression(Value: String);
    procedure InternalDelete2;
  protected
    procedure InternalDelete; override;
    procedure DoOnNewRecord; override;
    function OpenDataSet: TDataSet; override;
    function CurrentObjectClassName: String; override;
    procedure InternalCancel; override;
  public
    function NewObject: IovObject;
    procedure DeleteAllObjects;
    procedure DeleteCurrentObject;
    procedure RemoveAllObjects;
    procedure RemoveCurrentObject;
    procedure AddObject(Id: TOID); overload;
    constructor Create(AOwner: TComponent); override;
    function CreateObject: TOID;
    procedure AddObject(AObjectId: IovObject); overload;
    procedure ExposeObject(AObject: IovObject; Attributes: String = '[*]');
    property MasterObject: IovObject read GetMasterObject;
    property ModelObject: TModelObject read GetModelObject;
  published
    property ObjectClassName: String read GetObjectClassName write
      SetObjectClassName;
    property ObjectAttributes: String read FObjectAttributes write
      SetObjectAttributes;
    property ObjectFilter: String read FObjectFilter write SetObjectFilter;
    property ObjectOrder: String read FObjectOrder write SetObjectOrder;
    property JoinExpression: String read FJoinExpression write SetJoinExpression;
  end;

  TovQuery = class(TovDataSet)
  private
    FQuery: TStrings;
    procedure SetQuery(Value: TStrings);
    procedure QueryChange(Sender: TObject);
  protected
    function OpenDataSet: TDataSet; override;
    function CurrentObjectClassName: String; override;
    procedure InternalDelete; override;
    procedure InsertRecordInternal; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Query: TStrings read FQuery write SetQuery;
    property MasterFieldName;
  end;

  ERecordUnavailable = class (Exception)
  end;

implementation

uses Variants, AppEvnts, Forms, ovUtils;

resourcestring
  DefaultIdFieldName = 'Id';

constructor TDataSetFieldInfo.CreateSimple(const AFieldName, ASourceFieldName:
   String);
begin
  inherited Create;
  FFieldName := AFieldName;
  FSourceFieldName := ASourceFieldName;
end;

constructor TDataSetFieldInfo.Create(const AFieldName: String; AFieldInfo:
  TFieldInfo; AModelObject: TModelObject; AObjectName: String; ACalculated:
  Boolean = False);
begin
  inherited Create;
  FObjectName := AObjectName;
  FFieldName := AFieldName;
  FFieldInfo := AFieldInfo;
  FCalculated := ACalculated;
  FModelObject := AModelObject;
  if Assigned(FFieldInfo) and Assigned(FFieldInfo.Field) then
    FSourceFieldName := FFieldInfo.Field.FieldName
end;

function TDataSetFieldInfo.FieldIsAttribute(var AModelAttribute:
  TModelAttribute): Boolean;
begin
  Result := FieldInfo is TAttributeField;
  if Result then AModelAttribute := TAttributeField(FieldInfo).ModelAttribute
end;

function TDataSetFieldInfo.FieldIsReference(var AModelRelation:
  TModelRelation): Boolean;
begin
  Result := FieldInfo is TReferenceField;
  if Result then AModelRelation := TReferenceField(FieldInfo).ModelRelation
end;

var
  DataSets: TComponentList; //     TovDataSet

{ ============================================================================ }
{ TovDataSet                                                                   }
{ ============================================================================ }

constructor TovDataSet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAutoRefresh := True;
  FIdFieldName := DefaultIdFieldName;
  FDeleteMode := dmCheckRefs;
  FAttributes := TObjectList.Create(True);

  FParams := TStringList.Create;
  TStringList(FParams).OnChange := ParamsChange;

  FMasterLink := TMasterDataLink.Create(Self);
  FMasterLink.OnMasterChange := MasterChanged;
  FMasterLink.OnMasterDisable := MasterDisabled;

  FFieldInfoList := TObjectList.Create;

  ovDataSet.DataSets.Add(Self);
end;

destructor TovDataSet.Destroy;
begin
  if Assigned(Database) then
    Database.UnRegisterTableChangeListener(Self);

  inherited Destroy;
  FreeAndNil(FFieldInfoList);
  FParams.Free;
  FAttributes.Free;
  FMasterLink.Free;
end;

class function TovDataSet.DataSetCount: Integer;
begin
  if ovDataSet.DataSets <> nil then
    Result := ovDataSet.DataSets.Count
  else
    Result := 0
end;

class function TovDataSet.GetDataSet(Index: Integer): TovDataSet;
begin
  Result := TovDataSet(ovDataSet.DataSets[Index]);
end;

procedure TovDataSet.SetModel(Value: TovModel);
begin
  if FModel <> Value then
  begin
    Close;
    FModel := Value;
    if Value <> nil then begin
      Value.FreeNotification(Self);
      FModelName := Value.ModelName;
    end else
      FModelName := '';
  end
end;

function TovDataSet.IsModelStored: Boolean;
begin
  Result := ModelName = '';
end;

procedure TovDataSet.SetModelName(const Value: String);
begin
  if FModelName <> Value then begin
    Close;
    FModelName := Value;
    FModel := FindModel(FModelName);
  end
end;

procedure TovDataSet.SetMasterDataSet(Value: TovDataSet);
begin
  if FMasterDataSet <> Value then begin
    Close;
    FMasterDataSet := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end
end;

procedure TovDataSet.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = Model) and (Operation = opRemove) then Model := nil else
  if (AComponent = MasterDataSet) and (Operation = opRemove) then MasterDataSet := nil;
end;

function TovDataSet.GetDatabase: TovDatabase;
begin
  if Model <> nil
    then Result := Model.Database
    else Result := nil
end;

procedure TovDataSet.Loaded;
begin
  inherited Loaded;
  if AutoOpen and Not (csDesigning in ComponentState) then Active := True;
end;

procedure TovDataSet.SetMasterObjectName(const Value: String);
begin
  if FMasterObjectName <> Value then begin
    Close;
    FMasterObjectName := Value;
  end
end;

function TovDataSet.GetMasterObjectName: String;
begin
  if MasterObjectName <> '' then
    Result := MasterObjectName
  else
    if Assigned(MasterDataSet) then
      Result := MasterDataSet.ObjectNames[0]
    else
      raise Exception.Create('   MasterDataSet')
end;

procedure TovDataSet.SetMasterRelationName(const Value: String);
begin
  if FMasterRelationName <> Value then begin
    Close;
    FMasterRelationName := Value;
  end
end;

procedure TovDataSet.SetOptions(Value: TovDataSetOptions);
begin
  if FOptions <> Value then begin
    Close;
    FOptions  := Value;
  end
end;

procedure TovDataSet.SetIdFieldName(const Value: String);
begin
  if FIdFieldName <> Value then begin
    Close;
    FIdFieldName  := Value;
  end
end;

function TovDataSet.IsIdFieldNameStored: Boolean;
begin
  Result := FIdFieldName <> DefaultIdFieldName;
end;

{  Master-Detail                                                          }
{ ============================================================================ }

function TovDataSet.GetIsMasterDetail: Boolean;
begin
  Result := Assigned(MasterDataSet);
end;

procedure TovDataSet.MasterChanged(Sender: TObject);
begin
  //  MasterObjectId <> 0,       MasterDataSet
  if Active and (MasterObjectId = 0) and Not FInPost then
  begin
    Resync([]);
    DoAfterScroll;
  end;
end;

procedure TovDataSet.MasterDisabled(Sender: TObject);
begin

end;

function TovDataSet.GetMasterRelation: TModelRelation;
begin
  Result := nil;
  if Assigned(Model) and Assigned(MasterDataSet) and MasterDataSet.Active and
    (MasterRelationName <> '')
  then
    Result := Model.DomainModel.GetObjectRelation(
      MasterDataSet.GetObjectClass(GetMasterObjectName), MasterRelationName);
end;

{{
        MasterDataSet.KeyDataSet
(..        master-detail),
       .
}
function TovDataSet.GetMasterFieldName: String;
var
  ObjectInfo: TObjectInfo;
  Relation: TModelRelation;
  AncestorName: String;
begin
  if MasterFieldName <> ''
  then
    //    MasterFieldName
    Result := MasterDataSet.GetKeyDataSetField(MasterFieldName)
  else
    //    MasterRelationName
  with MasterDataSet do
  if Assigned(Model) then begin

    if Self.GetMasterObjectName = '' then
      ObjectInfo := FQueryInfo.Objects[0]
    else
      ObjectInfo := FQueryInfo.GetObject(Self.GetMasterObjectName);

    Relation := Model.DomainModel.GetObjectRelation(
      ObjectInfo.ModelObject.FullName, Self.MasterRelationName);
    //   ,    (  
    //   )   
    if Relation.IsInherited then begin
      AncestorName := Relation.ParentRelation.SourceObjectName;
      ObjectInfo := ObjectInfo.GetAncestor(AncestorName);
    end;

    if Relation.IsReference then
      Result := ObjectInfo.GetField(Relation.FieldName).Field.FieldName
    else
      Result := ObjectInfo.GetField(TObjectIdField).Field.FieldName

  end else
      raise Exception.Create('  MasterFieldName')
end;

{{
      :MasterFieldName   
   MasterDataSet.KeyDataSet,   MasterFieldName.
}
function TovDataSet.SetMasterField(const Expresssion: String): String;
begin
  Result := StringReplace(Expresssion, ':'+MasterFieldName,
    ':'+MasterDataSet.GetKeyDataSetField(MasterFieldName), [rfReplaceAll])
end;

{ ============================================================================ }
{                                                        }
{ ============================================================================ }

{{
     .
}
function TovDataSet.GetCurrentRecordId: TOID;
begin
  if (RecordIdFieldName <> '') and Assigned(KeyDataSet) then
    Result := KeyDataSet.FieldByName(RecordIdFieldName).AsInteger
  else
    raise Exception.Create('    :' + Name);
end;

{{
    .
}
function TovDataSet.GetCurrentObjectId: TOID;
begin
  if (ObjectIdFieldName <> '') and Assigned(KeyDataSet) then
    Result := KeyDataSet.FieldByName(ObjectIdFieldName).AsInteger
  else
    raise Exception.Create('    :' + Name);
end;

{{
     .
}
function TovDataSet.GetCurrentClassId: TOID;
begin
  if (ClassIdFieldName <> '') and Assigned(KeyDataSet) then
    Result := KeyDataSet.FieldByName(ClassIdFieldName).AsInteger
  else
    raise Exception.Create('    :' + Name);
end;

{{
    .
}
function TovDataSet.GetCurrentObject: IovObject;
begin
  if (CurrentObjectClassName <> '') and (ObjectIdFieldName <> '') and Assigned(KeyDataSet)
  then
    if Not VarIsNull(KeyDataSet[ObjectIdFieldName]) then
      Result := Database.Classes[CurrentObjectClassName].GetRealObject(KeyDataSet[ObjectIdFieldName])
    else
      Result := nil
  else
    raise Exception.Create('    :' + Name);
end;

{{
     .    
   ,       .
}
function TovDataSet.IsCurrentObjectAvailable: Boolean;
begin
  Result := (CurrentObjectClassName <> '') and (ObjectIdFieldName <> '') and
    Assigned(KeyDataSet) and Not VarIsNull(KeyDataSet[ObjectIdFieldName]);
end;

function TovDataSet.FindRecord(ARecordId: Integer): Boolean;
begin
  if RecordIdFieldName = '' then raise Exception.Create('   RecordIdFieldName');

  Result := KeyDataSet.Locate(RecordIdFieldName, ARecordId, []);
  if Result then
  begin
    Resync([]);
    DoAfterScroll;
  end;
end;

function TovDataSet.FindObject(AObjectId: TOID): Boolean;
begin
  if ObjectIdFieldName = '' then raise Exception.Create('   ObjectIdFieldName');

  Result := KeyDataSet.Locate(ObjectIdFieldName, AObjectId, []);
  if Result then
  begin
    Resync([]);
    DoAfterScroll;
  end;
end;

procedure TovDataSet.LocateObject(AObject: IovObject);
begin
{ TODO :
  if Not AObject.IsClassOf(CurrentClassId) then
    raise Exception.Create('  ')
}
  if Not FindObject(AObject.ObjectId) then
    raise Exception.Create('  ')
end;

{ ============================================================================ }
{                                                      }
{ ============================================================================ }

function TovDataSet.GetObjects(Index: Integer): IovObject;
begin
  if Assigned(FQueryInfo) then
    with FQueryInfo.Objects[Index] do
      Result := Database.Classes[ModelObject.Name].GetObject(
        GetField(TObjectIdField).Field.AsInteger)
  else
    raise Exception.Create('  ')
end;

function TovDataSet.GetObjectCount: Integer;
begin
  if Assigned(FQueryInfo) then
    Result := FQueryInfo.ObjectCount
  else
    Result := 0
end;

function TovDataSet.GetObjectNames(Index: Integer): String;
begin
  if Assigned(FQueryInfo) then
    Result := FQueryInfo.Objects[Index].Name
  else
    raise Exception.Create('  ')
end;

function TovDataSet.GetObjectClasses(Index: Integer): String;
begin
  if Assigned(FQueryInfo) then
    Result := FQueryInfo.Objects[Index].ModelObject.Name
  else
    raise Exception.Create('  ')
end;

function TovDataSet.ObjectByName(ObjectName: String): IovObject;
begin
  if Assigned(FQueryInfo) then
    with FQueryInfo.GetObject(ObjectName) do
      Result := Database.GetObject(ModelObject, GetField(TObjectIdField).Field.AsInteger)
  else
    raise Exception.Create('  ')
end;

{{
  ObjectByName,    ,    ,
       ,     .

   ,     .
}
function TovDataSet.ObjectIdByName(ObjectName: String): TOID;
begin
  if Assigned(FQueryInfo) then
    with FQueryInfo.GetObject2(ObjectName) do
      Result := GetField(TObjectIdField).Field.AsInteger
  else
    raise Exception.Create('  ')
end;

function TovDataSet.GetObjectClass(ObjectName: String): String;
begin
  if Assigned(FQueryInfo) then
    Result := FQueryInfo.GetObject(ObjectName).ModelObject.FullName
  else
    raise Exception.Create('  ')
end;

{ ============================================================================ }
{                                       }
{ ============================================================================ }

function TovDataSet.GetFieldInfoCount: Integer;
begin
  Result := FFieldInfoList.Count;
end;

function TovDataSet.GetFieldInfoListItem(Index: Integer): TDataSetFieldInfo;
begin
  Result := TDataSetFieldInfo(FFieldInfoList[Index])
end;

procedure TovDataSet.AddSimpleFieldInfo(const AFieldName, ASourceFieldName:
   String);
begin
  FFieldInfoList.Add(TDataSetFieldInfo.CreateSimple(AFieldName, ASourceFieldName));
end;

procedure TovDataSet.AddFieldInfo(const AFieldName: String; AFieldInfo:
  TFieldInfo; AModelObject: TModelObject; AObjectName: String; ACalculated:
  Boolean = False);
begin
  FFieldInfoList.Add(TDataSetFieldInfo.Create(AFieldName, AFieldInfo, AModelObject,
    AObjectName, ACalculated));
end;

{{
   FFieldInfoList    .
}
procedure TovDataSet.BindInfoListFields;
var
  I: Integer;
begin
  for I := 0 to FieldInfoCount - 1 do
  {           FieldInfo  
       - - ,     persistent-
       }
  with FieldInfo[I] do
    FField := FindField(FieldName);
end;

{{
  FieldInfo,   AFieldName  
 .
}
function TovDataSet.GetFieldInfo(AFieldName: String): TDataSetFieldInfo;
var
  I: Integer;
begin
  for I := 0 to FieldInfoCount - 1 do
    if FieldInfo[I].FieldName = AFieldName then
    begin
      Result := FieldInfo[I];
      Exit;
    end;
  Result := nil
end;

{{
     KeyDataSet,   AFieldName
   .
}
function TovDataSet.GetKeyDataSetField(AFieldName: String): String;
var
  AFieldInfo: TDataSetFieldInfo;
begin
  AFieldInfo := GetFieldInfo(AFieldName);
  if AFieldInfo <> nil then
    Result := AFieldInfo.SourceFieldName
  else
    Result := ''
end;

{ ============================================================================ }
{                                      }
{ ============================================================================ }

procedure TovDataSet.InternalOpen;
begin
  if Not Assigned(FModel) and (ModelName <> '') then FModel := FindModel(FModelName);
  if Not Assigned(FModel) then raise Exception.Create('    ' + Name);
  Model.Open;

  if IsMasterDetail then
  begin
    MasterDataSet.Open;
    FMasterDataSource := TDataSource.Create(Self);
    FMasterDataSource.DataSet := MasterDataSet.KeyDataSet;
    FMasterLink.FieldNames := '';
    FMasterLink.DataSource := FMasterDataSource;
  end;

  KeyDataSet := OpenDataSet;
  KeyDataSet.AfterOpen := KeyDataSetAfterOpen;
  //   ,       
  KeyDataSetAfterOpen(KeyDataSet);

  //    ReadOnly  KeyDataSet   
  // ,    
  FReadOnlyMode := Not Assigned(FQueryInfo.GetField(TObjectIdField, False)) or ReadOnly;

  CreateVirtualFields;

  inherited;  //    

  InitFields;
  if IsMasterDetail then FMasterLink.FieldNames := GetMasterFieldName;
end;

{{
     KeyDataSet   
( )        
(,    TQuery).
}
procedure TovDataSet.KeyDataSetAfterOpen(ADataSet: TDataSet);
begin
  FQueryInfo.BindFields(KeyDataSet);
  InitFields;
end;

procedure TovDataSet.InitFields;
var
  AFieldInfo: TFieldInfo;
begin
  with FQueryInfo do
  begin
    //  FObjectIdFieldName
    AFieldInfo := Objects[0].GetField(TObjectIdField, False);
    if Assigned(AFieldInfo) then
      FObjectIdFieldName := AFieldInfo.Field.FieldName
    else
      FObjectIdFieldName := '';

    //  FClassIdFieldName
    AFieldInfo := Objects[0].GetField(TClassIdField, False);
    if Assigned(AFieldInfo) then
      FClassIdFieldName := AFieldInfo.Field.FieldName
    else
      FClassIdFieldName := '';

    //  FRecordIdFieldName
    if Not IsMasterDetail or (MasterFieldName <> '') then
    begin
      FRecordIdFieldName := FObjectIdFieldName;
      AddSimpleFieldInfo(IdFieldName, FRecordIdFieldName);
    end
    else begin
      AFieldInfo := GetField(TRelationIdField, False);
      if Assigned(AFieldInfo) then
      begin
        FRecordIdFieldName := AFieldInfo.Field.FieldName;
        AddSimpleFieldInfo(IdFieldName, FRecordIdFieldName);
      end
      else
        //  TRelationIdField  , ,     
        FRecordIdFieldName := '';
    end;
    KeyDataSetKeyFieldName := FRecordIdFieldName;
  end;

  BindInfoListFields;
end;

{{
    ,   AddDataSet 
 .
}
procedure TovDataSet.CreateVirtualFields;
var
  AFields, AFieldAliases: TStringList;

  procedure PrepareFieldAliases(AFieldAliases: TStrings);
  var
    I: Integer;
  begin
    //     ,      /
    if Not (dsoShowClassNames in Options) then
      for I := 0 to AFieldAliases.Count - 1 do
        AFieldAliases[I] := ExtractLastName(AFieldAliases[I]);
  end;

  procedure ExposeObjectIds(AObjectInfo: TObjectInfo);
  var
    I: Integer;
    ThisDataSetField: String;
    KeyDataSetField: String;
  begin
    ThisDataSetField := ExtractObjectName(AObjectInfo.Name)+'_OID';
    KeyDataSetField := AObjectInfo.GetField(TObjectIdField).Field.FieldName;
    AFields.Add(KeyDataSetField);
    AFieldAliases.Add(ThisDataSetField);
    AddSimpleFieldInfo(ThisDataSetField, KeyDataSetField);
    for I := 0 to AObjectInfo.ChildCount - 1 do
      ExposeObjectIds(AObjectInfo.Children[I]);
  end;

  procedure DoCreateFields;
  var
    I: Integer;
    AFieldName: String;
    AFieldType: TFieldType;
    AFieldSize: Integer;
  begin
    if (dsoExposeIdField in Options) and (RecordIdFieldName <> '') then
    begin //       
      AFields.Add(RecordIdFieldName);
      AFieldAliases.Add(IdFieldName);
    end;

    if dsoExposeObjectIdField in Options then
    begin //       
      AFields.Add(ObjectIdFieldName);
      AFieldAliases.Add('OID');
      AddSimpleFieldInfo('OID', ObjectIdFieldName);
    end;

    if dsoExposeObjectIdFields in Options then
      for I := 0 to FQueryInfo.ObjectCount - 1 do
        ExposeObjectIds(FQueryInfo.Objects[I]);

    with FQueryInfo do
    for I := 0 to FieldCount - 1 do

      if Fields[I] is TExpressionField then
      with TExpressionField(Fields[I]) do
      begin
        AFields.Add(Field.FieldName);
        AFieldAliases.Add(Name);
      end else

      if (Fields[I] is TAttributeField) and Not Fields[I].AutoSelected then
      with TAttributeField(Fields[I]) do
      begin
        if ModelAttribute.Calculated and Not ModelAttribute.Persistent then
        begin
          ModelAttribute.GetFieldDef(AFieldType, AFieldSize);
          AFieldName := ModelAttribute.FieldName;
          AddCustomCalcField(ExtractLastName(Name), AFieldType, AFieldSize);
        end else
        begin
          AFields.Add(Field.FieldName);
          //        
          AFieldAliases.Add(Name);
        end;
        AddFieldInfo(ExtractLastName(Name), Fields[I], ObjectInfo.ModelObject,
          ObjectInfo.Path, ModelAttribute.Calculated and Not ModelAttribute.Persistent);
      end else

      if (Fields[I] is TReferenceField) and Not Fields[I].AutoSelected then
      with TReferenceField(Fields[I]) do
      begin
        AFields.Add(Field.FieldName);
        AFieldAliases.Add(Name);
        AddFieldInfo(ExtractLastName(Name), Fields[I], ObjectInfo.ModelObject,
          ObjectInfo.Path);
      end;

    if AFields.Count > 0 then
    begin
      PrepareFieldAliases(AFieldAliases);
      ExposeFields(AFields, AFieldAliases);
    end
  end;

begin
  AFields := TStringList.Create;
  AFieldAliases := TStringList.Create;
  try
    DoCreateFields;
  finally
    AFieldAliases.Free;
    AFields.Free;
  end;
end;

procedure TovDataSet.InternalClose;
var
  OldKeyDataSet: TDataSet;
  I: Integer;
begin
  for I := 0 to FieldInfoCount - 1 do
    if FieldInfo[I].Calculated then
      RemoveCustomCalcField(FieldInfo[I].FieldName);
  FFieldInfoList.Clear;

  FAttributes.Clear;

  FreeAndNil(FMasterDataSource);

  OldKeyDataSet := KeyDataSet;
  KeyDataSet := nil;
  OldKeyDataSet.Free;

  FreeAndNil(FQueryInfo);

  inherited;
end;

function TovDataSet.GetCanModify: Boolean;
begin
  Result := Not FReadOnlyMode and inherited GetCanModify;
end;

procedure TovDataSet.DoAfterOpen;
begin
  inherited;
  if Assigned(Database) then
    Database.RegisterTableChangeListener(Self);
end;

procedure TovDataSet.DoAfterClose;
begin
  if Assigned(Database) then
    Database.UnRegisterTableChangeListener(Self);
  inherited;
end;

{{
   ,      
  ,        (..
         )
}
procedure TovDataSet.GetFieldDef(FieldName: String; FieldDef: TFieldDef; var
  FieldType: TFieldType; var FieldSize: Integer; var Required: Boolean);
var
  AFieldInfo: TDataSetFieldInfo;
  AModelAttribute: TModelAttribute;
begin
  inherited;
  AFieldInfo := GetFieldInfo(FieldName);
  if Assigned(AFieldInfo) and AFieldInfo.FieldIsAttribute(AModelAttribute) and
     AModelAttribute.IsBoolean and Not Database.Adapter.SupportsBooleanType
  then FieldType := ftBoolean;
end;

procedure TovDataSet.CopyFieldData(const FieldName: String; var Buffer: PChar;
  SourceField: TField);
var
  AFieldInfo: TDataSetFieldInfo;
  P: ^ WordBool;
  AModelAttribute: TModelAttribute;
begin
  AFieldInfo := GetFieldInfo(FieldName);
  if Assigned(AFieldInfo) and AFieldInfo.FieldIsAttribute(AModelAttribute) and
     AModelAttribute.IsBoolean and Not Database.Adapter.SupportsBooleanType then
  begin
    if Not SourceField.IsNull
      then Buffer[0] := #1
      else Buffer[0] := #0; //   Null

    P := Pointer(Buffer+1);
    if SourceField.AsInteger <> 0 then
      P^ := True else P^ := False;
    Inc(Buffer, 3);
  end else
    inherited
end;

{ ============================================================================ }
{                                                        }
{ ============================================================================ }

procedure TovDataSet.RefreshKeyDataSet;
begin
  KeyDataSet.Close;
  KeyDataSet.Open;
end;

{{
   TDataSet.Refresh.    , .
      .
}
procedure TovDataSet.InternalRefresh;
var
  SaveRecordId: TOID;
begin
  //DisableControls;
  FIsRefreshing := True; //      
  //    ,  ,  Refresh   
  try

    if (RecordIdFieldName <> '')
      then SaveRecordId := KeyDataSet.FieldByName(RecordIdFieldName).AsInteger
      else SaveRecordId := 0;

    inherited;
    RefreshKeyDataSet;

    //    
    if SaveRecordId > 0 then
      KeyDataSet.Locate(RecordIdFieldName, SaveRecordId, []);

  finally
    //EnableControls;
    FIsRefreshing := False;
  end;
end;

procedure TovDataSet.BeginUpdate;
begin
  Inc(FUpdateLockCount);
end;

procedure TovDataSet.EndUpdate;
begin
  Dec(FUpdateLockCount);
end;

function TovDataSet.IsUpdating: Boolean;
begin
  Result := FUpdateLockCount > 0
end;

{{
 Refresh,    AutoRefresh 
         
.
}
procedure TovDataSet.TableChanged(ATableName: String);
begin
  if Active and AutoRefresh and Not FInPost and FQueryInfo.TableIsUsed(ATableName)
  then begin
    FRefreshPending := True;
    Exit;
  end
end;

procedure TovDataSet.Idle;
begin
  //  Refresh     
  if Active and FRefreshPending and (State = dsBrowse) then begin
    Refresh;
    FRefreshPending := False;
  end
end;

{ ============================================================================ }
{                                                    }
{ ============================================================================ }

{{
       Buffer  
 .  DataSetIndex  , ..  
  -  KeyDataSet.

:
-    KeyDataSet     
  ,    ,  :
        -       ,   
        -    ,   
        -    blob-,   blob-
-        
}
procedure TovDataSet.UpdateRecordData(Buffer: PChar);

  function GetObjectFromField(AFieldInfo: TDataSetFieldInfo): IovObject;
  var
    AObjectId: TOID;
  begin
    AObjectId := AFieldInfo.FieldInfo.ObjectInfo.GetField(TObjectIdField).Field.AsInteger;

    if AObjectId <> 0 then
      Result := Database.GetObject(AFieldInfo.ModelObject, AObjectId)
    else
      Result := nil
  end;

  procedure UpdateRecordDataInternal;
  var
    I: Integer;
    AObject: IovObject;
    ModelAttribute: TModelAttribute;
    ModelRelation: TModelRelation;
    AFieldInfo: TDataSetFieldInfo;
    BlobBuffer: PBlobFieldDataBuffer;
  begin
    for I := 0 to FieldCount - 1 do
    begin
      AFieldInfo := GetFieldInfo(Fields[I].FieldName);
      if AFieldInfo <> nil then
      begin
        if AFieldInfo.FieldIsAttribute(ModelAttribute) then
        begin
          AObject := GetObjectFromField(AFieldInfo);
          if Assigned(AObject) and Not ModelAttribute.Calculated then
            if Not Fields[I].IsBlob then
              AObject[ModelAttribute.Name].Value := Fields[I].Value
            else
            begin
              BlobBuffer := FBlobData.GetFieldBlobBuffer(AFieldInfo.Field.FieldName);
              if BlobBuffer <> nil then
                with AObject[ModelAttribute.Name].AsBlobStream do
                begin
                  GetStream(bmWrite).Write(BlobBuffer^.Data, BlobBuffer^.Size);
                  Post;
                end;
            end;
        end else
        if AFieldInfo.FieldIsReference(ModelRelation) then
        begin
          AObject := GetObjectFromField(AFieldInfo);
          if AObject <> nil then
            AObject[ModelRelation.Name].AsObjectId := Fields[I].AsInteger
        end
      end;
    end;
  end;

begin
  Database.BeginUpdateTables;
  try
    UpdateRecordDataInternal;
  finally
    Database.EndUpdateTables;
  end;
  inherited;
end;

procedure TovDataSet.InternalPost;
begin
  {        
    ITableChangeListener,   ,  
      Post.
  }
  FInPost := True;

  {          
    ,      
   ITableChangeListener,     
  Database.BeginUpdate  Database.EndUpdate.
  }
  Database.BeginUpdate;
  try
    UpdateRecordData(ActiveBuffer)
  finally
    Database.EndUpdate;
    FInPost := False;
  end;
end;

{ ============================================================================ }
{                                                                  }
{ ============================================================================ }

{{
       KeyDataSet,  
  ,   .  
   KeyDataSet,  ,     .
}
function TovDataSet.LocateRecord(const KeyFields: string; const KeyValues:
  Variant; Options: TLocateOptions; SyncCursor: Boolean): Boolean;
begin
  if CanModify then RefreshKeyDataSet;
  Result := inherited LocateRecord(KeyFields, KeyValues, Options, SyncCursor);
end;

{{
     KeyDataSet,  
 .      , ..
          
  KeyDataSet.
}
function TovDataSet.GetKeyDataSetFieldLocateFor(AFieldName: String): String;
begin
  Result := GetKeyDataSetField(AFieldName)
end;

{ ============================================================================ }
{                                                     }
{ ============================================================================ }

{{
     .     
IsMasterDetail         
(,    ,      
  ).
}
procedure TovDataSet.DeleteAll;
begin
  { TODO :    }
  DisableControls;
  try
    while Not IsEmpty do Delete;
  finally
    EnableControls
  end;
end;

procedure TovDataSet.SetMasterObjectId(Value: TOID);
begin
  if FMasterObjectId <> Value then
  begin
    Close;
    FMasterObjectId := Value;
  end;
end;

procedure TovDataSet.SetNameSpace(Value: String);
begin
  if FNameSpace <> Value then begin
    Close;
    FNameSpace := Value;
  end
end;

function TovDataSet.CurrentObjectClassName: String;
begin
  Result := '';
end;

{ ============================================================================ }
{                                                                     }
{ ============================================================================ }

function TovDataSet.GetParamByName(ParamName: String): String;
begin
  Result := FParams.Values[ParamName];
end;

procedure TovDataSet.SetParamByName(ParamName: String; const Value: String);
begin
  FParams.Values[ParamName] := Value;
end;

{{
       :   
.
}
function TovDataSet.SetParamValues(const Expresssion: String): String;
var
  I: Integer;
begin
  Result := Expresssion;
  with FParams do
  for I := 0 to Count - 1 do
    Result := StringReplace(Result, ':'+Names[I], Values[Names[I]], [rfReplaceAll])
end;

procedure TovDataSet.SetParams(Value: TStrings);
begin
  FParams.Assign(Value);
end;

procedure TovDataSet.ParamsChange(Sender: TObject);
begin
  Close;
end;

procedure TovDataSet.DoOnCalcFields;
var
  I: Integer;
  Value: Variant;
  ObjectId: TOID;
begin
  for I := 0 to FieldInfoCount - 1 do
  with FieldInfo[I] do
  if Calculated then
  begin
    ObjectId := ObjectIdByName(ObjectName);
    if ObjectId <> 0 then
    begin
      Database.CalculateAttribute(ObjectId, (FieldInfo as TAttributeField).ModelAttribute, Value);
      Field.Value := Value;
    end;
  end;

  inherited;
end;

{ ============================================================================ }
{ TovExposer                                                                   }
{ ============================================================================ }

constructor TovExposer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

function TovExposer.CurrentObjectClassName: String;
begin
  Result := ObjectClassName
end;

{ ============================================================================ }
{ ,                                  }
{ ============================================================================ }

procedure TovExposer.SetObjectFilter(Value: String);
begin
  if FObjectFilter <> Value then begin
    Close;
    FObjectFilter := Value;
  end
end;

procedure TovExposer.SetObjectAttributes(const Value: String);
begin
  if FObjectAttributes <> Value then begin
    Close;
    FObjectAttributes := Value;
  end
end;

function TovExposer.GetObjectClassName: String;
var
  Relation: TModelRelation;
begin
  if Not IsMasterDetail then
    Result := MakeFullObjectName(FObjectClassName, NameSpace)
  else begin
    Relation := MasterRelation;
    if Assigned(Relation) then
      Result := Relation.TargetObject.FullName
    else
      Result := ''
  end
end;

procedure TovExposer.SetObjectClassName(Value: String);
begin
  if Not IsMasterDetail then //   Master-Detail   
    if FObjectClassName <> Value then begin
      Close;
      FObjectClassName := Value;
    end
end;

procedure TovExposer.SetObjectOrder(Value: String);
begin
  if FObjectOrder <> Value then begin
    Close;
    FObjectOrder := Value;
  end
end;

procedure TovExposer.SetJoinExpression(Value: String);
begin
  if FJoinExpression <> Value then begin
    Close;
    FJoinExpression := Value;
  end
end;

function TovExposer.GetModelObject: TModelObject;
begin
  if Assigned(Model) and (ObjectClassName <> '')
    then Result := Model.DomainModel.GetObject(ObjectClassName)
    else Result := nil;
end;

{ ============================================================================ }
{                                                     }
{ ============================================================================ }

function TovExposer.NewObject: IovObject;
begin
  Append;
  Post;
  Result := CurrentObject;
end;

function TovExposer.CreateObject: TOID;
begin
  Append;
  Post;
  Result := CurrentObjectId;
end;

procedure TovExposer.AddObject(Id: TOID);
begin
  FNewObject := Id;
  Append;
  Post;
end;

procedure TovExposer.AddObject(AObjectId: IovObject);
begin
  { TODO :  AObject.ClassId }
  AddObject(AObjectId.ObjectId);
end;

procedure TovExposer.DoOnNewRecord;

  function DoRefresh(ARecordId: TOID): Boolean;
  var
    IdField: TField;
  begin
    RefreshKeyDataSet;
    Result := KeyDataSet.Locate(RecordIdFieldName, ARecordId, []);
    if Result then
    begin
      {   Id  ,         }
      IdField := FindField(IdFieldName);
      if Assigned(IdField) then IdField.AsInteger := ARecordId;
    end;
  end;

  function GetObjectFromField(AFieldInfo: TDataSetFieldInfo): IovObject;
  var
    AObjectId: TOID;
  begin
    AObjectId := AFieldInfo.FieldInfo.ObjectInfo.GetField(TObjectIdField).Field.AsInteger;

    if AObjectId <> 0 then
      Result := Database.GetObject(AFieldInfo.ModelObject, AObjectId)
    else
      Result := nil
  end;

  procedure GetRecordDataInternal;
  var
    I: Integer;
    AObject: IovObject;
    ModelAttribute: TModelAttribute;
    ModelRelation: TModelRelation;
    AFieldInfo: TDataSetFieldInfo;
  begin
    for I := 0 to FieldCount - 1 do
    begin
      AFieldInfo := GetFieldInfo(Fields[I].FieldName);
      if AFieldInfo <> nil then
      begin
        if AFieldInfo.FieldIsAttribute(ModelAttribute) then
        begin
          AObject := GetObjectFromField(AFieldInfo);
          if Assigned(AObject) and Not ModelAttribute.Calculated then
            if Not Fields[I].IsBlob then
              Fields[I].Value := AObject[ModelAttribute.Name].Value
            else
              FBlobData.SetBlobData(AFieldInfo.Field,
                AObject[ModelAttribute.Name].AsBlobStream.GetStream(bmRead));
        end else
        if AFieldInfo.FieldIsReference(ModelRelation) then
        begin
          AObject := GetObjectFromField(AFieldInfo);
          if AObject <> nil then
            Fields[I].AsInteger := AObject[ModelRelation.Name].AsObjectId
        end
      end;
    end;
  end;

var
  ACancel, NeedSaveData: Boolean;
  AObject, AMasterObject: IovObject;
  RecordId: TOID;
begin
  NeedSaveData := False;

  if IsMasterDetail then
  begin
    AMasterObject := MasterObject;
    if Not Assigned(AMasterObject) then raise Exception.Create('   ');

    ACancel := False;
    if Not MasterRelation.IsComposition and Assigned(FOnAddObject)
      then FOnAddObject(Self, FNewObject, ACancel);
    if ACancel then Cancel; //     Cancel ??
    NeedSaveData := FNewObject = 0; //    ,     
  end;

  if (Not IsMasterDetail or (dsoCreateChildObjects in Options)) and (FNewObject = 0) then
  begin
    RecordId := CurrentRecordId;
    AObject := Database.Classes[ObjectClassName].CreateObject;
    try
      //          
      //        
      if DoRefresh(AObject.ObjectId) then GetRecordDataInternal;

      //          
      if Assigned(FOnCreateObject) then FOnCreateObject(Self, AObject, ACancel);

      if Not ACancel then begin
      (*         -     Post
        //         -   
        AObject.Validate; //      
      *)
        FNewObject := AObject.ObjectId;
        AObject := nil;
     end else
        Exit;
    finally
        //          -   
      if AObject <> nil then begin
        AObject.Delete;
        if Not IsMasterDetail then DoRefresh(RecordId); //    -       
      end
    end;
  end;

  if IsMasterDetail then begin
    //         
    // (  Master-Detail     )
    with AMasterObject do
      Database.AddChildObject(ClassName, MasterRelationName, ObjectId, FNewObject, RecordId);
    //         
    //   ,         
    if DoRefresh(RecordId) and NeedSaveData then GetRecordDataInternal;
  end;
  FNewObject := 0;
  FPosition := posNormal;
end;

procedure TovExposer.InternalCancel;
begin
  if State = dsInsert then InternalDelete2
end;

procedure TovExposer.InternalDelete2;
begin
  if Not IsMasterDetail then
    Database.DeleteObject(ObjectClassName, CurrentObjectId)
  else
    with MasterObject do
      Database.DeleteObjectRelation(ClassName, MasterRelationName, ObjectId, CurrentObjectId, CurrentRecordId);
  RefreshKeyDataSet;
end;

procedure TovExposer.InternalDelete;

  {{
    , ,  ,    
  }
  procedure DoRefresh;
  var
    SaveRecordId: TOID;
  begin
//    if Not Bof then begin Prior; SaveRecordId := CurrentRecordId end else
//    if Not Eof then begin Next; SaveRecordId := CurrentRecordId end
//    else SaveRecordId := 0;
    RefreshKeyDataSet;
//    if SaveRecordId <> 0 then FindRecord(SaveRecordId);
  end;

var
  AObject: IovObject;
  ACancel, ADeleteObject: Boolean;
begin
  AObject := CurrentObject;

  if IsMasterDetail then
  begin
    ADeleteObject := dsoDeleteChildObjects in Options; //  DeleteChildObjects      OnRemoveObject
    ACancel := False;
    if Assigned(FOnRemoveObject) then FOnRemoveObject(Self, AObject, ACancel, ADeleteObject);
    if ACancel then Exit;

    if ADeleteObject then
    begin
      Database.StartTransaction;
      try
        with MasterObject do
          Database.DeleteObjectRelation(ClassName, MasterRelationName, ObjectId, CurrentObjectId, CurrentRecordId);
          //     
        //   ,    MasterDetail      
        if Assigned(FOnDeleteObject) then FOnDeleteObject(Self, AObject); //       ,   
        AObject.Delete;
        Database.CommitTransaction;
      except
        Database.RollbackTransaction; raise;
      end;
      DoRefresh; //       
    end else
    begin
      with MasterObject do
        Database.DeleteObjectRelation(ClassName, MasterRelationName, ObjectId, CurrentObjectId, CurrentRecordId);
      DoRefresh; //       
    end
  end else begin
    //   ,    MasterDetail      
    if Assigned(FOnDeleteObject) then FOnDeleteObject(Self, AObject); //       ,   
    AObject.Delete;
    DoRefresh; //       
  end
end;

procedure TovExposer.DeleteCurrentObject;
var
  SaveOptions: TovDataSetOptions;
begin
  SaveOptions := Options;
  FOptions := FOptions + [dsoDeleteChildObjects];
  try
    Delete;
  finally
    FOptions := SaveOptions;
  end;
end;

procedure TovExposer.RemoveCurrentObject;
var
  SaveOptions: TovDataSetOptions;
begin
  SaveOptions := Options;
  FOptions := FOptions - [dsoDeleteChildObjects];
  try
    Delete;
  finally
    FOptions := SaveOptions;
  end;
end;

procedure TovExposer.DeleteAllObjects;
begin
end;

procedure TovExposer.RemoveAllObjects;
begin
end;

{ ============================================================================ }
{  Master-Detail                                                          }
{ ============================================================================ }

function TovExposer.GetMasterObject: IovObject;
begin
  if Assigned(MasterDataSet) then
    if MasterObjectId = 0 then
      if MasterObjectName = '' then
        Result := MasterDataSet.CurrentObject
      else
        Result := MasterDataSet.ObjectByName(GetMasterObjectName)
      //    MasterDataSet     -   
      //     
      else Result := Database.Classes[MasterDataSet.GetObjectClass(GetMasterObjectName)].GetObject(MasterObjectId)
  else
    Result := nil
end;

{ ============================================================================ }
{                                               }
{ ============================================================================ }

function TovExposer.OpenDataSet: TDataSet;
begin
  if ObjectAttributes = '' then raise Exception.Create('     :' + Name);

  if MasterRelationName = '' then
  begin
    if ObjectClassName = '' then
      raise Exception.Create('      ' + Name);
    Result := Database.OpenObjectDataSet(NameSpace, ObjectClassName, ObjectAttributes,
      SetParamValues(ObjectFilter), ObjectOrder, JoinExpression, FQueryInfo)
  end else
    if MasterObjectId = 0 then
      Result := Database.OpenChildObjectDataSet(NameSpace,
        MasterDataSet.GetObjectClass(GetMasterObjectName),
        MasterRelationName, ObjectAttributes, SetParamValues(ObjectFilter), ObjectOrder, JoinExpression,
        FMasterDataSource, GetMasterFieldName, FQueryInfo)
    else
      Result := Database.OpenChildObjectDataSet(NameSpace,
        MasterDataSet.GetObjectClass(GetMasterObjectName),
        MasterRelationName, ObjectAttributes, SetParamValues(ObjectFilter), ObjectOrder, JoinExpression,
        MasterObjectId,
        FQueryInfo);
end;

{{
  ,   ,  
   ,  ,   
Attributes.  ,   ExposeObject(AObject) -   
,  ExposeObject(AObject, ''),     ,
   ObjectAttributes.
}
procedure TovExposer.ExposeObject(AObject: IovObject; Attributes: String =
  '[*]');
begin
  Close;
  ObjectClassName := AObject.ClassName;
  if Attributes <> '' then ObjectAttributes := Attributes;
  ObjectFilter := '[OID#] = ' + IntToStr(AObject.ObjectId);
  Open;
end;

{ ============================================================================ }
{ TovQuery                                                                     }
{ ============================================================================ }

constructor TovQuery.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FQuery := TStringList.Create;
  TStringList(FQuery).OnChange := QueryChange;
end;

destructor TovQuery.Destroy;
begin
  inherited Destroy;
  FQuery.Free;
end;

function TovQuery.CurrentObjectClassName: String;
begin
  Result := FQueryInfo.Objects[0].ModelObject.FullName
end;

procedure TovQuery.QueryChange(Sender: TObject);
begin
  Close
end;

procedure TovQuery.SetQuery(Value: TStrings);
begin
  FQuery.Assign(Value);
end;

function TovQuery.OpenDataSet: TDataSet;
var
  QueryText: String;
begin
  QueryText := Trim(Query.Text);
  if QueryText = '' then raise Exception.Create('     ' + Name);

  if FParams.Count > 0 then
    QueryText := SetParamValues(QueryText);

  if Assigned(MasterDataSet) and (MasterRelationName <> '') then
  begin
    //   ,   MasterRelationName
    //   master- 
    Result := Database.OpenChildQuery(NameSpace, QueryText,
      MasterDataSet.GetObjectClass(GetMasterObjectName),
      MasterRelationName, FMasterDataSource, GetMasterFieldName, FQueryInfo);
  end else
  begin
    //      , 
    //   MasterFieldName  master- 
    if MasterFieldName <> '' then
      QueryText := SetMasterField(QueryText);
    Result := Database.OpenQuery(NameSpace, QueryText, FQueryInfo, FMasterDataSource)
  end;
end;

procedure TovQuery.InternalDelete;

  {{
    , ,  ,    
  }
  procedure DoRefresh;
  var
    SaveRecordId: TOID;
  begin
    if Not Bof then begin Prior; SaveRecordId := CurrentRecordId end else
    if Not Eof then begin Next; SaveRecordId := CurrentRecordId end
    else SaveRecordId := 0;
    RefreshKeyDataSet;
    if SaveRecordId <> 0 then FindRecord(SaveRecordId);
  end;

var
  AObject: IovObject;
begin
  AObject := CurrentObject;
  if Assigned(FOnDeleteObject) then FOnDeleteObject(Self, AObject); //       ,   
  AObject.Delete(DeleteMode);
  DoRefresh; //       
end;

procedure TovQuery.InsertRecordInternal;
begin
end;

{
********************************************************************************
    .

   TovDataSet     
      Idle.
********************************************************************************
}

var
  AppEvents: TApplicationEvents;

procedure AppEventsIdle(Self, Sender: TObject; var Done: Boolean);
var
  I: Integer;
begin
  for I := 0 to TovDataSet.DataSetCount - 1 do
    TovDataSet.GetDataSet(I).Idle;
end;

var Method: TMethod;
initialization
  AppEvents := TApplicationEvents.Create(nil);
  Method.Data := nil;
  Method.Code := @AppEventsIdle;
  AppEvents.OnIdle := TIdleEvent(Method);
  DataSets := TComponentList.Create(False);
finalization
  FreeAndNil(DataSets);
  FreeAndNil(AppEvents);
end.
