unit ovDatabase;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB,
  ovTypes, ovModel, ovOperations, ovIObjects,
  ovQueryInfo, ovQueryMapper, ovDatabaseManager;

type
  TOnCalcAttributeEvent = procedure (AObjectId: TOID;
    const AModelAttribute: TModelAttribute; var Value: Variant) of object;
  TValidateObjectEvent = procedure (
    AModelObject: TModelObject; AObjectId: TOID) of object;

  //       -   
  // 
  TDeleteObjectErrorEvent = procedure (
    AModelObject: TModelObject; AObjectId: TOID) of object;

  TovDatabaseEvents = record
    OnAfterGenerateDatabase: TAfterGenerateDatabaseEvent;
    OnBeforeGenerateDatabase: TBeforeGenerateDatabaseEvent;
    OnGenerateDatabaseProgress: TGenerateDatabaseProgressEvent;
    OnCreateObject: TCreateObjectEvent;
    OnDeleteObject: TDeleteObjectEvent;
    OnDeleteObjectError: TDeleteObjectErrorEvent;
    OnValidateObject: TValidateObjectEvent;
    OnCalcAttribute: TOnCalcAttributeEvent;
  end;

  TovDatabase = class (TInterfacedObject, IovObjectSpace)
  private
    FAdapter: IDatabaseAdapter;
    FSelectQueryMapper: TSelectQueryMapper;
    FObjectOperations: TObjectOperations;
    FDatabaseManager: TovDatabaseManager;
    FModel: TDomainModel;
    FRegisteredListeners: IInterfaceList;
    FRegisteredClasses: TStrings;
    FUpdateLevel: Integer;
    FChangedTables: TStringList;
    FDatabaseEvents: TovDatabaseEvents;
    FUpdateQueryMapper: TUpdateQueryMapper;
    function CreateClassInstance(AClassName: String): IovClass;
    function GetClasses(AClassName: String): IovClass;
    procedure SetAdapter(Value: IDatabaseAdapter);
    function GetModel: TDomainModel;
    property ObjectOperations: TObjectOperations read FObjectOperations;
    property SelectQueryMapper: TSelectQueryMapper read FSelectQueryMapper write
      FSelectQueryMapper;
    property UpdateQueryMapper: TUpdateQueryMapper read FUpdateQueryMapper write
      FUpdateQueryMapper;
    function GetIsUpdating: Boolean;
    function IovObjectSpace.CreateObject = CreateClassObject;
    function CreateClassObject(AClassName: String): IovObject; overload;
    procedure TableChangedNotification(ATableName: String);
    function GetObject(AClassName: String; ObjectId: TOID): IovObject; overload;
    procedure ClassTableChanged(AModelObject: TModelObject);
    procedure AfterDeleteObject(AModelObject: TModelObject; AObjectId: TOID);
    function OpenQuery(AQuery: String): TDataSet; overload;
  public
    constructor Create(AModel: TDomainModel; const ADatabaseEvents:
      TovDatabaseEvents);
    destructor Destroy; override;
    procedure GenerateDatabase(TargetDatabase: IDatabaseAdapter; RebuildAllTables:
      Boolean);
    procedure GenerateObjectTables(AClassName: String);
    procedure CopyDatabase(SourceDatabase: IDatabaseAdapter; SourceModel:
      TDomainModel; TargetDatabase: IDatabaseAdapter; TargetModel: TDomainModel);
    procedure Close;
    procedure CommitTransaction;
    {{
       .    () 
         (  MinTargetCount) 
     .

          AClassName  
         TOnCreateObjectEvent,  
            
     .
    
          .
    }
    function CreateObject(AClassName: String): TOID; overload;
    procedure DeleteObject(AClassName: String; AObjectId: TOID; ADeleteMode:
      TDeleteMode = dmCheckRefs);
    {{
         ObjectId   
    AnAttributeName  AClassName  ParentObjectId.   
     ,         
       .   DeleteChildrenOnly  
    True,       ,     
     .
    }
    procedure DeleteObjects(AClassName: String; AnObjects: TOIDList); overload;
    function GetObjectRelation(AClassName, ARelationName: String; AParentObjectId,
      ARelationId: TOID): TOID;
    procedure Open;
    procedure RollbackTransaction;
    procedure StartTransaction;
    procedure UpdateObjectRelation(AClassName, ARelationName: String;
      AParentObjectId, ARelationId, ANewChildObjectId: TOID);
    procedure DeleteObjectRelation(AClassName, ARelationName: String;
      AParentObjectId, AChildObjectId, ARelationId: TOID);
    procedure RemoveChildObject(AClassName, ARelationName: String; AParentObjectId,
      AChildObjectId, ARelationId: TOID);
    procedure DeleteObjectRelations(AClassName: String; AObjectId: TOID);
    function AddChildObject(AClassName, ARelationName: String; AParentObjectId,
      AChildObjectId: TOID; var ARelationId: TOID): TOID;
    procedure GetChildObjects(AObjectClassName, AObjectRelationName: String;
      AParentObjectId: TOID; AObjects: TOIDList; AObjectFilter: String = ''; 
      AObjectOrder: String = '');
    procedure GetObjects(AObjectClassName: String; AObjects: TOIDList; 
      AObjectFilter: String = ''; AObjectOrder: String = '');
    function OpenChildObjectDataSet(ADefaultNameSpace, AObjectClassName,
       AObjectRelationName, AChildObjectAttributes, AObjectFilter, AObjectOrder,
       AJoinExpression: String; AMasterDataSource: TDataSource; AMasterFieldName:
       String; var AQueryInfo: TSelectQueryInfo): TDataSet; overload;
    function OpenChildQuery(ADefaultNameSpace, AQuery, AMasterObjectClassName,
      AMasterObjectRelationName: String; AMasterDataSource: TDataSource; 
      AMasterFieldName: String; var AQueryInfo: TSelectQueryInfo): TDataSet; 
      overload;
    function OpenObjectDataSet(ADefaultNameSpace, AObjectClassName,
       AObjectAttributes, AObjectFilter, AObjectOrder, AJoinExpression: String; var
       AQueryInfo: TSelectQueryInfo): TDataSet;
    function OpenQuery(ADefaultNameSpace, AQuery: String; var AQueryInfo:
      TSelectQueryInfo; AMasterDataSource: TDataSource = nil): TDataSet; overload;
    procedure RegisterTableChangeListener(Listener: ITableChangeListener);
    procedure UnRegisterTableChangeListener(Listener: ITableChangeListener);
    function GetAncestorObject(AModelObject, AParentObject: TModelObject;
      AObjectId: TOID): TOID;
    procedure BeginUpdate;
    procedure EndUpdate;
    procedure BeginUpdateTables;
    procedure EndUpdateTables;
    procedure CancelUpdateTables;
    procedure IovObjectSpace.BeginUpdate=BeginUpdateTables;
    procedure IovObjectSpace.EndUpdate=EndUpdateTables;
    procedure IovObjectSpace.CancelUpdate=CancelUpdateTables;

    function OpenChildObjectDataSet(ADefaultNameSpace, AObjectClassName,
       AObjectRelationName, AChildObjectAttributes, AObjectFilter, AObjectOrder,
       AJoinExpression: String; AMasterObjectId: TOID; var AQueryInfo:
       TSelectQueryInfo): TDataSet; overload;
    function InTransaction: Boolean;
    procedure GetReferencingObjects(AClassName: String; AObjectId: TOID; AObjects:
      TovObjectRefList; ImmediateObjects: Boolean);
    function GetClass(AClassId: Integer): IovClass;
    procedure SetObjectOwner(ChildObjectClass: String; ChildObjectId: TOID;
      ParentObjectId, RelationId: TOID);
    procedure RegisterClassImplementor(AClass: TClass; AClassName: String);
    procedure UnregisterClassImplementor(AClass: TClass; AClassName: String = '');
    function GetClassImplementor(AClassName: String; var AClass: TClass): Boolean;
    procedure GetObjectOwner(ChildObjectClass: String; ChildObjectId: TOID; var
      ParentObjectId, RelationId: TOID);
    function GetObjectAttributeField(AClassName, AttributeName: String; AObjectId:
      TOID; var FieldName: String; ReadOnly: Boolean): TDataSet;
    procedure CalculateAttribute(AObjectId: TOID; const AModelAttribute:
      TModelAttribute; var Value: Variant);
    procedure CustomValidateObject(AModelObject: TModelObject; AObjectId: TOID);
    function GetObject(AModelObject: TModelObject; ObjectId: TOID): IovObject;
      overload;
    procedure UpdateObjectAttribute(AModelAttribute: TModelAttribute; AObjectId:
      TOID; AValue: Variant);
    procedure UpdateObjectsAttributes(AClassName: String; const AttributeValues:
      Variant; WhereExpression: String);
    procedure DeleteObjects(AClassName: String; WhereExpression: String); overload;
    function OpenSQLQuery(Query: String; MasterDataSource: TDataSource = nil;
      LiveDataSet: Boolean = False): TDataSet;
    procedure ExecSQLQuery(AQuery: String);
    function GetAncestor(AObjectId: TOID; AGeneralization: TModelRelation): TOID;
      overload;
    function GetBooleanAttributeValue(Field: TField): Boolean;
    property DatabaseManager: TovDatabaseManager read FDatabaseManager;
    property Adapter: IDatabaseAdapter read FAdapter write SetAdapter;
    property Classes[AClassName: String]: IovClass read GetClasses;
    property Model: TDomainModel read GetModel;
    property IsUpdating: Boolean read GetIsUpdating;
  end;

implementation

uses ovConsts, ovObjects, Variants, ovQueryManager, ovOQLProcessor;

{
********************************* TovDatabase **********************************
}

constructor TovDatabase.Create(AModel: TDomainModel; const ADatabaseEvents:
  TovDatabaseEvents);
begin
  inherited Create;
  FModel := AModel;
  FDatabaseEvents := ADatabaseEvents;

  FDatabaseManager := TovDatabaseManager.Create;
  FObjectOperations := TObjectOperations.Create(DatabaseManager, Model);
  FObjectOperations.AfterCreateObject := ADatabaseEvents.OnCreateObject;
  FObjectOperations.BeforeDeleteObject := ADatabaseEvents.OnDeleteObject;
  FObjectOperations.AfterDeleteObject := AfterDeleteObject;

  FSelectQueryMapper := TSelectQueryMapper.Create(DatabaseManager, Model);
  FUpdateQueryMapper := TUpdateQueryMapper.Create(DatabaseManager, Model);

  FRegisteredListeners := TInterfaceList.Create;
  FChangedTables := TStringList.Create;
  FRegisteredClasses := TStringList.Create;
end;

destructor TovDatabase.Destroy;
begin
  FRegisteredClasses.Free;
  FChangedTables.Free;

  FUpdateQueryMapper.Free;
  FSelectQueryMapper.Free;
  FObjectOperations.Free;
  FDatabaseManager.Free;

  inherited Destroy;
end;

procedure TovDatabase.SetAdapter(Value: IDatabaseAdapter);
begin
  if FAdapter <> Value then
  begin
    FAdapter := Value;
    DatabaseManager.DatabaseAdapter := Value;
  end;
end;

procedure TovDatabase.Open;
begin
  DatabaseManager.OpenDatabase;
end;

procedure TovDatabase.Close;
begin
  FChangedTables.Clear;
  DatabaseManager.CloseDatabase;
end;

function TovDatabase.GetModel: TDomainModel;
begin
  Result := FModel;
end;

procedure TovDatabase.GenerateDatabase(TargetDatabase: IDatabaseAdapter;
  RebuildAllTables: Boolean);
begin
  with TDatabaseOperations.Create do
  try
    OnBeforeGenerateDatabase := FDatabaseEvents.OnBeforeGenerateDatabase;
    OnAfterGenerateDatabase := FDatabaseEvents.OnAfterGenerateDatabase;
    OnGenerateDatabaseProgress := FDatabaseEvents.OnGenerateDatabaseProgress;
    GenerateDatabase(TargetDatabase, Model, RebuildAllTables);
  finally
    Free
  end;
end;

procedure TovDatabase.CopyDatabase(SourceDatabase: IDatabaseAdapter;
  SourceModel: TDomainModel; TargetDatabase: IDatabaseAdapter; TargetModel:
  TDomainModel);
begin
  with TDatabaseOperations.Create do
  try
    OnBeforeGenerateDatabase := FDatabaseEvents.OnBeforeGenerateDatabase;
    OnAfterGenerateDatabase := FDatabaseEvents.OnAfterGenerateDatabase;
    OnGenerateDatabaseProgress := FDatabaseEvents.OnGenerateDatabaseProgress;
    CopyDatabase(SourceDatabase, SourceModel, TargetDatabase, TargetModel);
  finally
    Free
  end;
end;

{{
     (    )
}
procedure TovDatabase.GenerateObjectTables(AClassName: String);
begin
  with TDatabaseOperations.Create do
  try
    GenerateObjectTables(nil, nil, Model.GetObject(AClassName), Adapter, True);
  finally
    Free
  end;
end;

procedure TovDatabase.CustomValidateObject(AModelObject: TModelObject;
  AObjectId: TOID);
begin
  FDatabaseEvents.OnValidateObject(AModelObject, AObjectId)
end;

{
************************    *********************************
}

procedure TovDatabase.CalculateAttribute(AObjectId: TOID; const
  AModelAttribute: TModelAttribute; var Value: Variant);
begin
  if Not AModelAttribute.IsInherited then
    FDatabaseEvents.OnCalcAttribute(AObjectId, AModelAttribute, Value)
  else
    FDatabaseEvents.OnCalcAttribute(
      GetAncestorObject(AModelAttribute.Owner, AModelAttribute.ParentAttribute.Owner, AObjectId),
      AModelAttribute.ParentAttribute, Value)
end;

{{
   ,   ,
   AttributeName   AClassName 
 AObjectId.   FieldName   ,
  .
}
function TovDatabase.GetObjectAttributeField(AClassName, AttributeName: String;
  AObjectId: TOID; var FieldName: String; ReadOnly: Boolean): TDataSet;
var
  ModelObject: TModelObject;
begin
  ModelObject := Model.GetObject(AClassName);
  if AttributeName <> '' then
    FieldName := ModelObject.GetAttribute(AttributeName).FieldName;
  Result := FObjectOperations.GetObjectDataSet(ModelObject, AObjectId, ReadOnly);
end;

procedure TovDatabase.UpdateObjectAttribute(AModelAttribute: TModelAttribute;
  AObjectId: TOID; AValue: Variant);
begin
  FObjectOperations.UpdateObjectAttribute(AModelAttribute.Owner, AObjectId,
    AModelAttribute, AValue);
  ClassTableChanged(AModelAttribute.Owner);
end;

procedure TovDatabase.UpdateObjectsAttributes(AClassName: String; const
  AttributeValues: Variant; WhereExpression: String);
begin
  UpdateQueryMapper.UpdateObjects(Model.GetObject(AClassName),
    AttributeValues, WhereExpression)
end;

{
************************   **********************************
}

function TovDatabase.OpenQuery(ADefaultNameSpace, AQuery: String; var
  AQueryInfo: TSelectQueryInfo; AMasterDataSource: TDataSource = nil): TDataSet;
begin
  Result := SelectQueryMapper.OpenObjectQuery(ADefaultNameSpace, AQuery, AQueryInfo,
    nil, AMasterDataSource)
end;

function TovDatabase.OpenChildQuery(ADefaultNameSpace, AQuery,
  AMasterObjectClassName, AMasterObjectRelationName: String; AMasterDataSource:
  TDataSource; AMasterFieldName: String; var AQueryInfo: TSelectQueryInfo):
  TDataSet;
var
  MasterDetailRelation: TMasterDetailRelation;
begin
  MasterDetailRelation := TDynamicMasterDetailRelation.Create(
    AMasterObjectClassName, AMasterObjectRelationName, AMasterFieldName);
  try
    Result := SelectQueryMapper.OpenObjectQuery(ADefaultNameSpace, AQuery, AQueryInfo,
      MasterDetailRelation, AMasterDataSource)
  finally
    MasterDetailRelation.Free
  end;
end;

{***************************************************************************** }
{                                                                    }
{ **************************************************************************** }

procedure TovDatabase.StartTransaction;
begin
  DatabaseManager.StartTransaction;
end;

procedure TovDatabase.CommitTransaction;
begin
  DatabaseManager.CommitTransaction;
end;

procedure TovDatabase.RollbackTransaction;
begin
  DatabaseManager.RollbackTransaction;
end;

function TovDatabase.InTransaction: Boolean;
begin
  Result := DatabaseManager.InTransaction
end;

{***************************************************************************** }
{                                                   }
{ **************************************************************************** }

procedure TovDatabase.BeginUpdateTables;
begin
  DatabaseManager.QueryManager.StartUpdating;
end;

procedure TovDatabase.EndUpdateTables;
begin
  DatabaseManager.QueryManager.EndUpdating;
end;

procedure TovDatabase.CancelUpdateTables;
begin
  DatabaseManager.QueryManager.CancelUpdating;
end;

{***************************************************************************** }
{                                                              }
{ **************************************************************************** }

{{
   .    ( )
    (  MinTargetCount)  .
        
 .

         .
}
function TovDatabase.CreateObject(AClassName: String): TOID;
var
  ModelObject: TModelObject;
begin
  StartTransaction;
  try
    ModelObject := Model.GetObject(AClassName);
    Result := ObjectOperations.CreateObject(ModelObject);
    CommitTransaction;
    ClassTableChanged(ModelObject);
  except
    RollbackTransaction; raise
  end;
end;

function TovDatabase.CreateClassObject(AClassName: String): IovObject;
begin
  Result := GetClasses(AClassName).CreateObject
end;

function TovDatabase.GetObject(AClassName: String; ObjectId: TOID): IovObject;
begin
  Result := CreateClassInstance(AClassName).GetObject(ObjectId)
end;

function TovDatabase.GetObject(AModelObject: TModelObject; ObjectId: TOID):
  IovObject;
begin
  Result := CreateClassInstance(AModelObject.FullName).GetObject(ObjectId)
end;

{{
   AChildObjectId , 
 ARelationName  AClassName,   
AParentObjectId,   AClassName.  AChildObjectId = 0, 
    .

     .
}
function TovDatabase.AddChildObject(AClassName, ARelationName: String;
  AParentObjectId, AChildObjectId: TOID; var ARelationId: TOID): TOID;
begin
  StartTransaction;
  try
    Result := ObjectOperations.CreateObjectRelation(
      Model.GetObjectRelation(AClassName, ARelationName),
        AParentObjectId, AChildObjectId, ARelationId);
    CommitTransaction;
  except
    RollbackTransaction; raise
  end;
end;

{
************************   *************************************
}

{{
   ,       .
 ADeleteMode      
   .
}
procedure TovDatabase.DeleteObject(AClassName: String; AObjectId: TOID;
  ADeleteMode: TDeleteMode = dmCheckRefs);
var
  TransactionRollbacked: Boolean;
  AModelObject: TModelObject;
begin
  Screen.Cursor := crHourGlass;
  try
    AModelObject := Model.GetObject(AClassName);
    TransactionRollbacked := False;
    StartTransaction;
    try
      try
        ObjectOperations.DeleteObject(AModelObject, AObjectId, ADeleteMode);
      except on E: EDeleteObjectError do
        if Assigned(FDatabaseEvents.OnDeleteObjectError) then
        begin
          RollbackTransaction;
          TransactionRollbacked := True;
          FDatabaseEvents.OnDeleteObjectError(E.ModelObject, E.ObjectId);
          Abort;
        end else
          raise
      end;

      CommitTransaction;
    except
      if Not TransactionRollbacked then RollbackTransaction;
      raise
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TovDatabase.DeleteObjectRelation(AClassName, ARelationName: String;
  AParentObjectId, AChildObjectId, ARelationId: TOID);
var
  TransactionRollbacked: Boolean;
  AModelRelation: TModelRelation;
begin
  Screen.Cursor := crHourGlass;
  try
    TransactionRollbacked := False;
    StartTransaction;
    try
      AModelRelation := Model.GetObjectRelation(AClassName, ARelationName);
      try
        ObjectOperations.DeleteObjectRelation(AModelRelation,
          AParentObjectId, AChildObjectId, ARelationId, True);
      except on E: EDeleteObjectError do
        if Assigned(FDatabaseEvents.OnDeleteObjectError) then
        begin
          RollbackTransaction;
          TransactionRollbacked := True;
          FDatabaseEvents.OnDeleteObjectError(E.ModelObject, E.ObjectId);
          Abort;
        end else
          raise
      end;
      CommitTransaction;
    except
      if Not TransactionRollbacked then RollbackTransaction;
      raise
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TovDatabase.RemoveChildObject(AClassName, ARelationName: String;
  AParentObjectId, AChildObjectId, ARelationId: TOID);
begin
  StartTransaction;
  try
    ObjectOperations.DeleteObjectRelation(Model.GetObjectRelation(AClassName, ARelationName),
      AParentObjectId, AChildObjectId, ARelationId, False);
    CommitTransaction;
  except
    RollbackTransaction; raise
  end;
end;

procedure TovDatabase.DeleteObjects(AClassName: String; AnObjects: TOIDList);
begin
  StartTransaction;
  try
    ObjectOperations.DeleteObjects(Model.GetObject(AClassName), AnObjects);
    CommitTransaction;
    { TODO :       }
  except
    RollbackTransaction; raise
  end;
end;

procedure TovDatabase.DeleteObjects(AClassName: String; WhereExpression:
  String);
begin
  UpdateQueryMapper.DeleteObjects(Model.GetObject(AClassName),
    WhereExpression)
end;

{
**********************    ***********************************
}

{{
   ARelationName   AParentObjectId 
AClassName  ,      ANewChildObjectId.
}
procedure TovDatabase.UpdateObjectRelation(AClassName, ARelationName: String;
  AParentObjectId, ARelationId, ANewChildObjectId: TOID);
begin
  ObjectOperations.UpdateObjectRelation(Model.GetObjectRelation(
    AClassName, ARelationName),
    AParentObjectId, ARelationId, ANewChildObjectId)
end;

{{
    ParentObjectId  
 RelationId     ChildObjectId 
ChildObjectClass.
}
procedure TovDatabase.GetObjectOwner(ChildObjectClass: String; ChildObjectId:
  TOID; var ParentObjectId, RelationId: TOID);
begin
  ObjectOperations.GetObjectOwner(Model.GetObject(ChildObjectClass),
    ChildObjectId, ParentObjectId, RelationId)
end;

{{
    ChildObjectId  ChildObjectClass  
     ParentObjectId    
  RelationId.
}
procedure TovDatabase.SetObjectOwner(ChildObjectClass: String; ChildObjectId:
  TOID; ParentObjectId, RelationId: TOID);
begin
  ObjectOperations.SetObjectOwner(Model.GetObject(ChildObjectClass),
    ChildObjectId, ParentObjectId, RelationId)
end;

function TovDatabase.GetObjectRelation(AClassName, ARelationName: String;
  AParentObjectId, ARelationId: TOID): TOID;
begin
  Result := ObjectOperations.GetObjectRelation(Model.GetObjectRelation(
    AClassName, ARelationName), AParentObjectId, ARelationId)
end;

procedure TovDatabase.DeleteObjectRelations(AClassName: String; AObjectId:
  TOID);
begin
  StartTransaction;
  try
    ObjectOperations.DeleteObjectRelations(Model.GetObject(AClassName), AObjectId);
    CommitTransaction;
  except
    RollbackTransaction; raise
  end;
end;

{{
   AParentObject,  
AObjectId    AModelObject. .. AParentObject 
    AModelObject  AObjectId  
  AModelObject.
}
function TovDatabase.GetAncestorObject(AModelObject, AParentObject:
  TModelObject; AObjectId: TOID): TOID;
begin
  Result := ObjectOperations.GetAncestorObject(AModelObject, AParentObject, AObjectId);
end;

{{
    (  AGeneralization.TargetObjectName)
    AObjectId  AGeneralization.SourceObjectName.
}
function TovDatabase.GetAncestor(AObjectId: TOID; AGeneralization:
  TModelRelation): TOID;
begin
  Result := ObjectOperations.GetObjectRelation(AGeneralization, AObjectId, 0)
end;

{{
  ,    AObjectId  AClassName.
}
procedure TovDatabase.GetReferencingObjects(AClassName: String; AObjectId:
  TOID; AObjects: TovObjectRefList; ImmediateObjects: Boolean);
begin
  FObjectOperations.GetReferencingObjects(AClassName, AObjectId, AObjects,
    ImmediateObjects)
end;

{
********************************************************************************
                     
********************************************************************************
}

{{
  ,    AObjectClassName,
  AObjectFilter      AObjectOrder.
}
function TovDatabase.OpenObjectDataSet(ADefaultNameSpace, AObjectClassName,
   AObjectAttributes, AObjectFilter, AObjectOrder, AJoinExpression: String; var
   AQueryInfo: TSelectQueryInfo): TDataSet;
begin
  Result := SelectQueryMapper.OpenObjectDataSet(ADefaultNameSpace, AObjectClassName,
    AObjectAttributes, AObjectFilter, AObjectOrder, AJoinExpression, AQueryInfo)
end;

{{
  ,   ,
  AObjectRelationName   AObjectClassName 
  AObjectFilter,     AObjectOrder.

      Master-Detail   AMasterFieldName
      MasterDataSource.
}
function TovDatabase.OpenChildObjectDataSet(ADefaultNameSpace,
   AObjectClassName, AObjectRelationName, AChildObjectAttributes,
   AObjectFilter, AObjectOrder, AJoinExpression: String; AMasterDataSource:
   TDataSource; AMasterFieldName: String; var AQueryInfo: TSelectQueryInfo):
   TDataSet;
var
  FullClassName: String;
  MasterDetailRelation: TMasterDetailRelation;
begin
  FullClassName := MakeFullObjectName(AObjectClassName, ADefaultNameSpace);
  MasterDetailRelation := TDynamicMasterDetailRelation.Create(
    FullClassName, AObjectRelationName, AMasterFieldName);
  try
    Result := SelectQueryMapper.OpenObjectDataSet(ADefaultNameSpace,
      Model.GetObjectRelation(FullClassName,
      AObjectRelationName).TargetObject.FullName,
      AChildObjectAttributes,
      AObjectFilter, AObjectOrder, AJoinExpression, AQueryInfo, MasterDetailRelation, AMasterDataSource)
  finally
    MasterDetailRelation.Free
  end;
end;

{{
  ,   ,
  AObjectRelationName   AObjectClassName 
  AObjectFilter,     AObjectOrder.

         
AMasterObjectId  AObjectClassName.
}
function TovDatabase.OpenChildObjectDataSet(ADefaultNameSpace,
   AObjectClassName, AObjectRelationName, AChildObjectAttributes,
   AObjectFilter, AObjectOrder, AJoinExpression: String; AMasterObjectId: TOID;
   var AQueryInfo: TSelectQueryInfo): TDataSet;
var
  ModelRelation: TModelRelation;
  MasterDetailRelation: TMasterDetailRelation;
begin
  ModelRelation := Model.GetObjectRelation(AObjectClassName, AObjectRelationName);
  //   ,      -,
  //   
  if ModelRelation.IsInherited then begin
    AMasterObjectId := GetAncestorObject(ModelRelation.SourceObject,
      ModelRelation.ParentObject, AMasterObjectId);
    ModelRelation := Model.GetObjectRelation(ModelRelation.ParentObject.FullName,
      ModelRelation.Name); //   ,    
  end;

  MasterDetailRelation := TStaticMasterDetailRelation.Create(
    AObjectClassName, AObjectRelationName, AMasterObjectId);
  try
    Result := SelectQueryMapper.OpenObjectDataSet(ADefaultNameSpace,
      ModelRelation.TargetObject.FullName, AChildObjectAttributes,
      AObjectFilter, AObjectOrder, AJoinExpression, AQueryInfo, MasterDetailRelation)
  finally
    MasterDetailRelation.Free
  end;
end;

{{
     AObjectClassName, 
 AObjectFilter,     AObjectOrder.
}
procedure TovDatabase.GetObjects(AObjectClassName: String; AObjects: TOIDList;
  AObjectFilter: String = ''; AObjectOrder: String = '');
var
  QueryInfo: TSelectQueryInfo;
  DataSet: TDataSet;
begin
  DataSet := SelectQueryMapper.OpenObjectDataSet('', AObjectClassName, '',
    AObjectFilter, AObjectOrder, '', QueryInfo);
  try
    AObjects.LoadFromDataSet(DataSet, QueryInfo.GetObject(AObjectClassName).GetField(TObjectIdField).Field)
  finally
    DataSet.Free;
    QueryInfo.Free;
  end;
end;

{{
   ,   AObjectRelationName,
  AParentObjectId  AObjectClassName,  
AObjectFilter,     AObjectOrder.
}
procedure TovDatabase.GetChildObjects(AObjectClassName, AObjectRelationName:
  String; AParentObjectId: TOID; AObjects: TOIDList; AObjectFilter: String = '';
  AObjectOrder: String = '');
var
  ModelRelation: TModelRelation;
  QueryInfo: TSelectQueryInfo;
  MasterDetailRelation: TMasterDetailRelation;
  DataSet: TDataSet;
  AChildObjectId: TOID;
begin
  ModelRelation := Model.GetObjectRelation(AObjectClassName, AObjectRelationName);
  if ModelRelation.IsReference then
  begin
    AChildObjectId := GetObjectRelation(AObjectClassName, AObjectRelationName,
      AParentObjectId, 0);
    AObjects.Clear;
    if AChildObjectId <> 0 then AObjects.AddObject(AChildObjectId, 0);
  end else
  begin
    //   ,      -,
    //   
    if ModelRelation.IsInherited then begin
      AParentObjectId := GetAncestorObject(ModelRelation.SourceObject,
        ModelRelation.ParentObject, AParentObjectId);
      ModelRelation := Model.GetObjectRelation(ModelRelation.ParentObject.FullName,
        ModelRelation.Name); //   ,    
    end;

    QueryInfo := nil;
    MasterDetailRelation := nil;
    try
      MasterDetailRelation := TStaticMasterDetailRelation.Create(
        AObjectClassName, AObjectRelationName, AParentObjectId);
      DataSet := SelectQueryMapper.OpenObjectDataSet('', ModelRelation.TargetObject.FullName, '',
        AObjectFilter, AObjectOrder, '', QueryInfo, MasterDetailRelation);
      //       2- :
      //   -      -  
      try
        AObjects.LoadFromDataSet(DataSet,
          QueryInfo.GetObject(ModelRelation.TargetObject.FullName).GetField(TObjectIdField).Field,
          QueryInfo.GetField(TRelationIdField).Field);
      finally
        DataSet.Free
      end;
    finally
      MasterDetailRelation.Free;
      QueryInfo.Free
    end;
  end
end;

{
********************************************************************************
          
********************************************************************************
}

procedure TovDatabase.AfterDeleteObject(AModelObject: TModelObject; AObjectId:
   TOID);
begin
  ClassTableChanged(AModelObject);
end;

{{
        ( 
  IsUpdating.      
 .
}
procedure TovDatabase.ClassTableChanged(AModelObject: TModelObject);
begin
  if Not IsUpdating then
    TableChangedNotification(AModelObject.TableName)
  else
    if FChangedTables.IndexOf(AModelObject.TableName) < 0 then
      FChangedTables.Add(AModelObject.TableName);
end;

{{
   ""    
}
procedure TovDatabase.TableChangedNotification(ATableName: String);
var
  I: Integer;
begin
  for I := 0 to FRegisteredListeners.Count - 1 do
    ITableChangeListener(FRegisteredListeners[I]).TableChanged(ATableName);
end;

function TovDatabase.GetIsUpdating: Boolean;
begin
  Result := FUpdateLevel > 0;
end;

procedure TovDatabase.BeginUpdate;
begin
  Inc(FUpdateLevel);
end;

procedure TovDatabase.EndUpdate;
var
  I: Integer;
begin
  Dec(FUpdateLevel);
  if FUpdateLevel = 0 then
  begin
    for I := 0 to FChangedTables.Count - 1 do
      TableChangedNotification(FChangedTables[I]);
    FChangedTables.Clear;
  end;
end;

procedure TovDatabase.RegisterTableChangeListener(Listener:
  ITableChangeListener);
begin
  FRegisteredListeners.Add(Listener);
end;

procedure TovDatabase.UnRegisterTableChangeListener(Listener:
  ITableChangeListener);
begin
  FRegisteredListeners.Remove(Listener);
end;

{
********************************************************************************
                                
********************************************************************************
}

function TovDatabase.CreateClassInstance(AClassName: String): IovClass;
begin
  if Model.FindObject(AClassName) <> nil
    then Result := TovClass.Create(Self, AClassName)
    else raise Exception.CreateFmt(' "%s"  ', [AClassName]);
end;

function TovDatabase.GetClasses(AClassName: String): IovClass;
begin
  Result := CreateClassInstance(AClassName);
end;

function TovDatabase.GetClass(AClassId: Integer): IovClass;
begin
  Result := CreateClassInstance(Model.GetObject(AClassId).Name);
end;

procedure TovDatabase.RegisterClassImplementor(AClass: TClass; AClassName:
  String);

  { TODO :  ,    ,
      .
  }
  procedure RegisterForDerivedObjects(AModelObject: TModelObject);
  var
    I: Integer;
    DerivedObjects: TModelObjects;
  begin
    DerivedObjects := TModelObjects.Create;
    try
      AModelObject.GetDerivedObjects(DerivedObjects);
      for I := 0 to DerivedObjects.Count - 1 do
      begin
        FRegisteredClasses.AddObject(DerivedObjects[I].FullName, TObject(AClass));
        RegisterForDerivedObjects(DerivedObjects[I]);
      end;
    finally
      DerivedObjects.Free
    end;
  end;

var
  AModelObject: TModelObject;
begin
  if FRegisteredClasses.IndexOf(AClassName) >= 0 then
    raise Exception.Create(' "%s"  ')
  else
    FRegisteredClasses.AddObject(AClassName, TObject(AClass));

  AModelObject := Model.GetObject(AClassName);
  RegisterForDerivedObjects(AModelObject)
end;

function TovDatabase.GetClassImplementor(AClassName: String; var AClass:
  TClass): Boolean;
var
  I: Integer;
begin
  I := FRegisteredClasses.IndexOf(AClassName);
  if I >= 0 then
    AClass := TClass(FRegisteredClasses.Objects[I])
  else
    AClass := nil;
  Result := I >= 0;
end;

procedure TovDatabase.UnregisterClassImplementor(AClass: TClass; AClassName:
  String = '');
var
  I: Integer;
begin
  for I := FRegisteredClasses.Count - 1 downto 0 do
    if (AClassName = '') or (AClassName = FRegisteredClasses[I])
      then FRegisteredClasses.Delete(I);
end;

function TovDatabase.OpenSQLQuery(Query: String; MasterDataSource: TDataSource
  = nil; LiveDataSet: Boolean = False): TDataSet;
begin
  Result := DatabaseManager.QueryManager.OpenQuery(Query, MasterDataSource, LiveDataSet);
end;

procedure TovDatabase.ExecSQLQuery(AQuery: String);
begin
  DatabaseManager.QueryManager.ExecQuery(AQuery);
end;

{***************************************************************************** }
{                                             }
{ **************************************************************************** }

function TovDatabase.OpenQuery(AQuery: String): TDataSet;
var
  OQLProcessor: TQueryProcessor;
  SQLQuery: String;
begin
  OQLProcessor := TQueryProcessor.Create;
  try
    OQLProcessor.Model := Model;
    OQLProcessor.DatabaseAdapter := DatabaseManager.DatabaseAdapter;
    SQLQuery := OQLProcessor.ProcessQuery(AQuery);
    Result := DatabaseManager.DatabaseAdapter.OpenQuery(SQLQuery)
  finally
    OQLProcessor.Free;
  end;
end;

{{
       , 
      OpenQuery.  Field  
  .
}
function TovDatabase.GetBooleanAttributeValue(Field: TField): Boolean;
begin
  if Adapter.SupportsBooleanType then
    Result := Field.AsBoolean
  else
    Result := Not Field.IsNull and (Field.Value <> FalseValueReplacement)
end;

end.



