unit ovDatabaseManager;

interface

uses SysUtils, Classes, Variants, DB,
     ovIObjects, ovQueryManager;

type

  TovDatabaseManager = class(TObject)
  private
    FDatabaseAdapter: IDatabaseAdapter;
    FTransactionLevel: Integer;
    FQueryManager: TovQueryManager;
    procedure SetDatabaseAdapter(Value: IDatabaseAdapter);
    function GetDBConnected: Boolean;
  protected
  public
    destructor Destroy; override;
    // 
    procedure CommitTransaction;
    procedure RollbackTransaction;
    procedure StartTransaction;
    function InTransaction: Boolean;
    procedure CloseDatabase;
    procedure OpenDatabase;
    property TransactionLevel: Integer read FTransactionLevel;

    //  
    property DatabaseAdapter: IDatabaseAdapter read FDatabaseAdapter write
      SetDatabaseAdapter;
    property QueryManager: TovQueryManager read FQueryManager;
    property DBConnected: Boolean read GetDBConnected;
  end;

implementation

destructor TovDatabaseManager.Destroy;
begin
  FreeAndNil(FQueryManager);
  inherited Destroy;
end;

procedure TovDatabaseManager.SetDatabaseAdapter(Value: IDatabaseAdapter);
begin
  if FDatabaseAdapter <> Value then
  begin
    FDatabaseAdapter := Value;
    FreeAndNil(FQueryManager);
    if Value <> nil then begin
      FQueryManager := TovQueryManager.Create(Value);
    end
  end
end;

procedure TovDatabaseManager.OpenDatabase;
begin
  if Not Assigned(DatabaseAdapter) then raise Exception.Create('   ');
  DatabaseAdapter.OpenDatabase;
end;

procedure TovDatabaseManager.CloseDatabase;
begin
  if Assigned(DatabaseAdapter) then
  begin
    DatabaseAdapter.CloseDatabase;
  end
end;

function TovDatabaseManager.GetDBConnected: Boolean;
begin
  Result := Assigned(DatabaseAdapter);
end;

{
****************************    ***************************
}

{{
        
 .        
 .
}
procedure TovDatabaseManager.StartTransaction;
begin
  if (TransactionLevel > 0) and Not InTransaction then
    raise Exception.Create('     ');

  if Not InTransaction then
  begin
    DatabaseAdapter.StartTransaction;
    if WriteToLogEnabled then WriteMsgToLog('  ');
  end;
  Inc(FTransactionLevel);
end;

{{
        
 .       
  .
}
procedure TovDatabaseManager.CommitTransaction;
begin
  if Not InTransaction then
  begin
    if TransactionLevel <= 0 then
      raise Exception.Create('   ')
    else
      raise Exception.Create('     ');
  end;

  Dec(FTransactionLevel);
  if TransactionLevel = 0 then
  begin
    DatabaseAdapter.CommitTransaction;
    if WriteToLogEnabled then WriteMsgToLog('  ');
  end;
end;

{{
       
 .        
   .
}
procedure TovDatabaseManager.RollbackTransaction;
begin
  if TransactionLevel = 0 then
    raise Exception.Create('   ');

  if InTransaction then
  begin
    DatabaseAdapter.RollbackTransaction;
    if WriteToLogEnabled then WriteMsgToLog('  ');
  end;
  Dec(FTransactionLevel);
end;

function TovDatabaseManager.InTransaction: Boolean;
begin
  if Assigned(DatabaseAdapter) then
    Result := DatabaseAdapter.InTransaction
  else
    Result := False
end;

end.
