unit ovIObjects;

interface

uses DB, Classes, ovTypes, ovModel;

type
  TQuerySyntaxParams = record
    LeftTableDelimiter: String;         //      SQL-
    RightTableDelimiter: String;        //      SQL-
    LeftFieldDelimiter: String;         //      SQL-
    RightFieldDelimiter: String;        //      SQL-
    FilterLeftFieldDelimiter: String;   //        
    FilterRightFieldDelimiter: String;  //        
  end;

  {    (     ) }
  TDeleteMode =
    (dmCheckRefs,          //       
                           //  
     dmDeleteRefsObjects,  //       
                           //     
     dmDeleteRefs,         //     
     dmIgnoreRefs          //      
    );

  IDatabaseAdapter = interface(IUnknown)
    //  

    function GetOnCloseDatabase: TNotifyEvent;
    procedure SetOnCloseDatabase(const Value: TNotifyEvent);

    //  

    procedure CommitTransaction;
    procedure RollbackTransaction;
    procedure StartTransaction;
    function InTransaction: Boolean;

    //   

    {{
          TableName   
      .
    }
    procedure CreateTable(TableName: String;
      FieldsDefs: TFieldDefs; IndexDefs: TIndexDefs);
    {{
          TableName   
      ,        SourceTable 
     .    SourceTableFieldNames  
        SourceTable     
     ,  :
         i       
      SourceTableFieldNames[i]   .
    }
    procedure CloneTable(SourceTable: TDataSet; SourceTableFieldNames: TStrings;
      TableName: String; FieldsDefs: TFieldDefs; IndexDefs: TIndexDefs);

    //   

    function OpenTable(TableName: String; ReadOnly: Boolean = False; Filter: String = ''): TDataSet;
    function TableExists(TableName: String): Boolean;
    function OpenQuery(QueryText: String; MasterDataSource: TDataSource = nil;
      RequestLive: Boolean = False): TDataSet;
    procedure ExecQuery(SQLText: String);

    {{
          
      .      
    IdFieldName.
    }
    function GetLastKeyValue(TableName, IdFieldName: String): Integer;

    {{
           , 
          .
    }
    function GetNewRecordId(ATableName: String): Integer;
    procedure SetNewRecordId(ATableName: String; AValue: Integer);

    //     
    function GetQuerySyntaxParams: TQuerySyntaxParams;
    function ValueToSQLString(const Value: Variant; FieldType: TFieldType;
      FieldSize: Integer): String;
    {{
       True,     .
    }
    function SupportsBooleanType: Boolean;
    {{
       True,      
     ,   ..,   .
    }
    function SupportsLocalisedNames: Boolean;

    //    

    //    
    procedure OpenDatabase;
    //    
    procedure CloseDatabase;


    //          ,
    //  ..    CloseDatabase
    property OnCloseDatabase: TNotifyEvent read GetOnCloseDatabase write
        SetOnCloseDatabase;
  end;

  IovObject = interface;
  IovProperty = interface;
  IovClassObjects = interface;
  IovPropertyObjects = interface;
  IovObjectList = interface;
  IovObjectSpace = interface;

  IovClass = interface(IInterface)
  ['{2F0DCE95-33AE-4E38-AB50-227E9E8A12DF}']
    function CreateObject: IovObject;
    procedure DeleteAllObjects;
    procedure DeleteObject(ObjectId: TOID);
    function GetName: String;
    function FindObject(ObjectId: TOID): Boolean;
    function GetObject(ObjectId: TOID): IovObject;
    function GetRealObject(ObjectId: TOID): IovObject;
    function GetObjects: IovClassObjects;
    function GetObjectSpace: IovObjectSpace;
    {{
         
    IovObject.UpdateCalculatedAttributes.
    }
    procedure UpdateCalculatedAttributes;
    {{
       ,  
    WhereExpression.  AttributeValues    
    < ,  >.   ( ,  
      )     
    , .     .
    }
    procedure SetObjectsAttributes(const AttributeValues: Variant; const
      WhereExpression: String);
    {{
     ,   WhereExpression.  
    (  )     
    , .     .
    }
    procedure DeleteObjects(const WhereExpression: String);
    property Name: String read GetName;
    property Objects: IovClassObjects read GetObjects;
    property ObjectSpace: IovObjectSpace read GetObjectSpace;
  end;
  
  TCopyObjectEvent = procedure (Source, Target: IovObject) of object;

  IovObject = interface(IInterface)
  ['{65489B72-2A93-435A-9700-849EC1CD62B1}']
    procedure Assign(Source: IovObject; CopyAttributes: String = '*';
      DontCopyAttributes: String = ''; CopyRelations: String = '*';
      DontCopyRelations: String = ''; OnCopyObject: TCopyObjectEvent = nil);
    procedure BeginUpdate;
    function Copy: IovObject;
    procedure Delete(DeleteMode: TDeleteMode = dmCheckRefs);
    procedure EndUpdate;
    function GetObjectId: TOID;
    function GetParentClass: IovClass;
    function GetProperties(PropertyName: String): IovProperty;
    procedure Update;
    {{
            
    IovClass.CreateObject   Commit   ,  
         ( )    
     ,    .
      Commit       
     ,   Commit   .
         Commit   ,   
         .
    }
    procedure Commit;
    function GetClassName: String;
    procedure Validate; overload;
    function Validate(ValidateAttributes, ValidateRelations, ValidateUserRules:
      Boolean; ValidateErrors: TStrings): Boolean; overload;
    {{
           Persistent 
         .
    }
    procedure UpdateCalculatedAttributes;
    {{
      ,    .
    }
    function GetReferencingObjects(ImmediateObjects: Boolean = True): IovObjectList;
    function IsSame(AObject: IovObject): Boolean;
    function GetOwner: IovObject;

    {{
           .
    }
    function GetAncestor(AGeneralization: TModelRelation): IovObject;
    {{
         .   
    AParentObject     ARelationName, 
      .  ARelationName   , 
    AParentObject = nil.
    }
    procedure SetOwner(AParentObject: IovObject; ARelationName: String = '');
    function IsExist: Boolean;
    function GetModelSpace: TDomainModel;
    function GetModelObject: TModelObject;
    function GetClassId: TOID;
    property ObjectId: TOID read GetObjectId;
    property ClassId: TOID read GetClassId;
    property ParentClass: IovClass read GetParentClass;
    property Properties[PropertyName: String]: IovProperty read GetProperties;
            default;
    property ClassName: String read GetClassName;
    {{
       -    ( 
      ).
    }
    property Owner: IovObject read GetOwner;
    property ModelSpace: TDomainModel read GetModelSpace;
    property ModelObject: TModelObject read GetModelObject;
  end;

  IovBlobStream = interface(IUnknown)
    function GetStream(Mode: TBlobStreamMode): TStream;
    procedure Post;
    procedure Cancel;
    procedure LoadFromFile(FileName: String);
    procedure SaveToFile(FileName: String);
  end;

  IovProperty = interface(IUnknown)
    function Add: IovObject; overload;
    function Add(AnObject: IovObject): TOID; overload;
    procedure Delete(Index: Integer); overload;
    procedure Delete(AnObject: IovObject); overload;
    procedure DeleteAll;
    procedure Clear;
    function GetAsBoolean: Boolean;
    function GetAsDateTime: TDateTime;
    function GetAsFloat: Double;
    function GetAsInteger: Integer;
    function GetAsObject: IovObject;
    function GetAsObjectId: Integer;
    function GetAsObjects: IovPropertyObjects;
    function GetAsString: String;
    function GetCount: Integer;
    function GetIsObject: Boolean;
    function GetObjects(Index: Integer): IovObject;
    function GetPropertyName: String;
    function GetPropertyType: String;
    function GetValue: Variant;
    procedure RemoveAt(Index: Integer); overload;
    procedure Remove(AnObject: IovObject); overload;
    procedure Remove(AObjectId: TOID); overload;
    procedure SetAsBoolean(const Value: Boolean);
    procedure SetAsDateTime(const Value: TDateTime);
    procedure SetAsFloat(const Value: Double);
    procedure SetAsInteger(const Value: Integer);
    procedure SetAsObject(const Value: IovObject);
    procedure SetAsObjectId(Value: TOID);
    procedure SetAsString(const Value: string);
    procedure SetValue(Value: Variant);
    function Add(ObjectId: TOID): TOID; overload;
    function GetIsBlob: Boolean;
    function GetAsBlobStream: IovBlobStream;
    function GetIsNull: Boolean;
    property AsBoolean: Boolean read GetAsBoolean write SetAsBoolean;
    property AsDateTime: TDateTime read GetAsDateTime write SetAsDateTime;
    property AsFloat: Double read GetAsFloat write SetAsFloat;
    property AsInteger: Integer read GetAsInteger write SetAsInteger;
    property AsObject: IovObject read GetAsObject write SetAsObject;
    property AsObjectId: TOID read GetAsObjectId write SetAsObjectId;
    property AsObjects: IovPropertyObjects read GetAsObjects;
    property AsString: String read GetAsString write SetAsString;
    property Count: Integer read GetCount;
    property IsObject: Boolean read GetIsObject;
    property Objects[Index: Integer]: IovObject read GetObjects; default;
    property PropertyName: String read GetPropertyName;
    property PropertyType: String read GetPropertyType;
    property Value: Variant read GetValue write SetValue;
    property IsBlob: Boolean read GetIsBlob;
    property AsBlobStream: IovBlobStream read GetAsBlobStream;
    property IsNull: Boolean read GetIsNull;
  end;

  IovClassObjects = interface(IUnknown)
    function Find(FilterExpr, OrderExpr: String; Objects: TOIDList): Boolean;
            overload;
    function Find(FilterExpr: String; var ObjectId: TOID): Integer; overload;
    function Find(FilterExpr: String; var AObject: IovObject): Boolean; overload;
    function GetCount: Integer;
    function GetObjects(Index: Integer): IovObject;
    procedure Refresh;
    procedure Select(FilterExpr: String; OrderExpr: String = ''); overload;
    procedure SelectAll;
    function IndexOf(ObjectId: TOID): Integer;
    {{
       AttributeValues  ,   
    Attributes (       
     ) ,   FilterExpr  
    ,   OrderExpr.
    }
    procedure Select(AttributeValues: TStrings; Attributes: String; FilterExpr:
      String = ''; OrderExpr: String = ''; IdAttribute: String = ''); overload;
    property Count: Integer read GetCount;
    property Objects[Index: Integer]: IovObject read GetObjects; default;
  end;
  
  IovPropertyObjects = interface(IovClassObjects)
    procedure Assign(Source: IovPropertyObjects);
    function Add: IovObject; overload;
    function Add(ObjectId: TOID): TOID; overload;
    function Add(AnObject: IovObject): TOID; overload;
    procedure Delete(Index: Integer); overload;
    procedure Delete(AnObject: IovObject); overload;
    procedure DeleteAll;
    procedure RemoveAt(Index: Integer); overload;
    procedure Remove(AnObject: IovObject); overload;
    procedure Remove(AObjectId: TOID); overload;
    procedure Clear;
  end;

  ITableChangeListener = interface
    procedure TableChanged(ATableName: String);
  end;

  IovObjectSpace = interface(IUnknown)
    //  
    function GetClasses(ClassName: String): IovClass;
    // 
    procedure StartTransaction;
    procedure CommitTransaction;
    procedure RollbackTransaction;
    function InTransaction: Boolean;
    function CreateObject(ClassName: String): IovObject;
    function GetObject(ClassName: String; ObjectId: TOID): IovObject; overload;
    function GetObject(AModelObject: TModelObject; ObjectId: TOID): IovObject; overload;
    function GetModel: TDomainModel; 
    function GetIsUpdating: Boolean;
    procedure CancelUpdate;
    procedure EndUpdate;
    procedure BeginUpdate;
    function OpenQuery(AQuery: String): TDataSet;
    property Classes[ClassName: String]: IovClass read GetClasses; default;
    property Model: TDomainModel read GetModel;
    property IsUpdating: Boolean read GetIsUpdating;
  end;

  IovObjectList = interface(IUnknown)
    procedure Assign(AObjects: IovObjectList);
    procedure Clear;
    procedure Add(AObject: IovObject);
    procedure Delete(Index: Integer);
    function GetObjects(Index: Integer): IovObject;
    function IndexOfObject(AObject: IovObject): Integer;
    function GetCount: Integer;
    property Objects[Index: Integer]: IovObject read GetObjects; default;
    property Count: Integer read GetCount;
  end;

implementation

end.

