unit ovModel;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, Contnrs, ovConsts;

const
  Unlimited = MaxInt; //    TargetMaxCount

type

  TModelObjects = class;
  TModelObjectList = class;
  TModelRelation = class;
  TModelAttribute = class;
  TModelAttributeList = class;
  TDomainModel = class;

  EModelObjectNotFound = class(Exception)
  end;

  EModelAttributeNotFound = class(Exception)
  end;

  EModelRelationNotFound = class(Exception)
  end;

  TClassId = Integer;
  TRelationType = (rtAssociation, rtAggregation, rtComposition, rtGeneralization);
  TRelationTypes = set of TRelationType;

  TModelElement = class
  end;

  TModelDataType = class(TModelElement)
  private
    FName: String;
    FBaseType: String;
    FDescription: String;
    FSize: Integer;
    FNamespace: String;
  public
    constructor Create(ADataTypeName: String; ABaseType: String = ''; ASize:
      Integer = 0);
    property Name: String read FName write FName;
    property BaseType: String read FBaseType write FBaseType;
    property Description: String read FDescription write FDescription;
    property Size: Integer read FSize write FSize;
    property Namespace: String read FNamespace write FNamespace;
  end;

  TModelObjectIndex = class(TModelElement)
  private
    FAttributes: String;
    FName: String;
    FOptions: TIndexOptions;
  public
    constructor Create(const AName, AAttributes: String; AOptions: TIndexOptions);
    property Name: String read FName write FName;
    property Attributes: String read FAttributes write FAttributes;
    property Options: TIndexOptions read FOptions write FOptions;
  end;

  TModelObject = class(TModelElement)
  private
    FAttributes: TModelAttributeList;
    FParentRelations: TObjectList;
    FAncestorRelations: TObjectList;
    FIsAbstract: Boolean;
    FModel: TDomainModel;
    FName: String;
    FTableName: String;
    FChildCount: Integer;
    FId: Integer;
    FNamespace: String;
    FFullName: String;
    FRelations: TObjectList;
    FDescription: String;
    FIndexes: TObjectList;
    FVisibleForUser: Boolean;
    function AddAttributeInternal(AttributeName: String; AttributeType: String;
      AttributeSize: Integer; ParentObject: TModelObject): TModelAttribute;
    procedure DeleteAttribute(AttributeName: String; AllowDeleteInherited:
      Boolean); overload;
    procedure DeleteRelation(ARelationName: String; AllowDeleteInherited:
      Boolean); overload;
    function GetAttributeCount: Integer;
    function GetAttributes(Index: Integer): TModelAttribute;
    function AddRelationInternal(ARelationName, ATargetObjectName: String;
      ARelationType: TRelationType; RelationId: Integer; AParentObject: 
      TModelObject): TModelRelation;
    function GetParentCount: Integer;
    function GetAncestorCount: Integer;
    function GetParents(Index: Integer): TModelObject;
    function GetAncestors(Index: Integer): TModelObject;
    function GetAncestorRelations(Index: Integer): TModelRelation;
    function GetParentRelations(Index: Integer): TModelRelation;
    procedure AddAncestors(AParentObject: TModelObject);
    procedure DeleteAncestors(AParentObject: TModelObject);
    procedure SetName(const Value: String);
    procedure SetNamespace(const Value: String);
    function GetRelationCount: Integer;
    function GetRelations(Index: Integer): TModelRelation;
    procedure SetFullName(const NewFullName: String);
    function GetIndexCount: Integer;
    function GetIndexes(Index: Integer): TModelObjectIndex;
  public
    constructor Create(AModel: TDomainModel);
    destructor Destroy; override;
    function AddAttribute(AttributeName: String; AttributeType: String;
      AttributeSize: Integer = 0): TModelAttribute;
    function AddGeneralization(AParentObjectName: String): TModelRelation;
    {{
            . 
    
       ,     
    
      . FieldName     
     
     1        ,  
    . TableName      
     1    ,     
    .
    }
    function AddRelation(RelationName, TargetObjectName: String; RelationType: 
      TRelationType; RelationId: Integer = 0): TModelRelation;
    procedure DeleteAttribute(AttributeName: String); overload;
    procedure DeleteGeneralization(AParentObjectName: String);
    procedure DeleteRelation(ARelationName: String); overload;
    function FindAttribute(AttributeName: String): TModelAttribute;
    function FindRelation(RelationName: String): TModelRelation; overload;
    function GetAttribute(AttributeName: String): TModelAttribute;
    {{
       (  AnObjects.Objects)   
    
    (  AnObjects.Strings),     
    .
         AnObjects.Strings
     .  IncludeInherited=True,    
     
     .
    }
    procedure GetReferencedObjects(AnObjects: TStrings; IncludeInherited:
      Boolean);
    {{
       (  AnObjects.Objects)   
    (  AnObjects.Strings),     
    .
         AnObjects.Strings
     .  IncludeInherited=True,    
     
     .
    }
    procedure GetReferencingObjects(AnObjects: TStrings; IncludeInherited:
      Boolean);
    {{
           
    AParentName.
    }
    function GetParentRelation(AParentName: String): TModelRelation;

    property AttributeCount: Integer read GetAttributeCount;
    property Attributes[Index: Integer]: TModelAttribute read GetAttributes;
    property IsAbstract: Boolean read FIsAbstract write FIsAbstract;
    property Model: TDomainModel read FModel;
    property Name: String read FName write SetName;
    property TableName: String read FTableName write FTableName;

    {{
        .
    }
    property ChildCount: Integer read FChildCount;
    procedure GetDerivedObjects(AObjects: TModelObjects; Recursive: Boolean =
      False);
    procedure Delete;
    function IsInheritedFrom(AObject: TModelObject): Boolean;
    function GetRelation(RelationName: String): TModelRelation;
    function AddIndex(IndexName: String; const AAttributes: String = ''; AOptions:
      TIndexOptions = []): TModelObjectIndex; overload;
    procedure DeleteIndex(AModelObjectIndex: TModelObjectIndex);
    function FindRelation(Id: Integer): TModelRelation; overload;
    {{
        
    }
    property AncestorCount: Integer read GetAncestorCount;
    {{
         Index. 
       0  AncestorCount- 1.
    }
    property Ancestors[Index: Integer]: TModelObject read GetAncestors;
    {{
            Index. 
       0  AncestorCount- 1.
    }
    property AncestorRelations[Index: Integer]: TModelRelation read
      GetAncestorRelations;
    {{
        
    }
    property ParentCount: Integer read GetParentCount;
    {{
          Index. 
       0  ParentCount - 1.
    }
    property Parents[Index: Integer]: TModelObject read GetParents;
    {{
           
     Index.     0  ParentCount- 1.
    }
    property ParentRelations[Index: Integer]: TModelRelation read
      GetParentRelations;
    property Id: Integer read FId write FId;
    property Namespace: String read FNamespace write SetNamespace;
    property FullName: String read FFullName;
    property RelationCount: Integer read GetRelationCount;
    property Relations[Index: Integer]: TModelRelation read GetRelations;
    property Description: String read FDescription write FDescription;
    property IndexCount: Integer read GetIndexCount;
    property Indexes[Index: Integer]: TModelObjectIndex read GetIndexes;
    property VisibleForUser: Boolean read FVisibleForUser write FVisibleForUser;
  end;

  {{
      .    .
  }
  TModelObjects = class(TObjectList)
  private
    function GetItems(Index: Integer): TModelObject;
  public
    constructor Create;
    property Items[Index: Integer]: TModelObject read GetItems; default;
  end;
  
  TModelObjectList = class(TObjectList)
  private
    FModel: TDomainModel;
    FCachedIndex: Integer;
    function GetItems(Index: Integer): TModelObject;
  public
    constructor Create(AModel: TDomainModel);
    function Add: TModelObject;
    {{
         .    ,   nil.
    }
    function FindByName(ObjectName: String): TModelObject;
    function FindById(ObjectId: Integer): TModelObject;
    property Items[Index: Integer]: TModelObject read GetItems; default;
    property Model: TDomainModel read FModel;
  end;

  {       }
  TDeleteMode =
    (dmBlockDelete,    //       
                       //  
     dmDeleteRefs,     //      
     dmDeleteParents,  //       
                       //   
     dmIgnoreRefs      //      
    );

  TModelRelation = class(TModelElement)
  private
    FModel: TDomainModel;
    FName: String;
    FSourceObject: TModelObject;
    FTableName: String;
    FRelationType: TRelationType;
    FTargetMaxCount: Integer;
    FTargetMinCount: Integer;
    FTargetObject: TModelObject;
    FSourceObjectName: String;
    FTargetObjectName: String;
    FParentRelation: TModelRelation;
    FId: Integer;
    FParentObject: TModelObject;
    FIsInherited: Boolean;
    FFieldName: String;
    FRequired: Boolean;
    FRequiredMsg: String;
    FDescription: String;
    FDeleteMode: TDeleteMode;
    FVisibleForUser: Boolean;
    FDBMapping: TObject;
    FBiDirectional: Boolean;
    FDerived: Boolean;
    procedure SetTableName(const Value: String);
    procedure SetRelationType(Value: TRelationType);
    procedure SetSourceObjectName(const Value: String);
    procedure SetTargetObjectName(const Value: String);
    function GetSourceObject: TModelObject;
    function GetTargetObject: TModelObject;
    function GetTableName: String;
    function GetTargetMaxCount: Integer;
    procedure SetTargetMaxCount(Value: Integer);
    function GetTargetMinCount: Integer;
    procedure SetTargetMinCount(Value: Integer);
    function GetRelationType: TRelationType;
    function GetTargetObjectNameSpace: String;
    function GetFieldName: String;
    procedure SetFieldName(const Value: String);
    function GetName: String;
    procedure SetName(const Value: String);
    function GetSourceObjectNameSpace: String;
    function GetRequired: Boolean;
    procedure SetRequired(const Value: Boolean);
    function GetRequiredMsg: String;
    procedure SetRequiredMsg(const Value: String);
    function GetDescription: String;
    procedure SetDescription(const Value: String);
    function GetDeleteMode: TDeleteMode;
    procedure SetDeleteMode(const Value: TDeleteMode);
    function GetVisibleForUser: Boolean;
    procedure SetVisibleForUser(const Value: Boolean);
    procedure SetDBMapping(const Value: TObject);
    function GetBiDirectional: Boolean;
    procedure SetBiDirectional(const Value: Boolean);
    function GetDerived: Boolean;
    function GetBackwardRelation: TModelRelation;
    procedure SetDerived(const Value: Boolean);
    procedure SetId(const Value: Integer);
  public
    constructor Create(AModel: TDomainModel; ARelationName, ASourceObjectName, 
      ATargetObjectName: String; ARelationType: TRelationType);
    function IsAggregation: Boolean;
    function IsComposition: Boolean;
    function IsGeneralization: Boolean;
    {{
     True,       
      1, ..      
        1.
    }
    function IsReference: Boolean;
    function IsAssociation: Boolean;
    destructor Destroy; override;
    procedure DeleteBackwardRelation;
    procedure CreateBackwardRelation;
    property Model: TDomainModel read FModel;
    property Name: String read GetName write SetName;
    property SourceObject: TModelObject read GetSourceObject;
    property ParentObject: TModelObject read FParentObject;
    property TableName: String read GetTableName write SetTableName;
    property RelationType: TRelationType read GetRelationType write SetRelationType;
    property TargetMaxCount: Integer read GetTargetMaxCount write SetTargetMaxCount;
    property TargetMinCount: Integer read GetTargetMinCount write SetTargetMinCount;
    property TargetObject: TModelObject read GetTargetObject;
    {{
        
    }
    property FieldName: String read GetFieldName write SetFieldName;
    property SourceObjectName: String read FSourceObjectName write
      SetSourceObjectName;
    property TargetObjectName: String read FTargetObjectName write
      SetTargetObjectName;
    {{
       ,      
    ,    .    ,
        nil.
    }
    property ParentRelation: TModelRelation read FParentRelation;
    property IsInherited: Boolean read FIsInherited;
    property TargetObjectNameSpace: String read GetTargetObjectNameSpace;
    property Id: Integer read FId write SetId;
    property SourceObjectNameSpace: String read GetSourceObjectNameSpace;
    property Required: Boolean read GetRequired write SetRequired;
    property RequiredMsg: String read GetRequiredMsg write SetRequiredMsg;
    property Description: String read GetDescription write SetDescription;
    property DeleteMode: TDeleteMode read GetDeleteMode write SetDeleteMode;
    property VisibleForUser: Boolean read GetVisibleForUser write SetVisibleForUser;
    property DBMapping: TObject read FDBMapping write SetDBMapping;

    {    }

    {{
          
               
     BackwardRelation.
    }
    property BiDirectional: Boolean read GetBiDirectional write SetBiDirectional;
    {{
         ,   ,
        BiDirectional  Derived.
    }
    property BackwardRelation: TModelRelation read GetBackwardRelation;
    {{
          
               
     BackwardRelation.
    }
    property Derived: Boolean read GetDerived write SetDerived;
  end;

  TModelRelationList = class(TObjectList)
  private
    FModel: TDomainModel;
    function GetItems(Index: Integer): TModelRelation;
  public
    constructor Create(AModel: TDomainModel);
    function Add(ASourceObjectName, ATargetObjectName, ARelationName: String;
      ARelationType: TRelationType): TModelRelation; overload;
    procedure Delete(Relation: TModelRelation);
    function FindGeneralization(SourceObject, TargetObject: TModelObject):
      TModelRelation;
    function FindByName(SourceObjectName, RelationName: String): TModelRelation;
    function FindById(RelationId: Integer): TModelRelation;
    property Items[Index: Integer]: TModelRelation read GetItems; default;
    property Model: TDomainModel read FModel;
  end;

  TModelAttribute = class(TModelElement)
  private
    FFieldName: String;
    FName: String;
    FParentObject: TModelObject;
    FSize: Integer;
    FTypeName: String;
    FOwner: TModelObject;
    FIsInherited: Boolean;
    FParentAttribute: TModelAttribute;
    FCalculated: Boolean;
    FRequired: Boolean;
    FRequiredMsg: String;
    FDescription: String;
    FPersistent: Boolean;
    FVisibleForUser: Boolean;
    procedure SetFieldName(const Value: String);
    procedure SetSize(Value: Integer);
    procedure SetTypeName(const Value: String);
    function GetFieldName: String;
    function GetSize: Integer;
    function GetTypeName: String;
    function GetName: String;
    procedure SetName(const Value: String);
    function GetCalculated: Boolean;
    procedure SetCalculated(const Value: Boolean);
    function GetRequired: Boolean;
    procedure SetRequired(const Value: Boolean);
    function GetRequiredMsg: String;
    procedure SetRequiredMsg(const Value: String);
    function GetDescription: String;
    procedure SetDescription(const Value: String);
    function GetPersistent: Boolean;
    procedure SetPersistent(const Value: Boolean);
    function GetVisibleForUser: Boolean;
    procedure SetVisibleForUser(const Value: Boolean);
  public
    {{
     True,      
        1, ..    
    
        1.
    }
    constructor Create(AName: String; AOwner, AParent: TModelObject);
    procedure GetFieldDef(var AFieldType: TFieldType; var AFieldSize: Integer);
    function IsBlob: Boolean;
    function IsBoolean: Boolean;
    function IsMemo: Boolean;
    property FieldName: String read GetFieldName write SetFieldName;
    property Name: String read GetName write SetName;
    property Size: Integer read GetSize write SetSize;
    property TypeName: String read GetTypeName write SetTypeName;
    {{
     ,     .
    }
    property Owner: TModelObject read FOwner;
    {{
     ,      .   
    ,       ,   
     , ..   Owner.
    }
    property ParentObject: TModelObject read FParentObject;
    property IsInherited: Boolean read FIsInherited;
    {{
       ,      
     (..   Parent),    . 
      ,     nil.
    }
    property ParentAttribute: TModelAttribute read FParentAttribute;
    property Calculated: Boolean read GetCalculated write SetCalculated;
    property Required: Boolean read GetRequired write SetRequired;
    property RequiredMsg: String read GetRequiredMsg write SetRequiredMsg;
    property Description: String read GetDescription write SetDescription;
    property Persistent: Boolean read GetPersistent write SetPersistent;
    property VisibleForUser: Boolean read GetVisibleForUser write SetVisibleForUser;
  end;

  TModelAttributeList = class(TObjectList)
  private
    FOwner: TModelObject;
    function AddAttribute(AName, AType: String; ASize: Integer; AParent:
      TModelObject): TModelAttribute;
  protected
    function GetItems(Index: Integer): TModelAttribute;
  public
    procedure DeleteAttribute(Attribute: TModelAttribute);
    {{
        .
        ,   nil.
    }
    function FindAttribute(AName: String): TModelAttribute;
    constructor Create(AOwner: TModelObject);
    property Items[Index: Integer]: TModelAttribute read GetItems; default;
    property Owner: TModelObject read FOwner;
  end;

  {{
          
   .
  }
  TGetModelEvent = function (ModelName: String): TDomainModel of object;

  TGenerateObjectTableNameEvent = procedure (AModelObject: TModelObject;
    DefaultName: String; var NewName: String) of object;
  TGenerateAttributeFieldNameEvent = procedure (Attribute: TModelAttribute;
    DefaultName: String; var NewName: String) of object;
  TGenerateRelationTableOrFieldNameEvent = procedure (Relation: TModelRelation;
    DefaultTableName, DefaultFieldName: String;
    var NewTableName, NewFieldName: String) of object;

  TDomainModel = class(TObject)
  private
    FNewRelationId: Integer;
    FObjects: TModelObjectList;
    FRelations: TModelRelationList;
    FNewObjectId: Integer;
    FNameSpaces: TStrings;
    FAutoGenerateNames: Boolean;
    FOnGenerateObjectTableName: TGenerateObjectTableNameEvent;
    FOnGenerateAttributeFieldName: TGenerateAttributeFieldNameEvent;
    FOnGenerateRelationTableOrFieldName: TGenerateRelationTableOrFieldNameEvent;
    FDataTypes: TObjectList;
    function GetNewRelationId: Integer;
    function GetNewObjectId: Integer;
    procedure SetNewRelationId(Value: Integer);
    procedure SetNewObjectId(Value: Integer);
    function GetObjectCount: Integer;
    function GetObjects(Index: Integer): TModelObject;
    function GetRelationCount: Integer;
    function GetRelations(Index: Integer): TModelRelation;
    function GenerateUniqueFieldName(AObject: TModelObject; APrefix: String):
      string;
    procedure GenerateObjectTableName(AModelObject: TModelObject; RenameExist:
      Boolean; ObjectIndex: Integer);
    procedure GenerateAttributeFieldName(AModelObject: TModelObject;
      AModelAttribute: TModelAttribute; RenameExist: Boolean; AttributeIndex:
      Integer);
    procedure GenerateRelationTableOrFieldName(AModelObject: TModelObject;
      AModelRelation: TModelRelation; RenameExist: Boolean);
    function GetDataTypeCount: Integer;
    function GetDataTypes(Index: Integer): TModelDataType;
  protected
    function GenerateUniqueTableName(APrefix: String): String;
  public
    constructor Create;
    destructor Destroy; override;
    function AddObject(ObjectName: String; NameSpace: String = ''; ObjectId:
      Integer = 0): TModelObject;
    procedure Clear;
    procedure DeleteObject(AObject: TModelObject);
    function FindObject(AnObjectName: String): TModelObject; overload;
    function GetObject(ObjectName: String; NameSpace: String = ''): TModelObject;
      overload;
    function GetObject(AnObjectId: Integer): TModelObject; overload;
    function FindObject(AnObjectId: Integer): TModelObject; overload;
    function GetObjectAttribute(AnObjectName, AnAttributeName: String):
      TModelAttribute;
    function GetObjectRelation(AnObjectName, ARelationName: String): TModelRelation;
    procedure DeleteObjects(NameSpace: String);
    procedure RenameNameSpace(OldName, NewName: String);
    function GetRelation(RelationId: Integer): TModelRelation;
    procedure GenerateDBNames(RenameExist: Boolean);
    function FindDataType(ADataTypeName: String): TModelDataType;
    function AddDataType(ADataTypeName: String; ABaseType: String = ''; ASize:
      Integer = 0): TModelDataType; overload;
    procedure DeleteDataTypes;
    procedure DeleteDataType(AModelDataType: TModelDataType);
    function FindRelation(RelationId: Integer): TModelRelation;
    {{
           
         Tblnnn  Fldnnn .  ,
              
     .
    }
    property AutoGenerateNames: Boolean read FAutoGenerateNames write
        FAutoGenerateNames;
    property ObjectCount: Integer read GetObjectCount;
    property Objects[Index: Integer]: TModelObject read GetObjects;
    property RelationCount: Integer read GetRelationCount;
    property Relations[Index: Integer]: TModelRelation read GetRelations;
    property NameSpaces: TStrings read FNameSpaces;
    property DataTypeCount: Integer read GetDataTypeCount;
    property DataTypes[Index: Integer]: TModelDataType read GetDataTypes;

    {  }

    property OnGenerateObjectTableName: TGenerateObjectTableNameEvent read
      FOnGenerateObjectTableName write FOnGenerateObjectTableName;
    property OnGenerateAttributeFieldName: TGenerateAttributeFieldNameEvent read
      FOnGenerateAttributeFieldName write FOnGenerateAttributeFieldName;
    property OnGenerateRelationTableOrFieldName:
      TGenerateRelationTableOrFieldNameEvent read
      FOnGenerateRelationTableOrFieldName write FOnGenerateRelationTableOrFieldName;
  end;


function MakeFullObjectName(ObjectName, NameSpace: String; Separator: String =
  NameSpaceSeparator): String;
procedure ParseFullObjectName(FullName: String; var ObjectName, NameSpace:
  String);
function ExtractObjectName(FullName: String): String;

function CardinalityToString(Cardinality: Integer): String;

function StringToCardinality(Cardinality: String): Integer;

implementation

uses ovObjects, ovUtils, TypInfo;

resourcestring
  SClassTablePrefix = 'Class';
  SRelationTablePrefix = 'Relation';
  SAttributeFieldPrefix = 'Attr';
  SReferenceFieldPrefix = 'Ref';
  SParentFieldPrefix = 'Parent';

function MakeFullObjectName(ObjectName, NameSpace: String; Separator: String =
  NameSpaceSeparator): String;
begin
  if Pos(NameSpaceSeparator, ObjectName) > 0 then
    Result := ObjectName
  else
  if NameSpace = ''
    then Result := ObjectName
    else Result := NameSpace + Separator + ObjectName
end;

{{
          
.
}
procedure ParseFullObjectName(FullName: String; var ObjectName, NameSpace:
  String);
var
  P: Integer;
begin
  P := Pos(NameSpaceSeparator, FullName);
  if P > 0 then begin
    NameSpace := Copy(FullName, 1, P-1);
    ObjectName := Copy(FullName, P+Length(NameSpaceSeparator), Length(FullName));
  end else begin
    NameSpace := '';
    ObjectName := FullName;
  end
end;

{{
       .
}
function ExtractObjectName(FullName: String): String;
var
  P: Integer;
begin
  P := Pos(NameSpaceSeparator, FullName);
  if P > 0 then
    Result := Copy(FullName, P+Length(NameSpaceSeparator), Length(FullName))
  else
    Result := FullName;
end;

function CompareItems(Item1, Item2: Pointer): Integer;
begin
  Result := AnsiCompareStr(TModelObject(Item1).FullName, TModelObject(Item2).FullName)
end;

function CardinalityToString(Cardinality: Integer): String;
begin
  if Cardinality = Unlimited
    then Result := '*'
    else Result := IntToStr(Cardinality);
end;

function StringToCardinality(Cardinality: String): Integer;
begin
  if Cardinality = '*'
    then Result := Unlimited
    else begin
      Result := StrToInt(Cardinality);
      if Result < 0 then raise Exception.Create('   ');
    end
end;

{ TModelDataType }

constructor TModelDataType.Create(ADataTypeName: String; ABaseType: String =
  ''; ASize: Integer = 0);
begin
  inherited Create;
  FName := ADataTypeName;
  FBaseType := ABaseType;
  FSize := ASize;
end;

{ TModelObjectIndex }

constructor TModelObjectIndex.Create(const AName, AAttributes: String;
  AOptions: TIndexOptions);
begin
  inherited Create;
  FName := AName;
  FOptions := AOptions;
  FAttributes := AAttributes;
end;

{ TModelObject }

constructor TModelObject.Create(AModel: TDomainModel);
begin
  inherited Create;
  FVisibleForUser := True;
  FModel := AModel;
  FAttributes := TModelAttributeList.Create(Self);
  FRelations := TObjectList.Create(False);
  FAncestorRelations := TObjectList.Create(False);
  FParentRelations := TObjectList.Create(False);
  FIndexes := TObjectList.Create();
end;

destructor TModelObject.Destroy;
begin
  FreeAndNil(FIndexes);
  FParentRelations.Free;
  FAncestorRelations.Free;
  FRelations.Free;
  FAttributes.Free;
  inherited Destroy;
end;

{{
     .
}
function TModelObject.AddAttribute(AttributeName: String; AttributeType: String;
  AttributeSize: Integer = 0): TModelAttribute;
begin
  Result := AddAttributeInternal(AttributeName, AttributeType, AttributeSize, Self);
end;

{{
   ,   AddAttribute,    
  ParentObject.
}
function TModelObject.AddAttributeInternal(AttributeName: String; 
  AttributeType: String; AttributeSize: Integer; ParentObject: TModelObject): 
  TModelAttribute;
var
  I: Integer;
  ObjectList: TModelObjects;
begin
  Result := FAttributes.AddAttribute(AttributeName, AttributeType,
    AttributeSize, ParentObject);
  if Model.AutoGenerateNames then
    Result.FieldName := Model.GenerateUniqueFieldName(Self, SAttributeFieldPrefix);

  //        
  ObjectList := nil;
  if ChildCount > 0 then
  try
    ObjectList := TModelObjects.Create;
    GetDerivedObjects(ObjectList);
    for I := 0 to ObjectList.Count - 1 do
      ObjectList[I].AddAttributeInternal(AttributeName, AttributeType,
        AttributeSize, ParentObject)
  finally
    ObjectList.Free
  end;
end;

{{
        . 

   ,      
  . FieldName       
 1        ,  
. TableName      
 1    ,     
.
}
function TModelObject.AddRelation(RelationName, TargetObjectName: String;
  RelationType: TRelationType; RelationId: Integer = 0): TModelRelation;
begin
  Result := AddRelationInternal(RelationName, TargetObjectName,
    RelationType, RelationId, Self)
end;

function TModelObject.AddRelationInternal(ARelationName, ATargetObjectName:
  String; ARelationType: TRelationType; RelationId: Integer; AParentObject:
  TModelObject): TModelRelation;
var
  I: Integer;
  ObjectList: TModelObjects;
  Relation: TModelRelation;
begin
  if ARelationType = rtGeneralization then
    raise Exception.Create('  ');

  Relation := Model.FRelations.Add(FullName, ATargetObjectName,
    ARelationName, ARelationType);
  FRelations.Add(Relation);

  if RelationId = 0
    then RelationId := Model.GetNewRelationId
    else Model.SetNewRelationId(RelationId);
  Relation.FId := RelationId;
  Relation.FParentObject := AParentObject;
  Relation.FIsInherited := AParentObject <> Self;

  if Relation.IsInherited then
    //    ,    ,
    //        
    Relation.FParentRelation := AParentObject.GetRelation(Relation.Name)
  else begin
    Relation.FFieldName := ARelationName;
    Relation.FTableName := ARelationName;
  end;
  Result := Relation;

  //        
  ObjectList := nil;
  if ChildCount > 0 then
  try
    ObjectList := TModelObjects.Create;
    GetDerivedObjects(ObjectList);
    for I := 0 to ObjectList.Count - 1 do with Relation do
      ObjectList[I].AddRelationInternal(Name, TargetObjectName,
        RelationType, RelationId, AParentObject)
  finally
    ObjectList.Free
  end;
end;

procedure TModelObject.DeleteAttribute(AttributeName: String);
begin
  DeleteAttribute(AttributeName, False)
end;

procedure TModelObject.DeleteAttribute(AttributeName: String;
  AllowDeleteInherited: Boolean);
var
  Attribute: TModelAttribute;
  I: Integer;
  ObjectList: TModelObjects;
begin
  Attribute := GetAttribute(AttributeName);

  if Not AllowDeleteInherited and Attribute.IsInherited then
    raise Exception.Create('   ');

  //    
  ObjectList := nil;
  if ChildCount > 0 then
  try
    ObjectList := TModelObjects.Create;
    GetDerivedObjects(ObjectList);
    for I := 0 to ObjectList.Count - 1 do
      ObjectList[I].DeleteAttribute(AttributeName, True)
  finally
    ObjectList.Free
  end;

  FAttributes.DeleteAttribute(Attribute);
end;

procedure TModelObject.DeleteRelation(ARelationName: String);
begin
  DeleteRelation(ARelationName, False)
end;

procedure TModelObject.DeleteRelation(ARelationName: String;
  AllowDeleteInherited: Boolean);
var
  I: Integer;
  ObjectList: TModelObjects;
  Relation: TModelRelation;
begin
  Relation := Model.GetObjectRelation(FullName, ARelationName);

  if Not AllowDeleteInherited and Relation.IsInherited
  then
    raise Exception.Create('   ');
  
  //    
  ObjectList := nil;
  if ChildCount > 0 then
  try
    ObjectList := TModelObjects.Create;
    GetDerivedObjects(ObjectList);
    for I := 0 to ObjectList.Count - 1 do with Relation do
      ObjectList[I].DeleteRelation(ARelationName, True);
  finally
    ObjectList.Free
  end;

  FRelations.Remove(Relation);       //   
  Model.FRelations.Delete(Relation); //    
end;

function TModelObject.FindAttribute(AttributeName: String): TModelAttribute;
begin
  Result := FAttributes.FindAttribute(AttributeName);
end;

function TModelObject.GetAttribute(AttributeName: String): TModelAttribute;
begin
  Result := FAttributes.FindAttribute(AttributeName);
  if Result = nil then
    raise EModelAttributeNotFound.CreateFmt(' "%s"    "%s"',
      [Name, AttributeName]);
end;

function TModelObject.FindRelation(RelationName: String): TModelRelation;
var
  I: Integer;
begin
  for I := 0 to RelationCount - 1 do
    if Relations[I].Name = RelationName then
    begin
      Result := Relations[I];
      Exit;
    end;
  Result := nil;
end;

function TModelObject.FindRelation(Id: Integer): TModelRelation;
var
  I: Integer;
begin
  for I := 0 to RelationCount - 1 do
    if Relations[I].Id = Id then
    begin
      Result := Relations[I];
      Exit;
    end;
  Result := nil;
end;

function TModelObject.GetRelation(RelationName: String): TModelRelation;
begin
  Result := FindRelation(RelationName);
  if Result = nil then
    raise EModelRelationNotFound.CreateFmt(' "%s"    "%s"',
      [Name, RelationName]);
end;

function TModelObject.GetAttributeCount: Integer;
begin
  Result := FAttributes.Count
end;

function TModelObject.GetAttributes(Index: Integer): TModelAttribute;
begin
  Result := TModelAttribute(FAttributes[Index])
end;

{{
   (  AnObjects.Objects)    
(  AnObjects.Strings),      .
     AnObjects.Strings
 .  IncludeInherited=True,      
 .
}
procedure TModelObject.GetReferencedObjects(AnObjects: TStrings;
  IncludeInherited: Boolean);
var
  R: TModelRelation;
  O: TModelObject;
  I: Integer;
begin
  AnObjects.Clear;
  with Model do
  for I := 0 to RelationCount - 1 do
    if Relations[I].SourceObject = Self then
    begin
      R := Relations[I];
      O := R.TargetObject;
      if R.IsGeneralization then
        AnObjects.AddObject('', O)
      else
      if Not R.IsInherited or IncludeInherited then
        AnObjects.AddObject(R.Name, O)
    end
end;

{{
   (  AnObjects.Objects)   
(  AnObjects.Strings),      .
     AnObjects.Strings
 .  IncludeInherited=True,      
 .
}
procedure TModelObject.GetReferencingObjects(AnObjects: TStrings;
  IncludeInherited: Boolean);
var
  R: TModelRelation;
  O: TModelObject;
  I: Integer;
begin
  AnObjects.Clear;
  with Model do
  for I := 0 to RelationCount - 1 do
    if Relations[I].TargetObjectName = Self.FullName then
    begin
      R := Relations[I];
      O := R.SourceObject;
      if R.IsGeneralization then
        AnObjects.AddObject('', O)
      else
      if Not R.IsInherited or IncludeInherited then
        AnObjects.AddObject(R.Name, O)
    end
end;

{
*********************       ***************
}

function TModelObject.GetAncestorCount: Integer;
begin
  Result := FAncestorRelations.Count
end;

function TModelObject.GetAncestors(Index: Integer): TModelObject;
begin
  Result := AncestorRelations[Index].TargetObject;
end;

function TModelObject.GetAncestorRelations(Index: Integer): TModelRelation;
begin
  Result := TModelRelation(FAncestorRelations[Index]);
end;

function TModelObject.GetParentCount: Integer;
begin
  Result := FParentRelations.Count
end;

function TModelObject.GetParents(Index: Integer): TModelObject;
begin
  Result := ParentRelations[Index].TargetObject;
end;

function TModelObject.GetParentRelations(Index: Integer): TModelRelation;
begin
  Result := TModelRelation(FParentRelations[Index]);
end;

function TModelObject.GetParentRelation(AParentName: String): TModelRelation;
var
  I: Integer;
begin
  for I := 0 to ParentCount - 1 do
    if Parents[I].FullName = AParentName then
    begin
      Result := ParentRelations[I];
      Exit;
    end;
  raise Exception.CreateFmt(' "%s"      "%s"', [Name, AParentName]);
end;

{{
  ,    
.
}
procedure TModelObject.GetDerivedObjects(AObjects: TModelObjects; Recursive:
  Boolean = False);
var
  I: Integer;
begin
  with Model do
  for I := 0 to RelationCount - 1 do with Relations[I] do
    if IsGeneralization and (TargetObjectName = Self.FullName) then
    begin
      //   
      AObjects.Add(SourceObject);
      //     
      if Recursive then SourceObject.GetDerivedObjects(AObjects, True);
    end;
end;

{{
     AParentObjectName    .
      AParentObjectName  
         .
}
function TModelObject.AddGeneralization(AParentObjectName: String):
  TModelRelation;
var
  I: Integer;
  AParentObject: TModelObject;
begin
  Result := Model.FRelations.Add(FullName, AParentObjectName,
    '~~' + AParentObjectName, //     ,   
    rtGeneralization);        //          '~~' + AParentObjectName
  Result.FieldName := ExtractObjectName(AParentObjectName);

  FParentRelations.Add(Result);

  AParentObject := Model.FindObject(AParentObjectName);
  if AParentObject <> nil then
  begin
    //     AParentObject
    with AParentObject do
    for I := 0 to AttributeCount - 1 do with Attributes[I] do
      Self.AddAttributeInternal(Name, TypeName, Size, ParentObject);

    //    AParentObject (  )
    with AParentObject do
    for I := 0 to RelationCount - 1 do with Relations[I] do
      Self.AddRelationInternal(Name, TargetObject.FullName,
        RelationType, Id, ParentObject);

    FAncestorRelations.Add(Result);
    AddAncestors(AParentObject);

    Inc(AParentObject.FChildCount);
  end
end;

{{
    AParentObject  
 AncestorRelations        (
   ,   ,   
).
}
procedure TModelObject.AddAncestors(AParentObject: TModelObject);
var
  ObjectList: TModelObjects;
  I: Integer;
begin
  //       FAncestorRelations
  with AParentObject do
    for I := 0 to AncestorCount - 1 do
      Self.FAncestorRelations.Add(AncestorRelations[I]);

  //         
  ObjectList := nil;
  if ChildCount > 0 then
  try
    ObjectList := TModelObjects.Create;
    GetDerivedObjects(ObjectList);
    for I := 0 to ObjectList.Count - 1 do
      ObjectList[I].AddAncestors(Self);
  finally
    ObjectList.Free
  end;
end;

{{
     AParentObject    .
      AParentObject 
          .
}
procedure TModelObject.DeleteGeneralization(AParentObjectName: String);
var
  AParentObject: TModelObject;
  Relation: TModelRelation;
  I: Integer;
begin
  AParentObject := Model.GetObject(AParentObjectName);
  Relation := FModel.FRelations.FindGeneralization(Self, AParentObject);
  if Relation = nil then
    raise EModelRelationNotFound.CreateFmt(' %s     %s  ',
      [Name, AParentObject.Name]);

  FParentRelations.Remove(Relation);

  //      AParentObject
  for I := AttributeCount - 1 downto 0 do
    with Attributes[I] do
      if AParentObject.IsInheritedFrom(ParentObject)
        then DeleteAttribute(Name, True);

  //     AParentObject (  )
  with Model do
  for I := RelationCount - 1 downto 0 do
    with Relations[I] do
      if Not IsGeneralization and (SourceObject = Self) and
         AParentObject.IsInheritedFrom(ParentObject)
      then DeleteRelation(Name, True);

  Model.FRelations.Delete(Relation); //    

  FAncestorRelations.Remove(Relation);
  DeleteAncestors(Relation.TargetObject);

  Dec(AParentObject.FChildCount);
end;

{{
    AParentObject    AncestorRelations 
      (    ,
  ,    ).
}
procedure TModelObject.DeleteAncestors(AParentObject: TModelObject);
var
  ObjectList: TModelObjects;
  I: Integer;
begin
  //         
  ObjectList := nil;
  if ChildCount > 0 then
  try
    ObjectList := TModelObjects.Create;
    GetDerivedObjects(ObjectList);
    for I := 0 to ObjectList.Count - 1 do
      ObjectList[I].DeleteAncestors(Self);
  finally
    ObjectList.Free
  end;

  //       FAncestorRelations
  with AParentObject do
    for I := 0 to AncestorCount - 1 do
      Self.FAncestorRelations.Remove(AncestorRelations[I]);
end;

procedure TModelObject.SetName(const Value: String);
begin
  if FName <> Value then SetFullName(MakeFullObjectName(Value, Namespace));
end;

procedure TModelObject.SetNamespace(const Value: String);
begin
  if FNamespace <> Value then SetFullName(MakeFullObjectName(Name, Value));
end;

procedure TModelObject.SetFullName(const NewFullName: String);
var
  I: Integer;
  AName, ANamespace: String;
begin
  ParseFullObjectName(NewFullName, AName, ANamespace);
  if Model.FindObject(NewFullName) <> nil then
    raise Exception.CreateFmt('  "%s"    "%s"  ', [ANamespace, AName]);

  for I := 0 to Model.RelationCount - 1 do
  with Model.Relations[I] do
  begin
    if TargetObjectName = FullName then TargetObjectName := NewFullName;
    if SourceObjectName = FullName then SourceObjectName := NewFullName;
  end;

  FNamespace := ANamespace;
  FName := AName;
  FFullName := NewFullName;

  FModel.FObjects.Sort(CompareItems);
end;

procedure TModelObject.Delete;
begin
  Model.DeleteObject(Self);
end;

function TModelObject.IsInheritedFrom(AObject: TModelObject): Boolean;
var
  I: Integer;
begin
  Result := False;
  if AObject = Self then Result := True
  else begin
    for I := 0 to AncestorCount - 1 do
      if Ancestors[I] = AObject then
      begin
        Result := True;
        Break;
      end
  end
end;

function TModelObject.GetRelationCount: Integer;
begin
  Result := FRelations.Count
end;

function TModelObject.GetRelations(Index: Integer): TModelRelation;
begin
  Result := TModelRelation(FRelations[Index])
end;

{***************************************************************************** }
{                                                                       }
{ **************************************************************************** }

function TModelObject.GetIndexCount: Integer;
begin
  Result := FIndexes.Count
end;

function TModelObject.GetIndexes(Index: Integer): TModelObjectIndex;
begin
  Result := TModelObjectIndex(FIndexes[Index]);
end;

function TModelObject.AddIndex(IndexName: String; const AAttributes: String =
  ''; AOptions: TIndexOptions = []): TModelObjectIndex;
begin
  Result := TModelObjectIndex.Create(IndexName, AAttributes, AOptions);
  FIndexes.Add(Result);
end;

procedure TModelObject.DeleteIndex(AModelObjectIndex: TModelObjectIndex);
begin
  FIndexes.Remove(AModelObjectIndex);
end;

{
******************************** TModelObjects *********************************
}

constructor TModelObjects.Create;
begin
  inherited Create;
  OwnsObjects := False;
end;

function TModelObjects.GetItems(Index: Integer): TModelObject;
begin
  Result := TModelObject(inherited Items[Index])
end;

{
******************************* TModelObjectList *******************************
}
{<<Category>>: No category}
constructor TModelObjectList.Create(AModel: TDomainModel);
begin
  inherited Create(True);
  FModel := AModel;
end;

function TModelObjectList.Add: TModelObject;
begin
  Result := TModelObject.Create(Model);
  inherited Add(Result);
end;

{{
       .    ,
  nil.

       
        .
}
function TModelObjectList.FindByName(ObjectName: String): TModelObject;
var
  L, R, M, CompareResult: Integer;
begin
  if (FCachedIndex < Count) and (Items[FCachedIndex].FullName = ObjectName)
  then Result := Items[FCachedIndex]
  else begin
    L := 0;
    R := Count - 1;
    while L <= R do
    begin
      M := (L + R) div 2;
      CompareResult := AnsiCompareStr(Items[M].FullName, ObjectName);
      if CompareResult > 0 then R := M - 1 else
      if CompareResult < 0 then L := M + 1 else
      begin
        FCachedIndex := M;
        Result := Items[M];
        Exit;
      end;
    end;
    Result := nil;
  end
end;

{{
     .    ,   nil.
}
function TModelObjectList.FindById(ObjectId: Integer): TModelObject;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    if Items[I].Id = ObjectId then
    begin
      Result := Items[I];
      Exit;
    end;
  Result := nil;
end;

function TModelObjectList.GetItems(Index: Integer): TModelObject;
begin
  Result := TModelObject(inherited Items[Index])
end;

{
******************************** TModelRelation ********************************
}

constructor TModelRelation.Create(AModel: TDomainModel; ARelationName,
  ASourceObjectName, ATargetObjectName: String; ARelationType: TRelationType);
begin
  inherited Create;
  FModel := AModel;
  FName := ARelationName;
  FSourceObjectName := ASourceObjectName;
  FTargetObjectName := ATargetObjectName;
  FRelationType := ARelationType;

  FTargetMaxCount := 1;
  FVisibleForUser := True;
end;

destructor TModelRelation.Destroy;
begin
  DBMapping := nil;
  inherited Destroy;
end;

procedure TModelRelation.SetDBMapping(const Value: TObject);
begin
  if FDBMapping <> Value then
  begin
    FreeAndNil(FDBMapping);
    FDBMapping := Value;
  end;
end;

function TModelRelation.IsAssociation: Boolean;
begin
  Result := RelationType = rtAssociation
end;

function TModelRelation.IsAggregation: Boolean;
begin
  Result := RelationType = rtAggregation
end;

function TModelRelation.IsComposition: Boolean;
begin
  Result := RelationType = rtComposition
end;

function TModelRelation.IsGeneralization: Boolean;
begin
  Result := RelationType = rtGeneralization;
end;

{{
 True,        
  1.
}
function TModelRelation.IsReference: Boolean;
begin
  Result := (TargetMaxCount = 1) and (RelationType = rtAssociation);
end;

procedure TModelRelation.SetSourceObjectName(const Value: String);
begin
  if FSourceObjectName <> Value then
  begin
    FSourceObjectName := Value;
    FSourceObject := nil;
  end;
end;

procedure TModelRelation.SetTargetObjectName(const Value: String);
begin
  if FTargetObjectName <> Value then
  begin
    FTargetObjectName := Value;
    FTargetObject := nil;
  end;
end;

function TModelRelation.GetTargetObjectNameSpace: String;
var
  ObjectName, NameSpace: String;
begin
  ParseFullObjectName(TargetObjectName, ObjectName, NameSpace);
  Result := NameSpace;
end;

function TModelRelation.GetSourceObject: TModelObject;
begin
  if FSourceObject = nil then FSourceObject := Model.GetObject(SourceObjectName);
  Result := FSourceObject;
end;

function TModelRelation.GetSourceObjectNameSpace: String;
var
  ObjectName, NameSpace: String;
begin
  ParseFullObjectName(SourceObjectName, ObjectName, NameSpace);
  Result := NameSpace;
end;

function TModelRelation.GetTargetObject: TModelObject;
begin
  if FTargetObject = nil then FTargetObject := Model.GetObject(TargetObjectName);
  Result := FTargetObject;
end;

{    }

function TModelRelation.GetTableName: String;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.TableName
  else
    Result := FTableName;
end;

procedure TModelRelation.SetTableName(const Value: String);
begin
  if Assigned(ParentRelation) then
    ParentRelation.TableName := Value
  else
    FTableName := Value
end;

function TModelRelation.GetTargetMaxCount: Integer;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.TargetMaxCount
  else
    Result := FTargetMaxCount;
end;

procedure TModelRelation.SetTargetMaxCount(Value: Integer);
begin
  if Assigned(ParentRelation) then
    ParentRelation.TargetMaxCount := Value
  else
    FTargetMaxCount := Value
end;

function TModelRelation.GetTargetMinCount: Integer;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.TargetMinCount
  else
    Result := FTargetMinCount;
end;

procedure TModelRelation.SetTargetMinCount(Value: Integer);
begin
  if Assigned(ParentRelation) then
    ParentRelation.TargetMinCount := Value
  else
    FTargetMinCount := Value
end;

function TModelRelation.GetRelationType: TRelationType;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.RelationType
  else
    Result := FRelationType;
end;

procedure TModelRelation.SetRelationType(Value: TRelationType);
begin
  if Assigned(ParentRelation) then
    ParentRelation.RelationType := Value
  else
    if Value <> rtGeneralization then
      FRelationType := Value
    else
      raise Exception.Create('         RelationType');
end;

function TModelRelation.GetFieldName: String;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.FieldName
  else
    if Not Derived then
      Result := FFieldName
    else
      if BackwardRelation.IsComposition then Result := OwnerIdField
      else Result := BackwardRelation.FieldName
end;

procedure TModelRelation.SetFieldName(const Value: String);
begin
  if Assigned(ParentRelation) then
    ParentRelation.FieldName := Value
  else
    FFieldName := Value
end;

function TModelRelation.GetName: String;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.Name
  else
    Result := FName;
end;

procedure TModelRelation.SetName(const Value: String);
begin
  if Name <> Value then
  begin
    if SourceObject.FindRelation(Value) <> nil then
      raise Exception.CreateFmt(' "%s"   "%s"  ', [Value, SourceObjectName]);
    if IsInherited then
      raise Exception.Create('   :' + Name)
    else
      FName := Value
  end
end;

function TModelRelation.GetRequired: Boolean;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.Required
  else
    Result := FRequired;
end;

procedure TModelRelation.SetRequired(const Value: Boolean);
begin
  if Assigned(ParentRelation) then
    ParentRelation.Required := Value
  else
    FRequired := Value
end;

function TModelRelation.GetRequiredMsg: String;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.RequiredMsg
  else
    Result := FRequiredMsg;
end;

procedure TModelRelation.SetRequiredMsg(const Value: String);
begin
  if Assigned(ParentRelation) then
    ParentRelation.RequiredMsg := Value
  else
    FRequiredMsg := Value
end;

function TModelRelation.GetDescription: String;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.Description
  else
    Result := FDescription;
end;

procedure TModelRelation.SetDescription(const Value: String);
begin
  if Assigned(ParentRelation) then
    ParentRelation.Description := Value
  else
    FDescription := Value
end;

function TModelRelation.GetDeleteMode: TDeleteMode;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.DeleteMode
  else
    Result := FDeleteMode;
end;

procedure TModelRelation.SetDeleteMode(const Value: TDeleteMode);
begin
  if Assigned(ParentRelation) then
    ParentRelation.DeleteMode := Value
  else
    FDeleteMode := Value
end;

function TModelRelation.GetVisibleForUser: Boolean;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.VisibleForUser
  else
    Result := FVisibleForUser;
end;

procedure TModelRelation.SetVisibleForUser(const Value: Boolean);
begin
  if Assigned(ParentRelation) then
    ParentRelation.VisibleForUser := Value
  else
    FVisibleForUser := Value
end;

{***************************************************************************** }
{                                               }
{ **************************************************************************** }

function TModelRelation.GetBiDirectional: Boolean;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.BiDirectional
  else
    Result := FBiDirectional;
end;

procedure TModelRelation.SetBiDirectional(const Value: Boolean);
begin
  if Assigned(ParentRelation) then
    ParentRelation.BiDirectional := Value
  else
    FBiDirectional := Value;
end;

function TModelRelation.GetDerived: Boolean;
begin
  if Assigned(ParentRelation) then
    Result := ParentRelation.Derived
  else
    Result := FDerived;
end;

procedure TModelRelation.SetDerived(const Value: Boolean);
begin
  if Assigned(ParentRelation) then
    ParentRelation.Derived := Value
  else
    FDerived := Value;
end;

function TModelRelation.GetBackwardRelation: TModelRelation;
begin
  if (FTargetObject = nil) and (Model.FindObject(TargetObjectName) = nil)
    then Result := nil
    else Result := TargetObject.FindRelation(Id);
end;

{{
  ,   .
}
procedure TModelRelation.DeleteBackwardRelation;
begin
  if Assigned(BackwardRelation) then
    TargetObject.DeleteRelation(BackwardRelation.Name);
  Derived := False; {    ,   
        }
end;

{{
  ,     .
}
procedure TModelRelation.CreateBackwardRelation;
var
  ABackwardRelation: TModelRelation;
begin
  if Not Assigned(BackwardRelation) then
  begin
    ABackwardRelation := TargetObject.AddRelation(
      SourceObject.Name, SourceObjectName, rtAssociation);
    ABackwardRelation.TargetMinCount := 1;
    ABackwardRelation.TargetMaxCount := 1;
    if IsComposition then ABackwardRelation.FieldName := OwnerIdField else
    if IsAggregation then ABackwardRelation.FieldName := FieldName;
    ABackwardRelation.TableName := '';
    ABackwardRelation.Id := Id;
    ABackwardRelation.FBiDirectional := True;
    ABackwardRelation.FDerived := True;
  end
end;

procedure TModelRelation.SetId(const Value: Integer);
var
  ABackwardRelation: TModelRelation;
begin
  if FId <> Value then
  begin
    {       }
    ABackwardRelation := BackwardRelation;
    FId := Value;
    if ABackwardRelation <> nil then ABackwardRelation.Id := Value;
  end;
end;

{
****************************** TModelRelationList ******************************
}

constructor TModelRelationList.Create(AModel: TDomainModel);
begin
  inherited Create(True);
  FModel := AModel;
end;

function TModelRelationList.Add(ASourceObjectName, ATargetObjectName,
  ARelationName: String; ARelationType: TRelationType): TModelRelation;
begin
  if FindByName(ASourceObjectName, ARelationName) <> nil then
    raise Exception.CreateFmt(' "%s"   "%s"  ',
      [ARelationName, ASourceObjectName]);
  Result := TModelRelation.Create(Model, ARelationName,
    ASourceObjectName, ATargetObjectName, ARelationType);
  inherited Add(Result);
end;

procedure TModelRelationList.Delete(Relation: TModelRelation);
begin
  Remove(Relation);
end;

function TModelRelationList.FindGeneralization(SourceObject, TargetObject:
  TModelObject): TModelRelation;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    if Items[I].IsGeneralization  and
       (Items[I].SourceObject = SourceObject) and
       (Items[I].TargetObject = TargetObject) then
    begin
      Result := Items[I];
      Exit;
    end;
  Result := nil;
end;

function TModelRelationList.FindByName(SourceObjectName, RelationName: String):
  TModelRelation;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    if (Items[I].SourceObjectName = SourceObjectName) and
       (Items[I].Name = RelationName) then
    begin
      Result := Items[I];
      Exit;
    end;
  Result := nil;
end;

function TModelRelationList.FindById(RelationId: Integer): TModelRelation;
var
  I: Integer;
begin
  if RelationId <> 0 then
  for I := 0 to Count - 1 do
    if Items[I].Id = RelationId then
    begin
      Result := Items[I];
      Exit;
    end;
  Result := nil;
end;

function TModelRelationList.GetItems(Index: Integer): TModelRelation;
begin
  Result := TModelRelation(inherited Items[Index])
end;

{
******************************* TModelAttribute ********************************
}

{{
       . AOwner 
,    .   , 
AParent   AOwner    ,    
  .    , 
AParentRelation   .
}
constructor TModelAttribute.Create(AName: String; AOwner, AParent:
  TModelObject);
begin
  inherited Create;
  FName := AName;
  FPersistent := True;
  FVisibleForUser := True;
  FOwner := AOwner;
  FParentObject := AParent;
  FIsInherited := AParent <> AOwner;
  if FIsInherited then
    FParentAttribute := AParent.GetAttribute(Name)
  else
    FFieldName := AName;
end;

{    }

function TModelAttribute.GetFieldName: String;
begin
  if Assigned(ParentAttribute) then
    Result := ParentAttribute.FieldName
  else
    Result := FFieldName
end;

procedure TModelAttribute.SetFieldName(const Value: String);
begin
  if Assigned(ParentAttribute) then
    ParentAttribute.FieldName := Value
  else
    FFieldName := Value
end;

function TModelAttribute.GetSize: Integer;
begin
  if Assigned(ParentAttribute) then
    Result := ParentAttribute.Size
  else
    Result := FSize;
end;

procedure TModelAttribute.SetSize(Value: Integer);
begin
  if Assigned(ParentAttribute) then
    ParentAttribute.Size := Value
  else
    FSize := Value
end;

function TModelAttribute.GetTypeName: String;
begin
  if Assigned(ParentAttribute) then
    Result := ParentAttribute.TypeName
  else
    Result := FTypeName;
end;

procedure TModelAttribute.SetTypeName(const Value: String);
begin
  if Assigned(ParentAttribute) then
    ParentAttribute.TypeName := Value
  else
    FTypeName := Value
end;

function TModelAttribute.GetName: String;
begin
  if Assigned(ParentAttribute) then
    Result := ParentAttribute.Name
  else
    Result := FName;
end;

procedure TModelAttribute.SetName(const Value: String);
begin
  if Name <> Value then
  begin
    if Owner.FindAttribute(Value) <> nil then
      raise Exception.CreateFmt(' %s  ', [Value]);
    if IsInherited then
      raise Exception.Create('   :' + Name)
    else
      FName := Value
  end
end;

function TModelAttribute.GetCalculated: Boolean;
begin
  if Assigned(ParentAttribute) then
    Result := ParentAttribute.Calculated
  else
    Result := FCalculated;
end;

procedure TModelAttribute.SetCalculated(const Value: Boolean);
begin
  if Assigned(ParentAttribute) then
    ParentAttribute.Calculated := Value
  else
    FCalculated := Value
end;

function TModelAttribute.GetRequired: Boolean;
begin
  if Assigned(ParentAttribute) then
    Result := ParentAttribute.Required
  else
    Result := FRequired;
end;

procedure TModelAttribute.SetRequired(const Value: Boolean);
begin
  if Assigned(ParentAttribute) then
    ParentAttribute.Required := Value
  else
    FRequired := Value
end;

function TModelAttribute.GetRequiredMsg: String;
begin
  if Assigned(ParentAttribute) then
    Result := ParentAttribute.RequiredMsg
  else
    Result := FRequiredMsg;
end;

procedure TModelAttribute.SetRequiredMsg(const Value: String);
begin
  if Assigned(ParentAttribute) then
    ParentAttribute.RequiredMsg := Value
  else
    FRequiredMsg := Value
end;

{{
      ,     .
}
procedure TModelAttribute.GetFieldDef(var AFieldType: TFieldType; var
  AFieldSize: Integer);
var
  RealTypeName: String;
  DataType: TModelDataType;
  EnumValue: Integer;
begin
  DataType := Owner.Model.FindDataType(TypeName);
  if Assigned(DataType) then
  begin
    RealTypeName := DataType.BaseType;
    AFieldSize := DataType.Size;
    //       ,   ,
    //   
    if AFieldSize = 0 then AFieldSize := Size;
  end
  else begin
    RealTypeName := TypeName;
    AFieldSize := Size;
  end;
  EnumValue := GetEnumValue(TypeInfo(TFieldType), 'ft' + RealTypeName);
  if EnumValue < 0 then
    raise Exception.CreateFmt('   %s: "%s"', [Name, RealTypeName]);
  AFieldType := TFieldType(EnumValue);
end;

function TModelAttribute.IsBoolean: Boolean;
var
  AFieldType: TFieldType;
  AFieldSize: Integer;
begin
  GetFieldDef(AFieldType, AFieldSize);
  Result := AFieldType = ftBoolean;
end;

function TModelAttribute.IsMemo: Boolean;
var
  AFieldType: TFieldType;
  AFieldSize: Integer;
begin
  GetFieldDef(AFieldType, AFieldSize);
  Result := AFieldType in [ftMemo, ftFmtMemo];
end;

function TModelAttribute.IsBlob: Boolean;
var
  AFieldType: TFieldType;
  AFieldSize: Integer;
begin
  GetFieldDef(AFieldType, AFieldSize);
  Result := AFieldType in [ftBlob, ftGraphic, ftBytes, ftVarBytes, ftTypedBinary,
    ftOraBlob, ftOraClob, ftParadoxOle, ftDBaseOle, ftArray, ftDataSet];
end;

function TModelAttribute.GetDescription: String;
begin
  if Assigned(ParentAttribute) then
    Result := ParentAttribute.Description
  else
    Result := FDescription;
end;

procedure TModelAttribute.SetDescription(const Value: String);
begin
  if Assigned(ParentAttribute) then
    ParentAttribute.Description := Value
  else
    FDescription := Value
end;

function TModelAttribute.GetPersistent: Boolean;
begin
  if Assigned(ParentAttribute) then
    Result := ParentAttribute.Persistent
  else
    Result := FPersistent;
end;

procedure TModelAttribute.SetPersistent(const Value: Boolean);
begin
  if Assigned(ParentAttribute) then
    ParentAttribute.Persistent := Value
  else
    FPersistent := Value
end;

function TModelAttribute.GetVisibleForUser: Boolean;
begin
  if Assigned(ParentAttribute) then
    Result := ParentAttribute.VisibleForUser
  else
    Result := FVisibleForUser;
end;

procedure TModelAttribute.SetVisibleForUser(const Value: Boolean);
begin
  if Assigned(ParentAttribute) then
    ParentAttribute.VisibleForUser := Value
  else
    FVisibleForUser := Value
end;

{
***************************** TModelAttributeList ******************************
}
constructor TModelAttributeList.Create(AOwner: TModelObject);
begin
  inherited Create;
  FOwner := AOwner;
end;

function TModelAttributeList.AddAttribute(AName, AType: String; ASize: Integer;
  AParent: TModelObject): TModelAttribute;
begin
  if FindAttribute(AName) <> nil then
    raise Exception.CreateFmt(' "%s"  ', [AName]);

  Result := TModelAttribute.Create(AName, Owner, AParent);
  inherited Add(Result);
  with Result do begin
    FTypeName := AType;
    FSize := ASize;
  end
end;

procedure TModelAttributeList.DeleteAttribute(Attribute: TModelAttribute);
begin
  Remove(Attribute);
end;

{{
    .
    ,   nil.
}
function TModelAttributeList.FindAttribute(AName: String): TModelAttribute;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    if Items[I].Name = AName then
    begin
      Result := Items[I];
      Exit;
    end;
  Result := nil;
end;

function TModelAttributeList.GetItems(Index: Integer): TModelAttribute;
begin
  Result := TModelAttribute(inherited Items[Index])
end;

{
********************************* TDomainModel *********************************
}
{<<Category>>: No category}
constructor TDomainModel.Create;
begin
  inherited Create;
  FNameSpaces := TStringList.Create;
  FObjects := TModelObjectList.Create(Self);
  FRelations := TModelRelationList.Create(Self);
  FDataTypes := TObjectList.Create;
end;

destructor TDomainModel.Destroy;
begin
  FreeAndNil(FDataTypes);
  FObjects.Free;
  FRelations.Free;
  FNameSpaces.Free;
  inherited Destroy;
end;

{***************************************************************************** }
{                                                                    }
{ **************************************************************************** }

function TDomainModel.GetDataTypeCount: Integer;
begin
  Result := FDataTypes.Count
end;

function TDomainModel.GetDataTypes(Index: Integer): TModelDataType;
begin
  Result := TModelDataType(FDataTypes[Index])
end;

function TDomainModel.FindDataType(ADataTypeName: String): TModelDataType;
var
  I: Integer;
begin
  for I := 0 to DataTypeCount - 1 do
    if DataTypes[I].Name = ADataTypeName then
    begin
      Result := DataTypes[I];
      Exit;
    end;
  Result := nil;
end;

function TDomainModel.AddDataType(ADataTypeName: String; ABaseType: String =
  ''; ASize: Integer = 0): TModelDataType;
begin
  Result := TModelDataType.Create(ADataTypeName, ABaseType, ASize);
  try
    FDataTypes.Add(Result);
  except
    Result.Free; raise
  end;
end;

procedure TDomainModel.DeleteDataType(AModelDataType: TModelDataType);
begin
  FDataTypes.Remove(AModelDataType);
end;

procedure TDomainModel.DeleteDataTypes;
begin
  FDataTypes.Clear;
end;

{***************************************************************************** }
{                                                                              }
{ **************************************************************************** }

function TDomainModel.AddObject(ObjectName: String; NameSpace: String = '';
  ObjectId: Integer = 0): TModelObject;
begin
  if NameSpace <> '' then ObjectName := NameSpace + NameSpaceSeparator + ObjectName;

  if FindObject(ObjectName) <> nil then
    raise Exception.CreateFmt(' %s  ', [ObjectName]);

  Result := FObjects.Add;
  ParseFullObjectName(ObjectName, ObjectName, NameSpace);
  Result.FNameSpace := NameSpace;
  Result.Name := ObjectName; //     FullName
  if AutoGenerateNames then
    Result.FTableName := GenerateUniqueTableName(SClassTablePrefix)
  else
    Result.FTableName := ObjectName;

  if ObjectId = 0
    then Result.FId := GetNewObjectId
    else begin Result.FId := ObjectId; SetNewObjectId(ObjectId); end
end;

procedure TDomainModel.Clear;
begin
  FNewObjectId := 0;
  FNewRelationId := 0;
  FObjects.Clear;
  FRelations.Clear;
  FDataTypes.Clear;
  NameSpaces.Clear;
end;

procedure TDomainModel.DeleteObject(AObject: TModelObject);
var
  I: Integer;
  S: TStrings;
begin
  S := TStringList.Create;
  try
    //     AObject
    AObject.GetReferencingObjects(S, False);
    for I := 0 to S.Count - 1 do
      if S[I] <> ''
        then TModelObject(S.Objects[I]).DeleteRelation(S[I])
        else TModelObject(S.Objects[I]).DeleteGeneralization(AObject.FullName);

    //     AObject (    )
    AObject.GetReferencedObjects(S, False);
    for I := 0 to S.Count - 1 do
      if S[I] <> ''
        then AObject.DeleteRelation(S[I])
        else AObject.DeleteGeneralization(TModelObject(S.Objects[I]).FullName);

    //   
    FObjects.Remove(AObject);
  finally
    S.Free
  end;
end;

procedure TDomainModel.DeleteObjects(NameSpace: String);
var
  I: Integer;
begin
  for I := ObjectCount - 1 downto 0 do
    if Objects[I].NameSpace = NameSpace
      then DeleteObject(Objects[I]);
end;

function TDomainModel.FindObject(AnObjectName: String): TModelObject;
begin
  Result := FObjects.FindByName(AnObjectName);
end;

{{
       ,
    .
}
function TDomainModel.GetObject(ObjectName: String; NameSpace: String = ''):
  TModelObject;
begin
  if NameSpace <> '' then ObjectName := NameSpace + NameSpaceSeparator + ObjectName;

  Result := FObjects.FindByName(ObjectName);
  if Result = nil then
    raise EModelObjectNotFound.CreateFmt('  "%s"  ', [ObjectName]);
end;

{{
       ,
    .
}
function TDomainModel.GetObject(AnObjectId: Integer): TModelObject;
begin
  Result := FObjects.FindById(AnObjectId);
  if Result = nil then
    raise EModelObjectNotFound.CreateFmt('    "%d"  ', [AnObjectId]);
end;

function TDomainModel.FindObject(AnObjectId: Integer): TModelObject;
begin
  Result := FObjects.FindById(AnObjectId);
end;

{{
    AnAttributeName   
  ,       .
}
function TDomainModel.GetObjectAttribute(AnObjectName, AnAttributeName:
  String): TModelAttribute;
var
  O: TModelObject;
begin
  O := GetObject(AnObjectName);
  Result := O.GetAttribute(AnAttributeName);
end;

{{
       
,     .
}
function TDomainModel.GetRelation(RelationId: Integer): TModelRelation;
begin
  Result := FRelations.FindById(RelationId);
  if Result = nil then
    raise EModelRelationNotFound.CreateFmt(
      '   "%d"    ', [RelationId]);
end;

function TDomainModel.FindRelation(RelationId: Integer): TModelRelation;
begin
  Result := FRelations.FindById(RelationId);
end;

{{
    ARelationName   
     :
-    AnObjectName  
-       ARelationName
}
function TDomainModel.GetObjectRelation(AnObjectName, ARelationName: String):
  TModelRelation;
var
  O: TModelObject;
begin
  O := GetObject(AnObjectName);
  Result := O.GetRelation(ARelationName);
end;

function TDomainModel.GetObjectCount: Integer;
begin
  Result := FObjects.Count
end;

function TDomainModel.GetObjects(Index: Integer): TModelObject;
begin
  Result := TModelObject(FObjects[Index])
end;

function TDomainModel.GetRelationCount: Integer;
begin
  Result := FRelations.Count
end;

function TDomainModel.GetRelations(Index: Integer): TModelRelation;
begin
  Result := TModelRelation(FRelations[Index])
end;

{{
    ASourceObjectName  
ATargetObjectName ( ).
}
procedure TDomainModel.RenameNameSpace(OldName, NewName: String);
var
  I: Integer;
begin
  if OldName <> NewName then begin
    if NameSpaces.IndexOf(NewName) >= 0 then
      raise Exception.CreateFmt('    "%s"  ', [NewName]);

    I := NameSpaces.IndexOf(OldName);
    if I < 0 then
      raise Exception.CreateFmt('    "%s"  ', [OldName]);

    NameSpaces[I] := NewName;

    for I := 0 to ObjectCount - 1 do
      if Objects[I].NameSpace = OldName then
        Objects[I].NameSpace := NewName;
  end
end;

{
*****************************   *******************************
}
function TDomainModel.GetNewRelationId: Integer;
begin
  Inc(FNewRelationId);
  Result := FNewRelationId;
end;

function TDomainModel.GetNewObjectId: Integer;
begin
  Inc(FNewObjectId);
  Result := FNewObjectId;
end;

procedure TDomainModel.SetNewRelationId(Value: Integer);
begin
  if Value > FNewRelationId then FNewRelationId := Value;
end;

procedure TDomainModel.SetNewObjectId(Value: Integer);
begin
  if Value > FNewObjectId then FNewObjectId := Value;
end;

{***************************************************************************** }
{                                                     }
{ **************************************************************************** }

{{
          
    -  .   RenameExist
 True,       ,   
     ,     ,
..   .
}
procedure TDomainModel.GenerateDBNames(RenameExist: Boolean);
var
  I, J: Integer;
begin
  for I := 0 to ObjectCount - 1 do
  with Objects[I] do
  begin
    GenerateObjectTableName(Objects[I], RenameExist, I);

    for J := 0 to AttributeCount - 1 do
      GenerateAttributeFieldName(Objects[I], Attributes[J], RenameExist, J);

    for J := 0 to RelationCount - 1 do
      GenerateRelationTableOrFieldName(Objects[I], Relations[J], RenameExist);

    for J := 0 to AncestorCount - 1 do
      GenerateRelationTableOrFieldName(Objects[I], AncestorRelations[J], RenameExist);
  end
end;

{{
        .  
 OnGenerateObjectTableName,       ,
     .  RenameExist  
    ,    , ..  .
}
procedure TDomainModel.GenerateObjectTableName(AModelObject: TModelObject;
  RenameExist: Boolean; ObjectIndex: Integer);
var
  TableName: String;
begin
  if RenameExist or (AModelObject.TableName = '') then
  begin
    if Assigned(FOnGenerateObjectTableName) then
    begin
      FOnGenerateObjectTableName(AModelObject, AModelObject.Name, TableName);
      AModelObject.TableName := TableName;
    end
    else
      if RenameExist then
        {         , 
                 }
        AModelObject.TableName := SClassTablePrefix + IntToStr(ObjectIndex+1)
      else
        {         , 
              ,     
            }
        AModelObject.TableName := GenerateUniqueTableName(SClassTablePrefix)
  end;
end;

procedure TDomainModel.GenerateAttributeFieldName(AModelObject: TModelObject;
  AModelAttribute: TModelAttribute; RenameExist: Boolean; AttributeIndex:
  Integer);
var
  FieldName: String;
begin
  if RenameExist or (AModelAttribute.FieldName = '') then
  begin
    if Assigned(FOnGenerateAttributeFieldName) then
    begin
      FOnGenerateAttributeFieldName(AModelAttribute, AModelAttribute.Name, FieldName);
      AModelAttribute.FieldName := FieldName;
    end
    else
      if RenameExist then
        {         , 
                 }
        AModelAttribute.FieldName := SAttributeFieldPrefix + IntToStr(AttributeIndex+1)
      else
        {         , 
              ,     
            }
        AModelAttribute.FieldName := GenerateUniqueFieldName(AModelObject, SAttributeFieldPrefix)
  end;
end;

procedure TDomainModel.GenerateRelationTableOrFieldName(AModelObject:
  TModelObject; AModelRelation: TModelRelation; RenameExist: Boolean);
var
  NeedFieldName: Boolean;
  NeedTableName: Boolean;
  TableName, DefaultTableName: String;
  FieldName, DefaultFieldName: String;
begin
  NeedFieldName := False;
  NeedTableName := False;
  if AModelRelation.IsReference or AModelRelation.IsAggregation or AModelRelation.IsGeneralization then
    NeedFieldName := True;
  if Not AModelRelation.IsComposition then
    NeedTableName := True;

  if RenameExist
     or NeedFieldName and (AModelRelation.FieldName = '')
     or NeedTableName and (AModelRelation.TableName = '') then
  begin

    if Assigned(FOnGenerateRelationTableOrFieldName) then
    begin
      DefaultTableName := '';
      DefaultFieldName := '';

      if AModelRelation.IsReference then
        DefaultFieldName := AModelRelation.Name else
      if AModelRelation.IsAggregation then
        DefaultFieldName := AModelRelation.SourceObject.Name else
      if AModelRelation.IsAssociation then
        DefaultTableName := AModelRelation.TargetObject.Name else
      if AModelRelation.IsGeneralization then
        DefaultFieldName := AModelRelation.TargetObject.Name + IntToStr(AModelRelation.TargetObject.Id);

      FOnGenerateRelationTableOrFieldName(AModelRelation,
        DefaultTableName, DefaultFieldName, TableName, FieldName);
      AModelRelation.TableName := TableName;
      AModelRelation.FieldName := FieldName;
    end
    else
    begin
      if AModelRelation.IsReference then
        AModelRelation.FieldName := GenerateUniqueFieldName(AModelObject, SReferenceFieldPrefix) else
      if AModelRelation.IsAggregation then
        AModelRelation.FieldName := GenerateUniqueFieldName(AModelObject, SReferenceFieldPrefix) else
      if AModelRelation.IsAssociation then
        AModelRelation.TableName := GenerateUniqueTableName(SRelationTablePrefix) else
      if AModelRelation.IsGeneralization then
        AModelRelation.FieldName := GenerateUniqueFieldName(AModelObject, SParentFieldPrefix)
    end;
  end;
end;

{{
   .
}
function TDomainModel.GenerateUniqueTableName(APrefix: String): String;
var
  IsUnique: Boolean;
  I, C: Integer;
begin
  C := 0;
  repeat
    Inc(C);
    Result := APrefix + IntToStr(C);
    IsUnique := True;
    for I := 0 to ObjectCount - 1 do
      if UpperCase(Objects[I].TableName) = UpperCase(Result) then
      begin
        IsUnique := False;
        Break;
      end;
  until IsUnique;
end;

{{
       .
}
function TDomainModel.GenerateUniqueFieldName(AObject: TModelObject; APrefix:
  String): string;
var
  IsUnique: Boolean;
  I, C: Integer;
begin
  C := 0;
  repeat
    Inc(C);
    Result := APrefix + IntToStr(C);
    IsUnique := True;
    for I := 0 to AObject.AttributeCount - 1 do
      if UpperCase(AObject.Attributes[I].FieldName) = UpperCase(Result) then
      begin
        IsUnique := False;
        Break;
      end;
    for I := 0 to AObject.RelationCount - 1 do
      if UpperCase(AObject.Relations[I].FieldName) = UpperCase(Result) then
      begin
        IsUnique := False;
        Break;
      end;
  until IsUnique;
end;

end.

