unit ovModelEvents;

interface

uses SysUtils, Classes, ovTypes, ovComponents, ovModel, ovIObjects;

type
  TovModelEvent = class;
  TovModelEventCollection = class;

  {{
      : , , 
   ,   .

          
  :
  -         
     
  -        
       ,    
      ,   .
  }
  TovModelEvents = class(TovModelClient)
  private
    FActive: Boolean;
    FEvents: TovModelEventCollection;
    FActiveEvents: TStringList;
    procedure SetActive(Value: Boolean);
    procedure SetEvents(const Value: TovModelEventCollection);
    function FindAttributeEvent(const AModelAttribute: TModelAttribute;
      ActiveEventList: TStringList; var Event: TovModelEvent): Boolean;
    function FindObjectEvent(const AModelObject: TModelObject; ActiveEventList:
      TStringList; var Event: TovModelEvent): Boolean;
    function GetAttributeKey(const AModelAttribute: TModelAttribute): String;
    function GetObjectKey(const AModelObject: TModelObject): String;
    procedure RefreshActiveEventList;
    function GetEventKey(const Event: TovModelEvent; var Key: String): Boolean;
  protected
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure HandleDeleteObjectEvent(AModelObject: TModelObject; AObjectId: TOID);
    procedure HandleValidateObjectEvent(AModelObject: TModelObject; AObjectId:
      TOID);
    procedure HandleCalculateAttributeEvent(AModelObject: TModelObject; AObjectId:
      TOID; AModelAttribute: TModelAttribute; var Value: Variant);
    procedure HandleCreateObjectEvent(AModelObject: TModelObject; AObjectId: TOID);
    function GetObject(AModelObject: TModelObject; ObjectId: TOID): IovObject;
      overload;
  published
    property Active: Boolean read FActive write SetActive default True;
    property Events: TovModelEventCollection read FEvents write SetEvents;
  end;

  TCreateObjectEvent = procedure (Sender: TovModelEvents;
    AModelObject: TModelObject; AObjectId: TOID) of object;

  TDeleteObjectEvent = procedure (Sender: TovModelEvents;
    AModelObject: TModelObject; AObjectId: TOID) of object;

  TValidateObjectEvent = procedure (Sender: TovModelEvents;
    AModelObject: TModelObject; AObjectId: TOID) of object;

  TCalcAttributeEvent = procedure (Sender: TovModelEvents;
    AModelObject: TModelObject; AObjectId: TOID;
    AModelAttribute: TModelAttribute; var Value: Variant) of object;

  TovModelEvent = class(TCollectionItem)
  private
    FOwner: TovModelEvents;
    FOnCalcAttribute: TCalcAttributeEvent;
    FOnCreateObject: TCreateObjectEvent;
    FOnDeleteObject: TDeleteObjectEvent;
    FOnValidateObject: TValidateObjectEvent;
    FEnabled: Boolean;
    FObjectClassName: String;
    FObjectAttribute: String;
    procedure SetEnabled(Value: Boolean);
    procedure SetObjectClassName(const Value: String);
    procedure SetObjectAttribute(const Value: String);
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    property Owner: TovModelEvents read FOwner;
  published
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property ObjectClassName: String read FObjectClassName write SetObjectClassName;
    property ObjectAttribute: String read FObjectAttribute write SetObjectAttribute;
    property OnCalcAttribute: TCalcAttributeEvent read FOnCalcAttribute write
      FOnCalcAttribute;
    property OnCreateObject: TCreateObjectEvent read FOnCreateObject write
      FOnCreateObject;
    property OnDeleteObject: TDeleteObjectEvent read FOnDeleteObject write
      FOnDeleteObject;
    property OnValidateObject: TValidateObjectEvent read FOnValidateObject write
      FOnValidateObject;
  end;

  TovModelEventCollection = class(TCollection)
  private
    FOwner: TovModelEvents;
    function GetItems(Index: Integer): TovModelEvent;
  protected
    procedure Notify(Item: TCollectionItem; Action: TCollectionNotification);
      override;
  public
    property Items[Index: Integer]: TovModelEvent read GetItems; default;
  end;

implementation

{ TovModelEvents }

constructor TovModelEvents.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEvents := TovModelEventCollection.Create(TovModelEvent);
  FEvents.FOwner := Self;
  FActive := True;

  FActiveEvents := TStringList.Create;
  FActiveEvents.Sorted := True;
  FActiveEvents.Duplicates := dupError;
end;

destructor TovModelEvents.Destroy;
begin
  FreeAndNil(FActiveEvents);

  FreeAndNil(FEvents);
  inherited Destroy;
end;

procedure TovModelEvents.SetEvents(const Value: TovModelEventCollection);
begin
  FEvents.Assign(Value);
end;

procedure TovModelEvents.Loaded;
begin
  inherited;
  RefreshActiveEventList;
end;

procedure TovModelEvents.SetActive(Value: Boolean);
begin
  if FActive <> Value then
  begin
    FActive := Value;
    RefreshActiveEventList;
  end;
end;

{{
        
TovModelEvent.       ,
    .
}
function TovModelEvents.GetObject(AModelObject: TModelObject; ObjectId: TOID):
  IovObject;
begin
  Result := Model.ObjectSpace.GetObject(AModelObject, ObjectId)
end;

{***************************************************************************** }
{                                                        }
{ **************************************************************************** }

procedure TovModelEvents.HandleCreateObjectEvent(AModelObject: TModelObject;
  AObjectId: TOID);
var
  Event: TovModelEvent;
begin
  if FindObjectEvent(AModelObject, FActiveEvents, Event) and
     Assigned(Event.FOnCreateObject)
  then
    Event.FOnCreateObject(Self, AModelObject, AObjectId);
end;

procedure TovModelEvents.HandleDeleteObjectEvent(AModelObject: TModelObject;
  AObjectId: TOID);
var
  Event: TovModelEvent;
begin
  if FindObjectEvent(AModelObject, FActiveEvents, Event) and
     Assigned(Event.FOnDeleteObject)
  then
    Event.FOnDeleteObject(Self, AModelObject, AObjectId);
end;

procedure TovModelEvents.HandleValidateObjectEvent(AModelObject: TModelObject;
  AObjectId: TOID);
var
  Event: TovModelEvent;
begin
  if FindObjectEvent(AModelObject, FActiveEvents, Event) and
     Assigned(Event.FOnValidateObject)
  then
    Event.FOnValidateObject(Self, AModelObject, AObjectId);
end;

procedure TovModelEvents.HandleCalculateAttributeEvent(AModelObject:
  TModelObject; AObjectId: TOID; AModelAttribute: TModelAttribute; var Value:
  Variant);
var
  Event: TovModelEvent;
begin
  if FindAttributeEvent(AModelAttribute, FActiveEvents, Event) and
     Assigned(Event.FOnCalcAttribute)
  then
    Event.FOnCalcAttribute(Self, AModelObject, AObjectId, AModelAttribute, Value);
end;

{***************************************************************************** }
{                                           }
{ **************************************************************************** }

function TovModelEvents.FindAttributeEvent(const AModelAttribute:
  TModelAttribute; ActiveEventList: TStringList; var Event: TovModelEvent):
  Boolean;
var
  I: Integer;
begin
  Result := ActiveEventList.Find(GetAttributeKey(AModelAttribute), I);
  if Result then Event := TovModelEvent(ActiveEventList.Objects[I])
end;

function TovModelEvents.FindObjectEvent(const AModelObject: TModelObject;
  ActiveEventList: TStringList; var Event: TovModelEvent): Boolean;
var
  I: Integer;
begin
  Result := ActiveEventList.Find(GetObjectKey(AModelObject), I);
  if Result then Event := TovModelEvent(ActiveEventList.Objects[I])
end;

function TovModelEvents.GetObjectKey(const AModelObject: TModelObject): String;
begin
  Result := AModelObject.FullName
end;

function TovModelEvents.GetAttributeKey(const AModelAttribute:
  TModelAttribute): String;
begin
  Result := AModelAttribute.Owner.FullName + '@' + AModelAttribute.Name
end;

function TovModelEvents.GetEventKey(const Event: TovModelEvent; var Key:
  String): Boolean;
begin
  Result := Event.Enabled;
  if Result then
  with Event do
  begin
    if (ObjectClassName <> '') and (ObjectAttribute = '') then
      Key := Event.ObjectClassName
    else
    if (ObjectClassName <> '') and (ObjectAttribute <> '') then
      Key := ObjectClassName + '@' + ObjectAttribute
  end
end;

procedure TovModelEvents.RefreshActiveEventList;
var
  I: Integer;
  Key: String;
begin
  if Not (csLoading in ComponentState) and Not (csDestroying in ComponentState) and
     Not (csDesigning in ComponentState)
  then
  begin
    FActiveEvents.Clear;
    if Active then
    begin
      for I := 0 to Events.Count - 1 do
        if GetEventKey(Events[I], Key) then
          FActiveEvents.AddObject(Key, Events[I])
    end;
  end;
end;

{ TovModelEventCollection }

procedure TovModelEventCollection.Notify(Item: TCollectionItem; Action:
  TCollectionNotification);
begin
  if Action = cnAdded then
    (Item as TovModelEvent).FOwner := FOwner
  else
    (Item as TovModelEvent).FOwner := nil;
  FOwner.RefreshActiveEventList;
end;

function TovModelEventCollection.GetItems(Index: Integer): TovModelEvent;
begin
  Result := TovModelEvent(inherited Items[Index]);
end;

{ TovModelEvent }

constructor TovModelEvent.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FEnabled := True;
end;

procedure TovModelEvent.SetEnabled(Value: Boolean);
begin
  if FEnabled <> Value then
  begin
    FEnabled := Value;
    FOwner.RefreshActiveEventList;
  end;
end;

procedure TovModelEvent.SetObjectClassName(const Value: String);
begin
  if FObjectClassName <> Value then
  begin
    FObjectClassName := Value;
    FOwner.RefreshActiveEventList;
  end;
end;

procedure TovModelEvent.SetObjectAttribute(const Value: String);
begin
  if FObjectAttribute <> Value then
  begin
    FObjectAttribute := Value;
    FOwner.RefreshActiveEventList;
  end;
end;

function TovModelEvent.GetDisplayName: string;
begin
  if ObjectAttribute <> '' then
    Result := ObjectClassName + '.' + ObjectAttribute
  else
  if ObjectClassName <> '' then
    Result := ObjectClassName
  else
    Result := inherited GetDisplayName
end;

end.
