unit ovModelTreeViewBuilder;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ComCtrls, ovModel, ovComponents;

type
  TRootObjectType = (rtDefault, rtNamespace, rtObject, rtRelation);
  TRootObject = record
    RootObjectType: TRootObjectType;
    RootObject: TObject;
    Namespace: String;   // ,  RootObjectType = rtNamespace
  end;

  //  ,   NodeTypes ( ,   
  //     )
  TNodeType = (ntNone, ntNamespace, ntDataType, ntObject, ntGeneralization, ntAttribute,
    ntRelation,
    ntRelationNodes, // ,       
                     //   ,      +
    ntIndex
    );

  TModelTreeViewBuilderOption =
  ( tvbShowDataTypes,             //     
    tvbShowAttributes,            //    
    tvbShowRelations,             //    
    tvbShowGeneralizations,       //     
    tvbShowIndexes,               //    
    tvbShowAttributeTypes,        //     
    tvbShowRelationTypes,         //     
    tvbShowSingleRelationsOnly,   //       .   1
    tvbShowRoot,                  //   
    tvbExpandRelations,           //        -
    tvbShowVisibleForUserObjects  //   ,      
                                  //   VisibleForUser = True,    -  
  );
  TModelTreeViewBuilderOptions = set of TModelTreeViewBuilderOption;

  TNodeImageIndexes = class(TPersistent)
  private
    FAttributeNodeImageIndex: Integer;
    FDataTypeNodeImageIndex: Integer;
    FGeneralizationNodeImageIndex: Integer;
    FNamespaceNodeImageIndex: Integer;
    FClassNodeImageIndex: Integer;
    FRelationNodeImageIndex: Integer;
    FClassIndexNodeImageIndex: Integer;
  published
    property NamespaceNodeImageIndex: Integer read FNamespaceNodeImageIndex write
      FNamespaceNodeImageIndex default 0;
    property DataTypeNodeImageIndex: Integer read FDataTypeNodeImageIndex write
      FDataTypeNodeImageIndex default 0;
    property ClassNodeImageIndex: Integer read FClassNodeImageIndex write
      FClassNodeImageIndex default 0;
    property AttributeNodeImageIndex: Integer read FAttributeNodeImageIndex write
      FAttributeNodeImageIndex default 0;
    property RelationNodeImageIndex: Integer read FRelationNodeImageIndex write
      FRelationNodeImageIndex default 0;
    property GeneralizationNodeImageIndex: Integer read
      FGeneralizationNodeImageIndex write FGeneralizationNodeImageIndex default 0;
    property ClassIndexNodeImageIndex: Integer read FClassIndexNodeImageIndex write
      FClassIndexNodeImageIndex default 0;
  end;

  TVisitNamespaceEvent = procedure(ANamespace: String; var Skip: Boolean) of
    object;
  TVisitObjectEvent = procedure(AModelObject: TModelObject; var Skip: Boolean) of
    object;

  {{
  ,      .

    :
   -    tvbExpandRelations,  
      OnExpanding  TreeView (   )
   -  TTreeNode.Data      
       (,   ).     ntNamespace
      ntRelationNodes  TTreeNode.Data   
      TNodeType.
  }
  TovModelTreeViewBuilder = class(TovModelClient)
  private
    FTreeView: TCustomTreeView;
    FRootObject: TRootObject;
    FDefaultNameSpaceName: String;
    FOptions: TModelTreeViewBuilderOptions;
    FTreeNodeImageIndexes: TNodeImageIndexes;
    FOnVisitNamespace: TVisitNamespaceEvent;
    FOnVisitObject: TVisitObjectEvent;
    procedure SetTreeView(Value: TCustomTreeView);
    function GetTreeViewItems: TTreeNodes;
    procedure BuildTreeViewProc(ParentNode: Pointer); overload;
    procedure VisitNamespaces(ParentNode: TTreeNode; ShowRoot: Boolean);
    procedure VisitObjects(ANamespace: String; AParentNode: TTreeNode);
    procedure VisitAttributes(AModelObject: TModelObject; AParentNode: TTreeNode);
    procedure TreeViewCompare(Sender: TObject; Node1, Node2: TTreeNode; Data:
      Integer; var Compare: Integer);
    function GetNodeTypes(Node: TTreeNode): TNodeType;
    procedure VisitClass(AModelObject: TModelObject; AParentNode: TTreeNode;
      ShowRoot: Boolean);
    procedure VisitRelations(AModelObject: TModelObject; AParentNode: TTreeNode);
    procedure VisitGeneralizations(AModelObject: TModelObject; AParentNode:
      TTreeNode);
    procedure TreeViewExpanding(Sender: TObject; Node: TTreeNode; var
      AllowExpansion: Boolean);
    function GetNodeObjects(Node: TTreeNode): TObject;
    procedure SetNodeObjects(Node: TTreeNode; Value: TObject);
    function GetSelectedClassName: String;
    procedure VisitDataType(AModelDataType: TModelDataType; AParentNode: TTreeNode);
    procedure SetTreeNodeImageIndexes(const Value: TNodeImageIndexes);
    procedure VisitIndexes(AModelObject: TModelObject; AParentNode: TTreeNode);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    property TreeViewItems: TTreeNodes read GetTreeViewItems;
  public
    procedure BuildTreeView; overload;
    procedure BuildTreeView(AParentNode: TTreeNode); overload;
    constructor Create(AOwner: TComponent); override;
    function GetAttributeText(Attribute: TModelAttribute): String; virtual;
    function GetRelationText(Relation: TModelRelation): String; virtual;
    function GetNamespaceText(Namespace: String): String; virtual;
    function GetNamespaceFromNodeText(Text: String): String;
    function FindObjectNode(AObject: TObject): TTreeNode;
    destructor Destroy; override;
    property NodeTypes[Node: TTreeNode]: TNodeType read GetNodeTypes;
    property RootObject: TRootObject read FRootObject write FRootObject;
    property NodeObjects[Node: TTreeNode]: TObject read GetNodeObjects write
      SetNodeObjects;
    property SelectedClassName: String read GetSelectedClassName;
  published
    property TreeView: TCustomTreeView read FTreeView write SetTreeView;
    property DefaultNameSpaceName: String read FDefaultNameSpaceName write
      FDefaultNameSpaceName;
    property Options: TModelTreeViewBuilderOptions read FOptions write FOptions
      default [tvbShowDataTypes, tvbShowAttributes, tvbShowRelations, tvbShowGeneralizations,
               tvbShowAttributeTypes, tvbShowRelationTypes, tvbShowRoot, tvbShowVisibleForUserObjects];
    property TreeNodeImageIndexes: TNodeImageIndexes read FTreeNodeImageIndexes
      write SetTreeNodeImageIndexes;
    property OnVisitNamespace: TVisitNamespaceEvent read FOnVisitNamespace write
      FOnVisitNamespace;
    property OnVisitObject: TVisitObjectEvent read FOnVisitObject write
      FOnVisitObject;
  end;

  TBuildTreeViewProc = procedure (AData: Pointer) of object;

function MakeNodeText(Name, TypeName: String): String;
function GetNameFromNodeText(Text: String): String;
function GetTypeNameFromNodeText(Text: String): String;

procedure BuildTreeView(TreeView: TCustomTreeView; BuildTreeViewProc:
  TBuildTreeViewProc; Data: Pointer; SortItems: Boolean = True; ExpandTreeView:
  Boolean = True; SelectFirstItem: Boolean = True);

implementation

type
  TTreeViewFriend = class (TCustomTreeView)
  end;

function MakeNodeText(Name, TypeName: String): String;
begin
  Result := Name + ':' + TypeName
end;

function GetNameFromNodeText(Text: String): String;
var
  P: Integer;
begin
  P := Pos(':', Text);
  if P > 0
    then Result := Copy(Text, 1, P-1)
    else Result := Text
end;

function GetTypeNameFromNodeText(Text: String): String;
var
  P: Integer;
begin
  P := Pos(':', Text);
  if P > 0
    then Result := Copy(Text, P+1, Length(Text))
    else Result := ''
end;

procedure BuildTreeView(TreeView: TCustomTreeView; BuildTreeViewProc:
  TBuildTreeViewProc; Data: Pointer; SortItems: Boolean = True; ExpandTreeView:
  Boolean = True; SelectFirstItem: Boolean = True);
var
  Items: TTreeNodes;
begin
  with TreeView do
  begin
    Items := TTreeViewFriend(TreeView).Items;
    Items.BeginUpdate;
    try
      Items.Clear;
      BuildTreeViewProc(Data);

      if SortItems then AlphaSort;

      if ExpandTreeView then FullExpand;

      if SelectFirstItem and (Items.Count > 0) then
      begin
        TreeView.Selected := Items[0];
        TreeView.TopItem := Items[0];
      end;

    finally
      Items.EndUpdate
    end;
  end;
end;
{
*************************** TovModelTreeViewBuilder ****************************
}

constructor TovModelTreeViewBuilder.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  DefaultNameSpaceName := '< >';
  Options := [tvbShowDataTypes, tvbShowAttributes, tvbShowRelations, tvbShowGeneralizations,
    tvbShowAttributeTypes, tvbShowRelationTypes, tvbShowRoot, tvbShowVisibleForUserObjects];
  FTreeNodeImageIndexes := TNodeImageIndexes.Create;
end;

destructor TovModelTreeViewBuilder.Destroy;
begin
  FreeAndNil(FTreeNodeImageIndexes);
  inherited Destroy;
end;

procedure TovModelTreeViewBuilder.SetTreeView(Value: TCustomTreeView);
begin
  if FTreeView <> Value then
  begin
    FTreeView := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovModelTreeViewBuilder.SetTreeNodeImageIndexes(const Value:
  TNodeImageIndexes);
begin
end;

function TovModelTreeViewBuilder.GetSelectedClassName: String;
begin
  if Assigned(TreeView) and Assigned(TreeView.Selected) and
     (NodeTypes[TreeView.Selected] = ntObject)
  then
    Result := (NodeObjects[TreeView.Selected] as TModelObject).FullName
  else
    Result := ''
end;

function TovModelTreeViewBuilder.GetNamespaceText(Namespace: String): String;
begin
  if NameSpace = '' then
    Result := DefaultNameSpaceName
  else
    Result := NameSpace;
end;

function TovModelTreeViewBuilder.GetNamespaceFromNodeText(Text: String): String;
begin
  if Text <> DefaultNameSpaceName
    then Result := Text
    else Result := ''
end;

function TovModelTreeViewBuilder.GetAttributeText(Attribute: TModelAttribute):
  String;
begin
  if tvbShowAttributeTypes in Options
    then Result := MakeNodeText(Attribute.Name, Attribute.TypeName)
    else Result := Attribute.Name;
end;

function TovModelTreeViewBuilder.GetRelationText(Relation: TModelRelation):
  String;
begin
  if Relation.IsGeneralization then
    Result := Relation.TargetObjectName
  else
    if tvbShowRelationTypes in Options
      then Result := MakeNodeText(Relation.Name, Relation.TargetObjectName)
      else Result := Relation.Name;
end;

procedure TovModelTreeViewBuilder.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = TreeView)then TreeView := nil
end;

function TovModelTreeViewBuilder.GetTreeViewItems: TTreeNodes;
begin
  Result := TTreeViewFriend(TreeView).Items;
end;

procedure TovModelTreeViewBuilder.BuildTreeView;
begin
  BuildTreeView(nil)
end;

procedure TovModelTreeViewBuilder.BuildTreeView(AParentNode: TTreeNode);
begin
  if Not Assigned(TreeView) then raise
    Exception.Create('    TreeView');

  ModelNeeded;

  //       
  TTreeViewFriend(TreeView).OnCompare := TreeViewCompare;

  ovModelTreeViewBuilder.BuildTreeView(TreeView, BuildTreeViewProc, AParentNode,
    True, Not (tvbExpandRelations in Options), True);

  //       
  if tvbExpandRelations in Options then
    TTreeViewFriend(TreeView).OnExpanding := TreeViewExpanding
end;

procedure TovModelTreeViewBuilder.BuildTreeViewProc(ParentNode: Pointer);
begin
  case RootObject.RootObjectType of
    rtDefault: VisitNamespaces(ParentNode, tvbShowRoot in Options);
    rtNamespace: VisitObjects(RootObject.Namespace, ParentNode{, tvbShowRoot in Options});
    rtObject: VisitClass(RootObject.RootObject as TModelObject, ParentNode, tvbShowRoot in Options);
  end;
end;

procedure TovModelTreeViewBuilder.TreeViewCompare(Sender: TObject; Node1,
  Node2: TTreeNode; Data: Integer; var Compare: Integer);
begin
  //     -   
  if Node1.ImageIndex = Node2.ImageIndex then Compare := 0
  else if NodeTypes[Node1] < NodeTypes[Node2]
  then Compare := -1 else Compare := 1;

  if Compare = 0 then  //    
    if Node1.Text = Node2.Text then Compare := 0
    else if Node1.Text < Node2.Text
    then Compare := -1 else Compare := 1
end;

procedure TovModelTreeViewBuilder.TreeViewExpanding(Sender: TObject; Node:
  TTreeNode; var AllowExpansion: Boolean);
var
  Relation: TModelRelation;
begin
  if (Node.Count > 0) and (NodeTypes[Node.Item[0]] = ntRelationNodes) then
  with TreeViewItems do
  begin
    BeginUpdate;
    try
      Node.Item[0].Delete;
      Relation := TModelRelation(Node.Data);
      VisitClass(Relation.TargetObject, Node, False);
      Node.AlphaSort;
    finally
      EndUpdate;
    end;
  end
end;

function TovModelTreeViewBuilder.GetNodeTypes(Node: TTreeNode): TNodeType;
var
  I: Integer;
begin
  if Node = nil then Result := ntNone
  else begin
    I := Integer(Node.Data);
    if (I = Ord(ntNone)) or (I = Ord(ntNamespace)) or (I = Ord(ntRelationNodes))
    then Result := TNodeType(I)
    else
      if TObject(Node.Data) is TModelDataType then Result := ntDataType else
      if TObject(Node.Data) is TModelObject then Result := ntObject else
      if TObject(Node.Data) is TModelAttribute then Result := ntAttribute else
      if TObject(Node.Data) is TModelObjectIndex then Result := ntIndex else
      if TObject(Node.Data) is TModelRelation then
        if TModelRelation(Node.Data).IsGeneralization then
          Result := ntGeneralization
        else
          Result := ntRelation
      else raise Exception.Create('  ');
  end
end;

function TovModelTreeViewBuilder.GetNodeObjects(Node: TTreeNode): TObject;
begin
  if NodeTypes[Node] in [ntDataType, ntObject, ntAttribute, ntGeneralization, ntRelation, ntIndex] then
    Result := TObject(Node.Data)
  else
    Result := nil
end;

{{
  ,     .  
 ,   nil.
}
function TovModelTreeViewBuilder.FindObjectNode(AObject: TObject): TTreeNode;

  function CheckNode(ParentNode: TTreeNode): TTreeNode;
  var
    I: Integer;
  begin
    if ParentNode.Data = AObject then
      Result := ParentNode
    else begin
      for I := 0 to ParentNode.Count - 1 do
      begin
        Result := CheckNode(ParentNode[I]);
        if Result <> nil then Exit;
      end;
      Result := nil;
    end
  end;

var
  I: Integer;
begin
  for I := 0 to TreeViewItems.Count - 1 do
  begin
    Result := CheckNode(TreeViewItems[I]);
    if Result <> nil then Exit;
  end;
  Result := nil;
end;

procedure TovModelTreeViewBuilder.SetNodeObjects(Node: TTreeNode; Value:
  TObject);
begin
  Node.Data := Value;
end;

procedure TovModelTreeViewBuilder.VisitNamespaces(ParentNode: TTreeNode;
  ShowRoot: Boolean);
var
  NameSpaceNode: TTreeNode;
  I: Integer;
  Skip: Boolean;
begin
  with DomainModel do
  if NameSpaces.Count > 0 then
  for I := 0 to NameSpaces.Count - 1 do
  begin
    Skip := False;
    if Assigned(FOnVisitNamespace) then FOnVisitNamespace(DomainModel.NameSpaces[I], Skip);
    if Not Skip then
    begin
      if ShowRoot then
      begin
        NameSpaceNode := TreeViewItems.Add(ParentNode, GetNamespaceText(DomainModel.NameSpaces[I]));
        NameSpaceNode.ImageIndex := TreeNodeImageIndexes.NameSpaceNodeImageIndex;
        NameSpaceNode.SelectedIndex := TreeNodeImageIndexes.NameSpaceNodeImageIndex;
        NameSpaceNode.Data := Pointer(ntNamespace);
      end else
        NameSpaceNode := ParentNode;
      VisitObjects(NameSpaces[I], NameSpaceNode);
    end;
  end
end;

procedure TovModelTreeViewBuilder.VisitObjects(ANamespace: String; AParentNode:
  TTreeNode);
var
  ModelObject: TModelObject;
  I: Integer;
  DataType: TModelDataType;
  Skip: Boolean;
begin
  if tvbShowDataTypes in Options then
  for I := 0 to DomainModel.DataTypeCount - 1 do
  begin
    DataType := DomainModel.DataTypes[I];
    if DataType.Namespace = ANamespace then
      VisitDataType(DataType, AParentNode);
  end;

  for I := 0 to DomainModel.ObjectCount - 1 do
  begin
    ModelObject := DomainModel.Objects[I];
    if (ModelObject.Namespace = ANamespace) and
       (ModelObject.VisibleForUser or Not (tvbShowVisibleForUserObjects in Options)) then
    begin
      Skip := False;
      if Assigned(FOnVisitObject) then FOnVisitObject(ModelObject, Skip);
      if Not Skip then
        VisitClass(ModelObject, AParentNode, True);
    end;
  end
end;

procedure TovModelTreeViewBuilder.VisitDataType(AModelDataType: TModelDataType;
  AParentNode: TTreeNode);
var
  DataTypeNode: TTreeNode;
begin
  DataTypeNode := TreeViewItems.AddChildFirst(AParentNode, AModelDataType.Name);
  DataTypeNode.ImageIndex := TreeNodeImageIndexes.DataTypeNodeImageIndex;
  DataTypeNode.SelectedIndex := TreeNodeImageIndexes.DataTypeNodeImageIndex;
  DataTypeNode.Data := AModelDataType;
end;

procedure TovModelTreeViewBuilder.VisitClass(AModelObject: TModelObject;
  AParentNode: TTreeNode; ShowRoot: Boolean);
var
  ClassNode: TTreeNode;
begin
  if ShowRoot then
  begin
    ClassNode := TreeViewItems.AddChildFirst(AParentNode, AModelObject.Name);
    ClassNode.ImageIndex := TreeNodeImageIndexes.ClassNodeImageIndex;
    ClassNode.SelectedIndex := TreeNodeImageIndexes.ClassNodeImageIndex;
    ClassNode.Data := AModelObject;
  end else
    ClassNode := AParentNode;
  if tvbShowGeneralizations in Options then
    VisitGeneralizations(AModelObject, ClassNode);
  if tvbShowAttributes in Options then
    VisitAttributes(AModelObject, ClassNode);
  if tvbShowRelations in Options then
    VisitRelations(AModelObject, ClassNode);
  if tvbShowIndexes in Options then
    VisitIndexes(AModelObject, ClassNode);
end;

procedure TovModelTreeViewBuilder.VisitAttributes(AModelObject: TModelObject;
  AParentNode: TTreeNode);
var
  I: Integer;
  Attribute: TModelAttribute;
  AttributeNode: TTreeNode;
begin
  for I := 0 to AModelObject.AttributeCount - 1 do
  begin
    Attribute := AModelObject.Attributes[I];
    if Attribute.VisibleForUser or Not (tvbShowVisibleForUserObjects in Options) then
    begin
      AttributeNode := TreeViewItems.AddChildFirst(AParentNode, GetAttributeText(Attribute));
      AttributeNode.ImageIndex := TreeNodeImageIndexes.AttributeNodeImageIndex;
      AttributeNode.SelectedIndex := TreeNodeImageIndexes.AttributeNodeImageIndex;
      AttributeNode.Data := Attribute;
    end;
  end;
end;

procedure TovModelTreeViewBuilder.VisitRelations(AModelObject: TModelObject;
  AParentNode: TTreeNode);
var
  I: Integer;
  Relation: TModelRelation;
  RelationNode: TTreeNode;
begin
  for I := 0 to AModelObject.RelationCount - 1 do
  begin
    Relation := AModelObject.Relations[I];
    if (Relation.VisibleForUser or Not (tvbShowVisibleForUserObjects in Options))  and
       (Not (tvbShowSingleRelationsOnly in Options) or (Relation.TargetMaxCount = 1))
    then begin
      RelationNode := TreeViewItems.AddChildFirst(AParentNode, GetRelationText(Relation));
      RelationNode.ImageIndex := TreeNodeImageIndexes.RelationNodeImageIndex;
      RelationNode.SelectedIndex := TreeNodeImageIndexes.RelationNodeImageIndex;
      RelationNode.Data := Relation;
      //   ,       
      //     +
      if tvbExpandRelations in Options then
        TreeViewItems.AddChildFirst(RelationNode, '').Data := Pointer(ntRelationNodes);
    end;
  end;
end;

procedure TovModelTreeViewBuilder.VisitGeneralizations(AModelObject:
  TModelObject; AParentNode: TTreeNode);
var
  I: Integer;
  Relation: TModelRelation;
  RelationNode: TTreeNode;
begin
  for I := 0 to AModelObject.ParentCount - 1 do
  begin
    Relation := AModelObject.ParentRelations[I];
    RelationNode := TreeViewItems.AddChildFirst(AParentNode, GetRelationText(Relation));
    RelationNode.ImageIndex := TreeNodeImageIndexes.GeneralizationNodeImageIndex;
    RelationNode.SelectedIndex := TreeNodeImageIndexes.GeneralizationNodeImageIndex;
    RelationNode.Data := Relation;
  end;
end;

procedure TovModelTreeViewBuilder.VisitIndexes(AModelObject: TModelObject;
  AParentNode: TTreeNode);
var
  I: Integer;
  ModelIndex: TModelObjectIndex;
  IndexNode: TTreeNode;
begin
  for I := 0 to AModelObject.IndexCount - 1 do
  begin
    ModelIndex := AModelObject.Indexes[I];
    IndexNode := TreeViewItems.AddChildFirst(AParentNode, ModelIndex.Name);
    IndexNode.ImageIndex := TreeNodeImageIndexes.ClassIndexNodeImageIndex;
    IndexNode.SelectedIndex := TreeNodeImageIndexes.ClassIndexNodeImageIndex;
    IndexNode.Data := ModelIndex;
  end;
end;

end.
