unit ovModelValidator;

interface

uses
  SysUtils, Classes, ovComponents, ovModel, ovIObjects;

type
  TValidateProgressEvent = procedure (Sender: TObject;
    Done, Max: Integer; var Cancel: Boolean) of object;
  TValidateErrorEvent = procedure (Sender: TObject; AObject: IovObject; ErrorMsg: String) of object;
  TInvalidRelationEvent = procedure (Sender: TObject; AObject: IovObject;
    AModelRelation: TModelRelation) of object;
  TValidateRelationEvent = procedure (Sender: TObject; var Cancel: Boolean) of object;

  TovModelValidator = class(TovModelClient)
  private
    FOnInvalidRelation: TInvalidRelationEvent;
    FOnValidateError: TValidateErrorEvent;
    FOnProgress: TValidateProgressEvent;
    FOnValidateRelation: TValidateRelationEvent;
    { Private declarations }
  protected
    procedure InvalidRelation(AObject: IovObject; AModelRelation: TModelRelation);
      dynamic;
    procedure ValidateError(AObject: IovObject; AErrorMsg: String); dynamic;
    function ValidateObjects(AModelObject: TModelObject): Boolean;
    function ValidateObject(AModelObject: TModelObject; AObject: IovObject):
      Boolean; virtual;
    function DoProgress(Done, Max: Integer): Boolean; virtual;
    procedure DoValidateRelation(var Cancel: Boolean);
    { Protected declarations }
  public
    function Validate: Boolean;
    { Public declarations }
  published
    { Published declarations }

    {{
            
    }
    property OnInvalidRelation: TInvalidRelationEvent read FOnInvalidRelation write
      FOnInvalidRelation;
    {{
           .
    }
    property OnValidateError: TValidateErrorEvent read FOnValidateError write
        FOnValidateError;
    property OnProgress: TValidateProgressEvent read FOnProgress write FOnProgress;
    property OnValidateRelation: TValidateRelationEvent read FOnValidateRelation
        write FOnValidateRelation;

    { TODO : property OnValidateObject }
  end;

implementation

{{
     .  False,  
      True   .
}
function TovModelValidator.Validate: Boolean;
var
  I: Integer;
begin
  ModelNeeded;
  Result := True;
  for I := 0 to DomainModel.ObjectCount - 1 do
  begin
    if Not ValidateObjects(DomainModel.Objects[I]) then Result := False;
    if Not DoProgress(I, DomainModel.ObjectCount) then
    begin
      Result := False;
      Exit;
    end;
  end;
end;

function TovModelValidator.ValidateObjects(AModelObject: TModelObject): Boolean;
var
  I: Integer;
begin
  Result := True;
  with Model.ObjectSpace.Classes[AModelObject.FullName].Objects do
    for I := 0 to Count - 1 do
      if Not ValidateObject(AModelObject, Objects[I]) then Result := False
end;

{{
  AObject   AModelObject.
    , ..    .
}
function TovModelValidator.ValidateObject(AModelObject: TModelObject; AObject:
  IovObject): Boolean;
var
  I: Integer;
  CurrCount: Integer;
  Cancel: Boolean;
begin
  Result := True;
  try
    with AModelObject do
      for I := 0 to RelationCount - 1 do
      begin
        CurrCount := AObject[Relations[I].Name].Count;
        if CurrCount < Relations[I].TargetMinCount then
        begin
          InvalidRelation(AObject, Relations[I]);
          Result := False;
        end else
        if CurrCount > Relations[I].TargetMaxCount then
        begin
          InvalidRelation(AObject, Relations[I]);
          Result := False;
        end;

        Cancel := False;
        DoValidateRelation(Cancel);
        if Cancel then Abort;
      end;
  except
    on E: Exception do
    if Not (E is EAbort) then
    begin
      ValidateError(AObject, E.Message);
      Result := False;
    end else raise
  end;
end;

procedure TovModelValidator.InvalidRelation(AObject: IovObject; AModelRelation:
  TModelRelation);
begin
  if Assigned(FOnInvalidRelation) then FOnInvalidRelation(Self, AObject, AModelRelation);
end;

procedure TovModelValidator.ValidateError(AObject: IovObject; AErrorMsg:
    String);
begin
  if Assigned(FOnValidateError) then FOnValidateError(Self, AObject, AErrorMsg);
end;

function TovModelValidator.DoProgress(Done, Max: Integer): Boolean;
var
  Cancel: Boolean;
begin
  Cancel := False;
  if Assigned(FOnProgress) then FOnProgress(Self, Done+1, Max, Cancel);
  Result := Not Cancel;
end;

procedure TovModelValidator.DoValidateRelation(var Cancel: Boolean);
begin
  if Assigned(FOnValidateRelation) then FOnValidateRelation(Self, Cancel);
end;

end.
