{$I oView.inc}
unit ovOQLProcessor;

interface

uses
  SysUtils, Windows, Classes, Contnrs, ovIObjects, ovModel, ovOQLTypes, ovOQLGrammar;

type
  TQueryTreeRelationNode = class;

  TRelationImplementor = class;

  TQueryTreeNode = class
  private
    FChildNodes: TObjectList;
    FParentNode: TQueryTreeNode;
    FModelClass: TModelObject;
    FSQLTableAlias: String;
    function GetChildNodeCount: Integer;
    function GetChildNodes(Index: Integer): TQueryTreeRelationNode;
  protected
    property ParentNode: TQueryTreeNode read FParentNode;
  public
    constructor Create(AParentNode: TQueryTreeNode; AModelClass: TModelObject);
    destructor Destroy; override;
    function AddChildNode(AModelRelation: TModelRelation): TQueryTreeRelationNode;
    function FindChildNode(AModelRelation: TModelRelation; var Node:
      TQueryTreeNode): Boolean;
    property ModelClass: TModelObject read FModelClass;
    property ChildNodeCount: Integer read GetChildNodeCount;
    property ChildNodes[Index: Integer]: TQueryTreeRelationNode read GetChildNodes;

    property SQLTableAlias: String read FSQLTableAlias;
  end;

  TJoinType = (jtLeftOuter, jtRightOuter, jtInner);

  TSelectStatementProcessor = class;

  TRelationImplementor = class
  private
    FModelRelation: TModelRelation;
  protected
    procedure JoinTables(SelectStatementProcessor: TSelectStatementProcessor;
      ChildTableName, ChildTableAlias: String; JoinType: TJoinType); overload;
    procedure AddJoinCondition(SelectStatementProcessor: TSelectStatementProcessor;
      ParentTableAlias, ChildTableAlias: String; const JoinedFields: array of
      Variant; AdditionalConditions: TQueryStatementExpression = nil);
  public
    constructor Create(AModelRelation: TModelRelation);
    procedure JoinTables(SelectStatementProcessor: TSelectStatementProcessor;
      ParentNode: TQueryTreeNode; ChildNode: TQueryTreeRelationNode); overload;
      virtual; abstract;
    procedure AddRelationIdField(SelectStatementProcessor: TSelectStatementProcessor;
      ParentNode, ChildNode: TQueryTreeNode);
      virtual; abstract;
    property ModelRelation: TModelRelation read FModelRelation;
  end;

  {{
  ,    
  }
  TQueryTreeRelationNode = class(TQueryTreeNode)
  private
    FModelRelation: TModelRelation;
    FJoinType: TJoinType;
    FJoinConditions: TQueryStatementExpression;
    FDBRelation: TRelationImplementor;
  public
    constructor Create(AParentNode: TQueryTreeNode; AModelRelation: TModelRelation);
    destructor Destroy; override;
    property ModelRelation: TModelRelation read FModelRelation;
    property DBRelation: TRelationImplementor read FDBRelation;
    {{
      ,      
       JOIN   SQL-
    }
    property JoinType: TJoinType read FJoinType write FJoinType;
    property JoinConditions: TQueryStatementExpression read FJoinConditions;
  end;

  {{
  ,  ,    FROM 
  }
  TQueryTreeClassNode = class(TQueryTreeNode)
  private
    FName: String;
  public
    constructor Create(AParentNode: TQueryTreeNode; AModelClass: TModelObject;
      const AName: String);
    {{
    ,          .
       FROM  ,  Name    ,
     -   .
    }
    property Name: String read FName;
  end;

  TQueryTree = class
  private
    FRootNodes: TObjectList;
  protected
    function GetRootNodeCount: Integer;
    function GetRootNodes(Index: Integer): TQueryTreeClassNode;
  public
    constructor Create;
    destructor Destroy; override;
    function AddRootNode(AModelClass: TModelObject; ANodeName: String):
      TQueryTreeClassNode;
    function FindRootNode(NodeName: String; var Node: TQueryTreeNode): Boolean;
    property RootNodeCount: Integer read GetRootNodeCount;
    property RootNodes[Index: Integer]: TQueryTreeClassNode read GetRootNodes;
  end;

  TQueryProcessor = class;

  TSelectStatementProcessor = class
    procedure GetNextTokenCheck(TokenType: TTokenType; var TokenText: String);
  private
    FQueryTree: TQueryTree;
    FSourceStatement: TSelectStatement;
    FModel: TDomainModel;
    FQueryProcessor: TQueryProcessor;
    FTargetStatement: TSelectStatement;
    FCurrentTokenSource: TQueryStatementExpression;
    FSourceTokenIndex: Integer;
    FCurrentTokenTarget: TQueryStatementExpression;
    procedure GenerateFromExpression;
    function GetNextToken(var Token: TQueryStatementExpressionItem): Boolean;
    procedure ProcessIdent(AllowSelectAllToken: Boolean);
    procedure InitTokenStream(SourceExpression, TargetExpression:
      TQueryStatementExpression);
    function HasMoreTokens: Boolean;
    function NextToken: TTokenType;
    procedure GetToken(var Token: TQueryStatementExpressionItem);
    procedure PutToken(Token: TQueryStatementExpressionItem); overload;
    procedure PutToken(TokenType: TTokenType; const TokenText: String); overload;
    function GenerateSQL: String;
    function GenerateSelectExpressionText(SelectExpression:
      TQueryStatementExpression): String;
    procedure ProcessSelectExpression;
    procedure ProcessFromExpression;
    procedure ProcessFetchExpression;
    procedure ProcessWhereExpression;
    procedure ProcessGroupByExpression;
    procedure ProcessHavingExpression;
    procedure ProcessOrderByExpression;
    procedure ProcessUnionExpression;
    procedure ProcessFieldList;
    procedure ProcessSearchCondition(ToToken: TTokenType = ttNone);
    property QueryProcessor: TQueryProcessor read FQueryProcessor;
  public
    constructor Create(AQueryProcessor: TQueryProcessor; ASelectStatement:
      TSelectStatement);
    destructor Destroy; override;
    function Process: String;
    property Model: TDomainModel read FModel write FModel;
    {{
       SELECT
    }
    property SourceStatement: TSelectStatement read FSourceStatement;
    {{
      SELECT
    }
    property TargetStatement: TSelectStatement read FTargetStatement;
    property QueryTree: TQueryTree read FQueryTree;
  end;

  {{
   ,       .
  }
  TQueryProcessor = class(TObject)
  private
    FDatabaseAdapter: IDatabaseAdapter;
    FModel: TDomainModel;
    FOQLGrammar: TovOQLGrammar;
    function ProcessSubQuery(SelectStatementIndex: Integer): String;
    procedure Error(Msg: String);
    procedure Initialize;
  protected
    property OQLGrammar: TovOQLGrammar read FOQLGrammar;
  public
    constructor Create;
    destructor Destroy; override;
    {{
     ,       .
    }
    function ProcessQuery(Query: String): String;
    property DatabaseAdapter: IDatabaseAdapter read FDatabaseAdapter write
        FDatabaseAdapter;
    property Model: TDomainModel read FModel write FModel;
  end;

  EQueryProcessorError = class (Exception)
  end;

implementation

uses CocoBase, Variants, ovConsts, TypInfo;

procedure StrToStream(Str: String; Stream: TStream);
begin
  with Stream do
  begin
    Size := 0;
    Write(Str[1], Length(Str));
    Position := 0;
  end;
end;

function StreamToStr(Stream: TStream): String;
begin
  with Stream do
  begin
    Position := 0;
    SetLength(Result, Size);
    Read(Result[1], Size);
  end;
end;

var
  TableCount: Integer = 0;

function GetNewTableAlias: String;
begin
  Inc(TableCount);
  Result := Format('T%d', [TableCount]);
end;

function CreateRelationImplementor(ModelRelation: TModelRelation):
  TRelationImplementor; forward;

{ TQueryProcessor }

constructor TQueryProcessor.Create;
begin
  inherited Create;
  FOQLGrammar := TovOQLGrammar.Create(nil);
  FOQLGrammar.GenListWhen := glOnError;
end;

destructor TQueryProcessor.Destroy;
begin
  FreeAndNil(FOQLGrammar);
  inherited Destroy;
end;

procedure TQueryProcessor.Initialize;
begin
  TableCount := 0;
end;

function TQueryProcessor.ProcessQuery(Query: String): String;
begin
  Initialize;
  StrToStream(Query, OQLGrammar.SourceStream);
  OQLGrammar.Execute;
  if OQLGrammar.Successful then
  begin
    if OQLGrammar.SelectStatementCount > 0 then
      Result := ProcessSubQuery(0)
    else
      Error(' SELECT-')
  end else
    Error(StreamToStr(OQLGrammar.ListStream))
end;

function TQueryProcessor.ProcessSubQuery(SelectStatementIndex: Integer): String;
begin
  with TSelectStatementProcessor.Create(Self,
    OQLGrammar.SelectStatements[SelectStatementIndex]) do
  try
    Result := Process
  finally
    Free
  end;
end;

procedure TQueryProcessor.Error(Msg: String);
begin
  raise EQueryProcessorError.Create(Msg);
end;

{ TSelectStatementProcessor }

constructor TSelectStatementProcessor.Create(AQueryProcessor: TQueryProcessor;
  ASelectStatement: TSelectStatement);
begin
  inherited Create;
  FQueryProcessor := AQueryProcessor;
  FModel := AQueryProcessor.Model;
  FSourceStatement := ASelectStatement;
  FTargetStatement := TSelectStatement.Create;
  FQueryTree := TQueryTree.Create;
end;

destructor TSelectStatementProcessor.Destroy;
begin
  FreeAndNil(FTargetStatement);
  FreeAndNil(FQueryTree);
  inherited Destroy;
end;

function TSelectStatementProcessor.Process: String;
begin
  ProcessFromExpression;    //   From-  
  //   SQL-        
  ProcessSelectExpression;
  ProcessWhereExpression;
  ProcessGroupByExpression;
  ProcessHavingExpression;
  ProcessOrderByExpression;
  ProcessFetchExpression;  //      
  ProcessUnionExpression;
  GenerateFromExpression;  //   From- SQL-
  Result := GenerateSQL;
end;

{***************************************************************************** }
{                        }
{ **************************************************************************** }

{$IFDEF DELPHI2006}
{$REGION '   '}
{$ENDIF}

procedure TSelectStatementProcessor.InitTokenStream(SourceExpression,
  TargetExpression: TQueryStatementExpression);
begin
  FCurrentTokenSource := SourceExpression;
  FSourceTokenIndex := -1;
  FCurrentTokenTarget := TargetExpression;
  if Assigned(TargetExpression) then TargetExpression.Clear;
end;

function TSelectStatementProcessor.HasMoreTokens: Boolean;
begin
  Result := FSourceTokenIndex + 1 < FCurrentTokenSource.ItemCount ;
end;

function TSelectStatementProcessor.NextToken: TTokenType;
begin
  if FSourceTokenIndex+1 < FCurrentTokenSource.ItemCount then
    Result := FCurrentTokenSource.Items[FSourceTokenIndex+1].ItemType
  else
    Result := ttNone
end;

{{
   (  )  , 
 GetFirstToken.
}
function TSelectStatementProcessor.GetNextToken(var Token:
  TQueryStatementExpressionItem): Boolean;
begin
  Inc(FSourceTokenIndex);
  Result := FSourceTokenIndex < FCurrentTokenSource.ItemCount;
  if Result then
    Token := FCurrentTokenSource.Items[FSourceTokenIndex]
  else
    Token := nil;
end;

procedure TSelectStatementProcessor.GetNextTokenCheck(TokenType: TTokenType;
  var TokenText: String);
var
  Token: TQueryStatementExpressionItem;
begin
  if Not GetNextToken(Token) or (Token.ItemType <> TokenType)
    then QueryProcessor.Error(' :'+
      GetEnumName(TypeInfo(TTokenType), Integer(Token.ItemType)));
  TokenText := Token.ItemText;
end;

procedure TSelectStatementProcessor.GetToken(var Token:
  TQueryStatementExpressionItem);
begin
  Inc(FSourceTokenIndex);
  Token := FCurrentTokenSource.Items[FSourceTokenIndex];
end;

procedure TSelectStatementProcessor.PutToken(Token:
  TQueryStatementExpressionItem);
begin
  PutToken(Token.ItemType, Token.ItemText);
end;

procedure TSelectStatementProcessor.PutToken(TokenType: TTokenType; const
  TokenText: String);
begin
  with FCurrentTokenTarget.AddItem do
  begin
    ItemType := TokenType;
    ItemText := TokenText;
  end;
end;

{$IFDEF DELPHI2006}
{$ENDREGION '   '}
{$ENDIF}

{***************************************************************************** }
{                                       }
{ **************************************************************************** }

{$IFDEF DELPHI2006}
{$REGION '  '}
{$ENDIF}

{{
    FROM      
   .
}
procedure TSelectStatementProcessor.ProcessFromExpression;
var
  ClassName: String;
  Token: TQueryStatementExpressionItem;
begin
  InitTokenStream(SourceStatement.FromExpression, nil);
  while HasMoreTokens do
  begin
    GetNextTokenCheck(ttClassName, ClassName);
    if NextToken = ttClassAlias then
    begin
      GetToken(Token);
      QueryTree.AddRootNode(Model.GetObject(ClassName), Token.ItemText);
    end else
      QueryTree.AddRootNode(Model.GetObject(ClassName), ClassName);

    if HasMoreTokens then GetToken(Token) {     }
  end
end;

{{
    FETCH  :  
          
   FETCH      
JOIN   .
}
procedure TSelectStatementProcessor.ProcessFetchExpression;

  function FindQueryTreeNode: TQueryTreeRelationNode;
  var
    Token: TQueryStatementExpressionItem;
    Relation: TModelRelation;
    CurrNode: TQueryTreeNode;
  begin
    {   ()      }
    GetToken(Token);
    if (Token.ItemType = ttIdent) and QueryTree.FindRootNode(Token.ItemText, CurrNode) then
      {    ( ) -     }
      GetToken(Token)
    else
      {   ,     }
      CurrNode := QueryTree.RootNodes[0];

    { ,         }
    repeat
      Relation := CurrNode.ModelClass.GetRelation(Token.ItemText);
      if Not CurrNode.FindChildNode(Relation, CurrNode)
        then QueryProcessor.Error('  :' + Token.ItemText)
        else {      } ;

      {     }
      if NextToken = ttIdent then GetToken(Token) else Break;
    until False;
    Result := CurrNode as TQueryTreeRelationNode;
  end;

var
  Token: TQueryStatementExpressionItem;
  Node: TQueryTreeRelationNode;
begin
  InitTokenStream(SourceStatement.FetchExpression, nil);
  while HasMoreTokens do
  begin
    Node := FindQueryTreeNode; {         }

    {     }
    case NextToken of
      ttFetchAll:
      begin
        Node.JoinType := jtRightOuter;
        GetToken(Token);
      end;
      ttFetchExisting:
      begin
        Node.JoinType := jtInner;
        GetToken(Token);
      end;
    else
      {    }
      Node.JoinType := jtLeftOuter;
    end;

    {    }
    if NextToken = ttFetchCondition then
    begin
      GetToken(Token);
      {     ,   
          JoinConditions   }
      Node.JoinConditions.Clear;
      FCurrentTokenTarget := Node.JoinConditions;
      ProcessSearchCondition(ttComma); 
    end;
    if HasMoreTokens then GetToken(Token) {     }
  end;
end;

{{
   ,  ,  
      .
}
procedure TSelectStatementProcessor.ProcessSelectExpression;
var
  Token: TQueryStatementExpressionItem;
begin
  InitTokenStream(SourceStatement.SelectExpression, TargetStatement.SelectExpression);
  while HasMoreTokens do
  begin
    while HasMoreTokens and Not (NextToken in [ttIdent, ttSelectAll]) do
    begin
      GetToken(Token);
      PutToken(Token);
    end;
    case NextToken of
      ttIdent, ttSelectAll:
        begin
          ProcessIdent(True);
        end;
    end;
  end;
end;

procedure TSelectStatementProcessor.ProcessWhereExpression;
begin
  InitTokenStream(SourceStatement.WhereExpression, TargetStatement.WhereExpression);
  ProcessSearchCondition;
end;

procedure TSelectStatementProcessor.ProcessGroupByExpression;
begin
  InitTokenStream(SourceStatement.GroupByExpression, TargetStatement.GroupByExpression);
  ProcessFieldList;
end;

procedure TSelectStatementProcessor.ProcessHavingExpression;
begin
  InitTokenStream(SourceStatement.HavingExpression, TargetStatement.HavingExpression);
  ProcessSearchCondition;
end;

procedure TSelectStatementProcessor.ProcessOrderByExpression;
begin
  InitTokenStream(SourceStatement.OrderByExpression, TargetStatement.OrderByExpression);
  ProcessFieldList;
end;

procedure TSelectStatementProcessor.ProcessUnionExpression;
var
  Token: TQueryStatementExpressionItem;
begin
  InitTokenStream(SourceStatement.UnionExpression, TargetStatement.UnionExpression);
  while HasMoreTokens do
  begin
    if NextToken <> ttSubSelect then
    begin
      GetToken(Token);
      PutToken(Token);
    end else begin
      GetToken(Token);
      Token.ItemText := QueryProcessor.ProcessSubQuery(StrToInt(Token.ItemText));
      PutToken(Token);
    end;
  end;
end;

{{
  ,  ,  
      .
}
procedure TSelectStatementProcessor.ProcessFieldList;
var
  Token: TQueryStatementExpressionItem;
begin
  while HasMoreTokens do
  begin
    ProcessIdent(False);
    if NextToken = ttSortOrder then
    begin
      GetToken(Token);        
      PutToken(Token);
    end;
    if NextToken = ttComma then
    begin
      GetToken(Token);
      PutToken(Token);
    end
  end;
end;

{{
   ,   
     .   
ttSubSelect        
   .      
 ToToken (  -   ).
}
procedure TSelectStatementProcessor.ProcessSearchCondition(ToToken: TTokenType
  = ttNone);
var
  Token: TQueryStatementExpressionItem;
begin
  while Not (NextToken in [ttNone, ToToken]) do
  begin
    while Not (NextToken in [ttNone, ttIdent, ttSubSelect, ToToken]) do
    begin
      GetToken(Token);
      PutToken(Token);
    end;
    case NextToken of
      ttIdent:
        begin
          ProcessIdent(False);
        end;
      ttSubSelect:
      begin
        GetToken(Token);
        Token.ItemText := QueryProcessor.ProcessSubQuery(StrToInt(Token.ItemText));
        PutToken(Token);
      end;
    end;
  end;
end;

{$IFDEF DELPHI2006}
{$ENDREGION '  '}
{$ENDIF}

{***************************************************************************** }
{                   }
{ **************************************************************************** }

{$IFDEF DELPHI2006}
{$REGION 'Object-relational mapping'}
{$ENDIF}

{{
        .
     ,   
.      
 .
}
procedure TSelectStatementProcessor.ProcessIdent(AllowSelectAllToken: Boolean);
var
  CurrNode: TQueryTreeNode;

  function GetAttributeNode(Attribute: TModelAttribute): TQueryTreeNode;
  var
    Generalization: TModelRelation;
  begin
    if Not Attribute.IsInherited then
      Result := CurrNode
    else
    begin
      {      ,    
       , ..    }
      Generalization := CurrNode.ModelClass.GetParentRelation(Attribute.ParentObject.FullName);
      {     ,     }
      if Not CurrNode.FindChildNode(Generalization, Result) then
        Result := CurrNode.AddChildNode(Generalization);
    end;
  end;

  function GetRelationNode(Relation: TModelRelation): TQueryTreeNode;
  begin
    if Not CurrNode.FindChildNode(Relation, Result) then
      Result := CurrNode.AddChildNode(Relation);
  end;

  {{
      
  }
  procedure ProcessSimpleIdent(Token: TQueryStatementExpressionItem);
  var
    CurrIdent: String;
    I: Integer;
    Attribute: TModelAttribute;
    Relation: TModelRelation;
  begin
    if Token.ItemType = ttSelectAll then
    begin
      {   * }
      if NextToken in [ttIdent, ttSelectAll] then QueryProcessor.Error(' *    ');
      with CurrNode.ModelClass do
        for I := 0 to AttributeCount - 1 do
        begin
          PutToken(ttTableAlias, CurrNode.SQLTableAlias);
          PutToken(ttFieldName, Attributes[I].FieldName);
          if I < AttributeCount - 1 then PutToken(ttComma, ',');
        end

    end else begin
      CurrIdent := Token.ItemText;
      if UpperCase(CurrIdent) = 'OID' then
        {  OID }
        if NextToken = ttIdent then QueryProcessor.Error(' OID   ')
        else begin
          PutToken(ttTableAlias, CurrNode.SQLTableAlias);
          PutToken(ttFieldName, ObjectIdField);
        end
      else
      if UpperCase(CurrIdent) = 'CID' then
        {  CID }
        if NextToken = ttIdent then QueryProcessor.Error(' CID   ')
        else begin
          PutToken(ttTableAlias, CurrNode.SQLTableAlias);
          PutToken(ttFieldName, ClassIdField);
        end

      else
      begin
        Attribute := CurrNode.ModelClass.FindAttribute(CurrIdent);
        if Assigned(Attribute) then
          {   }
          if NextToken = ttIdent then QueryProcessor.Error('   ')
          else begin
            PutToken(ttTableAlias, GetAttributeNode(Attribute).SQLTableAlias);
            PutToken(ttFieldName, Attribute.FieldName);
          end

        else
        begin
          Relation := CurrNode.ModelClass.FindRelation(CurrIdent);
          if Assigned(Relation)
          then
            {   }

            {     ,     }
            CurrNode := GetRelationNode(Relation)
          else
            QueryProcessor.Error('    :'+CurrIdent);

          if (NextToken <> ttIdent) and (NextToken <> ttSelectAll) then
          {     -      }
            (CurrNode as TQueryTreeRelationNode).DBRelation.AddRelationIdField(Self, CurrNode.ParentNode, CurrNode);

        end;
      end;
    end;
  end;

var
  Token: TQueryStatementExpressionItem;
begin
  {   ()        }
  GetToken(Token);
  if (Token.ItemType = ttIdent) and QueryTree.FindRootNode(Token.ItemText, CurrNode) then
    {    ( ) }
    GetToken(Token)
  else
    {     ,     }
    CurrNode := QueryTree.RootNodes[0];
  ProcessSimpleIdent(Token);

  { ,         }
  while NextToken in [ttIdent, ttSelectAll] do
  begin
    {     }
    GetToken(Token);
    ProcessSimpleIdent(Token);
  end;
end;

{$IFDEF DELPHI2006}
{$ENDREGION 'Object-relational mapping'}
{$ENDIF}

{***************************************************************************** }
{                 SQL-                 }
{ **************************************************************************** }

{$IFDEF DELPHI2006}
{$REGION ' SQL-'}
{$ENDIF}

{{
  FROM     
     ,    JOIN.
}
procedure TSelectStatementProcessor.GenerateFromExpression;

  procedure ProcessNode(Node: TQueryTreeNode);
  var
    I: Integer;
  begin
    with Node do
    begin
      if Node is TQueryTreeClassNode then
      begin
        {    FROM     :
          < > <   SQL-> }
        PutToken(ttTableName, ModelClass.TableName);
        PutToken(ttTableAlias, SQLTableAlias);
      end else
      if Node is TQueryTreeRelationNode then
        TQueryTreeRelationNode(Node).DBRelation.JoinTables(Self, ParentNode, TQueryTreeRelationNode(Node));

      for I := 0 to ChildNodeCount - 1 do ProcessNode(ChildNodes[I]);
    end;
  end;

var
  I: Integer;
begin
  InitTokenStream(SourceStatement.FromExpression, TargetStatement.FromExpression);
  for I := 0 to QueryTree.RootNodeCount - 1 do
  begin
    ProcessNode(QueryTree.RootNodes[I]);
    if I <> QueryTree.RootNodeCount - 1 then PutToken(ttComma, ',');
  end;
end;

{{
    SQL-   
   ,   FCurrentTokenTarget
(  FROM)       .

  SELECT      
(   ).
}
function TSelectStatementProcessor.GenerateSQL: String;
begin
  with FTargetStatement do
  begin
    Result := 'SELECT' + GenerateSelectExpressionText(SelectExpression);
    Result := Result + ' FROM' + GenerateSelectExpressionText(FromExpression);
    if WhereExpression.ItemCount > 0 then
      Result := Result + ' WHERE' + GenerateSelectExpressionText(WhereExpression);
    if GroupByExpression.ItemCount > 0 then
      Result := Result + ' GROUP BY' + GenerateSelectExpressionText(GroupByExpression);
    if HavingExpression.ItemCount > 0 then
      Result := Result + ' HAVING' + GenerateSelectExpressionText(HavingExpression);
    if OrderByExpression.ItemCount > 0 then
      Result := Result + ' ORDER BY' + GenerateSelectExpressionText(OrderByExpression);
    if UnionExpression.ItemCount > 0 then
      Result := Result + ' UNION' + GenerateSelectExpressionText(UnionExpression);
  end;
end;

function TSelectStatementProcessor.GenerateSelectExpressionText(
  SelectExpression: TQueryStatementExpression): String;
var
  I: Integer;
  LT, RT, LF, RF: String;
begin
  with QueryProcessor.DatabaseAdapter.GetQuerySyntaxParams do
  begin
    LT := LeftTableDelimiter;
    RT := RightTableDelimiter;
    LF := LeftFieldDelimiter;
    RF := RightFieldDelimiter;
  end;

  Result := '';
  for I := 0 to SelectExpression.ItemCount - 1 do
    case SelectExpression.Items[I].ItemType of
      ttTableName:
        Result := Result + ' ' + LT+ SelectExpression.Items[I].ItemText + RT;
      ttFieldName:
      begin
        if (I > 0) and (SelectExpression.Items[I-1].ItemType = ttTableAlias) then
          Result := Result + '.';
        Result := Result + LF + SelectExpression.Items[I].ItemText + RF;
      end
      else
        Result := Result + ' ' + SelectExpression.Items[I].ItemText;
    end;
end;

{$IFDEF DELPHI2006}
{$ENDREGION ' SQL-'}
{$ENDIF}

{***************************************************************************** }
{                           Query tree management                              }
{ **************************************************************************** }

{$IFDEF DELPHI2006}
{$REGION 'Query tree management'}
{$ENDIF}

{ TQueryTree }

constructor TQueryTree.Create;
begin
  inherited Create;
  FRootNodes := TObjectList.Create;
end;

destructor TQueryTree.Destroy;
begin
  FreeAndNil(FRootNodes);
  inherited Destroy;
end;

function TQueryTree.GetRootNodeCount: Integer;
begin
  Result := FRootNodes.Count;
end;

function TQueryTree.GetRootNodes(Index: Integer): TQueryTreeClassNode;
begin
  Result := TQueryTreeClassNode(FRootNodes[Index]);
end;

function TQueryTree.AddRootNode(AModelClass: TModelObject; ANodeName: String):
  TQueryTreeClassNode;
begin
  Result := TQueryTreeClassNode.Create(nil, AModelClass, ANodeName);
  FRootNodes.Add(Result);
end;

function TQueryTree.FindRootNode(NodeName: String; var Node: TQueryTreeNode):
  Boolean;
var
  I: Integer;
begin
  for I := 0 to RootNodeCount - 1 do
    if RootNodes[I].Name = NodeName then
    begin
      Node := RootNodes[I];
      Result := True;
      Exit;
    end;
  Result := False;
end;

{ TQueryTreeNode }

constructor TQueryTreeNode.Create(AParentNode: TQueryTreeNode; AModelClass:
  TModelObject);
begin
  inherited Create;
  FParentNode := AParentNode;
  FChildNodes := TObjectList.Create;
  FModelClass := AModelClass;
  FSQLTableAlias := GetNewTableAlias;
end;

destructor TQueryTreeNode.Destroy;
begin
  FreeAndNil(FChildNodes);
  inherited Destroy;
end;

function TQueryTreeNode.GetChildNodeCount: Integer;
begin
  Result := FChildNodes.Count
end;

function TQueryTreeNode.GetChildNodes(Index: Integer): TQueryTreeRelationNode;
begin
  Result := TQueryTreeRelationNode(FChildNodes[Index])
end;

function TQueryTreeNode.FindChildNode(AModelRelation: TModelRelation; var Node:
  TQueryTreeNode): Boolean;
var
  I: Integer;
begin
  for I := 0 to ChildNodeCount - 1 do
    if ChildNodes[I].ModelRelation = AModelRelation then
    begin
      Node := ChildNodes[I];
      Result := True;
      Exit;
    end;
  Result := False;
end;

function TQueryTreeNode.AddChildNode(AModelRelation: TModelRelation):
  TQueryTreeRelationNode;
begin
  Result := TQueryTreeRelationNode.Create(Self, AModelRelation);
  FChildNodes.Add(Result);
end;

{ TQueryTreeClassNode }

constructor TQueryTreeClassNode.Create(AParentNode: TQueryTreeNode;
  AModelClass: TModelObject; const AName: String);
begin
  inherited Create(AParentNode, AModelClass);
  FName := AName;
end;

{ TQueryTreeRelationNode }

constructor TQueryTreeRelationNode.Create(AParentNode: TQueryTreeNode;
  AModelRelation: TModelRelation);
begin
  inherited Create(AParentNode, AModelRelation.TargetObject);
  FModelRelation := AModelRelation;
  FJoinType := jtLeftOuter;
  FJoinConditions := TQueryStatementExpression.Create;
  FDBRelation := CreateRelationImplementor(AModelRelation);
end;

destructor TQueryTreeRelationNode.Destroy;
begin
  FreeAndNil(FDBRelation);
  FreeAndNil(FJoinConditions);
  inherited Destroy;
end;

{$IFDEF DELPHI2006}
{$ENDREGION 'Query tree management'}
{$ENDIF}

{***************************************************************************** }
{                           Relation implementation                            }
{ **************************************************************************** }

{$IFDEF DELPHI2006}
{$REGION 'Relation implementation'}
{$ENDIF}

type
  TReferenceRelation = class(TRelationImplementor)
  public
    procedure JoinTables(SelectStatementProcessor: TSelectStatementProcessor;
      ParentNode: TQueryTreeNode; ChildNode: TQueryTreeRelationNode); override;
    procedure AddRelationIdField(SelectStatementProcessor:
      TSelectStatementProcessor; ParentNode, ChildNode: TQueryTreeNode); override;
  end;

  TAssociationRelation = class(TRelationImplementor)
  private
    FRelationTableAlias: String;
    function GetRelationTableAlias: String;
  public
    procedure JoinTables(SelectStatementProcessor: TSelectStatementProcessor;
      ParentNode: TQueryTreeNode; ChildNode: TQueryTreeRelationNode); override;
    procedure AddRelationIdField(SelectStatementProcessor:
      TSelectStatementProcessor; ParentNode, ChildNode: TQueryTreeNode); override;
    property RelationTableAlias: String read GetRelationTableAlias;
  end;

  TAggregationRelation = class(TRelationImplementor)
  public
    procedure JoinTables(SelectStatementProcessor: TSelectStatementProcessor;
      ParentNode: TQueryTreeNode; ChildNode: TQueryTreeRelationNode); override;
    procedure AddRelationIdField(SelectStatementProcessor:
      TSelectStatementProcessor; ParentNode, ChildNode: TQueryTreeNode); override;
  end;

  TCompositionRelation = class(TRelationImplementor)
  public
    procedure JoinTables(SelectStatementProcessor: TSelectStatementProcessor;
      ParentNode: TQueryTreeNode; ChildNode: TQueryTreeRelationNode); override;
    procedure AddRelationIdField(SelectStatementProcessor:
      TSelectStatementProcessor; ParentNode, ChildNode: TQueryTreeNode); override;
  end;

  TGeneralizationRelation = class(TRelationImplementor)
  public
    procedure JoinTables(SelectStatementProcessor: TSelectStatementProcessor;
      ParentNode: TQueryTreeNode; ChildNode: TQueryTreeRelationNode); override;
  end;

function CreateRelationImplementor(ModelRelation: TModelRelation):
  TRelationImplementor;
begin
  case ModelRelation.RelationType of
    rtAssociation:
      if ModelRelation.IsReference
        then Result := TReferenceRelation.Create(ModelRelation)
        else Result := TAssociationRelation.Create(ModelRelation);
    rtAggregation:
      Result := TAggregationRelation.Create(ModelRelation);
    rtComposition:
      Result := TCompositionRelation.Create(ModelRelation);
    rtGeneralization:
      Result := TGeneralizationRelation.Create(ModelRelation);
    else
      Result := nil;
  end;
end;

{ TRelationImplementor }

constructor TRelationImplementor.Create(AModelRelation: TModelRelation);
begin
  inherited Create;
  FModelRelation := AModelRelation;
end;

procedure TRelationImplementor.JoinTables(SelectStatementProcessor:
  TSelectStatementProcessor; ChildTableName, ChildTableAlias: String; JoinType:
  TJoinType);
const
  JoinTypeText: array [TJoinType] of String =
    ('LEFT OUTER JOIN', 'RIGHT OUTER JOIN', 'INNER JOIN');
begin
  with SelectStatementProcessor do
  begin
    PutToken(ttJoin, JoinTypeText[JoinType]);
    PutToken(ttTableName, ChildTableName);
    PutToken(ttTableAlias, ChildTableAlias);
    PutToken(ttJoinCondition, 'ON');
  end;
end;

procedure TRelationImplementor.AddJoinCondition(SelectStatementProcessor:
  TSelectStatementProcessor; ParentTableAlias, ChildTableAlias: String; const
  JoinedFields: array of Variant; AdditionalConditions:
  TQueryStatementExpression = nil);

  procedure AddJoinValue(TableAlias: String; JoinValue: Variant);
  begin
    if VarIsStr(JoinValue) then
    begin
      SelectStatementProcessor.PutToken(ttTableAlias, TableAlias);
      SelectStatementProcessor.PutToken(ttFieldName, JoinValue);
    end else
      SelectStatementProcessor.PutToken(ttInteger, IntToStr(JoinValue));
  end;

var
  I: Integer;
begin
  I := 0;
  while I + 1 < Length(JoinedFields) do
  begin
    if VarIsStr(JoinedFields[I]) then //      ,   - 
      AddJoinValue(ParentTableAlias, JoinedFields[I])
    else
      AddJoinValue(ChildTableAlias, JoinedFields[I+1]);

    SelectStatementProcessor.PutToken(ttRelation, '=');

    if VarIsStr(JoinedFields[I]) then
      AddJoinValue(ChildTableAlias, JoinedFields[I+1])
    else
      AddJoinValue(ParentTableAlias, JoinedFields[I]);

    Inc(I, 2);
    {          ,   AND }
    if (I + 1 < Length(JoinedFields)) or
       Assigned(AdditionalConditions) and (AdditionalConditions.ItemCount > 0) then
      SelectStatementProcessor.PutToken(ttLogicalOperator, 'AND');
  end;

  if Assigned(AdditionalConditions) then
  with AdditionalConditions do
  {        }
  for I := 0 to ItemCount - 1 do
    SelectStatementProcessor.PutToken(Items[I].ItemType, Items[I].ItemText);
end;

{ TReferenceRelation }

procedure TReferenceRelation.JoinTables(SelectStatementProcessor:
  TSelectStatementProcessor; ParentNode: TQueryTreeNode; ChildNode:
  TQueryTreeRelationNode);
begin
   JoinTables(SelectStatementProcessor,
     ChildNode.ModelClass.TableName, ChildNode.SQLTableAlias, ChildNode.JoinType);
   AddJoinCondition(SelectStatementProcessor,
     ParentNode.SQLTableAlias, ChildNode.SQLTableAlias,
     [ModelRelation.FieldName, ObjectIdField], ChildNode.JoinConditions);
end;

procedure TReferenceRelation.AddRelationIdField(SelectStatementProcessor:
  TSelectStatementProcessor; ParentNode, ChildNode: TQueryTreeNode);
begin
  SelectStatementProcessor.PutToken(ttTableAlias, ParentNode.SQLTableAlias);
  SelectStatementProcessor.PutToken(ttFieldName, ModelRelation.FieldName);
end;

{ TCompositionRelation }

procedure TCompositionRelation.JoinTables(
  SelectStatementProcessor: TSelectStatementProcessor;
  ParentNode: TQueryTreeNode; ChildNode: TQueryTreeRelationNode);
begin
  JoinTables(SelectStatementProcessor,
    ChildNode.ModelClass.TableName, ChildNode.SQLTableAlias, ChildNode.JoinType);
  AddJoinCondition(SelectStatementProcessor,
    ParentNode.SQLTableAlias, ChildNode.SQLTableAlias,
    [ObjectIdField, OwnerIdField, ModelRelation.Id, RelationIdField], ChildNode.JoinConditions);
end;

procedure TCompositionRelation.AddRelationIdField(SelectStatementProcessor:
  TSelectStatementProcessor; ParentNode, ChildNode: TQueryTreeNode);
begin
  SelectStatementProcessor.PutToken(ttTableAlias, ChildNode.SQLTableAlias);
  SelectStatementProcessor.PutToken(ttFieldName, ObjectIdField);
end;

{ TAggregationRelation }

procedure TAggregationRelation.JoinTables(
  SelectStatementProcessor: TSelectStatementProcessor;
  ParentNode: TQueryTreeNode; ChildNode: TQueryTreeRelationNode);
begin
  JoinTables(SelectStatementProcessor,
    ChildNode.ModelClass.TableName, ChildNode.SQLTableAlias, ChildNode.JoinType);
  AddJoinCondition(SelectStatementProcessor,
    ParentNode.SQLTableAlias, ChildNode.SQLTableAlias,
    [ObjectIdField, ModelRelation.FieldName], ChildNode.JoinConditions);
end;

procedure TAggregationRelation.AddRelationIdField(SelectStatementProcessor:
  TSelectStatementProcessor; ParentNode, ChildNode: TQueryTreeNode);
begin
  SelectStatementProcessor.PutToken(ttTableAlias, ChildNode.SQLTableAlias);
  SelectStatementProcessor.PutToken(ttFieldName, ObjectIdField);
end;

{ TAssociationRelation }

procedure TAssociationRelation.JoinTables(
  SelectStatementProcessor: TSelectStatementProcessor;
  ParentNode: TQueryTreeNode; ChildNode: TQueryTreeRelationNode);
begin
  JoinTables(SelectStatementProcessor,
    ModelRelation.TableName, RelationTableAlias, jtLeftOuter);
  AddJoinCondition(SelectStatementProcessor,
    ParentNode.SQLTableAlias, RelationTableAlias,
    [ObjectIdField, OwnerIdField, ModelRelation.Id, RelationIdField]);

  JoinTables(SelectStatementProcessor,
    ChildNode.ModelClass.TableName, ChildNode.SQLTableAlias, ChildNode.JoinType);
  AddJoinCondition(SelectStatementProcessor,
    RelationTableAlias, ChildNode.SQLTableAlias,
    [ObjectIdField, ObjectIdField], ChildNode.JoinConditions);
end;

procedure TAssociationRelation.AddRelationIdField(SelectStatementProcessor:
  TSelectStatementProcessor; ParentNode, ChildNode: TQueryTreeNode);
begin
  SelectStatementProcessor.PutToken(ttTableAlias, RelationTableAlias);
  SelectStatementProcessor.PutToken(ttFieldName, RelationIdField);
end;

function TAssociationRelation.GetRelationTableAlias: String;
begin
  if FRelationTableAlias = '' then FRelationTableAlias := GetNewTableAlias;
  Result := FRelationTableAlias;
end;

{ TGeneralizationRelation }

procedure TGeneralizationRelation.JoinTables(
  SelectStatementProcessor: TSelectStatementProcessor;
  ParentNode: TQueryTreeNode; ChildNode: TQueryTreeRelationNode);
begin
  JoinTables(SelectStatementProcessor,
    ChildNode.ModelClass.TableName, ChildNode.SQLTableAlias, ChildNode.JoinType);
  AddJoinCondition(SelectStatementProcessor,
    ParentNode.SQLTableAlias, ChildNode.SQLTableAlias, [ModelRelation.FieldName, ObjectIdField]);
end;

{$IFDEF DELPHI2006}
{$ENDREGION 'Relation implementation'}
{$ENDIF}

end.
