unit ovOQLTypes;

interface

uses SysUtils, Classes, Contnrs;

type
  TTokenType =
    //      OQL-    SQL-
    (ttComma, ttLBracket, ttRBracket,
    ttInteger, ttFloat, ttString, ttNull,
    ttIdent, ttParam, ttColumnFunc, ttFieldAlias, ttSelectAll, ttClassName, ttClassAlias,
    ttRelation, ttLogicalOperator, ttMathOperator, ttIsOperator, ttInSetOperator, ttStringOperator,
    ttDistinct, ttSortOrder, ttSubSelect, ttUnionAll, ttFetchExisting, ttFetchAll, ttFetchCondition,

    //       SQL-
    ttNone, ttTableName, ttTableAlias, ttFieldName, ttJoin, ttJoinCondition);

  TQueryStatementExpressionItem = class
  private
    FItemType: TTokenType;
    FItemText: String;
  public
    property ItemType: TTokenType read FItemType write FItemType;
    property ItemText: String read FItemText write FItemText;
  end;

  TQueryStatementExpression = class
  private
    FItems: TObjectList;
  protected
    function GetItemCount: Integer;
    function GetItems(Index: Integer): TQueryStatementExpressionItem;
  public
    constructor Create;
    destructor Destroy; override;
    function AddItem: TQueryStatementExpressionItem;
    procedure Clear;
    property ItemCount: Integer read GetItemCount;
    property Items[Index: Integer]: TQueryStatementExpressionItem read GetItems;
  end;

  TQueryStatement = class
  private
    FExpressions: TObjectList;
  protected
    function NewExpression: TQueryStatementExpression;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
  end;

  TSelectStatement = class(TQueryStatement)
  private
    FSelectExpression: TQueryStatementExpression;
    FFromExpression: TQueryStatementExpression;
    FWhereExpression: TQueryStatementExpression;
    FGroupByExpression: TQueryStatementExpression;
    FHavingExpression: TQueryStatementExpression;
    FOrderByExpression: TQueryStatementExpression;
    FUnionExpression: TQueryStatementExpression;
    FFetchExpression: TQueryStatementExpression;
  protected
  public
    constructor Create;
    property SelectExpression: TQueryStatementExpression read FSelectExpression;
    property FromExpression: TQueryStatementExpression read FFromExpression;
    property FetchExpression: TQueryStatementExpression read FFetchExpression;
    property WhereExpression: TQueryStatementExpression read FWhereExpression;
    property GroupByExpression: TQueryStatementExpression read FGroupByExpression;
    property HavingExpression: TQueryStatementExpression read FHavingExpression;
    property OrderByExpression: TQueryStatementExpression read FOrderByExpression;
    property UnionExpression: TQueryStatementExpression read FUnionExpression;
  end;

  TDeleteStatement = class(TQueryStatement)
  private
    FWhereExpression: TQueryStatementExpression;
    FDeleteExpression: TQueryStatementExpression;
  public
    constructor Create;
    property DeleteExpression: TQueryStatementExpression read FDeleteExpression;
    property WhereExpression: TQueryStatementExpression read FWhereExpression;
  end;

  TUpdateStatement = class(TQueryStatement)
  private
    FWhereExpression: TQueryStatementExpression;
    FUpdateExpression: TQueryStatementExpression;
  public
    constructor Create;
    property WhereExpression: TQueryStatementExpression read FWhereExpression;
    property UpdateExpression: TQueryStatementExpression read FUpdateExpression;
  end;

implementation

{ TQueryStatement }

constructor TQueryStatement.Create;
begin
  inherited Create;
  FExpressions := TObjectList.Create;
end;

destructor TQueryStatement.Destroy;
begin
  FreeAndNil(FExpressions);
  inherited Destroy;
end;

procedure TQueryStatement.Clear;
var
  I: Integer;
begin
  for I := 0 to FExpressions.Count - 1 do
    TQueryStatementExpression(FExpressions).Clear
end;

function TQueryStatement.NewExpression: TQueryStatementExpression;
begin
  Result := TQueryStatementExpression.Create;
  FExpressions.Add(Result);
end;

{ TQueryStatementExpression }

constructor TQueryStatementExpression.Create;
begin
  inherited Create;
  FItems := TObjectList.Create;
end;

destructor TQueryStatementExpression.Destroy;
begin
  FreeAndNil(FItems);
  inherited Destroy;
end;

function TQueryStatementExpression.GetItemCount: Integer;
begin
  Result := FItems.Count;
end;

function TQueryStatementExpression.GetItems(Index: Integer):
  TQueryStatementExpressionItem;
begin
  Result := TQueryStatementExpressionItem(FItems[Index]);
end;

function TQueryStatementExpression.AddItem: TQueryStatementExpressionItem;
begin
  Result := TQueryStatementExpressionItem.Create;
  FItems.Add(Result);
end;

procedure TQueryStatementExpression.Clear;
begin
  FItems.Clear;
end;

{ TSelectStatement }

constructor TSelectStatement.Create;
begin
  inherited Create;
  FSelectExpression := NewExpression;
  FFromExpression := NewExpression;
  FFetchExpression := NewExpression;
  FWhereExpression := NewExpression;
  FGroupByExpression := NewExpression;
  FHavingExpression := NewExpression;
  FOrderByExpression := NewExpression;
  FUnionExpression := NewExpression;
end;

{ TDeleteStatement }

constructor TDeleteStatement.Create;
begin
  inherited Create;
  FDeleteExpression := NewExpression;
  FWhereExpression := NewExpression;
end;

{ TUpdateStatement }

constructor TUpdateStatement.Create;
begin
  inherited Create;
  FUpdateExpression := NewExpression;
  FWhereExpression := NewExpression;
end;

end.
