unit ovObjectDeleteError;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls, ImgList, Menus, ActnList,
  ovTypes, ovModel, ovObjectInspector, ovIObjects, ovComponents;

type
  TfmObjectDeleteError = class(TForm)
    PageControl: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    Label1: TLabel;
    Image2: TImage;
    CancelBtn: TBitBtn;
    DetailsBtn: TBitBtn;
    DeletedObjectInspector: TovObjectInspector;
    ExplorerImages: TImageList;
    ReferencingObjectsInspector: TovObjectInspector;
    PopupMenu: TPopupMenu;
    DeleteObjectItem: TMenuItem;
    ActionList: TActionList;
    DeleteObjectAction: TAction;
    DeleteAllObjectsAction: TAction;
    N1: TMenuItem;
    DeleteAllBtn: TBitBtn;
    Bevel: TBevel;
    Bevel1: TBevel;
    procedure DetailsBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DeleteObjectItemClick(Sender: TObject);
    procedure DeleteObjectActionExecute(Sender: TObject);
    procedure DeleteAllObjectsActionExecute(Sender: TObject);
    procedure DeleteAllBtnClick(Sender: TObject);
  private
    FObjectToDelete: IovObject;
    function GetShowDetails: Boolean;
    procedure SetShowDetails(Value: Boolean);
    { Private declarations }
  public
    property ShowDetails: Boolean read GetShowDetails write SetShowDetails;
    { Public declarations }
  end;

procedure ShowDeleteObjectErrorHandler(AModel: TovModel; AModelObject:
  TModelObject; AObjectId: TOID);

implementation

uses ovDatabase, ovObjects, ovObjectDeleteDlg;

{$R *.dfm}

//  TovModel    ObjectSpace
procedure ShowDeleteObjectErrorHandler(AModel: TovModel; AModelObject:
  TModelObject; AObjectId: TOID);
begin
  with TfmObjectDeleteError.Create(Application) do
  try
    //   
    FObjectToDelete := AModel.ObjectSpace.GetObject(AModelObject.FullName, AObjectId);
    DeletedObjectInspector.ShowObject(FObjectToDelete);

    //   
    ReferencingObjectsInspector.ShowObjects(FObjectToDelete.GetReferencingObjects);

    ShowModal;
  finally
    Free
  end;
end;

{ TfmObjectDeleteError }

procedure TfmObjectDeleteError.FormCreate(Sender: TObject);
begin
  ShowDetails := False;
end;

procedure TfmObjectDeleteError.DetailsBtnClick(Sender: TObject);
begin
  ShowDetails := Not ShowDetails;
end;

function TfmObjectDeleteError.GetShowDetails: Boolean;
begin
  Result := PageControl.Visible;
end;

procedure TfmObjectDeleteError.SetShowDetails(Value: Boolean);
begin
  LockWindowUpdate(Handle);
  try
    if Value then
    begin
      Constraints.MaxHeight := 0;
      PageControl.Visible := True;
      PageControl.Height := 200;
    end
    else begin
      PageControl.Visible := False;
    end;
  finally
    LockWindowUpdate(0)
  end;
end;

procedure TfmObjectDeleteError.DeleteObjectItemClick(Sender: TObject);
begin
  if Application.MessageBox('  ?', '', MB_ICONWARNING or MB_YESNO)  = IDYES
  then begin
    ReferencingObjectsInspector.SelectedObject.Delete;
    ReferencingObjectsInspector.ShowObjects(FObjectToDelete.GetReferencingObjects);
  end
end;

procedure TfmObjectDeleteError.DeleteObjectActionExecute(Sender: TObject);
begin
  if Application.MessageBox('   ?', '',
    MB_ICONWARNING or MB_YESNO)  = IDYES
  then begin
    ReferencingObjectsInspector.SelectedObject.Delete;
    ReferencingObjectsInspector.ShowObjects(FObjectToDelete.GetReferencingObjects);
  end
end;

procedure TfmObjectDeleteError.DeleteAllObjectsActionExecute(
  Sender: TObject);
var
  I: Integer;
begin
  if Application.MessageBox('    ?', '',
    MB_ICONWARNING or MB_YESNO)  = IDYES
  then begin
    try
      with FObjectToDelete.GetReferencingObjects do
        for I := 0 to Count - 1 do Objects[I].Delete;
    finally
      ReferencingObjectsInspector.ShowObjects(FObjectToDelete.GetReferencingObjects);
    end;
  end
end;

procedure TfmObjectDeleteError.DeleteAllBtnClick(Sender: TObject);
begin
  if ShowDeleteObjectDialog(DeletedObjectInspector.SelectedObject) then Close
end;

end.
