unit ovObjectQuery;

interface

uses SysUtils, ovSQLQuery;

type
  TJoinType = (jtLeftOuter, jtRightOuter, jtInner);

  TJoinExpressionItem = class(TQueryExpressionItem)
  private
    FJoinType: TJoinType;
    FRelationPath: String;
    FAdditionalFilter: String;
  protected
    function GetText: String; override;
    procedure SetText(const Value: String); override;
  public
    property JoinType: TJoinType read FJoinType write FJoinType;
    property RelationPath: String read FRelationPath write FRelationPath;
    property AdditionalFilter: String read FAdditionalFilter write
      FAdditionalFilter;
  end;

  TJoinExpression = class(TQueryExpression)
  private
    function GetItems(Index: Integer): TJoinExpressionItem;
    {$IFDEF Delphi2006} inline; {$ENDIF}
  protected
    function CreateItem(ItemText: String): TQueryExpressionItem; override;
  public
    {{
       . APossibleKeywords   
     ,      . 
          Keyword.  
        ,  .
    }
    constructor Create(AQuery: TAbstractQuery);
    property Items[Index: Integer]: TJoinExpressionItem read GetItems;
  end;

  TObjectSelectQuery = class(TSelectQuery)
  private
    FJoin: TJoinExpression;
  public
    constructor Create;
    function ContainsAggregateFunctions: Boolean;
    property Join: TJoinExpression read FJoin;
  end;

implementation

function JoinTypeToStr(JoinType: TJoinType): String;
const
  JoinTypeNames: array [TJoinType] of String = ('LEFT OUTER', 'RIGHT OUTER', 'INNER');
begin
  Result := JoinTypeNames[JoinType]
end;

function MakeAdditionalFilter(const Filter: String): String;
{$IFDEF Delphi2006} inline; {$ENDIF}
begin
  if Filter <> '' then
    Result := Format(' ON (%s)', [Filter])
  else
    Result := ''
end;

{{
       Text   
 .
}
function ExtractAdditionalFilter(var Text: String): String;
var
  P: Integer;
begin
  P := Pos('ON (', Text);
  if P > 0 then
  begin
    Result := Trim(Copy(Text, P+4, Length(Text)));
    Text := Trim(Copy(Text, 1, P-1));
    P := Length(Result);
    while (P > 0) and (Result[P] <> ')') do Dec(P);
    Result := Copy(Result, 1, P-1);
  end else
    Result := ''
end;

{
****************************** TObjectSelectQuery ******************************
}

constructor TObjectSelectQuery.Create;
begin
  inherited Create;
  FJoin := TJoinExpression.Create(Self);
end;

function TObjectSelectQuery.ContainsAggregateFunctions: Boolean;
var
  SelectStr: String;
begin
  SelectStr := UpperCase(Select.Text);
  Result :=
    Select.Distinct or (Pos('AVG', SelectStr) > 0) or
    (Pos('COUNT', SelectStr) > 0) or (Pos('SUM', SelectStr) > 0) or
    (Pos('MIN', SelectStr) > 0) or (Pos('MAX', SelectStr) > 0);
end;

{
******************************** TJoinExpression *******************************
}

function TJoinExpression.CreateItem(ItemText: String): TQueryExpressionItem;
begin
  Result := TJoinExpressionItem.Create(ItemText)
end;

function TJoinExpression.GetItems(Index: Integer): TJoinExpressionItem;
begin
  Result := TJoinExpressionItem(inherited Items[Index])
end;

constructor TJoinExpression.Create(AQuery: TAbstractQuery);
begin
  inherited Create(AQuery, 'JOIN', ',', (AQuery as TObjectSelectQuery).Where);
end;

{
****************************** TJoinExpressionItem *****************************
}

function TJoinExpressionItem.GetText: String;
begin
  Result := JoinTypeToStr(JoinType) + ' ' + RelationPath +
    MakeAdditionalFilter(AdditionalFilter);
end;

procedure TJoinExpressionItem.SetText(const Value: String);
var
  S: String;
  KeywordPresent: Boolean;
begin
  //   
  S := SkipKeyword(Value, 'LEFT OUTER', KeywordPresent);
  if KeywordPresent then JoinType := jtLeftOuter else
  begin
    S := SkipKeyword(Value, 'RIGHT OUTER', KeywordPresent);
    if KeywordPresent then JoinType := jtRightOuter else
    begin
      S := SkipKeyword(Value, 'INNER', KeywordPresent);
      if KeywordPresent then JoinType := jtInner else
      begin
        JoinType := jtLeftOuter;
        S := Value;
      end;
    end;
  end;
  AdditionalFilter := ExtractAdditionalFilter(S);
  RelationPath := Trim(S);
end;

end.
