unit ovObjectStorage;

interface

uses SysUtils, Classes, ovTypes, ovIObjects, ovModel, ovComponents, Contnrs;

type
  TObjectHandle = Pointer;

  IovObjectWriter = interface
    {{
          . 
           WriteAttribute 
    StartWriteRelation.
    }
    function StartWriteObject(AObjectClassName: String; AObjectId: TOID):
      TObjectHandle;
    {{
          .
    }
    procedure WriteAttribute(Handle: TObjectHandle; Attribute: TModelAttribute;
      Value: Variant);

    {{
         , 
     Relation.       
     WriteReference  WriteChildObject.
    }
    function StartWriteRelation(Handle: TObjectHandle; Relation: TModelRelation):
      TObjectHandle;
    {{
           .
    }
    procedure WriteReference(Handle: TObjectHandle; AObjectClassName: String;
      AObjectId: TOID; var AddToRefObjects: Boolean);
    {{
          . 
           WriteAttribute 
    StartWriteRelation.
    }
    function StartWriteChildObject(Handle: TObjectHandle; AObjectClassName: String;
      AObjectId: TOID): TObjectHandle;
  end;

  IovObjectReader = interface
    {{
         .
    }
    function GetObjectCount: Integer;
    {{
             
        ReadAttribute  ReadRelation. 
          0  GetObjectCount-1.
    }
    function StartReadObject(Index: Integer): TObjectHandle;
    procedure GetObject(Handle: TObjectHandle; var AObjectClassName: String; var
      AObjectId: TOID);
    {{
          .  
       ,     Null.
    }
    function ReadAttribute(Handle: TObjectHandle; Attribute: TModelAttribute):
      Variant;
    {{
         ,
      Relation,   .
    }
    function GetRelationCount(Handle: TObjectHandle; Relation: TModelRelation): Integer;
    {{
        ,   Relation.
    }
    function ReadRelation(Handle: TObjectHandle; Relation: TModelRelation; Index:
      Integer): TObjectHandle;
    function ReadChildObject(Handle: TObjectHandle; Relation: TModelRelation;
      Index: Integer): TObjectHandle;
  end;

  {{
      .    .
  }
  TWriteObjectEvent = procedure (Sender: TObject; AObject: IovObject;
    var Skip: Boolean) of Object;
  {{
        .
     .
  }
  TWriteAttributeEvent = procedure(Sender: TObject; AObject: IovObject;
    Attribute: TModelAttribute; var Value: Variant; var Skip: Boolean) of object;
  {{
        .
     .
  }
  TWriteRelationEvent = procedure(Sender: TObject; AObject: IovObject; Relation:
    TModelRelation; var Skip: Boolean) of object;
  {{
           .
     .
  }
  TReadRelationEvent = procedure(Sender: TObject; AModelObject: TModelObject;
    Relation: TModelRelation; var Skip: Boolean) of object;

  {{
       .
  }
  TReadObjectEvent = procedure(Sender: TObject; Handle: TObjectHandle;
    var Skip: Boolean) of object;
  {{
       .
  }
  TFindObjectEvent = procedure(Sender: TObject; Handle: TObjectHandle;
    var AObject: IovObject) of object;

  {{
         
       .
  }
  TMatchObjectEvent = procedure(Sender: TObject; Handle: TObjectHandle;
    AObject: IovObject) of object;

  TShowProgressEvent = procedure(Sender: TObject; MaxIndex: Integer) of object;
  TUpdateProgressEvent = procedure(Sender: TObject; CurrIndex: Integer; var
    Cancel: Boolean) of object;
  THideProgressEvent = procedure(Sender: TObject) of object;

  TOperationState = (osWritingObjects, osWritingReferencedObjects, osReadingObjects, osReadingReferences);

  TovObjectStorage = class(TovModelClient)
  private
    FObjectWriter: IovObjectWriter;
    FOnWriteAttribute: TWriteAttributeEvent;
    FOnWriteObject: TWriteObjectEvent;
    FOnWriteRelation: TWriteRelationEvent;
    FReferencedObjects: IovObjectList;
    FWrittenObjects: IovObjectList;
    FObjectReader: IovObjectReader;
    FMatchedObjects: TObjectList;
    FOnFindObject: TFindObjectEvent;
    FOnMatchObject: TMatchObjectEvent;
    FOnUpdateProgress: TUpdateProgressEvent;
    FOnShowProgress: TShowProgressEvent;
    FOperationState: TOperationState;
    FOnHideProgress: THideProgressEvent;
    FOnReadObject: TReadObjectEvent;
    FOnReadRelation: TReadRelationEvent;
    procedure CheckObjectWriter;
    procedure WriteObject(AObject: IovObject; Handle: TObjectHandle = nil);
    procedure WriteObjectAttributes(AObject: IovObject; Handle: TObjectHandle);
    procedure WriteObjectRelations(AObject: IovObject; Handle: TObjectHandle);
    function DoOnWriteObject(AObject: IovObject): Boolean;
    function DoOnWriteAttribute(AObject: IovObject; Attribute: TModelAttribute; var
      Value: Variant): Boolean;
    function DoOnWriteRelation(AObject: IovObject; Relation: TModelRelation):
      Boolean;
    procedure CheckObjectReader;
    function ReadObject(Index: Integer): IovObject;
    function ReadNewObject(Handle: TObjectHandle): IovObject;
    function UpdateObject(Handle: TObjectHandle): IovObject;
    procedure ReadObjectAttributes(AObject: IovObject; Handle: TObjectHandle);
    procedure ReadChildObjects(AObject: IovObject; Handle: TObjectHandle;
      UpdateMode: Boolean);
    procedure ReadObjectRelations(AObject: IovObject; Handle: TObjectHandle;
      UpdateMode: Boolean);
    procedure ShowProgress(MaxIndex: Integer);
    procedure UpdateProgress(CurrIndex: Integer);
    procedure HideProgress;
    function DoOnReadRelation(AModelObject: TModelObject; Relation:
      TModelRelation): Boolean;
  protected
    function FindObject(Handle: TObjectHandle): IovObject; virtual;
    procedure MatchObjects(AObject: IovObject; Handle: TObjectHandle);
    procedure DoMatchObjects(AObjectClassName: String; ExternalObjectId,
      InternalObjectId: TOID); virtual;
    function DoFindObject(AObjectClassName: String; AObjectId: TOID): IovObject;
      virtual;
    procedure Initialize(ReadMode: Boolean); dynamic;
    {{
       
    }
    property WrittenObjects: IovObjectList read FWrittenObjects;
    {{
     ,    ,      
    
    }
    property ReferencedObjects: IovObjectList read FReferencedObjects;

    {      }

    property OnUpdateProgress: TUpdateProgressEvent read FOnUpdateProgress write
      FOnUpdateProgress;
    property OnShowProgress: TShowProgressEvent read FOnShowProgress write
      FOnShowProgress;
    property OnHideProgress: THideProgressEvent read FOnHideProgress write
      FOnHideProgress;

    {    }

    property OnWriteObject: TWriteObjectEvent read FOnWriteObject write
      FOnWriteObject;
    property OnWriteAttribute: TWriteAttributeEvent read FOnWriteAttribute write
      FOnWriteAttribute;
    property OnWriteRelation: TWriteRelationEvent read FOnWriteRelation write
      FOnWriteRelation;

    {    }

    property OnReadObject: TReadObjectEvent read FOnReadObject write FOnReadObject;
    property OnReadRelation: TReadRelationEvent read FOnReadRelation write
      FOnReadRelation;
    property OnFindObject: TFindObjectEvent read FOnFindObject write FOnFindObject;
    {{
           
         .
    }
    property OnMatchObject: TMatchObjectEvent read FOnMatchObject write
      FOnMatchObject;

  public
    constructor Create(AOwner: TComponent); override;
    procedure WriteObjectList(AObjectList: IovObjectList);
    procedure ReadObjectList(AObjectList: IovObjectList);
    destructor Destroy; override;
    property ObjectReader: IovObjectReader read FObjectReader write FObjectReader;
    property ObjectWriter: IovObjectWriter read FObjectWriter write FObjectWriter;
    property OperationState: TOperationState read FOperationState;
  end;

  EObjectReaderIsNil = class (Exception);
  EObjectWriterIsNil = class (Exception);
  EOperationCanceled = class (Exception);

implementation

uses Variants, ovObjects, ovProgress;

type
  TMatchedObject = class
  private
    FObjectClassName: String;
    FExternalObjectId: TOID;
    FInternalObjectId: TOID;
  public
    property ObjectClassName: String read FObjectClassName;
    property ExternalObjectId: TOID read FExternalObjectId;
    property InternalObjectId: TOID read FInternalObjectId;
  end;

  TMatchedObjects = class (TObjectList)
  private
    procedure AddObject(AObjectClassName: String; ExternalObjectId,
      InternalObjectId: TOID);
    function FindInternalObject(AObjectClassName: String; AExternalObjectId: TOID):
      TOID;
  end;

{ TMatchedObjects }

procedure TMatchedObjects.AddObject(AObjectClassName: String; ExternalObjectId,
  InternalObjectId: TOID);
var
  AMatchedObject: TMatchedObject;
begin
  AMatchedObject := TMatchedObject.Create;
  AMatchedObject.FObjectClassName := AObjectClassName;
  AMatchedObject.FInternalObjectId := InternalObjectId;
  AMatchedObject.FExternalObjectId := ExternalObjectId;
  inherited Add(AMatchedObject);
end;

function TMatchedObjects.FindInternalObject(AObjectClassName: String;
  AExternalObjectId: TOID): TOID;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    with TMatchedObject(Items[I]) do
      if (ObjectClassName = AObjectClassName) and (ExternalObjectId = AExternalObjectId) then
      begin
        Result := InternalObjectId;
        Exit;
      end;
  Result := 0;
end;

{ TovObjectStorage }

constructor TovObjectStorage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FReferencedObjects := NewObjectList;
  FWrittenObjects := NewObjectList;
  FMatchedObjects := TMatchedObjects.Create;
end;

destructor TovObjectStorage.Destroy;
begin
  FreeAndNil(FMatchedObjects);
  inherited Destroy;
end;

procedure TovObjectStorage.Initialize(ReadMode: Boolean);
begin
  if ReadMode then
  begin
    CheckObjectReader;
    FMatchedObjects.Clear;
  end
  else
  begin
    CheckObjectWriter;
    ReferencedObjects.Clear;
  end;
  ModelNeeded;
end;

procedure TovObjectStorage.ShowProgress(MaxIndex: Integer);
const
  ProgressTitle: array [TOperationState] of String =
    (' ..', '  ..', ' ..', ' ..');
begin
  if Assigned(FOnShowProgress) then
    FOnShowProgress(Self, MaxIndex)
  else
    ovProgress.ShowProgress(MaxIndex, ProgressTitle[OperationState]);
end;

procedure TovObjectStorage.UpdateProgress(CurrIndex: Integer);
var
  Cancel: Boolean;
begin
  Cancel := False;

  if Assigned(FOnUpdateProgress) then FOnUpdateProgress(Self, CurrIndex, Cancel)
  else Cancel := Not ovProgress.SetProgress(CurrIndex);

  if Cancel then raise EOperationCanceled.Create('  ');
end;

procedure TovObjectStorage.HideProgress;
begin
  if Assigned(FOnHideProgress) then FOnHideProgress(Self)
  else ovProgress.HideProgress;
end;

{***************************************************************************** }
{                                                            }
{ **************************************************************************** }

procedure TovObjectStorage.WriteObjectList(AObjectList: IovObjectList);
var
  I: Integer;
  ACount: Integer;
begin
  Initialize(False);

  {      }
  FOperationState := osWritingObjects;
  with AObjectList do
  begin
    ACount := Count;
    ShowProgress(ACount);
    try
      for I := 0 to ACount - 1 do
      begin
        WriteObject(Objects[I]);
        UpdateProgress(I);
      end;
    finally
      HideProgress;
    end;
  end;

  {   ,      ,
      ,     ,     
        }
  FOperationState := osWritingReferencedObjects;
  while ReferencedObjects.Count > 0 do
  begin
    WriteObject(ReferencedObjects[0]);
    ReferencedObjects.Delete(0);
  end
end;

procedure TovObjectStorage.WriteObject(AObject: IovObject; Handle:
  TObjectHandle = nil);
begin
  if (WrittenObjects.IndexOfObject(AObject) < 0) and DoOnWriteObject(AObject) then
  begin
    if Not Assigned(Handle) then
      Handle := ObjectWriter.StartWriteObject(AObject.ClassName, AObject.ObjectId)
    else
      Handle := ObjectWriter.StartWriteChildObject(Handle, AObject.ClassName, AObject.ObjectId);
    WriteObjectAttributes(AObject, Handle);
    WriteObjectRelations(AObject, Handle);
    WrittenObjects.Add(AObject);
  end;
end;

procedure TovObjectStorage.CheckObjectWriter;
begin
  if Not Assigned(ObjectWriter) then
    raise EObjectWriterIsNil.Create('   ObjectWriter  ' + Name);
end;

function TovObjectStorage.DoOnWriteObject(AObject: IovObject): Boolean;
var
  Skip: Boolean;
begin
  Skip := False;
  if Assigned(FOnWriteObject) then FOnWriteObject(Self, AObject, Skip);
  Result := Not Skip;
end;

procedure TovObjectStorage.WriteObjectAttributes(AObject: IovObject; Handle:
  TObjectHandle);
var
  AModelObject: TModelObject;
  I: Integer;
  Attribute: TModelAttribute;
  Value: Variant;
begin
  AModelObject := AObject.ModelObject;
  for I := 0 to AModelObject.AttributeCount - 1 do
  begin
    Attribute := AModelObject.Attributes[I];
    if DoOnWriteAttribute(AObject, Attribute, Value) then
      ObjectWriter.WriteAttribute(Handle, Attribute, Value);
  end
end;

function TovObjectStorage.DoOnWriteAttribute(AObject: IovObject; Attribute:
  TModelAttribute; var Value: Variant): Boolean;
var
  Skip: Boolean;
begin
  Skip := Attribute.Calculated or Attribute.IsBlob; {           }
  if Not Attribute.IsBlob then
    Value := AObject[Attribute.Name].Value
  else
    Value := Null;
  if Assigned(FOnWriteAttribute) then FOnWriteAttribute(Self, AObject, Attribute, Value, Skip);
  Result := Not Skip;
end;

procedure TovObjectStorage.WriteObjectRelations(AObject: IovObject; Handle:
  TObjectHandle);
var
  AModelObject: TModelObject;
  I, J: Integer;
  Relation: TModelRelation;
  RelationHandle: TObjectHandle;
  AddToRefObjects: Boolean;
begin
  AModelObject := AObject.ModelObject;
  for I := 0 to AModelObject.RelationCount - 1 do
  begin
    Relation := AModelObject.Relations[I];
    if DoOnWriteRelation(AObject, Relation) then
    begin
      RelationHandle := ObjectWriter.StartWriteRelation(Handle, Relation);
      with AObject[Relation.Name].AsObjects do
        for J := 0 to Count - 1 do
        begin
          if Not Relation.IsComposition then
          begin
            AddToRefObjects := True; {    , 
                  
            ReferencedObjects   ,    
                     
               ,      .
            ..        
            ,   (..  )
               . }
            ObjectWriter.WriteReference(
              RelationHandle, Objects[J].ClassName, Objects[J].ObjectId, AddToRefObjects);
            if AddToRefObjects then ReferencedObjects.Add(Objects[J])
          end
          else
            WriteObject(Objects[J], RelationHandle);
        end;
    end;
  end
end;

function TovObjectStorage.DoOnWriteRelation(AObject: IovObject; Relation:
  TModelRelation): Boolean;
var
  Skip: Boolean;
begin
  Skip := Relation.Derived;
  if Assigned(FOnWriteRelation) then FOnWriteRelation(Self, AObject, Relation, Skip);
  Result := Not Skip;
end;

{***************************************************************************** }
{                                                                }
{ **************************************************************************** }

procedure TovObjectStorage.CheckObjectReader;
begin
  if Not Assigned(ObjectReader) then
    raise EObjectReaderIsNil.Create('   ObjectReader  ' + Name);
end;

{{
    ,   ,   
  (..   ,  ).
}
procedure TovObjectStorage.ReadObjectList(AObjectList: IovObjectList);
var
  I: Integer;
  ACount: Integer;
  AObject: IovObject;
begin
  Initialize(True);

  with Model.ObjectSpace do
  begin
    StartTransaction;
    try

      with ObjectReader do
      begin
        ACount := GetObjectCount;

        //     
        FOperationState := osReadingObjects;
        ShowProgress(ACount);
        try
          for I := 0 to ACount - 1 do
          begin
            AObject := ReadObject(I);
            if AObject <> nil then AObjectList.Add(AObject);
            UpdateProgress(I);
          end;
        finally
          HideProgress;
        end;

        //     
        FOperationState := osReadingReferences;
        ShowProgress(ACount);
        try
          for I := 0 to ACount - 1 do
          begin
            ReadObject(I);
            UpdateProgress(I);
          end;
        finally
          HideProgress
        end;

        { TODO :         ,     }

        //      -   
        with AObjectList do
          for I := 0 to Count - 1 do
            Objects[I].UpdateCalculatedAttributes;
      end;

      CommitTransaction;
    except
      RollbackTransaction;
      raise;
    end;
  end
end;

{{
    ,      .
  ReadRelations  False,      
  ,      .
}
function TovObjectStorage.ReadObject(Index: Integer): IovObject;
var
  ObjectHandle: TObjectHandle;
  SkipObject: Boolean;
begin
  ObjectHandle := ObjectReader.StartReadObject(Index);

  SkipObject := False;
  if Assigned(FOnReadObject) then FOnReadObject(Self, ObjectHandle, SkipObject);
  if SkipObject then Result := nil
  else
    if Not Assigned(FindObject(ObjectHandle)) then
      Result := ReadNewObject(ObjectHandle)   {    }
    else
      Result := UpdateObject(ObjectHandle);   {    }
end;

{{
   (ReadRelations  False)  (   )
      .    (ReadRelations
 True)          .
}
function TovObjectStorage.ReadNewObject(Handle: TObjectHandle): IovObject;
var
  AObjectClassName: String;
  AObjectId: TOID;
  AObject: IovObject;
begin
  if OperationState = osReadingObjects then
  begin
    ObjectReader.GetObject(Handle, AObjectClassName, AObjectId);
    AObject := ObjectSpace.CreateObject(AObjectClassName);
    MatchObjects(AObject, Handle);     {     }
    ReadObjectAttributes(AObject, Handle);
    ReadChildObjects(AObject, Handle, False);
  end else { OperationState = osReadingReferences }
  begin
    AObject := FindObject(Handle);        {         }
    ReadObjectRelations(AObject, Handle, False);
  end;
  Result := AObject;
end;

{{
   (ReadRelations  False)   
 .   ,    
     ().
   (ReadRelations  True)    
     .
}
function TovObjectStorage.UpdateObject(Handle: TObjectHandle): IovObject;
var
  AObject: IovObject;
begin
  if OperationState = osReadingObjects then
  begin
    AObject := FindObject(Handle);
    MatchObjects(AObject, Handle);     {     }
    ReadObjectAttributes(AObject, Handle);
    ReadChildObjects(AObject, Handle, True);
  end else { OperationState = osReadingReferences }
  begin
    AObject := FindObject(Handle);
    ReadObjectRelations(AObject, Handle, True);
  end;
  Result := AObject;
end;

procedure TovObjectStorage.ReadObjectAttributes(AObject: IovObject; Handle:
  TObjectHandle);
var
  AModelObject: TModelObject;
  Attribute: TModelAttribute;
  I: Integer;
begin
  AModelObject := AObject.ModelObject;
  AObject.BeginUpdate;
  try
    for I := 0 to AModelObject.AttributeCount - 1 do
    begin
      Attribute := AModelObject.Attributes[I];
      AObject[Attribute.Name].Value := ObjectReader.ReadAttribute(Handle, Attribute);
    end
  finally
    AObject.EndUpdate;
  end;
end;

{{
           
.
}
procedure TovObjectStorage.ReadChildObjects(AObject: IovObject; Handle:
  TObjectHandle; UpdateMode: Boolean);
var
  AModelObject: TModelObject;
  ARelation: TModelRelation;
  ChildObject: IovObject;
  I, J: Integer;
  ChildObjectHandle: TObjectHandle;

  {{
    True,      ,    
    (     ).
  }
  function IsRedundantObject(AObject: IovObject): Boolean;
  var
    I: Integer;
    ObjectHandle: TObjectHandle;
    AObject2: IovObject;
  begin
    for I := 0 to ObjectReader.GetRelationCount(Handle, ARelation) - 1 do
    begin
      ObjectHandle := ObjectReader.ReadChildObject(Handle, ARelation, I);
      AObject2 := FindObject(ObjectHandle);
      if (AObject2 <> nil) and AObject2.IsSame(AObject) then
      begin
        Result := False;
        Exit;
      end;
    end;
    Result := True;
  end;

begin
  AModelObject := AObject.ModelObject;
  for I := 0 to AModelObject.RelationCount - 1 do
  begin
    ARelation := AModelObject.Relations[I];
    if ARelation.IsComposition then
    begin
      if UpdateMode then
      begin
        {       , :
               AObject   ,
                }

          (*         
                  
        with AObject[ARelation.Name].AsObjects do
          for J := Count - 1 downto 0 do
            if IsRedundantObject(Objects[J]) then Delete(J);
            *)
      end;

      for J := 0 to ObjectReader.GetRelationCount(Handle, ARelation) - 1 do
      begin
        ChildObjectHandle := ObjectReader.ReadChildObject(Handle, ARelation, J);
        if Not UpdateMode then
        begin
          if J >= ARelation.TargetMinCount then
            ChildObject := AObject[ARelation.Name].Add
          else
            {  ,     
               ,      0 }
            ChildObject := AObject[ARelation.Name].Objects[J];
          MatchObjects(ChildObject, ChildObjectHandle);
          ReadObjectAttributes(ChildObject, ChildObjectHandle);
          ReadChildObjects(ChildObject, ChildObjectHandle, UpdateMode);
        end else
        begin
          {        ,
               }
          ChildObject := FindObject(ChildObjectHandle);
          if ChildObject = nil then
          begin
            {    -   }
            ChildObject := AObject[ARelation.Name].Add;
            MatchObjects(ChildObject, ChildObjectHandle);
          end; {    -   }

          ReadObjectAttributes(ChildObject, ChildObjectHandle);
          ReadChildObjects(ChildObject, ChildObjectHandle, UpdateMode);
        end;
      end
    end;
  end
end;

{{
          .
,  ,     ( )  
  .
}
procedure TovObjectStorage.ReadObjectRelations(AObject: IovObject; Handle:
  TObjectHandle; UpdateMode: Boolean);
var
  AModelObject: TModelObject;
  ARelation: TModelRelation;
  I, J: Integer;
  ReferencedObject: IovObject;
  ChildObjectHandle: TObjectHandle;
  ChildObject: IovObject;
begin
  if Not AObject.IsExist then
    raise Exception.CreateFmt('    (%s,%d)', [AObject.ClassName, AObject.ObjectId]);

  AModelObject := AObject.ModelObject;
  for I := 0 to AModelObject.RelationCount - 1 do
  begin
    ARelation := AModelObject.Relations[I];
    if DoOnReadRelation(AModelObject, ARelation) then
    begin
      if Not ARelation.IsComposition then
      begin
        if UpdateMode then AObject[ARelation.Name].Clear;
        for J := 0 to ObjectReader.GetRelationCount(Handle, ARelation) - 1 do
        begin
          ReferencedObject := FindObject(ObjectReader.ReadRelation(Handle, ARelation, J));
          if ReferencedObject <> nil then
            AObject[ARelation.Name].Add(ReferencedObject);
        end
      end
      else
        for J := 0 to ObjectReader.GetRelationCount(Handle, ARelation) - 1 do
        begin
          ChildObjectHandle := ObjectReader.ReadChildObject(Handle, ARelation, J);
          ChildObject := FindObject(ChildObjectHandle);
          if ChildObject <> nil then
            ReadObjectRelations(ChildObject, ChildObjectHandle, UpdateMode);
        end;
    end;
  end
end;

function TovObjectStorage.DoOnReadRelation(AModelObject: TModelObject;
  Relation: TModelRelation): Boolean;
var
  Skip: Boolean;
begin
  Skip := Relation.Derived;
  if Assigned(FOnReadRelation) then FOnReadRelation(Self, AModelObject, Relation, Skip);
  Result := Not Skip;
end;

{***************************************************************************** }
{                                                       }
{ **************************************************************************** }

{{
  ,      
 .    ,    nil.
}
function TovObjectStorage.FindObject(Handle: TObjectHandle): IovObject;
var
  AObjectClassName: String;
  AObjectId: TOID;
begin
  Result := nil;
  if Assigned(FOnFindObject) then FOnFindObject(Self, Handle, Result);
  if Result = nil then
  begin
    ObjectReader.GetObject(Handle, AObjectClassName, AObjectId);
    Result := DoFindObject(AObjectClassName, AObjectId);
  end;
end;

{{
 .      .

     ,    
 .      
      ,  .
}
function TovObjectStorage.DoFindObject(AObjectClassName: String; AObjectId:
  TOID): IovObject;
var
  InternalObjectId: TOID;
begin
  InternalObjectId := TMatchedObjects(FMatchedObjects).FindInternalObject(AObjectClassName, AObjectId);
  if InternalObjectId <> 0 then
    Result := ObjectSpace.GetObject(AObjectClassName, InternalObjectId)
  else
    Result := nil;
end;

{{
        AObject  
     Handle.
}
procedure TovObjectStorage.MatchObjects(AObject: IovObject; Handle:
  TObjectHandle);
var
  AObjectClassName: String;
  AObjectId: TOID;
begin
  if Assigned(FOnMatchObject) then FOnMatchObject(Self, Handle, AObject);
  ObjectReader.GetObject(Handle, AObjectClassName, AObjectId);
  DoMatchObjects(AObjectClassName, AObjectId, AObject.ObjectId);
end;

{{
 .      .
}
procedure TovObjectStorage.DoMatchObjects(AObjectClassName: String;
  ExternalObjectId, InternalObjectId: TOID);
begin
  TMatchedObjects(FMatchedObjects).AddObject(AObjectClassName, ExternalObjectId,
    InternalObjectId);
end;

end.
