unit ovObjects;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, ovTypes, ovModel, ovIObjects, ovDatabase, ovQueryInfo;

type
  TovClassObjects = class;
  TovInterfacedObject = class(TInterfacedObject)
  private
    FDatabase: TovDatabase;
  public
    constructor Create(ADatabase: TovDatabase);
    property Database: TovDatabase read FDatabase;
  end;
  
  TovClass = class(TovInterfacedObject, IovClass)
    procedure DeleteAllObjects;
    procedure DeleteObject(ObjectId: TOID);
  private
    FName: String;
    FObjects: TovClassObjects;
    function GetObject(ObjectId: TOID): IovObject;
    function GetRealObject(ObjectId: TOID): IovObject;
    function CreateInstance(ObjectId: TOID): IovObject;
    function FindObject(ObjectId: TOID): Boolean;
    function GetObjectSpace: IovObjectSpace;
    procedure UpdateCalculatedAttributes;
    procedure SetObjectsAttributes(const AttributeValues: Variant; const
      WhereExpression: String);
    {{
     ,   WhereExpression.  
    (  )     
    , .     .
    }
    procedure DeleteObjects(const WhereExpression: String);
  protected
  public
    constructor Create(ADatabase: TovDatabase; AClassName: String);
    destructor Destroy; override;
    function CreateObject: IovObject;
    function GetName: String;
    function GetObjects: IovClassObjects;
    property Name: String read GetName;
    property Objects: IovClassObjects read GetObjects;
  end;
  
  TovObject = class(TovInterfacedObject, IovObject)
  private
    FClassName: String;
    FObjectId: Integer;
    FParentClass: TovClass;
    FAttributeValues: Variant;
    procedure ParseProperty(PropertyName: String; var NewClassName,
      NewPropertyName: String; var NewObjectId: TOID);
    procedure AssignInternal(Source, Target: IovObject; CopyAttributes: TStrings =
      nil; CopyRelations: TStrings = nil; OnCopyObject: TCopyObjectEvent = nil);
    procedure SaveState;
  protected
    constructor Create(AParentClass: TovClass; AnObjectId: Integer); virtual;
    procedure Validate; overload;
    function Validate(ValidateAttributes, ValidateRelations, ValidateUserRules:
      Boolean; ValidateErrors: TStrings): Boolean; overload;
    function GetObjectId: Integer;
    function GetProperties(PropertyName: String): IovProperty;
    function GetModelSpace: TDomainModel;
    function GetModelObject: TModelObject;
    function GetClassId: TOID;
    property ModelObject: TModelObject read GetModelObject;
  public
    destructor Destroy; override;
    procedure Assign(Source: IovObject; CopyAttributes: String = '*';
      DontCopyAttributes: String = ''; CopyRelations: String = '*';
      DontCopyRelations: String = ''; OnCopyObject: TCopyObjectEvent = nil);
    function GetParentClass: IovClass;
    function GetClassName: String;
    function GetOwner: IovObject;
    procedure SetOwner(AParentObject: IovObject; ARelationName: String = '');
    procedure BeginUpdate;
    function Copy: IovObject;
    procedure Delete(DeleteMode: TDeleteMode = dmCheckRefs);
    procedure EndUpdate;
    procedure Update;
    procedure Commit;
    function GetReferencingObjects(ImmediateObjects: Boolean = True): IovObjectList;
    function IsSame(AObject: IovObject): Boolean;
    function IsExist: Boolean;
    procedure RestoreState;
    procedure UpdateCalculatedAttributes;
    {{
           .
    }
    function GetAncestor(AGeneralization: TModelRelation): IovObject;
    property ClassName: String read GetClassName;
    property Owner: IovObject read GetOwner;
    property ObjectId: Integer read GetObjectId;
    property ClassId: TOID read GetClassId;
    property ParentClass: TovClass read FParentClass;
    property Properties[PropertyName: String]: IovProperty read GetProperties;
      default;
    property ModelSpace: TDomainModel read GetModelSpace;
  end;

  TovProperty = class(TovInterfacedObject, IovProperty)
  private
    FObjectClassName: String;
    FObjectId: TOID;
    FParentObject: TovObject;
    FPropertyName: String;
    FPropertyType: String;
    FModelAttribute: TModelAttribute;
    function GetAsBoolean: Boolean;
    function GetAsDateTime: TDateTime;
    function GetAsFloat: Double;
    function GetAsInteger: Integer;
    function GetAsBlobStream: IovBlobStream;
    function GetAsObject: IovObject;
    function GetAsObjectId: Integer;
    function GetAsObjects: IovPropertyObjects;
    function GetAsString: String;
    function GetCount: Integer;
    function GetIsObject: Boolean;
    function GetIsBlob: Boolean;
    function GetIsNull: Boolean;
    function GetObjects(Index: Integer): IovObject;
    function GetPropertyName: String;
    function GetPropertyType: String;
    function GetValueInternal(NullValue: Variant): Variant;
    procedure SetAsBoolean(const Value: Boolean);
    procedure SetAsDateTime(const Value: TDateTime);
    procedure SetAsFloat(const Value: Double);
    procedure SetAsInteger(const Value: Integer);
    procedure SetAsObject(const Value: IovObject);
    procedure SetAsObjectId(Value: TOID);
    procedure SetAsString(const Value: string);
    procedure SetValue(Value: Variant);
    procedure GetPropInfo(AClassName, APropName: String; var PropType: String; var
      AModelAttribute: TModelAttribute);
    procedure Remove(AObjectId: TOID); overload;
    procedure Remove(AnObject: IovObject); overload;
    function GetValue: Variant;
  protected
    constructor Create(AParentObject: TovObject; AClassName, APropertyName:
      String; AnObjectId: TOID);
    property ObjectClassName: String read FObjectClassName;
    property ObjectId: TOID read FObjectId;
    property ParentObject: TovObject read FParentObject;
  public
    destructor Destroy; override;
    function Add: IovObject; overload;
    function Add(AnObject: IovObject): TOID; overload;
    function Add(ObjectId: TOID): TOID; overload;
    procedure Clear;
    procedure Delete(Index: Integer); overload;
    procedure DeleteAll;
    procedure RemoveAt(Index: Integer); overload;
    procedure Delete(AnObject: IovObject); overload;
    property AsBoolean: Boolean read GetAsBoolean write SetAsBoolean;
    property AsDateTime: TDateTime read GetAsDateTime write SetAsDateTime;
    property AsFloat: Double read GetAsFloat write SetAsFloat;
    property AsInteger: Integer read GetAsInteger write SetAsInteger;
    property AsObject: IovObject read GetAsObject write SetAsObject;
    property AsObjectId: TOID read GetAsObjectId write SetAsObjectId;
    property AsObjects: IovPropertyObjects read GetAsObjects;
    property AsString: String read GetAsString write SetAsString;
    property Count: Integer read GetCount;
    property Objects[Index: Integer]: IovObject read GetObjects;
    property PropertyName: String read GetPropertyName;
    property PropertyType: String read GetPropertyType;
    property Value: Variant read GetValue write SetValue;
    property IsBlob: Boolean read GetIsBlob;
    property IsNull: Boolean read GetIsNull;
    property AsBlobStream: IovBlobStream read GetAsBlobStream;
  end;
  
  TovBlobStream = class(TInterfacedObject, IovBlobStream)
    procedure Cancel;
    function GetStream(Mode: TBlobStreamMode): TStream;
    procedure Post;
  private
    FDataSet: TDataSet;
    FBlobFieldName: String;
    FStream: TStream;
    procedure LoadFromFile(FileName: String);
    procedure SaveToFile(FileName: String);
  public
    constructor Create(ADataSet: TDataSet; ABlobFieldName: String);
    destructor Destroy; override;
  end;

  TovObjects = class(TInterfacedObject)
    function Find(FilterExpr: String; var AObject: IovObject): Boolean; overload;
  private
    FFilterExpr: String;
    FObjectClass: TovClass;
    FObjects: TOIDList;
    FOrderExpr: String;
    FTempObjects: TOIDList;
  protected
    procedure FindObjects(FilterExpr, OrderExpr: String; Objects: TOIDList);
      virtual; abstract;
  public
    constructor Create(ObjectClass: TovClass);
    destructor Destroy; override;
    function Find(FilterExpr, OrderExpr: String; Objects: TOIDList): Boolean;
      overload;
    function Find(FilterExpr: String; var ObjectId: TOID): Integer; overload;
    function GetCount: Integer;
    function GetObjectIds(Index: Integer): TOID;
    function GetObjects(Index: Integer): IovObject;
    procedure Refresh;
    procedure Select(FilterExpr: String; OrderExpr: String = ''); overload;
    procedure SelectAll;
    function IndexOf(ObjectId: TOID): Integer;
    procedure Select(Attributes, AttributeValues: TStrings; FilterExpr: String = '';
      OrderExpr: String = ''); overload; virtual; abstract;
    property Count: Integer read GetCount;
    property ObjectIds[Index: Integer]: TOID read GetObjectIds;
    property Objects[Index: Integer]: IovObject read GetObjects;
  end;

  TovClassObjects = class(TovObjects, IovClassObjects)
  private
  protected
    procedure FindObjects(FilterExpr, OrderExpr: String; Objects: TOIDList);
      override;
  public
    procedure Select(AttributeValues: TStrings; Attributes: String; FilterExpr:
      String = ''; OrderExpr: String = ''; IdAttribute: String = ''); overload;
  end;

  TovPropertyObjects = class(TovObjects, IovPropertyObjects)
    procedure Remove(AObjectId: TOID); overload;
    function Add(AnObjectId: TOID): TOID; overload;
    procedure Delete(AnObject: IovObject); overload;
    procedure Remove(AnObject: IovObject); overload;
    procedure Assign(Source: IovPropertyObjects);
    procedure Clear;
  private
    FClass: IovClass;
    FParentProperty: TovProperty;
    FRelation: TModelRelation;
    FDatabase: TovDatabase;
    function GetParentObject: TovObject;
    function GetMaxCount: Integer;
    function GetMinCount: Integer;
  protected
    constructor Create(AParentProperty: TovProperty);
    procedure FindObjects(FilterExpr, OrderExpr: String; Objects: TOIDList);
      override;
    property ParentObject: TovObject read GetParentObject;
  public
    destructor Destroy; override;
    function Add: IovObject; overload;
    function Add(AnObject: IovObject): TOID; overload;
    procedure Delete(Index: Integer); overload;
    procedure DeleteAll;
    function GetParentProperty: IovProperty;
    procedure RemoveAt(Index: Integer); overload;
    procedure Select(AttributeValues: TStrings; Attributes: String; FilterExpr:
      String = ''; OrderExpr: String = ''; IdAttribute: String = ''); overload;
    property MaxCount: Integer read GetMaxCount;
    property MinCount: Integer read GetMinCount;
    property ParentProperty: TovProperty read FParentProperty;
  end;

  TovObjectClass = class of TovObject;

  EValidateObjectError = class(Exception)
  end;

  ERequiredElementError = class(EValidateObjectError)
  end;
  ERequiredAttributeError = class(ERequiredElementError)
  end;
  ERequiredRelationError = class(ERequiredElementError)
  end;
  ERelationOutOfMaxCountError = class(EValidateObjectError)
  end;
  ERelationOutOfMinCountError = class(EValidateObjectError)
  end;

function NewObjectList: IovObjectList;

function IsSameObjects(AObject1, AObject2: IovObject): Boolean;

implementation

uses ovConsts, ovUtils, Variants;

type
  TovObjectList = class(TInterfaceList, IovObjectList)
  private
    function GetObjects(Index: Integer): IovObject;
  public
    function IndexOfObject(AObject: IovObject): Integer;
    procedure Add(AObject: IovObject);
    procedure Assign(AObjects: IovObjectList);
    property Objects[Index: Integer]: IovObject read GetObjects; default;
  end;

function NewObjectList: IovObjectList;
begin
  Result := TovObjectList.Create
end;

{{
      True,    
      .  AObject1,   AObject2  
 nil.
}
function IsSameObjects(AObject1, AObject2: IovObject): Boolean;
begin
  Result := Not Assigned(AObject1) and Not Assigned(AObject2) or
    Assigned(AObject1) and Assigned(AObject2) and AObject1.IsSame(AObject2)
end;

procedure LoadDataSetToStrings(DataSet: TDataSet; QueryInfo: TSelectQueryInfo;
  AFieldValues: TStrings; Database: TovDatabase; IdAttribute: String = '');
var
  Values: String;
  I: Integer;
  Value: Variant;
  ObjectIdField: TField;
begin
  if IdAttribute = '' then
    ObjectIdField := QueryInfo.Objects[0].GetField(TObjectIdField).Field
  else
    ObjectIdField := nil;

  with DataSet do
  while Not EOF do
  begin
    //    
    Values := '';
    with QueryInfo do
      for I := 0 to FieldCount - 1 do
      begin
        if (Fields[I] is TAttributeField) and Not Fields[I].AutoSelected then
        begin
          if TAttributeField(Fields[I]).ModelAttribute.Calculated and
            Not TAttributeField(Fields[I]).ModelAttribute.Persistent then
          begin
            Database.CalculateAttribute(
              ObjectIdField.AsInteger, TAttributeField(Fields[I]).ModelAttribute, Value);
            Values := Values + ' ' + VarToStr(Value);
          end else
            Values := Values + ' ' + QueryInfo.Fields[I].Field.AsString;
        end else
        if (Fields[I] is TReferenceField) and Not Fields[I].AutoSelected then
          Values := Values + ' ' + QueryInfo.Fields[I].Field.AsString;

        if (IdAttribute <> '') and (QueryInfo.Fields[I].FieldName = IdAttribute) then
          ObjectIdField := QueryInfo.Fields[I].Field;
      end;

    System.Delete(Values, 1, 1);
    if Assigned(ObjectIdField) then
      AFieldValues.AddObject(Values, Pointer(ObjectIdField.AsInteger))
    else
      AFieldValues.Add(Values);
    Next;
  end;
end;

{
***************************** TovInterfacedObject ******************************
}

constructor TovInterfacedObject.Create(ADatabase: TovDatabase);
begin
  inherited Create;
  FDatabase := ADatabase;
end;

{
*********************************** TovClass ***********************************
}

constructor TovClass.Create(ADatabase: TovDatabase; AClassName: String);
begin
  inherited Create(ADatabase);
  FObjects := TovClassObjects.Create(Self);
  FObjects._AddRef;

  FName := AClassName;
end;

destructor TovClass.Destroy;
begin
  FObjects._Release;
  inherited;
end;

function TovClass.CreateInstance(ObjectId: TOID): IovObject;
var
  AClass: TClass;
  AObjectClass: TovObjectClass;
begin
  if Database.GetClassImplementor(Name, AClass) then
    AObjectClass := TovObjectClass(AClass)
  else
    AObjectClass := TovObject;
  Result := AObjectClass.Create(Self, ObjectId)
end;

function TovClass.CreateObject: IovObject;
var
  OID: TOID;
begin
  OID := Database.CreateObject(Name);
  Result := CreateInstance(OID)
end;

procedure TovClass.DeleteAllObjects;
var
  Objects: TOIDList;
begin
  Objects := TOIDList.Create;
  try
    Database.GetObjects(Name, Objects);
    Database.DeleteObjects(Name, Objects);
  finally
    Objects.Free
  end;
end;

procedure TovClass.DeleteObject(ObjectId: TOID);
begin
  Database.DeleteObject(Name, ObjectId)
end;

function TovClass.GetName: String;
begin
  Result := FName;
end;

function TovClass.FindObject(ObjectId: TOID): Boolean;
var
  FieldName: String;
begin
  with Database.GetObjectAttributeField(Name, '', ObjectId, FieldName, True) do
  try
    Result := RecordCount > 0
  finally
    Free
  end;
end;

function TovClass.GetObject(ObjectId: TOID): IovObject;
begin
  if ObjectId = 0 then
    raise EObjectNotFound.CreateFmt('    "%s"', [Name]);

//  if Not FindObject(ObjectId) then
//    raise EObjectNotFound.CreateFmt('  "%s"   (%d)', [Name, ObjectId]);
  Result := CreateInstance(ObjectId);
end;

function TovClass.GetRealObject(ObjectId: TOID): IovObject;
var
  O: TModelObject;
  FieldName: String;
  ChildIdField: TField;

  procedure InvalidClassId;
  begin
    raise Exception.CreateFmt('   .'#13+
      '   "%s",     "%s".'#13+
      '  -       .',
      [Self.Name, O.FullName])
  end;

  procedure DerivedObjectNotFound;
  begin
    raise Exception.CreateFmt('   .'#13+
      '      "%s" (ID=%d).    "%s" (ID=%d).',
      [O.FullName, ChildIdField.AsInteger, Self.Name, ObjectId])
  end;

begin
  with Database.GetObjectAttributeField(Name, '', ObjectId, FieldName, True) do
  try
    O := Database.Model.GetObject(FieldByName(ClassIdField).AsInteger);
    if O.FullName = Self.Name then
      Result := CreateInstance(ObjectId)
    else
    begin
      ChildIdField := FindField(ovConsts.ChildIdField);
      if Not Assigned(ChildIdField) or (ChildIdField.AsInteger = 0) then
        InvalidClassId
      else
      begin
        Result := Database.Classes[O.FullName].GetObject(ChildIdField.AsInteger);
        if Not Result.IsExist then DerivedObjectNotFound
      end;
    end;
  finally
    Free
  end;
end;

function TovClass.GetObjects: IovClassObjects;
begin
  Result := FObjects
end;

function TovClass.GetObjectSpace: IovObjectSpace;
begin
  Result := Database;
end;

procedure TovClass.UpdateCalculatedAttributes;
var
  I: Integer;
begin
  with Objects do
    for I := 0 to Count - 1 do Objects[I].UpdateCalculatedAttributes;
end;

procedure TovClass.SetObjectsAttributes(const AttributeValues: Variant; const
  WhereExpression: String);
begin
  Database.UpdateObjectsAttributes(Name, AttributeValues, WhereExpression);
end;

procedure TovClass.DeleteObjects(const WhereExpression: String);
begin
  Database.DeleteObjects(Name, WhereExpression);
end;

{
********************************** TovObject ***********************************
}
constructor TovObject.Create(AParentClass: TovClass; AnObjectId: Integer);
begin
  inherited Create(AParentClass.Database);
  FParentClass := AParentClass;
  FParentClass._AddRef;
  
  FClassName := AParentClass.Name;
  FObjectId := AnObjectId;
end;

destructor TovObject.Destroy;
begin
  Update;

  FParentClass._Release;
  inherited;
end;

{ **************************************************************************** }
{                                       }
{ **************************************************************************** }

{{
          .
}
function TovObject.Copy: IovObject;
begin
  Database.StartTransaction;
  try
    Result := ParentClass.CreateObject;
    AssignInternal(Self, Result);
    Database.CommitTransaction;
  except
    Database.RollbackTransaction;
    raise;
  end;
end;

{{
         
 Copy.  CopyAttributes    ,
   .      "*", 
  .  DontCopyAttributes   
,     .   
   (  )  ";". 
  ,      .
}
procedure TovObject.Assign(Source: IovObject; CopyAttributes: String = '*';
  DontCopyAttributes: String = ''; CopyRelations: String = '*';
  DontCopyRelations: String = ''; OnCopyObject: TCopyObjectEvent = nil);

  procedure GetListToCopy(Items: TStrings; CopyItems, DontCopyItems: String);
  var
    DontCopyItemsList: TStrings;
    I, J: Integer;
  begin
    DontCopyItemsList := TStringList.Create;
    try
      {   ,    }
      if CopyItems <> '*' then {     Items   }
        AttributesToList(CopyItems, ';', Items);

      {   ,     }
      AttributesToList(DontCopyItems, ';', DontCopyItemsList);
      for I := 0 to DontCopyItemsList.Count - 1 do
      begin
        J := Items.IndexOf(DontCopyItemsList[I]);
        if J >= 0 then Items.Delete(J);
      end;
    finally
      DontCopyItemsList.Free;
    end;
  end;

var
  Attributes, Relations: TStrings;
  AModelObject: TModelObject;
  I: Integer;
begin
  Database.StartTransaction;
  try

    Attributes := TStringList.Create;
    Relations := TStringList.Create;
    try
      AModelObject := Database.Model.GetObject(ClassName);

      for I := 0 to AModelObject.AttributeCount - 1 do
        Attributes.Add(AModelObject.Attributes[I].Name);
      GetListToCopy(Attributes, CopyAttributes, DontCopyAttributes);

      for I := 0 to AModelObject.RelationCount - 1 do
        Relations.Add(AModelObject.Relations[I].Name);
      GetListToCopy(Relations, CopyRelations, DontCopyRelations);

      AssignInternal(Source, Self, Attributes, Relations, OnCopyObject);

    finally
      Relations.Free;
      Attributes.Free;
    end;

    Database.CommitTransaction;
  except
    Database.RollbackTransaction;
    raise;
  end;
end;

{{
        Source  Target.
  CopyRelations = False,     , 
     .   -
CopyAttributes,    ,    .
}
procedure TovObject.AssignInternal(Source, Target: IovObject; CopyAttributes:
  TStrings = nil; CopyRelations: TStrings = nil; OnCopyObject: TCopyObjectEvent
  = nil);
var
  Relation: TModelRelation;

  procedure AssignRelations(Relation: TModelRelation);
  var
    I: Integer;
  begin
    //     
    Target[Relation.Name].AsObjects.DeleteAll;

    if Relation.IsReference then
      Target[Relation.Name].AsObjectId := Source[Relation.Name].AsObjectId
    else
    if Relation.IsComposition then
      with Source[Relation.Name].AsObjects do
        for I := 0 to Count - 1 do
          //      Assign
          Target[Relation.Name].AsObjects.Add.Assign(Objects[I], '*', '', '*', '', OnCopyObject)
    else
    if Relation.IsAggregation then
      with Source[Relation.Name].AsObjects do
        for I := 0 to Count - 1 do
        begin
          //      Assign
          Target[Relation.Name].AsObjects.Add.Assign(Objects[I], '*', '', '*', '', OnCopyObject)
        end
    else
      with Source[Relation.Name].AsObjects do
        for I := 0 to Count - 1 do
          //   
          Target[Relation.Name].AsObjects.Add(Objects[I])
  end;

var
  AttributeName: String;
  I: Integer;
  RelationName: String;
  TargetObject: TModelObject;
  SourceObject: TModelObject;
  TargetAttribute: TModelAttribute;
begin
  Target.BeginUpdate;
  try
    with FParentClass.Database.Model do
    begin
      SourceObject := GetObject(Source.ClassName);
      TargetObject := GetObject(Target.ClassName);
    end;

    //      Source,    Target
    for I := 0 to SourceObject.AttributeCount - 1 do
    begin
      AttributeName := SourceObject.Attributes[I].Name;
      if Not Assigned(CopyAttributes) or (CopyAttributes.IndexOf(AttributeName) >= 0) then
      begin
        TargetAttribute := TargetObject.FindAttribute(AttributeName);
        if Assigned(TargetAttribute) and Not TargetAttribute.Calculated
          // source attribute can be calculated !!
          then Target[AttributeName].Value := Source[AttributeName].Value
      end;
    end;

    if Assigned(OnCopyObject) then OnCopyObject(Source, Target);

  finally
    //       Post   ,
    //       ,    
    //    SQL-
    Target.EndUpdate
  end;

  for I := 0 to SourceObject.RelationCount - 1 do
  begin
    RelationName := SourceObject.Relations[I].Name;
    if Not Assigned(CopyRelations) or (CopyRelations.IndexOf(RelationName) >= 0) then
    begin
      Relation := TargetObject.FindRelation(RelationName);
      if (Relation <> nil) and Not Relation.Derived then AssignRelations(Relation)
    end;
  end;

  Target.UpdateCalculatedAttributes;

end;

{{
     .   .
}
procedure TovObject.SaveState;
var
  ModelObject: TModelObject;
  I: Integer;
begin
  ModelObject := FParentClass.Database.Model.GetObject(ClassName);

  //    
  FAttributeValues := VarArrayCreate([0, ModelObject.AttributeCount - 1], varVariant);
  for I := 0 to ModelObject.AttributeCount - 1 do
    FAttributeValues[I] := Properties[ModelObject.Attributes[I].Name].Value;

  //     (  )
end;

procedure TovObject.RestoreState;
begin
  // TODO -cMM: TovObject.SaveState default body inserted
end;

{ **************************************************************************** }
{                                                                              }
{ **************************************************************************** }

{{
        
IovClass.CreateObject   Commit   ,  
     ( )    
 ,    .
  Commit       
 ,   Commit   .
     Commit   ,   
     .
}
procedure TovObject.Commit;
begin
end;

procedure TovObject.BeginUpdate;
begin
  Database.BeginUpdateTables
end;

procedure TovObject.Delete(DeleteMode: TDeleteMode = dmCheckRefs);
begin
  Database.DeleteObject(FClassName, ObjectId, DeleteMode)
end;

procedure TovObject.EndUpdate;
begin
  Database.EndUpdateTables
end;

function TovObject.GetObjectId: Integer;
begin
  Result := FObjectId
end;

function TovObject.GetParentClass: IovClass;
begin
  Result := ParentClass
end;

function TovObject.GetProperties(PropertyName: String): IovProperty;
var
  NewClassName, NewPropertyName: String;
  NewObjectId: TOID;
begin
  try
    ParseProperty(PropertyName, NewClassName, NewPropertyName, NewObjectId);
    Result := TovProperty.Create(Self, NewClassName, NewPropertyName, NewObjectId);
  except
    on E: Exception do
      raise Exception.CreateFmt(
        '    "%s"   "%s" (ID=%d):'#13'%s',
        [PropertyName, GetClassName, ObjectId, E.Message]);
  end;
end;

procedure TovObject.ParseProperty(PropertyName: String; var NewClassName,
  NewPropertyName: String; var NewObjectId: TOID);
var
  Relation: TModelRelation;
  Attribute: TModelAttribute;
  PropertyNamePart, CurClassName: String;
  CurObjectId: TOID;
  AObject: TModelObject;
begin
  //      ()   PropertyName
  CurClassName := FClassName;
  CurObjectId := ObjectId;
  while ParseAttribute(PropertyNamePart, PropertyName) do
  begin
    if CurObjectId <> 0 then
      CurObjectId := Database.GetObjectRelation(CurClassName, PropertyNamePart, CurObjectId, 0);
    CurClassName := ParentClass.Database.Model.GetObjectRelation(CurClassName, PropertyNamePart).TargetObject.FullName;
  end;

  if CurObjectId > 0 then
  begin
    //            
    AObject := Database.Model.GetObject(CurClassName);
    Attribute := AObject.FindAttribute(PropertyNamePart);
    if Attribute <> nil then
    begin
      if Attribute.IsInherited and Not Attribute.ParentObject.IsAbstract then
      begin
        //    
        CurObjectId := Database.GetAncestorObject(AObject, Attribute.ParentObject, CurObjectId);
        CurClassName := Attribute.ParentObject.FullName;
      end;
    end else
    begin
      Relation := AObject.GetRelation(PropertyNamePart);
      if Relation.IsInherited and Not Relation.ParentObject.IsAbstract then
      begin
        //    
        CurObjectId := Database.GetAncestorObject(AObject, Relation.ParentObject, CurObjectId);
        CurClassName := Relation.ParentObject.FullName;
      end;
    end;
  end;

  NewObjectId := CurObjectId;
  NewClassName := CurClassName;
  NewPropertyName := PropertyNamePart;
end;

procedure TovObject.Update;
begin
  { TODO :  }
  //  ParentClass.DataSet.CheckBrowseMode
end;

function TovObject.GetClassName: String;
begin
  Result := FClassName
end;

procedure TovObject.Validate;
begin
  {           }
  Validate(True, True, True, nil);
end;

{{
      .   
   ValidateErrors (       Objects).
}
function TovObject.Validate(ValidateAttributes, ValidateRelations,
  ValidateUserRules: Boolean; ValidateErrors: TStrings): Boolean;

  procedure HandleError(ExceptionClass: ExceptClass; ErrorMsg: String;
    ErrorSource: TObject);
  begin
    if Assigned(ValidateErrors) then
    begin
      ValidateErrors.AddObject(ErrorMsg, ErrorSource);
      Result := False;
    end
    else
      raise ExceptionClass.Create(ErrorMsg);
  end;

var
  I: Integer;
  ModelObject: TModelObject;
  Relation: TModelRelation;
  Attribute: TModelAttribute;
  TargetCount: Integer;
begin
  Result := True;
  ModelObject := ParentClass.Database.Model.GetObject(ClassName);

  with ModelObject do
  begin
    //   
    if ValidateAttributes then
    for I := 0 to AttributeCount - 1 do
    begin
      Attribute := Attributes[I];
      if Attribute.Required and Properties[Attribute.Name].IsNull then
        HandleError(ERequiredAttributeError, Attribute.RequiredMsg, Attribute);
    end;

    //      
    if ValidateRelations then
    for I := 0 to RelationCount - 1 do
    begin
      Relation := Relations[I];
      TargetCount := Properties[Relation.Name].Count;

      if Relation.Required and (TargetCount = 0) then
        HandleError(ERequiredRelationError, Relation.RequiredMsg, Relation);
      if TargetCount < Relation.TargetMinCount then
        HandleError(ERelationOutOfMinCountError,
          Format('   "%s"    (OID=%d)',
            [Relation.Name, ObjectId]), Relation);
      if TargetCount > Relation.TargetMaxCount then
        HandleError(ERelationOutOfMaxCountError,
          Format('     "%s" (OID=%d)',
            [Relation.Name, ObjectId]), Relation);
    end;

    //   -
    if ValidateUserRules then
    try
      ParentClass.Database.CustomValidateObject(ModelObject, ObjectId);
    except
      on E: Exception do HandleError(ExceptClass(E.ClassType), E.Message, ModelObject);
    end;
  end;
end;

{{
       Persistent 
     .
}
procedure TovObject.UpdateCalculatedAttributes;
var
  I: Integer;
  Attribute: TModelAttribute;
  Value: Variant;
begin
  with ParentClass.Database.Model.GetObject(ClassName) do
  begin
    for I := 0 to AttributeCount - 1 do
    begin
      Attribute := Attributes[I];
      if Attribute.Calculated and Attribute.Persistent then
      begin
        Value := Null;
        Database.CalculateAttribute(ObjectId, Attribute, Value);
        GetProperties(Attribute.Name).Value := Value;
      end;
    end;
  end;
end;

{{
  ,    .
}
function TovObject.GetReferencingObjects(ImmediateObjects: Boolean = True):
  IovObjectList;
var
  I: Integer;
  Objects: TovObjectRefList;
begin
  Objects := TovObjectRefList.Create;
  try
    Result := NewObjectList;
    Database.GetReferencingObjects(ClassName, ObjectId, Objects, ImmediateObjects);
    for I := 0 to Objects.Count - 1 do
      with Objects[I] do
      try
        Result.Add(Database.Classes[ClassName].GetObject(ObjectId));
      except
        //     ,     
        on E: EObjectNotFound do
      end;
  finally
    Objects.Free;
  end;
end;

function TovObject.GetOwner: IovObject;
var
  OwnerId, RelationId: TOID;
  ModelRelation: TModelRelation;
begin
  FParentClass.Database.GetObjectOwner(ParentClass.Name, ObjectId, OwnerId, RelationId);
  if (OwnerId > 0) and (RelationId > 0) then
  begin
    { TODO :   RelationId     Database,  
     ModelRelation? }
    ModelRelation := Database.Model.GetRelation(RelationId);
    Result := Database.Classes[ModelRelation.SourceObjectName].GetObject(OwnerId);
  end else
    Result := nil
end;

procedure TovObject.SetOwner(AParentObject: IovObject; ARelationName: String = 
  '');
var
  OwnerId, RelationId: TOID;
  ModelRelation: TModelRelation;
begin
  if AParentObject = nil then begin
    OwnerId := 0;
    RelationId := 0;
  end else begin
    OwnerId := AParentObject.ObjectId;
    ModelRelation := Database.Model.GetObject(AParentObject.ClassName).GetRelation(ARelationName);
    if Not ModelRelation.IsComposition then
      raise Exception.Create('    ');
    RelationId := ModelRelation.Id;
  end;
  FParentClass.Database.SetObjectOwner(ParentClass.Name, ObjectId, OwnerId, RelationId)
end;

{{
      True,    
      .
}
function TovObject.IsSame(AObject: IovObject): Boolean;
begin
  Result := Assigned(AObject) and
    (ClassName = AObject.ClassName) and (ObjectId = AObject.ObjectId)
end;

function TovObject.IsExist: Boolean;
begin
  Result := ParentClass.FindObject(ObjectId)
end;

function TovObject.GetAncestor(AGeneralization: TModelRelation): IovObject;
var
  OID: TOID;
begin
  OID := Database.GetAncestor(ObjectId, AGeneralization);
  Result := Database.Classes[AGeneralization.TargetObjectName].GetObject(OID);
end;

function TovObject.GetModelSpace: TDomainModel;
begin
  Result := Database.Model
end;

function TovObject.GetModelObject: TModelObject;
begin
  Result := Database.Model.GetObject(ClassName)
end;

function TovObject.GetClassId: TOID;
begin
  Result := ModelObject.Id
end;

{
********************************* TovProperty **********************************
}
constructor TovProperty.Create(AParentObject: TovObject; AClassName,
  APropertyName: String; AnObjectId: TOID);
begin
  inherited Create(AParentObject.ParentClass.Database);
  FParentObject := AParentObject;
  FParentObject._AddRef;

  FObjectClassName := AClassName;
  FPropertyName := APropertyName;
  FObjectId := AnObjectId;

  GetPropInfo(FObjectClassName, PropertyName, FPropertyType, FModelAttribute);
end;

destructor TovProperty.Destroy;
begin
  FParentObject._Release;
  inherited;
end;

{ **************************************************************************** }
{                                                              }
{ **************************************************************************** }

function TovProperty.GetAsBoolean: Boolean;
begin
  Result := VarAsType(GetValueInternal(False), varBoolean)
end;

function TovProperty.GetAsDateTime: TDateTime;
begin
  Result := VarAsType(GetValueInternal(0.0), varDate)
end;

function TovProperty.GetAsFloat: Double;
begin
  Result := VarAsType(GetValueInternal(0.0), varDouble)
end;

function TovProperty.GetAsInteger: Integer;
begin
  Result := VarAsType(GetValueInternal(0), varInteger)
end;

function TovProperty.GetAsString: String;
begin
  Result := GetValueInternal('')
end;

function TovProperty.GetValue: Variant;
begin
  Result := GetValueInternal(Null)
end;

function TovProperty.GetValueInternal(NullValue: Variant): Variant;
var
  DataSet: TDataSet;
  AFieldName: String;
begin
  if Not Assigned(FModelAttribute) then
    raise Exception.CreateFmt(' %s   ', [PropertyName]);

  if ObjectId = 0 then
    Result := NullValue
//    raise Exception.Create('     ')
  else begin
    if FModelAttribute.Calculated and Not FModelAttribute.Persistent then
      Database.CalculateAttribute(ObjectId, FModelAttribute, Result)
    else
    begin
      DataSet := Database.GetObjectAttributeField(ObjectClassName, PropertyName,
        ObjectId, AFieldName, True);
      try
        if DataSet.IsEmpty then
          raise Exception.Create('  : ' + IntToStr(ObjectId));
        Result := DataSet[AFieldName];

        if FModelAttribute.IsBoolean and Not Database.Adapter.SupportsBooleanType
        then
          Result := Not VarIsNull(Result) and (Result <> FalseValueReplacement)

      finally
        DataSet.Free;
      end;
    end;
    if Result = Null then Result := NullValue;
  end;
end;

function TovProperty.GetIsNull: Boolean;
begin
  Result := VarIsNull(Value);
end;

function TovProperty.GetIsBlob: Boolean;
var
  DataSet: TDataSet;
  AFieldName: String;
begin
  if FModelAttribute.Calculated then
    Result := False
  else begin
    DataSet := Database.GetObjectAttributeField(ObjectClassName, PropertyName,
      ObjectId, AFieldName, True);
    try
      Result := DataSet.FieldByName(AFieldName).IsBlob;
    finally
      DataSet.Free;
    end;
  end;
end;

function TovProperty.GetAsBlobStream: IovBlobStream;
var
  DataSet: TDataSet;
  BlobFieldName: String;
begin
  DataSet := Database.GetObjectAttributeField(ObjectClassName, PropertyName,
    ObjectId, BlobFieldName, False);
  Result := TovBlobStream.Create(DataSet, BlobFieldName)
end;

procedure TovProperty.SetAsBoolean(const Value: Boolean);
begin
  SetValue(Value)
end;

procedure TovProperty.SetAsDateTime(const Value: TDateTime);
begin
  SetValue(Value)
end;

procedure TovProperty.SetAsFloat(const Value: Double);
begin
  SetValue(Value)
end;

procedure TovProperty.SetAsInteger(const Value: Integer);
begin
  SetValue(Value)
end;

procedure TovProperty.SetAsString(const Value: string);
begin
  SetValue(Value)
end;

procedure TovProperty.SetValue(Value: Variant);
begin
  if Not Assigned(FModelAttribute) then
    raise Exception.CreateFmt(' %s   ', [PropertyName]);

  if ObjectId = 0 then
    raise EUnAssignedReference.CreateFmt(
      '    "%s"  "%s"   ',
      [PropertyName, ObjectClassName]);

  if FModelAttribute.Calculated and Not FModelAttribute.Persistent then
    raise Exception.CreateFmt('    "%s"', [PropertyName]);

  Database.UpdateObjectAttribute(FModelAttribute, ObjectId, Value);
end;

{ **************************************************************************** }
{                        1                             }
{ **************************************************************************** }

function TovProperty.GetAsObjectId: Integer;
begin
  if Assigned(FModelAttribute) then
    raise Exception.CreateFmt(' %s   ', [PropertyName]);

  if ObjectId = 0 then
    Result := 0
  else
    Result := Database.GetObjectRelation(ObjectClassName, PropertyName, ObjectId, 0);
end;

{ TODO :      }
function TovProperty.GetAsObject: IovObject;
var
  OID: TOID;
begin
  OID := AsObjectId;
  if OID <> 0 then
    Result := TovClass.Create(Database, PropertyType).CreateInstance(OID)
  else
    Result := nil
end;

procedure TovProperty.SetAsObject(const Value: IovObject);
var
  ModelObject: TModelObject;
begin
  if Value = nil then AsObjectId := 0 else
  if Value.ParentClass.Name = PropertyType then
    AsObjectId := Value.ObjectId
  else
  begin //     PropertyType
    ModelObject := Database.Model.GetObject(Value.ClassName);
    AsObjectId := Database.GetAncestor(Value.ObjectId, ModelObject.GetParentRelation(PropertyType));
  end
end;

procedure TovProperty.SetAsObjectId(Value: TOID);
begin
  Database.UpdateObjectRelation(ObjectClassName, PropertyName, ObjectId, 0, Value);
end;

{ **************************************************************************** }
{                          1                      }
{ **************************************************************************** }

function TovProperty.GetAsObjects: IovPropertyObjects;
begin
  { TODO :  }

  {
  BeforeGet(False);

  with ParentObject.ParentClass do
  begin
    ADataSet := Database.SelectChildObjects(Name, PropertyName,
      DataSource, False);
    ADataSet.Open;
    Result := Database.CreateClass(Database.GetAttributeClassName(Name, PropertyName),
      ADataSet, True);
  end;
  }
  if Assigned(FModelAttribute) then
    raise Exception.CreateFmt(' %s   ', [PropertyName]);
  Result := TovPropertyObjects.Create(Self);
end;

function TovProperty.GetCount: Integer;
begin
  Result := AsObjects.Count;
end;

function TovProperty.Add: IovObject;
begin
  Result := AsObjects.Add;
end;

function TovProperty.Add(AnObject: IovObject): TOID;
begin
  Result := AsObjects.Add(AnObject);
end;

function TovProperty.Add(ObjectId: TOID): TOID;
begin
  Result := AsObjects.Add(ObjectId);
end;

procedure TovProperty.Delete(Index: Integer);
begin
  AsObjects.Delete(Index);
end;

procedure TovProperty.Delete(AnObject: IovObject);
begin
  AsObjects.Delete(AnObject);
end;

procedure TovProperty.DeleteAll;
begin
  AsObjects.DeleteAll;
end;

procedure TovProperty.Clear;
begin
  AsObjects.Clear;
end;

procedure TovProperty.RemoveAt(Index: Integer);
begin
  AsObjects.RemoveAt(Index);
end;

procedure TovProperty.Remove(AnObject: IovObject);
begin
  AsObjects.Remove(AnObject);
end;

procedure TovProperty.Remove(AObjectId: TOID);
begin
  AsObjects.Remove(AObjectId);
end;

{ **************************************************************************** }
{                                                                              }
{ **************************************************************************** }

function TovProperty.GetIsObject: Boolean;
begin
  Result := Not Assigned(FModelAttribute)
end;

function TovProperty.GetObjects(Index: Integer): IovObject;
begin
  Result := AsObjects.Objects[Index];
end;

function TovProperty.GetPropertyName: String;
begin
  Result := FPropertyName;
end;

function TovProperty.GetPropertyType: String;
begin
  Result := FPropertyType;
end;
          
{{
        APropName
 AClassName.
}
procedure TovProperty.GetPropInfo(AClassName, APropName: String; var PropType:
  String; var AModelAttribute: TModelAttribute);
var
  O: TModelObject;
  R: TModelRelation;
begin
  O := Database.Model.GetObject(AClassName);
  AModelAttribute := O.FindAttribute(APropName);
  if Assigned(AModelAttribute) then
  begin
    PropType := AModelAttribute.TypeName;
  end else
  begin
    R := O.GetRelation(APropName);
    PropType := R.TargetObjectName;
  end;
end;

{
********************************** TovBlobStream *******************************
}
constructor TovBlobStream.Create(ADataSet: TDataSet; ABlobFieldName: String);
begin
  inherited Create;
  FDataSet := ADataSet;
  FBlobFieldName := ABlobFieldName;
end;

destructor TovBlobStream.Destroy;
begin
  FreeAndNil(FStream);
  FDataSet.Free;
  inherited;
end;

function TovBlobStream.GetStream(Mode: TBlobStreamMode): TStream;
begin
  Cancel;
  if Mode <> bmRead then FDataSet.Edit;
  FStream := FDataSet.CreateBlobStream(FDataSet.FieldByName(FBlobFieldName), Mode);
  Result := FStream;
end;

procedure TovBlobStream.Cancel;
begin
  FreeAndNil(FStream);
  FDataSet.Cancel;
end;

procedure TovBlobStream.Post;
begin
  FreeAndNil(FStream);
  FDataSet.CheckBrowseMode
end;

procedure TovBlobStream.LoadFromFile(FileName: String);
var
  FileStream: TStream;
begin
  FileStream := TFileStream.Create(FileName, fmOpenRead);
  try
    FDataSet.Edit;
    FStream := FDataSet.CreateBlobStream(FDataSet.FieldByName(FBlobFieldName), bmWrite);
    try
      FStream.CopyFrom(FileStream, 0);
    finally
      FreeAndNil(FStream);
    end;
    FDataSet.Post;
  finally
    FileStream.Free;
  end;
end;

procedure TovBlobStream.SaveToFile(FileName: String);
var
  FileStream: TFileStream;
begin
  FileStream := TFileStream.Create(FileName, fmCreate);
  try
    FStream := FDataSet.CreateBlobStream(FDataSet.FieldByName(FBlobFieldName), bmRead);
    try
      FileStream.CopyFrom(FStream, 0);
    finally
      FreeAndNil(FStream);
    end;
  finally
    FileStream.Free;
  end;
end;

{
********************************** TovObjects **********************************
}
constructor TovObjects.Create(ObjectClass: TovClass);
begin
  inherited Create;
  FObjectClass := ObjectClass;
  FTempObjects := TOIDList.Create;
end;

destructor TovObjects.Destroy;
begin
  FObjects.Free;
  FTempObjects.Free;
  inherited Destroy;
end;

function TovObjects.Find(FilterExpr, OrderExpr: String; Objects: TOIDList):
  Boolean;
begin
  Objects.Clear;
  FindObjects(FilterExpr, OrderExpr, Objects);
  Result := Objects.Count > 0;
end;

function TovObjects.Find(FilterExpr: String; var ObjectId: TOID): Integer;
begin
  FTempObjects.Clear;
  FindObjects(FilterExpr, '', FTempObjects);
  Result := FTempObjects.Count;
  if Result > 0
    then ObjectId := FTempObjects[0]
    else ObjectId := 0
end;

function TovObjects.Find(FilterExpr: String; var AObject: IovObject): Boolean;
begin
  FTempObjects.Clear;
  FindObjects(FilterExpr, '', FTempObjects);
  Result := FTempObjects.Count > 0;
  if Result then AObject := FObjectClass.GetObject(FTempObjects[0])
end;

function TovObjects.GetCount: Integer;
begin
  if Not Assigned(FObjects) then SelectAll;
  Result := FObjects.Count;
end;

function TovObjects.GetObjectIds(Index: Integer): TOID;
begin
  if Not Assigned(FObjects) then SelectAll;
  Result := FObjects[Index];
end;

function TovObjects.GetObjects(Index: Integer): IovObject;
begin
  if Not Assigned(FObjects) then SelectAll;
  Result := FObjectClass.CreateInstance(FObjects[Index]);
end;

procedure TovObjects.Refresh;
begin
  if Not Assigned(FObjects)
    then FObjects := TOIDList.Create
    else FObjects.Clear;
  Find(FFilterExpr, FOrderExpr, FObjects);
end;

procedure TovObjects.Select(FilterExpr: String; OrderExpr: String = '');
begin
  FFilterExpr := FilterExpr;
  FOrderExpr := OrderExpr;
  Refresh;
end;

procedure TovObjects.SelectAll;
begin
  Select('', '')
end;

function TovObjects.IndexOf(ObjectId: TOID): Integer;
begin
  if Not Assigned(FObjects) then SelectAll;
  Result := FObjects.IndexOf(ObjectId)
end;

{
******************************* TovClassObjects ********************************
}
procedure TovClassObjects.FindObjects(FilterExpr, OrderExpr: String; Objects:
  TOIDList);
begin
  FObjectClass.Database.GetObjects(FObjectClass.Name, Objects, FilterExpr, OrderExpr)
end;

procedure TovClassObjects.Select(AttributeValues: TStrings; Attributes: String;
  FilterExpr: String = ''; OrderExpr: String = ''; IdAttribute: String = '');
var
  QueryInfo: TSelectQueryInfo;
  DataSet: TDataSet;
begin
  DataSet := FObjectClass.Database.OpenObjectDataSet('',
    FObjectClass.Name, Attributes, FilterExpr, OrderExpr, '', QueryInfo);
  try
    LoadDataSetToStrings(DataSet, QueryInfo, AttributeValues, FObjectClass.Database, IdAttribute);
  finally
    QueryInfo.Free;
    DataSet.Free;
  end;
end;

{
****************************** TovPropertyObjects ******************************
}
constructor TovPropertyObjects.Create(AParentProperty: TovProperty);
var
  PropertyClass: TovClass;
begin
  with AParentProperty do
    PropertyClass := TovClass.Create(ParentObject.ParentClass.Database, PropertyType);

  inherited Create(PropertyClass);

  FClass := PropertyClass; //     -      
  FDatabase := PropertyClass.Database;
  FParentProperty := AParentProperty;
  FParentProperty._AddRef;

  with ParentProperty do
    FRelation := FDatabase.Model.GetObjectRelation(ObjectClassName, PropertyName);
end;

destructor TovPropertyObjects.Destroy;
begin
  FParentProperty._Release;
  inherited;
end;

function TovPropertyObjects.Add: IovObject;
var
  ChildId, RelationId: TOID;
begin
  with ParentObject do
    ChildId := FDatabase.AddChildObject(ParentProperty.ObjectClassName,
      ParentProperty.PropertyName, ParentProperty.ObjectId, 0, RelationId);
  Result := FDatabase.Classes[ParentProperty.PropertyType].GetObject(ChildId);
end;

function TovPropertyObjects.Add(AnObjectId: TOID): TOID;
var
  RelationId: TOID;
begin
  with ParentObject do
    FDatabase.AddChildObject(ParentProperty.ObjectClassName,
      ParentProperty.PropertyName, ParentProperty.ObjectId, AnObjectId, RelationId);
  Result := RelationId;
end;

function TovPropertyObjects.Add(AnObject: IovObject): TOID;
begin
  Result := Add(AnObject.ObjectId)
end;

procedure TovPropertyObjects.Delete(Index: Integer);
begin
  if Not Assigned(FObjects) then SelectAll;
{ TODO :     , ..   
     (Assign),     XML
   .       .
}
  with ParentObject do
    FDatabase.DeleteObjectRelation(ClassName, ParentProperty.PropertyName,
      ObjectId, FObjects.ObjectId[Index], FObjects.RelationId[Index]);

  FObjects.DeleteObject(Index);
end;

procedure TovPropertyObjects.Delete(AnObject: IovObject);
var
  I: Integer;
begin
  if Not Assigned(FObjects) then SelectAll;
  { TODO :     }
  for I := FObjects.Count - 1 downto 0 do
    if FObjects.ObjectId[I] = AnObject.ObjectId then Delete(I);
end;

procedure TovPropertyObjects.DeleteAll;
var
  I: Integer;
begin
  if Not Assigned(FObjects) then SelectAll;
  for I := FObjects.Count - 1 downto 0 do Delete(I);
end;

procedure TovPropertyObjects.Clear;
var
  I: Integer;
begin
  if Not Assigned(FObjects) then SelectAll;
  for I := FObjects.Count - 1 downto 0 do Delete(I);
end;

procedure TovPropertyObjects.FindObjects(FilterExpr, OrderExpr: String;
  Objects: TOIDList);
begin
  with ParentObject do FDatabase.GetChildObjects(ParentProperty.ObjectClassName,
    ParentProperty.PropertyName, ParentProperty.ObjectId, Objects, FilterExpr, OrderExpr);
end;

function TovPropertyObjects.GetParentObject: TovObject;
begin
  Result := ParentProperty.ParentObject;
end;

function TovPropertyObjects.GetParentProperty: IovProperty;
begin
  Result := FParentProperty;
end;

procedure TovPropertyObjects.RemoveAt(Index: Integer);
begin
  if Not Assigned(FObjects) then SelectAll;
  with ParentObject do
    FDatabase.RemoveChildObject(ParentProperty.ObjectClassName, ParentProperty.PropertyName,
      ParentProperty.ObjectId, FObjects.ObjectId[Index], FObjects.RelationId[Index]);
  FObjects.DeleteObject(Index);
end;

procedure TovPropertyObjects.Remove(AnObject: IovObject);
begin
{ TODO :
  if AnObject.ClassName <> ParentProperty.PropertyType then
    raise Exception.Create('   ');
}
  Remove(AnObject.ObjectId);
end;

procedure TovPropertyObjects.Remove(AObjectId: TOID);
var
  I: Integer;
begin
  if Not Assigned(FObjects) then SelectAll;
  for I := FObjects.Count - 1 downto 0 do
    if FObjects.ObjectId[I] = AObjectId then RemoveAt(I);
end;

procedure TovPropertyObjects.Assign(Source: IovPropertyObjects);
var
  I: Integer;
begin
  Clear;
  if FRelation.IsComposition then
    with Source do
      for I := 0 to Count - 1 do Self.Add.Assign(Objects[I])
  else
    with Source do
      for I := 0 to Count - 1 do Self.Add(Objects[I])
end;

function TovPropertyObjects.GetMaxCount: Integer;
begin
  Result := FRelation.TargetMaxCount;
end;

function TovPropertyObjects.GetMinCount: Integer;
begin
  Result := FRelation.TargetMinCount;
end;

procedure TovPropertyObjects.Select(AttributeValues: TStrings; Attributes:
  String; FilterExpr: String = ''; OrderExpr: String = ''; IdAttribute: String
  = '');
var
  QueryInfo: TSelectQueryInfo;
  DataSet: TDataSet;
begin
  with ParentObject do
    DataSet := FObjectClass.Database.OpenChildObjectDataSet('', ParentProperty.ObjectClassName,
      ParentProperty.PropertyName, Attributes, FilterExpr, OrderExpr, '', ParentProperty.ObjectId, QueryInfo);
  try
    LoadDataSetToStrings(DataSet, QueryInfo, AttributeValues, FObjectClass.Database, IdAttribute);
  finally
    QueryInfo.Free;
    DataSet.Free;
  end;
end;

{ TovObjectList }

procedure TovObjectList.Assign(AObjects: IovObjectList);
var
  I: Integer;
begin
  Clear;
  for I := 0 to AObjects.Count - 1 do Add(AObjects[I])
end;

procedure TovObjectList.Add(AObject: IovObject);
begin
  inherited Add(AObject)
end;

function TovObjectList.GetObjects(Index: Integer): IovObject;
begin
  Result := IovObject(Items[Index]);
end;

function TovObjectList.IndexOfObject(AObject: IovObject): Integer;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    if Objects[I].IsSame(AObject) then begin
      Result := I;
      Exit;
    end;
  Result := -1
end;

end.
