unit ovOperations;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs, DB,
  ovTypes, ovModel, ovRelations, ovDatabaseManager, ovIObjects;

type
  TCommonOperations = class(TObject)
  private
    FDatabaseManager: TovDatabaseManager;
    FModel: TDomainModel;
    function HasReferencingObjects(AModelRelation: TModelRelation; AChildObjectId:
      TOID): Boolean; overload;
    procedure GetReferencingObjects(AModelRelation: TModelRelation; AObjectId: TOID;
      AObjects: TOIDList); overload;
  protected
  public
    {{
        ,  
    AttributeName  ClassName,   ParentId
    }
    procedure GetReferencedObjects(AModelRelation: TModelRelation; AParentObjectId: 
      TOID; AChildObjects: TOIDList);
    procedure GetReferencingObjects(AClassName: String; AObjectId: TOID; AObjects:
      TovObjectRefList; Immediate: Boolean); overload;
    function HasReferencingObjects(AModelObject: TModelObject; AObjectId: TOID):
      Boolean; overload;
    constructor Create(ADatabaseManager: TovDatabaseManager; AModel: TDomainModel);
    function GetAncestorObject(AModelObject, AParentObject: TModelObject;
      AObjectId: TOID): TOID;
    function GetObjectRelation(AModelRelation: TModelRelation; AParentObjectId,
      ARelationId: TOID): TOID;
    function GetObjectDataSet(AModelObject: TModelObject; AObjectId: TOID;
      ReadOnly: Boolean): TDataSet;
    procedure UpdateObjectAttribute(AModelObject: TModelObject; AObjectId: TOID;
      AModelAttribute: TModelAttribute; AValue: Variant);
    property DatabaseManager: TovDatabaseManager read FDatabaseManager;
    property Model: TDomainModel read FModel;
  end;

  TCreateObjectEvent = procedure (AModelObject: TModelObject;
    AObjectId: TOID) of object;
  TDeleteObjectEvent = procedure (AModelObject: TModelObject;
    AObjectId: TOID) of object;

  TObjectOperations = class(TCommonOperations)
  private
    FBeforeDeleteObject: TDeleteObjectEvent;
    FAfterDeleteObject: TDeleteObjectEvent;
    FAfterCreateObject: TCreateObjectEvent;
    function IsValidObject(ADatabase: TObject; AClassName: String; AObjectId: TOID;
      var AErrorMsg: String): Boolean;
    function CreateObjectInstance(AModelObject: TModelObject; AClassId: Integer;
      ARealObjectId: TOID): TOID;
    procedure CreateObjectRelations(AModelObject: TModelObject; AObjectId: TOID);
    function CreateParentInstance(AModelRelation: TModelRelation; AClassId: Integer;
      ARealObjectId: TOID; AObjectId: TOID): TOID;
    procedure DeleteObjectInstance(AModelObject: TModelObject; AObjectId: TOID);
    procedure DeleteReferences(AClassName: String; AObjectId: TOID);
    procedure HandleObjectReferences(AModelObject: TModelObject; AObjectId: TOID;
        ForceCascadeDelete: Boolean);
    function CreateObjectInternal(AModelObject: TModelObject): TOID;
  public
    {{
       AChildObjectId,  
    ARelationName  
    AParentObjectId   AClassName.   
      
      .      .

     :

    )           (
      
    ARelationName   AParentObjectId) ,   ,  
     
    )   ARelationName
    )   ,  DeleteObject.
    }
    procedure DeleteChildObject(AModelRelation: TModelRelation; AParentObjectId, 
      AChildObjectId: TOID);
    {{
       .     
      
    .      .

     :

    )           , 
     ,  
     
    )          (  
       
         )
    )   .
    }
    procedure DeleteObject(AModelObject: TModelObject; AObjectId: TOID; DeleteMode:
      TDeleteMode = dmCheckRefs); overload;
    procedure DeleteObjectRelations(AModelObject: TModelObject; AObjectId: TOID);
    procedure DeleteObjects(AModelObject: TModelObject; AObjects: TOIDList);
    procedure UpdateObjectRelation(AModelRelation: TModelRelation; AParentObjectId, 
      ARelationId, ANewChildObjectId: TOID);
    function CreateObject(AModelObject: TModelObject): TOID;
    function CreateObjectRelation(AModelRelation: TModelRelation; AParentObjectId,
      AChildObjectId: TOID; var ARelationId: TOID): TOID;
    procedure DeleteObjectRelation(AModelRelation: TModelRelation; AParentObjectId,
      AChildObjectId, ARelationId: TOID; DeleteWithChildObjects: Boolean);
    procedure GetObjectOwner(ChildObject: TModelObject; ChildObjectId: TOID; var
      ParentObjectId, RelationId: TOID);
    procedure DeleteReferencingObjects(AClassName: String; AObjectId: TOID);
      overload;
    procedure SetObjectOwner(ChildObject: TModelObject; ChildObjectId,
      ParentObjectId, RelationId: TOID);
    property AfterCreateObject: TCreateObjectEvent read FAfterCreateObject write
      FAfterCreateObject;
    property AfterDeleteObject: TDeleteObjectEvent read FAfterDeleteObject write
       FAfterDeleteObject;
    property BeforeDeleteObject: TDeleteObjectEvent read FBeforeDeleteObject write
       FBeforeDeleteObject;
  end;

  TBeforeGenerateDatabaseEvent = procedure (Max: Integer) of object;
  TAfterGenerateDatabaseEvent = procedure(Exception: Exception) of object;
  TGenerateDatabaseProgressEvent = procedure (Step: Integer) of object;

  TDatabaseOperations = class
  private
    FOnAfterGenerateDatabase: TAfterGenerateDatabaseEvent;
    FOnBeforeGenerateDatabase: TBeforeGenerateDatabaseEvent;
    FOnGenerateDatabaseProgress: TGenerateDatabaseProgressEvent;
    procedure CreateRelationTable(ModelRelation: TModelRelation; Database:
      IDatabaseAdapter);
    procedure CreateObjectTable(ModelObject: TModelObject; Database:
      IDatabaseAdapter);
    procedure CopyRelationTable(SourceDatabaseRelation: TModelRelation;
      SourceDatabase: IDatabaseAdapter; TargetDatabaseRelation: TModelRelation;
      TargetDatabase: IDatabaseAdapter);
    function GetObjectTableDefs(ModelObject: TModelObject): TTableDefs;
    procedure CopyObjectTable(SourceModelObject: TModelObject; SourceDatabase:
      IDatabaseAdapter; TargetModelObject: TModelObject; TargetDatabase:
      IDatabaseAdapter);
    function GetRelationTableDefs(ModelRelation: TModelRelation): TTableDefs;
    procedure GenerateDatabaseInternal(SourceDatabase: IDatabaseAdapter;
      SourceModel: TDomainModel; TargetDatabase: IDatabaseAdapter; TargetModel:
      TDomainModel; RebuildAllTables: Boolean);
    procedure ShowGenerateObjectTableError(ModelObject: TModelObject; Exception:
      Exception);
    procedure CopyTable(SourceTableDefs: TTableDefs; SourceDatabase:
      IDatabaseAdapter; TargetTableDefs: TTableDefs; TargetDatabase:
      IDatabaseAdapter);
  public
    procedure GenerateDatabase(ModelDatabase: IDatabaseAdapter; Model:
      TDomainModel; RebuildAllTables: Boolean);
    procedure CopyDatabase(SourceDatabase: IDatabaseAdapter; SourceModel:
      TDomainModel; TargetDatabase: IDatabaseAdapter; TargetModel: TDomainModel);
    procedure GenerateObjectTables(SourceDatabaseObject: TModelObject;
      SourceDatabase: IDatabaseAdapter; TargetDatabaseObject: TModelObject;
      TargetDatabase: IDatabaseAdapter; RebuildAllTables: Boolean);
    property OnAfterGenerateDatabase: TAfterGenerateDatabaseEvent read
      FOnAfterGenerateDatabase write FOnAfterGenerateDatabase;
    property OnBeforeGenerateDatabase: TBeforeGenerateDatabaseEvent read
      FOnBeforeGenerateDatabase write FOnBeforeGenerateDatabase;
    property OnGenerateDatabaseProgress: TGenerateDatabaseProgressEvent read
      FOnGenerateDatabaseProgress write FOnGenerateDatabaseProgress;
  end;

implementation

uses ovConsts, TypInfo, ovUtils, ovDatabase;

{
****************************** TCommonOperations *******************************
}
constructor TCommonOperations.Create(ADatabaseManager: TovDatabaseManager; 
  AModel: TDomainModel);
begin
  inherited Create;
  FDatabaseManager := ADatabaseManager;
  FModel := AModel;
end;

{{
  ,       AParentObjectId
 AClassName   ARelationName.
}
procedure TCommonOperations.GetReferencedObjects(AModelRelation: TModelRelation;
  AParentObjectId: TOID; AChildObjects: TOIDList);
var
  DataSet: TDataSet;
begin
  //   ,      -,
  //   
  if AModelRelation.IsInherited and Not AModelRelation.ParentObject.IsAbstract then begin
    AParentObjectId := GetAncestorObject(AModelRelation.SourceObject, AModelRelation.ParentObject,
      AParentObjectId);
    AModelRelation := Model.GetObjectRelation(AModelRelation.ParentObject.FullName,
      AModelRelation.Name); //   ,    
  end;

  with CreateRelation(AModelRelation, DatabaseManager.QueryManager) do
  try
    DataSet := GetReferencedObjects(AParentObjectId);
    //    nil  ,   
    if DataSet = nil then AChildObjects.Clear else
    try
      if DataSet.Fields.Count > 1 then
        AChildObjects.LoadFromDataSet(DataSet, DataSet.Fields[0], DataSet.Fields[1])
      else
        AChildObjects.LoadFromDataSet(DataSet, DataSet.Fields[0], nil)
    finally
      DataSet.Free
    end;
  finally
    Free
  end;
end;

{{
    ,     AObjectId
 AClassName.       AObjects.

  Immediate  True,     
  ,       
       ..
}
procedure TCommonOperations.GetReferencingObjects(AClassName: String;
  AObjectId: TOID; AObjects: TovObjectRefList; Immediate: Boolean);
var
  J: Integer;
  I, StartCount: Integer;
  ObjectList: TOIDList;
begin
  ObjectList := TOIDList.Create;
  try
    StartCount := AObjects.Count;
    with Model do
    for I := 0 to RelationCount - 1 do
      with Relations[I] do
      if Not IsGeneralization and Not SourceObject.IsAbstract and Not IsInherited
         and (TargetObjectName = AClassName)
//         and (SourceObjectName <> AClassName) //    
      then
         begin
           GetReferencingObjects(Relations[I], AObjectId, ObjectList);
           for J := 0 to ObjectList.Count - 1 do
           // ,     
             if AObjects.IndexOf(SourceObject.FullName, ObjectList[J]) < 0 then
               AObjects.Add(SourceObject.FullName, ObjectList[J])
         end;

    //  ,    
    if Not Immediate then with AObjects do
      //        
      // (      )
      for I := StartCount to Count - 1 do
        with Objects[I] do GetReferencingObjects(ClassName, ObjectId, AObjects, False);
  finally
    ObjectList.Free
  end;
end;

{{
    AClassName ,    
AObjectId    ARelationName.
}
procedure TCommonOperations.GetReferencingObjects(AModelRelation: 
  TModelRelation; AObjectId: TOID; AObjects: TOIDList);
var
  DataSet: TDataSet;
begin
  with CreateRelation(AModelRelation, DatabaseManager.QueryManager) do
  try
    DataSet := GetReferencingObjects(AObjectId);
    try
      if DataSet.Fields.Count > 1 then
        AObjects.LoadFromDataSet(DataSet, DataSet.Fields[0], DataSet.Fields[1])
      else
        AObjects.LoadFromDataSet(DataSet, DataSet.Fields[0], nil);
    finally
      DataSet.Free
    end;
  finally
    Free
  end;
end;

{{
      AObjectId  AClassName 
-     .
}
function TCommonOperations.HasReferencingObjects(AModelObject: TModelObject;
  AObjectId: TOID): Boolean;
var
  I: Integer;
  AObjects: TovObjectRefList;
begin
  Result := False;
  with Model do
  for I := 0 to RelationCount - 1 do
    with Relations[I] do
    if Not IsGeneralization and Not SourceObject.IsAbstract and Not IsInherited
      and (TargetObjectName = AModelObject.FullName)
//      and (SourceObjectName <> AModelObject.FullName) //    
      and HasReferencingObjects(Relations[I], AObjectId)
    then begin
      Result := True;
      Break;
    end;

  if Result then
  begin
    //      
    AObjects := TovObjectRefList.Create;
    try
      GetReferencingObjects(AModelObject.FullName, AObjectId, AObjects, True);
      for I := 0 to AObjects.Count - 1 do
        if (AObjects[I].ClassName <> AModelObject.FullName) or
           (AObjects[I].ObjectId <> AObjectId)
        then Exit;
      //        ,  -
      Result := False;
    finally
      AObjects.Free;
    end;
  end;
end;

{{
      AChildObjectId   AClassName
  ARelationName.
}
function TCommonOperations.HasReferencingObjects(AModelRelation: TModelRelation;
  AChildObjectId: TOID): Boolean;
begin
  with CreateRelation(AModelRelation, DatabaseManager.QueryManager) do
  try
    Result := HasReferencingObjects(AChildObjectId);
  finally
    Free
  end;
end;

function TCommonOperations.GetObjectRelation(AModelRelation: TModelRelation;
  AParentObjectId, ARelationId: TOID): TOID;
begin
  //   ,      -,
  //   
  if AModelRelation.IsInherited and Not AModelRelation.ParentObject.IsAbstract then begin
    AParentObjectId := GetAncestorObject(AModelRelation.SourceObject,
      AModelRelation.ParentObject, AParentObjectId);
    AModelRelation := Model.GetObjectRelation(AModelRelation.ParentObject.FullName,
      AModelRelation.Name); //   ,    
  end;

  with CreateRelation(AModelRelation, DatabaseManager.QueryManager) do
  try
    Result := GetRelation(AParentObjectId, ARelationId);
  finally
    Free
  end;
end;

{{
   AParentObject,  
AObjectId    AModelObject. .. AParentObject 
    AModelObject  AObjectId  
  AModelObject.
}
function TCommonOperations.GetAncestorObject(AModelObject, AParentObject:
  TModelObject; AObjectId: TOID): TOID;
var
  I: Integer;
  Generalization: TModelRelation;
begin
  //   ,   -, 
  //      
  Generalization := nil;
  with AModelObject do
    for I := 0 to AncestorCount - 1 do
      if AncestorRelations[I].TargetObject = AParentObject then begin
        Generalization := AncestorRelations[I];
        Break;
      end;
  if Not Assigned(Generalization) then
    raise Exception.CreateFmt(' "%s"     "%s"',
      [AParentObject.FullName, AModelObject.FullName]);

  with CreateRelation(Generalization, DatabaseManager.QueryManager) as TGeneralizationRelation do
  try
    Result := GetAncestor(AModelObject, AObjectId, Generalization);
  finally
    Free
  end;
end;

{{
  ,   ,   ,
     AModelObject   AObjectId.
}
function TCommonOperations.GetObjectDataSet(AModelObject: TModelObject;
  AObjectId: TOID; ReadOnly: Boolean): TDataSet;
begin
  Result := DatabaseManager.QueryManager.SelectRecord(AModelObject.TableName,
    ObjectIdField, AObjectId, ReadOnly)
end;

procedure TCommonOperations.UpdateObjectAttribute(AModelObject: TModelObject;
  AObjectId: TOID; AModelAttribute: TModelAttribute; AValue: Variant);
var
  AFieldType: TFieldType;
  AFieldSize: Integer;
begin
  AModelAttribute.GetFieldDef(AFieldType, AFieldSize);
  DatabaseManager.QueryManager.UpdateRecord(AModelObject.TableName,
    ObjectIdField, AObjectId,
      MakeFieldValues([AModelAttribute.FieldName, AFieldType, AFieldSize, AValue]))
end;

{ **************************************************************************** }
{                                                              }
{ **************************************************************************** }

{{
         
.      ,  
 .
}
function TObjectOperations.CreateObject(AModelObject: TModelObject): TOID;
begin
  Result := CreateObjectInternal(AModelObject);
  AfterCreateObject(AModelObject, Result);
end;

function TObjectOperations.CreateObjectInternal(AModelObject: TModelObject):
  TOID;
begin
  Result := CreateObjectInstance(AModelObject, AModelObject.Id, 0);  //    (   )
  CreateObjectRelations(AModelObject, Result);          //     (  )
end;

{{
    (  , ,  
  ,          ).
}
function TObjectOperations.CreateObjectInstance(AModelObject: TModelObject;
  AClassId: Integer; ARealObjectId: TOID): TOID;
var
  I: Integer;
begin
  with AModelObject do
  begin
    if ARealObjectId = 0 then //        
    begin
      Result := DatabaseManager.QueryManager.AddRecord(TableName, ObjectIdField,
        MakeIntFieldValues([ClassIdField, AClassId]));
      ARealObjectId := Result;
    end
    else //       (..   ,   )
      Result := DatabaseManager.QueryManager.AddRecord(TableName, ObjectIdField,
        MakeIntFieldValues([ClassIdField, AClassId, ChildIdField, ARealObjectId]));

    //      ,     
    //  
    for I := 0 to ParentCount - 1 do
      if Not ParentRelations[I].TargetObject.IsAbstract then
        CreateParentInstance(ParentRelations[I], AClassId, ARealObjectId, Result);
  end
end;

{{
   ,   CreateObjectInstance,     
  AObjectId.
}
function TObjectOperations.CreateParentInstance(AModelRelation: TModelRelation;
  AClassId: Integer; ARealObjectId: TOID; AObjectId: TOID): TOID;
var
  Relation: TRelation;
begin
  Result := CreateObjectInstance(AModelRelation.TargetObject, AClassId, ARealObjectId);
  //   TGeneralizationRelation
  Relation := CreateRelation(AModelRelation, DatabaseManager.QueryManager);
  try
    Result := Relation.CreateRelation(AObjectId, Result)
  finally
    Relation.Free
  end;
end;

{{
         
AObjectId  AClassName.     
,    .
}
procedure TObjectOperations.CreateObjectRelations(AModelObject: TModelObject;
  AObjectId: TOID);
var
  Relation: TModelRelation;
  I, J: Integer;
  RelationId: TOID;
begin
  with AModelObject do
  begin
    //           0
    for I := 0 to RelationCount - 1 do
    begin
      Relation := Relations[I];
      if Relation.IsComposition then
        for J := 1 to Relation.TargetMinCount do
          CreateObjectRelation(Relation, AObjectId, 0, RelationId)
    end;
  end
end;

{{
   AModelRelation   AParentObjectId 
AChildObjectId.  AChildObjectId  0,    (
AModelRelation.TargetObject)  .
    (  AChildObjectId  
,  AChildObjectId  0).   ARelationId 
  .
}
function TObjectOperations.CreateObjectRelation(AModelRelation: TModelRelation;
  AParentObjectId, AChildObjectId: TOID; var ARelationId: TOID): TOID;
var
  Relation: TRelation;
begin
  //   ,      -,
  //   
  if AModelRelation.IsInherited and Not AModelRelation.ParentObject.IsAbstract then begin
    AParentObjectId := GetAncestorObject(AModelRelation.SourceObject,
      AModelRelation.ParentObject, AParentObjectId);
    AModelRelation := Model.GetObjectRelation(AModelRelation.ParentObject.FullName,
      AModelRelation.Name); //   ,    
  end;

  Relation := CreateRelation(AModelRelation, DatabaseManager.QueryManager);
  with Relation do
  try
    if AChildObjectId = 0 then
      AChildObjectId := CreateObjectInternal(AModelRelation.TargetObject);

    if AChildObjectId > 0 then
      ARelationId := CreateRelation(AParentObjectId, AChildObjectId)
    else
      ARelationId := 0;
    Result := AChildObjectId;

    //       
    if AModelRelation.RelationType = rtComposition then
      AfterCreateObject(AModelRelation.TargetObject, AChildObjectId);
  finally
    Free
  end;
end;

{ **************************************************************************** }
{                                                              }
{ **************************************************************************** }

procedure TObjectOperations.DeleteObjects(AModelObject: TModelObject; AObjects:
  TOIDList);
var
  I: Integer;
begin
  for I := 0 to AObjects.Count - 1 do
    DeleteObject(AModelObject, AObjects[I])
end;

{{
   ,      
.  DeleteMode      
    

  :

)          (  
            );
)          ;
)    .
}
procedure TObjectOperations.DeleteObject(AModelObject: TModelObject; AObjectId:
  TOID; DeleteMode: TDeleteMode = dmCheckRefs);
begin
  { TODO :  DeleteMode  ,     }

  {      }
  BeforeDeleteObject(AModelObject, AObjectId);

  {     -    
        }
  DeleteObjectRelations(AModelObject, AObjectId);

  {          
      }
  HandleObjectReferences(AModelObject, AObjectId, DeleteMode = dmDeleteRefsObjects);

  {    }
  DeleteObjectInstance(AModelObject, AObjectId);

  {      }
  AfterDeleteObject(AModelObject, AObjectId);
end;

{{
       .

   :
-       ,   
 ,      

-     ,      
  :
        -   , ..    
        -  
        -  ( )  ,    
        -  
}
procedure TObjectOperations.HandleObjectReferences(AModelObject: TModelObject;
    AObjectId: TOID; ForceCascadeDelete: Boolean);

  procedure HandleParents(ARelation: TModelRelation; DeleteParents: Boolean);
  var
    J: Integer;
    OIDList: TOIDList;
  begin
    OIDList := TOIDList.Create(False);
    try
      GetReferencingObjects(ARelation, AObjectId, OIDList);
      if DeleteParents then
        {  ,     }
        for J := 0 to OIDList.Count - 1 do
          Self.DeleteObject(ARelation.SourceObject, OIDList[J])
      else
        {      }
        for J := 0 to OIDList.Count - 1 do
          Self.DeleteObjectRelation(ARelation, OIDList[J],
            AObjectId, 0 {   }, False)
    finally
      OIDList.Free;
    end;
  end;

var
  I: Integer;
begin
  with Model do
  for I := 0 to RelationCount - 1 do
    with Relations[I] do
    if Not IsGeneralization and Not SourceObject.IsAbstract
      and Not IsInherited and (TargetObjectName = AModelObject.FullName)
    then
    begin
      if ForceCascadeDelete then
        HandleParents(Relations[I], True)
      else
      case Relations[I].DeleteMode of

        ovModel.dmIgnoreRefs:
          {    };

        ovModel.dmBlockDelete:
          {   ,      }
          if HasReferencingObjects(Relations[I], AObjectId) then
            raise EDeleteObjectError.Create(Format(
              '  ,        (%s, %d)',
              [AModelObject.FullName, AObjectId]), AModelObject, AObjectId);

        ovModel.dmDeleteRefs, ovModel.dmDeleteParents:
          {   ,     }
          HandleParents(Relations[I], Relations[I].DeleteMode = ovModel.dmDeleteParents);
      end;
    end;
end;

{{
    (  , ,  
  ,       
  ).
}
procedure TObjectOperations.DeleteObjectInstance(AModelObject: TModelObject;
  AObjectId: TOID);
var
  I: Integer;
  Generalization: TModelRelation;
begin
  with AModelObject do
  begin
    //     ,     
    //  
    for I := 0 to ParentCount - 1 do
    begin
      //   ,     
      //   ,      
      Generalization := ParentRelations[I];
      DeleteObject(Generalization.TargetObject,
        GetObjectRelation(Generalization, AObjectId, 0))
    end;
    DatabaseManager.QueryManager.DeleteRecord(TableName, ObjectIdField, AObjectId);
  end
end;

{{
        
AObjectId  AClassName.     
,    .
}
procedure TObjectOperations.DeleteObjectRelations(AModelObject: TModelObject;
  AObjectId: TOID);
var
  I, J: Integer;
  ChildObjects: TOIDList;
begin
  ChildObjects := TOIDList.Create;
  try
    //         
    // (     )
    with AModelObject do
    for I := 0 to RelationCount - 1 do
    begin
      Self.GetReferencedObjects(Relations[I], AObjectId, ChildObjects);
      for J := 0 to ChildObjects.Count - 1 do
        DeleteObjectRelation(Relations[I], AObjectId,
          ChildObjects.ObjectId[J], ChildObjects.RelationId[J], True);
    end;
  finally
    ChildObjects.Free
  end;
end;

procedure TObjectOperations.DeleteObjectRelation(AModelRelation: TModelRelation;
  AParentObjectId, AChildObjectId, ARelationId: TOID; DeleteWithChildObjects:
  Boolean);
var
  Relation: TRelation;
begin
  //   ,      -,
  //   
  if AModelRelation.IsInherited and Not AModelRelation.ParentObject.IsAbstract then begin
    AParentObjectId := GetAncestorObject(AModelRelation.SourceObject,
      AModelRelation.ParentObject, AParentObjectId);
    AModelRelation := Model.GetObjectRelation(AModelRelation.ParentObject.FullName,
      AModelRelation.Name); //   ,    
  end;

  Relation := CreateRelation(AModelRelation, DatabaseManager.QueryManager);
  with Relation do
  try
    DeleteRelation(AParentObjectId, AChildObjectId, ARelationId);

    if DeleteWithChildObjects and (AModelRelation.RelationType = rtComposition)
    then
      DeleteObject(AModelRelation.TargetObject, AChildObjectId)
  finally
    Free
  end;
end;

{{
   AChildObjectId,  
ARelationName  
AParentObjectId   AClassName.   
  
  .      .

 :

)           (
  
ARelationName   AParentObjectId) ,   ,  
 
)   ARelationName
)   ,  DeleteObject.
}
procedure TObjectOperations.DeleteChildObject(AModelRelation: TModelRelation;
  AParentObjectId, AChildObjectId: TOID);
var
  I: Integer;
  ChildObjects: TOIDList;
begin
  ChildObjects := TOIDList.Create;
  try
    GetReferencedObjects(AModelRelation, AParentObjectId, ChildObjects);
    for I := 0 to ChildObjects.Count - 1 do
      if ChildObjects.ObjectId[I] = AChildObjectId then
        DeleteObjectRelation(AModelRelation, AParentObjectId,
          ChildObjects.ObjectId[I], ChildObjects.RelationId[I], False);
  finally
    ChildObjects.Free
  end;
  DeleteObject(AModelRelation.TargetObject, AChildObjectId);
end;

{{
   ,      
    ..
}
procedure TObjectOperations.DeleteReferencingObjects(AClassName: String;
  AObjectId: TOID);
var
  I: Integer;
  Objects: TovObjectRefList;
begin
  Objects := TovObjectRefList.Create;
  try
    //     ,    
    //     
    GetReferencingObjects(AClassName, AObjectId, Objects, False);
    //     ,     
    //    , ..     
    for I := 0 to Objects.Count - 1 do
      with Objects[I] do
        DeleteObject(Model.GetObject(ClassName), ObjectId, dmIgnoreRefs);
  finally
    Objects.Free;
  end;
end;

{{
      .
}
procedure TObjectOperations.DeleteReferences(AClassName: String;
  AObjectId: TOID);
var
  I, J: Integer;
  ObjectList: TOIDList;
begin
  ObjectList := TOIDList.Create;
  try
    with Model do
    for I := 0 to RelationCount - 1 do
      with Relations[I] do
      if Not IsGeneralization and Not IsInherited
         and (TargetObjectName = AClassName)
         and (SourceObjectName <> AClassName) //    
      then
         begin
           GetReferencingObjects(Relations[I], AObjectId, ObjectList);
           for J := 0 to ObjectList.Count - 1 do
             DeleteObjectRelation(Relations[I], ObjectList[J], AObjectId,
               ObjectList.RelationId[J], False);
         end;
  finally
    ObjectList.Free
  end;
end;

{ **************************************************************************** }
{                                                           }
{ **************************************************************************** }

{{
   ARelationName   AParentObjectId  AClassName.
}
procedure TObjectOperations.UpdateObjectRelation(AModelRelation: TModelRelation;
  AParentObjectId, ARelationId, ANewChildObjectId: TOID);
begin
  //   ,      -,
  //   
  if AModelRelation.IsInherited and Not AModelRelation.ParentObject.IsAbstract then begin
    AParentObjectId := GetAncestorObject(AModelRelation.SourceObject,
      AModelRelation.ParentObject, AParentObjectId);
    AModelRelation := Model.GetObjectRelation(AModelRelation.ParentObject.FullName,
      AModelRelation.Name); //   ,    
  end;

  with CreateRelation(AModelRelation, DatabaseManager.QueryManager) do
  try
    UpdateRelation(AParentObjectId, ARelationId, ANewChildObjectId);
  finally
    Free
  end;
end;

{{
    ParentObjectId  
 RelationId     ChildObjectId 
ChildObject.
}
procedure TObjectOperations.GetObjectOwner(ChildObject: TModelObject;
  ChildObjectId: TOID; var ParentObjectId, RelationId: TOID);
begin
  with TCompositionRelation.Create(DatabaseManager.QueryManager, ChildObject) do
  try
    GetOwner(ChildObjectId, ParentObjectId, RelationId);
  finally
    Free
  end;
end;

{{
    ChildObjectId  ChildObject  
     ParentObjectId    
  RelationId.
}
procedure TObjectOperations.SetObjectOwner(ChildObject: TModelObject;
  ChildObjectId, ParentObjectId, RelationId: TOID);
begin
  with TCompositionRelation.Create(DatabaseManager.QueryManager, ChildObject) do
  try
    SetOwner(ChildObjectId, ParentObjectId, RelationId);
  finally
    Free
  end;
end;

{ **************************************************************************** }
{                                                     }
{ **************************************************************************** }

{{
     (  ).
         True.
}
function TObjectOperations.IsValidObject(ADatabase: TObject; AClassName: String;
  AObjectId: TOID; var AErrorMsg: String): Boolean;
var
  C: Integer;
  R: TModelRelation;
  I: Integer;
  O: TModelObject;
  Objects: TOIDList;
begin
  Result := True;
  AErrorMsg := '';

  //         
  //  
  Objects := TOIDList.Create;
  try
    O := Model.GetObject(AClassName);
    for I := 0 to O.RelationCount - 1 do
    begin
      R := O.Relations[I];
      TovDatabase(ADatabase).GetChildObjects(AClassName, R.Name, AObjectId, Objects);
      C := Objects.Count;
      if (C < R.TargetMinCount) or (C > R.TargetMaxCount)
      then begin
        Result := False;
        AErrorMsg := Format('     "%s"  "%s"',
          [R.Name, O.Name]);
        Break;
      end
    end;
  finally
    Objects.Free
  end;
end;

{
****************************** TDatabaseOperations *****************************
}

{{
     .
}
procedure TDatabaseOperations.GenerateDatabase(ModelDatabase: IDatabaseAdapter;
  Model: TDomainModel; RebuildAllTables: Boolean);
begin
  GenerateDatabaseInternal(nil, nil, ModelDatabase, Model, RebuildAllTables);
end;

procedure TDatabaseOperations.CopyDatabase(SourceDatabase: IDatabaseAdapter;
  SourceModel: TDomainModel; TargetDatabase: IDatabaseAdapter; TargetModel:
  TDomainModel);
begin
  GenerateDatabaseInternal(SourceDatabase, SourceModel, TargetDatabase, TargetModel, True);
end;

{{
      TargetModel  TargetDatabase.
   SourceDatabase,     
    SourceDatabase (..  
  SourceDatabase  TargetDatabase).
}
procedure TDatabaseOperations.GenerateDatabaseInternal(SourceDatabase:
  IDatabaseAdapter; SourceModel: TDomainModel; TargetDatabase:
  IDatabaseAdapter; TargetModel: TDomainModel; RebuildAllTables: Boolean);
var
  I: Integer;
  SourceObject: TModelObject;
begin
  with TargetModel do
  begin
    if Assigned(FOnBeforeGenerateDatabase) then FOnBeforeGenerateDatabase(ObjectCount);
    try
      for I := 0 to ObjectCount - 1 do
        if Not Objects[I].IsAbstract then
        begin
          if Assigned(SourceModel) then
            SourceObject := SourceModel.GetObject(Objects[I].FullName)
          else
            SourceObject := nil;
          GenerateObjectTables(SourceObject, SourceDatabase, Objects[I], TargetDatabase,
            RebuildAllTables);
          if Assigned(FOnGenerateDatabaseProgress) then FOnGenerateDatabaseProgress(I);
        end;
      if Assigned(FOnAfterGenerateDatabase) then FOnAfterGenerateDatabase(nil);
    except
      on E: Exception do
      begin
        if Assigned(FOnAfterGenerateDatabase) then FOnAfterGenerateDatabase(E);
        raise;
      end;
    end;
  end;
end;

{{
      TargetDatabaseObject,  
      .   
,   RebuildAllTables = False,    .
}
procedure TDatabaseOperations.GenerateObjectTables(SourceDatabaseObject:
  TModelObject; SourceDatabase: IDatabaseAdapter; TargetDatabaseObject:
  TModelObject; TargetDatabase: IDatabaseAdapter; RebuildAllTables: Boolean);
var
  I: Integer;
begin
  try
    if RebuildAllTables or Not TargetDatabase.TableExists(TargetDatabaseObject.TableName)
    then
      if Assigned(SourceDatabase) then
        CopyObjectTable(SourceDatabaseObject, SourceDatabase, TargetDatabaseObject, TargetDatabase)
      else
        CreateObjectTable(TargetDatabaseObject, TargetDatabase);

    for I := 0 to TargetDatabaseObject.RelationCount - 1 do
      with TargetDatabaseObject.Relations[I] do
        //       
        if IsAssociation and Not IsReference and Not IsInherited and
           (RebuildAllTables or
           Not TargetDatabase.TableExists(TableName))
        then
        if Assigned(SourceDatabase) then
          CopyRelationTable(SourceDatabaseObject.GetRelation(TargetDatabaseObject.Relations[I].Name),
            SourceDatabase, TargetDatabaseObject.Relations[I], TargetDatabase)
        else
          CreateRelationTable(TargetDatabaseObject.Relations[I], TargetDatabase);
  except
    on E: Exception do
      ShowGenerateObjectTableError(TargetDatabaseObject, E);
  end;
end;

procedure TDatabaseOperations.ShowGenerateObjectTableError(ModelObject:
  TModelObject; Exception: Exception);
begin
  if Application.MessageBox(
    PChar(Format('    "%s" '#13'( %s). '#13+
    '?',
      [ModelObject.Name, Exception.Message])), '  ',
      MB_ICONERROR or MB_YESNO) = idNo
  then
  raise Exception.CreateFmt('    "%s" ( %s)',
    [ModelObject.Name, Exception.Message]);
end;

procedure TDatabaseOperations.CreateRelationTable(ModelRelation:
  TModelRelation; Database: IDatabaseAdapter);
var
  TableDefs: TTableDefs;
begin
  TableDefs := GetRelationTableDefs(ModelRelation);
  try
    with TableDefs do
      Database.CreateTable(TableName, FieldDefs, IndexDefs)
  finally
    TableDefs.Free
  end;
end;

procedure TDatabaseOperations.CopyRelationTable(SourceDatabaseRelation:
  TModelRelation; SourceDatabase: IDatabaseAdapter; TargetDatabaseRelation:
  TModelRelation; TargetDatabase: IDatabaseAdapter);
var
  SourceTableDefs, TargetTableDefs: TTableDefs;
begin
  SourceTableDefs := nil;
  TargetTableDefs := nil;
  try
    SourceTableDefs := GetRelationTableDefs(SourceDatabaseRelation);
    TargetTableDefs := GetRelationTableDefs(TargetDatabaseRelation);
    CopyTable(SourceTableDefs, SourceDatabase, TargetTableDefs, TargetDatabase);
  finally
    TargetTableDefs.Free;
    SourceTableDefs.Free;
  end;
end;

{{
      ModelObject.
}
procedure TDatabaseOperations.CreateObjectTable(ModelObject: TModelObject;
  Database: IDatabaseAdapter);
var
  TableDefs: TTableDefs;
begin
  TableDefs := GetObjectTableDefs(ModelObject);
  try
    with TableDefs do
      Database.CreateTable(TableName, FieldDefs, IndexDefs)
  finally
    TableDefs.Free
  end;
end;

{{
      SourceModelObject, 
 SourceDatabase     TargetModelObject , 
 TargetDatabase.        ,
      (..   , ..
       ).
}
procedure TDatabaseOperations.CopyObjectTable(SourceModelObject: TModelObject;
  SourceDatabase: IDatabaseAdapter; TargetModelObject: TModelObject;
  TargetDatabase: IDatabaseAdapter);
var
  SourceTableDefs, TargetTableDefs: TTableDefs;
begin
  SourceTableDefs := nil;
  TargetTableDefs := nil;
  try
    SourceTableDefs := GetObjectTableDefs(SourceModelObject);
    TargetTableDefs := GetObjectTableDefs(TargetModelObject);
    CopyTable(SourceTableDefs, SourceDatabase, TargetTableDefs, TargetDatabase);
  finally
    TargetTableDefs.Free;
    SourceTableDefs.Free;
  end;
end;

procedure TDatabaseOperations.CopyTable(SourceTableDefs: TTableDefs;
  SourceDatabase: IDatabaseAdapter; TargetTableDefs: TTableDefs;
  TargetDatabase: IDatabaseAdapter);
var
  SourceTable: TDataSet;
  SourceTableFieldNames: TStrings;
  I: Integer;
begin
  SourceTable := SourceDatabase.OpenTable(SourceTableDefs.TableName, True);
  try
    SourceTableFieldNames := TStringList.Create;
    try
      for I := 0 to SourceTableDefs.FieldDefs.Count - 1 do
        SourceTableFieldNames.Add(SourceTableDefs.FieldDefs[I].Name);
      with TargetTableDefs do
        TargetDatabase.CloneTable(SourceTable, SourceTableFieldNames, TableName, FieldDefs, IndexDefs)
    finally
      SourceTableFieldNames.Free;
    end;
  finally
    SourceTable.Free
  end;
end;

{{
   ,    
   ().
}
function TDatabaseOperations.GetObjectTableDefs(ModelObject: TModelObject):
  TTableDefs;

  {{
         .
  }
  procedure GetIndexedFields(ModelObject: TModelObject;
    IndexedFields: TStrings);
  var
    I, J: Integer;
    ModelAttribute: TModelAttribute;
    ModelRelation: TModelRelation;
    AttributeList: TStringList;
    Fields: String;
  begin
    AttributeList := TStringList.Create;
    try
      IndexedFields.Clear;
      with ModelObject do
      for I := 0 to IndexCount - 1 do
      begin
        //       IndexedFields  
        //    
        AttributesToList(Indexes[I].Attributes, AttributeList);
        Fields := '';
        for J := 0 to AttributeList.Count - 1  do
        begin
          ModelAttribute := FindAttribute(AttributeList[J]);
          if Assigned(ModelAttribute) then
            Fields := CombineSubStr(Fields, ';', ModelAttribute.FieldName) else
          begin
            ModelRelation := ModelObject.FindRelation(AttributeList[J]);
            if Assigned(ModelRelation) and ModelRelation.IsReference then
              Fields := CombineSubStr(Fields, ';', ModelRelation.FieldName)
            else
              Exception.Create('        :'#13+
                Indexes[I].Attributes);
          end;
        end;
        IndexedFields.Add(Fields);
      end;
    finally
      AttributeList.Free;
    end;
  end;

var
  AFieldName: String;
  FieldType: TFieldType;
  I, FieldSize: Integer;
  AObjects: TStringList;
  R: TModelRelation;
  IndexedFields: TStrings;
begin
  Result := TTableDefs.Create(ModelObject.TableName);
  with Result do
  try
    //   
    AddField(ObjectIdField, ftAutoInc);
    AddField(OwnerIdField, ftInteger);
    AddField(RelationIdField, ftInteger);
    AddField(ClassIdField, ftInteger);

    //    
    for I := 0 to ModelObject.AttributeCount - 1 do
      //   ,       
      //  ,  ,      - 
      with ModelObject.Attributes[I] do
      if (Not IsInherited or ParentObject.IsAbstract) and Persistent then
      begin
        ModelObject.Attributes[I].GetFieldDef(FieldType, FieldSize);
        AFieldName := ModelObject.Attributes[I].FieldName;
        AddField(AFieldName, FieldType, FieldSize);
      end;

    //      
    for I := 0 to ModelObject.RelationCount - 1 do
      with ModelObject.Relations[I] do
      if (Not IsInherited or ParentObject.IsAbstract) and IsReference and Not Derived then
      begin
        AFieldName := FieldName;
        FieldType := ftInteger;
        FieldSize := 0;
        AddField(AFieldName, FieldType, FieldSize);
      end;

    //      
    with ModelObject do
    begin
      AObjects := TStringList.Create;
      try
        GetReferencingObjects(AObjects, False);
        for I := 0 to AObjects.Count - 1 do
          if AObjects[I] <> '' then
          begin
            R := TModelObject(AObjects.Objects[I]).GetRelation(AObjects[I]);
            if R.IsAggregation then AddField(R.FieldName, ftInteger);
          end;
      finally
        AObjects.Free;
      end;
    end;

    //      
    with ModelObject do
    begin
      if ChildCount > 0 then AddField(ChildIdField, ftInteger);
      for I := 0 to AncestorCount - 1 do
        if Not AncestorRelations[I].TargetObject.IsAbstract then
          AddField(AncestorRelations[I].FieldName, ftInteger);
    end;

    //    
    AddIndex('ovPrimaryIdx', ObjectIdField, [ixPrimary, ixUnique]);
    AddIndex('ovOwnerIdx', OwnerIdField+';'+RelationIdField, []);

    //   
    IndexedFields := TStringList.Create;
    try
      //     (  i- 
      //      IndexedFields    )
      GetIndexedFields(ModelObject, IndexedFields);
      for I := 0 to ModelObject.IndexCount - 1 do
        AddIndex('ovUserIndex'+IntToStr(I+1), IndexedFields[I], ModelObject.Indexes[I].Options);
    finally
      IndexedFields.Free;
    end;

  except
    Free; raise;
  end;
end;

{{
   ,    
 .
}
function TDatabaseOperations.GetRelationTableDefs(ModelRelation:
  TModelRelation): TTableDefs;
begin
  Result := TTableDefs.Create(ModelRelation.TableName);
  with Result do
  try
    //  
    AddField(RelationInstanceIdField, ftAutoInc);
    AddField(OwnerIdField, ftInteger);
    AddField(ObjectIdField, ftInteger);
    AddField(RelationIdField, ftInteger);

    //  
    AddIndex('Primary', RelationInstanceIdField, [ixPrimary]);
    AddIndex('IDS_INDEX', OwnerIdField+';'+ObjectIdField+';'+RelationIdField, []);
  except
    Free; raise;
  end;
end;

end.
