unit ovProgress;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls;

type
  TfrmProgress = class(TForm)
    Panel1: TPanel;
    MessageLabel: TLabel;
    Bar: TProgressBar;
    CancelButton: TButton;
    procedure WMNCHitTest(var Message :TWMNCHitTest); message WM_NCHITTEST;
    procedure FormCreate(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FActiveForm: TForm;
    FTerminated: Boolean;
    FPosition: Integer;
    FMessage: String;
    FProgress: Boolean;
    procedure SetPosition(Value: Integer);
    procedure SetMessage(const Value: String);
    procedure SetTerminated(Value: Boolean);
    procedure SetProgress(Value: Boolean);
  public
    procedure Reset;
    procedure Execute(MaxValue: Integer; const Msg: String;
      Canceled: Boolean; Progress: Boolean);
    procedure Step;
    property Terminated: Boolean read FTerminated write SetTerminated;
    property Position: Integer read FPosition write SetPosition;
    property ShowProgress: Boolean read FProgress write SetProgress;
    property Message: String read FMessage write SetMessage;
  end;

  procedure ShowProgress(ProgressMax: Integer; ProgressCaption: String);
  function SetProgress(ProgressValue: Integer): Boolean;
  function ProgressStep: Boolean;
  function ProgressVisible: Boolean;
  procedure HideProgress;

implementation

{$R *.DFM}

var
  frmProgress: TfrmProgress;

procedure ShowProgress(ProgressMax: Integer; ProgressCaption: String);
begin
  if Not Assigned(frmProgress) then
    frmProgress := TfrmProgress.Create(Application);
  frmProgress.Execute(ProgressMax, ProgressCaption, True, True);
end;

procedure HideProgress;
begin
  FreeAndNil(frmProgress)
end;

function ProgressStep: Boolean;
begin
  if Not Assigned(frmProgress) then raise Exception.Create('    ');
  frmProgress.Step;
  Result := Not frmProgress.Terminated;
end;

function SetProgress(ProgressValue: Integer): Boolean;
begin
  if Assigned(frmProgress) then
  begin
    frmProgress.Position := ProgressValue;
    Result := Not frmProgress.Terminated;
  end else
    Result := False;
end;

function ProgressVisible: Boolean;
begin
  Result := Assigned(frmProgress)
end;

{ TfrmProgress }

procedure TfrmProgress.WMNCHitTest(var Message: TWMNCHitTest);
begin
  inherited;
  if Message.Result = htClient then Message.Result := htCaption;
end;

procedure TfrmProgress.FormCreate(Sender: TObject);
begin
  FActiveForm := Screen.ActiveForm;
  if FActiveForm <> nil then
    FActiveForm.Enabled := False;
  Bar.Min := 0;
  Bar.Max := 100;
  Position := 0;
end;

procedure TfrmProgress.FormDestroy(Sender: TObject);
begin
  if FActiveForm <> nil then FActiveForm.Enabled := True;
end;

procedure TfrmProgress.FormHide(Sender: TObject);
begin
  if FActiveForm <> nil then FActiveForm.Enabled := True;
end;

procedure TfrmProgress.Reset;
begin
  Position := 0;
end;

procedure TfrmProgress.SetPosition(Value: Integer);
begin
  FPosition := Value;
  Bar.Position := Value;
  BringToFront;
  Application.ProcessMessages;
end;

procedure TfrmProgress.Execute(MaxValue: Integer; const Msg: String;
  Canceled: Boolean; Progress: Boolean);
begin
  Terminated := False;
  CancelButton.Visible := Canceled;
  ShowProgress := Progress;
  Bar.Min := 0;
  Reset;
  Bar.Max := MaxValue;
  Message := Msg;
  Show;
  Application.ProcessMessages;
end;

procedure TfrmProgress.Step;
begin
  if (Position < Bar.Max) and (Position >= Bar.Min) then
    Position := Position + 1;
end;

procedure TfrmProgress.SetMessage(const Value: String);
begin
  FMessage := Value;
  MessageLabel.Caption := Value;
  MessageLabel.Refresh;
end;

procedure TfrmProgress.CancelButtonClick(Sender: TObject);
begin
  Terminated := True;
end;

procedure TfrmProgress.SetTerminated(Value: boolean);
begin
  FTerminated := Value;
  if Value then Close;
end;

procedure TfrmProgress.SetProgress(Value: boolean);
begin
  Bar.Visible := Value;
  FProgress := Value;
  if Value then
    MessageLabel.Top := 15
  else
    MessageLabel.Top := 35;
end;

end.


