unit ovQueryMapper;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Contnrs, DB,
  ovTypes, ovIObjects, ovModel, ovRelations, ovOperations,
  ovQueryInfo, ovSQLQuery, ovDatabaseManager, ovObjectQuery;

type
  TMasterDetailRelation = class
  private
    FObjectClassName: String;
    FObjectRelationName: String;
  public
    constructor Create(AObjectClassName, AObjectRelationName: String);
    property ObjectClassName: String read FObjectClassName;
    property ObjectRelationName: String read FObjectRelationName;
  end;

  TDynamicMasterDetailRelation = class(TMasterDetailRelation)
  private
    FMasterFieldName: String;
  public
    constructor Create(AObjectClassName, AObjectRelationName: String;
      AMasterFieldName: String);
    property MasterFieldName: String read FMasterFieldName;
  end;

  TStaticMasterDetailRelation = class(TMasterDetailRelation)
  private
    FParentObjectId: TOID;
  public
    constructor Create(AObjectClassName, AObjectRelationName: String;
      AParentObjectId: TOID);
    property ParentObjectId: TOID read FParentObjectId;
  end;

  {{
  ,      SQL-.
  }
  TSelectQueryMapper = class(TCommonOperations)
  private
    FAllowSelectSystemFields: Boolean;
    FFieldId: Integer;
    FObjectQuery: TObjectSelectQuery;
    FQueryInfo: TSelectQueryInfo;
    FSQLQuery: TSelectQuery;
    procedure MapAttribute(RootObjectName, AttributePath: String; var AObjectInfo:
      TObjectInfo; var AttributeName: String);
    function ProcessQueryPart(Expression: String; AutoSelectField: Boolean = False;
      IsWherePart: Boolean = False): String;
    procedure ProcessQueryTree;
    procedure ProcessQuery(DefaultNameSpace: String; MasterDetailRelation:
      TMasterDetailRelation);
    function ProcessSelectPart: String;
    procedure ProcessSystemFunctions;
    procedure ProcessOIDs(QueryPart: TQueryExpression);
    procedure ProcessFromPart(DefaultNameSpace: String);
    procedure ProcessMasterDetail(MasterDetailRelation: TMasterDetailRelation);
    function ExtractAttribute(Expression: String; var ObjectName, AttributePath:
      String; var CurrPos, CurrLen: Integer): Boolean;
    function GetNewFieldId: Integer;
    procedure SelectAncestors(AObject: TObjectInfo);
    procedure SelectSystemFields(AObject: TObjectInfo);

    procedure SelectAttributeField(AObject: TObjectInfo; Attribute:
      TModelAttribute; FieldAlias: String; AutoSelected: Boolean);
    procedure SelectClassIdField(AObject: TObjectInfo);
    procedure SelectExpressionField(Expression, FieldAlias: String);
    function SelectObject(AObjectName, AObjectAlias: String): TObjectInfo;
    procedure SelectObjectIdField(AObject: TObjectInfo);
    procedure SelectReferenceField(AObject: TObjectInfo; Relation: TModelRelation;
      FieldAlias: String; AutoSelected: Boolean);
    function SelectRelation(AParentObject: TObjectInfo; ARelationName: String):
      TChildObjectInfo;
    procedure SelectRelationIdField(TableAlias, FieldName: String);
    procedure ProcessJoinPart;
    property AllowSelectSystemFields: Boolean read FAllowSelectSystemFields write
      FAllowSelectSystemFields;
  protected
    property ObjectQuery: TObjectSelectQuery read FObjectQuery;
    property QueryInfo: TSelectQueryInfo read FQueryInfo;
    property SQLQuery: TSelectQuery read FSQLQuery;
  public
    constructor Create(ADatabaseManager: TovDatabaseManager; AModel: TDomainModel);
    destructor Destroy; override;
    function OpenObjectDataSet(ADefaultNameSpace, AObjectClassName,
       AObjectAttributes, AObjectFilter, AObjectOrder, AJoinExpression: String; var
       AQueryInfo: TSelectQueryInfo; AMasterDetailRelation: TMasterDetailRelation =
       nil; AMasterDataSource: TDataSource = nil): TDataSet;
    function OpenObjectQuery(ADefaultNameSpace, AQuery: String; var AQueryInfo:
      TSelectQueryInfo; AMasterDetailRelation: TMasterDetailRelation = nil;
      AMasterDataSource: TDataSource = nil): TDataSet;
  end;

  {{
  ,      SQL-.
  }
  TUpdateQueryMapper = class(TCommonOperations)
  private
    function MapAttributesToFields(ModelObject: TModelObject; Expression: String):
      String;
  public
    procedure UpdateObjects(ModelObject: TModelObject; const AttributeValues:
      Variant; WhereExpression: String);
    procedure DeleteObjects(ModelObject: TModelObject; WhereExpression: String);
  end;

implementation

uses TypInfo, ovConsts, ovUtils, ovComponents, Variants, ovQueryManager;

function IsExpression(S: String): Boolean;
begin
  //           
  Result := Not ((S <> '') and (S[1] = '[') and (S[Length(S)] = ']') and
    (Pos('[', Copy(S, 2, Length(S)-2)) = 0))
end;

{{
   
}
function ExtractArgument(const Expression: String): String;
begin
  Result := Copy(Expression, 2, Length(Expression)-2) //   
end;

{
***************      MasterDetail *************
}

constructor TMasterDetailRelation.Create(AObjectClassName, AObjectRelationName: 
  String);
begin
  FObjectClassName := AObjectClassName;
  FObjectRelationName := AObjectRelationName;
end;

constructor TDynamicMasterDetailRelation.Create(AObjectClassName, 
  AObjectRelationName: String; AMasterFieldName: String);
begin
  inherited Create(AObjectClassName, AObjectRelationName);
  FMasterFieldName := AMasterFieldName;
end;

constructor TStaticMasterDetailRelation.Create(AObjectClassName,
  AObjectRelationName: String; AParentObjectId: TOID);
begin
  inherited Create(AObjectClassName, AObjectRelationName);
  FParentObjectId := AParentObjectId;
end;


{
************************** TSelectQueryMapper ************************************
}

constructor TSelectQueryMapper.Create(ADatabaseManager: TovDatabaseManager;
  AModel: TDomainModel);
begin
  inherited Create(ADatabaseManager, AModel);
  FSQLQuery := TSelectQuery.Create;
  FObjectQuery := TObjectSelectQuery.Create;
end;

destructor TSelectQueryMapper.Destroy;
begin
  FreeAndNil(FObjectQuery);
  FreeAndNil(FSQLQuery);
  inherited Destroy;
end;

{{
  ,    AObjectClassName,
  AObjectFilter      AObjectOrder.
   AMasterDetailRelation,    
  .
}
function TSelectQueryMapper.OpenObjectDataSet(ADefaultNameSpace,
   AObjectClassName, AObjectAttributes, AObjectFilter, AObjectOrder,
   AJoinExpression: String; var AQueryInfo: TSelectQueryInfo;
   AMasterDetailRelation: TMasterDetailRelation = nil; AMasterDataSource:
   TDataSource = nil): TDataSet;
var
  Query: String;
begin
  if Not DatabaseManager.DBConnected then
    raise Exception.Create('    ');

  Query := Format('SELECT %s FROM [%s]', [AObjectAttributes, AObjectClassName]);
  if AJoinExpression <> '' then Query := Query + #13'JOIN ' + AJoinExpression;
  if AObjectFilter <> '' then Query := Query + #13'WHERE ' + AObjectFilter;
  if AObjectOrder <> '' then Query := Query + #13'ORDER BY ' + AObjectOrder;
  Result := OpenObjectQuery(ADefaultNameSpace, Query,
    AQueryInfo, AMasterDetailRelation, AMasterDataSource);
end;

{{
********************************************************************************
                       .
********************************************************************************

1.    TClass1.ObjectQuery   
    .
2. BuildQuery       
 (SQL).
3.    TClass1.SQLQuery    
 ,     .
4. DatabaseBroker         
         .
5.      AQueryInfo    .
}
function TSelectQueryMapper.OpenObjectQuery(ADefaultNameSpace, AQuery: String;
  var AQueryInfo: TSelectQueryInfo; AMasterDetailRelation:
  TMasterDetailRelation = nil; AMasterDataSource: TDataSource = nil): TDataSet;
begin
  Result := nil;
  FQueryInfo := TSelectQueryInfo.Create;
  try
    ObjectQuery.Text := AQuery;
    SQLQuery.Text := '';

    {        DISTINCT, 
         ,      
       SQL-.      .
    }
    FAllowSelectSystemFields := Not ObjectQuery.ContainsAggregateFunctions;

    ProcessQuery(ADefaultNameSpace, AMasterDetailRelation);

    {$IFDEF Debug}
    WriteToLogEnabled := True;  
    {$ENDIF}

    Result := DatabaseManager.QueryManager.OpenQuery(SQLQuery.Text, AMasterDataSource);
    //         
    FQueryInfo.BindFields(Result);

    AQueryInfo := FQueryInfo;
    FQueryInfo := nil;
  except
    Result.Free;
    FreeAndNil(FQueryInfo);
    raise;
  end;
end;

{{
 ,       .
}
procedure TSelectQueryMapper.ProcessQuery(DefaultNameSpace: String;
  MasterDetailRelation: TMasterDetailRelation);
begin
  FFieldId := 0;
  ovRelations.AliasCount := 0;

  ProcessFromPart(DefaultNameSpace);
  ProcessSelectPart;

  //        

  SQLQuery.Where.ItemsText := ProcessQueryPart(ObjectQuery.Where.ItemsText, True, True);

  //   ORDER BY, GROUP BY, HAVING   ( SELECT)
  //  ,      .   
  //  ,    ,  
  //      SELECT.

  //  GROUP BY      ,
                            //      
  SQLQuery.GroupBy.Add(ProcessQueryPart(ObjectQuery.GroupBy.ItemsText, True));
  SQLQuery.Having.ItemsText := ProcessQueryPart(ObjectQuery.Having.ItemsText, True);
  SQLQuery.OrderBy.ItemsText := ProcessQueryPart(ObjectQuery.OrderBy.ItemsText, True);

  //         
  // ,    JOIN
  ProcessJoinPart;

  //        FROM
  //        
  ProcessQueryTree;

  //     OID
  // (     ,    
  //  , ..,     ).
  ProcessSystemFunctions;

  if Assigned(MasterDetailRelation) then
    ProcessMasterDetail(MasterDetailRelation);
end;

{{
  QueryPart         
    ,   .

       :
  "[ . ]",
 " "       
    ( ),    
     .

    ,    ,   
     FROM.
}
function TSelectQueryMapper.ProcessQueryPart(Expression: String;
  AutoSelectField: Boolean = False; IsWherePart: Boolean = False): String;
var
  ClassAlias, AttributePath, AttributeName, FieldName: String;
  NewText: String;
  CurrPos, CurrLen: Integer;
  AObjectInfo: TObjectInfo;
  Attribute: TModelAttribute;
  Relation: TModelRelation;
  ParentObject: TModelObject;
begin
  CurrPos := 1;
  while ExtractAttribute(Expression, ClassAlias, AttributePath, CurrPos, CurrLen) do
  begin
    MapAttribute(ClassAlias, AttributePath, AObjectInfo, AttributeName);

    Attribute := nil;
    Relation := nil;

    //   AttributeName     
    //      OID#
    if AttributeName = 'OID#' then begin
      FieldName := ObjectIdField;
      ParentObject := nil;
    end else
    if AttributeName = 'OWNER#' then begin
      FieldName := OwnerIdField;
      ParentObject := nil;
    end else begin
      Attribute := AObjectInfo.ModelObject.FindAttribute(AttributeName);
      if Attribute <> nil then begin
        FieldName := Attribute.FieldName;
        if Attribute.IsInherited then
          ParentObject := Attribute.ParentObject
        else
          ParentObject := nil;
      end else begin
        Relation := AObjectInfo.ModelObject.GetRelation(AttributeName);
        if Not Relation.IsReference then
          raise Exception.CreateFmt('  "%s"        ', [Expression]);
        FieldName := Relation.FieldName;
        if Relation.IsInherited then
          ParentObject := Relation.ParentObject
        else
          ParentObject := nil;
      end;
    end;

    //   (  )       ,
    //        
    if Assigned(ParentObject) and Not ParentObject.IsAbstract then
      AObjectInfo := AObjectInfo.GetAncestor(ParentObject.FullName);

    if IsWherePart and (Attribute <> nil) and Attribute.IsBoolean and
      Not DatabaseManager.DatabaseAdapter.SupportsBooleanType
    then
      //  ,       WHERE 
      //         
      //  "([] <> FalseValue and Not [] is Null)"
      NewText := Format('({%s}.[%s] <> %d and Not {%0:s}.[%s] is Null)',
        [AObjectInfo.TableAlias, FieldName, FalseValueReplacement])
    else
      //       (..   )
      NewText := Format('{%s}.[%s]', [AObjectInfo.TableAlias, FieldName]);

    //      ,   NewText
    ReplaceSubStr(Expression, NewText, CurrPos, CurrLen);
    Inc(CurrPos, Length(NewText));

    if AutoSelectField and AllowSelectSystemFields then
      if Attribute <> nil then
        SelectAttributeField(AObjectInfo, Attribute, Attribute.Name, True)
      else
      if Relation <> nil then
        SelectReferenceField(AObjectInfo, Relation, Relation.Name, True);
  end;
  Result := Expression;
end;

{{
      .    
 FROM      .
}
procedure TSelectQueryMapper.ProcessFromPart(DefaultNameSpace: String);
var
  AliasName, AClassName: String;
  I: Integer;
begin
  with ObjectQuery.From do
  for I := 0 to ItemCount - 1 do
  begin
    if Items[I].Expression = '' then raise Exception.Create('     FROM');

    //     ,   ,      
    AClassName := ExtractArgument(Items[I].Expression);
    if Items[I].Alias = '' then AliasName := AClassName else AliasName := Items[I].Alias;

    SelectObject(MakeFullObjectName(AClassName, DefaultNameSpace), AliasName)
  end;
end;

procedure TSelectQueryMapper.ProcessJoinPart;
var
  I: Integer;
  ChildObjectInfo: TChildObjectInfo;
  Path: String;
begin
  with ObjectQuery.Join do
  begin
    //        
    // 
    for I := 0 to ItemCount - 1 do
      Items[I].AdditionalFilter := ProcessQueryPart(Items[I].AdditionalFilter);

    //    
    for I := 0 to ItemCount - 1 do
    begin
      Path := Copy(Items[I].RelationPath, 2, Length(Items[I].RelationPath)-2); //   []
      ChildObjectInfo := QueryInfo.FindObjectByPath(Path);
      if ChildObjectInfo = nil then
        raise Exception.Create('      JOIN :'+Items[I].RelationPath);
      if Not ChildObjectInfo.ModelRelation.IsGeneralization then
        ChildObjectInfo.JoinExpressionItem := Items[I];
    end;
  end;
end;

{{
  SelectText   ","    -  
      .    
  TAttributeField  QueryInfo.   
       MapAttribute    
 SQL- (     TExpressionFieldInfo 
FieldInfo)
}
function TSelectQueryMapper.ProcessSelectPart: String;

  {{
  Argument         
   "*".    QueryInfo   
  (   "*")  TAttributeField.
  }
  procedure SelectAttributes(Argument: String; FieldAlias: String);


    procedure SelectAttribute(AObjectInfo: TObjectInfo; Attribute: TModelAttribute; FieldAlias: String);
    var
      RealObjectInfo: TObjectInfo;
    begin
      if FieldAlias = '' then
        FieldAlias := AObjectInfo.Path + '.' + Attribute.Name;

      with Attribute do
      begin
        if IsInherited and Not ParentObject.IsAbstract then
          //   ,      ( ),
          //    ,       
          // (        ,  
          //      )
          RealObjectInfo := AObjectInfo.GetAncestor(ParentObject.FullName)
        else
          RealObjectInfo := AObjectInfo;
        SelectAttributeField(RealObjectInfo, Attribute, FieldAlias, False);
      end
    end;

    procedure SelectReference(AObjectInfo: TObjectInfo;
      Relation: TModelRelation; FieldAlias: String);
    var
      RealObjectInfo: TObjectInfo;
    begin
      if FieldAlias = '' then
        FieldAlias := AObjectInfo.Path + '.' + Relation.Name;

      with Relation do
      begin
        if IsInherited and Not ParentObject.IsAbstract then
          //   ,      ( ),
          //    ,       
          // (        ,  
          //      )
          RealObjectInfo := AObjectInfo.GetAncestor(ParentObject.FullName)
        else
          RealObjectInfo := AObjectInfo;
        SelectReferenceField(RealObjectInfo, Relation, FieldAlias, False);
      end
    end;

  var
    ObjectName, AttributeName, AttributePath: String;
    CurrPos, CurrLen: Integer;
    AObjectInfo: TObjectInfo;
    I: Integer;
    Attribute: TModelAttribute;
  begin
    CurrPos := 1;
    if Not ExtractAttribute(Argument, ObjectName, AttributePath, CurrPos, CurrLen)
      then raise Exception.Create('   SELECT');

    MapAttribute(ObjectName, AttributePath, AObjectInfo, AttributeName);

    if AttributeName = '*' then
    begin
      if FieldAlias <> '' then
        raise Exception.Create(' AS    *');
      //     
      with AObjectInfo.ModelObject do
        for I := 0 to AttributeCount - 1 do
            SelectAttribute(AObjectInfo, Attributes[I], '')
    end else
    begin
      //  ,    
      Attribute := AObjectInfo.ModelObject.FindAttribute(AttributeName);
      if Attribute <> nil then
        SelectAttribute(AObjectInfo, Attribute, FieldAlias)
      else
        SelectReference(AObjectInfo,
          AObjectInfo.ModelObject.GetRelation(AttributeName), FieldAlias)
    end
  end;

  procedure ProcessSelectExpression(Expression, Alias: String);
  var
    NewArgument: String;
  begin
    if Expression = '' then raise Exception.Create('    SELECT');
    if IsExpression(Expression) then begin
      NewArgument := ProcessQueryPart(Expression); //       <.>
      if Alias = '' then Alias := 'Expression';
      SelectExpressionField(NewArgument, Alias);
    end else
      SelectAttributes(Expression, Alias);
  end;

var
  I: Integer;
begin
  with ObjectQuery.Select do
  for I := 0 to ItemCount - 1do
    ProcessSelectExpression(Items[I].Expression, Items[I].Alias);
  SQLQuery.Select.Distinct := ObjectQuery.Select.Distinct; //  DISTINCT,   
end;

{{
  Expression       (  ),
   CurrPos.      CurrLen 
 CurrPos.  ,  False,   Expression
  .
}
function TSelectQueryMapper.ExtractAttribute(Expression: String; var
  ObjectName, AttributePath: String; var CurrPos, CurrLen: Integer): Boolean;
var
  FirstPart, RestPart: String;
begin
  Result := ExtractAttributeRef(Expression, RestPart, CurrPos, CurrLen);
  if Result then begin
    ParseAttribute(FirstPart, RestPart);

    //         
    //     .    (.. 
    //    ) ,      
    //     FROM.

    if QueryInfo.GetObject(FirstPart, False) <> nil then begin
      ObjectName := FirstPart;
      AttributePath := RestPart;
    end else begin
      ObjectName := QueryInfo.Objects[0].Name;
      AttributePath := CombineSubStr(FirstPart, '.', RestPart);
    end;
  end
end;

function TSelectQueryMapper.GetNewFieldId: Integer;
begin
  Inc(FFieldId);
  Result := FFieldId;
end;

{{
         
       .   
     ,   , 
     .

    RootObjectName      AttributePath,
       .

       , 
  ,   AttributePath.
}
procedure TSelectQueryMapper.MapAttribute(RootObjectName, AttributePath:
  String; var AObjectInfo: TObjectInfo; var AttributeName: String);
var
  Relation: TModelRelation;
  CurrObject: TObjectInfo;
  NextObject: TChildObjectInfo;
  CurrAttribute, CurrClass, CurrPath: String;
begin
  //      RootObjectName
  CurrObject := QueryInfo.GetObject(RootObjectName, False);
  if CurrObject = nil then
    raise Exception.Create('   : ' + RootObjectName);

  //    ,     
  CurrPath := RootObjectName;
  CurrClass := CurrObject.ModelObject.FullName;

  while ParseAttribute(CurrAttribute, AttributePath) do
  begin
    CurrPath := CurrPath + '.' + CurrAttribute;

    //      
    Relation := Model.GetObjectRelation(CurrClass, CurrAttribute);
    if Relation.IsInherited and Not Relation.ParentObject.IsAbstract then
      CurrObject := CurrObject.GetAncestor(Relation.ParentObject.FullName);

    //           AttributeName
    NextObject := CurrObject.GetObject(CurrAttribute, False);
    if NextObject = nil then
      NextObject := SelectRelation(CurrObject, CurrAttribute);

    CurrObject := NextObject;
    CurrClass := CurrObject.ModelObject.FullName;
  end;

  //  
  AttributeName := CurrAttribute;
  AObjectInfo := CurrObject;
end;

{{
      .
}
procedure TSelectQueryMapper.ProcessSystemFunctions;
begin
  ProcessOIDs(SQLQuery.Where);
  ProcessOIDs(SQLQuery.GroupBy);
  ProcessOIDs(SQLQuery.Having);
  ProcessOIDs(SQLQuery.OrderBy);
end;

{{
     OID(Class).    
  ObjectId - .

   "Master",      
 ObjectId  MasterDataSource.  MasterDataSource  
TovDataSource,      ObjectId  
    "ObjectId".
}
procedure TSelectQueryMapper.ProcessOIDs(QueryPart: TQueryExpression);

  function ExtractOID(Expression: String; var Argument: String; var
    CurrPos, CurrLen: Integer): Boolean;
  var
    P1, P2: Integer;
  begin
    P1 := Pos('OID(', Copy(UpperCase(Expression), CurrPos, Length(Expression)));
    Result := P1 > 0;
    if Result then begin
      P2 := Pos(')', Copy(Expression, CurrPos+P1, Length(Expression)));
      if P2 = 0 then raise Exception.Create('   )  : ' + Expression);
      //     
      Inc(CurrPos, P1-1);
      CurrLen := P2+1;
      Argument := Copy(Expression, CurrPos+4, CurrLen-5);
    end
  end;

  function ProcessOID(Argument: String): String;
  begin
    Result := Format('{%s}.[%s]', [QueryInfo.GetObject(Argument).TableAlias, ObjectIdField]);
  end;

var
  Argument, NewText, Expression: String;
  CurrPos, CurrLen: Integer;
begin
  CurrPos := 1;
  Expression := QueryPart.ItemsText;
  while ExtractOID(Expression, Argument, CurrPos, CurrLen) do
  begin
    NewText := ProcessOID(Argument);
    ReplaceSubStr(Expression, NewText, CurrPos, CurrLen);
    Inc(CurrPos, Length(NewText));
  end;
  QueryPart.ItemsText := Expression;
end;

{{
        
FROM ( JOIN)  WHERE ( ,    ).

//       ObjectId  .
}
procedure TSelectQueryMapper.ProcessQueryTree;

  {{
     FROM        
  .    
  ,       ,
    .      , 
     .

//  :          
//  ObjectId.
  }
  function VisitNode(Node: TObjectInfo; CurrExpression: String): String;
  var
    I: Integer;
  begin
    with Node do
    begin
      //    FROM     :
      // < > <   SQL->
      Result := Format('{%s} %s', [ModelObject.TableName, TableAlias]);

      if Node is TChildObjectInfo then
      with TChildObjectInfo(Node) do
      begin
        with CreateRelation(ModelRelation, DatabaseManager.QueryManager) do
        try
          if Assigned(JoinExpressionItem) then
            Result := GetJoinText(CurrExpression, Result,
              ParentObject.TableAlias, TableAlias, JoinExpressionItem.JoinType, JoinExpressionItem.AdditionalFilter)
          else
            Result := GetJoinText(CurrExpression, Result,
              ParentObject.TableAlias, TableAlias, jtLeftOuter, '')
        finally
          Free
        end;
      end;
      for I := 0 to ChildCount - 1 do
        Result := VisitNode(Children[I], Result);
    end;
  end;

var
  I: Integer;
begin
  for I := 0 to QueryInfo.ObjectCount - 1 do
    SQLQuery.From.Add(VisitNode(QueryInfo.Objects[I], ''));
end;

{{
   WHERE  FROM  , 
Master-Detail .
}
procedure TSelectQueryMapper.ProcessMasterDetail(MasterDetailRelation:
  TMasterDetailRelation);
var
  RelIdTableAlias, RelIdFieldName: String;
  FromExpr, WhereExpr: String;
  Relation: TRelation;
begin
  with MasterDetailRelation do
    Relation := CreateRelation(Model.GetObjectRelation(
      ObjectClassName, ObjectRelationName), DatabaseManager.QueryManager);
  try
    WhereExpr := Relation.GetMasterDetailText(QueryInfo.Objects[0].TableAlias,
      FromExpr, RelIdTableAlias, RelIdFieldName);
    if MasterDetailRelation is TStaticMasterDetailRelation then
      //   WHERE   <> = <  >
      WhereExpr := Format(WhereExpr, [IntToStr(TStaticMasterDetailRelation(MasterDetailRelation).ParentObjectId)])
    else
    if MasterDetailRelation is TDynamicMasterDetailRelation then
      //   WHERE   <> = :< >
      WhereExpr := Format(WhereExpr, [':'+TDynamicMasterDetailRelation(MasterDetailRelation).MasterFieldName]);

    //       (   ,
    //      "or", ..  
    //      "and")
    if SQLQuery.Where.ItemsText <> '' then
      SQLQuery.Where.ItemsText := '(' + SQLQuery.Where.ItemsText + ')';

    SQLQuery.Where.Add(WhereExpr);

    if FromExpr <> '' then SQLQuery.From.Add(FromExpr);

    if AllowSelectSystemFields then
      SelectRelationIdField(RelIdTableAlias, RelIdFieldName);
  finally
    Relation.Free;
  end;
end;

{{
     ,    
(   )   AObject.ModelObject.
}
procedure TSelectQueryMapper.SelectAncestors(AObject: TObjectInfo);
var
  ChildObject: TObjectInfo;
  I: Integer;
begin
  with AObject.ModelObject do
    for I := 0 to AncestorCount - 1 do
      if Not AncestorRelations[I].TargetObject.IsAbstract then
      begin
        ChildObject := QueryInfo.AddChildObject(AObject, AncestorRelations[I], AObject.Path);
        SelectSystemFields(ChildObject);
      end
end;

{{
  ,    .

  SelectFieldToSQL  False,      SQL- 
    ,      FieldInfo.

SelectFieldToSQL   True    ,
,      ORDER BY    
     SELECT.
}
procedure TSelectQueryMapper.SelectAttributeField(AObject: TObjectInfo;
  Attribute: TModelAttribute; FieldAlias: String; AutoSelected: Boolean);
var
  FieldStr: String;
begin
  if QueryInfo.AddField(AObject, TAttributeField.Create(
    FieldAlias, Attribute.FieldName, AutoSelected, Attribute), Attribute.Persistent) then
  begin
    //      
    FieldStr := Format('{%s}.[%s]', [AObject.TableAlias, Attribute.FieldName]);
    SQLQuery.Select.Add(FieldStr);
  end;
end;

{{
  ,   ClassId  AObject.TableAlias.
}
procedure TSelectQueryMapper.SelectClassIdField(AObject: TObjectInfo);
var
  FieldStr: String;
begin
  if QueryInfo.AddField(AObject, TClassIdField.Create(ClassIdField, ClassIdField, True)) then
  begin
    //      
    FieldStr := Format('{%s}.[%s]', [AObject.TableAlias, ClassIdField]);
    SQLQuery.Select.Add(FieldStr);
  end;
end;

procedure TSelectQueryMapper.SelectExpressionField(Expression, FieldAlias:
  String);
begin
  if QueryInfo.AddField(TExpressionField.Create(FieldAlias, Expression, False)) then
  begin
    SQLQuery.Select.Add(Expression);
  end
end;

{
************************     **************************
}

function TSelectQueryMapper.SelectObject(AObjectName, AObjectAlias: String):
  TObjectInfo;
begin
  Result := QueryInfo.AddObject(AObjectAlias, Model.GetObject(AObjectName));
  SelectSystemFields(Result); //       
  SelectAncestors(Result);    //    -
end;

{
*********************************   ********************************
}

{{
  ,   ObjectId  AObject.TableAlias.
}
procedure TSelectQueryMapper.SelectObjectIdField(AObject: TObjectInfo);
var
  FieldStr: String;
begin
  if QueryInfo.AddField(AObject, TObjectIdField.Create(ObjectIdField, ObjectIdField, True)) then
  begin
    //      
    FieldStr := Format('{%s}.[%s] as ObjectId%d', [AObject.TableAlias, ObjectIdField, GetNewFieldId]);
    SQLQuery.Select.Add(FieldStr);
  end;
end;

{{
  ,    , 
    .
}
procedure TSelectQueryMapper.SelectReferenceField(AObject: TObjectInfo;
  Relation: TModelRelation; FieldAlias: String; AutoSelected: Boolean);
var
  FieldStr: String;
begin
  if QueryInfo.AddField(AObject, TReferenceField.Create(
    FieldAlias, Relation.FieldName, AutoSelected, Relation)) then
  begin
    //      
    FieldStr := Format('{%s}.[%s] as Ref%d', [AObject.TableAlias, Relation.FieldName, GetNewFieldId]);
    {
      !      (Ref)    
    (   ,     ).   
        Master-Detail. ..  
      ,       
    (  SELECT ... WHERE ...= :MyField )       .
        ,        
       ,     .
    }
    SQLQuery.Select.Add(FieldStr);
  end;
end;

function TSelectQueryMapper.SelectRelation(AParentObject: TObjectInfo;
  ARelationName: String): TChildObjectInfo;
var
  APath: String;
begin
  APath := AParentObject.Path + '.' + ARelationName;
  Result := QueryInfo.AddChildObject(AParentObject,
    Model.GetObjectRelation(AParentObject.ModelObject.FullName, ARelationName), APath);
  SelectSystemFields(Result); //       
  SelectAncestors(Result);    //    -
end;

procedure TSelectQueryMapper.SelectRelationIdField(TableAlias, FieldName:
  String);
var
  FieldStr: String;
begin
  if QueryInfo.AddField(TRelationIdField.Create(FieldName, FieldName, True, TableAlias, '')) then
  begin
    FieldStr := Format('{%s}.[%s]', [TableAlias, FieldName]) ; //      DatabaseBroker'e
    SQLQuery.Select.Add(FieldStr);
  end
end;

{{
      AObject.
(  ObjectId       ).
}
procedure TSelectQueryMapper.SelectSystemFields(AObject: TObjectInfo);
var
  I: Integer;
begin
  if AllowSelectSystemFields then
  begin
    SelectObjectIdField(AObject);

    //   ClassId    
    if AObject.IsRoot then SelectClassIdField(AObject);

    //       
    with AObject.ModelObject do
      for I := 0 to RelationCount - 1 do
        with Relations[I] do
          if Not IsInherited and IsReference then
            SelectReferenceField(AObject, Relations[I], AObject.Path + '.' + Relations[I].Name, True);
  end
end;

{ TUpdateQueryMapper }

procedure TUpdateQueryMapper.UpdateObjects(ModelObject: TModelObject; const
  AttributeValues: Variant; WhereExpression: String);
var
  I, J: Integer;
  FieldValues: TFieldValues;
  Attribute: TModelAttribute;
  AFieldType: TFieldType;
  AFieldSize: Integer;
begin
  SetLength(FieldValues, VarArrayHighBound(AttributeValues, 1) + 1 div 2);
  J := 0;
  for I := 0 to VarArrayHighBound(AttributeValues, 1) - 1 do
  begin
    Attribute := ModelObject.GetAttribute(AttributeValues[I div 2]);
    Attribute.GetFieldDef(AFieldType, AFieldSize);
    FieldValues[J].FieldName := Attribute.FieldName;
    FieldValues[J].FieldType := AFieldType;
    FieldValues[J].FieldSize := AFieldSize;
    FieldValues[J].Value := AttributeValues[I div 2 + 1];
    Inc(J);
  end;

  DatabaseManager.QueryManager.UpdateRecords(ModelObject.TableName,
    FieldValues, MapAttributesToFields(ModelObject, WhereExpression));
end;

procedure TUpdateQueryMapper.DeleteObjects(ModelObject: TModelObject;
  WhereExpression: String);
begin
  DatabaseManager.QueryManager.DeleteRecords(ModelObject.TableName,
    MapAttributesToFields(ModelObject, WhereExpression));
end;

{{
       - 
    .    
   .
}
function TUpdateQueryMapper.MapAttributesToFields(ModelObject: TModelObject;
  Expression: String): String;
var
  AttributeOrRelationName, FieldName: String;
  NewText: String;
  CurrPos, CurrLen: Integer;
  Attribute: TModelAttribute;
  Relation: TModelRelation;
begin
  CurrPos := 1;
  while ExtractAttributeRef(Expression, AttributeOrRelationName, CurrPos, CurrLen) do
  begin
    Attribute := ModelObject.FindAttribute(AttributeOrRelationName);
    if Attribute <> nil then
    begin
      FieldName := Attribute.FieldName;
      Relation := nil;
    end else
    if AttributeOrRelationName = 'OID#' then
      FieldName := ObjectIdField
    else
    begin
      Relation := ModelObject.GetRelation(AttributeOrRelationName);
      if Not Relation.IsReference then
        raise Exception.CreateFmt('  "%s"        ', [Expression]);
      FieldName := Relation.FieldName;
    end;

    if (Attribute <> nil) and Attribute.IsBoolean and
      Not DatabaseManager.DatabaseAdapter.SupportsBooleanType
    then
      //  ,       WHERE 
      //         
      //  "([] <> FalseValue and Not [] is Null)"
      NewText := Format('([%s] <> %d and Not [%s] is Null)',
        [FieldName, FalseValueReplacement])
    else
      //       (..   )
      NewText := '['+FieldName+']';

    //      ,   NewText
    ReplaceSubStr(Expression, NewText, CurrPos, CurrLen);
    Inc(CurrPos, Length(NewText));
  end;
  Result := Expression;
end;


end.





