{$WRITEABLECONST ON}
unit ovRelations;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, ovTypes, ovModel, ovObjectQuery, ovQueryManager;

type
  TRelation = class(TObject)
  private
    FQueryManager: TovQueryManager;
    FModelRelation: TModelRelation;
    function GetSourceObject: TModelObject;
    function GetTargetObject: TModelObject;
    function NewTableAlias: String;
  public
    constructor Create(AQueryManager: TovQueryManager; AModelRelation:
      TModelRelation);
    {{
        ParentObjectId   
     ChildId        .
    }
    function CreateRelation(ParentObjectId, ChildObjectId: TOID): TOID; virtual;
      abstract;
    {{
        ,  
     ,     
       ObjectId.
    }
    function GetReferencingObjects(AChildObjectId: TOID): TDataSet; virtual;
      abstract;
    {{
         RelationId   ParentObjectId, 
       ,       NewChildId.
    }
    procedure UpdateRelation(ParentObjectId: TOID; RelationId: Integer; NewChildId: TOID);
      virtual; abstract;
    procedure DeleteRelation(ParentObjectId, ChildObjectId, RelationId: TOID);
      virtual; abstract;
    {{
              
     
    }
    function HasReferencingObjects(AChildObjectId: TOID): Boolean; virtual;
      abstract;
    function GetReferencedObjects(ParentObjectId: TOID): TDataSet; virtual;
      abstract;
    function GetRelation(ParentObjectId: TOID; RelationId: Integer): TOID; virtual;
      abstract;
    {{
        JOIN     
      .      
    FROM .       
        WhereExpr,     
    WHERE .
    ParentTable -     
    ChildTable -     
    }
    function GetJoinText(LeftExpression, RightExpression, ParentTable, ChildTable:
      String; JoinType: TJoinType; AdditionalConditions: String): String; virtual;
      abstract;
    {{
             WHERE   ,
      Master-Detail     .
                  ,
          .
               FROM.
    }
    function GetMasterDetailText(ChildTable: String; var FromExpr,
      RelationIdTable, RelationIdFieldName: String): String; virtual; abstract;
    property QueryManager: TovQueryManager read FQueryManager;
    property SourceObject: TModelObject read GetSourceObject;
    property TargetObject: TModelObject read GetTargetObject;
    property ModelRelation: TModelRelation read FModelRelation;
  end;

  TAssociationRelation = class(TRelation)
  private
    FRelationTableAlias: String;
  public
    constructor Create(AQueryManager: TovQueryManager; AModelRelation:
      TModelRelation);
    function CreateRelation(ParentObjectId, ChildObjectId: TOID): TOID; override;
    function GetReferencingObjects(AChildObjectId: TOID): TDataSet; override;
    procedure DeleteRelation(ParentObjectId, ChildObjectId, RelationId: TOID);
      override;
    function HasReferencingObjects(AChildObjectId: TOID): Boolean; override;
    function GetReferencedObjects(ParentObjectId: TOID): TDataSet; override;
    procedure UpdateRelation(ParentObjectId: TOID; RelationId: Integer; NewChildId: TOID);
      override;
    function GetRelation(ParentObjectId: TOID; RelationId: Integer): TOID; override;
    function GetJoinText(LeftExpression, RightExpression, ParentTable, ChildTable:
      String; JoinType: TJoinType; AdditionalConditions: String): String; override;
    function GetMasterDetailText(ChildTable: String; var FromExpr, RelationIdTable,
      RelationIdFieldName: String): String; override;
    property RelationTableAlias: String read FRelationTableAlias;
  end;

  TCompositionRelation = class(TRelation)
  private
    FModelObject: TModelObject;
  public
    function CreateRelation(ParentObjectId, ChildObjectId: TOID): TOID; override;
    function GetReferencingObjects(AChildObjectId: TOID): TDataSet; override;
    procedure DeleteRelation(ParentObjectId, ChildObjectId, RelationId: TOID);
      override;
    function HasReferencingObjects(AChildObjectId: TOID): Boolean; override;
    function GetReferencedObjects(ParentObjectId: TOID): TDataSet; override;
    procedure SetOwner(ChildObjectId: TOID; ParentObjectId, RelationId: TOID);
    function GetRelation(ParentObjectId: TOID; RelationId: Integer): TOID; override;
    function GetJoinText(LeftExpression, RightExpression, ParentTable, ChildTable:
      String; JoinType: TJoinType; AdditionalConditions: String): String; override;
    function GetMasterDetailText(ChildTable: String; var FromExpr,
      RelationIdTable, RelationIdFieldName: String): String; override;
    procedure GetOwner(ChildObjectId: TOID; var ParentObjectId, RelationId: TOID);
    constructor Create(AQueryManager: TovQueryManager; AModelObject: TModelObject);
      overload;
    property ModelObject: TModelObject read FModelObject;
  end;

  TReferenceRelation = class(TRelation)
  public
    function CreateRelation(ParentObjectId, ChildObjectId: TOID): TOID; override;
    function GetReferencingObjects(AChildObjectId: TOID): TDataSet; override;
    procedure DeleteRelation(ParentObjectId, ChildObjectId, RelationId: TOID);
      override;
    function HasReferencingObjects(AChildObjectId: TOID): Boolean; override;
    function GetReferencedObjects(ParentObjectId: TOID): TDataSet; override;
    procedure UpdateRelation(ParentObjectId: TOID; RelationId: Integer; NewChildId: TOID);
      override;
    function GetRelation(ParentObjectId: TOID; RelationId: Integer): TOID; override;
    function GetJoinText(LeftExpression, RightExpression, ParentTable, ChildTable:
      String; JoinType: TJoinType; AdditionalConditions: String): String; override;
    function GetMasterDetailText(ChildTable: String; var FromExpr,
      RelationIdTable, RelationIdFieldName: String): String; override;
  end;

  TGeneralizationRelation = class(TRelation)
  public
    function GetJoinText(LeftExpression, RightExpression, ParentTable, ChildTable:
      String; JoinType: TJoinType; AdditionalConditions: String): String; override;
    function CreateRelation(ParentObjectId, ChildObjectId: TOID): TOID; override;
    function GetRelation(ParentObjectId: TOID; RelationId: Integer): TOID; override;
    function GetAncestor(AModelObject: TModelObject; AObjectId: TOID;
      AModelRelation: TModelRelation): TOID;
  end;

  TAggregationRelation = class(TRelation)
  private
    FModelObject: TModelObject;
  public
    function CreateRelation(ParentObjectId, ChildObjectId: TOID): TOID; override;
    function GetReferencingObjects(AChildObjectId: TOID): TDataSet; override;
    procedure DeleteRelation(ParentObjectId, ChildObjectId, RelationId: TOID);
      override;
    function HasReferencingObjects(AChildObjectId: TOID): Boolean; override;
    function GetReferencedObjects(ParentObjectId: TOID): TDataSet; override;
    procedure SetOwner(ChildObjectId: TOID; ParentObjectId, RelationId: TOID);
    function GetRelation(ParentObjectId: TOID; RelationId: Integer): TOID; override;
    function GetJoinText(LeftExpression, RightExpression, ParentTable, ChildTable:
      String; JoinType: TJoinType; AdditionalConditions: String): String; override;
    function GetMasterDetailText(ChildTable: String; var FromExpr,
      RelationIdTable, RelationIdFieldName: String): String; override;
    procedure GetOwner(ChildObjectId: TOID; var ParentObjectId, RelationId: TOID);
    constructor Create(AQueryManager: TovQueryManager; AModelObject: TModelObject);
      overload;
    property ModelObject: TModelObject read FModelObject;
  end;

  EObjectNotFound = class(Exception)
  end;

function CreateRelation(AModelRelation: TModelRelation; AQueryManager:
  TovQueryManager): TRelation; overload;

var
  AliasCount: Integer;

implementation

uses ovConsts, Variants;

{{
   TRelation,   AModelRelation
.
}
function CreateRelation(AModelRelation: TModelRelation; AQueryManager:
  TovQueryManager): TRelation;
begin
  case AModelRelation.RelationType of
    rtAssociation:
      if AModelRelation.IsReference
        then Result := TReferenceRelation.Create(AQueryManager, AModelRelation)
        else Result := TAssociationRelation.Create(AQueryManager, AModelRelation);
    rtAggregation:
      Result := TAggregationRelation.Create(AQueryManager, AModelRelation);
    rtComposition:
      Result := TCompositionRelation.Create(AQueryManager, AModelRelation);
    rtGeneralization:
      Result := TGeneralizationRelation.Create(AQueryManager, AModelRelation)
  else
    raise Exception.Create('  ');
  end;
end;

{{
  : LeftExpression JOIN RightExpression ON Fields
 Fields  : LeftTable.JoinedFields[i] = RightTable.JoinedFields[i+1] AND ...
 ,   JoinedFields  ,     , 
   . JoinType  , 
 .
}
function JoinTables(LeftExpression, RightExpression: String; JoinType:
  TJoinType; LeftTable, RightTable: String; const JoinedFields: array of
  Variant; AdditionalConditions: String = ''): String;

  function MakeCondition(LeftValue, RightValue: Variant): String;
  var
    LeftStr, RightStr: String;
  begin
    if VarIsStr(LeftValue)
      then LeftStr := Format('{%s}.[%s]', [LeftTable, String(LeftValue)])
      else LeftStr := Format('%d', [Integer(LeftValue)]);
    if VarIsStr(RightValue)
      then RightStr := Format('{%s}.[%s]', [RightTable, String(RightValue)])
      else RightStr := Format('%d', [Integer(RightValue)]);

    Result := Format('%s = %s', [LeftStr, RightStr])
  end;

const
  JoinNames: array [TJoinType] of String =
    ('LEFT OUTER JOIN', 'RIGHT OUTER JOIN', 'INNER JOIN');
var
  I: Integer;
  Conditions: String;
begin
  if Length(JoinedFields) > 0 then begin
    Conditions := '';
    I := 0;
    while I < Length(JoinedFields) do
    begin
      Conditions := Conditions + MakeCondition(JoinedFields[I], JoinedFields[I+1]) + ' AND ';
      Inc(I, 2);
    end;
    Delete(Conditions, Length(Conditions) - 4, 5);
    if AdditionalConditions <> '' then
      Conditions := Conditions + ' AND ' + AdditionalConditions;
    Result := Format('%s %s %s ON %s',
      [LeftExpression, JoinNames[JoinType], RightExpression, Conditions]);
  end else
    raise Exception.Create('    ');
end;


{{
,   ,   
  ,     
}
{
********************************** TRelation ***********************************
}
{<<Category>>: No category}
constructor TRelation.Create(AQueryManager: TovQueryManager; AModelRelation:
  TModelRelation);
begin
  inherited Create;
  FQueryManager := AQueryManager;
  FModelRelation := AModelRelation;
end;

function TRelation.GetSourceObject: TModelObject;
begin
  Result := ModelRelation.SourceObject;
end;

function TRelation.GetTargetObject: TModelObject;
begin
  Result := ModelRelation.TargetObject;
end;

function TRelation.NewTableAlias: String;
begin
  Inc(AliasCount);
  Result := 'R_' + IntToStr(AliasCount);
end;

{
***************************** TAssociationRelation *****************************
}
constructor TAssociationRelation.Create(AQueryManager: TovQueryManager;
  AModelRelation: TModelRelation);
begin
  inherited Create(AQueryManager, AModelRelation);
  if ModelRelation.TableName = ''
    then raise Exception.CreateFmt('     "%s"  "%s"',
      [ModelRelation.Name, SourceObject.Name]);
  FRelationTableAlias := NewTableAlias;
end;

{<<Category>>: No category}
function TAssociationRelation.CreateRelation(ParentObjectId, ChildObjectId:
  TOID): TOID;
begin
  Result := QueryManager.AddRecord(ModelRelation.TableName,
    RelationInstanceIdField, MakeIntFieldValues([OwnerIdField, ParentObjectId,
    ObjectIdField, ChildObjectId, RelationIdField, ModelRelation.Id]));
end;

procedure TAssociationRelation.DeleteRelation(ParentObjectId, ChildObjectId,
  RelationId: TOID);
begin
  if RelationId <> 0 then
    //    
    QueryManager.DeleteRecord(ModelRelation.TableName,
      RelationInstanceIdField, RelationId)
  else
    //      
    QueryManager.DeleteRecords(ModelRelation.TableName,
      MakeIntFieldValues([OwnerIdField, ParentObjectId, ObjectIdField, ChildObjectId,
       RelationIdField, ModelRelation.Id]));
end;

procedure TAssociationRelation.UpdateRelation(ParentObjectId: TOID; RelationId:
  Integer; NewChildId: TOID);
begin
  QueryManager.UpdateRecord(ModelRelation.TableName, RelationInstanceIdField,
    RelationId, MakeIntFieldValues([ObjectIdField, NewChildId]));
end;

function TAssociationRelation.GetRelation(ParentObjectId: TOID; RelationId: Integer):
  TOID;
begin
  with QueryManager.SelectRecords(ModelRelation.TableName,
    [ObjectIdField], MakeIntFieldValues([RelationInstanceIdField, RelationId,
      OwnerIdField, ParentObjectId, RelationIdField, ModelRelation.Id])) do
  try
    if IsEmpty then raise Exception.Create('   ');
    Result := Fields[0].AsInteger;
  finally
    Free
  end;
end;

function TAssociationRelation.HasReferencingObjects(AChildObjectId: TOID):
  Boolean;
begin
  Result := QueryManager.CountRecords(
    ModelRelation.TableName, RelationInstanceIdField,
    MakeIntFieldValues([ObjectIdField, AChildObjectId,
      RelationIdField, ModelRelation.Id])) > 0;
end;

function TAssociationRelation.GetReferencingObjects(AChildObjectId: TOID):
  TDataSet;
begin
  Result := QueryManager.SelectRecords(ModelRelation.TableName,
    [OwnerIdField], MakeIntFieldValues([ObjectIdField, AChildObjectId,
      RelationIdField, ModelRelation.Id]));
end;

function TAssociationRelation.GetReferencedObjects(ParentObjectId: TOID):
  TDataSet;
begin
  Result := QueryManager.SelectRecords(ModelRelation.TableName,
    [ObjectIdField, RelationInstanceIdField],
    MakeIntFieldValues([OwnerIdField, ParentObjectId,
      RelationIdField, ModelRelation.Id]));
end;

function TAssociationRelation.GetJoinText(LeftExpression, RightExpression,
  ParentTable, ChildTable: String; JoinType: TJoinType; AdditionalConditions:
  String): String;
var
  InnerExpression: String;
begin
  InnerExpression := JoinTables(
    LeftExpression, Format('{%s} %s', [ModelRelation.TableName, FRelationTableAlias]),
      { jtInner -     ,      }
      jtLeftOuter,
      ParentTable, FRelationTableAlias, [ObjectIdField, OwnerIdField, ModelRelation.Id, RelationIdField]);
  Result := JoinTables(InnerExpression, RightExpression,
    JoinType, FRelationTableAlias, ChildTable, [ObjectIdField, ObjectIdField], AdditionalConditions);
end;

function TAssociationRelation.GetMasterDetailText(ChildTable: String; var
  FromExpr, RelationIdTable, RelationIdFieldName: String): String;
begin
  FromExpr := Format('{%s} %s', [ModelRelation.TableName, RelationTableAlias]);
  Result := Format('{%s}.[%s] = {%s}.[%s] AND {%s}.[%s] = %s AND {%s}.[%s] = %d',
    [ChildTable, ObjectIdField, RelationTableAlias, ObjectIdField,
     RelationTableAlias, OwnerIdField, '%s', RelationTableAlias, RelationIdField, ModelRelation.Id]);

  (*  MS Access
  FromExpr := Format('INNER JOIN {%s} %s ON {%s}.%s = {%s}.%s',
    [ModelRelation.TableName, RelationTableAlias,
     ChildTable, ObjectIdField, RelationTableAlias, ObjectIdField]);
  Result := Format('{%s}.%s = %s AND {%s}.%s = %d',
    [RelationTableAlias, OwnerIdField, '%s', RelationTableAlias, RelationIdField, ModelRelation.Id]);
  *)

  RelationIdTable := RelationTableAlias;
  RelationIdFieldName := RelationInstanceIdField;
end;

{
***************************** TCompositionRelation *****************************
}

{{
        GetOwner,
SetOwner.
}
constructor TCompositionRelation.Create(AQueryManager: TovQueryManager;
  AModelObject: TModelObject);
begin
  inherited Create(AQueryManager, nil);
  FModelObject := AModelObject;
end;

function TCompositionRelation.CreateRelation(ParentObjectId, ChildObjectId:
  TOID): TOID;
begin
  QueryManager.UpdateRecord(TargetObject.TableName, ObjectIdField, ChildObjectId,
    MakeIntFieldValues([OwnerIdField, ParentObjectId, RelationIdField, ModelRelation.Id]));
  Result := ChildObjectId;
end;

{{
    ParentObjectId  
 RelationId     ChildObjectId.
}
procedure TCompositionRelation.GetOwner(ChildObjectId: TOID; var ParentObjectId,
  RelationId: TOID);
begin
  with QueryManager.SelectRecords(ModelObject.TableName,
    [OwnerIdField, RelationIdField],
    MakeIntFieldValues([ObjectIdField, ChildObjectId])) do
  try
    if IsEmpty then begin
      ParentObjectId := 0;
      RelationId := 0
    end else begin
      ParentObjectId := Fields[0].AsInteger;
      RelationId := Fields[1].AsInteger;
    end
  finally
    Free
  end;
end;

{{
    ChildObjectId   
    ParentObjectId      
RelationId.
}
procedure TCompositionRelation.SetOwner(ChildObjectId: TOID; ParentObjectId,
  RelationId: TOID);
begin
  if ParentObjectId <> 0 then
    QueryManager.UpdateRecord(ModelObject.TableName,
      ObjectIdField, ChildObjectId,
      MakeIntFieldValues([OwnerIdField, ParentObjectId, RelationIdField, RelationId]))
  else
    QueryManager.UpdateRecord(ModelObject.TableName,
      ObjectIdField, ChildObjectId,
      MakeIntFieldValues([OwnerIdField, Null, RelationIdField, Null]))
end;

procedure TCompositionRelation.DeleteRelation(ParentObjectId, ChildObjectId,
  RelationId: TOID);
begin
  QueryManager.UpdateRecord(TargetObject.TableName,
    ObjectIdField, ChildObjectId,
    MakeIntFieldValues([OwnerIdField, Null, RelationIdField, Null]));
end;

function TCompositionRelation.HasReferencingObjects(AChildObjectId: TOID):
  Boolean;
begin
  Result := QueryManager.CountRecords(
    TargetObject.TableName, OwnerIdField,
    MakeIntFieldValues([ObjectIdField, AChildObjectId,
    RelationIdField, ModelRelation.Id]),
     Format('Not [%s] is null', [OwnerIdField])) > 0
end;

function TCompositionRelation.GetRelation(ParentObjectId: TOID; RelationId: Integer):
  TOID;
begin
  with QueryManager.SelectRecords(TargetObject.TableName,
    [ObjectIdField], MakeIntFieldValues([OwnerIdField, ParentObjectId,
    RelationIdField, ModelRelation.Id])) do
  try
    if IsEmpty then
      Result := 0
    else
      Result := Fields[0].AsInteger;
  finally
    Free
  end;
end;

function TCompositionRelation.GetReferencingObjects(AChildObjectId: TOID):
  TDataSet;
begin
  Result := QueryManager.SelectRecords(TargetObject.TableName, [OwnerIdField],
    MakeIntFieldValues([ObjectIdField, AChildObjectId,
    RelationIdField, ModelRelation.Id]),
    Format('[%s] <> 0', [OwnerIdField]));
end;

function TCompositionRelation.GetReferencedObjects(ParentObjectId: TOID):
  TDataSet;
begin
  Result := QueryManager.SelectRecords(TargetObject.TableName, [ObjectIdField],
    MakeIntFieldValues([OwnerIdField, ParentObjectId,
      RelationIdField, ModelRelation.Id]));
end;

function TCompositionRelation.GetJoinText(LeftExpression, RightExpression,
  ParentTable, ChildTable: String; JoinType: TJoinType; AdditionalConditions:
  String): String;
begin
  Result := JoinTables(LeftExpression, RightExpression, JoinType,
    ParentTable, ChildTable, [ObjectIdField, OwnerIdField, ModelRelation.Id, RelationIdField], AdditionalConditions);
end;

function TCompositionRelation.GetMasterDetailText(ChildTable: String; var
  FromExpr, RelationIdTable, RelationIdFieldName: String): String;
begin
  FromExpr := '';
  Result := Format('{%s}.[%s] = %s AND {%s}.[%s] = %d',
    [ChildTable, OwnerIdField, '%s', ChildTable, RelationIdField, ModelRelation.Id]);
  RelationIdTable := ChildTable;
  RelationIdFieldName := ObjectIdField;
end;

{
***************************** TAggregationRelation *****************************
}

{{
        GetOwner,
SetOwner.
}
constructor TAggregationRelation.Create(AQueryManager: TovQueryManager;
  AModelObject: TModelObject);
begin
  inherited Create(AQueryManager, nil);
  FModelObject := AModelObject;
end;

function TAggregationRelation.CreateRelation(ParentObjectId, ChildObjectId:
  TOID): TOID;
begin
  QueryManager.UpdateRecord(TargetObject.TableName,
    ObjectIdField, ChildObjectId,
    MakeIntFieldValues([ModelRelation.FieldName, ParentObjectId]));
  Result := ChildObjectId;
end;

procedure TAggregationRelation.DeleteRelation(ParentObjectId, ChildObjectId,
  RelationId: TOID);
begin
  QueryManager.UpdateRecord(TargetObject.TableName,
    ObjectIdField, ChildObjectId,
    MakeIntFieldValues([ModelRelation.FieldName, Null]));
end;

function TAggregationRelation.GetJoinText(LeftExpression, RightExpression,
  ParentTable, ChildTable: String; JoinType: TJoinType; AdditionalConditions:
  String): String;
begin
  Result := JoinTables(LeftExpression, RightExpression, JoinType,
    ParentTable, ChildTable, [ObjectIdField, ModelRelation.FieldName], AdditionalConditions);
end;

function TAggregationRelation.GetMasterDetailText(ChildTable: String; var
  FromExpr, RelationIdTable, RelationIdFieldName: String): String;
begin
  FromExpr := '';
  Result := Format('{%s}.[%s] = %s', [ChildTable, ModelRelation.FieldName, '%s']);
  RelationIdTable := ChildTable;
  RelationIdFieldName := ObjectIdField;
end;

{{
    ParentObjectId  
 RelationId     ChildObjectId.
}
procedure TAggregationRelation.GetOwner(ChildObjectId: TOID; var
  ParentObjectId, RelationId: TOID);
begin
  with QueryManager.SelectRecords(ModelObject.TableName,
    [ModelRelation.FieldName, ObjectIdField],
    MakeIntFieldValues([ObjectIdField, ChildObjectId])) do
  try
    if IsEmpty then begin
      ParentObjectId := 0;
      RelationId := 0
    end else begin
      ParentObjectId := Fields[0].AsInteger;
      RelationId := Fields[1].AsInteger;
    end
  finally
    Free
  end;
end;

function TAggregationRelation.GetReferencedObjects(ParentObjectId: TOID):
  TDataSet;
begin
  Result := QueryManager.SelectRecords(TargetObject.TableName,
    [ObjectIdField], MakeIntFieldValues([ModelRelation.FieldName, ParentObjectId]));
end;

function TAggregationRelation.GetReferencingObjects(AChildObjectId: TOID):
  TDataSet;
begin
  Result := QueryManager.SelectRecords(TargetObject.TableName,
    [ModelRelation.FieldName], MakeIntFieldValues([ObjectIdField, AChildObjectId]),
    Format('[%s] <> 0', [ModelRelation.FieldName]));
end;

function TAggregationRelation.GetRelation(ParentObjectId: TOID; RelationId:
  Integer): TOID;
begin
  with QueryManager.SelectRecords(TargetObject.TableName,
    [ObjectIdField], MakeIntFieldValues([ModelRelation.FieldName, ParentObjectId])) do
  try
    if IsEmpty then
      Result := 0
    else
      Result := Fields[0].AsInteger;
  finally
    Free
  end;
end;

function TAggregationRelation.HasReferencingObjects(AChildObjectId: TOID):
  Boolean;
begin
  Result := QueryManager.CountRecords(
    TargetObject.TableName, ModelRelation.FieldName,
    MakeIntFieldValues([ObjectIdField, AChildObjectId]),
     Format('Not [%s] is null', [ModelRelation.FieldName])) > 0
end;

{{
    ChildObjectId   
    ParentObjectId      
RelationId.
}
procedure TAggregationRelation.SetOwner(ChildObjectId: TOID; ParentObjectId,
  RelationId: TOID);
begin
  if ParentObjectId <> 0 then
    QueryManager.UpdateRecord(ModelObject.TableName,
      ObjectIdField, ChildObjectId,
      MakeIntFieldValues([ModelRelation.FieldName, ParentObjectId]))
  else
    QueryManager.UpdateRecord(ModelObject.TableName,
      ObjectIdField, ChildObjectId,
      MakeIntFieldValues([ModelRelation.FieldName, Null]))
end;

{
****************************** TReferenceRelation ******************************
}
{<<Category>>: No category}
function TReferenceRelation.CreateRelation(ParentObjectId, ChildObjectId: TOID):
  TOID;
begin
  QueryManager.UpdateRecord(SourceObject.TableName,
    ObjectIdField, ParentObjectId,
    MakeIntFieldValues([ModelRelation.FieldName, ChildObjectId]));
  Result := 0;
end;

procedure TReferenceRelation.DeleteRelation(ParentObjectId, ChildObjectId,
  RelationId: TOID);
begin
  QueryManager.UpdateRecord(SourceObject.TableName,
    ObjectIdField, ParentObjectId,
    MakeIntFieldValues([ModelRelation.FieldName, Null]));
end;

function TReferenceRelation.GetRelation(ParentObjectId: TOID; RelationId: Integer):
  TOID;
begin
  with QueryManager.SelectRecords(SourceObject.TableName,
    [ModelRelation.FieldName], MakeIntFieldValues([ObjectIdField, ParentObjectId])) do
  try
    if IsEmpty then
      raise EObjectNotFound.CreateFmt('   "%s"  ' +
        ' %d   "%s"  ',
        [SourceObject.Name, ParentObjectId, ModelRelation.Name]);
    Result := Fields[0].AsInteger;
  finally
    Free
  end;
end;

procedure TReferenceRelation.UpdateRelation(ParentObjectId: TOID; RelationId: Integer;
  NewChildId: TOID);
begin
  if NewChildId <> 0 then
    QueryManager.UpdateRecord(SourceObject.TableName,
      ObjectIdField, ParentObjectId,
      MakeIntFieldValues([ModelRelation.FieldName, NewChildId]))
  else
    QueryManager.UpdateRecord(SourceObject.TableName,
      ObjectIdField, ParentObjectId,
      MakeIntFieldValues([ModelRelation.FieldName, Null]))
end;

function TReferenceRelation.HasReferencingObjects(AChildObjectId: TOID):
  Boolean;
begin
  Result := QueryManager.CountRecords(SourceObject.TableName,
    ObjectIdField, MakeIntFieldValues([ModelRelation.FieldName, AChildObjectId])) > 0
end;

function TReferenceRelation.GetReferencingObjects(AChildObjectId: TOID):
  TDataSet;
begin
  Result := QueryManager.SelectRecords(SourceObject.TableName,
    [ObjectIdField], MakeIntFieldValues([ModelRelation.FieldName, AChildObjectId]));
end;

function TReferenceRelation.GetReferencedObjects(ParentObjectId: TOID):
  TDataSet;
begin
  Result := QueryManager.SelectRecords(SourceObject.TableName,
    [ModelRelation.FieldName], MakeIntFieldValues([ObjectIdField, ParentObjectId]));
  if Result.Fields[0].IsNull then FreeAndNil(Result);
end;

function TReferenceRelation.GetJoinText(LeftExpression, RightExpression,
  ParentTable, ChildTable: String; JoinType: TJoinType; AdditionalConditions:
  String): String;
begin
  Result := JoinTables(LeftExpression, RightExpression, JoinType,
    ParentTable, ChildTable, [ModelRelation.FieldName, ObjectIdField], AdditionalConditions);
end;

function TReferenceRelation.GetMasterDetailText(ChildTable: String; var
  FromExpr, RelationIdTable, RelationIdFieldName: String): String;
begin
  FromExpr := '';
  Result := Format('{%s}.[%s] = %s', [ChildTable, ObjectIdField, '%s']);
  RelationIdTable := ChildTable;
  RelationIdFieldName := ObjectIdField;
end;

{
*************************** TGeneralizationRelation ****************************
}

function TGeneralizationRelation.GetJoinText(LeftExpression, RightExpression,
  ParentTable, ChildTable: String; JoinType: TJoinType; AdditionalConditions:
  String): String;
begin
  if ModelRelation.FieldName = '' then
    raise Exception.CreateFmt('       "%s"   "%s"',
      [ModelRelation.TargetObjectName, ModelRelation.SourceObjectName]);

  Result := JoinTables(LeftExpression, RightExpression, JoinType,
    ParentTable, ChildTable, [ModelRelation.FieldName, ObjectIdField]);
end;

{{
    SourceObject.TableName (  ),
       .
}
function TGeneralizationRelation.CreateRelation(ParentObjectId, ChildObjectId:
  TOID): TOID;
var
  I: Integer;
  NewValues: array of Variant;
begin
  //          
  //     (  SourceObject)
  SetLength(NewValues, (1 + ModelRelation.TargetObject.AncestorCount) * 2);
  NewValues[0] := ModelRelation.FieldName;
  NewValues[1] := ChildObjectId;

  with ModelRelation do
    for I := 0 to TargetObject.AncestorCount - 1 do
    begin
      NewValues[I * 2 + 2] := TargetObject.AncestorRelations[I].FieldName;
      NewValues[I * 2 + 3] := GetAncestor(TargetObject, ChildObjectId, TargetObject.AncestorRelations[I])
    end;

  QueryManager.UpdateRecord(SourceObject.TableName,
    ObjectIdField, ParentObjectId, MakeIntFieldValues(NewValues));
  Result := 0;
end;

{{
      ParentObjectId.
}
function TGeneralizationRelation.GetRelation(ParentObjectId: TOID; RelationId:
  Integer): TOID;
begin
  Result := GetAncestor(SourceObject, ParentObjectId, ModelRelation)
end;

{{
     AObjectId, 
 AModelObject.
}
function TGeneralizationRelation.GetAncestor(AModelObject: TModelObject;
  AObjectId: TOID; AModelRelation: TModelRelation): TOID;
begin
  with QueryManager.SelectRecords(AModelObject.TableName,
    [AModelRelation.FieldName], MakeIntFieldValues([ObjectIdField, AObjectId])) do
  try
    if IsEmpty then
      raise Exception.CreateFmt('    "%s" %d  ', [AModelObject.FullName, AObjectId]);
    Result := Fields[0].AsInteger;
  finally
    Free
  end;
end;


end.
