{$INCLUDE oView.inc}
unit ovSQLQuery;

interface

uses SysUtils, Contnrs;

type
  TQueryExpression = class;

  TAbstractQuery = class(TObject)
  private
    FExpressions: TObjectList;
    function GetExpressionCount: Integer;
    function GetExpressions(Index: Integer): TQueryExpression;
  protected
    function GetText: String; virtual;
    procedure SetText(Value: String); virtual;
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddExpression(Expression: TQueryExpression; BeforeExpression:
      TQueryExpression = nil);
    procedure Validate; virtual; abstract;
    property ExpressionCount: Integer read GetExpressionCount;
    property Expressions[Index: Integer]: TQueryExpression read GetExpressions;
    property Text: String read GetText write SetText;
  end;

  TQueryExpressionItem = class(TObject)
  private
    FText: String;
  protected
    function GetText: String; virtual;
    procedure SetText(const Value: String); virtual;
  public
    constructor Create(const AText: String);
    property Text: String read GetText write SetText;
  end;

  TSelectExpressionItem = class(TQueryExpressionItem)
  private
    FExpression: String;
    FAlias: String;
  protected
    function GetText: String; override;
    procedure SetText(const Value: String); override;
  public
    property Expression: String read FExpression write FExpression;
    property Alias: String read FAlias write FAlias;
  end;

  TFromExpressionItem = class(TQueryExpressionItem)
  private
    FExpression: String;
    FAlias: String;
  protected
    function GetText: String; override;
    procedure SetText(const Value: String); override;
  public
    property Expression: String read FExpression write FExpression;
    property Alias: String read FAlias write FAlias;
  end;

  TQueryExpression = class(TObject)
  private
    FItems: TObjectList;
    FItemSeparator: String;
    FKeyword: String;
    function GetItemCount: Integer;
    function GetItems(Index: Integer): TQueryExpressionItem;
    function GetText: String;
    procedure SetText(Value: String);
  protected
    function CreateItem(ItemText: String): TQueryExpressionItem; virtual;
    procedure SetItemsText(Value: String); virtual;
    function GetItemsText: String; virtual;
  public
    {{
       . AKeyword   ,  
       .      
      ,  .  ABeforeExpression
            . 
       nil,       
      .
    }
    constructor Create(AQuery: TAbstractQuery; AKeyword: String; ASeparator: String
      = ''; ABeforeExpression: TQueryExpression = nil);
    destructor Destroy; override;
    procedure Add(Value: String);
    function Find(ItemText: String): TQueryExpressionItem;
    property ItemCount: Integer read GetItemCount;
    property Items[Index: Integer]: TQueryExpressionItem read GetItems;
    property ItemSeparator: String read FItemSeparator write FItemSeparator;
    property Keyword: String read FKeyword write FKeyword;
    property Text: String read GetText write SetText;
    property ItemsText: String read GetItemsText write SetItemsText;
  end;

  TSelectExpression = class(TQueryExpression)
  private
    FDistinct: Boolean;
    function GetItems(Index: Integer): TSelectExpressionItem;
    {$IFDEF Delphi2006} inline; {$ENDIF}
  protected
    function CreateItem(ItemText: String): TQueryExpressionItem; override;
    function GetItemsText: String; override;
    procedure SetItemsText(Value: String); override;
  public              
    property Distinct: Boolean read FDistinct write FDistinct;
    property Items[Index: Integer]: TSelectExpressionItem read GetItems;
  end;

  TFromExpression = class(TQueryExpression)
  private
    function GetItems(Index: Integer): TFromExpressionItem;
    {$IFDEF Delphi2006} inline; {$ENDIF}
  protected
    function CreateItem(ItemText: String): TQueryExpressionItem; override;
  public
    property Items[Index: Integer]: TFromExpressionItem read GetItems;
  end;

  TSelectQuery = class(TAbstractQuery)
  private
    FFrom: TFromExpression;
    FGroupBy: TQueryExpression;
    FHaving: TQueryExpression;
    FOrderBy: TQueryExpression;
    FSelect: TSelectExpression;
    FWhere: TQueryExpression;
  protected
  public
    constructor Create;
    procedure Validate; override;
    property From: TFromExpression read FFrom;
    property GroupBy: TQueryExpression read FGroupBy;
    property Having: TQueryExpression read FHaving;
    property OrderBy: TQueryExpression read FOrderBy;
    property Select: TSelectExpression read FSelect;
    property Where: TQueryExpression read FWhere;
  end;

  TUpdateQuery = class(TAbstractQuery)
  private
    FUpdateExpression: TQueryExpression;
    FWhereExpression: TQueryExpression;
    FSetExpression: TQueryExpression;
  protected
  public
    constructor Create;
    procedure Validate; override;
    property UpdateExpression: TQueryExpression read FUpdateExpression;
    property SetExpression: TQueryExpression read FSetExpression;
    property WhereExpression: TQueryExpression read FWhereExpression;
  end;

  TDeleteQuery = class(TAbstractQuery)
  private
    FDelete: TQueryExpression;
    FWhere: TQueryExpression;
  protected
  public
    constructor Create;
    property Delete: TQueryExpression read FDelete;
    property Where: TQueryExpression read FWhere;
  end;

  TAbstractQueryBuilder = class(TObject)
  private
    FQuery: TAbstractQuery;
    function GetText: String;
    procedure SetText(Value: String);
  public
    destructor Destroy; override;
    property Query: TAbstractQuery read FQuery write FQuery;
    property Text: String read GetText write SetText;
  end;

  TSQLQueryBuilder = class(TAbstractQueryBuilder)
  end;

function SkipKeyword(Text, Keyword: String; var KeywordPresent: Boolean):
  String;

implementation

uses Variants;

{{
   Text   Keyword,     
 .   Text      
     .
}
function FindKeyword(const Text, Keyword: String): Integer;
var
  P, I: Integer;
  S: String;
begin
  P := Pos(' ', Keyword);
  if P = 0 then Result := Pos(Keyword, Text)
  else begin
    Result := Pos(Copy(Keyword, 1, P-1), Text);
    if Result > 0 then
    begin
      //      
      I := Result + P;
      while (I <= Length(Text)) and (Text[I] = ' ') do Inc(I);
      S := Copy(Keyword, P+1, Length(Keyword)); //    
      if Copy(Text, I, Length(S)) <> S then Result := 0
    end;
  end;
end;

{{
  Text     Keyword.  
    (  ).  KeywordPresent 
       .
}
function SkipKeyword(Text, Keyword: String; var KeywordPresent: Boolean):
  String;
var
  StartPos, EndPos, P, I: Integer;
  S: String;
begin
  S := Text;
  Text := UpperCase(Text);
  Keyword := UpperCase(Keyword);

  StartPos := 1;
  while (StartPos <= Length(Text)) and (Text[StartPos] = ' ') do Inc(StartPos);

  P := Pos(' ', Keyword);
  if P = 0 then
  begin
    //    
    KeywordPresent := Copy(Text, StartPos, Length(Keyword)) = Keyword;
    EndPos := StartPos + Length(Keyword) - 1;
  end
  else begin
    //    
    I := StartPos + P - 1;
    while (I <= Length(Text)) and (Text[I] = ' ') do Inc(I);
    KeywordPresent :=
      //     
      (Copy(Text, StartPos, P-1) = Copy(Keyword, 1, P-1)) and
      //     
      (Copy(Text, I, Length(Keyword)-P) = Copy(Keyword, P+1, Length(Keyword)-P));
    EndPos := I + Length(Keyword)-P - 1;
  end;

  if KeywordPresent then
    Result := Copy(S, EndPos+1, Length(S))
  else
    Result := S;
end;

const
  LeftTerminators: array [1..3] of Char  = ('''', '"', '[');
  RightTerminators: array [1..3] of Char  = ('''', '"', ']');

{{
    C      
  .  CheckLeft = True,    
  ,    - .
}
function CheckForTerminator(C: Char; CheckLeft: Boolean; var PairTerminator:
  Char): Boolean;
var
  I: Integer;
begin
  Result := False;
  if CheckLeft then
    for I := Low(LeftTerminators) to High(LeftTerminators) do
    begin
      if LeftTerminators[I] = C then
      begin
        Result := True;
        PairTerminator := RightTerminators[I];
      end
    end
  else
    for I := Low(RightTerminators) to High(RightTerminators) do
    begin
      if RightTerminators[I] = C then
      begin
        Result := True;
        PairTerminator := LeftTerminators[I];
      end;
    end
end;

{{
    Text    
Separator      StartPart,    -  RestPart.
 ,     ,
  LeftTerminators  RightTerminators.  
 ,      StartPart.
}
procedure ExtractLeftPart(Text, Separator: String;
  var StartPart, RestPart: String);
var
  CurrPos, TextLength, SeparatorLenght: Integer;
  RightTerminator: Char;
begin
  TextLength := Length(Text);
  SeparatorLenght := Length(Separator);

  CurrPos := 1;
  while CurrPos <= TextLength do
  begin
    if CheckForTerminator(Text[CurrPos], True, RightTerminator) then
    begin
      Inc(CurrPos);
      while (CurrPos <= TextLength) and (Text[CurrPos] <> RightTerminator) do Inc(CurrPos);
    end else
    if Copy(Text, CurrPos, SeparatorLenght) = Separator then
    begin
      StartPart := Trim(Copy(Text, 1, CurrPos-1));
      RestPart := Trim(Copy(Text, CurrPos+SeparatorLenght, TextLength));
      Exit;
    end;
    Inc(CurrPos);
  end;
  StartPart := Text;
  RestPart := '';
end;

{{
,  ExtractLeftPart,     .  
 ,      RestPart.
}
procedure ExtractRightPart(Text, Separator: String;
  var StartPart, RestPart: String);
var
  CurrPos, TextLength, SeparatorLenght: Integer;
  LeftTerminator: Char;
begin
  TextLength := Length(Text);
  SeparatorLenght := Length(Separator);

  CurrPos := TextLength;
  while CurrPos > 0 do
  begin
    if CheckForTerminator(Text[CurrPos], False, LeftTerminator) then
    begin
      Dec(CurrPos);
      while (CurrPos > 0) and (Text[CurrPos] <> LeftTerminator) do Dec(CurrPos);
    end else
    if UpperCase(Copy(Text, CurrPos-SeparatorLenght, SeparatorLenght)) =
       UpperCase(Separator) then
    begin
      StartPart := Trim(Copy(Text, CurrPos, TextLength));
      RestPart := Trim(Copy(Text, 1, CurrPos-SeparatorLenght-1));
      Exit;
    end;
    Dec(CurrPos);
  end;
  StartPart := '';
  RestPart := Text;
end;

{
******************************** TAbstractQuery ********************************
}
constructor TAbstractQuery.Create;
begin
  inherited Create;
  FExpressions := TObjectList.Create;
end;

destructor TAbstractQuery.Destroy;
begin
  FExpressions.Free;
  inherited Destroy;
end;

procedure TAbstractQuery.AddExpression(Expression: TQueryExpression;
  BeforeExpression: TQueryExpression = nil);
begin
  if Assigned(BeforeExpression) then
    FExpressions.Insert(FExpressions.IndexOf(BeforeExpression), Expression)
  else
    FExpressions.Add(Expression);
end;

function TAbstractQuery.GetExpressionCount: Integer;
begin
  Result := FExpressions.Count
end;

function TAbstractQuery.GetExpressions(Index: Integer): TQueryExpression;
begin
  Result := TQueryExpression(FExpressions[Index])
end;

function TAbstractQuery.GetText: String;
var
  I: Integer;
  ExpressionText: String;
begin
  if ExpressionCount > 0 then Result := Expressions[0].Text;
  for I := 1 to ExpressionCount - 1 do
  begin
    ExpressionText := Expressions[I].Text;
    if ExpressionText <> '' then
      Result := Result + ' ' + ExpressionText;
  end;
end;

{{
         
        
    .
}
procedure TAbstractQuery.SetText(Value: String);
var
  I, J, P1, P2: Integer;
begin
  for I := 0 to ExpressionCount - 1 do
  begin

    P1 := FindKeyword(Value, Expressions[I].Keyword);
    if P1 > 0 then
    begin
      P2 := 0;
      for J := I+1 to ExpressionCount - 1 do
      begin
        P2 := FindKeyword(Value, Expressions[J].Keyword);
        if P2 > 0 then
          if P2 < P1 then
            raise Exception.CreateFmt(
              '  %s        %s',
              [Expressions[I].Keyword, Expressions[J].Keyword])
          else begin
            Expressions[I].Text := Copy(Value, P1, P2-P1);
            Break;
          end;
      end;
      //        -    
      if P2 = 0 then Expressions[I].Text := Copy(Value, P1, Length(Value))
    end else
      Expressions[I].Text := ''

  end;
end;

{
********************************* TSelectQuery *********************************
}
constructor TSelectQuery.Create;
begin
  inherited Create;
  FSelect := TSelectExpression.Create(Self, 'SELECT', ',');
  FFrom := TFromExpression.Create(Self, 'FROM', ',');
  FWhere := TQueryExpression.Create(Self, 'WHERE', ' AND ');
  FGroupBy := TQueryExpression.Create(Self, 'GROUP BY', ',');
  FHaving := TQueryExpression.Create(Self, 'HAVING', ',');
  FOrderBy := TQueryExpression.Create(Self, 'ORDER BY', ',');
end;

procedure TSelectQuery.Validate;
begin
  if Select.Text = '' then
    raise Exception.Create('  SELECT ');
  if From.Text = '' then
    raise Exception.Create('  FROM ');
end;

{
********************************* TUpdateQuery *********************************
}
constructor TUpdateQuery.Create;
begin
  inherited Create;
  FUpdateExpression := TQueryExpression.Create(Self, 'UPDATE');
  FSetExpression := TQueryExpression.Create(Self, 'SET', ',');
  FWhereExpression := TQueryExpression.Create(Self, 'WHERE', ' AND ');
end;

procedure TUpdateQuery.Validate;
begin
  if UpdateExpression.Text = '' then
    raise Exception.Create('  UPDATE ');
  if SetExpression.Text = '' then
    raise Exception.Create('  SET ');
end;

{
********************************* TDeleteQuery *********************************
}
constructor TDeleteQuery.Create;
begin
  inherited Create;
  FDelete := TQueryExpression.Create(Self, 'DELETE FROM');
  FWhere := TQueryExpression.Create(Self, 'WHERE', ',');
end;

{
******************************* TQueryExpression *******************************
}
constructor TQueryExpression.Create(AQuery: TAbstractQuery; AKeyword: String;
  ASeparator: String = ''; ABeforeExpression: TQueryExpression = nil);
begin
  inherited Create;
  FKeyword := AKeyword;
  FItemSeparator := ASeparator;
  FItems := TObjectList.Create;
  AQuery.AddExpression(Self, ABeforeExpression);
end;

destructor TQueryExpression.Destroy;
begin
  FItems.Free;
  inherited Destroy;
end;

function TQueryExpression.GetItemCount: Integer;
begin
  Result := FItems.Count
end;

function TQueryExpression.GetItems(Index: Integer): TQueryExpressionItem;
begin
  Result := TQueryExpressionItem(FItems[Index])
end;

function TQueryExpression.CreateItem(ItemText: String): TQueryExpressionItem;
begin
  Result := TQueryExpressionItem.Create(ItemText)
end;

function TQueryExpression.GetText: String;
begin
  Result := ItemsText;
  if Result <> '' then Result := Keyword + ' ' + Result;
end;

{{
      SetItemsText
}
procedure TQueryExpression.SetText(Value: String);
var
  KeywordPresent: Boolean;
begin
  ItemsText := SkipKeyword(Value, Keyword, KeywordPresent)
end;

{{
        .
}
function TQueryExpression.GetItemsText: String;
var
  I: Integer;
begin
  if ItemCount > 0 then
  begin
    Result := Items[0].Text;
    for I := 1 to ItemCount - 1 do
      Result := Result + ItemSeparator + Items[I].Text;
  end else
    Result := '';
end;

{{
         
        
TQueryExpressionItem.       
""  '' .
}
procedure TQueryExpression.SetItemsText(Value: String);
begin
  FItems.Clear;
  Add(Trim(Value));
end;

procedure TQueryExpression.Add(Value: String);
var
  ItemText, S: String;
begin
  while Value <> '' do
  begin
    ExtractLeftPart(Value, ItemSeparator, ItemText, S);
    Value := S;
    FItems.Add(CreateItem(Trim(ItemText)));
  end;
end;

function TQueryExpression.Find(ItemText: String): TQueryExpressionItem;
var
  I: Integer;
begin
  for I := 0 to ItemCount - 1 do
    if UpperCase(ItemText) = UpperCase(Items[I].Text) then
    begin
      Result := Items[I];
      Exit;
    end;
  Result := nil;
end;

{
***************************** TQueryExpressionItem *****************************
}

constructor TQueryExpressionItem.Create(const AText: String);
begin
  inherited Create;
  Text := AText;
end;

function TQueryExpressionItem.GetText: String;
begin
  Result := FText;
end;

procedure TQueryExpressionItem.SetText(const Value: String);
begin
  FText := Value;
end;

{
******************** TSelectExpression & TSelectExpressionItem *****************
}

function TSelectExpression.CreateItem(ItemText: String): TQueryExpressionItem;
begin
  Result := TSelectExpressionItem.Create(ItemText)
end;

function TSelectExpression.GetItems(Index: Integer): TSelectExpressionItem;
begin
  Result := TSelectExpressionItem(inherited Items[Index])
end;

function TSelectExpression.GetItemsText: String;
begin
  Result := inherited GetItemsText;
  if Distinct then Result := 'DISTINCT ' + Result;
end;

procedure TSelectExpression.SetItemsText(Value: String);
begin
  inherited SetItemsText(SkipKeyword(Value, 'DISTINCT', FDistinct));
end;

function TSelectExpressionItem.GetText: String;
begin
  Result := Expression;
  if Alias <> '' then Result := Result + ' as ' + Alias;
end;

procedure TSelectExpressionItem.SetText(const Value: String);
begin
  ExtractRightPart(Value, ' as ', FAlias, FExpression)
end;

{
******************** TFromExpression & TFromExpressionItem *********************
}

function TFromExpression.CreateItem(ItemText: String): TQueryExpressionItem;
begin
  Result := TFromExpressionItem.Create(ItemText)
end;

function TFromExpression.GetItems(Index: Integer): TFromExpressionItem;
begin
  Result := TFromExpressionItem(inherited Items[Index])
end;

function TFromExpressionItem.GetText: String;
begin
  Result := Expression;
  if Alias <> '' then Result := Result + ' ' + Alias;
end;

procedure TFromExpressionItem.SetText(const Value: String);
begin
  ExtractRightPart(Trim(Value), ' ', FAlias, FExpression)
end;

{
**************************** TAbstractQueryBuilder *****************************
}

destructor TAbstractQueryBuilder.Destroy;
begin
  FQuery.Free;
  inherited Destroy;
end;

function TAbstractQueryBuilder.GetText: String;
begin
  if Assigned(Query) then Result := Query.Text else Result := ''
end;

{{
         
     .
}
procedure TAbstractQueryBuilder.SetText(Value: String);
begin
  FreeAndNil(FQuery);
  Value := TrimLeft(Value);
  if Value <> '' then
  begin
    if Length(Value) > 0 then
    case UpCase(Value[1]) of
      'S': if UpperCase(Copy(Value, 1, 6)) = 'SELECT' then
             FQuery := TSelectQuery.Create;
      'U': if UpperCase(Copy(Value, 1, 6)) = 'UPDATE' then
             FQuery := TUpdateQuery.Create;
      'D': if UpperCase(Copy(Value, 1, 6)) = 'DELETE' then
             FQuery := TDeleteQuery.Create;
    end;
    if Not Assigned(Query) then
      raise Exception.Create('  :'+Value)
    else
      Query.Text := Value
  end;
end;


end.
