{***************************************************************************** }
{                                     }
{ **************************************************************************** }

unit ovStdInspectorItems;

interface

uses SysUtils, Classes, ovInspector, Controls, StdCtrls, ExtCtrls, Forms;

type
  TEditItemProperties = class(TInspectorItemProperties)
  private
    function GetMaxLength: Integer;
    procedure SetMaxLength(const Value: Integer);
    function GetHideSelection: Boolean;
    procedure SetHideSelection(const Value: Boolean);
    function GetPasswordChar: Char;
    procedure SetPasswordChar(const Value: Char);
    function GetCharCase: TEditCharCase;
    procedure SetCharCase(const Value: TEditCharCase);
  protected
    function GetValue: Variant; override;
    procedure SetValue(const Value: Variant); override;
    function GetReadOnly: Boolean; override;
    procedure SetReadOnly(const Value: Boolean); override;
  public
    constructor Create(Item: TInspectorItem); override;
  published
    property MaxLength: Integer read GetMaxLength write SetMaxLength default 0;
    property HideSelection: Boolean read GetHideSelection write SetHideSelection
      default True;
    property PasswordChar: Char read GetPasswordChar write SetPasswordChar default
      #0;
    property CharCase: TEditCharCase read GetCharCase write SetCharCase default
      ecNormal;
  end;

  TButtonEditItemProperties = class(TEditItemProperties)
  private
    FButton: TControl;
    FOnButtonClick: TNotifyEvent;
    procedure HandleButtonClick(Sender: TObject);
  protected
    procedure PlaceControlAt(ALeft, ATop, AWidth, AHeight: Integer); override;
  public
    constructor Create(Item: TInspectorItem); override;
    destructor Destroy; override;
  published
    property OnButtonClick: TNotifyEvent read FOnButtonClick write FOnButtonClick;
  end;

  TCheckItemProperties = class(TInspectorItemProperties)
  private
    function GetState: TCheckBoxState;
    procedure SetState(const Value: TCheckBoxState);
    function GetAllowGrayed: Boolean;
    procedure SetAllowGrayed(const Value: Boolean);
  protected
    function GetValue: Variant; override;
    procedure SetValue(const Value: Variant); override;
  public
    constructor Create(Item: TInspectorItem); override;
  published
    property AllowGrayed: Boolean read GetAllowGrayed write SetAllowGrayed default
      False;
    property State: TCheckBoxState read GetState write SetState default cbUnchecked;
  end;

  TComboItemProperties = class(TInspectorItemProperties)
  private
    FUseItemIndex: Boolean;
    procedure SetItems(const Value: TStrings);
    function GetItems: TStrings;
    function GetStyle: TComboBoxStyle;
    procedure SetStyle(const Value: TComboBoxStyle);
    function GetAutoComplete: Boolean;
    procedure SetAutoComplete(const Value: Boolean);
    function GetAutoDropDown: Boolean;
    procedure SetAutoDropDown(const Value: Boolean);
    function GetAutoCloseUp: Boolean;
    procedure SetAutoCloseUp(const Value: Boolean);
    function GetSorted: Boolean;
    procedure SetSorted(const Value: Boolean);
  protected
    function GetValue: Variant; override;
    procedure SetValue(const AValue: Variant); override;
  public
    constructor Create(Item: TInspectorItem); override;
  published
    property AutoComplete: Boolean read GetAutoComplete write SetAutoComplete
      default True;
    property AutoDropDown: Boolean read GetAutoDropDown write SetAutoDropDown
      default False;
    property AutoCloseUp: Boolean read GetAutoCloseUp write SetAutoCloseUp default
      False;
    property Items: TStrings read GetItems write SetItems;
    property Sorted: Boolean read GetSorted write SetSorted default False;
    property Style: TComboBoxStyle read GetStyle write SetStyle default csDropDown;
    property UseItemIndex: Boolean read FUseItemIndex write FUseItemIndex default
      False;
  end;

  TMemoItemProperties = class(TInspectorItemProperties)
  private
    function GetHideSelection: Boolean;
    procedure SetHideSelection(const Value: Boolean);
    function GetScrollBars: TScrollStyle;
    procedure SetScrollBars(const Value: TScrollStyle);
    function GetWantReturns: Boolean;
    procedure SetWantReturns(const Value: Boolean);
    function GetWantTabs: Boolean;
    procedure SetWantTabs(const Value: Boolean);
    function GetWordWrap: Boolean;
    procedure SetWordWrap(const Value: Boolean);
    function GetHeight: Integer;
    procedure SetHeight(const Value: Integer);
  protected
    function GetValue: Variant; override;
    procedure SetValue(const Value: Variant); override;
    function GetReadOnly: Boolean; override;
    procedure SetReadOnly(const Value: Boolean); override;
  public
    constructor Create(Item: TInspectorItem); override;
  published
    property HideSelection: Boolean read GetHideSelection write SetHideSelection
      default True;
    property ScrollBars: TScrollStyle read GetScrollBars write SetScrollBars
      default ssNone;
    property WantReturns: Boolean read GetWantReturns write SetWantReturns default
      True;
    property WantTabs: Boolean read GetWantTabs write SetWantTabs default False;
    property WordWrap: Boolean read GetWordWrap write SetWordWrap default True;
    property Height: Integer read GetHeight write SetHeight;
  end;

  TIntegerEditItemProperties = class(TInspectorItemProperties)
  private
  protected
    function GetValue: Variant; override;
    procedure SetValue(const Value: Variant); override;
    function GetReadOnly: Boolean; override;
    procedure SetReadOnly(const Value: Boolean); override;
  public
    constructor Create(Item: TInspectorItem); override;
  end;

implementation

uses Mask, Buttons;

{ TEditItemProperties }

constructor TEditItemProperties.Create(Item: TInspectorItem);
begin
  inherited Create(Item, TEdit.Create(nil));
  with TEdit(Control) do
  begin
    BorderStyle := bsNone;
    BevelKind := bkNone;
    ParentColor := True;
  end;
end;

function TEditItemProperties.GetMaxLength: Integer;
begin
  Result := TEdit(Control).MaxLength;
end;

procedure TEditItemProperties.SetMaxLength(const Value: Integer);
begin
  TEdit(Control).MaxLength := Value;
end;

function TEditItemProperties.GetHideSelection: Boolean;
begin
  Result := TEdit(Control).HideSelection;
end;

procedure TEditItemProperties.SetHideSelection(const Value: Boolean);
begin
  TEdit(Control).HideSelection := Value;
end;

function TEditItemProperties.GetPasswordChar: Char;
begin
  Result := TEdit(Control).PasswordChar;
end;

procedure TEditItemProperties.SetPasswordChar(const Value: Char);
begin
  TEdit(Control).PasswordChar := Value;
end;

function TEditItemProperties.GetCharCase: TEditCharCase;
begin
  Result := TEdit(Control).CharCase;
end;

procedure TEditItemProperties.SetCharCase(const Value: TEditCharCase);
begin
  TEdit(Control).CharCase := Value;
end;

function TEditItemProperties.GetValue: Variant;
begin
  Result := TEdit(Control).Text
end;

procedure TEditItemProperties.SetValue(const Value: Variant);
begin
  TEdit(Control).Text := Value
end;

function TEditItemProperties.GetReadOnly: Boolean;
begin
  Result := TEdit(Control).ReadOnly
end;

procedure TEditItemProperties.SetReadOnly(const Value: Boolean);
begin
  TEdit(Control).ReadOnly := Value
end;

{ TButtonEditItemProperties }

constructor TButtonEditItemProperties.Create(Item: TInspectorItem);
begin
  inherited Create(Item);
  FButton := TBitBtn.Create(nil);
  with TBitBtn(FButton) do
  begin
    Anchors := [akRight, akTop];
    Caption := '...';
    OnClick := HandleButtonClick;
  end;
end;

destructor TButtonEditItemProperties.Destroy;
begin
  FreeAndNil(FButton);
  inherited Destroy;
end;

procedure TButtonEditItemProperties.PlaceControlAt(ALeft, ATop, AWidth,
  AHeight: Integer);
begin
  inherited PlaceControlAt(ALeft, ATop, AWidth-16, AHeight);
  FButton.Parent := Control.Parent;
  FButton.SetBounds(Control.Left + Control.Width, ATop, 16, Control.Height);
end;

procedure TButtonEditItemProperties.HandleButtonClick(Sender: TObject);
begin
  if Assigned(FOnButtonClick) then FOnButtonClick(Item)
end;

{ TIntegerEditItemProperties }

constructor TIntegerEditItemProperties.Create(Item: TInspectorItem);
begin
  inherited Create(Item, TMaskEdit.Create(nil));
  with TMaskEdit(Control) do
  begin
    BorderStyle := bsNone;
    BevelKind := bkNone;
    ParentColor := True;
    EditMask := '9999999999;0; ';
  end;
end;

function TIntegerEditItemProperties.GetValue: Variant;
begin
  Result := StrToInt(TMaskEdit(Control).Text)
end;

procedure TIntegerEditItemProperties.SetValue(const Value: Variant);
begin
  TMaskEdit(Control).Text := IntToStr(Value)
end;

function TIntegerEditItemProperties.GetReadOnly: Boolean;
begin
  Result := TMaskEdit(Control).ReadOnly
end;

procedure TIntegerEditItemProperties.SetReadOnly(const Value: Boolean);
begin
  TMaskEdit(Control).ReadOnly := Value
end;

{ TChec++kItemProperties }

constructor TCheckItemProperties.Create(Item: TInspectorItem);
begin
  inherited Create(Item, TCheckBox.Create(nil));
  with TCheckBox(Control) do
  begin
    Caption := '   ';
  end;
end;

function TCheckItemProperties.GetState: TCheckBoxState;
begin
  Result := TCheckBox(Control).State;
end;

procedure TCheckItemProperties.SetState(const Value: TCheckBoxState);
begin
  TCheckBox(Control).State := Value;
end;

function TCheckItemProperties.GetAllowGrayed: Boolean;
begin
  Result := TCheckBox(Control).AllowGrayed;
end;

procedure TCheckItemProperties.SetAllowGrayed(const Value: Boolean);
begin
  TCheckBox(Control).AllowGrayed := Value;
end;

function TCheckItemProperties.GetValue: Variant;
begin
  Result := TCheckBox(Control).Checked
end;

procedure TCheckItemProperties.SetValue(const Value: Variant);
begin
  TCheckBox(Control).Checked := Value
end;

{ TComboItemProperties }

constructor TComboItemProperties.Create(Item: TInspectorItem);
begin
  inherited Create(Item, TComboBox.Create(nil));
  with TComboBox(Control) do
  begin
    BevelKind := bkSoft;
    BevelInner := bvNone;
    BevelOuter := bvRaised;
    BevelEdges := [beRight];
    ParentColor := True;
  end;
end;

procedure TComboItemProperties.SetItems(const Value: TStrings);
begin
  TComboBox(Control).Items := Value;
end;

function TComboItemProperties.GetItems: TStrings;
begin
  Result := TComboBox(Control).Items;
end;

procedure TComboItemProperties.SetStyle(const Value: TComboBoxStyle);
begin
  TComboBox(Control).Style := Value;
end;

function TComboItemProperties.GetStyle: TComboBoxStyle;
begin
  Result := TComboBox(Control).Style;
end;

function TComboItemProperties.GetAutoComplete: Boolean;
begin
  Result := TComboBox(Control).AutoComplete;
end;

procedure TComboItemProperties.SetAutoComplete(const Value: Boolean);
begin
  TComboBox(Control).AutoComplete := Value;
end;

function TComboItemProperties.GetAutoDropDown: Boolean;
begin
  Result := TComboBox(Control).AutoDropDown;
end;

procedure TComboItemProperties.SetAutoDropDown(const Value: Boolean);
begin
  TComboBox(Control).AutoDropDown := Value;
end;

function TComboItemProperties.GetAutoCloseUp: Boolean;
begin
  Result := TComboBox(Control).AutoCloseUp;
end;

procedure TComboItemProperties.SetAutoCloseUp(const Value: Boolean);
begin
  TComboBox(Control).AutoCloseUp := Value;
end;

function TComboItemProperties.GetSorted: Boolean;
begin
  Result := TComboBox(Control).Sorted;
end;

procedure TComboItemProperties.SetSorted(const Value: Boolean);
begin
  TComboBox(Control).Sorted := Value;
end;

function TComboItemProperties.GetValue: Variant;
begin
  if UseItemIndex then
    Result := TComboBox(Control).ItemIndex
  else
    Result := TComboBox(Control).Text
end;

procedure TComboItemProperties.SetValue(const AValue: Variant);
begin
  with TComboBox(Control) do
  if UseItemIndex then
    ItemIndex := AValue
  else
    if Style = csDropDownList then
      ItemIndex := Items.IndexOf(AValue)  //     Text  
    else
      Text := AValue
end;

{ TMemoItemProperties }

constructor TMemoItemProperties.Create(Item: TInspectorItem);
begin
  inherited Create(Item, TMemo.Create(nil));
  with TMemo(Control) do
  begin
    BorderStyle := bsNone;
    BevelKind := bkNone;
    ParentColor := True;
  end;
end;

function TMemoItemProperties.GetHideSelection: Boolean;
begin
  Result := TMemo(Control).HideSelection;
end;

procedure TMemoItemProperties.SetHideSelection(const Value: Boolean);
begin
  TMemo(Control).HideSelection := Value;
end;

function TMemoItemProperties.GetScrollBars: TScrollStyle;
begin
  Result := TMemo(Control).ScrollBars;
end;

procedure TMemoItemProperties.SetScrollBars(const Value: TScrollStyle);
begin
  TMemo(Control).ScrollBars := Value;
end;

function TMemoItemProperties.GetWantReturns: Boolean;
begin
  Result := TMemo(Control).WantReturns;
end;

procedure TMemoItemProperties.SetWantReturns(const Value: Boolean);
begin
  TMemo(Control).WantReturns := Value;
end;

function TMemoItemProperties.GetWantTabs: Boolean;
begin
  Result := TMemo(Control).WantTabs;
end;

procedure TMemoItemProperties.SetWantTabs(const Value: Boolean);
begin
  TMemo(Control).WantTabs := Value;
end;

function TMemoItemProperties.GetWordWrap: Boolean;
begin
  Result := TMemo(Control).WordWrap;
end;

procedure TMemoItemProperties.SetWordWrap(const Value: Boolean);
begin
  TMemo(Control).WordWrap := Value;
end;

function TMemoItemProperties.GetHeight: Integer;
begin
  Result := Control.Height;
end;

procedure TMemoItemProperties.SetHeight(const Value: Integer);
begin
  Control.Height := Value;
end;

function TMemoItemProperties.GetValue: Variant;
begin
  Result := TMemo(Control).Text
end;

procedure TMemoItemProperties.SetValue(const Value: Variant);
begin
  TMemo(Control).Text := Value
end;

function TMemoItemProperties.GetReadOnly: Boolean;
begin
  Result := TMemo(Control).ReadOnly
end;

procedure TMemoItemProperties.SetReadOnly(const Value: Boolean);
begin
  TMemo(Control).ReadOnly := Value
end;

initialization
  RegisterPropertiesClass(TEditItemProperties);
  RegisterPropertiesClass(TIntegerEditItemProperties);
  RegisterPropertiesClass(TCheckItemProperties);
  RegisterPropertiesClass(TComboItemProperties);
  RegisterPropertiesClass(TMemoItemProperties);
  RegisterPropertiesClass(TButtonEditItemProperties);
end.
