unit ovStdOperations;

interface

uses Classes, SysUtils, Forms, ovIObjects, ovSysOperations;

type
  TEditObjectOperationMethod = function(AObject: IovObject; IsNew: Boolean; const
    Args: array of Variant): Boolean of object;
  TExecObjectOperationMethod = function (AObject: IovObject; AOperation: String;
    const Args: array of Variant): Variant;
  TSelectObjectOperationMethod = function (AClassName: String;
    const Args: array of Variant; var AObject: IovObject): Boolean of object;
  TUserOperationMethod = function (const Args: array of Variant): Boolean of object;

//    
procedure RegisterObjectEditor(AClassName: String;
  AFormClass: TFormClass; AMethodProc: Pointer;
  AutoCreate: Boolean = False; FreeAfterUse: Boolean = True);
procedure UnregisterObjectEditor(AClassName: String);
procedure RegisterEditObjectOperation(AClassName: String; AOperationArgCount:
  SmallInt; AOperation: TSysOperation);
procedure UnregisterEditObjectOperation(AClassName: String; AOperationArgCount:
  SmallInt);

function EditObject(AObject: IovObject; const Args: array of Variant;
  CloseAction: TCloseFormAction = caDefault): Boolean;
{{
  ,   EditObject,      
 IsNew   True.      , 
      NewObject, .. 
  -    ,   
,     ,    .
}
function EditObjectAsNew(AObject: IovObject; const Args: array of Variant;
   CloseAction: TCloseFormAction = caDefault): Boolean;
function NewObject(AModelName, AClassName: String; const Args: array of Variant;
  var AObject: IovObject; CloseAction: TCloseFormAction = caDefault): Boolean;
  overload;
{{
,    ,   DefaultModel.
}
function NewObject(AClassName: String; const Args: array of Variant; var
  AObject: IovObject; CloseAction: TCloseFormAction = caDefault): Boolean;
  overload;
function NewChildObject(AParentObject: IovObject; ARelationName: String; const
  Args: array of Variant; var AObject: IovObject; CloseAction: TCloseFormAction
  = caDefault): Boolean;

//  
procedure RegisterObjectSelector(AClassName: String; AFormClass: TFormClass;
  AMethodProc: Pointer; AutoCreate: Boolean = False; FreeAfterUse: Boolean =
  True);
procedure RegisterSelectObjectOperation(AClassName: String; AOperationArgCount:
  SmallInt; AOperation: TSysOperation);
procedure UnregisterSelectObjectOperation(AClassName: String;
  AOperationArgCount: SmallInt);
function SelectObject(AClassName: String; const Args: array of Variant; var
  AObject: IovObject; CloseAction: TCloseFormAction = caDefault): Boolean;

//  
procedure RegisterUserOperation(AOperationName: String; AOperationArgCount:
  SmallInt; AOperation: TSysOperation);
procedure UnregisterUserOperation(AOperationName: String; AOperationArgCount:
  SmallInt);
function ExecUserOperation(AOperationName: String; const Args: array of Variant;
  CloseAction: TCloseFormAction = caDefault): Boolean;

procedure RegisterObjectOperation(AClassName, AOperation: String; AMethodProc:
  Pointer);
procedure UnregisterObjectOperation(AClassName, AOperation: String);

function ExecObjectOperation(AObject: IovObject; AOperation: String; const
  Args: array of Variant): Variant;

function NewObjectCopy(SourceObject: IovObject; var AObject: IovObject; const
  Args: array of Variant; CopyAttributes: String = '*'; DontCopyAttributes:
  String = ''; CopyRelations: String = '*'; DontCopyRelations: String = '';
  CloseAction: TCloseFormAction = caDefault): Boolean; overload;

function NewObjectCopy(SourceObject: IovObject; const Args: array of Variant;
  var AObject: IovObject; CloseAction: TCloseFormAction = caDefault): Boolean;
  overload;



implementation

uses Variants, ovComponents;

const
  otEditObject    = 1;
  otSelectObject  = 2;
  otUserOperation = 3;
  otObjectOperation = 4;

{ **************************************************************************** }
{                                               }
{ **************************************************************************** }

procedure RegisterObjectEditor(AClassName: String;
  AFormClass: TFormClass; AMethodProc: Pointer;
  AutoCreate: Boolean = False; FreeAfterUse: Boolean = True);
begin
  RegisterEditObjectOperation(AClassName, -1,
    TSysOperation.Create(AFormClass, AMethodProc, AutoCreate, FreeAfterUse));
end;

procedure UnregisterObjectEditor(AClassName: String);
begin
  UnregisterSysOperation(otEditObject, AClassName, -1);
end;

procedure RegisterEditObjectOperation(AClassName: String; AOperationArgCount:
  SmallInt; AOperation: TSysOperation);
begin
  RegisterSysOperation(otEditObject, AClassName, AOperationArgCount, AOperation);
end;

procedure UnregisterEditObjectOperation(AClassName: String; AOperationArgCount:
  SmallInt);
begin
  UnregisterSysOperation(otEditObject, AClassName, AOperationArgCount);
end;

function EditObjectInternal(AObject: IovObject; IsNew: Boolean;
  const Args: array of Variant; CloseAction: TCloseFormAction): Boolean;
var
  Method: TMethod;
  Operation: TSysOperation;
begin
  Operation := GetSysOperation(otEditObject, AObject.ClassName, High(Args) + 1);
  Operation.CreateForm;
  try
    Method := Operation.GetMethod;
    Result := TEditObjectOperationMethod(Method)(AObject, IsNew, Args);
  finally
    Operation.ReleaseForm(CloseAction);
  end;
end;

function EditObject(AObject: IovObject; const Args: array of Variant;
  CloseAction: TCloseFormAction = caDefault): Boolean;
begin
  Result := EditObjectInternal(AObject, False, Args, CloseAction)
end;

function EditObjectAsNew(AObject: IovObject; const Args: array of Variant;
   CloseAction: TCloseFormAction = caDefault): Boolean;
begin
  Result := EditObjectInternal(AObject, True, Args, CloseAction)
end;

function NewObject(AModelName, AClassName: String; const Args: array of Variant;
  var AObject: IovObject; CloseAction: TCloseFormAction = caDefault): Boolean;
begin
  Result := False;
  AObject := ObjectSpace(AModelName).CreateObject(AClassName);
  try
    Result := EditObjectInternal(AObject, True, Args, CloseAction)
  finally
    //     
    if Not Result then begin
      AObject.Delete;
      AObject := nil;
    end
  end;
end;

function NewObject(AClassName: String; const Args: array of Variant; var
  AObject: IovObject; CloseAction: TCloseFormAction = caDefault): Boolean;
begin
  Result := NewObject(DefaultModel.ModelName, AClassName, Args, AObject, CloseAction)
end;

function NewObjectCopy(SourceObject: IovObject; const Args: array of Variant;
  var AObject: IovObject; CloseAction: TCloseFormAction = caDefault): Boolean;
  overload;
begin
  Result := False;
  AObject := SourceObject.Copy;
  try
    Result := EditObjectInternal(AObject, True, Args, CloseAction)
  finally
    //     
    if Not Result then begin
      AObject.Delete;
      AObject := nil;
    end
  end;
end;

{{
,      NewObjectCopy,   
  ,   
}
function NewObjectCopy(SourceObject: IovObject; var AObject: IovObject; const
  Args: array of Variant; CopyAttributes: String = '*'; DontCopyAttributes:
  String = ''; CopyRelations: String = '*'; DontCopyRelations: String = '';
  CloseAction: TCloseFormAction = caDefault): Boolean;
begin
  Result := False;
  AObject := SourceObject.ParentClass.CreateObject;
  try
    AObject.Assign(SourceObject, CopyAttributes, DontCopyAttributes, CopyRelations, DontCopyRelations);
    Result := EditObjectInternal(AObject, True, Args, CloseAction)
  finally
    //     
    if Not Result then begin
      AObject.Delete;
      AObject := nil;
    end
  end;
end;

function NewChildObject(AParentObject: IovObject; ARelationName: String; const
  Args: array of Variant; var AObject: IovObject; CloseAction: TCloseFormAction
  = caDefault): Boolean;
begin
  Result := False;
  AObject := AParentObject[ARelationName].AsObjects.Add;
  try
    Result := EditObjectInternal(AObject, True, Args, CloseAction)
  finally
    //     
    if Not Result then begin
      AParentObject[ARelationName].AsObjects.Delete(AObject);
      AObject := nil;
    end
  end;
end;

{ **************************************************************************** }
{ **************************************************************************** }

//  

procedure RegisterObjectOperation(AClassName, AOperation: String; AMethodProc:
  Pointer);
begin
  RegisterSysOperation(otObjectOperation, AClassName+'$$'+AOperation, -1,
    TSysOperation.Create(nil, AMethodProc));
end;

procedure UnregisterObjectOperation(AClassName, AOperation: String);
begin
  UnregisterSysOperation(otObjectOperation, AClassName+'$$'+AOperation, -1);
end;

function ExecObjectOperation(AObject: IovObject; AOperation: String; const
  Args: array of Variant): Variant;
var
  Method: TMethod;
  Operation: TSysOperation;
begin
  Operation := GetSysOperation(otObjectOperation,
    AObject.ClassName+'$$'+AOperation, High(Args) + 1);

  Method := Operation.GetMethod;
  Result := TExecObjectOperationMethod(Method.Code)(AObject, AOperation, Args);
end;

//  

procedure RegisterObjectSelector(AClassName: String; AFormClass: TFormClass;
  AMethodProc: Pointer; AutoCreate: Boolean = False; FreeAfterUse: Boolean =
  True);
begin
  RegisterSelectObjectOperation(AClassName, -1,
    TSysOperation.Create(AFormClass, AMethodProc, AutoCreate, FreeAfterUse));
end;

procedure RegisterSelectObjectOperation(AClassName: String; AOperationArgCount:
  SmallInt; AOperation: TSysOperation);
begin
  RegisterSysOperation(otSelectObject, AClassName, AOperationArgCount, AOperation);
end;

procedure UnregisterSelectObjectOperation(AClassName: String;
  AOperationArgCount: SmallInt);
begin
  UnregisterSysOperation(otSelectObject, AClassName, AOperationArgCount);
end;

function SelectObject(AClassName: String; const Args: array of Variant; var
  AObject: IovObject; CloseAction: TCloseFormAction = caDefault): Boolean;
var
  Method: TMethod;
  Operation: TSysOperation;
begin
  Operation := GetSysOperation(otSelectObject, AClassName, High(Args) + 1);
  Operation.CreateForm;
  try
    Method := Operation.GetMethod;
    Result := TSelectObjectOperationMethod(Method)(AClassName, Args, AObject);
  finally
    Operation.ReleaseForm(CloseAction);
  end;
end;

//  

procedure RegisterUserOperation(AOperationName: String; AOperationArgCount:
  SmallInt; AOperation: TSysOperation);
begin
  RegisterSysOperation(otUserOperation, AOperationName, AOperationArgCount, AOperation);
end;

procedure UnregisterUserOperation(AOperationName: String; AOperationArgCount:
  SmallInt);
begin
  UnregisterSysOperation(otUserOperation, AOperationName, AOperationArgCount);
end;

function ExecUserOperation(AOperationName: String; const Args: array of Variant;
  CloseAction: TCloseFormAction = caDefault): Boolean;
var
  Method: TMethod;
  Operation: TSysOperation;
begin
  Operation := GetSysOperation(otUserOperation, AOperationName, High(Args) + 1);
  Operation.CreateForm;
  try
    Method := Operation.GetMethod;
    Result := TUserOperationMethod(Method)(Args);
  finally
    Operation.ReleaseForm(CloseAction);
  end;
end;

end.
