unit ovTypes;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, Contnrs, ovModel;

type

  TOID = Integer;

  EObjectNotFound = class (Exception)
  end;

  EDeleteObjectError = class (Exception)
  private
    FModelObject: TModelObject;
    FObjectId: Integer;
  public
    constructor Create(AMessage: String; AModelObject: TModelObject; AObjectId:
      TOID);
    property ModelObject: TModelObject read FModelObject;
    property ObjectId: Integer read FObjectId;
  end;

  EAddObjectError = class(Exception)
  public
    constructor Create(AClassName, ARelationName: String; AObjectId: TOID; ACount:
      Integer);
  end;

  EUnAssignedReference = class (Exception)
  end;

  {{
     GetModel,     
  }
  EModelNotFound = class (Exception)
  end;

  {{
     IovObject.Commit,     
     - .
  }
  EInvalidObject = class (Exception)
  end;

  TOIDList = class (TObject)
  private
    FDuplicates: Boolean;
    FObjectIdList: TList;
    FRelationIdList: TList;
    function GetCount: Integer;
    function GetObjectId(Index: Integer): TOID;
    function GetRelationId(Index: Integer): TOID;
    procedure SetObjectId(Index: Integer; Value: TOID);
    procedure SetRelationId(Index: Integer; Value: TOID);
  public
    constructor Create(ADuplicates: Boolean = True);
    destructor Destroy; override;
    procedure AddObject(ObjectId: TOID; RelationId: TOID = 0);
    procedure Clear;
    procedure DeleteObject(Index: Integer);
    function IndexOf(ObjectId: TOID): Integer;
    procedure LoadFromDataSet(DataSet: TDataSet; ObjectIdField: TField; 
      RelationIdField: TField = nil);
    procedure Add(ObjectId: TOID);
    function ToString(Separator: String = ','): String;
    property Count: Integer read GetCount;
    property Duplicates: Boolean read FDuplicates write FDuplicates;
    property ObjectId[Index: Integer]: TOID read GetObjectId write SetObjectId;
      default;
    property RelationId[Index: Integer]: TOID read GetRelationId write 
      SetRelationId;
  end;

  TObjectData = class
  private
    FObjects: TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddObject(Attribute: String; AObject: TOID);
    function GetObject(Attribute: String): TOID;
  end;

  {{
        
  IDatabaseAdapter.CreateTable (     
     ).
  }
  TTableDefs = class(TObject)
  private
    FIndexDefs: TIndexDefs;
    FDataSet: TDataSet;
    FTableName: String;
    function GetFieldDefs: TFieldDefs;
  public
    constructor Create(const ATableName: String);
    destructor Destroy; override;
    procedure AddField(FieldName: String; FieldType: TFieldType; FieldSize: Integer
      = 0; Required: Boolean = False);
    procedure AddIndex(IndexName, IndexFields: String; IndexOptions: TIndexOptions);
    property FieldDefs: TFieldDefs read GetFieldDefs;
    property IndexDefs: TIndexDefs read FIndexDefs;
    property TableName: String read FTableName;
  end;

  TovObjectRef = record
    ClassName: String;
    ObjectId: TOID;
  end;

  {{
    ,      .
  }
  TovObjectRefList = class
  private
    FObjects: TStringList;
    function GetCount: Integer;
    function GetObjects(Index: Integer): TovObjectRef;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(AClassName: String; AObjectId: TOID);
    procedure Delete(Index: Integer);
    function IndexOf(AClassName: String; AObjectId: TOID): Integer;
    property Count: Integer read GetCount;
    property Objects[Index: Integer]: TovObjectRef read GetObjects; default;
  end;

  TFieldValue =
  record
    FieldName: String;
    FieldType: TFieldType;
    FieldSize: Integer;
    Value: Variant;
  end;
  TFieldValues = array of TFieldValue;

function MakeFieldValues(const AFieldValues: array of Variant): TFieldValues;
function MakeIntFieldValues(const AFieldValues: array of Variant): TFieldValues;

implementation

uses ovConsts, ovUtils;

{{
   AFieldValues   TFieldValues. 
AFieldValues     :  ,  ,
      .
}
function MakeFieldValues(const AFieldValues: array of Variant): TFieldValues;
var
  I: Integer;
begin
  SetLength(Result, Length(AFieldValues) div 3);
  for I := 0 to High(Result) do
  begin
    Result[I].FieldName := AFieldValues[I*3];
    Result[I].FieldType := AFieldValues[I*3+1];
    Result[I].FieldSize := AFieldValues[I*3+2];
    Result[I].Value := AFieldValues[I*3+3];
  end;
end;

{{
   AFieldValues   TFieldValues. 
AFieldValues     :  ,   
  .   ,      ftInteger.
}
function MakeIntFieldValues(const AFieldValues: array of Variant): TFieldValues;
var
  I: Integer;
begin
  SetLength(Result, Length(AFieldValues) div 2);
  for I := 0 to High(Result) do
  begin
    Result[I].FieldName := AFieldValues[I*2];
    Result[I].FieldType := ftInteger;
    Result[I].FieldSize := 0;
    Result[I].Value := AFieldValues[I*2+1];
  end;
end;

{ EDeleteObjectError }

constructor EDeleteObjectError.Create(AMessage: String; AModelObject:
  TModelObject; AObjectId: TOID);
begin
  inherited Create(AMessage);
  FModelObject := AModelObject;
  FObjectId := AObjectId;
end;

constructor EAddObjectError.Create(AClassName, ARelationName: String;
  AObjectId: TOID; ACount: Integer);
begin
  inherited CreateFmt(
    '      "%s"    "%s" (id=%d).'#13+
    '    %d.',
    [ARelationName, AClassName, AObjectId, ACount]);
end;

{
*********************************** TOIDList ***********************************
}

constructor TOIDList.Create(ADuplicates: Boolean = True);
begin
  inherited Create;
  FDuplicates := ADuplicates;
  FObjectIdList := TList.Create;
  FRelationIdList := TList.Create;
end;

destructor TOIDList.Destroy;
begin
  FRelationIdList.Free;
  FObjectIdList.Free;
  inherited Destroy;
end;

procedure TOIDList.AddObject(ObjectId: TOID; RelationId: TOID = 0);
begin
  if Duplicates or (IndexOf(ObjectId) < 0) then
  begin
    FObjectIdList.Add(Pointer(ObjectId));
    FRelationIdList.Add(Pointer(RelationId));
  end;
end;

procedure TOIDList.Add(ObjectId: TOID);
begin
  if Duplicates or (IndexOf(ObjectId) < 0) then
  begin
    FObjectIdList.Add(Pointer(ObjectId));
    FRelationIdList.Add(nil);
  end;
end;

procedure TOIDList.Clear;
begin
  FObjectIdList.Clear;
  FRelationIdList.Clear;
end;

procedure TOIDList.DeleteObject(Index: Integer);
begin
  FObjectIdList.Delete(Index);
  FRelationIdList.Delete(Index);
end;

function TOIDList.GetCount: Integer;
begin
  Result := FObjectIdList.Count
end;

function TOIDList.GetObjectId(Index: Integer): TOID;
begin
  Result := Integer(FObjectIdList[Index])
end;

function TOIDList.GetRelationId(Index: Integer): TOID;
begin
  Result := Integer(FRelationIdList[Index])
end;

procedure TOIDList.SetObjectId(Index: Integer; Value: TOID);
begin
  FObjectIdList[Index] := TObject(Value)
end;

procedure TOIDList.SetRelationId(Index: Integer; Value: TOID);
begin
  FRelationIdList[Index] := TObject(Value)
end;

function TOIDList.IndexOf(ObjectId: TOID): Integer;
begin
  Result := FObjectIdList.IndexOf(Pointer(ObjectId))
end;

{{
      .    
 ObjectIdField  ,    -   RelationIdField
(  ).
}
procedure TOIDList.LoadFromDataSet(DataSet: TDataSet; ObjectIdField: TField;
  RelationIdField: TField = nil);
begin
  Clear;
  with DataSet do begin
    First;
    while Not EOF do begin
      if Not Assigned(RelationIdField)
        then AddObject(ObjectIdField.AsInteger, 0)
        else AddObject(ObjectIdField.AsInteger, RelationIdField.AsInteger);
      Next;
    end;
  end
end;

{{
      ,   Separator.
}
function TOIDList.ToString(Separator: String = ','): String;
var
  I: Integer;
begin
  Result := '';
  for I := 0 to Count - 1 do
    Result := Result + Separator + IntToStr(ObjectId[I]);
  Delete(Result, 1, Length(Separator));
end;

{ ******************************** TObjectData ******************************* }

constructor TObjectData.Create;
begin
  inherited Create;
  FObjects := TStringList.Create;
end;

destructor TObjectData.Destroy;
begin
  FObjects.Free;
  inherited Destroy;
end;

procedure TObjectData.AddObject(Attribute: String; AObject: TOID);
begin
  FObjects.AddObject(Attribute, TObject(AObject))
end;

{{
   ,    .  
  , ..   ".".      ,
      ,      
     ".".

      ,    .
}
function TObjectData.GetObject(Attribute: String): TOID;
var
  S: String;
  I: Integer;
begin
  I := FObjects.IndexOf(Attribute);
  if I < 0 then raise Exception.Create('    : ' + Attribute);

  Result := TOID(FObjects.Objects[I]);
  FObjects.Delete(I);

  for I := 0 to FObjects.Count - 1 do
  begin
    S := FObjects[I];
    if ExtractSubStr(S, '.') = Attribute then FObjects[I] := S
  end;
end;

{ TTableDefs }

constructor TTableDefs.Create(const ATableName: String);
begin
  inherited Create;
  FTableName := ATableName;
  FDataSet := TDataSet.Create(nil);
  FIndexDefs := TIndexDefs.Create(FDataSet);
end;

destructor TTableDefs.Destroy;
begin
  FreeAndNil(FIndexDefs);
  FreeAndNil(FDataSet);
  inherited Destroy;
end;

function TTableDefs.GetFieldDefs: TFieldDefs;
begin
  Result := FDataSet.FieldDefs;
end;

procedure TTableDefs.AddField(FieldName: String; FieldType: TFieldType;
  FieldSize: Integer = 0; Required: Boolean = False);
begin
  with FieldDefs do
    if IndexOf(FieldName) < 0 then
      Add(FieldName, FieldType, FieldSize, Required);
end;

procedure TTableDefs.AddIndex(IndexName, IndexFields: String; IndexOptions:
  TIndexOptions);
begin
  IndexDefs.Add(IndexName, IndexFields, IndexOptions);
end;

{ TovObjectRefList }

constructor TovObjectRefList.Create;
begin
  inherited Create;
  FObjects := TStringList.Create;
end;

destructor TovObjectRefList.Destroy;
begin
  FreeAndNil(FObjects);
  inherited Destroy;
end;

procedure TovObjectRefList.Add(AClassName: String; AObjectId: TOID);
begin
  FObjects.AddObject(AClassName, Pointer(AObjectId))
end;

procedure TovObjectRefList.Delete(Index: Integer);
begin
  FObjects.Delete(Index);
end;

function TovObjectRefList.GetCount: Integer;
begin
  Result := FObjects.Count;
end;

function TovObjectRefList.GetObjects(Index: Integer): TovObjectRef;
begin
  Result.ClassName := FObjects.Strings[Index];
  Result.ObjectId := Integer(FObjects.Objects[Index]);
end;

function TovObjectRefList.IndexOf(AClassName: String; AObjectId: TOID): Integer;
var
  I: Integer;
begin
  with FObjects do
    for I := 0 to Count - 1 do
      if (FObjects[I] = AClassName) and (Integer(FObjects.Objects[I]) = AObjectId)
      then begin
        Result := I;
        Exit;
      end;
  Result := -1;
end;

end.
