unit ovUtils;

interface

uses SysUtils, Windows, Classes, ovComponents, DB, ovModel;

function StringToList(Separator, S: String): TStrings;
function ListToString(L: TStrings; Separator, ItemFormat: String): String;

function MakeNodeText(Name, TypeName: String): String;

function GetNodeName(Text: String): String;

procedure AttributesToList(Attributes, Separator: String; AttributeNames:
  TStrings); overload;

function ParseAttribute(var FirstPart, RestPart: String): Boolean;

function ExtractSubStr(var S: String; Delimiter: String): String;
function CombineSubStr(SubStr1, Delimiter, SubStr2: String): String;

procedure AttributesToList(Attributes: String; AttributeNames: TStrings);
  overload;

function AttributesFromList(AttributeNames: TStrings): String;

type
  TCharSet = set of Char;

function ExtractWord(N: Integer; const S: string; const WordDelims: TCharSet):
  string;

function WordPosition(const N: Integer; const S: string; const WordDelims:
  TCharSet): Integer;

function ExtractWordPos(N: Integer; const S: string; const WordDelims: TCharSet;
  var Pos: Integer): string;

procedure ReplaceSubStr(var S: String; NewText: String; Position, Length:
  Integer);

function ExtractAttributeRef(Expression: String; var AttributeRef: String; var
  CurrPos, CurrLen: Integer): Boolean;

function ExtractSelectFromQuery(Query: TStrings): String;

procedure GenerateComponentName(AComponent: TComponent; Prefix: String = '');


function ExtractLastName(FullName: String): String;

function CopyData(SourceDataSet: TDataSet; SourceDataSetFields: TStrings;
  SourceDataSetName: String; TargetDataSet: TDataSet; MaxValueField: TField =
  nil): Integer;

procedure SaveComponentToFile(AComponent: TComponent; AFileName: String);
procedure LoadComponentFromFile(AComponent: TComponent; AFileName: String);

procedure SaveStringsToRegistry(Strings: TStrings; ARegKey: String);
procedure LoadStringsFromRegistry(Strings: TStrings; ARegKey: String);

procedure SaveStringToRegistry(ARegKey, AName, AValue: String);
function LoadStringFromRegistry(ARegKey, AName: String; ADefaultValue: String =
  ''): String;

function TransliterateText(Text: String): String;

type
  ECopyData = class (Exception);

implementation

uses ovConsts, TypInfo, Registry, Forms;

function ListToString(L: TStrings; Separator, ItemFormat: String): String;
//    ,   Separator.  
//  ,     ,  ItemFormat
var I: Integer;
begin
  Result := '';
  for I := 0 to L.Count - 1 do
    Result := Result + Separator + Format(ItemFormat, [L[I]]);
  if Result <> '' then Delete(Result, 1, 1);
end;

var StringList: TStrings;

function StringToList(Separator, S: String): TStrings;
//  ,   Separator  
var
  P: Integer;
  W: String;
begin
  { TODO :   :
    -   -  
    - BeginUpdate, EndUpdate
    -   - . ListToString }
  Result := StringList;
  Result.Clear;
  while S <> '' do begin
    P := Pos(Separator, S);
    if P > 0
      then begin W := Copy(S, 1, P-1); Delete(S, 1, P) end
      else begin W := S; S := '' end;
    Result.Add(Copy(Trim(W), 2, Length(Trim(W))-2));
  end;
end;

{{
     ,   ,
      .
}
procedure AttributesToList(Attributes, Separator: String; AttributeNames: 
  TStrings);
var
  L: Integer;
  S: String;
begin
  AttributeNames.Clear;
  while Attributes <> '' do
  begin
    S := Trim(ExtractSubStr(Attributes, Separator));
    if Copy(S, 1, 1) = '[' then Delete(S, 1, 1);
    L := Length(S);
    if Copy(S, L, 1) = ']' then Delete(S, L, 1);
    AttributeNames.Add(S);
  end
end;

procedure AttributesToList(Attributes: String; AttributeNames: TStrings);
begin
  AttributesToList(Attributes, ',', AttributeNames);
end;

function AttributesFromList(AttributeNames: TStrings): String;
begin
  Result := ListToString(AttributeNames, ',', '[%s]')
end;

{{
  Expression    "[]",    CurrPos.
     CurrLen   CurrPos   .
 ,  False,   Expression    .
}
function ExtractAttributeRef(Expression: String; var AttributeRef: String; var
  CurrPos, CurrLen: Integer): Boolean;
var
  P1, P2: Integer;
begin
  P1 := Pos('[', Copy(Expression, CurrPos, Length(Expression)));
  Result := P1 > 0;
  if Result then begin
    P2 := Pos(']', Copy(Expression, CurrPos+P1, Length(Expression)));
    if P2 = 0 then raise Exception.Create('   ]  : ' + Expression);
    //      ( )
    Inc(CurrPos, P1-1);
    CurrLen := P2+1;
    AttributeRef := Copy(Expression, CurrPos+1, CurrLen-2); //   []
  end
end;

function MakeNodeText(Name, TypeName: String): String;
begin
  Result := Name + ':' + TypeName
end;

function GetNodeName(Text: String): String;
var
  P: Integer;
begin
  P := Pos(':', Text);
  if P > 0
    then Result := Copy(Text, 1, P-1)
    else Result := Text
end;

{{
    ,   RestPart  
( )     FirstPart.    RestPart.
}
function ParseAttribute(var FirstPart, RestPart: String): Boolean;
var
  P: Integer;
begin
  P := Pos(NameSeparator, RestPart);
  Result := P > 0;
  if Result then
  begin
    FirstPart := Copy(RestPart, 1, P-1);
    RestPart := Copy(RestPart, P+1, Length(RestPart));
  end
  else
  begin
    FirstPart := RestPart;
    RestPart := '';
  end;
end;

{{
   ,      .
  ,    .
}
function ExtractLastName(FullName: String): String;
var
  P: Integer;
begin
  P := Pos(NameSeparator, FullName);
  if P > 0 then
    Result := Copy(FullName, P+1, Length(FullName))
  else
    Result := FullName
end;

function ExtractSubStr(var S: String; Delimiter: String): String;
var
  P: Integer;
begin
  P := Pos(Delimiter, S);
  if P > 0 then begin
    Result := Copy(S, 1, P-1);
    S := Copy(S, P+1, Length(S));
  end
  else
  begin
    Result := S;
    S := '';
  end;
end;

function CombineSubStr(SubStr1, Delimiter, SubStr2: String): String;
begin
  if (SubStr1 <> '') and (SubStr2 <> '') then
    Result := SubStr1 + Delimiter + SubStr2 else
  if SubStr1 <> '' then Result := SubStr1 else
  if SubStr2 <> '' then Result := SubStr2 else Result := '';
end;

{{
  S   Length   Position  NewText.
}
procedure ReplaceSubStr(var S: String; NewText: String; Position, Length:
  Integer);
begin
  Delete(S, Position, Length);
  Insert(NewText, S, Position);
end;

function ExtractWord(N: Integer; const S: string; const WordDelims: TCharSet):
  string;
var
  I: Integer;
  Len: Integer;
begin
  Len := 0;
  I := WordPosition(N, S, WordDelims);
  if I <> 0 then
    { find the end of the current word }
    while (I <= Length(S)) and not(S[I] in WordDelims) do begin
      { add the I'th character to result }
      Inc(Len);
      SetLength(Result, Len);
      Result[Len] := S[I];
      Inc(I);
    end;
  SetLength(Result, Len);
end;

{{
 N-    S,  WordDelims ( TCharSet) 
  .     ExtractWord 
            Pos.
}
function ExtractWordPos(N: Integer; const S: string; const WordDelims: TCharSet;
  var Pos: Integer): string;
var
  I, Len: Integer;
begin
  Len := 0;
  I := WordPosition(N, S, WordDelims);
  Pos := I;
  if I <> 0 then
    { find the end of the current word }
    while (I <= Length(S)) and not(S[I] in WordDelims) do begin
      { add the I'th character to result }
      Inc(Len);
      SetLength(Result, Len);
      Result[Len] := S[I];
      Inc(I);
    end;
  SetLength(Result, Len);
end;

function WordPosition(const N: Integer; const S: string; const WordDelims: 
  TCharSet): Integer;
var
  Count, I: Integer;
begin
  Count := 0;
  I := 1;
  Result := 0;
  while (I <= Length(S)) and (Count <> N) do begin
    { skip over delimiters }
    while (I <= Length(S)) and (S[I] in WordDelims) do Inc(I);
    { if we're not beyond end of S, we're at the start of a word }
    if I <= Length(S) then Inc(Count);
    { if not finished, find the end of the current word }
    if Count <> N then
      while (I <= Length(S)) and not (S[I] in WordDelims) do Inc(I)
    else Result := I;
  end;
end;

{{
      SELECT .
}
function ExtractSelectFromQuery(Query: TStrings): String;
var
  S: String;
  P1, P2: Integer;
begin
  S := UpperCase(Query.Text);
  P1 := Pos('SELECT', S);
  P2 := Pos('FROM', S);
  if (P1 = 0) or (P2 = 0) then
    Result := ''
  else begin
    Result := Trim(Copy(Query.Text, P1+Length('SELECT'), P2-(P1+Length('SELECT'))));
    if UpperCase(Copy(Result, 1, Length('DISTINCT'))) = 'DISTINCT' then
    begin
      Delete(Result, 1, Length('DISTINCT'));
      Result := Trim(Result);
    end
  end
end;

procedure GenerateComponentName(AComponent: TComponent; Prefix: String = '');
var
  I: Integer;
begin
  if Prefix = '' then Prefix := Copy(AComponent.ClassName, 2, Length(AComponent.ClassName) - 1);
  I := 0;
  while True do
  try
    Inc(I);
    //   
    if I > 1000 then raise Exception.Create('   ');
    AComponent.Name := Prefix + IntToStr(I);
    Break;
  except
    on E: EComponentError do ;
  end;
end;

{***************************************************************************** }
{                                                                              }
{ **************************************************************************** }

{{
        SourceDataSet  
  TargetDataSet.    SourceDataSetFields 
   SourceDataSet,    
      , .
    TargetDataSet 

  TargetDataSet[i] <- SourceDataSet[SourceDataSetFields[I]]

   MaxValueField,     
  MaxValueField.

 SourceDataSetName      
   ,    .
}
function CopyData(SourceDataSet: TDataSet; SourceDataSetFields: TStrings;
  SourceDataSetName: String; TargetDataSet: TDataSet; MaxValueField: TField =
  nil): Integer;
var
  ShowErrors: Boolean;

  procedure ProcessCopyDataError(SourceErrorMsg: String);
  var
    RecordTextMsg, NewErrorMsg: String;
    I: Integer;
  begin
    RecordTextMsg := '';
    with TargetDataSet do
      for I := 0 to Fields.Count - 1 do
        RecordTextMsg := RecordTextMsg +
          SourceDataSetFields[I] + '=' + SourceDataSet.FieldByName(SourceDataSetFields[I]).AsString + ';';

    NewErrorMsg := Format('     %s'#13'%s'+
    #13'   "%s"', [SourceDataSetName, RecordTextMsg, SourceErrorMsg]);

    case Application.MessageBox(
      PChar(NewErrorMsg+#13#10+
      '    ?'+#13#10+
      ' -   '#13#10+
      ' -       '#13#10+
      ' -     '), '  ',
      MB_ICONWARNING or MB_YESNOCANCEL) of

      ID_YES:
        ShowErrors := True;

      ID_NO:
        ShowErrors := False;

      ID_CANCEL:
          raise ECopyData.Create(NewErrorMsg);
    end

  end;

var
  I: Integer;
begin
  Result := 0;
  ShowErrors := True;
  with SourceDataSet do
  begin
    First;
    if Assigned(MaxValueField) and Not Eof then Result := Low(Integer);
    while Not Eof do
    begin
      try
        TargetDataSet.Append;
        with TargetDataSet do
          for I := 0 to Fields.Count - 1 do
            Fields[I].Value :=
              SourceDataSet.FieldByName(SourceDataSetFields[I]).Value;
        TargetDataSet.Post;
        if Assigned(MaxValueField) and (MaxValueField.AsInteger > Result) then
          Result := MaxValueField.AsInteger;
      except
        on E: Exception do
        begin
          TargetDataSet.Cancel;
          if ShowErrors then  {       }
            ProcessCopyDataError(E.Message);
        end;
      end;
      Next;
    end
  end;
end;

{***************************************************************************** }
{                       /                              }
{ **************************************************************************** }

procedure SaveComponentToFile(AComponent: TComponent; AFileName: String);
var
  FileStream, BinStream: TStream;
begin
  BinStream := TMemoryStream.Create;
  try
    BinStream.WriteComponent(AComponent);
    FileStream := TFileStream.Create(AFileName, fmCreate);
    try
      BinStream.Position := 0;
      ObjectBinaryToText(BinStream, FileStream);
    finally
      FileStream.Free
    end;
  finally
    BinStream.Free;
  end;
end;

procedure LoadComponentFromFile(AComponent: TComponent; AFileName: String);
var
  FileStream, BinStream: TStream;
begin
  FileStream := TFileStream.Create(AFileName, fmOpenRead);
  try
    BinStream := TMemoryStream.Create;
    try
      ObjectTextToBinary(FileStream, BinStream);
      BinStream.Position := 0;
      BinStream.ReadComponent(AComponent);
    finally
      BinStream.Free;
    end;
  finally
    FileStream.Free
  end;
end;

procedure SaveStringsToRegistry(Strings: TStrings; ARegKey: String);
var
  I: Integer;
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(ARegKey, True) then
      for I := 0 to Strings.Count - 1 do
        Reg.WriteString(IntToStr(I+1), Strings[I]);
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

procedure LoadStringsFromRegistry(Strings: TStrings; ARegKey: String);
var
  I: Integer;
  Reg: TRegistry;
  Names: TStrings;
begin
  Strings.Clear;
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(ARegKey, False) then
    begin
      Names := TStringList.Create;
      try
        Reg.GetValueNames(Names);
        for I := 0 to Names.Count - 1 do
          Strings.Add(Reg.ReadString(Names[I]))
      finally
        Names.Free;
      end;
    end;
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

procedure SaveStringToRegistry(ARegKey, AName, AValue: String);
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(ARegKey, True) then
      Reg.WriteString(AName, AValue);
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

function LoadStringFromRegistry(ARegKey, AName: String; ADefaultValue: String =
  ''): String;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(ARegKey, False) then
      Result := Reg.ReadString(AName)
    else
      Result := ADefaultValue
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

{***************************************************************************** }
{                                                                              }
{ **************************************************************************** }

function TransliterateLetter(Letter: Char): String;

  function FindLetter(InLetter: Char; var OutLetter: String): Boolean;
  const
    ConversionTable: array [1..34, 1..2] of String =
    (('', 'A'),
    ('', 'B'),
    ('', 'V'),
    ('', 'G'),
    ('', 'D'),
    ('', 'E'),
    ('', 'ZH'),
    ('', 'Z'),
    ('', 'I'),
    ('', 'J'),
    ('', 'K'),
    ('', 'L'),
    ('', 'M'),
    ('', 'N'),
    ('', 'O'),
    ('', 'P'),
    ('', 'R'),
    ('', 'S'),
    ('', 'T'),
    ('', 'U'),
    ('', 'F'),
    ('', 'H'),
    ('', 'TS'),
    ('', 'CH'),
    ('', 'SH'),
    ('', 'CHSH'),
    ('', 'J'),
    ('', 'Y'),
    ('', 'Y'),
    ('', 'E'),
    ('', 'JU'),
    ('', 'JA'),
    ('-', '_'),
    (' ', '_'));

  var
    I: Integer;
  begin
    for I := Low(ConversionTable) to High(ConversionTable) do
      if ConversionTable[I][1] = AnsiUpperCase(InLetter) then
      begin
        OutLetter := ConversionTable[I][2];
        Result := True;
        Exit;
      end;
    Result := False
  end;

begin
  if Not FindLetter(Letter, Result) then Result := Letter
end;

{{
   Text,     ,
   .
}
function TransliterateText(Text: String): String;
var
  I: Integer;
begin
  Result := '';
  for I := 1 to Length(Text) do
    Result := Result + TransliterateLetter(Text[I])
end;

initialization
  StringList := TStringList.Create;
finalization
  StringList.Free;
end.
