unit ovVirtualDataSet;

{
   Any TDataSet implementation must provide Bookmark capabilities and
 implement all functions which directly access the record buffer.  The base
 TDataSet manages a group of record buffers, but has no requirements regarding
 what is contained in each record buffer.  The base TDataSet also manages
 the communcation with any attached TDataSource components and the respective
 data aware controls.

}

interface

uses DB, Classes, Contnrs;

type
  TBookmark = Pointer;
  PBookmark = ^TBookmark;
  TKey = Integer;
  PKey = ^TKey;

{   This structure is used to access additional information stored in
  each record buffer which follows the actual record data.

    Buffer: PChar;
   ||
   \/
    ---------------------------------------------------------
    |  Record Data  | CalcFields | Bookmark | Bookmark Flag |
    ---------------------------------------------------------
                                 ^-- FRecInfoOfs = Buffer + FRecordSize + CalcFieldsSize

  Keep in mind that this is just an example of how the record buffer
  can be used to store additional information besides the actual record
  data.  There is no requirement that TDataSet implementations do it this
  way.

  For the purposes of this demo, the bookmark format used is just an integer
  value.  For an actual implementation the bookmark would most likely be
  a native bookmark type (as with BDE), or a fabricated bookmark for
  data providers which do not natively support bookmarks (this might be
  a variant array of key values for instance).

  The BookmarkFlag is used to determine if the record buffer contains a
  valid bookmark and has special values for when the dataset is positioned
  on the "cracks" at BOF and EOF. }

  PRecInfo = ^TRecInfo;
  TRecInfo = packed record
    BookmarkFlag: TBookmarkFlag;
    Bookmark: TBookmark;
  end;

  TBlobFieldDataBuffer = record
    Size: Integer;
    Data: record end;
  end;

  PBlobFieldDataBuffer = ^TBlobFieldDataBuffer;

  TovCustomVirtualDataSet = class;

  TovBlobFieldData = class(TMemoryStream)
  private
    FBuffers: TStrings;
    FDataSet: TovCustomVirtualDataSet;
    class procedure FreeBuffer(BufferList: TStrings; BufferIndex: Integer);
    class procedure FreeBuffers(BufferList: TStrings);
    procedure GetBlobData(Field: TField; Stream: TStream);
    function LoadBlobDataToBuffer(Field: TField): PBlobFieldDataBuffer;
    class function NewBufferFromStream(Stream: TStream): PBlobFieldDataBuffer;
    procedure StoreBlobDataFromBuffers;
    class procedure CopyBuffers(Source, Destination: TStrings);
    procedure CopyToList(AData: TStrings);
    procedure CopyFromList(AData: TStrings);
  protected
  public
    constructor Create(ADataSet: TovCustomVirtualDataSet);
    destructor Destroy; override;
    function GetFieldBlobBuffer(AFieldName: String): PBlobFieldDataBuffer;
    procedure Clear;
    procedure SetBlobData(Field: TField; Stream: TStream);
  end;

  {{
      
  }
  TChangedRecordList = class
  private
    FRecordSize: Integer;
    FRecords: TStringList;
    FBlobBuffers: TObjectList;
    FDataSet: TovCustomVirtualDataSet;
    procedure SetRecordSize(const Value: Integer);
  public
    procedure Clear;
    procedure PutRecord(ARecordKey: String; ABuffer: Pointer);
    function GetRecord(ARecordKey: String; ABuffer: Pointer): Boolean;
    constructor Create(ADataSet: TovCustomVirtualDataSet);
    destructor Destroy; override;
    property RecordSize: Integer read FRecordSize write SetRecordSize;
    property DataSet: TovCustomVirtualDataSet read FDataSet;
  end;

  TFieldMapping = class
  private
    FThisDataSetFieldName: String;
    FKeyDataSetFieldName: String;
    FKeyDataSetFieldIndex: Integer;
  public
    {{
        .   ,   
         ExposeFields.
          ,     
      .
    }
    property ThisDataSetFieldName: String read FThisDataSetFieldName;
    {{
        KeyDataSet,   , ..
          KeyDataSet,  
       .
    }
    property KeyDataSetFieldName: String read FKeyDataSetFieldName;
  end;

  TDataSetPosition = (posBOF, posEOF, posNormal);

  TOnDeleteRecordEvent = procedure (Sender: TObject; DataSet: TDataSet;
    RecordId: TKey; var DeleteRecord: Boolean) of object;
  TOnCreateRecordEvent = procedure (Sender: TObject; DataSet: TDataSet;
    var NewRecordId: TKey) of object;

  {{
     TovCustomDataSet    , 
          
  (  ),      
       (Master-).

           Master-, 
   ParentDataSet.   ,  
         ,  
  ExposeFields    :

    procedure ExposeFields(DataSet: TDataSet; KeyField, MasterKeyField: String;
      Fields, FieldAliases: TStrings);

      DataSet     , 
  KeyField          , 
   MasterKeyField      ParentDataSet, 
    DataSet.
     Fields         DataSet,
        .
     FieldAliases    ,    
  Fields    DataSet    . 
   ,   FieldAliases   
     Fields.

             OnCreateRecord
      :

  TOnCreateRecordEvent = procedure (Sender: TObject; DataSet: TDataSet;
    var NewRecordId: TKey) of object;

           DataSet 
   NewRecordId.      (   
  )    ,       NewRecordId.
      NewRecordId <= 0 (   )
     ,    DataSet    
          .

             OnDeleteRecord
      :

  TOnDeleteRecordEvent = procedure (Sender: TObject; DataSet: TDataSet;
    RecordId: TKey; var DeleteRecord: Boolean) of object;

         DeleteRecord, 
          RecordId  
  DataSet.     DeleteRecord = True (   )
     ,     RecordId  
     DataSet.
  }
  TovCustomVirtualDataSet = class(TDataSet)
  private
    FRecBufSize: Integer;
    FRecInfoOfs: Integer;
    FRecordSize: Integer;
    FReadOnly: Boolean;
    FKeyDataSet: TDataSet;
    FCanModify: Boolean;
    FOnCreateRecord: TOnCreateRecordEvent;
    FOnDeleteRecord: TOnDeleteRecordEvent;
    FCustomCalcFields: TFieldDefs;
    FOnCustomCalcFields: TDataSetNotifyEvent;
    FCustomCalcFieldsState: Boolean;
    FCustomCalcFieldsBuffer: PChar;
    FKeyDataSetReadOnly: Boolean;
    FUniqueFieldNames: TStrings;
    FCustomLookupFields: TObjectList;
    FChangedRecords: TChangedRecordList;
    FKeyDataSetKeyFieldName: String;
    FFieldMappings: TObjectList;
    function GetFieldBuffer(FieldNo: Integer; Buffer: Pointer): PChar;
    procedure SetKeyDataSet(Value: TDataSet);
    function GetActiveRecBuf(var RecBuf: PChar): Boolean;
    procedure InitBufferPointers;
    function GetUniqueFieldName(OriginalFieldName: String): String;
    procedure SetupFields;
    procedure InternalGetBookmark(Buffer: PChar);
    procedure DoFilterRecord(DataSet: TDataSet; var Accept: Boolean);
    function IsBlobFieldNull(Field: TField): Boolean;
    function GetFieldMappingCount: Integer;
    function GetFieldMappings(Index: Integer): TFieldMapping;
    function GetKeyDataSetField(FieldIndex: Integer): TField;
    function KeyDataSetFieldToThisDataSetField(FieldIndex: Integer): TField;
    procedure ThisDataSetFieldToKeyDataSetField(FieldName: String; var FieldIndex:
      Integer);
  protected
    { Overriden abstract methods (required) }

    { TODO :      (    ),
           }
    FPosition: TDataSetPosition;
    FBlobData: TovBlobFieldData;
    function AllocRecordBuffer: PChar; override;
    procedure FreeRecordBuffer(var Buffer: PChar); override;
    procedure GetBookmarkData(Buffer: PChar; Data: Pointer); override;
    function GetBookmarkFlag(Buffer: PChar): TBookmarkFlag; override;
    function GetRecord(Buffer: PChar; GetMode: TGetMode; DoCheck: Boolean): TGetResult; override;
    function GetRecordSize: Word; override;
    procedure InternalAddRecord(Buffer: Pointer; Append: Boolean); override;
    procedure InternalClose; override;
    procedure InternalDelete; override;
    procedure InternalFirst; override;
    procedure InternalGotoBookmark(Bookmark: Pointer); override;
    procedure InternalHandleException; override;
    procedure InternalInitFieldDefs; override;
    procedure InternalInitRecord(Buffer: PChar); override;
    procedure InternalLast; override;
    procedure InternalOpen; override;
    procedure InternalPost; override;
    procedure InternalSetToRecord(Buffer: PChar); override;
    function IsCursorOpen: Boolean; override;
    procedure SetBookmarkFlag(Buffer: PChar; Value: TBookmarkFlag); override;
    procedure SetBookmarkData(Buffer: PChar; Data: Pointer); override;
    procedure SetFieldData(Field: TField; Buffer: Pointer); override;
  protected
    { Additional overrides (optional) }
    function GetRecordCount: Integer; override;
    function GetRecNo: Integer; override;
    procedure SetRecNo(Value: Integer); override;

    function GetCanModify: Boolean; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;

    procedure ExposeFields(KeyDataSetFields, ThisDataSetFields: TStrings);
    procedure DoCreateRecord(DataSet: TDataSet; var RecordId: TKey); dynamic;
    procedure DoDeleteRecord(DataSet: TDataSet; RecordId: TKey; var DeleteRecord:
      Boolean); dynamic;
    procedure UpdateRecordData(Buffer: PChar); virtual;
    procedure ClearCalcFields(Buffer: PChar); override;
    function LocateRecord(const KeyFields: string; const KeyValues: Variant;
      Options: TLocateOptions; SyncCursor: Boolean): Boolean; virtual;
    procedure GetRecordData(Buffer: PChar); overload;
    procedure GetCalcFields(Buffer: PChar); override;
    function GetKeyDataSetFieldLocateFor(AFieldName: String): String; virtual;
    procedure CreateFields; override;
    procedure SetOnFilterRecord(const Value: TFilterRecordEvent); override;
    procedure SetFiltered(Value: Boolean); override;
    procedure DoOnCalcFields; override;
    procedure CopyFieldData(const FieldName: String; var Buffer: PChar;
      SourceField: TField); virtual;
    procedure GetFieldDef(FieldName: String; FieldDef: TFieldDef; var FieldType:
      TFieldType; var FieldSize: Integer; var Required: Boolean); virtual;
    procedure InternalRefresh; override;
    property ReadOnly: Boolean read FReadOnly write FReadOnly;
    property KeyDataSet: TDataSet read FKeyDataSet write SetKeyDataSet;
    {{
            .
          CustomCalcFields   
            
    OnCustomCalcFields.  ,    CustomCalcFields
     Calculated  False    FieldNo > 0.
    }
    property CustomCalcFields: TFieldDefs read FCustomCalcFields;
    property CustomLookupFields: TObjectList read FCustomLookupFields;
    property KeyDataSetReadOnly: Boolean read FKeyDataSetReadOnly write
        FKeyDataSetReadOnly;
    property FieldMappingCount: Integer read GetFieldMappingCount;
    property FieldMappings[Index: Integer]: TFieldMapping read GetFieldMappings;

    // 
    property OnCreateRecord: TOnCreateRecordEvent read FOnCreateRecord write
      FOnCreateRecord;
    property OnDeleteRecord: TOnDeleteRecordEvent read FOnDeleteRecord write
      FOnDeleteRecord;
    property OnCustomCalcFields: TDataSetNotifyEvent read FOnCustomCalcFields write
      FOnCustomCalcFields;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function CreateBlobStream(Field: TField; Mode: TBlobStreamMode): TStream;
        override;
    function GetFieldData(Field: TField; Buffer: Pointer): Boolean; override;
    function GetFieldData(FieldNo: Integer; Buffer: Pointer): Boolean; overload;
      override;
    function Lookup(const KeyFields: string; const KeyValues: Variant; const
      ResultFields: string): Variant; override;
    function Locate(const KeyFields: string; const KeyValues: Variant; Options:
      TLocateOptions): Boolean; override;
    procedure AddCustomLookupField(const Name: string; DataType: TFieldType; Size: 
      Integer; KeyFields, LookupKeyFields, LookupResultField: String; 
      LookupDataSet: TDataSet);
    procedure AddCustomCalcField(const Name: string; DataType: TFieldType; Size:
      Integer);
    function BookmarkValid(Bookmark: TBookmark): Boolean; override;
    function CompareBookmarks(Bookmark1, Bookmark2: TBookmark): Integer; override;
    function FindCustomCalcField(const Name: string): TFieldDef;
    procedure RemoveCustomCalcField(const Name: string);
    property KeyDataSetKeyFieldName: String read FKeyDataSetKeyFieldName write
      FKeyDataSetKeyFieldName;
  end;

  TCustomLookupFieldDef = class
  private
    FName: String;
    FLookupDataSet: TDataSet;
    FKeyFields: String;
    FLookupKeyFields: String;
    FLookupResultField: String;
    FFieldType: TFieldType;
    FSize: Integer;
    FRealName: String;
  public
    property Name: String read FName;
    property RealName: String read FRealName;
    property FieldType: TFieldType read FFieldType;
    property Size: Integer read FSize;
    property KeyFields: String read FKeyFields;
    property LookupDataSet: TDataSet read FLookupDataSet;
    property LookupKeyFields: String read FLookupKeyFields;
    property LookupResultField: String read FLookupResultField;
  end;

  {{
          TovCustomVirtualDataSet.
       ,    
  -  .
  }
  TovVirtualDataSet = class(TovCustomVirtualDataSet)
  protected
  public
    property CustomCalcFields;
    property CustomLookupFields;
  published
    //    TDataSet
    property Active;
    property BeforeOpen;
    property AfterOpen;
    property BeforeClose;
    property AfterClose;
    property BeforeInsert;
    property AfterInsert;
    property BeforeEdit;
    property AfterEdit;
    property BeforePost;
    property AfterPost;
    property BeforeCancel;
    property AfterCancel;
    property BeforeDelete;
    property AfterDelete;
    property BeforeScroll;
    property AfterScroll;
    property BeforeRefresh;
    property AfterRefresh;
    property OnDeleteError;
    property OnEditError;
    property OnNewRecord;
    property OnPostError;
    property OnFilterRecord;

    //    TovCustomVirtualDataSet
    property ReadOnly;
    property KeyDataSet;
    property KeyDataSetReadOnly;
    property OnCreateRecord;
    property OnDeleteRecord;
    property OnCustomCalcFields;
  end;


implementation

uses Windows, SysUtils, Forms, DBConsts, Variants;

resourcestring
  SNoFieldAccess = 'Cannot access field ''%s'' in a filter';

type
  TovBlobStream = class(TMemoryStream)
  private
    FField: TBlobField;
    FDataSet: TovCustomVirtualDataSet;
    FMode: TBlobStreamMode;
    FOpened: Boolean;
    FModified: Boolean;
  protected
    procedure SetSize(NewSize: Longint); overload; override;
  public
    constructor Create(Field: TBlobField; Mode: TBlobStreamMode);
    destructor Destroy; override;
    function Write(const Buffer; Count: Longint): Longint; override;
  end;

{ ============================================================================ }
{ TovCustomDataSet                                                             }
{ ============================================================================ }

constructor TovCustomVirtualDataSet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  // FUniqueFieldNames      , 
  //        
  // ExposeFields,      
  FUniqueFieldNames := TStringList.Create;
  FCustomCalcFields := TFieldDefs.Create(Self);
  FCustomLookupFields := TObjectList.Create;
  FFieldMappings := TObjectList.Create;

  // KeyDataSet   ,    
  //   .       
  //   .
  FKeyDataSetReadOnly := True;
  FBlobData := TovBlobFieldData.Create(Self);
  FChangedRecords := TChangedRecordList.Create(Self);
end;

destructor TovCustomVirtualDataSet.Destroy;
begin
  inherited Destroy;
  //   Destroy  Close
  FreeAndNil(FChangedRecords);
  FreeAndNil(FBlobData);
  FFieldMappings.Free;
  FCustomLookupFields.Free;
  FCustomCalcFields.Free;
  FUniqueFieldNames.Free;
end;

{ ============================================================================ }
{ /                                                }
{ ============================================================================ }

{{
This method is called by TDataSet.Open and also when FieldDefs need to
  be updated (usually by the DataSet designer).  Everything which is
  allocated or initialized in this method should also be freed or
  uninitialized in the InternalClose method.
}
procedure TovCustomVirtualDataSet.InternalOpen;
begin
  //  KeyDataSet
  if Not Assigned(KeyDataSet) then
    DatabaseError('    KeyDataSet', Self);
  with KeyDataSet do
  if Not Active then Open else First;

  FCanModify := Not ReadOnly and
    //      ,
    //    KeyDataSet
    (KeyDataSetKeyFieldName <> '') or KeyDataSet.CanModify;

  //     
  SetupFields;

  //      ,    
  InitBufferPointers;

  FPosition := posBOF;
end;

procedure TovCustomVirtualDataSet.SetupFields;
begin
  { Initialize the FieldDefs }
  InternalInitFieldDefs;

  { Create TField components when no persistent fields have been created }
  if DefaultFields then CreateFields;

  { Bind the TField components to the physical fields }
  BindFields(True);
end;

procedure TovCustomVirtualDataSet.AddCustomCalcField(const Name: string;
  DataType: TFieldType; Size: Integer);
begin
  CustomCalcFields.Add(Name, DataType, Size)
end;

procedure TovCustomVirtualDataSet.RemoveCustomCalcField(const Name: string);
var
  I: Integer;
begin
  I := CustomCalcFields.IndexOf(Name);
  if I >= 0 then CustomCalcFields.Delete(I);
end;

function TovCustomVirtualDataSet.FindCustomCalcField(const Name: string):
  TFieldDef;
var
  I: Integer;
begin
  I := CustomCalcFields.IndexOf(Name);
  if I < 0 then Result := nil else Result := CustomCalcFields[I];
end;

procedure TovCustomVirtualDataSet.AddCustomLookupField(const Name: string;
  DataType: TFieldType; Size: Integer; KeyFields, LookupKeyFields,
  LookupResultField: String; LookupDataSet: TDataSet);
var
  LookupField: TCustomLookupFieldDef;
begin
  LookupField := TCustomLookupFieldDef.Create;
  LookupField.FName := Name;
  LookupField.FFieldType := DataType;
  LookupField.FSize := Size;
  LookupField.FKeyFields := KeyFields;
  LookupField.FLookupDataSet := LookupDataSet;
  LookupField.FLookupKeyFields := LookupKeyFields;
  LookupField.FLookupResultField := LookupResultField;
  CustomLookupFields.Add(LookupField);
end;

{{
   FieldDefs    .

       ,  
   ExposeFields,     ,
  CustomCalcFields.
}
procedure TovCustomVirtualDataSet.InternalInitFieldDefs;
var
  I: Integer;
  AFieldName: String;
  AFieldType: TFieldType; AFieldSize: Integer;
  ARequired: Boolean;
  FieldDef: TFieldDef;
  RealFieldName: String;
begin
  FieldDefs.Clear;
  FUniqueFieldNames.Clear;

  if KeyDataSet <> nil then
  begin
    KeyDataSet.FieldDefs.Update;
    for I := 0 to FieldMappingCount - 1 do
    begin
      FieldDef := KeyDataSet.FieldDefs.Find(FieldMappings[I].KeyDataSetFieldName);
      RealFieldName := GetUniqueFieldName(FieldMappings[I].ThisDataSetFieldName);
      FieldMappings[I].FThisDataSetFieldName := RealFieldName;
      FUniqueFieldNames.Add(RealFieldName);

      GetFieldDef(RealFieldName, FieldDef, AFieldType, AFieldSize, ARequired);
      Self.FieldDefs.Add(RealFieldName, AFieldType, AFieldSize, ARequired);

      //       
      FieldMappings[I].FKeyDataSetFieldIndex :=
        KeyDataSet.FieldByName(FieldMappings[I].KeyDataSetFieldName).Index;
    end;
  end;

  //     
  for I := 0 to CustomCalcFields.Count - 1 do
  begin
    FieldDef := CustomCalcFields[I];
    RealFieldName := GetUniqueFieldName(FieldDef.Name);
    FUniqueFieldNames.Add(RealFieldName);
    Self.FieldDefs.Add(RealFieldName, FieldDef.DataType, FieldDef.Size, FieldDef.Required);
  end;

  //     
  for I := 0 to CustomLookupFields.Count - 1 do
  with TCustomLookupFieldDef(CustomLookupFields[I]) do
  begin
    //     (  ,      )
    FRealName := GetUniqueFieldName(Name);
    FUniqueFieldNames.Add(FRealName);
    Self.FieldDefs.Add(FRealName, FieldType, Size);
  end
end;

{{
         .
}
procedure TovCustomVirtualDataSet.GetFieldDef(FieldName: String; FieldDef:
  TFieldDef; var FieldType: TFieldType; var FieldSize: Integer; var Required:
  Boolean);
begin
  FieldType := FieldDef.DataType;
  FieldSize := FieldDef.Size;
  Required := FieldDef.Required;
end;

procedure TovCustomVirtualDataSet.CreateFields;
var
  I: Integer;
  Field: TField;
begin
  inherited;

  for I := 0 to CustomCalcFields.Count - 1 do
  begin
    Field := FieldByName(CustomCalcFields[I].Name);
    Field.Calculated := True;
  end;

  //     
  for I := 0 to CustomLookupFields.Count - 1 do
  with TCustomLookupFieldDef(CustomLookupFields[I]) do
  begin
    Field := FieldByName(RealName);
    Field.Lookup := True;
    Field.KeyFields := KeyFields;
    Field.LookupDataSet := LookupDataSet;
    Field.LookupKeyFields := LookupKeyFields;
    Field.LookupResultField := LookupResultField;
  end;
end;

procedure TovCustomVirtualDataSet.InternalClose;
begin
  FChangedRecords.Clear;

  if Assigned(KeyDataSet) then KeyDataSet.Close;

  { Destroy the TField components if no persistent fields }
  if DefaultFields then DestroyFields;

  { Reset these internal flags }
  FPosition := posBOF;
end;

{{
This property is used while opening the dataset.
It indicates if data is available even though the
current state is still dsInActive.
}
function TovCustomVirtualDataSet.IsCursorOpen: Boolean;
begin
  Result := Assigned(KeyDataSet) and KeyDataSet.Active;
end;

{{
This is the exception handler which is called if an exception is raised
while the component is being stream in or streamed out.  In most cases this
should be implemented useing the application exception handler as follows.
}
procedure TovCustomVirtualDataSet.InternalHandleException;
begin
  Application.HandleException(Self);
end;

procedure TovCustomVirtualDataSet.InternalRefresh;
begin
  FChangedRecords.Clear
end;

{ ============================================================================ }
{ Field mapping                                                                }
{ ============================================================================ }

{{
      KeyDataSet,    
  ,      ,   
ThisDataSetFields.        .
}
procedure TovCustomVirtualDataSet.ExposeFields(KeyDataSetFields,
  ThisDataSetFields: TStrings);
var
  I: Integer;
  Field: TFieldMapping;
begin
  Close;

  if KeyDataSetFields.Count <> ThisDataSetFields.Count then
    raise Exception.Create(' KeyDataSetFields  ThisDataSetFields ' +
      '    ');

  FFieldMappings.Clear;
  for I := 0 to KeyDataSetFields.Count - 1 do
  begin
    Field := TFieldMapping.Create;
    FFieldMappings.Add(Field);
    Field.FKeyDataSetFieldName := KeyDataSetFields[I];
    Field.FThisDataSetFieldName := ThisDataSetFields[I];
  end;
end;

function TovCustomVirtualDataSet.GetFieldMappingCount: Integer;
begin
  Result := FFieldMappings.Count;
end;

function TovCustomVirtualDataSet.GetFieldMappings(Index: Integer):
  TFieldMapping;
begin
  Result := TFieldMapping(FFieldMappings[Index]);
end;

{{
     ,      
FieldIndex  KeyDataSet.   FieldIndex  
   KeyDataSetFields  ExposeFields.

      ,    
     KeyDataSet     .
}
function TovCustomVirtualDataSet.KeyDataSetFieldToThisDataSetField(FieldIndex:
  Integer): TField;
begin
  Result := FindField(FieldMappings[FieldIndex].ThisDataSetFieldName);
end;

{{
      KeyDataSet,   ,
  FieldIndex,    OriginFields.  
   ,   OriginFields 
  ,       DataSet  
. ..       
           
 .

    - ,    
   (/)    
  .
}
function TovCustomVirtualDataSet.GetKeyDataSetField(FieldIndex: Integer):
  TField;
begin
  Result := KeyDataSet.Fields[FieldMappings[FieldIndex].FKeyDataSetFieldIndex];
end;

{{
    KeyDataSet,     FieldName
  .     ,  
 .
}
procedure TovCustomVirtualDataSet.ThisDataSetFieldToKeyDataSetField(FieldName:
  String; var FieldIndex: Integer);
var
  I: Integer;
begin
  for I := 0 to FieldMappingCount - 1 do
    if FieldMappings[I].ThisDataSetFieldName = FieldName then
    begin
      FieldIndex := I;
      Exit;
    end;
  DatabaseErrorFmt(' "%s"  ', [FieldName], Self);
end;

{ ============================================================================ }
{                                                                      }
{ ============================================================================ }

procedure TovCustomVirtualDataSet.InternalGotoBookmark(Bookmark: Pointer);
begin
  KeyDataSet.GotoBookmark(PBookmark(Bookmark)^);
  FPosition := posNormal;
end;

{{
This function does the same thing as InternalGotoBookmark, but it takes
  a record buffer as a parameter instead
}
procedure TovCustomVirtualDataSet.InternalSetToRecord(Buffer: PChar);
begin
  InternalGotoBookmark(@PRecInfo(Buffer + FRecInfoOfs).Bookmark);
end;

{{
    ,   .
      -    .
}
procedure TovCustomVirtualDataSet.InternalGetBookmark(Buffer: PChar);
begin
  with PRecInfo(Buffer + FRecInfoOfs)^ do
  begin
    BookmarkFlag := bfCurrent;
    Bookmark := KeyDataSet.GetBookmark;
  end;
end;

{{
Bookmark flags are used to indicate if a particular record is the first
or last record in the dataset.  This is necessary for "crack" handling.
If the bookmark flag is bfBOF or bfEOF then the bookmark is not actually
used; InternalFirst, or InternalLast are called instead by TDataSet.
}
function TovCustomVirtualDataSet.GetBookmarkFlag(Buffer: PChar): TBookmarkFlag;
begin
  Result := PRecInfo(Buffer + FRecInfoOfs).BookmarkFlag;
end;

procedure TovCustomVirtualDataSet.SetBookmarkFlag(Buffer: PChar; Value: TBookmarkFlag);
begin
  PRecInfo(Buffer + FRecInfoOfs).BookmarkFlag := Value;
end;

{{
These methods provide a way to read and write bookmark data into the
record buffer without actually repositioning the current record.

 Data     TBookmark.
}
procedure TovCustomVirtualDataSet.GetBookmarkData(Buffer: PChar; Data: Pointer);
begin
  PBookmark(Data)^ := PRecInfo(Buffer + FRecInfoOfs).Bookmark;
end;

procedure TovCustomVirtualDataSet.SetBookmarkData(Buffer: PChar; Data: Pointer);
begin
  PRecInfo(Buffer + FRecInfoOfs).Bookmark := PBookmark(Data)^;
end;

function TovCustomVirtualDataSet.BookmarkValid(Bookmark: TBookmark): Boolean;
begin
  Result := KeyDataSet.BookmarkValid(PBookmark(Bookmark)^)
end;

function TovCustomVirtualDataSet.CompareBookmarks(Bookmark1, Bookmark2:
  TBookmark): Integer;
begin
  Result := KeyDataSet.CompareBookmarks(PBookmark(Bookmark1)^, PBookmark(Bookmark2)^)
end;

{ ============================================================================ }
{ Record / Field Access                                                        }
{ ============================================================================ }

{{
This method returns the size of just the data in the record buffer.
Do not confuse this with RecBufSize which also includes any additonal
structures stored in the record buffer (such as TRecInfo).
}
function TovCustomVirtualDataSet.GetRecordSize: Word;
begin
  Result := FRecordSize;
end;

procedure TovCustomVirtualDataSet.ClearCalcFields(Buffer: PChar);
begin
  FillChar(Buffer[RecordSize], CalcFieldsSize, 0);
end;

{{
  ,       
     .
}
procedure TovCustomVirtualDataSet.InitBufferPointers;
var
  I: Integer;
begin
  { Tell TDataSet how big our Bookmarks are (REQUIRED) }
  BookmarkSize := SizeOf(TBookmark);

  //   ,       
  // (   BLOB-)
  FRecordSize := 0;
  for I := 0 to Fields.Count - 1 do
    with Fields[I] do
      if Not IsBlob and Not (FieldKind in [fkCalculated, fkLookup]) then
        Inc(FRecordSize, DataSize + 1); //     Null-

  //    ,      TRecInfo
  // (CalcFieldsSize    ,   
  //      BindFields)
  FRecInfoOfs := FRecordSize + CalcFieldsSize;

  { Calculate the size of the record buffers.
    Note: This is NOT the same as the RecordSize property which
    only gets the size of the data in the record buffer }
  FRecBufSize := FRecInfoOfs + SizeOf(TRecInfo);

  FChangedRecords.RecordSize := FRecordSize;
end;

{{
TDataSet calls this method to allocate the record buffer.  Here we use
FRecBufSize which is equal to the size of the data plus the size of the
TRecInfo structure.
}
function TovCustomVirtualDataSet.AllocRecordBuffer: PChar;
begin
  GetMem(Result, FRecBufSize);
  FillChar(Result^, FRecBufSize, 0);
end;

{{
Again, TDataSet calls this method to free the record buffer.
Note: Make sure the value of FRecBufSize does not change before all
allocated buffers are freed.
}
procedure TovCustomVirtualDataSet.FreeRecordBuffer(var Buffer: PChar);
var
  P: Pointer;
begin
  P := PRecInfo(Buffer + FRecInfoOfs)^.Bookmark;
  if P <> nil then KeyDataSet.FreeBookmark(P);
  FreeMem(Buffer, FRecBufSize);
end;

{{
This multi-purpose function does 3 jobs.  It retrieves data for either
the current, the prior, or the next record.  It must return the status
(TGetResult), and raise an exception if DoCheck is True.
}
function TovCustomVirtualDataSet.GetRecord(Buffer: PChar; GetMode: TGetMode;
  DoCheck: Boolean): TGetResult;
begin
  if Not Assigned(KeyDataSet) then raise Exception.Create('KeyDataSet  ');

  if RecordCount < 1 then
    Result := grEOF else
  begin
    Result := grOK;
    case GetMode of
      gmNext:
        if FPosition = posBOF
          then FPosition := posNormal
          else if Not KeyDataSet.Eof
             then begin
               KeyDataSet.Next;
               if KeyDataSet.Eof then
                 begin Result := grEOF; FPosition := posEOF end
             end else
               begin Result := grEOF; FPosition := posEOF end;
      gmPrior:
        if FPosition = posEOF
          then FPosition := posNormal
          else if Not KeyDataSet.Bof
             then begin
               KeyDataSet.Prior;
               if KeyDataSet.Bof then
                 begin Result := grBOF; FPosition := posBOF end
             end else
               begin Result := grBOF; FPosition := posBOF end;
      gmCurrent:
        if FPosition in [posEOF, posBOf] then
          Result := grError;
    end;
    if Result = grOK then
    begin
      GetRecordData(Buffer);
      InternalGetBookmark(Buffer);
      GetCalcFields(Buffer);
    end else
      if (Result = grError) and DoCheck then DatabaseError('No Records', Self);
  end;
end;

{{
   KeyDataSet   Buffer.
}
procedure TovCustomVirtualDataSet.GetRecordData(Buffer: PChar);
var
  Field: TField;
  I: Integer;
begin
  if Not FChangedRecords.GetRecord(KeyDataSet.Bookmark, Buffer) then
  begin
    InternalInitRecord(Buffer);
    for I := 0 to FieldMappingCount - 1 do
    begin
      //  ,        
      if Assigned(KeyDataSetFieldToThisDataSetField(I)) then
      begin
        Field := GetKeyDataSetField(I);
        if Not Field.IsBlob then
          CopyFieldData(FieldMappings[I].ThisDataSetFieldName, Buffer, Field)
      end
    end;
  end;
end;

procedure TovCustomVirtualDataSet.CopyFieldData(const FieldName: String; var
  Buffer: PChar; SourceField: TField);
begin
  if SourceField.GetData(Buffer+1)
    then Buffer[0] := #1
    else Buffer[0] := #0; //   Null
  Inc(Buffer, SourceField.DataSize+1);
end;

{{
      Buffer.
     DataSetIndex > 0,   
      DataSetIndex.
     DataSetIndex < 0,    
  ,     .
}
procedure TovCustomVirtualDataSet.UpdateRecordData(Buffer: PChar);
var
  Field: TField;
  I: Integer;
begin
  if Not KeyDataSet.CanModify or KeyDataSetReadOnly then
    FChangedRecords.PutRecord(KeyDataSet.Bookmark, Buffer)
  else
  begin
    KeyDataSet.Edit;
    try
      for I := 0 to FieldMappingCount - 1 do
      begin
        Field := GetKeyDataSetField(I);
        if Not Field.IsBlob then
        begin
          if Buffer[0] <> #0
            then Field.SetData(Buffer+1)
            else Field.Clear; //   Null
          Inc(Buffer, Field.DataSize+1);
        end
      end;

      FBlobData.StoreBlobDataFromBuffers;

      //     KeyDataSet.Post    InternalPost
      if State = dsEdit then KeyDataSet.Post;
    except
      if State = dsEdit then KeyDataSet.Cancel;
      raise
    end;
  end
end;

{ This routine is called to initialize a record buffer.  In this sample,
  we fill the buffer with zero values, but we might have code to initialize
  default values or do other things as well. }
procedure TovCustomVirtualDataSet.InternalInitRecord(Buffer: PChar);
begin
  FillChar(Buffer^, RecordSize, 0);

  //   Blob-    
  FBlobData.Clear;
end;

function TovCustomVirtualDataSet.GetActiveRecBuf(var RecBuf: PChar): Boolean;
begin
  if FCustomCalcFieldsState then RecBuf := FCustomCalcFieldsBuffer
  else
  case State of
    dsBrowse: if IsEmpty then RecBuf := nil else RecBuf := ActiveBuffer;
    dsEdit, dsInsert: RecBuf := ActiveBuffer;
    dsCalcFields: RecBuf := CalcBuffer;
    dsNewValue: RecBuf := ActiveBuffer;
  else
    RecBuf := nil;
  end;
  Result := RecBuf <> nil;
end;

{{
         .  
   (  FieldNo < 0      
  Field.Offset).
}
function TovCustomVirtualDataSet.GetFieldBuffer(FieldNo: Integer; Buffer:
  Pointer): PChar;
var
  Field: TField;
  I: Integer;
begin
  Result := Buffer;
  for I := 1 to FieldNo - 1 do
  begin
    Field := Fields.FieldByNumber(I);
    // ,         (   )
    if Field <> nil then
    with Field do
      if Not IsBlob then Inc(Result, DataSize + 1);
  end
end;

{
Here we copy the data from the record buffer into a field's buffer.
}
function TovCustomVirtualDataSet.GetFieldData(Field: TField; Buffer: Pointer):
  Boolean;
var
  RecBuf: PChar;
begin
  if Field.FieldNo > 0 then
  begin
    if Field.IsBlob and Not Assigned(Buffer) then
      //     Blob- (  Buffer = nil) - 
      //        Blob-
      Result := Not IsBlobFieldNull(Field)
    else
      Result := GetFieldData(Field.FieldNo, Buffer)
  end else {fkCalculated, fkLookup}
  begin
    //        Field.Offset,
    //    
    Result := GetActiveRecBuf(RecBuf);
    if Result and (State in [dsBrowse, dsEdit, dsInsert, dsCalcFields]) then
    begin
      Inc(RecBuf, FRecordSize + Field.Offset);
      Result := Boolean(RecBuf[0]);
      if Result and (Buffer <> nil) then
        Move(RecBuf[1], Buffer^, Field.DataSize);
    end;
  end;
end;

{{
-      GetFieldData, 
 
}
function TovCustomVirtualDataSet.GetFieldData(FieldNo: Integer; Buffer:
  Pointer): Boolean;
var
  RecBuf, FieldBuf: PChar;
begin
  Result := GetActiveRecBuf(RecBuf);
  if Result then
  begin
    FieldBuf := GetFieldBuffer(FieldNo, RecBuf);
    Result := Boolean(FieldBuf[0]);
    if Result and (Buffer <> nil) then
      Move(FieldBuf[1], Buffer^, Fields.FieldByNumber(FieldNo).DataSize);
  end
end;

procedure TovCustomVirtualDataSet.SetFieldData(Field: TField; Buffer: Pointer);
var
  RecBuf, FieldBuf: PChar;
begin
  with Field do
  begin
    //     State,     DBTables
    if not (State in dsWriteModes) and Not FCustomCalcFieldsState
      then DatabaseError(SNotEditing, Self);
    GetActiveRecBuf(RecBuf);
    if FieldNo > 0 then
    begin
      if State = dsCalcFields then DatabaseError(SNotEditing, Self);
      if ReadOnly and not (State in [dsSetKey, dsFilter]) then
        DatabaseErrorFmt(SFieldReadOnly, [DisplayName]);
      Validate(Buffer);
      if FieldKind <> fkInternalCalc then
      begin
        FieldBuf := GetFieldBuffer(Field.FieldNo, RecBuf);
        if Buffer = nil    //   TField.Clear  SetFieldData  Buffer  nil
          then FillChar(FieldBuf^, Field.DataSize+1, 0)  //     Null
          else begin
            FieldBuf[0] := #1;
            Move(Buffer^, (FieldBuf+1)^, Field.DataSize);
          end;
      end;
    end else {fkCalculated, fkLookup}
    begin
      Inc(RecBuf, FRecordSize + Offset);
      Boolean(RecBuf[0]) := LongBool(Buffer);
      if Boolean(RecBuf[0]) then Move(Buffer^, RecBuf[1], DataSize);
    end;
    if not (State in [dsCalcFields, dsFilter, dsNewValue]) and Not FCustomCalcFieldsState then
      DataEvent(deFieldChange, Longint(Field));
  end;
end;

procedure TovCustomVirtualDataSet.GetCalcFields(Buffer: PChar);
begin
  inherited GetCalcFields(Buffer);

  FCustomCalcFieldsState := True;
  try
    FCustomCalcFieldsBuffer := Buffer;
    if Assigned(FOnCustomCalcFields) then FOnCustomCalcFields(Self);
  finally
    FCustomCalcFieldsState := False;
  end;
end;

procedure TovCustomVirtualDataSet.DoOnCalcFields;
begin
  inherited;
  if Assigned(FOnCustomCalcFields) then FOnCustomCalcFields(Self);
end;

{ ============================================================================ }
{                                               }
{ ============================================================================ }

{{
This method is called by TDataSet.First.  Crack behavior is required.
That is we must position to a special place *before* the first record.
Otherwise, we will actually end up on the second record after Resync
is called.
}
procedure TovCustomVirtualDataSet.InternalFirst;
begin
  FPosition := posBOF;
  KeyDataSet.First;
end;

{{
Again, we position to the crack *after* the last record here.
}
procedure TovCustomVirtualDataSet.InternalLast;
begin
  FPosition := posEOF;
  KeyDataSet.Last;
end;

{{
This method is called by TDataSet.Post.
}
procedure TovCustomVirtualDataSet.InternalPost;
begin
  if State = dsEdit then
    UpdateRecordData(ActiveBuffer)
  else
  begin
    KeyDataSet.Append;
    try
      UpdateRecordData(Self.ActiveBuffer); //          DataSets[I]
      KeyDataSet.Post;
    except
      KeyDataSet.Cancel; raise;
    end;
    FPosition := posNormal;
  end;
end;

{ ============================================================================ }
{                                                    }
{ ============================================================================ }

procedure TovCustomVirtualDataSet.DoCreateRecord(DataSet: TDataSet; var RecordId:
  TKey);
begin
  if Assigned(FOnCreateRecord) then
    FOnCreateRecord(Self, DataSet, RecordId)
end;

procedure TovCustomVirtualDataSet.DoDeleteRecord(DataSet: TDataSet; RecordId: TKey;
  var DeleteRecord: Boolean);
begin
  if Assigned(FOnDeleteRecord) then
    FOnDeleteRecord(Self, DataSet, RecordId, DeleteRecord)
end;

{{
 This method is similar to InternalPost above, but the operation is always
  an insert or append and takes a pointer to a record buffer as well.

     TDataSet.AddRecord     
.
 }
procedure TovCustomVirtualDataSet.InternalAddRecord(Buffer: Pointer; Append: Boolean);
begin
  DatabaseError('InternalAddRecord  ', Self);
end;

{{
This method is called by TDataSet.Delete to delete the current record
}
procedure TovCustomVirtualDataSet.InternalDelete;
begin
  KeyDataSet.Delete; //      
end;

{ ============================================================================ }
{ Optional Methods                                                             }
{ ============================================================================ }

{ The following methods are optional.  When provided they will allow the
  DBGrid and other data aware controls to track the current cursor postion
  relative to the number of records in the dataset.  Because we are dealing
  with a small, static data store (a stringlist), these are very easy to
  implement.  However, for many data sources (SQL servers), the concept of
  record numbers and record counts do not really apply. }

function TovCustomVirtualDataSet.GetRecordCount: Longint;
begin
  if Assigned(FKeyDataSet) then
    Result := FKeyDataSet.RecordCount
  else
    Result := 0
end;

function TovCustomVirtualDataSet.GetRecNo: Longint;
begin
  if Assigned(FKeyDataSet) then begin
    UpdateCursorPos;
    Result := FKeyDataSet.RecNo;
  end else
    Result := 0
end;

procedure TovCustomVirtualDataSet.SetRecNo(Value: Integer);
begin
  if Assigned(FKeyDataSet) and (Value >= 1) and (Value <= RecordCount) then
  begin
    FKeyDataSet.RecNo := Value;
    FPosition := posNormal;
    Resync([]);
  end;
end;

{ ============================================================================ }
{                                                                  }
{ ============================================================================ }

function TovCustomVirtualDataSet.Lookup(const KeyFields: string; const
  KeyValues: Variant; const ResultFields: string): Variant;
begin
  Result := Null;
  if LocateRecord(KeyFields, KeyValues, [], False) then
  begin
    SetTempState(dsCalcFields);
    try
      CalculateFields(TempBuffer);
      Result := FieldValues[ResultFields];
    finally
      RestoreState(dsBrowse);
    end;
  end;
end;

function TovCustomVirtualDataSet.Locate(const KeyFields: string; const
  KeyValues: Variant; Options: TLocateOptions): Boolean;
begin
  DoBeforeScroll;
  Result := LocateRecord(KeyFields, KeyValues, Options, True);
  if Result then
  begin
    Resync([rmExact, rmCenter]);
    DoAfterScroll;
  end;
end;

{{
       KeyFields,  KeyValues 
    True   TempBuffer    .
 SyncCursor  True    Locate   False  
 Lookup.

        , 
   KeyDataSet.   ,   
    .
}
function TovCustomVirtualDataSet.LocateRecord(const KeyFields: string; const
  KeyValues: Variant; Options: TLocateOptions; SyncCursor: Boolean): Boolean;
var
  Pos: Integer;
  KeyDataSetSearchFields: String;
  FieldName: String;
  ResultValues: Variant;
begin
  //   KeyDataSetSearchFields    KeyDataSet,
  //   
  Pos := 1;
  KeyDataSetSearchFields := '';
  while Pos <= Length(KeyFields) do
  begin
    FieldName := ExtractFieldName(KeyFields, Pos);
    KeyDataSetSearchFields := KeyDataSetSearchFields + ';' +
      GetKeyDataSetFieldLocateFor(FieldName)
  end;
  System.Delete(KeyDataSetSearchFields, 1, 1);

  CheckBrowseMode;
  CursorPosChanged;

  Result := KeyDataSet.Locate(KeyDataSetSearchFields, KeyValues, Options);

  if Result then begin
    GetRecordData(TempBuffer);
    if SyncCursor then FPosition := posNormal;
  end;
end;

{{
       KeyDataSet, 
  AFieldName.      
    KeyDataSet.
}
function TovCustomVirtualDataSet.GetKeyDataSetFieldLocateFor(AFieldName:
  String): String;
var
  FieldIndex: Integer;
begin
  ThisDataSetFieldToKeyDataSetField(AFieldName, FieldIndex);
  Result := GetKeyDataSetField(FieldIndex).FieldName
end;

procedure TovCustomVirtualDataSet.SetFiltered(Value: Boolean);
begin
  if Active then
  begin
    CheckBrowseMode;
    KeyDataSet.OnFilterRecord := DoFilterRecord;
    KeyDataSet.Filtered := Value;
    inherited SetFiltered(Value);
    First;
  end else
    inherited SetFiltered(Value);
end;

procedure TovCustomVirtualDataSet.SetOnFilterRecord(const Value:
  TFilterRecordEvent);
begin
  if Active then
  begin
    CheckBrowseMode;
    inherited SetOnFilterRecord(Value);
    if Filtered then First;
  end else
    inherited SetOnFilterRecord(Value);
end;

procedure TovCustomVirtualDataSet.DoFilterRecord(DataSet: TDataSet; var Accept:
  Boolean);
begin
  if Assigned(OnFilterRecord) then OnFilterRecord(Self, Accept)
end;

{ ============================================================================ }

procedure TovCustomVirtualDataSet.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = KeyDataSet) then KeyDataSet := nil
end;

procedure TovCustomVirtualDataSet.SetKeyDataSet(Value: TDataSet);
begin
  if FKeyDataSet <> Value then
  begin
    Close;
    FKeyDataSet := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

{{
   .      
FUniqueFieldNames.
}
function TovCustomVirtualDataSet.GetUniqueFieldName(OriginalFieldName: String):
  String;
var
  Count: Integer;
begin
  Result := OriginalFieldName;
  if FUniqueFieldNames.IndexOf(Result) >= 0 then begin
    Count := 0;
    repeat
      Inc(Count); Result := OriginalFieldName + '_' + IntToStr(Count);
    until FUniqueFieldNames.IndexOf(Result) < 0;
  end
end;

function TovCustomVirtualDataSet.GetCanModify: Boolean;
begin
  Result := FCanModify;
end;

{ ============================  blob- ========================= }

function TovCustomVirtualDataSet.CreateBlobStream(Field: TField; Mode:
    TBlobStreamMode): TStream;
begin
  Result := TovBlobStream.Create(Field as TBlobField, Mode);
end;

{{
     blob-    .
}
function TovCustomVirtualDataSet.IsBlobFieldNull(Field: TField): Boolean;
var
  Stream: TStream;
begin
  Stream := TMemoryStream.Create;
  try
    FBlobData.GetBlobData(Field, Stream);
    Result := Stream.Size = 0;
  finally
    Stream.Free;
  end;
end;

{ TovBlobFieldData }

constructor TovBlobFieldData.Create(ADataSet: TovCustomVirtualDataSet);
begin
  inherited Create;
  FDataSet := ADataSet;
  {     blob-  .    ()
    .  Objects     . }
  FBuffers := TStringList.Create;
end;

destructor TovBlobFieldData.Destroy;
begin
  Clear;
  FreeAndNil(FBuffers);
  inherited Destroy;
end;

procedure TovBlobFieldData.Clear;
begin
  FreeBuffers(FBuffers);
end;

class procedure TovBlobFieldData.FreeBuffer(BufferList: TStrings; BufferIndex:
  Integer);
var
  Buffer: PBlobFieldDataBuffer;
begin
  Buffer := PBlobFieldDataBuffer(BufferList.Objects[BufferIndex]);
  if Buffer <> nil then
  begin
    FreeMem(Buffer, SizeOf(TBlobFieldDataBuffer) + Buffer^.Size);
    BufferList.Objects[BufferIndex] := nil;
  end;
end;

class procedure TovBlobFieldData.FreeBuffers(BufferList: TStrings);
var
  I: Integer;
begin
  for I := 0 to BufferList.Count - 1 do FreeBuffer(BufferList, I);
  BufferList.Clear;
end;

{{
 Blob-        Destination.
}
class procedure TovBlobFieldData.CopyBuffers(Source, Destination: TStrings);
var
  I: Integer;
  NewBuffer, Buffer: PBlobFieldDataBuffer;
begin
  FreeBuffers(Destination);
  for I := 0 to Source.Count - 1 do
  begin
    Buffer := PBlobFieldDataBuffer(Source.Objects[I]);
    GetMem(NewBuffer, SizeOf(TBlobFieldDataBuffer) + Buffer^.Size);
    NewBuffer^.Size := Buffer^.Size;
    Move(Buffer^.Data, NewBuffer^.Data, Buffer^.Size);
    Destination.AddObject(Source[I], TObject(NewBuffer));
  end;
end;

procedure TovBlobFieldData.CopyToList(AData: TStrings);
begin
  CopyBuffers(FBuffers, AData);
end;

procedure TovBlobFieldData.CopyFromList(AData: TStrings);
begin
  CopyBuffers(AData, FBuffers);
end;

class function TovBlobFieldData.NewBufferFromStream(Stream: TStream):
  PBlobFieldDataBuffer;
begin
  GetMem(Result, SizeOf(TBlobFieldDataBuffer) + Stream.Size);
  try
    Result.Size := Stream.Size;
    Stream.Position := 0;
    Stream.Read(Result^.Data, Result.Size);
  except
    FreeMem(Result, SizeOf(TBlobFieldDataBuffer) + Stream.Size);
    raise;
  end;
end;

{{
        .  , 
   ,     LoadBlobDataToBuffer  
        .
}
procedure TovBlobFieldData.GetBlobData(Field: TField; Stream: TStream);
var
  I: Integer;
  Buffer: PBlobFieldDataBuffer;
begin
  I := FBuffers.IndexOf(UpperCase(Field.FieldName));
  if I >= 0 then
    Buffer := PBlobFieldDataBuffer(FBuffers.Objects[I])
  else
  //           -
  //   blob- -       blob-
  if FDataSet.State = dsInsert then Buffer := nil else
  begin
    Buffer := LoadBlobDataToBuffer(Field);
    if Buffer <> nil then
      FBuffers.AddObject(UpperCase(Field.FieldName), TObject(Buffer));
  end;
  Stream.Size := 0;
  if Buffer <> nil then
    Stream.Write(Buffer^.Data, Buffer^.Size);
end;

{{
 Blob-    Field   Stream  .
}
procedure TovBlobFieldData.SetBlobData(Field: TField; Stream: TStream);
var
  I: Integer;
  Buffer: PBlobFieldDataBuffer;
begin
  Buffer := NewBufferFromStream(Stream);
  I := FBuffers.IndexOf(UpperCase(Field.FieldName));
  if I >= 0 then
  begin
    FreeBuffer(FBuffers, I);
    FBuffers.Objects[I] := TObject(Buffer);
  end else
    FBuffers.AddObject(UpperCase(Field.FieldName), TObject(Buffer));
end;

{{
     blob-  .
}
function TovBlobFieldData.LoadBlobDataToBuffer(Field: TField):
  PBlobFieldDataBuffer;
var
  FieldIndex: Integer;
  BlobStream: TStream;
begin
  FDataSet.UpdateCursorPos;
  FDataSet.ThisDataSetFieldToKeyDataSetField(Field.FieldName, FieldIndex);
  with FDataSet do
  begin
    BlobStream := KeyDataSet.CreateBlobStream(GetKeyDataSetField(FieldIndex), bmRead);
    try
      Result := NewBufferFromStream(BlobStream);
    finally
      BlobStream.Free;
    end;
  end
end;

{{
       blob-,  
,          DataSetIndex.
}
procedure TovBlobFieldData.StoreBlobDataFromBuffers;
var
  BlobStream: TStream;
  Field: TField;
  I: Integer;
  J: Integer;
  Buffer: PBlobFieldDataBuffer;
begin
  with FDataSet do
  for I := 0 to FieldMappingCount - 1 do
  begin
    //      
    Field := FDataSet.KeyDataSetFieldToThisDataSetField(I);
    if Field <> nil then
    begin
      J := FBuffers.IndexOf(UpperCase(Field.FieldName));
      if J >= 0 then //       ,   
      begin
        Buffer := PBlobFieldDataBuffer(FBuffers.Objects[J]);
        BlobStream := KeyDataSet.CreateBlobStream(GetKeyDataSetField(I), bmWrite);
        try
          BlobStream.Size := 0;
          BlobStream.Write(Buffer^.Data, Buffer^.Size);
        finally
          BlobStream.Free;
        end;
      end;
    end;
  end;
end;

function TovBlobFieldData.GetFieldBlobBuffer(AFieldName: String):
  PBlobFieldDataBuffer;
var
  I: Integer;
begin
  I := FBuffers.IndexOf(UpperCase(AFieldName));
  if I >= 0 then
    Result := PBlobFieldDataBuffer(FBuffers.Objects[I])
  else
    Result := nil
end;

{ TovBlobStream }

constructor TovBlobStream.Create(Field: TBlobField; Mode: TBlobStreamMode);
begin
  inherited Create;

  FMode := Mode;
  FField := Field;
  FDataSet := FField.DataSet as TovCustomVirtualDataSet;
  if FDataSet.State = dsFilter then
    DatabaseErrorFmt(SNoFieldAccess, [FField.DisplayName], FDataSet);

  if (Mode <> bmWrite) and (FDataSet.State <> dsInsert) then
    FDataSet.FBlobData.GetBlobData(Field, Self);
  {      bmWrite  ,   
    , ..     ,  ,   
     . . TBlobField.Clear }
  FModified := Mode = bmWrite;
  Position := 0;

  FOpened := True;
end;

function TovBlobStream.Write(const Buffer; Count: Longint): Longint;
begin
  Result := inherited Write(Buffer, Count);
  FModified := True;
end;

procedure TovBlobStream.SetSize(NewSize: Longint);
begin
  inherited SetSize(NewSize);
  FModified := True;
end;

destructor TovBlobStream.Destroy;
begin
  if FOpened and FModified then
  begin
    FField.Modified := True;

    if FMode in [bmWrite, bmReadWrite] then
      FDataSet.FBlobData.SetBlobData(FField, Self);

    //     ,       
    //  Modified  TDataSet.CheckBrowseMode   , 
    //  TDataSet.Cancel
    try
      FDataSet.DataEvent(deFieldChange, Longint(FField));
    except
      ApplicationHandleException(Self);
    end;
  end;
  inherited;
end;

{ TChangedRecordList }

constructor TChangedRecordList.Create(ADataSet: TovCustomVirtualDataSet);
begin
  inherited Create;
  FDataSet := ADataSet;
  FRecords := TStringList.Create;
  FBlobBuffers := TObjectList.Create;
end;

destructor TChangedRecordList.Destroy;
begin
  Clear;
  FreeAndNil(FBlobBuffers);
  FreeAndNil(FRecords);
  inherited Destroy;
end;

procedure TChangedRecordList.Clear;
var
  I: Integer;
begin
  for I := 0 to FRecords.Count - 1 do
    FreeMem(Pointer(FRecords.Objects[I]), RecordSize);
  FRecords.Clear;

  for I := 0 to FBlobBuffers.Count - 1 do
    DataSet.FBlobData.FreeBuffers(TStrings(FBlobBuffers[I]));
  FBlobBuffers.Clear;
end;

procedure TChangedRecordList.SetRecordSize(const Value: Integer);
begin
  if FRecordSize <> Value then
  begin
    Clear;
    FRecordSize := Value;
  end;
end;

procedure TChangedRecordList.PutRecord(ARecordKey: String; ABuffer: Pointer);
var
  I: Integer;
  P: Pointer;
  S: TStringList;
begin
  I := FRecords.IndexOf(ARecordKey);
  if I < 0 then
  begin
    GetMem(P, RecordSize);
    I := FRecords.AddObject(ARecordKey, P);

    S := TStringList.Create;
    FBlobBuffers.Add(S);
  end;
  Move(ABuffer^, Pointer(FRecords.Objects[I])^, RecordSize);
  DataSet.FBlobData.CopyToList(TStringList(FBlobBuffers[I]));
end;

function TChangedRecordList.GetRecord(ARecordKey: String; ABuffer: Pointer):
  Boolean;
var
  I: Integer;
begin
  I := FRecords.IndexOf(ARecordKey);
  Result := I >= 0;
  if Result then
  begin
    Move(Pointer(FRecords.Objects[I])^, ABuffer^, RecordSize);
    DataSet.FBlobData.CopyFromList(TStringList(FBlobBuffers[I]));
  end;
end;

end.
