unit ovXMLImportExport;

interface

uses Classes, SysUtils, XMLIntf, Contnrs,
     ovIObjects, ovObjects, ovModel, ovTypes, ovComponents, ovDataObjects;

type
  //   
  TObjectExportOption = (eoObjectId, eoAttributes, eoRelations, eoExportReferencedObjects);
  TObjectExportOptions = set of TObjectExportOption;

  TObjectExportEvent = procedure (Sender: TObject; AObject: IovObject;
    var ExportOptions: TObjectExportOptions) of Object;
  TAfterObjectExportEvent = procedure (Sender: TObject; AObject: IovObject;
    AObjectNode: IXMLNode) of Object;
  TAttributeExportEvent = procedure (Sender: TObject; AObject: IovObject;
    Attribute: String; var Value: String; var Skip: Boolean) of Object;
  TRelationExportEvent = procedure (Sender: TObject; AObject: IovObject;
    ARelation: String; var Skip: Boolean; var ExportOptions: TObjectExportOptions) of Object;

  //   
  TBeforeObjectImportEvent = procedure (Sender: TObject; AObjectClassName: String;
    AObjectNode: IXMLNode; var AObjectId: TOID) of Object;
  TAfterObjectImportEvent = procedure (Sender: TObject; AObject: IovObject) of Object;
  TChildObjectImportEvent = procedure (Sender: TObject; AObject: IovObject;
    ARelation: String; AChildObjectNode: IXMLNode; var AObjectId: TOID) of Object;
  TAttributesImportEvent = procedure (Sender: TObject; AObject: IovObject;
    AObjectNode: IXMLNode; var Skip: Boolean) of Object;
  TRelationImportEvent = procedure (Sender: TObject; AObject: IovObject;
    ARelation: String; AChildObjectNode: IXMLNode; var Skip: Boolean) of Object;

  //  
  TProgressStage = (psMain, psAuxilary);
  TProgressEvent = procedure (Sender: TObject; Stage: TProgressStage; 
    Done, Max: Integer; var Cancel: Boolean) of object;
  EOperationCanceled = class (Exception)
  end;

  TovXMLImportExport = class(TovModelClient)
  private
    FXMLDocument: IXMLDocument;
    FDecimalSeparator: Char;
    FSaveDecimalSeparator: Char;
    FOnProgress: TProgressEvent;
    FObjectIdAttributeName: String;
    FRootObjectsTagName: String;
  protected
    procedure AfterExecute(Success: Boolean); virtual;
    procedure BeforeExecute; virtual;
    procedure DoExecute; virtual; abstract;

    function GetOriginalName(XMLName: String): String;
    function GetValidXMLName(OriginalName: String): String;
    procedure DoProgress(Stage: TProgressStage; Index, Count: Integer); virtual;
    property DecimalSeparator: Char read FDecimalSeparator write FDecimalSeparator;

    property XMLDocument: IXMLDocument read FXMLDocument;
  public
    constructor Create(AOwner: TComponent); override;

    function Execute: Boolean;
    procedure Reset; dynamic;
    procedure SaveToStream(AStream: TStream);
    procedure SaveToFile(FileName: String);
    procedure LoadFromStream(AStream: TStream);
    procedure LoadFromFile(FileName: String);
  published
    property OnProgress: TProgressEvent read FOnProgress write FOnProgress;
    property ObjectIdAttributeName: String read FObjectIdAttributeName write
      FObjectIdAttributeName;
    property RootObjectsTagName: String read FRootObjectsTagName write 
      FRootObjectsTagName;
  end;

  TovCustomXMLSchemaGenerator = class(TovXMLImportExport)
  public
    procedure GenerateSchema(FileName: String); virtual; abstract;
  end;

  TXMLExportOption = (eoKeepEmptyAttributes,
    eoEmbedReferencedObjects);
  TXMLExportOptions = set of TXMLExportOption;

  TovCustomXMLExport = class(TovXMLImportExport)
  private
    FOutputFileName: String;
    FOutputStream: TStream;
    FOnAfterExport: TNotifyEvent;
    FOnAfterObjectExport: TAfterObjectExportEvent;
    FOnObjectExport: TObjectExportEvent;
    FOnRelationExport: TRelationExportEvent;
    FOnAttributeExport: TAttributeExportEvent;
    FOptions: TXMLExportOptions;
    FReferencedObjects: IovObjectList;
    FSchemaGenerator: TovCustomXMLSchemaGenerator;
    FSchemaFileName: String;
    FWrittenObjects: IovObjectList;
    procedure SetSchemaGenerator(Value: TovCustomXMLSchemaGenerator);
  protected
    procedure WriteObject(AObject: IovObject; ARootNode: IXMLNode; ExportOptions:
      TObjectExportOptions);
    procedure WriteObjectAttributes(AObject: IovObject; AModelObject: TModelObject;
      AObjectNode: IXMLNode);
    procedure WriteObjectRelations(AObject: IovObject; AModelObject: TModelObject;
      AObjectNode: IXMLNode);
    procedure AfterExecute(Success: Boolean); override;
    procedure BeforeExecute; override;
    procedure WriteChildObject(AObject: IovObject; AModelRelation: TModelRelation; 
      ARelationNode: IXMLNode; ExportOptions: TObjectExportOptions);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    {{
         XML-    
    }
    property WrittenObjects: IovObjectList read FWrittenObjects;
    {{
         XML-,   
    .      
     ,   XMLDocument.
    }
    property OnAfterExport: TNotifyEvent read FOnAfterExport write FOnAfterExport;
    property OnAfterObjectExport: TAfterObjectExportEvent read FOnAfterObjectExport
      write FOnAfterObjectExport;
    property OnObjectExport: TObjectExportEvent read FOnObjectExport write
      FOnObjectExport;
    property OnRelationExport: TRelationExportEvent read FOnRelationExport write
      FOnRelationExport;
    property OnAttributeExport: TAttributeExportEvent read FOnAttributeExport write
      FOnAttributeExport;
  public
    procedure Save; dynamic;
    procedure Reset; override;
    constructor Create(AOwner: TComponent); override;
    property OutputStream: TStream read FOutputStream write FOutputStream;
    property ReferencedObjects: IovObjectList read FReferencedObjects;
  published
    property OutputFileName: String read FOutputFileName write FOutputFileName;
    property Options: TXMLExportOptions read FOptions write FOptions
      default [];
    property SchemaGenerator: TovCustomXMLSchemaGenerator read FSchemaGenerator
      write SetSchemaGenerator;
    property SchemaFileName: String read FSchemaFileName write FSchemaFileName;
  end;

  {{
       XML   ,
     Objects.
  }
  TovXMLExport = class(TovCustomXMLExport)
  private
    FObjects: IovObjectList;
  protected
    procedure DoExecute; override;
    procedure AfterExecute(Success: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
    property Objects: IovObjectList read FObjects;

    property XMLDocument;
  published
    // properties
    property DecimalSeparator;

    // events
    property OnAfterExport;
    property OnAfterObjectExport;
    property OnObjectExport;
    property OnAttributeExport;
    property OnRelationExport;
  end;

  {{
       XML  ,  
    DataObjectName  DataObjects.
  }
  TovDataObjectsXMLExport = class(TovCustomXMLExport)
  private
    FDataObjects: TovDataObjects;
    FDataObjectName: String;
    procedure SetDataObjects(Value: TovDataObjects);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DoExecute; override;
  public
    property XMLDocument;
  published
    // properties
    property DataObjects: TovDataObjects read FDataObjects write SetDataObjects;
    property DataObjectName: String read FDataObjectName write FDataObjectName;
    property DecimalSeparator;

    // events
    property OnAfterExport;
    property OnAfterObjectExport;
    property OnObjectExport;
    property OnAttributeExport;
    property OnRelationExport;
  end;

  {{
     . ,   ,   
          XML-,
  ..  XML-    , 
           ,
       .
  }
  TReadObjects = class (TObjectList)
  private
    function FindObject(AObjectClassName: String; ADocumentObjectId: TOID; var
      AObject: IovObject): Boolean;
    function GetObjects(Index: Integer): IovObject;
  public
    property Objects[Index: Integer]: IovObject read GetObjects;
  end;

  {{
         . 
          
  XML-.
  }
  TReadObject = class
  private
    FDatabaseObject: IovObject;
    FDocumentObjectId: TOID;
  public
    constructor Create(ADatabaseObject: IovObject; ADocumentObjectId: TOID);
    {{
         
    }
    property DatabaseObject: IovObject read FDatabaseObject;
    {{
      DatabaseObject  XML-.   
      DatabaseObject.ObjectId.
    }
    property DocumentObjectId: TOID read FDocumentObjectId;
  end;

  TovCustomXMLImport = class(TovXMLImportExport)
  private
    FInputFileName: String;
    FInputStream: TStream;
    FOnBeforeImport: TNotifyEvent;
    FOnBeforeObjectImport: TBeforeObjectImportEvent;
    FOnRelationImport: TRelationImportEvent;
    FOnChildObjectImport: TChildObjectImportEvent;
    FOnAfterObjectImport: TAfterObjectImportEvent;
    FOnAttributesImport: TAttributesImportEvent;
    FReadObjects: TReadObjects;
    procedure ReadObject(AObject: IovObject; AObjectNode: IXMLNode; AReadRelations:
      Boolean);
    procedure ReadObjectAttributes(AObject: IovObject; AObjectNode: IXMLNode);
    procedure ReadObjectRelations(AObject: IovObject; AObjectNode: IXMLNode;
      AReadRelations: Boolean);
    procedure GetObjectFromNode(AObjectNode: IXMLNode; var ObjectClassName: String;
      var ObjectId: TOID);
  protected
    property InputFileName: String read FInputFileName write FInputFileName;
    property InputStream: TStream read FInputStream write FInputStream;
    {{
         XML-    
    }
    property ReadObjects: TReadObjects read FReadObjects;
    {{
        XML-,    .
           
    ,   XMLDocument.
    }
    property OnBeforeImport: TNotifyEvent read FOnBeforeImport write
      FOnBeforeImport;
    property OnBeforeObjectImport: TBeforeObjectImportEvent read
      FOnBeforeObjectImport write FOnBeforeObjectImport;
    property OnAfterObjectImport: TAfterObjectImportEvent read FOnAfterObjectImport
      write FOnAfterObjectImport;
    property OnRelationImport: TRelationImportEvent read FOnRelationImport write
      FOnRelationImport;
    property OnChildObjectImport: TChildObjectImportEvent read FOnChildObjectImport
      write FOnChildObjectImport;
    property OnAttributesImport: TAttributesImportEvent read FOnAttributesImport
        write FOnAttributesImport;

    procedure BeforeExecute; override;
    function GetObjectInstance(ObjectClassName: String; ObjectId: TOID): IovObject;
    function GetChildObject(AObject: IovObject; ARelationName: String;
      AChildObjectNode: IXMLNode): IovObject; virtual;
  public
    procedure Open; dynamic;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
  end;

  TovXMLImport = class(TovCustomXMLImport)
  private
    procedure ReadRootObject(AObjectNode: IXMLNode; AReadRelations: Boolean);
  protected
    procedure DoExecute; override;
    function GetChildObject(AObject: IovObject; ARelationName: String;
      AChildObjectNode: IXMLNode): IovObject; override;
  public
    property XMLDocument;
    property InputStream;
  published
    property DecimalSeparator;
    property InputFileName;
    property OnBeforeImport;
    property OnAfterObjectImport;
    property OnBeforeObjectImport;
    property OnChildObjectImport;
    property OnAttributesImport;
    property OnRelationImport;
  end;

implementation

uses XMLDoc, Variants, DB, ovConsts;

{{
        nil, 
  .
}
function GetTopOwner(AObject: IovObject): IovObject;
var
  Owner: IovObject;
begin
  Owner := AObject.Owner;
  if Owner = nil then Result := nil
  else
  begin
    while Owner <> nil do
    begin
      AObject := Owner;
      Owner := AObject.Owner;
    end;
    Result := AObject;
  end;
end;

{ TovCustomXMLImportExport }

constructor TovXMLImportExport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDecimalSeparator := '.';
  FRootObjectsTagName := 'Objects';
  FObjectIdAttributeName := 'ID';
end;

function TovXMLImportExport.Execute: Boolean;
var
  Success: Boolean;
begin
  Success := False;
  Result := True;
  ModelNeeded;

  BeforeExecute;
  try
    try
      DoExecute;
      Success := True;
    except
      on EOperationCanceled do Result := False;
    end;
  finally
    AfterExecute(Success)
  end;
end;

procedure TovXMLImportExport.BeforeExecute;
begin
  FSaveDecimalSeparator := SysUtils.DecimalSeparator;
  SysUtils.DecimalSeparator := Self.DecimalSeparator;
end;

procedure TovXMLImportExport.AfterExecute(Success: Boolean);
begin
  SysUtils.DecimalSeparator := FSaveDecimalSeparator;
end;

procedure TovXMLImportExport.Reset;
begin
  FXMLDocument := NewXMLDocument;
  FXMLDocument.Encoding := 'UTF-8';
  FXMLDocument.Options := FXMLDocument.Options + [doAttrNull];
end;

procedure TovXMLImportExport.LoadFromStream(AStream: TStream);
begin
  Reset;
  XMLDocument.LoadFromStream(AStream);
end;

procedure TovXMLImportExport.SaveToStream(AStream: TStream);
begin
  if Not Assigned(XMLDocument) then raise Exception.Create(' ');
  XMLDocument.SaveToStream(AStream);
  Reset;
end;

procedure TovXMLImportExport.LoadFromFile(FileName: String);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead);
  try
    LoadFromStream(Stream)
  finally
    Stream.Free
  end;
end;

procedure TovXMLImportExport.SaveToFile(FileName: String);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(Stream)
  finally
    Stream.Free
  end;
end;

{{
     ,   XML-.
,       "_".
}
function TovXMLImportExport.GetValidXMLName(OriginalName: String): String;
begin
  Result := StringReplace(OriginalName, ' ', '_', [rfReplaceAll]);
  Result := StringReplace(Result, '::', ':', [rfReplaceAll]);
end;

{{
  .
}
function TovXMLImportExport.GetOriginalName(XMLName: String): String;
begin
  Result := StringReplace(XMLName, ':', '::', [rfReplaceAll]);
  Result := StringReplace(Result, '_', ' ', [rfReplaceAll]);
end;

procedure TovXMLImportExport.DoProgress(Stage: TProgressStage; Index, Count:
  Integer);
var
  Cancel: Boolean;
begin
  Cancel := False;
  if Assigned(FOnProgress) then FOnProgress(Self, Stage, Index+1, Count, Cancel);
  if Cancel then
    raise EOperationCanceled.CreateFmt('   %d', [Index+1]);
end;

{ TovCustomXMLExport }

constructor TovCustomXMLExport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FReferencedObjects := NewObjectList;
  FWrittenObjects := NewObjectList;
  Options := [];
end;

procedure TovCustomXMLExport.SetSchemaGenerator(Value: TovCustomXMLSchemaGenerator);
begin
  if SchemaGenerator <> Value then
  begin
    FSchemaGenerator := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end
end;

procedure TovCustomXMLExport.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = SchemaGenerator) and (Operation = opRemove) then SchemaGenerator := nil
end;

procedure TovCustomXMLExport.BeforeExecute;
begin
  inherited;
  if Not Assigned(XMLDocument) then Reset;
end;

procedure TovCustomXMLExport.Reset;
var
  I: Integer;
  RootNode: IXMLNode;
begin
  inherited;
  RootNode := FXMLDocument.AddChild(RootObjectsTagName);

  //   XML-,      XML

  //   ,    
  with Model.DomainModel do
    for I := 0 to NameSpaces.Count - 1 do
      if NameSpaces[I] <> '' then
        { TODO :     URI (   ) }
        RootNode.Attributes['xmlns:'+GetValidXMLName(NameSpaces[I])] := NameSpaces[I];

  if SchemaFileName <> ''then
  begin
    RootNode.Attributes['xmlns:xsi'] := 'http://www.w3.org/2001/XMLSchema-instance';
    RootNode.Attributes['xsi:noNamespaceSchemaLocation'] := SchemaFileName;
  end
end;

procedure TovCustomXMLExport.AfterExecute(Success: Boolean);
begin
  if Success then
  begin
    if Assigned(FOnAfterExport) then FOnAfterExport(Self);
    Save;
  end;

  WrittenObjects.Clear;

  inherited;
end;

procedure TovCustomXMLExport.Save;
begin
  if OutputStream <> nil then SaveToStream(OutputStream) else
  if OutputFileName <> '' then SaveToFile(OutputFileName);

  if (SchemaFileName <> '') and Assigned(SchemaGenerator) then
    SchemaGenerator.GenerateSchema(SchemaFileName);
end;

procedure TovCustomXMLExport.WriteObject(AObject: IovObject; ARootNode:
  IXMLNode; ExportOptions: TObjectExportOptions);
var
  ObjectNode: IXMLNode;
  ModelObject: TModelObject;
begin
  try
    ObjectNode := ARootNode.AddChild(GetValidXMLName(AObject.ClassName));
    ModelObject := AObject.ModelObject;

    if Assigned(FOnObjectExport) then FOnObjectExport(Self, AObject, ExportOptions);

    if eoObjectId in ExportOptions then
      ObjectNode.Attributes[ObjectIdAttributeName] := AObject.ObjectId;

    //    
    if eoAttributes in ExportOptions then
      WriteObjectAttributes(AObject, ModelObject, ObjectNode);

    //   
    if eoRelations in ExportOptions then
      WriteObjectRelations(AObject, ModelObject, ObjectNode);

    if Assigned(FOnAfterObjectExport) then FOnAfterObjectExport(Self, AObject, ObjectNode);

    //     ,        
    if (eoAttributes in ExportOptions) and (WrittenObjects.IndexOfObject(AObject) < 0)
      then WrittenObjects.Add(AObject);
  except
    on E: Exception do raise Exception.CreateFmt(
      '    "%s"   %d.'#13' "%s"',
      [AObject.ClassName, AObject.ObjectId, E.Message]);
  end
end;

procedure TovCustomXMLExport.WriteObjectAttributes(AObject: IovObject;
  AModelObject: TModelObject; AObjectNode: IXMLNode);
var
  AttributeNode: IXMLNode;
  Skip: Boolean;
  I: Integer;
  AttributeName, Value: String;
  Attribute: TModelAttribute;
  IsBinary: Boolean;
begin
  for I := 0 to AModelObject.AttributeCount - 1 do
  begin
    Attribute := AModelObject.Attributes[I];
    if Not Attribute.Calculated then
    begin
      AttributeName := Attribute.Name;
      IsBinary := Attribute.IsBlob;
  
      if Attribute.IsBoolean then
        //    XML-
        Value := LowerCase(AObject[AttributeName].AsString)
      else
        if Not IsBinary then
          Value := AObject[AttributeName].AsString
        else
          Value := '';
  
      Skip := False;
      if Assigned(FOnAttributeExport) then
        FOnAttributeExport(Self, AObject, AttributeName, Value, Skip);
  
      if Not Skip and ((Value <> '') or (eoKeepEmptyAttributes in Options)) then
        if Not AObject[AttributeName].IsBlob then
          AObjectNode.Attributes[GetValidXMLName(AttributeName)] := Value
        else if Not IsBinary then begin
          AttributeNode := AObjectNode.AddChild(GetValidXMLName(AttributeName));
          AttributeNode.Attributes['type'] := 'memo';
          AttributeNode.NodeValue := Value;
  
          { TODO :   Blob-   "blob" }
        end
    end;
  end;
end;

procedure TovCustomXMLExport.WriteObjectRelations(AObject: IovObject;
  AModelObject: TModelObject; AObjectNode: IXMLNode);
var
  Skip: Boolean;
  RelationNode: IXMLNode;
  ModelRelation: TModelRelation;
  I, J: Integer;
  ExportOptions: TObjectExportOptions;
begin
  for I := 0 to AModelObject.RelationCount - 1 do
  begin
    ModelRelation := AModelObject.Relations[I];

    if ModelRelation.IsComposition then
      //         
      ExportOptions := [eoObjectId, eoAttributes, eoRelations]
    else begin
      //      
      //   (   eoEmbedReferencedObjects)
      if eoEmbedReferencedObjects in Options then
        ExportOptions := [eoObjectId, eoAttributes, eoRelations]
      else
        ExportOptions := [eoObjectId, eoExportReferencedObjects];
    end;

    Skip := False;
    if Assigned(FOnRelationExport) then
      FOnRelationExport(Self, AObject, ModelRelation.Name, Skip, ExportOptions);

    if Not Skip then
    with AObject[ModelRelation.Name].AsObjects do
    if Count > 0 then
    begin
      RelationNode := AObjectNode.AddChild(GetValidXMLName(ModelRelation.Name));
      for J := 0 to Count - 1 do
        WriteChildObject(Objects[J], ModelRelation, RelationNode, ExportOptions);
    end
  end
end;

procedure TovCustomXMLExport.WriteChildObject(AObject: IovObject;
  AModelRelation: TModelRelation; ARelationNode: IXMLNode; ExportOptions:
  TObjectExportOptions);
begin
  WriteObject(AObject, ARelationNode, ExportOptions);
  if (eoExportReferencedObjects in ExportOptions) and Not AModelRelation.IsComposition and
     (ReferencedObjects.IndexOfObject(AObject) < 0)
  then ReferencedObjects.Add(AObject)
end;


{ TovXMLExport }

constructor TovXMLExport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FObjects := NewObjectList;
end;

procedure TovXMLExport.AfterExecute(Success: Boolean);
begin
  inherited;
  Objects.Clear;
end;

{{
   .

:
1.          .
       .   
( .. )     .
2.          
     .    
    .    ( .. )
    .

 :
-           .
  -     .
-           
  ( )  .  ,  -
   ()      .
}
procedure TovXMLExport.DoExecute;
var
  Count: Integer;
  I: Integer;
  AOwner: IovObject;
begin
  ReferencedObjects.Clear;

  //     , .   
  Count := Objects.Count;
  for I := 0 to Count - 1 do
  begin
    WriteObject(Objects[I], XMLDocument.DocumentElement,
      [eoObjectId, eoAttributes, eoRelations]);
    DoProgress(psMain, I, Count);
  end;

  //   ,      ,
  //   ,     
  I := 0;
  while ReferencedObjects.Count > 0 do
  begin
    AOwner := GetTopOwner(ReferencedObjects[0]);
    if (AOwner <> nil) and (WrittenObjects.IndexOfObject(AOwner) < 0) then
    begin
      ReferencedObjects.Add(AOwner); //      
      ReferencedObjects.Delete(0);
    end
    else
    begin
      //      
      if WrittenObjects.IndexOfObject(ReferencedObjects[0]) < 0 then
        WriteObject(ReferencedObjects[0], XMLDocument.DocumentElement,
          [eoObjectId, eoAttributes, eoRelations]);
      ReferencedObjects.Delete(0);
    end;

    DoProgress(psAuxilary, I, ReferencedObjects.Count);
    Inc(I);
  end;
end;

{ TovDataObjectsXMLExport }

procedure TovDataObjectsXMLExport.SetDataObjects(Value: TovDataObjects);
begin
  if FDataObjects <> Value then
  begin
    FDataObjects := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TovDataObjectsXMLExport.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = DataObjects)
    then DataObjects := nil
end;

procedure TovDataObjectsXMLExport.DoExecute;
var
  Index: Integer;
  Count: Integer;
begin
  if Not Assigned(DataObjects) then
    raise Exception.Create('   DataObjects');

  with DataObjects[DataObjectName] do
  begin
    Count := RecordCount;
    Index := 0;
    First;

    while Not Eof do
    begin
      WriteObject(CurrentObject, XMLDocument.DocumentElement,
        [eoObjectId, eoAttributes, eoRelations]);
      DoProgress(psMain, Index, Count);
      
      Inc(Index);
      Next;
    end
  end
end;

{ TovCustomXMLImport }

constructor TovCustomXMLImport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FReadObjects := TReadObjects.Create;
end;

destructor TovCustomXMLImport.Destroy;
begin
  FreeAndNil(FReadObjects);
  inherited Destroy;
end;

procedure TovCustomXMLImport.BeforeExecute;
begin
  FReadObjects.Clear;
  inherited;
  Open;
  if Assigned(FOnBeforeImport) then FOnBeforeImport(Self);
end;

procedure TovCustomXMLImport.Open;
begin
  if InputStream <> nil then LoadFromStream(InputStream) else
  if InputFileName <> '' then LoadFromFile(InputFileName);
end;

procedure TovCustomXMLImport.ReadObject(AObject: IovObject; AObjectNode:
  IXMLNode; AReadRelations: Boolean);
var
  ObjectClassName: String;
  DocumentObjectId: TOID;
begin
  if Not AReadRelations then ReadObjectAttributes(AObject, AObjectNode);
  ReadObjectRelations(AObject, AObjectNode, AReadRelations);
  GetObjectFromNode(AObjectNode, ObjectClassName, DocumentObjectId);
  ReadObjects.Add(TReadObject.Create(AObject, DocumentObjectId));
  {        ,
    ..     }
  if AReadRelations and Assigned(FOnAfterObjectImport) then
    FOnAfterObjectImport(Self, AObject);
end;

procedure TovCustomXMLImport.ReadObjectAttributes(AObject: IovObject;
    AObjectNode: IXMLNode);
var
  AttributeName: String;
  I: Integer;
  Skip: Boolean;
begin
  Skip := False;
  if Assigned(FOnAttributesImport) then
    FOnAttributesImport(Self, AObject, AObjectNode, Skip);
  if Not Skip then
  begin

    { TODO :  BeginUpdate/EndUpdate, ..  
      EndUpdate      
      .  ,    . }

    //AObject.BeginUpdate;
    try
      for I := 0 to AObjectNode.AttributeNodes.Count - 1 do
      begin
        if AObjectNode.AttributeNodes[I].NodeName <> ObjectIdAttributeName then
        begin
          AttributeName := AObjectNode.AttributeNodes[I].NodeName;
          AObject[GetOriginalName(AttributeName)].AsString := AObjectNode.Attributes[AttributeName];
        end
      end;
      //    Memo-
      for I := 0 to AObjectNode.ChildNodes.Count - 1 do
      if AObjectNode.ChildNodes[I].Attributes['type'] = 'memo' then
      begin
        AttributeName := GetOriginalName(AObjectNode.ChildNodes[I].LocalName);
        AObject[AttributeName].Value := AObjectNode.ChildNodes[I].NodeValue;
      end;
    finally
      //AObject.EndUpdate;
    end;
  end;
end;

procedure TovCustomXMLImport.ReadObjectRelations(AObject: IovObject;
  AObjectNode: IXMLNode; AReadRelations: Boolean);
var
  RelationName: String;
  Skip: Boolean;
  Child: IovObject;
  RelationNode: IXMLNode;
  Relation: TModelRelation;
  I, J: Integer;
  AModelObject: TModelObject;
begin
  AModelObject := DomainModel.GetObject(AObject.ClassName);
  for I := 0 to AObjectNode.ChildNodes.Count - 1 do
  if VarIsNull(AObjectNode.ChildNodes[I].Attributes['type']) then
  begin
    RelationNode := AObjectNode.ChildNodes[I];
    RelationName := GetOriginalName(RelationNode.LocalName);
    Relation := AModelObject.GetRelation(RelationName);

    if Not AReadRelations then AObject[RelationName].AsObjects.DeleteAll;
    //    
    for J := 0 to RelationNode.ChildNodes.Count - 1 do
    begin
      Skip := False;
      if Assigned(FOnRelationImport) then
        FOnRelationImport(Self, AObject, RelationName, RelationNode.ChildNodes[J], Skip);
      if Not Skip then
      begin
        if Relation.IsComposition then
          if Not AReadRelations then
            ReadObject(AObject[RelationName].Add, RelationNode.ChildNodes[J], AReadRelations)
          else
          begin
            //           ReadObject
            Child := GetChildObject(AObject, RelationName, RelationNode.ChildNodes[J]);
            if Child <> nil then
              ReadObject(Child, RelationNode.ChildNodes[J], AReadRelations);
          end
        else
          if AReadRelations then //      
          begin
            Child := GetChildObject(AObject, RelationName, RelationNode.ChildNodes[J]);
            if Child <> nil then begin
              ReadObject(Child, RelationNode.ChildNodes[J], AReadRelations);
              AObject[RelationName].Add(Child);
            end
          end
      end;
    end
  end
end;

function TovCustomXMLImport.GetChildObject(AObject: IovObject; ARelationName:
  String; AChildObjectNode: IXMLNode): IovObject;
var
  ObjectId: TOID;
  ObjectClassName: String;
begin
  //  ObjectId,   XML-
  GetObjectFromNode(AChildObjectNode, ObjectClassName, ObjectId);

  //    ObjectId
  if Assigned(FOnChildObjectImport) then
    FOnChildObjectImport(Self, AObject, ARelationName, AChildObjectNode, ObjectId);

  Result := GetObjectInstance(ObjectClassName, ObjectId);
end;

{{
    ,    AObjectNode
}
procedure TovCustomXMLImport.GetObjectFromNode(AObjectNode: IXMLNode; var
  ObjectClassName: String; var ObjectId: TOID);
begin
  if VarIsNull(AObjectNode.Attributes[ObjectIdAttributeName]) then
    ObjectId := 0
  else
    ObjectId := AObjectNode.Attributes[ObjectIdAttributeName];

  ObjectClassName := GetOriginalName(AObjectNode.NodeName);
end;

{{
       .
}
function TovCustomXMLImport.GetObjectInstance(ObjectClassName: String;
  ObjectId: TOID): IovObject;
begin
  if ObjectId < 0 then
    //  
    Result := nil
  else
  if ObjectId = 0 then
    //    
    Result := Model.ObjectSpace.CreateObject(ObjectClassName)
  else
    //    
    begin
      Result := Model.ObjectSpace.GetObject(ObjectClassName, ObjectId);
      if Not Result.IsExist then
        raise Exception.CreateFmt('    "%s"   %d', [ObjectClassName, ObjectId]);
    end;
end;

{ TovXMLImport }

{{
      AObjectNode.  AReadRelations
   :   (AReadRelations = True) 
   ,    (AReadRelations = False).
}
procedure TovXMLImport.ReadRootObject(AObjectNode: IXMLNode; AReadRelations:
  Boolean);
var
  AObject: IovObject;
  ObjectId, DocumentObjectId: TOID;
  ObjectClassName: String;
begin
  //  ObjectId,   XML-
  GetObjectFromNode(AObjectNode, ObjectClassName, DocumentObjectId);

  if Not AReadRelations then
  begin

    ObjectId := 0;
    if Assigned(FOnBeforeObjectImport) then
      FOnBeforeObjectImport(Self, ObjectClassName, AObjectNode, ObjectId);
  
    AObject := GetObjectInstance(ObjectClassName, ObjectId);

    if AObject <> nil then //    
      ReadObject(AObject, AObjectNode, AReadRelations);
  end else begin
    if ReadObjects.FindObject(ObjectClassName, DocumentObjectId, AObject) then
      ReadObject(AObject, AObjectNode, AReadRelations);
  end
end;

{{
    ,   AChildObjectNode,
   AObject,   ARelationName.
}
function TovXMLImport.GetChildObject(AObject: IovObject; ARelationName: String;
  AChildObjectNode: IXMLNode): IovObject;
var
  ObjectId: TOID;
  ObjectClassName: String;
begin
  //  ObjectId,   XML-
  GetObjectFromNode(AChildObjectNode, ObjectClassName, ObjectId);
  if Not ReadObjects.FindObject(ObjectClassName, ObjectId, Result) then
    Result := inherited GetChildObject(AObject, ARelationName, AChildObjectNode);
end;

{{
  :

1.    (       ()   )
2.   ,       
3.           
(       ,
  ),       
  .
}
procedure TovXMLImport.DoExecute;
var
  Count: Integer;
  I: Integer;
begin
  with Model.ObjectSpace do
  begin
    StartTransaction;
    try

      with XMLDocument.DocumentElement do
      begin
        //   
        Count := ChildNodes.Count;
        for I := 0 to Count - 1 do
        begin
          ReadRootObject(ChildNodes[I], False);
          DoProgress(psMain, I, Count);
        end;

        //     
        for I := 0 to Count - 1 do
        begin
          ReadRootObject(ChildNodes[I], True);
          DoProgress(psAuxilary, I, Count);
        end;

        //      -   
        with ReadObjects do
          for I := 0 to Count - 1 do
            Objects[I].UpdateCalculatedAttributes;
      end;

      CommitTransaction;
    except
      RollbackTransaction;
      raise;
    end;
  end
end;

{ TReadObject }

constructor TReadObject.Create(ADatabaseObject: IovObject; ADocumentObjectId:
  TOID);
begin
  inherited Create;
  FDatabaseObject := ADatabaseObject;
  FDocumentObjectId := ADocumentObjectId;
end;

{ TReadObjects }

{{
      AObjectClassName, 
 ADocumentObjectId,    XML-.
}
function TReadObjects.FindObject(AObjectClassName: String; ADocumentObjectId:
  TOID; var AObject: IovObject): Boolean;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    with TReadObject(inherited Items[I]) do
      if (AObjectClassName = DatabaseObject.ClassName) and
         (ADocumentObjectId = DocumentObjectId) then
      begin
        Result := True;
        AObject := DatabaseObject;
        Exit;
      end;
  Result := False;
end;

function TReadObjects.GetObjects(Index: Integer): IovObject;
begin
  Result := TReadObject(inherited Items[Index]).DatabaseObject;
end;

end.
