unit ovXMLObjectStorage;

interface

uses SysUtils, Classes, XMLIntf, ovObjectStorage, ovIObjects, ovModel, ovTypes;

type
  TXMLObjectReaderWriter = class(TInterfacedObject)
  private
    FXMLDocument: IXMLDocument;
  protected
    property XMLDocument: IXMLDocument read FXMLDocument;
  public
    constructor Create(AXMLDocument: IXMLDocument);
    function GetNodeFromHandle(Handle: TObjectHandle): IXMLNode;
    function GetHandleFromNode(Node: IXMLNode): TObjectHandle;
  end;

  TOnWriteObjectEvent = procedure (Sender: TObject;
    AObjectClassName: String; AObjectId: TOID; AObjectNode: IXMLNode) of object;
  TOnWriteReferenceEvent = procedure(Sender: TObject; AObjectClassName: String;
    AObjectId: TOID; AReferenceNode: IXMLNode; var AddToRefObjects: Boolean) of
    object;

  TXMLObjectWriter = class(TXMLObjectReaderWriter, IovObjectWriter)
  private
    FOnWriteObject: TOnWriteObjectEvent;
    FOnWriteReference: TOnWriteReferenceEvent;
  public
    function StartWriteObject(AObjectClassName: String; AObjectId: TOID):
      TObjectHandle;
    procedure WriteAttribute(Handle: TObjectHandle; Attribute: TModelAttribute;
      Value: Variant);
    function StartWriteRelation(Handle: TObjectHandle; Relation: TModelRelation):
      TObjectHandle;
    function StartWriteChildObject(Handle: TObjectHandle; AObjectClassName: String;
      AObjectId: TOID): TObjectHandle;
    procedure WriteReference(Handle: TObjectHandle; AObjectClassName: String;
      AObjectId: TOID; var AddToRefObjects: Boolean);
    property OnWriteObject: TOnWriteObjectEvent read FOnWriteObject write
      FOnWriteObject;
    property OnWriteReference: TOnWriteReferenceEvent read FOnWriteReference write
      FOnWriteReference;
  end;

  TXMLObjectReader = class(TXMLObjectReaderWriter, IovObjectReader)
  public
    {{
         .
    }
    function GetObjectCount: Integer;
    {{
             
        ReadAttribute  ReadRelation. 
          0  GetObjectCount-1.
    }
    function StartReadObject(Index: Integer): TObjectHandle;
    function GetRelationCount(Handle: TObjectHandle; Relation: TModelRelation):
      Integer;
    {{
          .  
       ,     Null.
    }
    function ReadAttribute(Handle: TObjectHandle; Attribute: TModelAttribute):
      Variant;
    {{
        ,   Relation.
    }
    function ReadRelation(Handle: TObjectHandle; Relation: TModelRelation; Index:
      Integer): TObjectHandle;
    function ReadChildObject(Handle: TObjectHandle; Relation: TModelRelation;
      Index: Integer): TObjectHandle;
    function FindNode(ParentNode: IXMLNode; NodeName: String; NodeType: String =
      ''): IXMLNode;
    procedure GetObject(Handle: TObjectHandle; var AObjectClassName: String; var
      AObjectId: TOID);
  end;

  {{
        XML-.

     :

  with TovXMLObjectStorage.Create(AOwner) do
  try
    WriteObjectList(AObjectList);
    SaveToFile('Sample.xml');
  finally
    Free
  end

     :

  with TovXMLObjectStorage.Create(AOwner) do
  try
    LoadFromFile('Sample.xml');
    ReadObjectList(AObjectList);
  finally
    Free
  end
  }
  TovXMLObjectStorage = class(TovObjectStorage)
  private
    FXMLDocument: IXMLDocument;
    FOnWriteObjectToXML: TOnWriteObjectEvent;
    FXMLObjectReader: TXMLObjectReader;
    FXMLObjectWriter: TXMLObjectWriter;
    FOnWriteReferenceToXML: TOnWriteReferenceEvent;
  protected
    procedure GetReaderWriter(var AReader: IovObjectReader; var AWriter:
      IovObjectWriter); dynamic;
    procedure Initialize(ReadMode: Boolean); override;
    property XMLDocument: IXMLDocument read FXMLDocument;
    property XMLObjectReader: TXMLObjectReader read FXMLObjectReader;
    property XMLObjectWriter: TXMLObjectWriter read FXMLObjectWriter;
  public
    procedure SaveToStream(AStream: TStream); dynamic;
    procedure Clear; dynamic;
    procedure LoadFromFile(FileName: String);
    procedure LoadFromStream(AStream: TStream); dynamic;
    procedure SaveToFile(FileName: String);
  published
    property OnWriteObject;
    property OnWriteAttribute;
    property OnWriteRelation;

    property OnReadObject;
    property OnReadRelation;
    property OnFindObject;
    property OnMatchObject;

    property OnUpdateProgress;
    property OnShowProgress;
    property OnHideProgress;

    property OnWriteObjectToXML: TOnWriteObjectEvent read FOnWriteObjectToXML write
      FOnWriteObjectToXML;
    property OnWriteReferenceToXML: TOnWriteReferenceEvent read
      FOnWriteReferenceToXML write FOnWriteReferenceToXML;
  end;

implementation

uses XMLDoc, Variants, ovComponents, DB;

{{
     ,   XML-.
,       "_".
}
function GetValidXMLName(OriginalName: String): String;
begin
  Result := StringReplace(OriginalName, ' ', '_', [rfReplaceAll]);
  Result := StringReplace(Result, '::', ':', [rfReplaceAll]);
end;

{{
  .
}
function GetOriginalName(XMLName: String): String;
begin
  Result := StringReplace(XMLName, ':', '::', [rfReplaceAll]);
  Result := StringReplace(Result, '_', ' ', [rfReplaceAll]);
end;

function BlobToStr(Value: Variant): String;
begin
  Result := ''
  { TODO :
         
    (    )
  }
end;

function StrToBlob(Value: String): Variant;
begin
  Result := ''
  { TODO :
         
    (    )
  }
end;

const
  ObjectIdAttributeName = 'ID';
  RootElementTagName = 'Objects';

{ TXMLObjectReaderWriter }

constructor TXMLObjectReaderWriter.Create(AXMLDocument: IXMLDocument);
begin
  inherited Create;
  FXMLDocument := AXMLDocument;
end;

function TXMLObjectReaderWriter.GetNodeFromHandle(Handle: TObjectHandle):
  IXMLNode;
begin
  Result := IXMLNode(Handle);
end;

function TXMLObjectReaderWriter.GetHandleFromNode(Node: IXMLNode):
  TObjectHandle;
begin
  Result := TObjectHandle(Node);
end;

{ TXMLObjectWriter }

function TXMLObjectWriter.StartWriteObject(AObjectClassName: String; AObjectId:
  TOID): TObjectHandle;
var
  ObjectNode: IXMLNode;
begin
  ObjectNode := XMLDocument.DocumentElement.AddChild(GetValidXMLName(AObjectClassName));
  ObjectNode.Attributes[ObjectIdAttributeName] := AObjectId;
  Result := TObjectHandle(ObjectNode);
  if Assigned(FOnWriteObject) then
    FOnWriteObject(Self, AObjectClassName, AObjectId, ObjectNode);
end;

procedure TXMLObjectWriter.WriteAttribute(Handle: TObjectHandle;
  Attribute: TModelAttribute; Value: Variant);
var
  AttributeNode: IXMLNode;
  AFieldType: TFieldType;
  AFieldSize: Integer;
begin
  if Attribute.IsMemo then
  begin
    AttributeNode := IXMLNode(Handle).AddChild(GetValidXMLName(Attribute.Name));
    AttributeNode.Attributes['type'] := 'memo';
    AttributeNode.NodeValue := Value;
  end else
  if Attribute.IsBlob then
  begin
    AttributeNode := IXMLNode(Handle).AddChild(GetValidXMLName(Attribute.Name));
    AttributeNode.Attributes['type'] := 'blob';
    AttributeNode.NodeValue := BlobToStr(Value);
  end else
  begin
    Attribute.GetFieldDef(AFieldType, AFieldSize);
    if VarIsNull(Value) then
      IXMLNode(Handle).Attributes[GetValidXMLName(Attribute.Name)] := Value
    else
      case AFieldType of
        ftDate:
          IXMLNode(Handle).Attributes[GetValidXMLName(Attribute.Name)] := DateToStr(Value);
        ftTime:
          IXMLNode(Handle).Attributes[GetValidXMLName(Attribute.Name)] := TimeToStr(Value);
        else
          IXMLNode(Handle).Attributes[GetValidXMLName(Attribute.Name)] := Value
      end
  end;
end;

function TXMLObjectWriter.StartWriteRelation(Handle: TObjectHandle; Relation:
  TModelRelation): TObjectHandle;
begin
  Result := TObjectHandle(IXMLNode(Handle).AddChild(GetValidXMLName(Relation.Name)));
end;

procedure TXMLObjectWriter.WriteReference(Handle: TObjectHandle;
  AObjectClassName: String; AObjectId: TOID; var AddToRefObjects: Boolean);
var
  ReferenceNode: IXMLNode;
begin
  ReferenceNode := IXMLNode(Handle).AddChild(GetValidXMLName(AObjectClassName));
  ReferenceNode.Attributes[ObjectIdAttributeName] := AObjectId;
  if Assigned(FOnWriteReference) then
    FOnWriteReference(Self, AObjectClassName, AObjectId, ReferenceNode, AddToRefObjects);
end;

function TXMLObjectWriter.StartWriteChildObject(Handle: TObjectHandle;
  AObjectClassName: String; AObjectId: TOID): TObjectHandle;
var
  ObjectNode: IXMLNode;
begin
  ObjectNode := IXMLNode(Handle).AddChild(GetValidXMLName(AObjectClassName));
  ObjectNode.Attributes[ObjectIdAttributeName] := AObjectId;
  Result := TObjectHandle(ObjectNode);
  if Assigned(FOnWriteObject) then
    FOnWriteObject(Self, AObjectClassName, AObjectId, ObjectNode);
end;

{ TXMLObjectReader }

function TXMLObjectReader.GetObjectCount: Integer;
begin
  Result := XMLDocument.DocumentElement.ChildNodes.Count
end;

function TXMLObjectReader.StartReadObject(Index: Integer): TObjectHandle;
begin
  Result := TObjectHandle(XMLDocument.DocumentElement.ChildNodes[Index]);
end;

function TXMLObjectReader.ReadAttribute(Handle: TObjectHandle; Attribute:
  TModelAttribute): Variant;
var
  AttributeNode: IXMLNode;
  AFieldType: TFieldType;
  AFieldSize: Integer;
  AValue: Variant;
begin
  if Attribute.IsMemo then
  begin
    AttributeNode := FindNode(IXMLNode(Handle), GetValidXMLName(Attribute.Name), 'memo');
    if Assigned(AttributeNode) then
      Result := AttributeNode.NodeValue
    else
      Result := Null
  end else
  if Attribute.IsBlob then
  begin
    AttributeNode := FindNode(IXMLNode(Handle), GetValidXMLName(Attribute.Name), 'blob');
    if Assigned(AttributeNode) then
      Result := StrToBlob(AttributeNode.NodeValue)
    else
      Result := Null
  end else
  begin
    Attribute.GetFieldDef(AFieldType, AFieldSize);
    AValue := IXMLNode(Handle).Attributes[GetValidXMLName(Attribute.Name)];
    if VarIsNull(AValue) then Result := Null
    else
    case AFieldType of
      ftDate:
        Result := StrToDate(AValue);
      ftTime:
        Result := StrToTime(AValue);
      else
        Result := IXMLNode(Handle).Attributes[GetValidXMLName(Attribute.Name)];
    end
  end;
end;

{{
        
IXMLNode(Handle),   Relation
}
function TXMLObjectReader.GetRelationCount(Handle: TObjectHandle; Relation:
  TModelRelation): Integer;
var
  RelationNode: IXMLNode;
begin
  {     ,      }
  RelationNode := FindNode(IXMLNode(Handle), GetValidXMLName(Relation.Name));
  if Assigned(RelationNode) then
    Result := RelationNode.ChildNodes.Count
  else
    Result := 0;
end;

function TXMLObjectReader.ReadRelation(Handle: TObjectHandle; Relation:
  TModelRelation; Index: Integer): TObjectHandle;
var
  RelationNode: IXMLNode;
begin
  {     ,      }
  RelationNode := FindNode(IXMLNode(Handle), GetValidXMLName(Relation.Name));
  if Assigned(RelationNode) then
    Result := TObjectHandle(RelationNode.ChildNodes[Index])
  else
    Result := nil;
end;

function TXMLObjectReader.ReadChildObject(Handle: TObjectHandle; Relation:
  TModelRelation; Index: Integer): TObjectHandle;
var
  RelationNode: IXMLNode;
begin
  {     ,      }
  RelationNode := FindNode(IXMLNode(Handle), GetValidXMLName(Relation.Name));
  if Assigned(RelationNode) then
    Result := TObjectHandle(RelationNode.ChildNodes[Index])
  else
    Result := nil;
end;

function TXMLObjectReader.FindNode(ParentNode: IXMLNode; NodeName: String;
  NodeType: String = ''): IXMLNode;
var
  I: Integer;
begin
  with ParentNode.ChildNodes do
    for I := 0 to Count - 1 do
    if (Nodes[I].LocalName = NodeName) and ((NodeType = '') or (Nodes[I].Attributes['type'] = NodeType)) then
  begin
    Result := Nodes[I];
    Exit;
  end;
  Result := nil;
end;

procedure TXMLObjectReader.GetObject(Handle: TObjectHandle; var
  AObjectClassName: String; var AObjectId: TOID);
begin
  with IXMLNode(Handle) do
  begin
    AObjectClassName := GetOriginalName(NodeName);
    AObjectId := Attributes[ObjectIdAttributeName];
  end;
end;

{ TovXMLObjectStorage }

procedure TovXMLObjectStorage.Initialize(ReadMode: Boolean);

  {{
    XML-,      XML
  }
  procedure AddXMLNamespaces;
  var
    I: Integer;
  begin
    ModelNeeded;
    {   ,     }
    with Model.DomainModel do
      for I := 0 to NameSpaces.Count - 1 do
        if NameSpaces[I] <> '' then
          { TODO :     URI (   ) }
          XMLDocument.DocumentElement.Attributes['xmlns:'+GetValidXMLName(NameSpaces[I])] := NameSpaces[I];
  end;

begin
  if Not ReadMode then
  begin
    Clear;
    AddXMLNamespaces;
  end;
  inherited;
end;

procedure TovXMLObjectStorage.Clear;
var
  AObjectReader: IovObjectReader;
  AObjectWriter: IovObjectWriter;
begin
  FXMLDocument := NewXMLDocument;
  FXMLDocument.Encoding := 'UTF-8';
  FXMLDocument.Options := FXMLDocument.Options + [doAttrNull];
  FXMLDocument.AddChild(RootElementTagName);

  { Reader  Writer   , ..   
    XMLDocument,   }
  GetReaderWriter(AObjectReader, AObjectWriter);
  ObjectReader := AObjectReader;
  ObjectWriter := AObjectWriter;
end;

procedure TovXMLObjectStorage.GetReaderWriter(var AReader: IovObjectReader; var
  AWriter: IovObjectWriter);
begin
  FXMLObjectReader := TXMLObjectReader.Create(XMLDocument);
  AReader := FXMLObjectReader;

  FXMLObjectWriter := TXMLObjectWriter.Create(XMLDocument);
  FXMLObjectWriter.OnWriteObject := OnWriteObjectToXML;
  FXMLObjectWriter.OnWriteReference := OnWriteReferenceToXML;
  AWriter := FXMLObjectWriter;
end;

procedure TovXMLObjectStorage.SaveToStream(AStream: TStream);
begin
  XMLDocument.SaveToStream(AStream);
end;

procedure TovXMLObjectStorage.LoadFromStream(AStream: TStream);
begin
  Clear; {  XMLDocument }
  XMLDocument.LoadFromStream(AStream);
end;

procedure TovXMLObjectStorage.SaveToFile(FileName: String);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(Stream)
  finally
    Stream.Free
  end;
end;

procedure TovXMLObjectStorage.LoadFromFile(FileName: String);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead);
  try
    LoadFromStream(Stream)
  finally
    Stream.Free
  end;
end;

end.
