unit ovAttributeSelection;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ImgList, StdCtrls, Buttons, ExtCtrls,
  ovModel, ovModelTreeViewBuilder, ovCheckTreeView, Menus, ovComponents;

type
  {{
         ,
        , 
      .
  }
  TAttributeSelection = class
  private
    FList: TStringList;
    function GetRootItemCount: Integer;
    function GetRootItems(Index: Integer): String;
    function GetAttributeCount(RootItemIndex: Integer): Integer;
    function GetAttributes(RootItemIndex, Index: Integer): String;
  public
    function AddRootItem(ItemName: String): Integer;
    procedure AddAttribute(RootItemIndex: Integer; AttributeName: String);
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    {{
        (    ) 
      .
    }
    function GetAttributeList(RootItemIndex: Integer): String;
    function GetAttributeList2: String;
    property RootItemCount: Integer read GetRootItemCount;
    property RootItems[Index: Integer]: String read GetRootItems;
    property AttributeCount[RootItemIndex: Integer]: Integer read GetAttributeCount;
    property Attributes[RootItemIndex, Index: Integer]: String read GetAttributes;
  end;

  TAttributeSelectionDlg = class(TForm)
    ObjectTree: TovCheckTreeView;
    TreeImages: TImageList;
    BottomPanel: TPanel;
    TopPanel: TPanel;
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    cbShowAttributeTypes: TCheckBox;
    SelectAllBtn: TBitBtn;
    UnselectAllBtn: TBitBtn;
    PopupMenu: TPopupMenu;
    ToggleStateItem: TMenuItem;
    TreeViewBuilder: TovModelTreeViewBuilder;
    cbShowRelationTypes: TCheckBox;
    procedure ObjectTreeStateChange(Sender: TObject; Node: TTreeNode;
      NewState: TCheckState);
    procedure OptionsChanged(Sender: TObject);
    procedure SelectAllBtnClick(Sender: TObject);
    procedure ToggleStateItemClick(Sender: TObject);
    procedure ObjectTreeDblClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FChangeStates: Boolean;
    procedure SetChildrenStates(ParentNode: TTreeNode; NewState: TCheckState);
    procedure SetParentStates(Node: TTreeNode; NewState: TCheckState);
    procedure GetSelection(Selection: TAttributeSelection);
    function GetSelectedAttributes(Node: TTreeNode; RootItemIndex: Integer; 
      Attributes: TAttributeSelection; CurrPath: String = ''): Integer;
    procedure SelectAll(SetChecked: Boolean);
  protected
  public
    { Public declarations }
    procedure RefreshObjectTree; dynamic;
  end;

function SelectModelObjects(AModel: TovModel; ASelection: TAttributeSelection):
  Boolean;
function SelectObjectAttributes(AModel: TovModel; AModelObject: TModelObject;
  ASelection: TAttributeSelection; ShowRelations: Boolean = True): Boolean;

implementation

uses ovUtils;

{$R *.DFM}

function SelectModelObjects(AModel: TovModel; ASelection: TAttributeSelection):
  Boolean;
begin
  with TAttributeSelectionDlg.Create(Application) do
  try
    TreeViewBuilder.Model := AModel;
    Result := ShowModal = mrOk;
    if Result then GetSelection(ASelection)
  finally
    Free
  end;
end;

function SelectObjectAttributes(AModel: TovModel; AModelObject: TModelObject;
  ASelection: TAttributeSelection; ShowRelations: Boolean = True): Boolean;
var
  RootObject: TRootObject;
begin
  with TAttributeSelectionDlg.Create(Application) do
  try
    TreeViewBuilder.Model := AModel;
    RootObject.RootObjectType := rtObject;
    RootObject.RootObject := AModelObject;
    TreeViewBuilder.RootObject := RootObject;
    TreeViewBuilder.Options := TreeViewBuilder.Options - [tvbShowRoot];
    if Not ShowRelations then
      TreeViewBuilder.Options := TreeViewBuilder.Options - [tvbShowRelations];
    Result := ShowModal = mrOk;
    if Result then GetSelection(ASelection)
  finally
    Free
  end;
end;

{ TAttributeSelection }

constructor TAttributeSelection.Create;
begin
  inherited;
  FList := TStringList.Create;
end;

destructor TAttributeSelection.Destroy;
begin
  Clear;
  FList.Free;
  inherited;
end;

procedure TAttributeSelection.Clear;
var
  I: Integer;
begin
  for I := FList.Count - 1 downto 0 do FList.Objects[I].Free;
  FList.Clear;
end;

function TAttributeSelection.GetRootItemCount: Integer;
begin
  Result := FList.Count
end;

function TAttributeSelection.GetRootItems(Index: Integer): String;
begin
  Result := FList[Index];
end;

function TAttributeSelection.GetAttributeCount(RootItemIndex: Integer): Integer;
begin
  Result := TStrings(FList.Objects[RootItemIndex]).Count
end;

function TAttributeSelection.GetAttributes(RootItemIndex, Index: Integer):
  String;
begin
  Result := TStrings(FList.Objects[RootItemIndex])[Index]
end;

function TAttributeSelection.AddRootItem(ItemName: String): Integer;
begin
  Result := FList.AddObject(ItemName, TStringList.Create);
end;

procedure TAttributeSelection.AddAttribute(RootItemIndex: Integer;
  AttributeName: String);
begin
  TStrings(FList.Objects[RootItemIndex]).Add(AttributeName)
end;

function TAttributeSelection.GetAttributeList(RootItemIndex: Integer): String;
var
  I: Integer;
begin
  Result := '';
  for I := 0 to AttributeCount[RootItemIndex] - 1 do
    Result := Result + Format(', [%s]', [Attributes[RootItemIndex, I]]);
  Delete(Result, 1, 2);
end;

function TAttributeSelection.GetAttributeList2: String;
var
  I, J: Integer;
  RootItemName: String;
begin
  Result := '';
  for I := 0 to RootItemCount - 1 do
  begin
    RootItemName := RootItems[I];
    if AttributeCount[I] > 0 then
      // RootItemName -   
      for J := 0 to AttributeCount[I] - 1 do
        Result := Result + ', ' +
          Format('[%s.%s]', [RootItemName, Attributes[I, J]])
    else
      // RootItemName -   
      Result := Result + ', ' + Format('[%s]', [RootItemName])
  end;
  Delete(Result, 1, 2);
end;

{ TAttributeSelectionDlg }

procedure TAttributeSelectionDlg.FormShow(Sender: TObject);
begin
  RefreshObjectTree;
end;

// --------------------------   ------------------------------

procedure TAttributeSelectionDlg.RefreshObjectTree;
begin
  TreeViewBuilder.BuildTreeView;
  if ObjectTree.Items.Count > 0 then
    ObjectTree.Items[0].Selected := True;
end;

procedure TAttributeSelectionDlg.OptionsChanged(Sender: TObject);
begin
  if cbShowAttributeTypes.Checked then
    TreeViewBuilder.Options := TreeViewBuilder.Options + [tvbShowAttributeTypes]
  else
    TreeViewBuilder.Options := TreeViewBuilder.Options - [tvbShowAttributeTypes];

  if cbShowRelationTypes.Checked then
    TreeViewBuilder.Options := TreeViewBuilder.Options + [tvbShowRelationTypes]
  else
    TreeViewBuilder.Options := TreeViewBuilder.Options - [tvbShowRelationTypes];

  RefreshObjectTree
end;

// --------------------------    ---------------------------

procedure TAttributeSelectionDlg.SelectAll(SetChecked: Boolean);
var
  Node: TTreeNode;
begin
  with ObjectTree do begin
    Node := Items.GetFirstNode;
    while Node <> nil do
    begin
      if SetChecked
        then ItemState[Node.AbsoluteIndex] := csChecked
        else ItemState[Node.AbsoluteIndex] := csUnChecked;
      Node := Node.GetNextSibling;
    end;
  end
end;

procedure TAttributeSelectionDlg.SelectAllBtnClick(Sender: TObject);
begin
  SelectAll(Sender = SelectAllBtn);
end;

procedure TAttributeSelectionDlg.ObjectTreeStateChange(Sender: TObject;
  Node: TTreeNode; NewState: TCheckState);
begin
  if Not FChangeStates then
  begin
    FChangeStates := True;
    try
      SetChildrenStates(Node, NewState);
      if NewState = csChecked then SetParentStates(Node, NewState);
    finally
      FChangeStates := False;
    end;
  end;
end;

procedure TAttributeSelectionDlg.SetChildrenStates(ParentNode: TTreeNode;
  NewState: TCheckState);
var
  Node: TTreeNode;
begin
  Node := ParentNode.GetFirstChild;
  while Node <> nil do
  begin
    ObjectTree.ItemState[Node.AbsoluteIndex] := NewState;
    if Node.HasChildren then SetChildrenStates(Node, NewState);
    Node := Node.GetNextSibling;
  end;
end;

procedure TAttributeSelectionDlg.SetParentStates(Node: TTreeNode; NewState:
  TCheckState);
begin
  while Node.Parent <> nil do
  begin
    Node := Node.Parent;
    ObjectTree.ItemState[Node.AbsoluteIndex] := NewState;
  end;
end;

{{
       
}
procedure TAttributeSelectionDlg.ToggleStateItemClick(
  Sender: TObject);
var
  NewState: TCheckState;
  Node: TTreeNode;
begin
  Node := ObjectTree.Selected;
  if Node <> nil then begin
    if ObjectTree.ItemState[Node.AbsoluteIndex] = csUnchecked
      then NewState := csChecked
      else NewState := csUnchecked;

    FChangeStates := True;
    try
      ObjectTree.ItemState[Node.AbsoluteIndex] := NewState;
      if NewState = csUnchecked then SetChildrenStates(Node, NewState)
      else SetParentStates(Node, NewState);
    finally
      FChangeStates := False;
    end;
  end
end;

procedure TAttributeSelectionDlg.ObjectTreeDblClick(Sender: TObject);
begin
  ToggleStateItemClick(nil);
end;

// ---------------------    ---------------------------

procedure TAttributeSelectionDlg.GetSelection(Selection: TAttributeSelection);
var
  I: Integer;
  RootNode, ClassNode: TTreeNode;
begin
  Selection.Clear;
  RootNode := ObjectTree.Items.GetFirstNode;
  with ObjectTree do
    while RootNode <> nil do
    begin
      if TreeViewBuilder.NodeTypes[RootNode] = ntNameSpace then
      begin
        //    
        ClassNode := RootNode.GetFirstChild;
        while ClassNode <> nil do
        begin
          if GetNodeState(ClassNode) <> csUnChecked then
          begin
            I := Selection.AddRootItem(
              TModelObject(TreeViewBuilder.NodeObjects[ClassNode]).FullName); //     
            GetSelectedAttributes(ClassNode, I, Selection);
          end;
          ClassNode := ClassNode.GetNextSibling;
        end
      end else
        if GetNodeState(RootNode) <> csUnChecked then begin
          I := Selection.AddRootItem(GetNodeName(RootNode.Text)); //     
          GetSelectedAttributes(RootNode, I, Selection);
        end;

      RootNode := RootNode.GetNextSibling;
    end
end;

{{
  Attributes     Node, 
   .        ,
    .
}
function TAttributeSelectionDlg.GetSelectedAttributes(Node: TTreeNode; 
  RootItemIndex: Integer; Attributes: TAttributeSelection; CurrPath: String =
  ''): Integer;
var
 I, N: Integer;
 CurrName: String;
begin
  Result := 0;

  //    ,    (expanded)
  // ,      -    
  if (Node.Count = 1) and
     (TreeViewBuilder.NodeTypes[Node.Item[0]] = ntRelationNodes) then Exit;

  for I := 0 to Node.Count - 1 do
    if ObjectTree.GetNodeState(Node.Item[I]) <> csUnChecked then
    begin
      if CurrPath <> ''
        then CurrName := CurrPath + '.' + GetNodeName(Node.Item[I].Text)
        else CurrName := GetNodeName(Node.Item[I].Text);

     if TreeViewBuilder.NodeTypes[Node.Item[I]] = ntRelation
       //   -  
       then begin
         N := GetSelectedAttributes(Node.Item[I], RootItemIndex, Attributes, CurrName);
         //      ,     
         if N = 0 then begin
           Attributes.AddAttribute(RootItemIndex, CurrName); Inc(Result);
         end else
           Inc(Result, N);
       end else begin
         Attributes.AddAttribute(RootItemIndex, CurrName);
         Inc(Result);
       end
   end
end;


end.




