unit ovDBControlsDataLinksExplorer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, ComCtrls, DBGrids,
  AppEvnts, ToolWin, ImgList, ovDataLinks, ovDataLinksExplorer;

type
  TfrmDBControlsDataLinksExplorer = class(TfrmDataLinksExplorer)
    procedure FormCreate(Sender: TObject);
  private
    function FindColumnLink(AColumn: TColumn): TovDataLink;
    { Private declarations }
  protected
    function CanLink(AComponent: TComponent): Boolean; override;
    function DeleteObject(AObjectIndex: Integer): Boolean; override;
    function FindLink(AObject: TPersistent): TovDataLink; override;
    function GetChildObject(AObject: TPersistent; Index: Integer): TPersistent;
      override;
    function GetChildObjectCount(AObject: TPersistent): Integer; override;
    function GetObjectInfo(AObject: TPersistent; var ATitle, ADataObject,
      AnAttribute, ALookupDataObject, ALookupAttributes: String): Boolean; override;
  public
    function AddColumnLink(AColumn: TColumn; ADataObjectName, AttributeName:
      String): TovDataLink;
    procedure AddAttributeLink(DataObjectName: String; Attributes: TStrings; 
      ObjectIndex: Integer); override;
    { Public declarations }
  end;

var
  frmDBControlsDataLinksExplorer: TfrmDBControlsDataLinksExplorer;

implementation

uses TypInfo, ovDBControlLinks, ovDataLinksEditor;

{$R *.dfm}

procedure TfrmDBControlsDataLinksExplorer.FormCreate(Sender: TObject);
begin
  inherited;
  DataLinkControllerClass := TDBControlDataLinkController;
end;

procedure TfrmDBControlsDataLinksExplorer.AddAttributeLink(DataObjectName:
  String; Attributes: TStrings; ObjectIndex: Integer);
var
  Link: TovDataLink;
  Column: TColumn;
  I: Integer;
  AObject: TObject;
begin
  AObject := GetObjectAt(ObjectIndex);
  if AObject is TDBGrid then
  begin
    for I := 0 to Attributes.Count - 1 do
    begin
      Column := TDBGrid(AObject).Columns.Add;
      Column.Title.Caption := Attributes[I];
      AddColumnLink(Column, DataObjectName, Attributes[I]);
    end;
    ShowComponentList
  end else
  if AObject is TColumn then
  begin
    Link := GetLinkAt(ObjectIndex);
    if Link <> nil then begin
      Link.DataObject := DataObjectName;
      Link.Attribute := Attributes[0];
    end else
      AddColumnLink(TColumn(AObject), DataObjectName, Attributes[0]);
    ShowComponentList;
  end else
   inherited
end;

{{
  ,   DataSource: TDataSource
}
function TfrmDBControlsDataLinksExplorer.CanLink(AComponent: TComponent): Boolean;
var
  PropInfo: PPropInfo;
begin
  PropInfo := GetPropInfo(AComponent, 'DataSource');
  Result := (PropInfo <> nil) and (PropInfo.PropType^.Name = 'TDataSource')
end;

function TfrmDBControlsDataLinksExplorer.DeleteObject(AObjectIndex: Integer): 
  Boolean;
var
  AObject: TObject;
begin
  AObject := GetObjectAt(AObjectIndex);
  if (AObject is TColumn) and ((TColumn(AObject).Collection as TDBGridColumns).State = csDefault)
  then begin
    MessageDlg('      TDBGrid', mtWarning, [mbOk], 0);
    Result := False;
  end else begin
    AObject.Free;
    Result := True;
  end
end;

function TfrmDBControlsDataLinksExplorer.AddColumnLink(AColumn: TColumn; 
  ADataObjectName, AttributeName: String): TovDataLink;
var
  Link: TovDataLink;
begin
  Result := DataLinks.AddLink(TColumnDataLinkController, AColumn.Grid, ADataObjectName, AttributeName);
  TColumnDataLinkController(Result.Controller).Column := AColumn;

  //  Grid-      ,   
  Link := DataLinks.FindLink(AColumn.Grid, TDBControlDataLinkController);
  if Link <> nil
    then Link.DataObject := ADataObjectName
    else DataLinks.AddLink(TDBControlDataLinkController, AColumn.Grid, ADataObjectName)
end;

function TfrmDBControlsDataLinksExplorer.FindColumnLink(AColumn: TColumn): TovDataLink;
var
  I: Integer;
begin
  with DataLinks do
  for I := 0 to Links.Count - 1 do
    if (Links[I].Controller is TColumnDataLinkController) and
       (TColumnDataLinkController(Links[I].Controller).Column = AColumn) then
    begin
      Result :=	Links[I];
      Exit;
    end;
  Result := nil;
end;

function TfrmDBControlsDataLinksExplorer.FindLink(AObject: TPersistent):
  TovDataLink;
begin
  if AObject is TColumn
    then Result := FindColumnLink(TColumn(AObject))
    else Result := inherited FindLink(AObject)
end;

function TfrmDBControlsDataLinksExplorer.GetChildObject(AObject: TPersistent; Index:
  Integer): TPersistent;
begin
  if AObject is TDBGrid
    then Result :=  TDBGrid(AObject).Columns[Index]
    else Result := nil;
end;

function TfrmDBControlsDataLinksExplorer.GetChildObjectCount(AObject: TPersistent):
  Integer;
begin
  if AObject is TDBGrid
    then Result := TDBGrid(AObject).Columns.Count
    else Result := 0;
end;

function TfrmDBControlsDataLinksExplorer.GetObjectInfo(AObject: TPersistent; 
  var ATitle, ADataObject, AnAttribute, ALookupDataObject, ALookupAttributes: 
  String): Boolean;
begin
  Result := inherited GetObjectInfo(AObject,
    ATitle, ADataObject, AnAttribute, ALookupDataObject, ALookupAttributes);
  if AObject is TColumn then ATitle := TColumn(AObject).Title.Caption
end;



initialization
  RegisterDataLinkExplorer('  DB-', TfrmDBControlsDataLinksExplorer);
finalization
  UnregisterDataLinkExplorer(TfrmDBControlsDataLinksExplorer);
end.
