unit ovDataEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Contnrs,
  ImgList, ComCtrls, ToolWin, Menus, ExtCtrls, Db, StdCtrls, ActnList,
  Grids, DBGrids, DBCtrls,
  ovTypes, ovDataSet, ovVirtualDataSet, ovComponents, ovModelTreeViewBuilder,
  ovModel, ovIObjects;

type
  TfrmDataEditor = class(TForm)
    ClassTree: TTreeView;
    PageControl: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    VSplitter: TSplitter;
    ToolBar2: TToolBar;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    StatusBar: TStatusBar;
    Splitter2: TSplitter;
    ToolImages: TImageList;
    ActionList: TActionList;
    NewQueryCmd: TAction;
    RunQueryCmd: TAction;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    DataNavigator: TDBNavigator;
    ToolButton8: TToolButton;
    NextQueryCmd: TAction;
    PredQueryCmd: TAction;
    ToolButton9: TToolButton;
    LiveDataSetCmd: TAction;
    MainMenu: TMainMenu;
    N1: TMenuItem;
    N2: TMenuItem;
    NewQueryCmd1: TMenuItem;
    PredQueryCmd1: TMenuItem;
    NextQueryCmd1: TMenuItem;
    LiveDataSetCmd1: TMenuItem;
    RunQueryCmd1: TMenuItem;
    N3: TMenuItem;
    QueryNavigator: TDBNavigator;
    ToolButton10: TToolButton;
    MasterGrid: TDBGrid;
    AddObjectCmd: TAction;
    DeleteObjectCmd: TAction;
    DeleteAllObjectsCmd: TAction;
    ToolButton11: TToolButton;
    AddObjectCmd1: TMenuItem;
    DeleteObjectCmd1: TMenuItem;
    DeleteAllObjectsCmd1: TMenuItem;
    MasterLabel: TLabel;
    AddAttributesToQueryCmd: TAction;
    N4: TMenuItem;
    N5: TMenuItem;
    SaveQueryBtn: TToolButton;
    CancelQueryBtn: TToolButton;
    ToolButton14: TToolButton;
    pcQuery: TPageControl;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    mmSQLQuery: TMemo;
    MasterObjectExposer: TovExposer;
    Query: TovQuery;
    MasterDataSource: TDataSource;
    QuerySource: TDataSource;
    ImageList: TImageList;
    TreeViewBuilder: TovModelTreeViewBuilder;
    TabSheet5: TTabSheet;
    ToolBar3: TToolBar;
    ToolButton12: TToolButton;
    ToolButton13: TToolButton;
    mmData: TMemo;
    Label1: TLabel;
    LoadDataCmd: TAction;
    ImportDataCmd: TAction;
    OpenDialog: TOpenDialog;
    ToolButton15: TToolButton;
    NewDataCmd: TAction;
    QueryGrid: TDBGrid;
    mmQuery: TMemo;
    By1: TMenuItem;
    RecalcAttributesItem: TMenuItem;
    N6: TMenuItem;
    DeleteRecordsItem: TMenuItem;
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ClassTreeChange(Sender: TObject; Node: TTreeNode);
    procedure NewQueryCmdExecute(Sender: TObject);
    procedure RunQueryCmdExecute(Sender: TObject);
    procedure ClassTreeDblClick(Sender: TObject);
    procedure LiveDataSetCmdExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ActiveControlChange(Sender: TObject);
    procedure NextQueryCmdExecute(Sender: TObject);
    procedure PredQueryCmdExecute(Sender: TObject);
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure AddAttributesToQueryCmdExecute(Sender: TObject);
    procedure AddObjectCmdExecute(Sender: TObject);
    procedure DeleteObjectCmdExecute(Sender: TObject);
    procedure DeleteAllObjectsCmdExecute(Sender: TObject);
    procedure SaveQueryBtnClick(Sender: TObject);
    procedure CancelQueryBtnClick(Sender: TObject);
    procedure DeleteRecordsItemClick(Sender: TObject);
    procedure ExposerCreateObject(Sender: TObject;
      AObject: IovObject; var Cancel: Boolean);
    procedure ExposerAddObject(Sender: TObject; var AObjectId: Integer; var ACancel:
      Boolean);
    procedure LoadDataCmdExecute(Sender: TObject);
    procedure ImportDataCmdExecute(Sender: TObject);
    procedure N6Click(Sender: TObject);
    procedure RecalcAttributesItemClick(Sender: TObject);
    procedure NewDataCmdExecute(Sender: TObject);
  private
    FModel: TovModel;
    FIsRefreshing: Boolean;
    FObjects: TObjectList;
    FHistory: TStrings;
    FHistoryIndex: Integer;
    FSQLQuery: TDataSet;
    procedure SetModel(Value: TovModel);
    procedure InsertTextToQuery(AText: String);
    function GetFullPathName(Node: TTreeNode): String;
    procedure UpdateStatus;
    procedure UpdateDataGrids;
    procedure BuildGrid(Node: TTreeNode; var Exposer: TovExposer; var Grid:
      TDBGrid);
    procedure AddToHistory(Query: String);
    procedure SaveHistory(ModelName: String);
    procedure LoadHistory(ModelName: String);
    function GetActiveExposer: TovExposer;
    { Private declarations }
  protected
    property ActiveExposer: TovExposer read GetActiveExposer;
  public
    procedure RefreshClassTree; dynamic;
    function GetHistory(Next: Boolean): String;
    property Model: TovModel read FModel write SetModel;
    { Public declarations }
  end;

function ShowDataEditor(AModel: TovModel): Boolean;
function ShowQueryEditor(AQuery: TovQuery): Boolean;

procedure ImportData(AModel: TovModel; ObjectClassName: String; Attributes,
  Data: TStrings; Separator: String);

var
  frmDataEditor: TfrmDataEditor;

implementation

uses ovConsts, ovQueryManager, ovUtils, ovObjectSelection, ovObjectInspectorDlg;

{$R *.DFM}

function ShowDataEditor(AModel: TovModel): Boolean;
begin
  with TfrmDataEditor.Create(Application) do
  try
    Model := AModel;
    Model.LoadModel;
    Result := ShowModal = mrOk;
  finally
    Free
  end;
end;

function ShowQueryEditor(AQuery: TovQuery): Boolean;
begin
  with TfrmDataEditor.Create(Application) do
  try
    Model := AQuery.Model;
    Model.LoadModel;

    PageControl.ActivePageIndex := 1;
    SaveQueryBtn.Show;
    CancelQueryBtn.Show;

    mmQuery.Lines := AQuery.Query;
    Result := ShowModal = mrOk;
    if Result then AQuery.Query := mmQuery.Lines;
  finally
    Free
  end;
end;

procedure ImportData(AModel: TovModel; ObjectClassName: String; Attributes,
  Data: TStrings; Separator: String);

  procedure ImportObjectData(AObject: IovObject; ObjectData: String);
  var
    I: Integer;
  begin
    with AObject do
    begin
      BeginUpdate;
      try
        for I := 0 to Attributes.Count - 1 do
        with Properties[Attributes[I]] do
          if Not IsObject then
            //    
            AsString := ExtractSubStr(ObjectData, Separator)
          else
            //    - ,   
            AsObjectId := StrToInt(ExtractSubStr(ObjectData, Separator))
      finally
        EndUpdate
      end;
    end;
  end;

var
  I: Integer;
begin
  with AModel.ObjectSpace do
  begin
    StartTransaction;
    try
      for I := 0 to Data.Count - 1 do
        ImportObjectData(Classes[ObjectClassName].CreateObject, Data[I]);
      CommitTransaction;
    except
      on E: Exception do
      begin
        RollbackTransaction; raise;
      end;
    end;
  end;
end;

{ TDataEditorForm }

procedure TfrmDataEditor.FormCreate(Sender: TObject);
begin
  FObjects := TObjectList.Create(False);
  FHistory := TStringList.Create;
  PageControl.ActivePageIndex := 0;
end;

procedure TfrmDataEditor.FormDestroy(Sender: TObject);
begin
  try
    SaveHistory(Model.ModelName)
  except
  end;
  FreeAndNil(FSQLQuery);
  FHistory.Free;
  FObjects.Free;
end;

procedure TfrmDataEditor.FormShow(Sender: TObject);
begin
  try
    LoadHistory(Model.ModelName)
  except
  end;

  TreeViewBuilder.Model := Model;
  RefreshClassTree;

  //      
  if ClassTree.Items.Count > 1 then ClassTree.Items[1].Selected := True;
end;

procedure TfrmDataEditor.SetModel(Value: TovModel);
begin
  FModel := Value;
  Query.Model := Value;
  MasterObjectExposer.Model := Value;
end;

procedure TfrmDataEditor.ActiveControlChange(Sender: TObject);
begin
  if (PageControl.ActivePageIndex = 0) and (ActiveControl is TDBGrid)
    then DataNavigator.DataSource := TDBGrid(ActiveControl).DataSource
end;

{    --------------------------------------------------- }

procedure TfrmDataEditor.RefreshClassTree;
begin
  FIsRefreshing := True;
  try
    TreeViewBuilder.BuildTreeView;
  finally
    FIsRefreshing := False
  end;
  ClassTreeChange(ClassTree, nil)
end;

procedure TfrmDataEditor.ClassTreeChange(Sender: TObject;
  Node: TTreeNode);
begin
  if PageControl.ActivePageIndex = 0 then UpdateDataGrids
end;

procedure TfrmDataEditor.ClassTreeDblClick(Sender: TObject);
begin
  AddAttributesToQueryCmdExecute(Self)
end;

procedure TfrmDataEditor.UpdateStatus;
var
  C: Integer;
begin
  if (PageControl.ActivePageIndex = 1) and Query.Active
  then C := Query.RecordCount
  else
  if (PageControl.ActivePageIndex = 0) and (ActiveControl is TDBGrid) and
     Assigned(TDBGrid(ActiveControl).DataSource) and
     Assigned(TDBGrid(ActiveControl).DataSource.DataSet) 
    then C := TDBGrid(ActiveControl).DataSource.DataSet.RecordCount
    else
  if (PageControl.ActivePageIndex = 0) and
      Assigned(MasterGrid.DataSource) and Assigned(MasterGrid.DataSource.DataSet)
    then C := MasterGrid.DataSource.DataSet.RecordCount
  else begin StatusBar.SimpleText := ''; Exit end;

  StatusBar.SimpleText := ' : ' + IntToStr(C);
end;

{    --------------------------------------------------------- }

procedure TfrmDataEditor.NewQueryCmdExecute(Sender: TObject);
begin
  with mmQuery.Lines do
  begin
    Clear;
    Add('SELECT ');
    Add('FROM ');
    Add('WHERE ');
    Add('ORDER BY ');
  end;
  mmQuery.SetFocus;
end;

procedure TfrmDataEditor.AddAttributesToQueryCmdExecute(Sender: TObject);
var
  S: String;
begin
  if PageControl.ActivePageIndex <> 1 then Exit;

  if ClassTree.Selected <> nil then
  begin
    if TreeViewBuilder.NodeTypes[ClassTree.Selected] <> ntAttribute then
    begin
      //   
      S := Format('[%s]', [GetFullPathName(ClassTree.Selected)]);
    end
    else
    begin
      //    
      S := Format('[%s]', [GetFullPathName(ClassTree.Selected)]);
    end;

    InsertTextToQuery(S);
  end
end;

procedure TfrmDataEditor.RunQueryCmdExecute(Sender: TObject);
begin
  if pcQuery.ActivePageIndex = 0 then
  begin
    Query.Close;
    Query.Query := mmQuery.Lines;
    try
      Query.Open;
    finally
      //   SQL- ,    
      mmSQLQuery.Lines.Text := LastSQL;
    end;
    QuerySource.DataSet := Query;
    AddToHistory(mmQuery.Lines.Text);
  end
  else
  begin
    FreeAndNil(FSQLQuery);
    FSQLQuery := IDatabaseAdapter(Model.DatabaseAdapter).OpenQuery(mmSQLQuery.Lines.Text);
    QuerySource.DataSet := FSQLQuery;
  end;
end;

procedure TfrmDataEditor.InsertTextToQuery(AText: String);
begin
  with mmQuery do begin
    SelText := AText;
    SetFocus;
  end
end;

procedure TfrmDataEditor.LiveDataSetCmdExecute(Sender: TObject);
begin
  LiveDataSetCmd.Checked := Not LiveDataSetCmd.Checked;
//  Query.LiveDataSet := LiveDataSetCmd.Checked;
  QueryGrid.ReadOnly := Not LiveDataSetCmd.Checked;
end;

procedure TfrmDataEditor.DeleteRecordsItemClick(Sender: TObject);
var
  I: Integer;
begin
  if Application.MessageBox('   ?', ' ', MB_ICONQUESTION or MB_YESNO) = mrYes
  then begin
    for I := 1 to Query.RecordCount do Query.Delete;
  end
end;

{    ---------------------------------------------------------- }

procedure TfrmDataEditor.NextQueryCmdExecute(Sender: TObject);
begin
  mmQuery.Lines.Text := GetHistory(True)
end;

procedure TfrmDataEditor.PredQueryCmdExecute(Sender: TObject);
begin
  mmQuery.Lines.Text := GetHistory(False)
end;

procedure TfrmDataEditor.AddToHistory(Query: String);
begin
  if FHistory.IndexOf(Query) < 0 then begin
    FHistory.Add(Query);
    FHistoryIndex := FHistory.Count - 1;
  end
end;

function TfrmDataEditor.GetHistory(Next: Boolean): String;
begin
  Result := '';

  if Next and (FHistoryIndex < FHistory.Count) then begin
    Result := FHistory[FHistoryIndex];
    Inc(FHistoryIndex)
  end else
  if Not Next and (FHistoryIndex > 0) then begin
    Dec(FHistoryIndex);
    Result := FHistory[FHistoryIndex];
  end
end;

procedure TfrmDataEditor.ActionListUpdate(Action: TBasicAction;
  var Handled: Boolean);
begin
  PredQueryCmd.Enabled := FHistoryIndex > 0;
  NextQueryCmd.Enabled := FHistoryIndex < FHistory.Count;
  UpdateStatus;

  ImportDataCmd.Enabled := (TreeViewBuilder.SelectedClassName <> '') and (mmData.Lines.Count > 1);
  DeleteRecordsItem.Enabled := (PageControl.ActivePageIndex = 1) and (pcQuery.ActivePageIndex = 0) and
    Query.Active
end;

procedure TfrmDataEditor.SaveHistory(ModelName: String);
begin
  SaveStringsToRegistry(FHistory,
    ObjectViewRegKey+'\DataEditor\SQLHistory\' + ModelName);
end;

procedure TfrmDataEditor.LoadHistory(ModelName: String);
begin
  LoadStringsFromRegistry(FHistory,
    ObjectViewRegKey+'\DataEditor\SQLHistory\' + ModelName);
  FHistoryIndex := FHistory.Count;
end;

{    ----------------------------------------------------------- }

procedure TfrmDataEditor.UpdateDataGrids;
var
  I: Integer;
  Exposer: TovExposer;
  Grid: TDBGrid;
begin
  MasterObjectExposer.Close;

  {         Grid'     
       (EditorMode = True),     Grid'
     AV.      . }
  if MasterGrid.Visible then MasterGrid.SetFocus;

  //     
  for I := 0 to FObjects.Count - 1 do FObjects[I].Free;
  FObjects.Clear; 

  MasterGrid.Visible := ClassTree.Selected <> nil;

  if (ClassTree.Selected <> nil) and (ClassTree.Selected.Level > 0) then
  begin
    LockWindowUpdate(TabSheet1.Handle);
    try
      if TreeViewBuilder.NodeTypes[ClassTree.Selected] <> ntAttribute
        then BuildGrid(ClassTree.Selected, Exposer, Grid)
        else BuildGrid(ClassTree.Selected.Parent, Exposer, Grid);
      //   ,    
      // (   ,     
      //  ).
      Exposer.Open;
    finally
      LockWindowUpdate(0);
    end;

    MasterGrid.SetFocus;
  end;
end;

procedure TfrmDataEditor.BuildGrid(Node: TTreeNode; var Exposer: TovExposer;
  var Grid: TDBGrid);
var
  ParentExposer: TovExposer;
  ParentGrid: TDBGrid;
  Splitter: TSplitter;
  InfoLabel: TLabel;
  ADataSource: TDataSource;
begin
  if TreeViewBuilder.NodeTypes[Node.Parent] <> ntNameSpace then begin
    //    
    BuildGrid(Node.Parent, ParentExposer, ParentGrid);

    Exposer := TovExposer.Create(Self);
    FObjects.Add(Exposer);
    ADataSource := TDataSource.Create(Self);
    FObjects.Add(ADataSource);
    ADataSource.DataSet := Exposer;

    with Exposer do begin
      Model := ParentExposer.Model;
      ObjectAttributes := ParentExposer.ObjectAttributes;
      MasterDataSet := ParentExposer;
      MasterRelationName := GetNodeName(Node.Text);
      OnCreateObject := ExposerCreateObject;
      OnAddObject := ExposerAddObject;
      Options := Options + [dsoExposeObjectIdField];
    end;

    InfoLabel := TLabel.Create(Self);
    FObjects.Add(InfoLabel);
    with InfoLabel do begin
      Parent := TabSheet1;
      Align := alBottom;
      Top := ParentGrid.Top + ParentGrid.Height;
      Layout := MasterLabel.Layout;
      Font.Style := MasterLabel.Font.Style;
      Caption := ' : ' + Node.Text;
      AutoSize := False;
      Height := MasterLabel.Height;
    end;

    Splitter := TSplitter.Create(Self);
    FObjects.Add(Splitter);
    with Splitter do begin
      Parent := TabSheet1;
      Beveled := True;
      Align := alBottom;
      Top := InfoLabel.Top + InfoLabel.Height;
    end;

    Grid := TDBGrid.Create(Self);
    FObjects.Add(Grid);
    with Grid do begin
      Parent := TabSheet1;
      Align := alBottom;
      DataSource := ADataSource;
      Columns.RebuildColumns;
      Top := Splitter.Top + Splitter.Height;
      OnEnter := ActiveControlChange;
      OnExit := ActiveControlChange;
      Options := Options - [dgTabs];
    end;

  end else begin
    //    
    MasterObjectExposer.ObjectClassName :=
      (TreeViewBuilder.NodeObjects[Node] as TModelObject).FullName;

    MasterGrid.Columns.RebuildColumns;

    Exposer := MasterObjectExposer;
    Grid := MasterGrid;

    MasterLabel.Caption := ' : ' + Node.Text;
  end;
end;

procedure TfrmDataEditor.AddObjectCmdExecute(Sender: TObject);
begin
  if ActiveExposer <> nil then ActiveExposer.CreateObject
end;

procedure TfrmDataEditor.DeleteObjectCmdExecute(Sender: TObject);
begin
  if ActiveExposer <> nil then
    if MessageDlg(Format('   ?', []),
      mtConfirmation, [mbYes,mbNo], 0) = mrYes
    then ActiveExposer.Delete
end;

procedure TfrmDataEditor.DeleteAllObjectsCmdExecute(Sender: TObject);
begin
  if ActiveExposer <> nil then
    if MessageDlg(Format('   ?', []), mtConfirmation, [mbYes,mbNo], 0) = mrYes
    then ActiveExposer.DeleteAll
end;

function TfrmDataEditor.GetActiveExposer: TovExposer;
begin
  if DataNavigator.DataSource <> nil
    then Result := DataNavigator.DataSource.DataSet as TovExposer
    else Result := nil
end;

{    ----------------------------------------------------------- }

function TfrmDataEditor.GetFullPathName(Node: TTreeNode): String;
begin
  Result := Node.Text;
  while Node.Parent <> nil do
  begin
    if TreeViewBuilder.NodeTypes[Node.Parent] = ntNamespace then
    begin
      if Node.Parent.Text <> TreeViewBuilder.DefaultNameSpaceName then
        Result := Node.Parent.Text + '::' + Result
    end
    else
      Result := Node.Parent.Text + '.' + Result;
    Node := Node.Parent;
  end;
end;

procedure TfrmDataEditor.ExposerCreateObject(Sender: TObject;
  AObject: IovObject; var Cancel: Boolean);
var
  ObjectId: TOID;
  J: Integer;
  I: Integer;
begin
  //    
  with Model.DomainModel.GetObject(AObject.ClassName) do
    for I := 0 to RelationCount - 1 do
      with Relations[I] do
      if IsAggregation then
        for J := 1 to TargetMinCount do
        if SelectObject(Format('  "%s"    "%s"',
            [TargetObjectName, Name]), Self.Model,
             TargetObjectName, ObjectId)
        then AObject[Name].Add(ObjectId)
        else begin
          Cancel := True;
          Exit
        end
end;

procedure TfrmDataEditor.ExposerAddObject(Sender: TObject; var AObjectId: Integer;
  var ACancel: Boolean);
begin
  with Sender as TovExposer do
    ACancel := SelectObject(Format('  "%s"    "%s"',
              [ObjectClassName, MasterRelationName]), Self.Model,
               ObjectClassName, AObjectId)
end;

procedure TfrmDataEditor.SaveQueryBtnClick(Sender: TObject);
begin
  ModalResult := mrOk
end;

procedure TfrmDataEditor.CancelQueryBtnClick(Sender: TObject);
begin
  ModalResult := mrCancel
end;

{   -------------------------------------------------------------- }

procedure TfrmDataEditor.NewDataCmdExecute(Sender: TObject);
begin
  mmData.Lines.Clear;
end;

procedure TfrmDataEditor.LoadDataCmdExecute(Sender: TObject);
begin
  if OpenDialog.Execute then mmData.Lines.LoadFromFile(OpenDialog.FileName);
end;

procedure TfrmDataEditor.ImportDataCmdExecute(Sender: TObject);
var
  Attributes: TStrings;
  Data: TStrings;
  S: String;
begin
  Attributes := TStringList.Create;
  Data := TStringList.Create;
  try
    Data.Assign(mmData.Lines);
    S := Data[0];
    Data.Delete(0);
    while S <> '' do Attributes.Add(ExtractSubStr(S, #9));
    ImportData(Model, TreeViewBuilder.SelectedClassName, Attributes, Data, #9)
  finally
    Data.Free;
    Attributes.Free;
  end;
end;

procedure TfrmDataEditor.N6Click(Sender: TObject);
begin
  ShowObjectInspector(MasterObjectExposer.CurrentObject)
end;

procedure TfrmDataEditor.RecalcAttributesItemClick(Sender: TObject);
begin
  if (TreeViewBuilder.SelectedClassName <> '') and
     (MessageDlg(Format('    '#13+
     '   ?', []), mtConfirmation, [mbYes,mbNo], 0) = mrYes)
  then with Model.ObjectSpace.Classes[TreeViewBuilder.SelectedClassName] do
  begin
    Screen.Cursor := crHourGlass;
    try
      UpdateCalculatedAttributes;
    finally
      Screen.Cursor := crDefault;
    end;
  end;
end;

end.
