unit ovDataLinksEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ovDataObjectsEditor, Menus, ActnList, ImgList, ComCtrls, DBCtrls, ExtCtrls, Grids, DBGrids, StdCtrls,
  ToolWin, DesignIntf, ovDataLinks, ovDataLinksExplorer, ovComponents,
  ovDataObjects;

type
  TfrmDataLinksEditor = class(TfrmDataObjectsEditor)
    tsComponents: TTabSheet;
    MainMenu: TMainMenu;
    N14: TMenuItem;
    AddLinkCmd: TAction;
    DeleteLinkCmd: TAction;
    DeleteAllLinksCmd: TAction;
    N15: TMenuItem;
    N16: TMenuItem;
    N17: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure AddLinkCmdExecute(Sender: TObject);
    procedure DeleteLinkCmdExecute(Sender: TObject);
    procedure DeleteAllLinksCmdExecute(Sender: TObject);
    procedure DataObjectsTreeDragDrop(Sender, Source: TObject; X,
      Y: Integer);
    procedure DataObjectsTreeDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
  private
    FExplorer: TfrmDataLinksExplorer;
    FSaveActiveState: Boolean;
    { Private declarations }
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure AddLink(Node: TTreeNode; ObjectIndex: Integer);
  public
    procedure SelectDataObject(DataObjectName, AttributeName: String);
    property Explorer: TfrmDataLinksExplorer read FExplorer;
    { Public declarations }
  end;

procedure ShowDataLinksEditor(ADataLinks: TovDataLinks;  ExplorerClass: 
  TfrmDataLinksExplorerClass; ADesigner: IDesigner=nil);

implementation

{$R *.dfm}

procedure DockForm(AForm: TForm; AParent: TWinControl);
begin
  with AForm do
  begin
    BorderStyle := bsNone;
    Align := alClient;
    Parent := AParent;
    Show;
  end;
end;

procedure ShowDataLinksEditor(ADataLinks: TovDataLinks;  ExplorerClass: 
  TfrmDataLinksExplorerClass; ADesigner: IDesigner=nil);
var
  Editor: TfrmDataLinksEditor;
begin
  if Not Assigned(ADataLinks.DataObjects) then
    raise Exception.Create('   DataObjects');

  Editor := TfrmDataLinksEditor.Create(Application);
  with Editor do
  begin
    DataObjects := ADataLinks.DataObjects;
    FormStyle := fsStayOnTop;
    FExplorer := GetDataLinkExplorer(ADataLinks, ExplorerClass, Editor, ADesigner);
    FExplorer.DataLinksEditor := Editor;
    DockForm(FExplorer, tsComponents);
    Show;
  end;
  if ADesigner <> nil then ADesigner.Modified;
end;

{ TfrmDataLinksEditor }

procedure TfrmDataLinksEditor.FormShow(Sender: TObject);
begin
  inherited;
  FSaveActiveState := Explorer.DataLinks.Active;
  Explorer.DataLinks.Active := False;
  DataObjects.FreeNotification(Self);
end;

procedure TfrmDataLinksEditor.FormDestroy(Sender: TObject);
begin
  inherited;
  Explorer.DataLinks.Active := FSaveActiveState;
end;

procedure TfrmDataLinksEditor.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  if (Operation = opRemove) and (AComponent = DataObjects) then Release;

  inherited Notification(AComponent, Operation);
end;

procedure TfrmDataLinksEditor.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  inherited;
  Action := caFree
end;

{ ------------------------     ------------------------ }

procedure TfrmDataLinksEditor.ActionListUpdate(Action: TBasicAction;
  var Handled: Boolean);
begin
  inherited;
  AddLinkCmd.Enabled := (DataObjectsTree.Selected <> nil) and (Explorer.ComponentList.ItemIndex >= 0)
end;

procedure TfrmDataLinksEditor.AddLinkCmdExecute(Sender: TObject);
begin
  inherited;
  AddLink(DataObjectsTree.Selected, Explorer.ComponentList.ItemIndex);
end;

procedure TfrmDataLinksEditor.AddLink(Node: TTreeNode; ObjectIndex: Integer);
var
  Attributes: TStrings;
  DataObjectName: String;
begin
  with Node do
    if ImageIndex = 1 then begin
      DataObjectName := Parent.Text;
      Attributes := TStringList.Create;
      try
        Attributes.Add(Text);
        Explorer.AddAttributeLink(DataObjectName, Attributes, ObjectIndex);
      finally
        Attributes.Free;
      end;
    end else begin
      DataObjectName := Text;
      Explorer.AddDataObjectLink(DataObjectName, ObjectIndex);
    end
end;

procedure TfrmDataLinksEditor.DeleteLinkCmdExecute(Sender: TObject);
begin
  inherited;
  Explorer.DeleteSelectedLinks
end;

procedure TfrmDataLinksEditor.DeleteAllLinksCmdExecute(Sender: TObject);
begin
  inherited;
  if (MessageDlg('   ?', mtConfirmation, [mbYes, mbNo], 0) = mrYes)
    then Explorer.DeleteAllLinks;
end;

procedure TfrmDataLinksEditor.DataObjectsTreeDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
var
  Node: TTreeNode;
begin
  inherited;
  if Not Accept then begin
    Node := DataObjectsTree.GetNodeAt(X, Y);
    Accept := (Source = Explorer.ComponentList) and (Node <> nil);
  end;
end;

procedure TfrmDataLinksEditor.DataObjectsTreeDragDrop(Sender,
  Source: TObject; X, Y: Integer);
var
  Node: TTreeNode;
begin
  inherited;
  if Source = Explorer.ComponentList then
  begin
    Node := DataObjectsTree.GetNodeAt(X, Y);
    if (Node <> nil) and (Explorer.ComponentList.ItemIndex >= 0) then
      AddLink(Node, Explorer.ComponentList.ItemIndex);
  end
end;

{ ----------------------------   ------------------------ }

{{
      AttributeName    DataObjectName.
 AttributeName = '',    .
}
procedure TfrmDataLinksEditor.SelectDataObject(DataObjectName, AttributeName:
  String);

  {{
    ,   ItemName.   .
  }
  function FindDataObjectNode(DataObjectName: String): TTreeNode;
  var
    I: Integer;
  begin
    for I := 0 to DataObjectsTree.Items.Count - 1 do
      if (DataObjectsTree.Items[I].ImageIndex = 0) and
         (DataObjectsTree.Items[I].Text = DataObjectName) then
      begin
        Result := DataObjectsTree.Items[I];
        Exit;
      end;
    Result := nil;
  end;

  {{
    ,   AttributeName.  
  .
  }
  function FindAttributeNode(DataObjectNode: TTreeNode; AttributeName: String):
    TTreeNode;
  var
    I: Integer;
  begin
    for I := 0 to DataObjectNode.Count - 1 do
      if (DataObjectNode[I].ImageIndex = 1) and
         (DataObjectNode[I].Text = AttributeName) then
      begin
        Result := DataObjectNode[I];
        Exit;
      end;
    Result := nil;
  end;

var
  Node: TTreeNode;
begin
  Node := FindDataObjectNode(DataObjectName);
  if Node <> nil then begin
    if AttributeName <> '' then
      Node := FindAttributeNode(Node, AttributeName);
    if Node <> nil then Node.Selected := True;
  end;
end;



end.
