unit ovDataLinksExplorer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, ComCtrls, AppEvnts, ToolWin, ImgList,
  DesignIntf, ovDataLinks;

type
  TfrmDataLinksExplorer = class(TForm)
    ComponentList: TListView;
    ActionList: TActionList;
    DeleteComponentCmd: TAction;
    ApplicationEvents: TApplicationEvents;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    DisabledCmdImages: TImageList;
    CmdImages: TImageList;
    RefreshComponentListCmd: TAction;
    ComponentListImages: TImageList;
    procedure ComponentListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure ComponentListColumnClick(Sender: TObject;
      Column: TListColumn);
    procedure ComponentListCompare(Sender: TObject; Item1,
      Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure FormShow(Sender: TObject);
    procedure ComponentListChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure DeleteComponentCmdExecute(Sender: TObject);
    procedure RefreshComponentListCmdExecute(Sender: TObject);
    procedure ComponentListDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure ComponentListDragDrop(Sender, Source: TObject; X,
      Y: Integer);
    procedure ComponentListDblClick(Sender: TObject);
  private
    FColumnToSort: Integer;
    FDataLinks: TovDataLinks;
    FDesigner: IDesigner;
    FNeedRefreshList: Boolean;
    FDataLinksEditor: TComponent;
    FDataLinkControllerClass: TovDataLinkControllerClass;
    procedure SetDataLinks(Value: TovDataLinks);
    procedure SelectListItem(Component: TPersistent);
    procedure AddLink(DataObjectName, AttributeName: String; ObjectIndex: Integer);
    { Private declarations }
  protected
    procedure ShowComponentList(Container: TComponent = nil); dynamic;
    function GetObjectInfo(AObject: TPersistent; var ATitle, ADataObject, 
      AnAttribute, ALookupDataObject, ALookupAttributes: String): Boolean; virtual;
    {{
             AComponent
    }
    function CanLink(AComponent: TComponent): Boolean; virtual; abstract;
    function FindLink(AObject: TPersistent): TovDataLink; virtual;
    function GetChildObjectCount(AObject: TPersistent): Integer; virtual;
    function GetChildObject(AObject: TPersistent; Index: Integer): TPersistent; 
      virtual;
    procedure UpdateSelection;
    function DeleteObject(AObjectIndex: Integer): Boolean; dynamic;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function FindComponentInList(AComponent: TComponent): Integer;
    function GetObjectAt(ListIndex: Integer): TPersistent;
    function GetLinkAt(ListIndex: Integer): TovDataLink;
    function GetParentObject(ObjectIndex: Integer): Integer;
    procedure UpdateComponentListItem(AItem: TListItem; AObject: TPersistent; 
      APrefix: String);
    property Designer: IDesigner read FDesigner;
  public
    procedure AddDataObjectLink(DataObjectName: String; ObjectIndex: Integer);
      virtual;
    procedure DeleteSelectedLinks;
    procedure DeleteAllLinks;
    procedure AddAttributeLink(DataObjectName: String; Attributes: TStrings; 
      ObjectIndex: Integer); virtual;
    procedure AddLookupLink(ALookupDataObjectName, ALookupAttributes: String; 
      ObjectIndex: Integer); virtual;
    property DataLinks: TovDataLinks read FDataLinks write SetDataLinks;
    property DataLinksEditor: TComponent read FDataLinksEditor write
      FDataLinksEditor;
    property DataLinkControllerClass: TovDataLinkControllerClass read 
      FDataLinkControllerClass write FDataLinkControllerClass;
    { Public declarations }
  end;

  TfrmDataLinksExplorerClass = class of TfrmDataLinksExplorer;

procedure RegisterDataLinkExplorer(Title: String; ExplorerClass:
  TfrmDataLinksExplorerClass);
procedure UnregisterDataLinkExplorer(ExplorerClass: TfrmDataLinksExplorerClass);
procedure GetDataLinkExplorers(ExplorerClassNames: TStrings);

function GetDataLinkExplorer(ADataLinks: TovDataLinks; ExplorerClass:
  TfrmDataLinksExplorerClass; AOwner: TComponent; ADesigner: IDesigner=nil):
  TfrmDataLinksExplorer;
function ShowDataLinksExplorer(ADataLinks: TovDataLinks; ExplorerClass: 
  TfrmDataLinksExplorerClass; ADesigner: IDesigner=nil): TfrmDataLinksExplorer;

implementation

uses ovDataSet, ovDBControlLinks, ovDataObjectsEditor, ovDataLinksEditor,
  ovDataObjects;

{$R *.dfm}

var
  RegisteredExplorers: TStringList;

procedure RegisterDataLinkExplorer(Title: String; ExplorerClass:
  TfrmDataLinksExplorerClass);
begin
  RegisteredExplorers.AddObject(Title, TObject(ExplorerClass));
end;

procedure UnregisterDataLinkExplorer(ExplorerClass: TfrmDataLinksExplorerClass);
var
  I: Integer;
begin
  with RegisteredExplorers do
  for I := Count - 1 downto 0 do
    if TfrmDataLinksExplorerClass(Objects[I]) = ExplorerClass then Delete(I)
end;

procedure GetDataLinkExplorers(ExplorerClassNames: TStrings);
begin
  ExplorerClassNames.Assign(RegisteredExplorers);
end;

function GetDataLinkExplorer(ADataLinks: TovDataLinks; ExplorerClass: 
  TfrmDataLinksExplorerClass; AOwner: TComponent; ADesigner: IDesigner=nil): 
  TfrmDataLinksExplorer;
begin
  Result := ExplorerClass.Create(AOwner);
  with Result do
  begin
    FDataLinks := ADataLinks;
    FDesigner := ADesigner;
  end;
end;

function ShowDataLinksExplorer(ADataLinks: TovDataLinks; ExplorerClass: 
  TfrmDataLinksExplorerClass; ADesigner: IDesigner=nil): TfrmDataLinksExplorer;
begin
  Result := ExplorerClass.Create(Application);
  with Result do
  begin
    FDataLinks := ADataLinks;
    FDesigner := ADesigner;
    Show;
  end;
  if ADesigner <> nil then ADesigner.Modified;
end;

{ TfrmDataLinksExplorer }

procedure TfrmDataLinksExplorer.FormShow(Sender: TObject);
begin
  DataLinks.FreeNotification(Self);
  ShowComponentList;
end;

procedure TfrmDataLinksExplorer.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree
end;

procedure TfrmDataLinksExplorer.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if (Operation = opRemove) and (AComponent = DataLinks) then Release
  else
  if (Operation = opRemove) and Not (csDestroying in ComponentState) and
     (FindComponentInList(AComponent) >= 0) then FNeedRefreshList := True;

  inherited Notification(AComponent, Operation);
end;

procedure TfrmDataLinksExplorer.ComponentListKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_DELETE then begin
    DeleteComponentCmd.Execute;
    Key := 0;
  end;
end;

procedure TfrmDataLinksExplorer.ActionListUpdate(Action: TBasicAction;
  var Handled: Boolean);
begin
  DeleteComponentCmd.Enabled := ComponentList.ItemIndex >= 0;
end;

{ ------------------------     ------------- }

procedure TfrmDataLinksExplorer.ComponentListChange(Sender: TObject;
  Item: TListItem; Change: TItemChange);
begin
  UpdateSelection;
end;

{{
  ,    .
}
procedure TfrmDataLinksExplorer.ApplicationEventsIdle(Sender: TObject;
  var Done: Boolean);
var
  List: IDesignerSelections;
begin
  inherited;
  if DataLinks = nil then Exit;

  if Designer <> nil then begin
    List := TDesignerSelections.Create;
    Designer.GetSelections(List);
    if List.Count > 0 then SelectListItem(List[0])
  end;

  if FNeedRefreshList then
  begin
    ShowComponentList;
    FNeedRefreshList := False;
  end;
end;

{{
     .
}
procedure TfrmDataLinksExplorer.SelectListItem(Component: TPersistent);
var
  I: Integer;
begin
  for I := 0 to ComponentList.Items.Count - 1 do
    if GetObjectAt(I) = Component then begin
      if I <> ComponentList.ItemIndex then begin
        ComponentList.ClearSelection;
        ComponentList.ItemIndex := I;
        ComponentList.Items[I].MakeVisible(False);
        UpdateSelection;
      end;
      Exit;
    end
end;

{ ------------------------    --------------------- }

procedure TfrmDataLinksExplorer.ComponentListColumnClick(
  Sender: TObject; Column: TListColumn);
begin
  FColumnToSort := Column.Index;
  (Sender as TCustomListView).AlphaSort;
end;

procedure TfrmDataLinksExplorer.ComponentListCompare(Sender: TObject;
  Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
var
  I: Integer;
begin
  if FColumnToSort = 0 then
    Compare := CompareText(IntToStr(Item1.ImageIndex), IntToStr(Item2.ImageIndex))
  else begin
   I := FColumnToSort - 1;
   Compare := CompareText(Item1.SubItems[I], Item2.SubItems[I]);
  end;
end;

{ -----------------------    --------------------- }

procedure TfrmDataLinksExplorer.RefreshComponentListCmdExecute(
  Sender: TObject);
begin
  ShowComponentList;
end;

{{
     Container.
    ,     .
}
procedure TfrmDataLinksExplorer.ShowComponentList(Container: TComponent =
  nil);

  procedure AddVisualObject(AObject: TPersistent; Prefix: String);
  var
    I: Integer;
    Item: TListItem;
  begin
    Item := ComponentList.Items.Add;
    UpdateComponentListItem(Item, AObject, Prefix);
    for I := 0 to GetChildObjectCount(AObject) - 1 do
      AddVisualObject(GetChildObject(AObject, I), Prefix + '   ')
  end;

var
  I: Integer;
  CurrIndex: Integer;
begin
  CurrIndex := ComponentList.ItemIndex;

  ComponentList.Items.BeginUpdate;
  try
    ComponentList.Items.Clear;

    if Not Assigned(Container) then Container := DataLinks.Owner;

    with Container do
      for I := 0 to ComponentCount - 1 do
        if CanLink(Components[I]) then begin
          AddVisualObject(Components[I], '');
          Components[I].FreeNotification(Self);
        end
  finally
    ComponentList.Items.EndUpdate;
  end;

  if CurrIndex < ComponentList.Items.Count then
    ComponentList.ItemIndex := CurrIndex;
  if ComponentList.Selected <> nil then begin
    ComponentList.Selected.MakeVisible(False);
    UpdateSelection;
  end
end;

procedure TfrmDataLinksExplorer.UpdateComponentListItem(AItem: TListItem;
  AObject: TPersistent; APrefix: String);
var
  HasLink: Boolean;
  ATitle, ADataObject, AnAttribute, ALookupDataObject, ALookupAttributes: String;
begin
  HasLink := GetObjectInfo(AObject,
    ATitle, ADataObject, AnAttribute, ALookupDataObject, ALookupAttributes);
  with AItem do begin
    Data := AObject;
    SubItems.Clear;
    SubItems.Add(APrefix + ATitle);
    SubItems.Add(AObject.ClassName);
    if HasLink then begin
      SubItems.Add(ADataObject);
      SubItems.Add(AnAttribute);
      if ALookupDataObject <> '' then
        SubItems[2] := SubItems[2] + '<-' + ALookupDataObject;
      if ALookupAttributes <> '' then
        SubItems[3] := SubItems[3] + '<-' + ALookupAttributes;
      ImageIndex := 0;
    end else begin
      SubItems.Add('');
      SubItems.Add('');
      ImageIndex := -1;
    end
  end;
end;



{{
       , 
  .
}
function TfrmDataLinksExplorer.GetObjectInfo(AObject: TPersistent; var ATitle,
  ADataObject, AnAttribute, ALookupDataObject, ALookupAttributes: String):
  Boolean;
var
  Link: TovDataLink;
begin
  if AObject is TComponent
    then ATitle := TComponent(AObject).Name
  else
    ATitle := AObject.ClassName;

  Link := FindLink(AObject);
  if Link <> nil then begin
    ADataObject := Link.DataObject;
    AnAttribute := Link.Attribute;
    if Link.Controller is TDBControlDataLinkController then
    with TDBControlDataLinkController(Link.Controller) do
    begin
      ALookupDataObject := LookupDataObject;
      ALookupAttributes := LookupAttributes;
    end;
    Result := True
  end else
    Result := False
end;

{ ------------------------     ------------------------ }

procedure TfrmDataLinksExplorer.AddLink(DataObjectName, AttributeName: String;
  ObjectIndex: Integer);
var
  AObject: TPersistent;
  Link: TovDataLink;
begin
  Link := GetLinkAt(ObjectIndex);
  AObject := GetObjectAt(ObjectIndex);
  if Link <> nil then begin
    Link.DataObject := DataObjectName;
    Link.Attribute := AttributeName;
  end else
    if AObject is TComponent then
      DataLinks.AddLink(DataLinkControllerClass, TComponent(AObject), DataObjectName, AttributeName)
    else
      Exit;
  UpdateComponentListItem(ComponentList.Items[ObjectIndex], AObject, '');
end;

procedure TfrmDataLinksExplorer.AddDataObjectLink(DataObjectName: String;
  ObjectIndex: Integer);
begin
  AddLink(DataObjectName, '', ObjectIndex);
end;

procedure TfrmDataLinksExplorer.AddAttributeLink(DataObjectName: String;
  Attributes: TStrings; ObjectIndex: Integer);
begin
  AddLink(DataObjectName, Attributes[0], ObjectIndex);
end;

procedure TfrmDataLinksExplorer.AddLookupLink(ALookupDataObjectName, 
  ALookupAttributes: String; ObjectIndex: Integer);
var
  AObject: TObject;
  Link: TovDataLink;
begin
  Link := GetLinkAt(ObjectIndex);
  if Link = nil then begin
    AObject := GetObjectAt(ObjectIndex);
    if AObject is TComponent then
      Link := DataLinks.AddLink(DataLinkControllerClass, TComponent(AObject), '', '')
    else
      Exit;
  end;
  if Link.Controller is TDBControlDataLinkController then
  with TDBControlDataLinkController(Link.Controller) do
  begin
    LookupDataObject := ALookupDataObjectName;
    LookupAttributes := ALookupAttributes;
  end;
  ShowComponentList;
end;

procedure TfrmDataLinksExplorer.ComponentListDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := Assigned(DataLinksEditor) and (DataLinksEditor is TfrmDataLinksEditor)
    and (Source = TfrmDataLinksEditor(DataLinksEditor).DataObjectsTree)
end;

procedure TfrmDataLinksExplorer.ComponentListDragDrop(Sender,
  Source: TObject; X, Y: Integer);
var
  ItemIndex: Integer;
  Item: TListItem;
  I: Integer;
  DataObjectName, LookupAttributes: String;
  Attributes: TStrings;
begin
  inherited;
  if Assigned(DataLinksEditor) then
  with (DataLinksEditor as TfrmDataLinksEditor) do
  if Source = DataObjectsTree then
  begin //       
    Item := ComponentList.GetItemAt(X, Y);
    if Item = nil then ItemIndex := -1 else ItemIndex := Item.Index;

    if DataObjectsTree.Selected.ImageIndex = 0 then
    begin
      DataObjectName := DataObjectsTree.Selected.Text;
      AddDataObjectLink(DataObjectName, ItemIndex);
    end else
    begin
      DataObjectName := DataObjectsTree.Selected.Parent.Text;
      Attributes := TStringList.Create;
      try
        for I := 0 to DataObjectsTree.SelectionCount - 1 do
          Attributes.Add(DataObjectsTree.Selections[I].Text);

        //   Alt,   Lookup-
        if GetKeyState(VK_MENU) and 256 = 256 then
        begin
          //   ,   ";"
          LookupAttributes := '';
          for I := 0 to Attributes.Count - 1 do
            LookupAttributes := LookupAttributes + ';' + Attributes[I];
          Delete(LookupAttributes, 1, 1);
          AddLookupLink(DataObjectName, LookupAttributes, ItemIndex);
          with DataObjects.ObjectByName[DataObjectName] do
            Options := Options + [dsoExposeIdField];
        end else
          AddAttributeLink(DataObjectName, Attributes, ItemIndex);
      finally
        Attributes.Free;
      end;
    end
  end
end;

procedure TfrmDataLinksExplorer.DeleteSelectedLinks;
var
  Link: TovDataLink;
  I: Integer;
begin
  with ComponentList do
    for I := 0 to Items.Count - 1 do
      if Items[I].Selected then begin
        Link := GetLinkAt(I);
        if Link <> nil then Link.Free;
        //    
        with Items[I] do begin
          ImageIndex := -1;
          while SubItems.Count > 2 do SubItems.Delete(SubItems.Count - 1);
        end
      end;
end;

procedure TfrmDataLinksExplorer.DeleteAllLinks;
begin
  DataLinks.Links.Clear;
  ShowComponentList;
end;

procedure TfrmDataLinksExplorer.ComponentListDblClick(Sender: TObject);
begin
  DeleteSelectedLinks
end;

{ ----------------------------   ------------------------ }

{{
  ,    .  , 
 ,   Link.Component = AObject.
}
function TfrmDataLinksExplorer.FindLink(AObject: TPersistent):
  TovDataLink;
begin
  if AObject is TComponent then
    Result := DataLinks.FindLink(TComponent(AObject), DataLinkControllerClass)
  else
    Result := nil
end;

{{
        -1,    
.
}
function TfrmDataLinksExplorer.FindComponentInList(AComponent: TComponent):
  Integer;
var
  I: Integer;
begin
  for I := 0 to ComponentList.Items.Count - 1 do
    if GetObjectAt(I) = AComponent then begin
      Result := I;
      Exit;
    end;
  Result := -1;
end;

{{
     ,   
    
}
function TfrmDataLinksExplorer.GetChildObjectCount(AObject: TPersistent):
  Integer;
begin
  Result := 0;
end;

function TfrmDataLinksExplorer.GetChildObject(AObject: TPersistent; Index:
  Integer): TPersistent;
begin
  Result := nil;
end;

function TfrmDataLinksExplorer.GetParentObject(ObjectIndex: Integer): Integer;
var
  I: Integer;
begin
  for I := ObjectIndex - 1 downto 0 do
    if Copy(ComponentList.Items[I].SubItems[0], 1, 2) <> '  ' then
    begin
      Result := I;
      Exit;
    end;
  raise Exception.Create('   ');
end;

procedure TfrmDataLinksExplorer.UpdateSelection;
var
  Link: TovDataLink;
begin
  if ComponentList.ItemIndex >= 0 then
  begin
    Link := GetLinkAt(ComponentList.ItemIndex);

    if Assigned(DataLinksEditor) then
    with (DataLinksEditor as TfrmDataLinksEditor) do
      if Assigned(Link) and Not DataObjectsTree.Dragging then
        with Link do SelectDataObject(DataObject, Attribute);

    if Designer <> nil then
      Designer.SelectComponent(GetObjectAt(ComponentList.ItemIndex));
  end;
end;

{{
 ,       Index
}
function TfrmDataLinksExplorer.GetObjectAt(ListIndex: Integer): TPersistent;
begin
  if ListIndex < 0 then
    Result := nil
  else
    Result := TPersistent(ComponentList.Items[ListIndex].Data)
end;

{{
       Index
}
function TfrmDataLinksExplorer.GetLinkAt(ListIndex: Integer):
  TovDataLink;
begin
  Result := FindLink(GetObjectAt(ListIndex))
end;

procedure TfrmDataLinksExplorer.SetDataLinks(Value: TovDataLinks);
begin
  // TODO -cMM: TfrmDataLinksExplorer.SetFormObjects default body inserted
end;

function TfrmDataLinksExplorer.DeleteObject(AObjectIndex: Integer): Boolean;
var
  Link: TovDataLink;
begin
  Link := GetLinkAt(AObjectIndex);
  if Link <> nil then Link.Free;
  GetObjectAt(AObjectIndex).Free;
  Result := True;
end;

procedure TfrmDataLinksExplorer.DeleteComponentCmdExecute(
  Sender: TObject);
begin
  if DeleteObject(ComponentList.ItemIndex) then ShowComponentList;
end;




initialization
  RegisteredExplorers := TStringList.Create;
finalization
  RegisteredExplorers.Free;
end.
