unit ovDataObjectsEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  ImgList, ActnList, ToolWin, Menus, Grids, DBGrids, Contnrs, DBCtrls,
  ovAttributeSelection,
  ovIObjects, ovComponents, ovDataSet, ovDataObjects, ComCtrls, ExtCtrls;

type
  TfrmDataObjectsEditor = class(TForm)
    DataObjectsTree: TTreeView;
    TreeImages: TImageList;
    ActionList: TActionList;
    AddExposerCmd: TAction;
    DeleteObjectCmd: TAction;
    DeleteAlIObjectsCmd: TAction;
    AddQueryCmd: TAction;
    PageControl: TPageControl;
    VSplitter: TSplitter;
    tsProperties: TTabSheet;
    tsData: TTabSheet;
    EditItemCmd: TAction;
    MainGrid: TDBGrid;
    MainLabel: TLabel;
    TreeViewPopup: TPopupMenu;
    AddAttributeToFilterCmd: TMenuItem;
    AddAttributeToOrderCmd: TMenuItem;
    CmdImages: TImageList;
    DisabledCmdImages: TImageList;
    Panel1: TPanel;
    DataNavigator: TDBNavigator;
    mmQuery: TMemo;
    AddDetailExposerCmd: TAction;
    AddDetailQueryCmd: TAction;
    StatusBar: TStatusBar;
    N9: TMenuItem;
    N10: TMenuItem;
    N11: TMenuItem;
    N12: TMenuItem;
    N13: TMenuItem;
    PropertiesPanel: TPanel;
    Bevel5: TBevel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Bevel6: TBevel;
    Bevel7: TBevel;
    Bevel8: TBevel;
    Label10: TLabel;
    Label11: TLabel;
    Bevel9: TBevel;
    Bevel10: TBevel;
    edObjectName: TEdit;
    edAttributes: TEdit;
    cbAutoRefresh: TCheckBox;
    Label1: TLabel;
    Label2: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    edFilter: TEdit;
    edOrder: TEdit;
    edRelationName: TEdit;
    cbReadOnly: TCheckBox;
    cbExposeId: TCheckBox;
    Bevel3: TBevel;
    N24: TMenuItem;
    ToolBar: TToolBar;
    ToolButton: TToolButton;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    N1: TMenuItem;
    N2: TMenuItem;
    N3: TMenuItem;
    NewDataObjectPopup: TPopupMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    DataObjects: TovDataObjects;
    ToolButton3: TToolButton;
    SaveButton: TToolButton;
    CancelButton: TToolButton;
    Label3: TLabel;
    edMasterFieldName: TEdit;
    Bevel4: TBevel;
    Bevel11: TBevel;
    cbAutoOpen: TCheckBox;
    Label4: TLabel;
    Bevel12: TBevel;
    Label9: TLabel;
    edJoin: TEdit;
    procedure FormShow(Sender: TObject);
    procedure DataObjectsTreeChange(Sender: TObject; Node: TTreeNode);
    procedure AddExposerCmdExecute(Sender: TObject);
    procedure DeleteObjectCmdExecute(Sender: TObject);
    procedure DeleteAlIObjectsCmdExecute(Sender: TObject);
    procedure DataObjectsTreeEdited(Sender: TObject; Node: TTreeNode;
      var S: String);
    procedure AddQueryCmdExecute(Sender: TObject);
    procedure DataObjectsTreeKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure DataObjectsTreeDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure DataObjectsTreeDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure EditItemCmdExecute(Sender: TObject);
    procedure DataObjectsTreeDblClick(Sender: TObject);
    procedure PageControlChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AddAttributeToFilterCmdClick(Sender: TObject);
    procedure AddAttributeToOrderCmdClick(Sender: TObject);
    procedure GridEnter(Sender: TObject);
    procedure AddDetailExposerCmdExecute(Sender: TObject);
    procedure AddDetailQueryCmdExecute(Sender: TObject);
    procedure ControlExit(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ControlEnter(Sender: TObject);
    procedure UpdateObjectProperties;
    procedure PageControlChanging(Sender: TObject;
      var AllowChange: Boolean);
    procedure DataObjectsTreeEnter(Sender: TObject);
    procedure DataObjectsTreeExit(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
  private
    FObjects: TObjectList;
    FEditObject: TovDataObject;
    FNeedRefresh: Boolean;
    FLockUpdateProperties: Boolean;
    procedure BuildTreeView;
    procedure CheckModel;
    procedure ShowObjectData(ADataObject: TovDataObject);
    procedure UpdateItemAttributes(ItemNode: TTreeNode);
    procedure HideObjectData;
    function InputObjectName(var AObjectName: String): Boolean;
    procedure AddQuery(AParentObjectName: String);
    procedure AddExposers(AParentObjectName: String; Selection: 
      TAttributeSelection);
    procedure DataObjectsAddObject(Sender: TObject; ADataObjectName: String; var
      AObjectId: Integer; var ACancel: Boolean);
    procedure DataObjectsCreateObject(Sender: TObject; ADataObjectName: String; 
      AObject: IovObject; var Cancel: Boolean);
    procedure ClearObjectProperties; virtual;
    { Private declarations }
  protected
    procedure ShowObjectProperties(ADataObject: TovDataObject); virtual;
    procedure BuildItems(RootNode: TTreeNode; RootObject: TovDataObject); virtual;
    procedure DeleteNode(Node: TTreeNode); dynamic;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure RenameObject(ADataObject: TovDataObject; NewName: String); dynamic;
  public
    procedure RefreshTreeView;
    { Public declarations }
  end;

function ShowDataObjectsEditor(ADataObjects: TovDataObjects): Boolean;

implementation

uses ovUtils, ovTypes, ovModel, ovObjectSelection;

{$R *.DFM}

function ShowDataObjectsEditor(ADataObjects: TovDataObjects): Boolean;
begin
  with TfrmDataObjectsEditor.Create(Application) do
  begin
    try
      DataObjects.Assign(ADataObjects);
      Result := ShowModal = mrOk;
      if Result then
        ADataObjects.Assign(DataObjects);
    finally
      Free
    end;
  end;
end;

function FindNodeByText(TreeNodes: TTreeNodes; Text: String): TTreeNode;
var
  I: Integer;
begin
  for I := 0 to TreeNodes.Count - 1 do
    if TreeNodes[I].Text = Text then begin
      Result := TreeNodes[I];
      Exit;
    end;
  Result := nil;
end;

{{
    ,   
ADataObject.
}
function GetObjectClassName(ADataObjects: TovDataObjects; ADataObject: TovDataObject): String;
begin
  if Not ADataObject.IsChild
  then Result := MakeFullObjectName(ADataObject.ObjectClassName, ADataObjects.NameSpace)
  else Result := ADataObjects.Model.DomainModel.GetObjectRelation(
         GetObjectClassName(ADataObjects, ADataObject.Parent), ADataObject.MasterRelationName).TargetObjectName
end;

function MakeObjectAttributes(AObjectAttributes: String; AttributeNames: TStrings): String;
var
  I: Integer;
begin
  Result := AObjectAttributes;
  for I := 0 to AttributeNames.Count - 1 do
    Result := Result + Format(', [%s]', [AttributeNames[I]]);
  if AObjectAttributes = '' then Delete(Result, 1, 2);
end;

const
  ntDataSet = 0;
  ntQuery = 2;
  ntAttribute = 1;

{ TfrmDataObjectsEditor }

procedure TfrmDataObjectsEditor.FormCreate(Sender: TObject);
begin
  //    ,     
  FObjects := TObjectList.Create(False);
  PageControl.ActivePageIndex := 0;
end;

procedure TfrmDataObjectsEditor.FormShow(Sender: TObject);
begin
  DataObjects.OnCreateObject := DataObjectsCreateObject;
  DataObjects.OnAddObject := DataObjectsAddObject;

  RefreshTreeView;
  ShowObjectProperties(nil);
end;

procedure TfrmDataObjectsEditor.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  ControlExit(ActiveControl);
end;

procedure TfrmDataObjectsEditor.FormDestroy(Sender: TObject);
begin
  DataObjects.OnCreateObject := nil;
  DataObjects.OnAddObject := nil;

  FObjects.Free
end;

procedure TfrmDataObjectsEditor.ActionListUpdate(Action: TBasicAction;
  var Handled: Boolean);
begin
  DeleteAlIObjectsCmd.Enabled := DataObjectsTree.Items.Count > 0;
  EditItemCmd.Enabled := DataObjectsTree.Selected <> nil;
  DeleteObjectCmd.Enabled := DataObjectsTree.Selected <> nil;
  AddDetailExposerCmd.Enabled := DeleteObjectCmd.Enabled;
  AddDetailQueryCmd.Enabled := DeleteObjectCmd.Enabled;

  AddAttributeToFilterCmd.Enabled := (DataObjectsTree.Selected <> nil) and
    (DataObjectsTree.Selected.ImageIndex = ntAttribute);
  AddAttributeToOrderCmd.Enabled := AddAttributeToFilterCmd.Enabled;

  if FNeedRefresh then
  begin
    FNeedRefresh := False;
    RefreshTreeView;
  end;

end;

{{
       .
}
procedure TfrmDataObjectsEditor.DataObjectsTreeExit(Sender: TObject);
begin
  DeleteObjectCmd.ShortCut := 0;
  AddExposerCmd.ShortCut := 0;
end;

{{
 .
}
procedure TfrmDataObjectsEditor.DataObjectsTreeEnter(Sender: TObject);
begin
  DeleteObjectCmd.ShortCut := TextToShortCut('Del');
  AddExposerCmd.ShortCut := TextToShortCut('Ins');
end;


{ ============================================================================ }
{                                                     }
{ ============================================================================ }

{{
   , -   .
}
{ TODO : 
   , ..    
,     ,    . ..  
  . }
procedure TfrmDataObjectsEditor.RefreshTreeView;
var
  NewSelected: TTreeNode;
  CurrItem: String;
begin
  with DataObjectsTree do
  begin
    if Selected <> nil then
      CurrItem := Selected.Text
    else
      CurrItem := '';

    Items.BeginUpdate;
    try
      Items.Clear;
      BuildTreeView;
      FullExpand;

      if CurrItem <> '' then
      begin
        NewSelected := FindNodeByText(Items, CurrItem);
        if NewSelected <> nil then NewSelected.Selected := True;
      end;

    finally
      Items.EndUpdate;
    end;
  end;
end;

procedure TfrmDataObjectsEditor.BuildTreeView;
var
  I: Integer;
begin
  with DataObjects do
  for I := 0 to RootObjectCount - 1 do
    BuildItems(nil, RootObjects[I]);
end;

procedure TfrmDataObjectsEditor.BuildItems(RootNode: TTreeNode; RootObject:
  TovDataObject);

  procedure BuildAttributes(ParentNode: TTreeNode; Item: TovDataObject);
  var
    I: Integer;
    S: TStrings;
    Node: TTreeNode;
  begin
    S := TStringList.Create;
    try
      Item.GetSelectedAttributes(S);
      for I := 0 to S.Count - 1 do begin
        Node := DataObjectsTree.Items.AddChild(ParentNode, S[I]);
        Node.ImageIndex := ntAttribute;
        Node.SelectedIndex := ntAttribute;
      end
    finally
      S.Free;
    end;
  end;

var
  Node: TTreeNode;
  I: Integer;
begin
  Node := DataObjectsTree.Items.AddChild(RootNode, RootObject.Name);
  if RootObject.DataSetClass = TovExposer then
  begin
    Node.ImageIndex := ntDataSet;
    Node.SelectedIndex := ntDataSet;
  end else
  begin
    Node.ImageIndex := ntQuery;
    Node.SelectedIndex := ntQuery;
  end;
  BuildAttributes(Node, RootObject);
  for I := 0 to RootObject.Count - 1 do
    BuildItems(Node, RootObject.Objects[I]);
end;

{ ============================================================================ }
{                                          }
{ ============================================================================ }

{{
     ItemNode.Text  ObjectAttributes,
   ,   -
 ItemNode.
}
procedure TfrmDataObjectsEditor.UpdateItemAttributes(ItemNode: TTreeNode);
var
  S: TStrings;
  DataObject: TovDataObject;
  I: Integer;
begin
  DataObject := DataObjects.ObjectByName[ItemNode.Text];
  S := TStringList.Create;
  try
    for I := 0 to ItemNode.Count - 1 do
      if ItemNode.Item[I].ImageIndex = ntAttribute
        then S.Add(ItemNode.Item[I].Text);
    DataObject.ObjectAttributes := ListToString(S, ', ', '[%s]')
  finally
    S.Free
  end;
end;

procedure TfrmDataObjectsEditor.DataObjectsTreeChange(Sender: TObject; Node: TTreeNode);
var
  DataObject: TovDataObject;
begin
  if Node <> nil then begin
    if Node.ImageIndex = ntAttribute
      then Node := Node.Parent //   
      else ;                   //    
    DataObject := DataObjects.FindObjectByName(Node.Text);
  end else
    DataObject := nil;

  if PageControl.ActivePage = tsProperties then begin
    ControlExit(ActiveControl);
    ShowObjectProperties(DataObject);
  end else
  if PageControl.ActivePage = tsData then ShowObjectData(DataObject)
end;

procedure TfrmDataObjectsEditor.ShowObjectProperties(ADataObject:
  TovDataObject);
begin
  LockWindowUpdate(PropertiesPanel.Handle);

  if ADataObject <> nil then
  begin
    mmQuery.Enabled := ADataObject.DataSetClass = TovQuery;

    edAttributes.Enabled := ADataObject.DataSetClass = TovExposer;
    edFilter.Enabled := edAttributes.Enabled;
    edOrder.Enabled := edAttributes.Enabled;
    edObjectName.Enabled := edAttributes.Enabled and Not ADataObject.IsChild;
    edRelationName.Enabled := ADataObject.IsChild;

    mmQuery.Lines := ADataObject.Query;

    edObjectName.Text := ADataObject.ObjectClassName;
    edAttributes.Text := ADataObject.ObjectAttributes;
    edFilter.Text := ADataObject.ObjectFilter;
    edOrder.Text := ADataObject.ObjectOrder;
    edJoin.Text := ADataObject.JoinExpression;
    edRelationName.Text := ADataObject.MasterRelationName;
    edMasterFieldName.Text := ADataObject.MasterFieldName;
    cbReadOnly.Checked := ADataObject.ReadOnly;
    cbAutoOpen.Checked := ADataObject.AutoOpen;
    cbAutoRefresh.Checked := ADataObject.AutoRefresh;
    cbExposeId.Checked := dsoExposeIdField in ADataObject.Options;

    FEditObject := ADataObject;
  end else
    ClearObjectProperties;

  LockWindowUpdate(0);
end;

procedure TfrmDataObjectsEditor.ClearObjectProperties;
begin
  FEditObject := nil;
  edObjectName.Text := '';
  edAttributes.Text := '';
  edFilter.Text := '';
  edOrder.Text := '';
  edJoin.Text := '';
  edRelationName.Text := '';
  edMasterFieldName.Text := '';
  cbReadOnly.Checked := False;
  cbAutoOpen.Checked := False;
  cbAutoRefresh.Checked := False;
  cbExposeId.Checked := False;
  mmQuery.Lines.Clear;
end;

procedure TfrmDataObjectsEditor.UpdateObjectProperties;
begin
  if FEditObject <> nil then
  begin
    FEditObject.Query := mmQuery.Lines;
    FEditObject.ObjectClassName := edObjectName.Text;
    FEditObject.ObjectAttributes := edAttributes.Text;
    FEditObject.ObjectFilter := edFilter.Text;
    FEditObject.ObjectOrder := edOrder.Text;
    FEditObject.JoinExpression := edJoin.Text;
    FEditObject.MasterRelationName := edRelationName.Text;
    FEditObject.MasterFieldName := edMasterFieldName.Text;
    FEditObject.ReadOnly := cbReadOnly.Checked;
    FEditObject.AutoOpen := cbAutoOpen.Checked;
    FEditObject.AutoRefresh := cbAutoRefresh.Checked;
    if cbExposeId.Checked then
      FEditObject.Options := FEditObject.Options + [dsoExposeIdField]
    else
      FEditObject.Options := FEditObject.Options - [dsoExposeIdField];
  end;
end;

procedure TfrmDataObjectsEditor.ControlEnter(Sender: TObject);
begin
  if Sender is TCheckBox then
    TCheckBox(Sender).Color := cl3DLight else
  if Sender is TEdit then
    TEdit(Sender).Color := cl3DLight
end;

procedure TfrmDataObjectsEditor.ControlExit(Sender: TObject);
begin
  if Sender <> nil then begin
    if Sender is TCheckBox then
      TCheckBox(Sender).Color := clWindow else
    if Sender is TEdit then
      TEdit(Sender).Color := clWindow
  end;

  if Not FLockUpdateProperties then
  begin
    UpdateObjectProperties;
  
    //     ,    ObjectAttributes
    if (Sender = edAttributes) and edAttributes.Modified
    //     , ..   
    //    (    AV)
      then FNeedRefresh := True
  end;
end;

procedure TfrmDataObjectsEditor.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if ActiveControl.Parent = PropertiesPanel then
  case Key of
    VK_UP:
    begin
      Key := 0;
      SelectNext(ActiveControl, False, True);
    end;
    VK_DOWN:
    begin
      Key := 0;
      SelectNext(ActiveControl, True, True);
    end;
  end
end;

procedure TfrmDataObjectsEditor.DataObjectsTreeEdited(Sender: TObject; Node: TTreeNode;
  var S: String);
begin
  if Node.ImageIndex <> ntAttribute then
  begin
    //   
    RenameObject(DataObjects.ObjectByName[Node.Text], S);
    Node.Text := S;
  end else begin
    //  
    Node.Text := S;
    UpdateItemAttributes(Node.Parent);
  end;
  FEditObject := nil;
  DataObjectsTreeChange(DataObjectsTree, DataObjectsTree.Selected);
end;

{{
 .    .
}
procedure TfrmDataObjectsEditor.RenameObject(ADataObject: TovDataObject;
  NewName: String);
begin
  ADataObject.Name := NewName;
end;

{ ============================================================================ }
{                                                        }
{ ============================================================================ }

procedure TfrmDataObjectsEditor.CheckModel;
begin
  if DataObjects.Model = nil then raise Exception.Create('  ');
  DataObjects.Model.LoadModel;
end;

function TfrmDataObjectsEditor.InputObjectName(var AObjectName: String):
  Boolean;
begin
  Result := False;
  while True do
  begin
    if Not InputQuery('  ', '', AObjectName)
      then Exit;
    if DataObjects.FindObjectByName(AObjectName) <> nil then
      MessageDlg(Format('    "%s"  ', [AObjectName]), mtError, [mbOK], 0)
    else Break
  end;
  Result := True;
end;

procedure TfrmDataObjectsEditor.AddExposerCmdExecute(Sender: TObject);
var
  Selection: TAttributeSelection;
begin
  CheckModel;
  Selection := TAttributeSelection.Create;
  try
    if SelectModelObjects(DataObjects.Model, Selection) then
    begin
      AddExposers('', Selection);
      RefreshTreeView;
    end
  finally
    Selection.Free
  end;
end;

procedure TfrmDataObjectsEditor.AddDetailExposerCmdExecute(
  Sender: TObject);
var
  Selection: TAttributeSelection;
  AParentObjectName: String;
begin
  CheckModel;
  Selection := TAttributeSelection.Create;
  try
    with DataObjectsTree.Selected do
    if ImageIndex = ntAttribute
      then AParentObjectName := Parent.Text //  
      else AParentObjectName := Text;       //   

    if SelectObjectAttributes(DataObjects.Model,
         DataObjects.Model.DomainModel.GetObject(
         GetObjectClassName(DataObjects, DataObjects.ObjectByName[AParentObjectName])),
         Selection)
    then
    begin
      AddExposers(AParentObjectName, Selection);
      RefreshTreeView;
    end;
  finally
    Selection.Free
  end;
end;

{{
          
 ,   Selection.
}
procedure TfrmDataObjectsEditor.AddExposers(AParentObjectName: String;
  Selection: TAttributeSelection);
var
  I: Integer;
  AObjectName, AObjectClassName, AMasterRelationName: String;
begin
  for I := 0 to Selection.RootItemCount - 1 do
  begin
    //   
    AObjectName := ExtractObjectName(Selection.RootItems[I]); //    
    if Not InputObjectName(AObjectName) then Exit;
    AObjectClassName := Selection.RootItems[I];
    AMasterRelationName := Selection.RootItems[I]; //     
                                         //        
    //   
    with DataObjects.AddObject(AObjectName, AParentObjectName) do begin
      if Not IsChild
        then ObjectClassName := AObjectClassName
        else MasterRelationName := AMasterRelationName;
      ObjectAttributes := Selection.GetAttributeList(I);
    end;
  end
end;

procedure TfrmDataObjectsEditor.AddQueryCmdExecute(Sender: TObject);
begin
  AddQuery('');
end;

procedure TfrmDataObjectsEditor.AddDetailQueryCmdExecute(Sender: TObject);
var
  AParentObjectName: String;
begin
  with DataObjectsTree.Selected do
  if ImageIndex = ntAttribute
    then AParentObjectName := Parent.Text //  
    else AParentObjectName := Text;       //   
  AddQuery(AParentObjectName);
end;

procedure TfrmDataObjectsEditor.AddQuery(AParentObjectName: String);
var
  AObjectName: String;
begin
  if InputObjectName(AObjectName) then
  begin
    with DataObjects.AddObject(AObjectName, AParentObjectName) do
      DataSetClass := TovQuery;
    RefreshTreeView;
  end;
end;

{ ============================================================================ }
{                                                                              }
{ ============================================================================ }

procedure TfrmDataObjectsEditor.EditItemCmdExecute(Sender: TObject);
var
  ItemNode: TTreeNode;
  ItemName: String;
  Selection: TAttributeSelection;
begin
  CheckModel;

  Selection := TAttributeSelection.Create;
  try
    if DataObjectsTree.Selected.ImageIndex = ntAttribute
      then ItemNode := DataObjectsTree.Selected.Parent //  
      else ItemNode := DataObjectsTree.Selected;

    ItemName := ItemNode.Text;

    if SelectObjectAttributes(DataObjects.Model,
         DataObjects.Model.DomainModel.GetObject(
           GetObjectClassName(DataObjects, DataObjects.ObjectByName[ItemName])),
         Selection)
        then
    with DataObjects.ObjectByName[ItemName] do
    begin
      if ObjectAttributes <> '' then
        ObjectAttributes := ObjectAttributes + ', ' + Selection.GetAttributeList2
      else
        ObjectAttributes := Selection.GetAttributeList2;
      FEditObject := nil;
      RefreshTreeView;
    end;

  finally
    Selection.Free
  end;
end;

procedure TfrmDataObjectsEditor.DeleteObjectCmdExecute(Sender: TObject);
var
  NextNode: TTreeNode;
begin
  //        
  // (        
  //   )
  FLockUpdateProperties := True;
  try
    NextNode := DataObjectsTree.Selected;
    if NextNode.GetNextSibling = nil then
      NextNode := NextNode.GetPrevSibling
    else
      NextNode := NextNode.GetNextSibling;
    DeleteNode(DataObjectsTree.Selected);
    if NextNode <> nil then NextNode.Selected := True;
    DataObjectsTreeChange(DataObjectsTree, DataObjectsTree.Selected);
  finally
    FLockUpdateProperties := False
  end;
end;

procedure TfrmDataObjectsEditor.DeleteNode(Node: TTreeNode);
var
  ItemNode: TTreeNode;
begin
  FEditObject := nil;
  if Node.ImageIndex <> ntAttribute then
  begin
    //  
    DataObjects.DeleteObject(Node.Text);
    Node.Delete;
  end else begin
    //  
    ItemNode := Node.Parent;
    Node.Delete;
    UpdateItemAttributes(ItemNode);
  end;
end;

procedure TfrmDataObjectsEditor.DeleteAlIObjectsCmdExecute(Sender: TObject);
begin
  if MessageDlg('   ?', mtConfirmation, [mbYes,mbNo], 0) = mrYes then
  begin
    DataObjects.DeleteObjects;
    DataObjectsTree.Items.Clear;

    ShowObjectProperties(nil);
    ShowObjectData(nil);
  end
end;

procedure TfrmDataObjectsEditor.DataObjectsTreeDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var
  Node: TTreeNode;
begin
  Node := DataObjectsTree.GetNodeAt(X, Y);
  Accept := (Source = DataObjectsTree) and (Node <> nil) and (DataObjectsTree.Selected.Parent = Node.Parent)
end;

procedure TfrmDataObjectsEditor.DataObjectsTreeDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  Node: TTreeNode;
begin
  if Source = DataObjectsTree then
  begin
    Node := DataObjectsTree.GetNodeAt(X, Y);
    DataObjectsTree.Selected.MoveTo(Node, naInsert);
    if DataObjectsTree.Selected.ImageIndex <> ntAttribute
      //   
      then DataObjects.ObjectByName[DataObjectsTree.Selected.Text].Index :=
             DataObjects.ObjectByName[Node.Text].Index
      else begin
      //  
        UpdateItemAttributes(DataObjectsTree.Selected.Parent);
        DataObjectsTreeChange(DataObjectsTree, DataObjectsTree.Selected.Parent);
      end
  end;
end;

procedure TfrmDataObjectsEditor.DataObjectsTreeDblClick(Sender: TObject);
begin
  if EditItemCmd.Enabled then EditItemCmd.Execute
end;

procedure TfrmDataObjectsEditor.PageControlChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  if PageControl.ActivePage = tsProperties then ControlExit(ActiveControl);
end;

procedure TfrmDataObjectsEditor.PageControlChange(Sender: TObject);
begin
  if PageControl.ActivePage <> tsData then DataObjects.Close;
  DataObjectsTreeChange(DataObjectsTree, DataObjectsTree.Selected);
end;

procedure TfrmDataObjectsEditor.AddAttributeToFilterCmdClick(Sender: TObject);
var
  AObjectNode: TTreeNode;
begin
  AObjectNode := DataObjectsTree.Selected.Parent;
  if AObjectNode = nil then Exit;
  with DataObjects.ObjectByName[AObjectNode.Text] do
    ObjectFilter := ObjectFilter + ' [' + DataObjectsTree.Selected.Text + ']';
  FEditObject := nil;
  DataObjectsTreeChange(DataObjectsTree, DataObjectsTree.Selected);
end;

procedure TfrmDataObjectsEditor.AddAttributeToOrderCmdClick(Sender: TObject);
var
  AObjectNode: TTreeNode;
begin
  AObjectNode := DataObjectsTree.Selected.Parent;
  if AObjectNode = nil then Exit;
  with DataObjects.ObjectByName[AObjectNode.Text] do
    ObjectOrder := ObjectOrder + ' [' + DataObjectsTree.Selected.Text + ']';
  FEditObject := nil;
  DataObjectsTreeChange(DataObjectsTree, DataObjectsTree.Selected);
end;

procedure TfrmDataObjectsEditor.DataObjectsTreeKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_F2 then begin
    if DataObjectsTree.Selected <> nil then DataObjectsTree.Selected.EditText;
    Key := 0;
  end
end;

procedure TfrmDataObjectsEditor.Notification(AComponent: TComponent; Operation:
  TOperation);
begin
  if (Operation = opRemove) and (AComponent = DataObjects)
    then DataObjects := nil;
  inherited Notification(AComponent, Operation);
end;

{ ============================================================================ }
{                                                          }
{ ============================================================================ }

{{
 Grid'    ,   Node
     ,   .
}
procedure TfrmDataObjectsEditor.ShowObjectData(ADataObject: TovDataObject);
var
  InfoLabel: TLabel;
  Splitter: TSplitter;
  Grid: TDBGrid;
begin
  LockWindowUpdate(tsData.Handle);
  try
    HideObjectData;
    if ADataObject = nil then Exit;

    while ADataObject.IsChild do
    begin
      Grid := TDBGrid.Create(Self);
      FObjects.Add(Grid);
      with Grid do begin
        Parent := tsData;
        Align := alBottom;
        DataSource := DataObjects.DataSources[ADataObject.Name];
        Columns.RebuildColumns;
        if Grid = nil
          then Top := MainGrid.Top + MainGrid.Height
          else Top := Grid.Top - 1;
        Options := Options - [dgTabs];
        OnEnter := GridEnter;
        OnExit := GridEnter;
      end;

      InfoLabel := TLabel.Create(Self);
      FObjects.Add(InfoLabel);
      with InfoLabel do begin
        Parent := tsData;
        Align := alBottom;
        Layout := MainLabel.Layout;
        Font.Style := MainLabel.Font.Style;
        Color := MainLabel.Color;
        Caption := Format('  "%s"', [ADataObject.Name]);
        AutoSize := False;
        Top := Grid.Top - 1;
        Height := MainLabel.Height;
        Constraints.MaxHeight := Height;
      end;

      Splitter := TSplitter.Create(Self);
      FObjects.Add(Splitter);
      with Splitter do begin
        Parent := tsData;
        Beveled := True;
        Align := alBottom;
        Top := Grid.Top - 1;
      end;

      ADataObject := ADataObject.Parent;
    end;

    MainGrid.DataSource := DataObjects.DataSources[ADataObject.Name];
    MainGrid.Show;
    MainLabel.Caption := Format('  "%s"', [ADataObject.Name]);
    MainLabel.Show;
  finally
    LockWindowUpdate(0);
  end;
end;

procedure TfrmDataObjectsEditor.HideObjectData;
var
  I: Integer;
begin
  {         Grid'     
       (EditorMode = True),     Grid'
     AV.      . }
  if MainGrid.Visible and (PageControl.ActivePage = tsData)
    then MainGrid.SetFocus;

  //    Grid'   
  for I := 0 to FObjects.Count - 1 do FObjects[I].Free;
  FObjects.Clear;

  DataObjects.Close;

  StatusBar.SimpleText := '';
  MainLabel.Caption := '';
end;

procedure TfrmDataObjectsEditor.GridEnter(Sender: TObject);
begin
  if (PageControl.ActivePage = tsData) and (ActiveControl is TDBGrid) then
  begin
    DataNavigator.DataSource := TDBGrid(ActiveControl).DataSource;
    if Assigned(DataNavigator.DataSource) and
       Assigned(DataNavigator.DataSource.DataSet) and
       DataNavigator.DataSource.DataSet.Active
    then
      StatusBar.SimpleText := ' : ' +
        IntToStr(DataNavigator.DataSource.DataSet.RecordCount);
  end
end;

procedure TfrmDataObjectsEditor.DataObjectsAddObject(Sender: TObject;
  ADataObjectName: String; var AObjectId: Integer; var ACancel: Boolean);
begin
  with (Sender as TovDataObjects).DataSets[ADataObjectName] as TovExposer do
    ACancel := SelectObject(Format('  "%s"    "%s"',
              [ObjectClassName, MasterRelationName]), DataObjects.Model,
               ObjectClassName, AObjectId)
end;

procedure TfrmDataObjectsEditor.DataObjectsCreateObject(Sender: TObject; 
  ADataObjectName: String; AObject: IovObject; var Cancel: Boolean);
var
  ObjectId: TOID;
  J: Integer;
  I: Integer;
begin
  //    
  with DataObjects.Model.DomainModel.GetObject(AObject.ClassName) do
    for I := 0 to RelationCount - 1 do
      with Relations[I] do
      if IsAggregation then
        for J := 1 to TargetMinCount do
        if SelectObject(Format('  "%s"    "%s"',
            [TargetObjectName, Name]), DataObjects.Model,
             TargetObjectName, ObjectId)
        then AObject[Name].Add(ObjectId)
        else begin
          Cancel := True;
          Exit
        end
end;

procedure TfrmDataObjectsEditor.SaveButtonClick(Sender: TObject);
begin
  ModalResult := mrOk
end;

procedure TfrmDataObjectsEditor.CancelButtonClick(Sender: TObject);
begin
  ModalResult := mrCancel
end;

end.
