unit ovDatabaseToolsDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, Buttons, ExtCtrls, ovComponents, ovModel,
  ovModelEditorTypes, DB, ovVirtualDataSet, ovDataSet;

type
  TDatabaseOperation = procedure(AModelObject: TModelObject) of object;

  TfmDatabaseToolsDialog = class(TForm)
    CancelBtn: TBitBtn;
    OkBtn: TBitBtn;
    GroupBox1: TGroupBox;
    rbSelectedClass: TRadioButton;
    rbSelectedNamespace: TRadioButton;
    rbWholeModel: TRadioButton;
    GroupBox2: TGroupBox;
    cbCheckClassIds: TCheckBox;
    cbCheckRelationIds: TCheckBox;
    cbCheckReferences: TCheckBox;
    cbValidateObjects: TCheckBox;
    cbGenerateTables: TCheckBox;
    cbUpdateAttributes: TCheckBox;
    Bevel1: TBevel;
    Query: TovQuery;
    cbCheckGenerators: TCheckBox;
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FModel: TovModel;
    FMessageLogger: IMessageLogger;
    FSelectedModelObject: TModelObject;
    FSelectedNamespace: String;
    FCheckedTables: TStringList;
    procedure CheckClassIds(Scope: Integer);
    procedure CheckRelationIds(Scope: Integer);
    procedure CheckReferences(Scope: Integer); overload;
    procedure ValidateObjects(Scope: Integer); overload;
    procedure CheckClassId(AModelObject: TModelObject);
    procedure InspectInvalidClassIdMsg(ModelObject: TModelElement; Data: TObject);
    procedure FixInvalidClassId(ModelObject: TModelElement; Data: TObject);
    procedure GenerateClassTables(Scope: Integer); overload;
    procedure UpdateCalculatedAttributes(Scope: Integer); overload;
    procedure UpdateCalculatedAttributes(AModelObject: TModelObject); overload;
    function GetOperationScope: Integer;
    procedure GenerateClassTables(AModelObject: TModelObject); overload;
    procedure ExecOperation(OperationScope: Integer; Operation: TDatabaseOperation;
      ProgressCaption, SuccessText, ErrorText: String); overload;
    procedure ValidateObjects(AModelObject: TModelObject); overload;
    procedure InspectInvalidObject(ModelObject: TModelElement; Data: TObject);
    procedure CheckReferences(AModelObject: TModelObject); overload;
    procedure FixInvalidReference(ModelObject: TModelElement; Data: TObject);
    procedure InspectInvalidReferenceMsg(ModelObject: TModelElement; Data: TObject);
    procedure CheckGenerators(Scope: Integer); overload;
    procedure CheckGenerators(AModelObject: TModelObject); overload;
    procedure FixInvalidGenerator(ModelObject: TModelElement; Data: TObject);
    procedure InspectInvalidGeneratorMsg(ModelObject: TModelElement; Data: TObject);
    function GetMaxValue(TableName, IdFieldName: String): Integer;
    procedure CheckRelationTableRelationId(AModelRelation: TModelRelation);
    procedure CheckClassTableRelationId(AModelObject: TModelObject);
  protected
    property Model: TovModel read FModel;
    property MessageLogger: IMessageLogger read FMessageLogger;
    property SelectedModelObject: TModelObject read FSelectedModelObject;
    property OperationScope: Integer read GetOperationScope;
    property SelectedNamespace: String read FSelectedNamespace;
  public
    { Public declarations }
    procedure Execute;
  end;

function ShowVerifyDatabaseDialog(AModel: TovModel; ASelectedModelObject:
  TModelObject; ASelectedNamespace: String; AMessageLogger: IMessageLogger):
  Boolean;
function ShowGenerateDatabaseDialog(AModel: TovModel; ASelectedModelObject:
  TModelObject; ASelectedNamespace: String; AMessageLogger: IMessageLogger):
  Boolean;

implementation

uses ovProgress, ovConsts, ovDataSetExplorer, ovObjectInspectorDlg, ovIObjects,
  ovRelations;

{$R *.dfm}

function ShowVerifyDatabaseDialog(AModel: TovModel; ASelectedModelObject:
  TModelObject; ASelectedNamespace: String; AMessageLogger: IMessageLogger):
  Boolean;
begin
  with TfmDatabaseToolsDialog.Create(Application) do
  try
//    cbCheckClassIds.Checked := True;
//    cbCheckRelationIds.Checked := True;
//    cbCheckReferences.Checked := True;
//    cbValidateObjects.Checked := True;

    FModel := AModel;
    FSelectedModelObject := ASelectedModelObject;
    FSelectedNamespace := ASelectedNamespace;
    FMessageLogger := AMessageLogger;
    Result := ShowModal = mrOk;
    if Result then Execute;
  finally
    Free
  end;
end;

function ShowGenerateDatabaseDialog(AModel: TovModel; ASelectedModelObject:
  TModelObject; ASelectedNamespace: String; AMessageLogger: IMessageLogger):
  Boolean;
begin
  with TfmDatabaseToolsDialog.Create(Application) do
  try
    cbGenerateTables.Checked := True;

    FModel := AModel;
    FSelectedModelObject := ASelectedModelObject;
    FSelectedNamespace := ASelectedNamespace;
    FMessageLogger := AMessageLogger;
    Result := ShowModal = mrOk;
    if Result then Execute;
  finally
    Free
  end;
end;

const
  // Operation scope
  osSelectedClass     = 0;
  osSelectedNamespace = 1;
  osWholeModel        = 2;

type
  TObjectIdData = class
  private
    FObjectId: Integer;
  public
    constructor Create(AObjectId: Integer);
    property ObjectId: Integer read FObjectId;
  end;

  TInvalidGeneratorData = class
  private
    FModelElement: TModelElement;
  public
    constructor Create(AModelElement: TModelElement);
    property ModelElement: TModelElement read FModelElement;
  end;

  TInvalidReferenceData = class(TObjectIdData)
  private
    FRelation: TModelRelation;
    FTargetObjectId: Integer;
  public
    constructor Create(AObjectId: Integer; ARelation: TModelRelation;
      ATargetObjectId: Integer);
    property Relation: TModelRelation read FRelation;
    property TargetObjectId: Integer read FTargetObjectId;
  end;

{ TObjectIdData }

constructor TObjectIdData.Create(AObjectId: Integer);
begin
  inherited Create;
  FObjectId := AObjectId;
end;

{ TInvalidGeneratorData }

constructor TInvalidGeneratorData.Create(AModelElement: TModelElement);
begin
  inherited Create;
  FModelElement := AModelElement;
end;

{ TInvalidReferenceData }

constructor TInvalidReferenceData.Create(AObjectId: Integer; ARelation:
  TModelRelation; ATargetObjectId: Integer);
begin
  inherited Create(AObjectId);
  FRelation := ARelation;
  FTargetObjectId := ATargetObjectId;
end;

{ TfmDatabaseToolsDialog }

procedure TfmDatabaseToolsDialog.FormCreate(Sender: TObject);
begin
  FCheckedTables := TStringList.Create();
end;

procedure TfmDatabaseToolsDialog.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FCheckedTables);
end;

procedure TfmDatabaseToolsDialog.FormShow(Sender: TObject);
begin
  rbSelectedClass.Enabled := Assigned(SelectedModelObject);
  if Assigned(SelectedModelObject) then
    rbSelectedClass.Caption := rbSelectedClass.Caption + ' (' + SelectedModelObject.Name + ')';
  rbSelectedNamespace.Enabled := SelectedNamespace <> '';
  if SelectedNamespace <> '' then
    rbSelectedNamespace.Caption := rbSelectedNamespace.Caption + ' (' + SelectedNamespace + ')';
end;

procedure TfmDatabaseToolsDialog.Execute;
begin
  if cbCheckClassIds.Checked then CheckClassIds(OperationScope);
  if cbCheckRelationIds.Checked then CheckRelationIds(OperationScope);
  if cbCheckReferences.Checked then CheckReferences(OperationScope);
  if cbCheckGenerators.Checked then CheckGenerators(OperationScope);
  if cbValidateObjects.Checked then ValidateObjects(OperationScope);

  if cbGenerateTables.Checked then GenerateClassTables(OperationScope);
  if cbUpdateAttributes.Checked then UpdateCalculatedAttributes(OperationScope);
end;

function TfmDatabaseToolsDialog.GetOperationScope: Integer;
begin
  if rbSelectedClass.Checked then Result := osSelectedClass else
  if rbSelectedNamespace.Checked then Result := osSelectedNamespace else
  if rbWholeModel.Checked then Result := osWholeModel else Result := -1
end;

procedure TfmDatabaseToolsDialog.ExecOperation(OperationScope: Integer;
  Operation: TDatabaseOperation; ProgressCaption, SuccessText, ErrorText:
  String);
var
  I: Integer;
begin
  case OperationScope of

  osSelectedClass:
    if Assigned(SelectedModelObject) then
    begin
      Screen.Cursor := crHourGlass;
      try
        Operation(SelectedModelObject);
      finally
        Screen.Cursor := crDefault;
      end;
      MessageLogger.AddMessage('', SuccessText, SelectedModelObject);
    end else ShowMessage('  ');

  osWholeModel, osSelectedNamespace:
    begin
      ShowProgress(Model.DomainModel.ObjectCount, ProgressCaption);
      try
        with Model.DomainModel do
        for I := 0 to ObjectCount - 1 do
        begin
          try
            if (OperationScope = osWholeModel) or (Objects[I].Namespace = SelectedNamespace) then
              Operation(Objects[I]);
          except
            on E: Exception do
              MessageLogger.AddMessage(ErrorText, E.Message, Objects[I]);
          end;
          if Not ProgressStep then Exit;
        end;
      finally
        HideProgress
      end;
    end;
  end
end;

{***************************************************************************** }
{                                          }
{ **************************************************************************** }

resourcestring
  SModelValidation = ' ';
  SReferenceValidation = ' ';
  SClassIdValidation = ' ClassID';
  SInvalidClassID = '  ';
  SInvalidClassIDMsg = '    (ObjectID=%d,ClassID=%d)   ' +
    '    ';
  SInvalidRelationID = '  ';
  SInvalidRelationIDMsg = '    "%s" (ID =%d)   ' +
    '    ';
  SInvalidReference = '   ';
  SInvalidReferenceMsg = ' %d     %d.   "%s"';
  SInvalidGeneralization = '    ';
  SInvalidGeneralizationMsg1 = ' %d      %d  "%s".';
  SInvalidGeneralizationMsg2 = ' %d      %d  "%s".';
  STableGeneration = ' ';

  SInvalidGenerator = '  ';
  SInvalidGeneratorMsg1 = '      >= 0 (  %d)';
  SInvalidGeneratorMsg2 = '          (  %d,  %d)';

{***************************************************************************** }
{                                        }
{ **************************************************************************** }

{{
    ,      
 ClassID,       
  .
}
procedure TfmDatabaseToolsDialog.CheckClassIds(Scope: Integer);
begin
  ExecOperation(Scope, CheckClassId,
    '    ', '  ', '  ClassId');
end;

procedure TfmDatabaseToolsDialog.CheckClassId(AModelObject: TModelObject);
var
  I: Integer;
  ClassList: TModelObjects;
  IdList: String;
  Query: String;
begin
  ClassList := TModelObjects.Create;
  try
    //    
    ClassList.Add(AModelObject);
    AModelObject.GetDerivedObjects(ClassList, True);

    //     
    IdList := '';
    for I := 0 to ClassList.Count - 1 do
      IdList := IdList + ',' +  IntToStr(ClassList[I].Id);
    Delete(IdList, 1, 1);

    //  
    Query := Format('SELECT [%s], [%s] FROM {%s} WHERE Not [%1:s] in (%3:s)',
      [ObjectIdField, ClassIdField, AModelObject.TableName, IdList]);
    with Model.OpenSQLQuery(Query) do
    try
      //     
      while Not Eof do
      begin
        MessageLogger.AddMessage(
          SInvalidClassID, Format(SInvalidClassIDMsg,
            [Fields[0].AsInteger, Fields[1].AsInteger]),
          AModelObject, FixInvalidClassId, InspectInvalidClassIdMsg, TObjectIdData.Create(Fields[0].AsInteger));
        Next;
      end;
    finally
      Free
    end;

  finally
    ClassList.Free;
  end;
end;

{{
   " "   "  "
}
procedure TfmDatabaseToolsDialog.InspectInvalidClassIdMsg(ModelObject:
  TModelElement; Data: TObject);
var
  DataSet: TDataSet;
  Query: String;
begin
  Query := Format('SELECT * FROM {%s}', [(ModelObject as TModelObject).TableName]);
  DataSet := Model.OpenSQLQuery(Query);
  try
    if Not DataSet.Locate(ObjectIdField, TObjectIdData(Data).ObjectId, []) then ShowMessage('  ');
    ExploreDataSetModal(DataSet, '   "'+TModelObject(ModelObject).FullName);
  finally
    DataSet.Free
  end;
end;

{{
   ""   "  "
}
procedure TfmDatabaseToolsDialog.FixInvalidClassId(ModelObject: TModelElement;
  Data: TObject);
var
  Query: String;
begin
  Query := Format('UPDATE {%s} SET [%s] = %d WHERE [%s] = %d',
    [TModelObject(ModelObject).TableName, ClassIdField, TModelObject(ModelObject).Id,
      ObjectIdField, TObjectIdData(Data).ObjectId]);
  Model.ExecSQLQuery(Query);
end;

{***************************************************************************** }
{                                      }
{ **************************************************************************** }

{{
 -    RelationId   ,  ..  
 -   ,    -   .

:
    :
  -   
       ,   
       )   ,       
             ,       ( )
       )    ,  ,   , 
              (  )
       )    

    :
  -    
        ,   
       )   ,         
       )    ,  ,   , 
              (  )
       )    
}
procedure TfmDatabaseToolsDialog.CheckRelationIds(Scope: Integer);
var
  I: Integer;
begin
  FCheckedTables.Clear;
  ExecOperation(Scope, CheckClassTableRelationId,
    '  ', '  ', '  RelationId  ');

  { TODO :    Scope    }
  try
    FCheckedTables.Clear;
    ShowProgress(Model.DomainModel.RelationCount, '  ');
    with Model.DomainModel do
    for I := 0 to RelationCount - 1 do
    begin
      try
        CheckRelationTableRelationId(Relations[I]);
      except
        on E: Exception do
          MessageLogger.AddMessage('  RelationId  ', E.Message, Relations[I]);
      end;
      if Not ProgressStep then Exit;
    end
  finally
    HideProgress
  end;
end;

{{
   :
    ,   
    )   ,       
          ,       ( )
    )    ,  ,   , 
           (  )
    )    
}
procedure TfmDatabaseToolsDialog.CheckClassTableRelationId(AModelObject:
  TModelObject);

  procedure CheckTable(ATableName, AIdList: String);
  var
    Query: String;
  begin
    //  
    if AIdList <> '' then
      Query := Format('SELECT [%s] FROM {%s} WHERE Not [%s] in (%s)',
        [ObjectIdField, ATableName, RelationIdField, AIdList])
    else
      Query := Format('SELECT [%s] FROM {%s} WHERE Not [%s] is Null',
        [ObjectIdField, ATableName, RelationIdField]);
    with Model.OpenSQLQuery(Query) do
    try
      //     
      while Not Eof do
      begin
        MessageLogger.AddMessage(
          SInvalidRelationID, Format(SInvalidRelationIDMsg,
            [ATableName, Fields[0].AsInteger]), AModelObject,
            nil, nil, nil);
        Next;
      end;
    finally
      Free
    end;
  end;

  {{
         ( ),
      ,    .
  }
  procedure FindRelationIdsForTable(ATableName: String; AIdList: TStrings);
  var
    I, J: Integer;
  begin
    ATableName := UpperCase(ATableName);
    with Model.DomainModel do
    for I := 0 to ObjectCount - 1 do
      if UpperCase(Objects[I].TableName) = ATableName then
      begin
        {        }
        for J := 0 to RelationCount - 1 do
          if Relations[J].IsComposition and (Relations[J].TargetObject = Objects[I])
            {      }
            then AIdList.Add(IntToStr(Relations[J].Id))
      end;
  end;

var
  AIdList: TStringList;
begin
  with AModelObject do
  if FCheckedTables.IndexOf(UpperCase(TableName)) < 0 then
  begin
    AIdList := TStringList.Create;
    try
      FindRelationIdsForTable(TableName, AIdList);
      CheckTable(TableName, AIdList.CommaText);
    finally
      AIdList.Free;
    end;
    FCheckedTables.Add(UpperCase(TableName));
  end;
end;

{{
   :
     ,   
    )   ,         
    )    ,  ,   , 
           (  )
    )    
}
procedure TfmDatabaseToolsDialog.CheckRelationTableRelationId(AModelRelation:
  TModelRelation);

  procedure CheckTable(ATableName, AIdList: String);
  var
    Query: String;
  begin
    //  
    if AIdList <> '' then
      Query := Format('SELECT [%s] FROM {%s} WHERE Not [%s] in (%s)',
        [RelationInstanceIdField, ATableName, RelationIdField, AIdList])
    else
      Query := Format('SELECT [%s] FROM {%s} WHERE Not [%s] is Null',
        [RelationInstanceIdField, ATableName, RelationIdField]);
    with Model.OpenSQLQuery(Query) do
    try
      //     
      while Not Eof do
      begin
        MessageLogger.AddMessage(
          SInvalidRelationID, Format(SInvalidRelationIDMsg,
            [ATableName, Fields[0].AsInteger]), AModelRelation,
            nil, nil, nil);
        Next;
      end;
    finally
      Free
    end;
  end;

  {{
       ,  
   .
  }
  procedure FindRelationIdsForTable(ATableName: String; AIdList: TStrings);
  var
    I: Integer;
  begin
    ATableName := UpperCase(ATableName);
    with Model.DomainModel do
    for I := 0 to RelationCount - 1 do
      with Relations[I] do
      if IsAssociation and (TargetMaxCount > 1) and (UpperCase(TableName) = ATableName)
        then AIdList.Add(IntToStr(Id))
  end;

var
  AIdList: TStrings;
begin
  with AModelRelation do
  if IsAssociation and (TargetMaxCount > 1) and (FCheckedTables.IndexOf(UpperCase(TableName)) < 0) then
  begin
    AIdList := TStringList.Create;
    try
      FindRelationIdsForTable(TableName, AIdList);
      CheckTable(TableName, AIdList.CommaText);
    finally
      AIdList.Free;
    end;
    FCheckedTables.Add(UpperCase(TableName));
  end;
end;

{***************************************************************************** }
{                                                        }
{ **************************************************************************** }

{{
    ,     
  , ..  " ".
}
procedure TfmDatabaseToolsDialog.CheckReferences(Scope: Integer);
begin
  ExecOperation(Scope, CheckReferences,
    ' ', '  ', '  ');
end;

{{
    ,     
  , ..  " ".

:
        
,   .
}
procedure TfmDatabaseToolsDialog.CheckReferences(AModelObject: TModelObject);
var
  CurrObject: IovObject;

  procedure CheckChildren;

    function CheckGeneralization(AGeneralization: TModelRelation; var ChildId: Integer): Boolean;
    var
      Query: String;
      DataSet: TDataSet;
    begin
      Query := Format('SELECT [%s] FROM {%s} WHERE [%s] = %d and [%s] = %d',
        [ChildIdField, AModelObject.TableName, ObjectIdField, CurrObject.ObjectId, ClassIdField, AGeneralization.SourceObject.Id]);
      DataSet := Model.OpenSQLQuery(Query);
      try
        if DataSet.IsEmpty then begin
          Result := True;
          Exit;
        end;
        ChildId := DataSet.Fields[0].AsInteger;
      finally
        DataSet.Free
      end;

      Query := Format('SELECT [%s] FROM {%s} WHERE [%s] = %d',
        [ObjectIdField, AGeneralization.SourceObject.TableName, ObjectIdField, ChildId]);
      DataSet := Model.OpenSQLQuery(Query);
      try
        Result := Not DataSet.IsEmpty
      finally
        DataSet.Free
      end;
    end;

  var
    I, ChildId: Integer;
  begin
    with Model.DomainModel do
    for I := 0 to RelationCount - 1 do with Relations[I] do
      if IsGeneralization and (TargetObjectName = AModelObject.FullName) then
        if Not CheckGeneralization(Relations[I], ChildId) then
          MessageLogger.AddMessage(SInvalidGeneralization, Format(SInvalidGeneralizationMsg2,
            [CurrObject.ObjectId, ChildId, SourceObjectName]), AModelObject);
  end;

var
  I, J, K: Integer;
begin
  {      }
  with Model.ObjectSpace.Classes[AModelObject.FullName].Objects do
    for I := 0 to Count - 1 do
    begin
      CurrObject := Objects[I];

      {      }
      with AModelObject do
      begin
        for J := 0 to AncestorCount - 1 do
          if Not CurrObject.GetAncestor(AncestorRelations[J]).IsExist then
            MessageLogger.AddMessage(SInvalidGeneralization, Format(SInvalidGeneralizationMsg1,
               [CurrObject.ObjectId, CurrObject.GetAncestor(AncestorRelations[J]).ObjectId, AncestorRelations[J].TargetObjectName]),
               AModelObject, FixInvalidReference, InspectInvalidReferenceMsg,
                 TInvalidReferenceData.Create(CurrObject.ObjectId, AncestorRelations[J], 0));
      end;

      {      }
      CheckChildren;

      with AModelObject do
      {     }
      for J := 0 to RelationCount - 1 do
        {     }
        try
          with CurrObject[Relations[J].Name].AsObjects do
            for K := 0 to Count - 1 do
              if Not Objects[K].IsExist then
                MessageLogger.AddMessage(SInvalidReference, Format(SInvalidReferenceMsg,
                  [CurrObject.ObjectId, Objects[K].ObjectId, Relations[J].Name]),
                  AModelObject, FixInvalidReference, InspectInvalidReferenceMsg,
                  TInvalidReferenceData.Create(CurrObject.ObjectId, Relations[J], Objects[K].ObjectId));
        except
          {      ,     , 
                    }
          on E: ovRelations.EObjectNotFound do
            MessageLogger.AddMessage(SInvalidReference, E.Message, AModelObject);
    end;
    end;
end;

{{
   " "   " "
}
procedure TfmDatabaseToolsDialog.InspectInvalidReferenceMsg(ModelObject:
  TModelElement; Data: TObject);
begin
end;

{{
   ""   " "
}
procedure TfmDatabaseToolsDialog.FixInvalidReference(ModelObject:
  TModelElement; Data: TObject);
var
  ASourceObject: IovObject;
  Query: String;
begin
  with TInvalidReferenceData(Data) do
    if Relation.IsGeneralization then
    begin
      {    (      ) }
      Query := Format('DELETE FROM {%s} WHERE [%s] = %d',
       [TModelObject(ModelObject).TableName, ObjectIdField, ObjectId]);
      Model.ExecSQLQuery(Query);
    end else
    case Application.MessageBox('   ()      () ?', ' ',
      MB_ICONQUESTION or MB_YESNOCANCEL) of
      idYes:
        begin
          {     }
          ASourceObject := Model.ObjectSpace.GetObject(TModelObject(ModelObject), ObjectId);
          ASourceObject.Delete;
        end;
      idNo:
        begin
          {      }
          ASourceObject := Model.ObjectSpace.GetObject(TModelObject(ModelObject), ObjectId);
          ASourceObject[Relation.Name].AsObjects.Remove(TargetObjectId);
        end;
    end
end;

{***************************************************************************** }
{                                      }
{ **************************************************************************** }

{{
       
.
}
procedure TfmDatabaseToolsDialog.CheckGenerators(Scope: Integer);
begin
  ExecOperation(Scope, CheckGenerators,
    ' ', '  ', '');
end;

{{
       
.

:
-           :
        -     
        -       Id
        - ,  Id > 0  Id > max (  )
}
procedure TfmDatabaseToolsDialog.CheckGenerators(AModelObject: TModelObject);

  procedure CheckTable(TableName, IdFieldName: String; ModelElement:
    TModelElement);
  var
    MaxId, NewId: Integer;
  begin
    MaxId := GetMaxValue(TableName, IdFieldName);
    NewId := Model.DatabaseAdapter.GetNewRecordId(TableName);

    if NewId < 0 then
      MessageLogger.AddMessage(SInvalidGenerator, Format(SInvalidGeneratorMsg1, [NewId]), ModelElement,
        FixInvalidGenerator, InspectInvalidGeneratorMsg, TInvalidGeneratorData.Create(ModelElement))
    else
    if NewId < MaxId then
      MessageLogger.AddMessage(SInvalidGenerator, Format(SInvalidGeneratorMsg2, [NewId, MaxId]), ModelElement,
        FixInvalidGenerator, InspectInvalidGeneratorMsg, TInvalidGeneratorData.Create(ModelElement));
  end;

var
  I: Integer;
begin
  CheckTable(AModelObject.TableName, ObjectIdField, AModelObject);
  with AModelObject do
  {     }
  for I := 0 to RelationCount - 1 do
    if Relations[I].TableName <> '' then
      CheckTable(Relations[I].TableName, RelationInstanceIdField, Relations[I]);
end;

{{
   " "   "  "
}
procedure TfmDatabaseToolsDialog.InspectInvalidGeneratorMsg(ModelObject:
  TModelElement; Data: TObject);
begin
  Application.MessageBox('    '#13+
    '      '#13+
    '   ', '', MB_ICONINFORMATION or MB_OK);
end;

{{
   ""   "  "
}
procedure TfmDatabaseToolsDialog.FixInvalidGenerator(ModelObject:
  TModelElement; Data: TObject);
var
  TableName: String;
  FieldName: String;
begin
  if TInvalidGeneratorData(Data).ModelElement is TModelObject then
  begin
    TableName := TModelObject(TInvalidGeneratorData(Data).ModelElement).TableName;
    FieldName := ObjectIdField;
  end else
  if TInvalidGeneratorData(Data).ModelElement is TModelRelation then
  begin
    TableName := TModelRelation(TInvalidGeneratorData(Data).ModelElement).TableName;
    FieldName := RelationInstanceIdField;
  end else
    raise Exception.Create('Invalid Data');

  Model.DatabaseAdapter.SetNewRecordId(TableName, GetMaxValue(TableName, FieldName));
end;

{***************************************************************************** }
{                                                     }
{ **************************************************************************** }

procedure TfmDatabaseToolsDialog.ValidateObjects(Scope: Integer);
begin
  ExecOperation(Scope, ValidateObjects,
    ' ', '  ', '');
end;

procedure TfmDatabaseToolsDialog.ValidateObjects(AModelObject: TModelObject);
var
  I, J: Integer;
  Errors: TStringList;
begin
  Errors := TStringList.Create;
  try
    with Model.ObjectSpace.Classes[AModelObject.FullName].Objects do
      for I := 0 to Count - 1 do
      begin
        Errors.Clear;
        if Not Objects[I].Validate(True, True, True, Errors) then
          for J := 0 to Errors.Count - 1 do
            MessageLogger.AddMessage('  ', Errors[J],
              Errors.Objects[J] as TModelElement, nil, InspectInvalidObject, TObjectIdData.Create(Objects[I].ObjectId));
      end;
  finally
    Errors.Free;
  end;
end;

{{
   " "      
}
procedure TfmDatabaseToolsDialog.InspectInvalidObject(ModelObject:
  TModelElement; Data: TObject);
var
  ObjectToInspect: IovObject;
  Save: TovModel;
begin
  Save := Model;
  if ModelObject is TModelObject then
    ObjectToInspect := Model.ObjectSpace.GetObject(TModelObject(ModelObject), TObjectIdData(Data).ObjectId)
  else
  if ModelObject is TModelAttribute then
    ObjectToInspect := Model.ObjectSpace.GetObject(TModelAttribute(ModelObject).Owner, TObjectIdData(Data).ObjectId)
  else
  if ModelObject is TModelRelation then
    ObjectToInspect := Model.ObjectSpace.GetObject(TModelRelation(ModelObject).SourceObject, TObjectIdData(Data).ObjectId)
  else raise Exception.Create('  ');

  ShowObjectInspector(ObjectToInspect, True);

  { TODO :         FModel  }
  FModel := Save;
end;

{***************************************************************************** }
{                                        }
{ **************************************************************************** }

procedure TfmDatabaseToolsDialog.UpdateCalculatedAttributes(Scope: Integer);
begin
  ExecOperation(Scope, UpdateCalculatedAttributes,
    '  ', '  ', '   ');
end;

procedure TfmDatabaseToolsDialog.UpdateCalculatedAttributes(AModelObject:
  TModelObject);
begin
  try
    Model.ObjectSpace.Classes[AModelObject.FullName].UpdateCalculatedAttributes;
  except
    on E: Exception do
      MessageLogger.AddMessage('   ', E.Message, AModelObject);
  end;
end;

{***************************************************************************** }
{                                                }
{ **************************************************************************** }

procedure TfmDatabaseToolsDialog.GenerateClassTables(Scope: Integer);
begin
  ExecOperation(Scope, GenerateClassTables,
    '    ', '  ', '');
end;

procedure TfmDatabaseToolsDialog.GenerateClassTables(AModelObject:
  TModelObject);
begin
  Model.Open;
  try
    Model.Database.GenerateObjectTables(AModelObject.FullName);
  except
    on E: Exception do
      MessageLogger.AddMessage('   ', E.Message, AModelObject);
  end;
end;

{***************************************************************************** }
{                                                       }
{ **************************************************************************** }

function TfmDatabaseToolsDialog.GetMaxValue(TableName, IdFieldName: String): Integer;
var
  DataSet: TDataSet;
  Query: String;
begin
  Query := Format('SELECT Max([%s]) FROM {%s}', [IdFieldName, TableName]);
  DataSet := Model.OpenSQLQuery(Query);
  try
    if Not DataSet.IsEmpty then
      Result := DataSet.Fields[0].AsInteger
    else
      Result := 0
  finally
    DataSet.Free
  end;
end;

end.
