unit ovModelEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ovModelExplorer, ImgList, ComCtrls, ExtCtrls,
  ovComponents, ovModel, ToolWin, ActnList, Menus, AppEvnts, StdCtrls,
  ovModelTreeViewBuilder, ovXMLImportExport;

type
  TModelEditorForm = class(TModelExplorerForm)
    ToolBar: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolbarImages: TImageList;
    ActionList: TActionList;
    AddClassCmd: TAction;
    ToolButton5: TToolButton;
    AddDataTypeCmd: TAction;
    DeleteAllUserTypesCmd: TAction;
    AddAttributeCmd: TAction;
    AddCompositionCmd: TAction;
    SaveAndCloseToolButton: TToolButton;
    ToolButton9: TToolButton;
    SaveAndCloseCmd: TAction;
    CancelCmd: TAction;
    SaveToFileCmd: TAction;
    ToolButton10: TToolButton;
    MainMenu: TMainMenu;
    N1: TMenuItem;
    N2: TMenuItem;
    N3: TMenuItem;
    N4: TMenuItem;
    N6: TMenuItem;
    N7: TMenuItem;
    N8: TMenuItem;
    N9: TMenuItem;
    N10: TMenuItem;
    N11: TMenuItem;
    StatusBar: TStatusBar;
    DeleteObjectCmd: TAction;
    N16: TMenuItem;
    DeleteAllClassesCmd: TAction;
    NewModelCmd: TAction;
    N21: TMenuItem;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton11: TToolButton;
    ToolButton12: TToolButton;
    ToolButton13: TToolButton;
    AddAssociationCmd: TAction;
    AddGeneralizationCmd: TAction;
    N34: TMenuItem;
    N35: TMenuItem;
    XMLSchemaSaveDialog: TSaveDialog;
    AddAggregationCmd: TAction;
    ToolButton4: TToolButton;
    ToolButton14: TToolButton;
    ToolButton15: TToolButton;
    OpenFromFileCmd: TAction;
    N14: TMenuItem;
    DisabledImages: TImageList;
    OpenFileCmd1: TMenuItem;
    AddIndexCmd: TAction;
    N5: TMenuItem;
    N31: TMenuItem;
    N30: TMenuItem;
    DeleteButton: TToolButton;
    PopupMenu: TPopupMenu;
    ExpandAllItem: TMenuItem;
    CollapseAllItem: TMenuItem;
    procedure AddClassCmdExecute(Sender: TObject);
    procedure AddDataTypeCmdExecute(Sender: TObject);
    procedure DeleteAllUserTypesCmdExecute(Sender: TObject);
    procedure AddAttributeCmdExecute(Sender: TObject);
    procedure AddCompositionCmdExecute(Sender: TObject);
    procedure SaveAndCloseCmdExecute(Sender: TObject);
    procedure CancelCmdExecute(Sender: TObject);
    procedure SaveToFileCmdExecute(Sender: TObject);
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure AddAggregationCmdExecute(Sender: TObject);
    procedure DeleteObjectCmdExecute(Sender: TObject);
    procedure NewModelCmdExecute(Sender: TObject);
    procedure DeleteAllClassesCmdExecute(Sender: TObject);
    procedure AddGeneralizationCmdExecute(Sender: TObject);
    procedure AddAssociationCmdExecute(Sender: TObject);
    procedure AddIndexCmdExecute(Sender: TObject);
    procedure ClassTreeDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure ClassTreeDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure CollapseAllItemClick(Sender: TObject);
    procedure ExpandAllItemClick(Sender: TObject);
    procedure ModelTreeViewKeyDown(Sender: TObject; var Key: Word; Shift:
      TShiftState);
    procedure OpenFromFileCmdExecute(Sender: TObject);
  private
    { Private declarations }
    function GetCurrentAttributeName: String;
    function ConfirmDeleteClass(AObject: TModelObject; var NeedRefresh: Boolean):
      Boolean;
    function GetCurrentNamespace: String;
    function AddDataTypeNode(AModelDataType: TModelDataType): TTreeNode;
    procedure AddIndex;
    function GetCurrentClassNode: TTreeNode;
    function GetCurrentModelObject: TModelObject;
  protected
    procedure AddAttribute;
    procedure AddRelation(DefaultRelationType: TRelationType);
    function GetRootOf(Node: TTreeNode): TTreeNode;
  public
    procedure DeleteObject(Node: TTreeNode);
    property CurrentAttributeName: String read GetCurrentAttributeName;
    property CurrentNamespace: String read GetCurrentNamespace;
    property CurrentModelObject: TModelObject read GetCurrentModelObject;
    { Public declarations }
  published
  end;

  TModelEditorFormClass = class of TModelEditorForm;

function ShowModelEditor(AModel: TovModel): Boolean;

var
  ModelEditorFormClass: TModelEditorFormClass = TModelEditorForm;

implementation

uses ovInfoDlg, ovNewRelation, ovIObjects, ovConsts, ovUtils;

{$R *.DFM}

function ShowModelEditor(AModel: TovModel): Boolean;
begin
  with ModelEditorFormClass.Create(Application) do
  try
    Model := AModel;
    Model.LoadModel;
    Result := ShowModal = mrOk;
    if Result then Model.SaveModel;
  finally
//    Model.UnloadModel;
    Free
  end;
end;

function GetNodeAfterDeletion(ASelectedNode: TTreeNode): TTreeNode;
begin
  if ASelectedNode.GetPrevSibling <> nil
    then Result := ASelectedNode.GetPrevSibling
    else if ASelectedNode.GetNextSibling <> nil
      then Result := ASelectedNode.GetNextSibling
      else Result := ASelectedNode.Parent;
end;

{ TModelEditorForm }

{***************************************************************************** }
{                                                   }
{ **************************************************************************** }

function TModelEditorForm.GetCurrentNamespace: String;
var
  RootNode: TTreeNode;
begin
  Result := '';
  if ModelTreeView.Selected <> nil then
  begin
    RootNode := GetRootOf(ModelTreeView.Selected);
    if RootNode.Text <> TreeViewBuilder.DefaultNameSpaceName
      then Result := RootNode.Text
  end
end;

function TModelEditorForm.GetCurrentAttributeName: String;
begin
  if (ModelTreeView.Selected <> nil) and (ModelTreeView.Selected.Level = 1)
    then Result := GetNameFromNodeText(ModelTreeView.Selected.Text)
    else Result := ''
end;

function TModelEditorForm.GetRootOf(Node: TTreeNode): TTreeNode;
begin
  Result := Node;
  while Result.Parent <> nil do Result := Result.Parent;
end;

function TModelEditorForm.GetCurrentClassNode: TTreeNode;
begin
  case TreeViewBuilder.NodeTypes[ModelTreeView.Selected] of
    ntObject:
      Result := ModelTreeView.Selected;
    ntAttribute, ntRelation, ntGeneralization, ntIndex:
      Result := ModelTreeView.Selected.Parent
  else
    Result := nil
  end;
end;

function TModelEditorForm.GetCurrentModelObject: TModelObject;
var
  Node: TTreeNode;
begin
  Node := GetCurrentClassNode;
  if Assigned(Node) then
    Result := TModelObject(TreeViewBuilder.NodeObjects[Node])
  else
    Result := nil;
end;

{***************************************************************************** }
{                                                                              }
{ **************************************************************************** }

procedure TModelEditorForm.AddClassCmdExecute(Sender: TObject);
var
  AObject: TModelObject;
  Name: String;
begin
  inherited;
  Name := InputBox('  ', '', '');
  if Trim(Name) <> '' then
  begin
    Name := Trim(Name);
    AObject := Model.DomainModel.AddObject(Name, CurrentNamespace);
    RefreshModelView;
    ModelTreeView.Selected := TreeViewBuilder.FindObjectNode(AObject)
  end;
end;

procedure TModelEditorForm.DeleteObjectCmdExecute(Sender: TObject);
begin
  inherited;
  DeleteObject(ModelTreeView.Selected)
end;

procedure TModelEditorForm.DeleteObject(Node: TTreeNode);
var
  AObject: TModelObject;
  ANewObject: TObject;
  NeedRefresh: Boolean;
begin
  ANewObject := TreeViewBuilder.NodeObjects[GetNodeAfterDeletion(Node)];

  case TreeViewBuilder.NodeTypes[Node] of
    ntDataType:
      begin
        if MessageDlg(Format('   "%s" ?', [Node.Text]),
          mtConfirmation, [mbYes, mbNo], 0) = mrYes
        then
        begin
          Model.DomainModel.DeleteDataType(TModelDataType(TreeViewBuilder.NodeObjects[Node]))
        end
        else
          Exit
      end;
    ntObject:
      begin
        AObject := TreeViewBuilder.NodeObjects[Node] as TModelObject;
        if ConfirmDeleteClass(AObject, NeedRefresh) then
          AObject.Delete
        else
          Exit
      end;
    ntAttribute:
      begin //  
        AObject := TreeViewBuilder.NodeObjects[Node.Parent] as TModelObject;
        NeedRefresh := AObject.ChildCount > 0;
        AObject.DeleteAttribute(GetNameFromNodeText(Node.Text));
      end;
    ntRelation:
      begin //  
        AObject := TreeViewBuilder.NodeObjects[Node.Parent] as TModelObject;
        NeedRefresh := AObject.ChildCount > 0;
        AObject.DeleteRelation(GetNameFromNodeText(Node.Text));
      end;
    ntGeneralization:
      begin
        AObject := TreeViewBuilder.NodeObjects[Node.Parent] as TModelObject;
        NeedRefresh := AObject.ChildCount > 0;
        AObject.DeleteGeneralization(Node.Text);
      end;
    ntIndex:
      begin
        AObject := TreeViewBuilder.NodeObjects[Node.Parent] as TModelObject;
        NeedRefresh := False;
        AObject.DeleteIndex(TModelObjectIndex(TreeViewBuilder.NodeObjects[Node]));
      end;
    else Exit;
  end;

  ObjectInspector.ClearCurrentObject;
  if Not NeedRefresh then Node.Delete
  else
    //      ,  ,  
    //    , ..    
    //       
    RefreshModelView;

  ModelTreeView.Selected := TreeViewBuilder.FindObjectNode(ANewObject);
end;

function TModelEditorForm.ConfirmDeleteClass(AObject: TModelObject; var
  NeedRefresh: Boolean): Boolean;
var
  I: Integer;
  M, S1, S2: String;
  S: TStrings;
begin
  S := TStringList.Create;
  try
    NeedRefresh := False;
    AObject.GetReferencingObjects(S, True);
    if S.Count = 0 then //     

      Result := MessageDlg(Format('  %s ?', [AObject.FullName]), mtConfirmation, [mbYes,mbNo], 0) = mrYes

    else begin

      M := '        :'#13#13;
      S1 := '';
      S2 := '';

      for I := 0 to S.Count - 1 do
      begin
        if S[I] <> ''
        then S1 := S1 + Format(' %s   %s'#13, [S[I], TModelObject(S.Objects[I]).FullName])
        else S2 := S2 + Format('    %s'#13, [TModelObject(S.Objects[I]).FullName])
      end;

      Result := ShowInfoDlg('  ' + AObject.FullName, M+S1+S2);
      NeedRefresh := True;
    end
  finally
    S.Free
  end;
end;

{***************************************************************************** }
{                                                     }
{ **************************************************************************** }

function TModelEditorForm.AddDataTypeNode(AModelDataType: TModelDataType):
  TTreeNode;
begin
  Result := ModelTreeView.Items.Add(nil, AModelDataType.Name);
  with Result  do
  begin
    ImageIndex := TreeViewBuilder.TreeNodeImageIndexes.DataTypeNodeImageIndex;
    SelectedIndex := TreeViewBuilder.TreeNodeImageIndexes.DataTypeNodeImageIndex;
    Data := AModelDataType;
  end;
end;

procedure TModelEditorForm.AddDataTypeCmdExecute(Sender: TObject);
var
  Node: TTreeNode;
  DataTypeName: String;
  DataType: TModelDataType;
begin
  inherited;
  if InputQuery('  ', ' :', DataTypeName) then
  begin
    DataType := Model.DomainModel.AddDataType(DataTypeName);
    DataType.Namespace := CurrentNamespace;
//    Node := AddDataTypeNode(DataType);
//    Node.Parent.AlphaSort;
//    Node.Selected := True;
    RefreshModelView;
    ModelTreeView.Selected := TreeViewBuilder.FindObjectNode(DataType)
  end;
end;

procedure TModelEditorForm.DeleteAllUserTypesCmdExecute(Sender: TObject);
begin
  inherited;
//  if MessageDlg('    ?', mtConfirmation, [mbYes,mbNo], 0) = mrYes then
//  begin
//    DataTypesTree.Items.Clear;
//    Model.DomainModel.DeleteDataTypes;
//  end;
end;

{
********************************************************************************
}

procedure TModelEditorForm.ActionListUpdate(Action: TBasicAction;
  var Handled: Boolean);
var
  NodeType: TNodeType;
  E: Boolean;
begin
  inherited;
  E := ModelTreeView.Selected <> nil;
  NodeType := TreeViewBuilder.NodeTypes[ModelTreeView.Selected];
  AddAttributeCmd.Enabled := E and (NodeType <> ntNamespace) and (NodeType <> ntDataType);
  AddAssociationCmd.Enabled := AddAttributeCmd.Enabled;
  AddAggregationCmd.Enabled := AddAttributeCmd.Enabled;
  AddCompositionCmd.Enabled := AddAttributeCmd.Enabled;
  AddGeneralizationCmd.Enabled := AddAttributeCmd.Enabled;
  AddIndexCmd.Enabled := AddAttributeCmd.Enabled;

  DeleteObjectCmd.Enabled := E and (ModelTreeView.Selected.Level > 0);
  DeleteAllClassesCmd.Enabled := E and (ModelTreeView.Selected.Level = 0);
end;

procedure TModelEditorForm.DeleteAllClassesCmdExecute(Sender: TObject);
begin
  inherited;
  if MessageDlg(Format('    "%s" ?', [ModelTreeView.Selected.Text]),
    mtConfirmation, [mbYes,mbNo], 0) = mrYes then
  begin
    Model.DomainModel.DeleteObjects(CurrentNamespace);
    RefreshModelView;
  end;
end;

procedure TModelEditorForm.AddAttributeCmdExecute(Sender: TObject);
begin
  inherited;
  AddAttribute;
end;

procedure TModelEditorForm.AddIndexCmdExecute(Sender: TObject);
begin
  inherited;
  AddIndex;
end;

{{
         
}
procedure TModelEditorForm.AddAttribute;
var
  Attribute: TModelAttribute;
  AttributeNode, ClassNode: TTreeNode;
  AObject: TModelObject;
  AttributeName: String;
begin
  ClassNode := GetCurrentClassNode;
  if ClassNode = nil then Exit;

  AObject := TreeViewBuilder.NodeObjects[ClassNode] as TModelObject;
  AttributeName := Trim(InputBox('  ', '', ''));
  if AttributeName <> '' then
  begin
    Attribute := AObject.AddAttribute(AttributeName, '');
    if AObject.ChildCount > 0 then
      RefreshModelView
    else begin
      AttributeNode := ModelTreeView.Items.AddChild(ClassNode, AttributeName);
      AttributeNode.ImageIndex := TreeViewBuilder.TreeNodeImageIndexes.AttributeNodeImageIndex;
      AttributeNode.SelectedIndex := TreeViewBuilder.TreeNodeImageIndexes.AttributeNodeImageIndex;
      AttributeNode.Text := TreeViewBuilder.GetAttributeText(Attribute);
      TreeViewBuilder.NodeObjects[AttributeNode] := Attribute;
      ClassNode.AlphaSort;
    end;
    ModelTreeView.Selected := TreeViewBuilder.FindObjectNode(Attribute)
  end;
end;

procedure TModelEditorForm.AddIndex;
var
  ModelIndex: TModelObjectIndex;
  IndexNode, ClassNode: TTreeNode;
  AObject: TModelObject;
  IndexName: String;
begin
  ClassNode := GetCurrentClassNode;
  if ClassNode = nil then Exit;

  AObject := TreeViewBuilder.NodeObjects[ClassNode] as TModelObject;
  IndexName := Trim(InputBox(' ', ' ', ''));
  if IndexName <> '' then
  begin
    ModelIndex := AObject.AddIndex(IndexName);
    IndexNode := ModelTreeView.Items.AddChild(ClassNode, IndexName);
    IndexNode.ImageIndex := TreeViewBuilder.TreeNodeImageIndexes.ClassIndexNodeImageIndex;
    IndexNode.SelectedIndex := TreeViewBuilder.TreeNodeImageIndexes.ClassIndexNodeImageIndex;
    IndexNode.Text := ModelIndex.Name;
    TreeViewBuilder.NodeObjects[IndexNode] := ModelIndex;
    ClassNode.AlphaSort;
    ModelTreeView.Selected := IndexNode;
  end;
end;

procedure TModelEditorForm.AddAssociationCmdExecute(Sender: TObject);
begin
  inherited;
  AddRelation(rtAssociation);
end;

procedure TModelEditorForm.AddAggregationCmdExecute(Sender: TObject);
begin
  inherited;
  AddRelation(rtAggregation);
end;

procedure TModelEditorForm.AddCompositionCmdExecute(Sender: TObject);
begin
  inherited;
  AddRelation(rtComposition);
end;

procedure TModelEditorForm.AddGeneralizationCmdExecute(Sender: TObject);
begin
  inherited;
  AddRelation(rtGeneralization);
end;

procedure TModelEditorForm.AddRelation(DefaultRelationType: TRelationType);
var
  ClassNode: TTreeNode;
  RelationNode: TTreeNode;
  AObject: TModelObject;
  Relation: TModelRelation;
begin
  ClassNode := GetCurrentClassNode;
  if ClassNode = nil then Exit;

  AObject := TreeViewBuilder.NodeObjects[ClassNode] as TModelObject;

  Relation := ovNewRelation.AddRelation(Model.DomainModel, AObject, DefaultRelationType);
  if Relation <> nil then
  begin
    if AObject.ChildCount > 0 then
      RefreshModelView
    else with TreeViewBuilder.TreeNodeImageIndexes do
    begin
      RelationNode := ModelTreeView.Items.AddChild(ClassNode, '');
      if Relation.IsGeneralization then begin
        RelationNode.ImageIndex := GeneralizationNodeImageIndex;
        RelationNode.SelectedIndex := GeneralizationNodeImageIndex;
      end else begin
        RelationNode.ImageIndex := RelationNodeImageIndex;
        RelationNode.SelectedIndex := RelationNodeImageIndex;
      end;
      RelationNode.Text := TreeViewBuilder.GetRelationText(Relation);
      TreeViewBuilder.NodeObjects[RelationNode] := Relation;
      ClassNode.AlphaSort;
    end;
    ModelTreeView.Selected := TreeViewBuilder.FindObjectNode(Relation);
  end
end;

procedure TModelEditorForm.ModelTreeViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  inherited;
  if (Key = VK_DELETE) and DeleteObjectCmd.Enabled then DeleteObjectCmd.Execute
end;

procedure TModelEditorForm.ClassTreeDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var
  Node: TTreeNode;
begin
  inherited;
  Node := ModelTreeView.GetNodeAt(X, Y);
  Accept :=
    (TreeViewBuilder.NodeTypes[ModelTreeView.Selected] = ntObject) and
    (TreeViewBuilder.NodeTypes[Node] = ntNameSpace);
end;

procedure TModelEditorForm.ClassTreeDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  ANode: TTreeNode;
  AObject: TModelObject;
begin
  inherited;
  AObject := TreeViewBuilder.NodeObjects[ModelTreeView.Selected] as TModelObject;
  ANode := ModelTreeView.GetNodeAt(X, Y);
  if AObject.NameSpace <> TreeViewBuilder.GetNamespaceFromNodeText(ANode.Text) then
  begin
    AObject.NameSpace := TreeViewBuilder.GetNamespaceFromNodeText(ANode.Text);
    RefreshModelView;
  end;
end;

procedure TModelEditorForm.CollapseAllItemClick(Sender: TObject);
begin
  inherited;
  ModelTreeView.FullCollapse
end;

procedure TModelEditorForm.ExpandAllItemClick(Sender: TObject);
begin
  inherited;
  ModelTreeView.FullExpand
end;

{***************************************************************************** }
{                                                        }
{ **************************************************************************** }

procedure TModelEditorForm.NewModelCmdExecute(Sender: TObject);
var
  I: Integer;
begin
  inherited;
  if MessageDlg('   .  ?', mtConfirmation, [mbYes,mbNo], 0) = mrYes then
  begin
    ObjectInspector.CurrentObject := nil;
    Model.DomainModel.Clear;
    with Model do
    for I := 0 to StorageCount - 1 do
      DomainModel.NameSpaces.Add(Storages[I].NameSpace);
    RefreshModelView;
  end;
end;

procedure TModelEditorForm.SaveAndCloseCmdExecute(Sender: TObject);
begin
  inherited;
  ModalResult := mrOk
end;

procedure TModelEditorForm.CancelCmdExecute(Sender: TObject);
begin
  inherited;
  ModalResult := mrCancel
end;

procedure TModelEditorForm.SaveToFileCmdExecute(Sender: TObject);
begin
  inherited;
  Model.SaveModel
end;

procedure TModelEditorForm.OpenFromFileCmdExecute(Sender: TObject);
begin
  inherited;
  //
end;

end.
