unit ovModelEditorEx;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ovModelEditor, Menus, ActnList, ImgList, ovComponents,
  ovModelTreeViewBuilder, ComCtrls, ToolWin, ExtCtrls, ovModelViewFilter, ovModel,
  ovConsts, ovModelToolsDlg, ovDatabaseToolsDlg, ovModelEditorTypes;

type
  TExModelEditorForm = class(TModelEditorForm, IMessageLogger)
    FilterButton: TToolButton;
    FilterModelViewAction: TAction;
    N12: TMenuItem;
    FilterModelViewCmd1: TMenuItem;
    N13: TMenuItem;
    ShowDependentObjectsAction: TAction;
    GotoTargetObjectAction: TAction;
    N15: TMenuItem;
    N17: TMenuItem;
    ModelVerificationAction: TAction;
    ModelGenerationAction: TAction;
    DatabaseVerificationAction: TAction;
    DatabaseGenerationAction: TAction;
    N18: TMenuItem;
    N19: TMenuItem;
    N20: TMenuItem;
    N22: TMenuItem;
    N23: TMenuItem;
    HorizSplitter: TSplitter;
    PageControl: TPageControl;
    tsDependentObjects: TTabSheet;
    tsMessages: TTabSheet;
    DependentObjectsTree: TTreeView;
    MessageList: TListView;
    MessagesToolBar: TToolBar;
    ToolButton8: TToolButton;
    ToolButton16: TToolButton;
    FixProblemAction: TAction;
    DeleteMessageAction: TAction;
    ShowMessagesAction: TAction;
    N25: TMenuItem;
    ToolButton17: TToolButton;
    InspectMessageAction: TAction;
    ClearMessagesAction: TAction;
    ToolButton18: TToolButton;
    FindClassByIdAction: TAction;
    N24: TMenuItem;
    FindRelationByIdAction: TAction;
    N26: TMenuItem;
    procedure ActionListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure ClearMessagesActionExecute(Sender: TObject);
    procedure ModelVerificationActionExecute(Sender: TObject);
    procedure ModelGenerationActionExecute(Sender: TObject);
    procedure DatabaseVerificationActionExecute(Sender: TObject);
    procedure DatabaseGenerationActionExecute(Sender: TObject);
    procedure DeleteMessageActionExecute(Sender: TObject);
    procedure DependentObjectsTreeDblClick(Sender: TObject);
    procedure DependentObjectsTreeKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure FilterModelViewActionExecute(Sender: TObject);
    procedure FindClassByIdActionExecute(Sender: TObject);
    procedure FindRelationByIdActionExecute(Sender: TObject);
    procedure FixProblemActionExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure GotoTargetObjectActionExecute(Sender: TObject);
    procedure MessageListDblClick(Sender: TObject);
    procedure MessageListDeletion(Sender: TObject; Item: TListItem);
    procedure MessageListKeyDown(Sender: TObject; var Key: Word; Shift:
      TShiftState);
    procedure ModelTreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure ShowDependentObjectsActionExecute(Sender: TObject);
    procedure ShowMessagesActionExecute(Sender: TObject);
    procedure TreeViewBuilderVisitNamespace(ANamespace: String;
      var Skip: Boolean);
    procedure TreeViewBuilderVisitObject(AModelObject: TModelObject;
      var Skip: Boolean);
    procedure InspectMessageActionExecute(Sender: TObject);
  private
    { Private declarations }
    FModelViewFilter: TfmModelViewFilter;
    function GetTargetModelElement: TModelElement;
    procedure FindModelElement(AModelElement: TModelElement);
    procedure ShowDependentObjectsPanel(ShowPanel: Boolean);
    procedure ShowDependentObjects(AModelElement: TModelElement);
    procedure ShowDependentAttributes(Data: Pointer); overload;
    procedure ShowDependentClasses(Data: Pointer);
    function AddNodeToDependentObjectsTree(AParentNode: TTreeNode;
      AText: String; AModelElement: TModelElement;
      AImageIndex: Integer): TTreeNode;
    procedure AddMessage(AMsgType, AMsgText: String; AModelElement: TModelElement;
      AProcessMsgProc: TProcessMsgProc = nil; AInspectMsgProc: TInspectMsgProc =
      nil; AData: TObject = nil);
  protected
    procedure ShowMessages(ShowPanel: Boolean = True);
    property TargetModelElement: TModelElement read GetTargetModelElement;
  public
    { Public declarations }
    property ModelViewFilter: TfmModelViewFilter read FModelViewFilter;
  end;

var
  ExModelEditorForm: TExModelEditorForm;
  FilterRegistryKey: String = ObjectViewRegKey + '\ModelEditor\ModelViewFilter';

implementation

uses ovModelExplorer;

resourcestring
  STargetElementNotFoundCaption = '   ';
  STargetElementNotFound = '   .'+#13#10+
    '       .';

{$R *.dfm}

type
  TMessageInfo = class
  private
    FModelElement: TModelElement;
    FProcessMsgProc: TProcessMsgProc;
    FInspectMsgProc: TInspectMsgProc;
    FUserData: TObject;
  public
    constructor Create(AModelElement: TModelElement; AProcessMsgProc:
      TProcessMsgProc; AInspectMsgProc: TInspectMsgProc; AData: TObject);
    destructor Destroy; override;
    property ModelElement: TModelElement read FModelElement;
    property ProcessMsgProc: TProcessMsgProc read FProcessMsgProc;
    property InspectMsgProc: TInspectMsgProc read FInspectMsgProc;
    property UserData: TObject read FUserData;
  end;

constructor TMessageInfo.Create(AModelElement: TModelElement; AProcessMsgProc:
  TProcessMsgProc; AInspectMsgProc: TInspectMsgProc; AData: TObject);
begin
  inherited Create;
  FModelElement := AModelElement;
  FProcessMsgProc := AProcessMsgProc;
  FInspectMsgProc := AInspectMsgProc;
  FUserData := AData;
end;

destructor TMessageInfo.Destroy;
begin
  FreeAndNil(FUserData);
  inherited Destroy;
end;

function ObjectDescription(ModelElement: TModelElement): String;
begin
  if ModelElement is TModelObject then
    Result := ': ' + TModelObject(ModelElement).FullName else
  if ModelElement is TModelAttribute then
    Result := ': ' + TModelAttribute(ModelElement).Name else
  if ModelElement is TModelRelation then
    Result := Format(' "%s"  "%s"',
      [TModelRelation(ModelElement).Name,
       TModelRelation(ModelElement).SourceObjectName])
  else
    Result := '';
end;

function GetNextItemAfterDeletion(AListView: TListView; AListItem: TListItem):
  TListItem;
begin
  Result := AListItem;
  if AListView.GetNextItem(Result, sdBelow, []) <> nil then
    Result := AListView.GetNextItem(Result, sdBelow, [])
  else
    Result := AListView.GetNextItem(Result, sdAbove, [])
end;

{ TExModelEditorForm }

procedure TExModelEditorForm.FormCreate(Sender: TObject);
begin
  inherited;
  FilterButton.Left := ToolButton2.Left - 1;

  FModelViewFilter := TfmModelViewFilter.Create(Self);
end;

procedure TExModelEditorForm.FormShow(Sender: TObject);
var
  Options: TModelTreeViewBuilderOptions;
begin
  ModelViewFilter.Model := Model.DomainModel; //   
  //       FormShow
  ModelViewFilter.LoadFromRegistry(FilterRegistryKey);
  Options := TreeViewBuilder.Options;
  ModelViewFilter.GetFilterOptions(Options);
  TreeViewBuilder.Options := Options;
  inherited;
end;

procedure TExModelEditorForm.ActionListUpdate(Action: TBasicAction; var
  Handled: Boolean);
begin
  inherited;
  FixProblemAction.Enabled := (MessageList.Selected <> nil) and
    Assigned(TMessageInfo(MessageList.Selected.Data).ProcessMsgProc);
  InspectMessageAction.Enabled := (MessageList.Selected <> nil) and
    Assigned(TMessageInfo(MessageList.Selected.Data).InspectMsgProc);
  DeleteMessageAction.Enabled := MessageList.SelCount > 0;
end;

{***************************************************************************** }
{                                                    }
{ **************************************************************************** }

procedure TExModelEditorForm.FilterModelViewActionExecute(Sender: TObject);
var
  Options: TModelTreeViewBuilderOptions;
begin
  inherited;
  if Assigned(ModelViewFilter) then
  with ModelViewFilter do
  begin
    Options := TreeViewBuilder.Options;
    if ShowDialog(Options) then
    begin
      TreeViewBuilder.Options := Options;
      RefreshModelView;
      SaveToRegistry(FilterRegistryKey);
    end;
  end;
end;

procedure TExModelEditorForm.TreeViewBuilderVisitNamespace(ANamespace: String;
  var Skip: Boolean);
begin
  inherited;
  Skip := Assigned(ModelViewFilter) and ModelViewFilter.IsNamespaceFiltered(ANamespace)
end;

procedure TExModelEditorForm.TreeViewBuilderVisitObject(AModelObject:
  TModelObject; var Skip: Boolean);
begin
  inherited;
  Skip := Assigned(ModelViewFilter) and ModelViewFilter.IsObjectFiltered(AModelObject)
end;

{***************************************************************************** }
{                                                        }
{ **************************************************************************** }

procedure TExModelEditorForm.GotoTargetObjectActionExecute(Sender: TObject);
begin
  inherited;
  FindModelElement(TargetModelElement)
end;

function TExModelEditorForm.GetTargetModelElement: TModelElement;
var
  AModelElement: TModelElement;
begin
  AModelElement := SelectedModelElement;
  if AModelElement is TModelAttribute then
    Result := Model.DomainModel.FindDataType(TModelAttribute(AModelElement).TypeName) else
  if AModelElement is TModelRelation then
    Result := TModelRelation(AModelElement).TargetObject
  else
    Result := nil
end;

procedure TExModelEditorForm.FindModelElement(AModelElement: TModelElement);
var
  ANode: TTreeNode;
begin
  if AModelElement <> nil then
  begin
    Screen.Cursor := crHourGlass;
    try
      ANode := TreeViewBuilder.FindObjectNode(AModelElement);
      if Assigned(ANode)
      then
        ANode.Selected := True
      else
        Application.MessageBox(PChar(STargetElementNotFound),
          PChar(STargetElementNotFoundCaption), MB_ICONWARNING or MB_OK)
    finally
      Screen.Cursor := crDefault;
    end;
  end;
end;

procedure TExModelEditorForm.FindClassByIdActionExecute(Sender: TObject);
var
  S: String;
  AModelObject: TModelObject;
begin
  inherited;
  if InputQuery('   ', ' ', S) then
  begin
    AModelObject := Model.DomainModel.FindObject(StrToInt(S));
    if AModelObject <> nil then
      FindModelElement(AModelObject)
    else
      Application.MessageBox('     ',
        ' ', MB_ICONWARNING or MB_OK)
  end;
end;

procedure TExModelEditorForm.FindRelationByIdActionExecute(Sender: TObject);
var
  S: String;
  AModelRelation: TModelRelation;
begin
  inherited;
  if InputQuery('   ', ' ', S) then
  begin
    AModelRelation := Model.DomainModel.FindRelation(StrToInt(S));
    if AModelRelation <> nil then
      FindModelElement(AModelRelation)
    else
      Application.MessageBox('     ',
        ' ', MB_ICONWARNING or MB_OK)
  end;
end;

{***************************************************************************** }
{                                            }
{ **************************************************************************** }

procedure TExModelEditorForm.ShowDependentObjectsActionExecute(Sender: TObject);
begin
  inherited;
  ShowDependentObjectsPanel(Not PageControl.Visible)
end;

procedure TExModelEditorForm.ModelTreeViewChange(Sender: TObject; Node:
  TTreeNode);
begin
  inherited;
  if DependentObjectsTree.Visible then
    ShowDependentObjects(SelectedModelElement)
end;

procedure TExModelEditorForm.ShowDependentObjectsPanel(ShowPanel: Boolean);
begin
  if ShowPanel then ShowDependentObjects(SelectedModelElement); //     
  PageControl.Visible := ShowPanel;
  HorizSplitter.Visible := ShowPanel;
  if ShowPanel then
  begin
    HorizSplitter.Top := PageControl.Top - 1;
    PageControl.ActivePage := tsDependentObjects;
    DependentObjectsTree.SetFocus;
  end;
end;

{{
  DependentObjectList  ,    AModelElement.
}
procedure TExModelEditorForm.ShowDependentObjects(AModelElement: TModelElement);
begin
  if AModelElement is TModelDataType then
    BuildTreeView(DependentObjectsTree, ShowDependentAttributes, AModelElement)
  else
  if AModelElement is TModelObject then
    BuildTreeView(DependentObjectsTree, ShowDependentClasses, AModelElement)
  else
    DependentObjectsTree.Items.Clear;
end;

procedure TExModelEditorForm.ShowDependentAttributes(Data: Pointer);
var
  I, J: Integer;
  RootNode: TTreeNode;
begin
  RootNode := AddNodeToDependentObjectsTree(nil, ',   - ' +
    TModelDataType(Data).Name, nil, TreeViewBuilder.TreeNodeImageIndexes.NamespaceNodeImageIndex);
  with Model.DomainModel do
    for I := 0 to ObjectCount - 1 do
      with Objects[I] do
        for J := 0 to AttributeCount - 1 do
        if Attributes[J].TypeName = TModelDataType(Data).Name then
          AddNodeToDependentObjectsTree(RootNode,
            Attributes[J].Owner.FullName + '.' + Attributes[J].Name, Attributes[J],
            TreeViewBuilder.TreeNodeImageIndexes.AttributeNodeImageIndex)
end;

procedure TExModelEditorForm.ShowDependentClasses(Data: Pointer);
var
  I: Integer;
  DerivedClasses, DependentClasses: TTreeNode;
begin
  DerivedClasses := AddNodeToDependentObjectsTree(nil, ' ',
    nil, TreeViewBuilder.TreeNodeImageIndexes.NamespaceNodeImageIndex);

  DependentClasses := AddNodeToDependentObjectsTree(nil, ' ',
    nil, TreeViewBuilder.TreeNodeImageIndexes.NamespaceNodeImageIndex);

  with Model.DomainModel do
    for I := 0 to RelationCount - 1 do
      if Relations[I].TargetObjectName = TModelObject(Data).FullName then
        if Relations[I].IsGeneralization then
          {    }
          AddNodeToDependentObjectsTree(DerivedClasses,
            Relations[I].SourceObjectName, Relations[I].SourceObject,
            TreeViewBuilder.TreeNodeImageIndexes.ClassNodeImageIndex)
        else
          {    }
          AddNodeToDependentObjectsTree(DependentClasses,
            Relations[I].SourceObjectName + ' ('+Relations[I].Name+')', Relations[I].SourceObject,
            TreeViewBuilder.TreeNodeImageIndexes.ClassNodeImageIndex);

  if DerivedClasses.Count = 0 then DerivedClasses.Delete;
  if DependentClasses.Count = 0 then DependentClasses.Delete;
end;

function TExModelEditorForm.AddNodeToDependentObjectsTree(AParentNode: TTreeNode; AText: String;
  AModelElement: TModelElement; AImageIndex: Integer): TTreeNode;
begin
  Result := DependentObjectsTree.Items.AddChild(AParentNode, AText);
  Result.Data := AModelElement;
  Result.ImageIndex := AImageIndex;
  Result.SelectedIndex := Result.ImageIndex;
end;

procedure TExModelEditorForm.DependentObjectsTreeKeyPress(Sender: TObject; var
  Key: Char);
begin
  inherited;
  if (Key = #13) and Assigned(DependentObjectsTree.Selected) and
    Assigned(DependentObjectsTree.Selected.Data) then
  begin
    Key := #0;
    FindModelElement(TModelElement(DependentObjectsTree.Selected.Data));
  end;
end;

procedure TExModelEditorForm.DependentObjectsTreeDblClick(Sender: TObject);
var
  Key: Char;
begin
  inherited;
  Key := #13;
  DependentObjectsTreeKeyPress(Sender, Key)
end;

{***************************************************************************** }
{                                             }
{ **************************************************************************** }

procedure TExModelEditorForm.ModelVerificationActionExecute(Sender: TObject);
begin
  inherited;
  ShowMessages;
  ShowVerifyModelDialog(Model.DomainModel, Self)
end;

procedure TExModelEditorForm.ModelGenerationActionExecute(Sender: TObject);
begin
  inherited;
  ShowMessages;
  ShowGenerateModelDialog(Model.DomainModel, Self)
end;

procedure TExModelEditorForm.DatabaseVerificationActionExecute(Sender: TObject);
begin
  inherited;
  ShowMessages;
  ShowVerifyDatabaseDialog(Model, CurrentModelObject, CurrentNamespace, Self)
end;

procedure TExModelEditorForm.DatabaseGenerationActionExecute(Sender: TObject);
begin
  inherited;
  ShowMessages;
  ShowGenerateDatabaseDialog(Model, CurrentModelObject, CurrentNamespace, Self)
end;

{***************************************************************************** }
{                                                       }
{ **************************************************************************** }

procedure TExModelEditorForm.ShowMessagesActionExecute(Sender: TObject);
begin
  inherited;
  ShowMessages(Not PageControl.Visible);
end;

{{
  
}
procedure TExModelEditorForm.ShowMessages(ShowPanel: Boolean = True);
begin
  PageControl.Visible := ShowPanel;
  HorizSplitter.Visible := ShowPanel;
  if ShowPanel then
  begin
    HorizSplitter.Top := PageControl.Top - 1;
    PageControl.ActivePage := tsMessages;
    MessageList.SetFocus;
  end;
end;

{{
    
}
procedure TExModelEditorForm.AddMessage(AMsgType, AMsgText: String;
  AModelElement: TModelElement; AProcessMsgProc: TProcessMsgProc = nil;
  AInspectMsgProc: TInspectMsgProc = nil; AData: TObject = nil);
begin
  with MessageList do
  begin
    with Items.Add do
    begin
      Caption  := AMsgType;
      SubItems.Add(ObjectDescription(AModelElement));
      SubItems.Add(AMsgText);
      Data := TMessageInfo.Create(AModelElement, AProcessMsgProc, AInspectMsgProc, AData);
    end;
  end
end;

procedure TExModelEditorForm.ClearMessagesActionExecute(Sender: TObject);
begin           
  inherited;
  MessageList.Clear;
end;

{{
   ,    
}
procedure TExModelEditorForm.MessageListDblClick(Sender: TObject);
begin
  inherited;
  if Assigned(MessageList.Selected) then
    FindModelElement(TMessageInfo(MessageList.Selected.Data).ModelElement);
end;

{{
      
}
procedure TExModelEditorForm.FixProblemActionExecute(Sender: TObject);
var
  Item: TListItem;
begin
  inherited;
  with MessageList do
  begin
    Item := Selected;
    while Item <> nil do
    begin
      if Not Item.Checked then
      with TMessageInfo(Item.Data) do
        if Assigned(ProcessMsgProc) then
        begin
          ProcessMsgProc(ModelElement, UserData);
          Item.Checked := True;
        end;
      Item := GetNextItem(Item, sdBelow, [isSelected]);
    end;
  end;
  {      -    
   }
  ObjectInspector.RefreshObjectProperties;
end;

{{
       
}
procedure TExModelEditorForm.InspectMessageActionExecute(Sender: TObject);
begin
  inherited;
  with TMessageInfo(MessageList.Selected.Data) do
    if Assigned(InspectMsgProc) then InspectMsgProc(ModelElement, UserData);
end;

{{
   
}
procedure TExModelEditorForm.DeleteMessageActionExecute(Sender: TObject);
var
  Item, NextItem, NewSelectedItem: TListItem;
begin
  inherited;
  with MessageList do
  begin
    Item := Selected;
    while Item <> nil do
    begin
      NextItem := GetNextItem(Item, sdBelow, [isSelected]);
      if NextItem = nil then NewSelectedItem := GetNextItemAfterDeletion(MessageList, Item);
      Item.Delete;
      Item := NextItem;
    end;
    if NewSelectedItem <> nil then NewSelectedItem.Selected := True
  end;
  MessageList.SetFocus;
end;

procedure TExModelEditorForm.MessageListDeletion(Sender: TObject; Item:
  TListItem);
begin
  inherited;
  TObject(Item.Data).Free
end;

procedure TExModelEditorForm.MessageListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  inherited;
  if (Key = VK_DELETE) and DeleteMessageAction.Enabled then DeleteMessageAction.Execute
end;

initialization
  ModelEditorFormClass := TExModelEditorForm
end.
