unit ovModelExplorer;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ovModel, ovComponents, ovModelTreeViewBuilder, ImgList, StdCtrls,
  ExtCtrls, Buttons, AppEvnts, ovModelObjectInspector, ovStdInspectorItems;

type
  TModelExplorerForm = class(TForm)
    Splitter: TSplitter;
    PropertiesPanel: TPanel;
    TreeViewBuilder: TovModelTreeViewBuilder;
    ImageList: TImageList;
    ModelTreeView: TTreeView;
    procedure ModelTreeViewChanging(Sender: TObject; Node: TTreeNode; var AllowChange:
      Boolean);
    procedure FormShow(Sender: TObject);
    procedure ModelTreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    FObjectInspector: TfrmModelObjectInspector;
    FModel: TovModel;
    procedure SetModel(const Value: TovModel);
    procedure ShowCurrentObjectProperties;
    procedure CreateObjectInspector;
    procedure InspectorRenameObjectHandler(Sender: TObject);
    procedure AttributeTypeNameChanged(Sender: TObject);
    procedure IndexAttributesButtonClick(Sender: TObject);
    function GetSelectedModelElement: TModelElement;
    procedure BidirectionalChanged(Sender: TObject);
  protected
    procedure RefreshModelView; dynamic;
  public
    { Public declarations }
    property Model: TovModel read FModel write SetModel;
    property ObjectInspector: TfrmModelObjectInspector read FObjectInspector;
    property SelectedModelElement: TModelElement read GetSelectedModelElement;
  published
  end;

function ShowModelExplorer(AModel: TovModel): Boolean;

implementation

uses ovAttributeSelectionDlg, ovInspector;

{$R *.DFM}

function ShowModelExplorer(AModel: TovModel): Boolean;
begin
  with TModelExplorerForm.Create(Application) do
  try
    Model := AModel;
    Result := ShowModal = mrOk
  finally
    Free
  end;
end;

{ TModelExplorerForm }

procedure TModelExplorerForm.FormCreate(Sender: TObject);
begin
  CreateObjectInspector;
end;

procedure TModelExplorerForm.FormShow(Sender: TObject);
begin
  RefreshModelView;
  ModelTreeView.SetFocus;
end;

procedure TModelExplorerForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  ObjectInspector.UpdateCurrentObject;  //  
end;

procedure TModelExplorerForm.CreateObjectInspector;
begin
  FObjectInspector := TfrmModelObjectInspector.Create(Self);
  FObjectInspector.Parent := PropertiesPanel;
  FObjectInspector.Align := alClient;

  FObjectInspector.OnRenameObject := InspectorRenameObjectHandler;
  FObjectInspector.AttributeProperties.Items.ItemByCaption[
    ' '].OnEdited := AttributeTypeNameChanged;
  (FObjectInspector.IndexProperties.Items.ItemByCaption[
    ''].Properties as TButtonEditItemProperties).OnButtonClick := IndexAttributesButtonClick;
  FObjectInspector.RelationProperties.Items.ItemByCaption[
    ''].OnEdited := BidirectionalChanged;
end;

procedure TModelExplorerForm.SetModel(const Value: TovModel);
begin
  FModel := Value;
  ObjectInspector.Model := Model.DomainModel;
  TreeViewBuilder.Model := Model;
end;

procedure TModelExplorerForm.RefreshModelView;
begin
  {     }
  TreeViewBuilder.BuildTreeView;
  if ModelTreeView.Items.Count > 0 then ModelTreeView.Items[0].Selected := True;

  {    }
  ShowCurrentObjectProperties;
end;

{***************************************************************************** }
{                                             }
{ **************************************************************************** }

function TModelExplorerForm.GetSelectedModelElement: TModelElement;
begin
  case TreeViewBuilder.NodeTypes[ModelTreeView.Selected] of
    ntDataType, ntObject, ntAttribute, ntGeneralization, ntRelation, ntIndex:
      Result := TModelElement(TreeViewBuilder.NodeObjects[ModelTreeView.Selected])
    else
      Result := nil
  end
end;

procedure TModelExplorerForm.ModelTreeViewChanging(Sender: TObject; Node:
  TTreeNode; var AllowChange: Boolean);
begin
  ObjectInspector.UpdateCurrentObject;
end;

procedure TModelExplorerForm.ModelTreeViewChange(Sender: TObject; Node:
  TTreeNode);
begin
  ShowCurrentObjectProperties;
end;

procedure TModelExplorerForm.ShowCurrentObjectProperties;
begin
  ObjectInspector.CurrentObject := SelectedModelElement;
end;

{***************************************************************************** }
{                                         }
{ **************************************************************************** }

{{
        .
        . , 
       .  
  ObjectInspector.UpdateCurrentObject  ModelTreeViewChanging.
}
procedure TModelExplorerForm.InspectorRenameObjectHandler(Sender: TObject);
begin
  if ObjectInspector.CurrentObject is TModelDataType then
    ModelTreeView.Selected.Text := TModelDataType(ObjectInspector.CurrentObject).Name else
  if ObjectInspector.CurrentObject is TModelObject then
    ModelTreeView.Selected.Text := TModelObject(ObjectInspector.CurrentObject).Name else
  if ObjectInspector.CurrentObject is TModelAttribute then
    ModelTreeView.Selected.Text := TreeViewBuilder.GetAttributeText(
      TModelAttribute(ObjectInspector.CurrentObject)) else
  if ObjectInspector.CurrentObject is TModelRelation then
    ModelTreeView.Selected.Text := TreeViewBuilder.GetRelationText(
      TModelRelation(ObjectInspector.CurrentObject)) else
  if ObjectInspector.CurrentObject is TModelObjectIndex then
    ModelTreeView.Selected.Text := TModelObjectIndex(ObjectInspector.CurrentObject).Name
end;

{{
        .
      .
}
procedure TModelExplorerForm.AttributeTypeNameChanged(Sender: TObject);
begin
  ObjectInspector.UpdateCurrentObject;
  if ObjectInspector.CurrentObject is TModelAttribute then
    ModelTreeView.Selected.Text := TreeViewBuilder.GetAttributeText(
      TModelAttribute(ObjectInspector.CurrentObject)) else
end;

procedure TModelExplorerForm.IndexAttributesButtonClick(Sender: TObject);
var
  Selection: String;
begin
  Selection := (Sender as TInspectorItem).Value;
  if SelectObjectAttributes(Model,
    TreeViewBuilder.NodeObjects[ModelTreeView.Selected.Parent] as TModelObject,
    Selection, '  ', [soAllowSelectRelations])
  then (Sender as TInspectorItem).Value := Selection;
end;

{{
     "" 
  .        .
}
procedure TModelExplorerForm.BidirectionalChanged(Sender: TObject);
var
  AModelRelation: TModelRelation;
  Item: TInspectorItem;
begin
  Item := FObjectInspector.RelationProperties.Items.ItemByCaption[''];
  AModelRelation := FObjectInspector.CurrentObject as TModelRelation;

  AModelRelation.BiDirectional := Item.Value;
  if Item.Value then
    AModelRelation.CreateBackwardRelation
  else
    AModelRelation.DeleteBackwardRelation;

  FObjectInspector.UpdateCurrentObject; //      
  FObjectInspector.ClearCurrentObject;  //     
  RefreshModelView;
end;

end.
