unit ovModelObjectInspector;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ovInspector, ovModel;

type
  TRenameObjectEvent = procedure(Sender: Tobject) of
    object;

  TfrmModelObjectInspector = class(TFrame)
    ClassProperties: TovInspector;
    EmptyProperties: TovInspector;
    AttributeProperties: TovInspector;
    RelationProperties: TovInspector;
    GeneralizationProperties: TovInspector;
    DataTypeProperties: TovInspector;
    IndexProperties: TovInspector;
    procedure AttributeTypeNameEnter(Sender: TObject);
    procedure ObjectRenamed(Sender: TObject);
  private
    { Private declarations }
    FCurrentObject: TModelElement;
    FModel: TDomainModel;
    FOnRenameObject: TRenameObjectEvent;
    FActiveInspector: TovInspector;
    procedure SetCurrentObject(const Value: TModelElement);
    procedure ShowDataTypeProperties(AModelDataType: TModelDataType);
    procedure ShowObjectProperties(AModelElement: TModelElement);
    procedure ShowClassProperties(AModelObject: TModelObject);
    procedure ShowAttributeProperties(AModelAttribute: TModelAttribute);
    procedure ShowGeneralizationProperties(AModelRelation: TModelRelation);
    procedure ShowRelationProperties(AModelRelation: TModelRelation);
    procedure UpdateDataTypeProperties(AModelDataType: TModelDataType);
    procedure UpdateObjectProperties(AModelElement: TModelElement);
    procedure UpdateRelationProperties(AModelRelation: TModelRelation);
    procedure UpdateAttributeProperties(AModelAttribute: TModelAttribute);
    procedure UpdateClassProperties(AModelObject: TModelObject);
    procedure UpdateGeneralizationProperties(AModelRelation: TModelRelation);
    procedure GetAttributeTypes(ATypes: TStrings);
    procedure ShowIndexProperties(AModelIndex: TModelObjectIndex);
    procedure UpdateIndexProperties(AModelIndex: TModelObjectIndex);
  protected
    property ActiveInspector: TovInspector read FActiveInspector;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure UpdateCurrentObject;
    procedure ClearCurrentObject;
    procedure RefreshObjectProperties;
    property CurrentObject: TModelElement read FCurrentObject write
      SetCurrentObject;
    property Model: TDomainModel read FModel write FModel;
  published
    property OnRenameObject: TRenameObjectEvent read FOnRenameObject write
      FOnRenameObject;
  end;

procedure GetBaseTypes(ABaseTypes: TStrings);

implementation

uses ovStdInspectorItems, DB;  //     

{$R *.dfm}

procedure GetBaseTypes(ABaseTypes: TStrings);
const
  SimpleTypes: array [1..21] of String = (
    'String', 'Smallint', 'Integer', 'Word', 'Boolean', 'Float', 'Currency', 'BCD',
    'Date', 'Time', 'DateTime', 'Bytes', 'VarBytes', 'Blob', 'Memo', 'Graphic',
    'FmtMemo', 'FixedChar', 'WideString', 'LargeInt', 'Variant');
var
  I: Integer;
begin
  ABaseTypes.Clear;
  for I := Low(SimpleTypes) to High(SimpleTypes) do
    ABaseTypes.Add(SimpleTypes[I])
end;

{ TfrmModelObjectInspector }

constructor TfrmModelObjectInspector.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Color := clWindow;
  ParentFont := True;
  DataTypeProperties.Align := alClient;
  ClassProperties.Align := alClient;
  AttributeProperties.Align := alClient;
  RelationProperties.Align := alClient;
  GeneralizationProperties.Align := alClient;
  IndexProperties.Align := alClient;
  EmptyProperties.Align := alClient;
  EmptyProperties.BringToFront;
  FActiveInspector := EmptyProperties;

  GetBaseTypes((DataTypeProperties.Items.ItemByCaption[
    ' '].Properties as TComboItemProperties).Items);
end;

procedure TfrmModelObjectInspector.SetCurrentObject(const Value: TModelElement);
begin
  if FCurrentObject <> Value then
  begin
    UpdateObjectProperties(CurrentObject);
    FCurrentObject := Value;
    ShowObjectProperties(Value);
  end;
end;

{{
          .
  , ,  ,     .
}
procedure TfrmModelObjectInspector.ClearCurrentObject;
begin
  FCurrentObject := nil;
  ShowObjectProperties(nil);
end;

{{
       
}
procedure TfrmModelObjectInspector.RefreshObjectProperties;
begin
  ShowObjectProperties(CurrentObject);
end;

procedure TfrmModelObjectInspector.ObjectRenamed(Sender: TObject);
begin
  UpdateObjectProperties(CurrentObject);
  if Assigned(FOnRenameObject) then FOnRenameObject(Self);
end;

{***************************************************************************** }
{                                                    }
{ **************************************************************************** }

procedure TfrmModelObjectInspector.ShowObjectProperties(AModelElement:
  TModelElement);
begin
  if AModelElement = nil then
    FActiveInspector := EmptyProperties
  else
  if AModelElement is TModelDataType then
  begin
    ShowDataTypeProperties(TModelDataType(AModelElement));
    FActiveInspector := DataTypeProperties
  end else
  if AModelElement is TModelObject then
  begin
    ShowClassProperties(TModelObject(AModelElement));
    FActiveInspector := ClassProperties
  end else
  if AModelElement is TModelAttribute then
  begin
    ShowAttributeProperties(TModelAttribute(AModelElement));
    FActiveInspector := AttributeProperties
  end else
  if AModelElement is TModelRelation then
  begin
    if Not TModelRelation(AModelElement).IsGeneralization then
    begin
      ShowRelationProperties(TModelRelation(AModelElement));
      FActiveInspector := RelationProperties
    end else
    begin
      ShowGeneralizationProperties(TModelRelation(AModelElement));
      FActiveInspector := GeneralizationProperties
    end
  end else
  if AModelElement is TModelObjectIndex then
  begin
    ShowIndexProperties(TModelObjectIndex(AModelElement));
    FActiveInspector := IndexProperties;
  end else
    FActiveInspector := EmptyProperties;
  FActiveInspector.BringToFront;
end;

procedure TfrmModelObjectInspector.ShowDataTypeProperties(AModelDataType:
  TModelDataType);
begin
  with DataTypeProperties.Items do
  begin
    Items[0].Value := AModelDataType.Name;
    Items[1].Value := AModelDataType.BaseType;
    Items[2].Value := IntToStr(AModelDataType.Size);
    Items[3].Value := AModelDataType.Description;
  end;
end;

procedure TfrmModelObjectInspector.ShowClassProperties(AModelObject:
  TModelObject);
begin
  with ClassProperties.Items do
  begin
    Items[0].Value := AModelObject.Name;
    Items[1].Value := AModelObject.IsAbstract;
    Items[2].Value := IntToStr(AModelObject.Id);
    Items[3].Value := AModelObject.Description;
    Items[4].Value := AModelObject.TableName;
    Items[5].Value := AModelObject.VisibleForUser;
  end;
end;

procedure TfrmModelObjectInspector.ShowAttributeProperties(AModelAttribute:
  TModelAttribute);
begin
  with AttributeProperties.Items do
  begin
    Items[0].Value := AModelAttribute.Name;
    Items[1].Value := AModelAttribute.TypeName;
    Items[2].Value := IntToStr(AModelAttribute.Size);
    Items[3].Value := AModelAttribute.Persistent;
    Items[4].Value := AModelAttribute.Calculated;
    Items[5].Value := AModelAttribute.IsInherited;
    if AModelAttribute.IsInherited then
      Items[6].Value := AModelAttribute.ParentObject.FullName
    else
      Items[6].Value := '';
    Items[7].Value := AModelAttribute.Required;
    Items[8].Value := AModelAttribute.RequiredMsg;
    Items[9].Value := AModelAttribute.Description;
    Items[10].Value := AModelAttribute.FieldName;
    Items[11].Value := AModelAttribute.VisibleForUser;
  end;
end;

procedure TfrmModelObjectInspector.ShowRelationProperties(AModelRelation:
  TModelRelation);
begin
  with RelationProperties.Items do
  begin
    Items[0].Value := AModelRelation.Name;
    Items[1].Value := AModelRelation.TargetObjectName;
    Items[2].Value := Integer(AModelRelation.RelationType);
    Items[3].Value := CardinalityToString(AModelRelation.TargetMinCount);
    Items[4].Value := CardinalityToString(AModelRelation.TargetMaxCount);
    Items[5].Value := Integer(AModelRelation.DeleteMode);
    Items[6].Value := IntToStr(AModelRelation.Id);
    Items[7].Value := AModelRelation.IsInherited;
    if AModelRelation.IsInherited then
      Items[8].Value := AModelRelation.ParentObject.FullName
    else
      Items[8].Value := '';
    Items[9].Value := AModelRelation.Required;
    Items[10].Value := AModelRelation.RequiredMsg;
    Items[11].Value := AModelRelation.Description;
    Items[12].Value := AModelRelation.TableName;
    Items[13].Value := AModelRelation.FieldName;
    Items[14].Value := AModelRelation.VisibleForUser;
    Items[15].Value := AModelRelation.BiDirectional;
  end;
end;

procedure TfrmModelObjectInspector.ShowGeneralizationProperties(AModelRelation:
  TModelRelation);
begin
  with GeneralizationProperties.Items do
  begin
    Items[0].Value := AModelRelation.FieldName;
  end
end;

procedure TfrmModelObjectInspector.ShowIndexProperties(AModelIndex:
  TModelObjectIndex);
begin
  with IndexProperties.Items do
  begin
    Items[0].Value := AModelIndex.Name;
    Items[1].Value := AModelIndex.Attributes;
    Items[2].Value := ixUnique in AModelIndex.Options;
    Items[3].Value := ixDescending in AModelIndex.Options;
  end;
end;

{***************************************************************************** }
{                                                      }
{ **************************************************************************** }

{{
    
}
procedure TfrmModelObjectInspector.UpdateCurrentObject;
begin
  UpdateObjectProperties(CurrentObject);

//  {   OnRenameObject,     
//    (CurrentObject)      , 
//       (     OnEdited
//      ObjectRenamed)
//  }
  if Assigned(FOnRenameObject) and Assigned(ActiveInspector.FocusedItem) and
    Assigned(@ActiveInspector.FocusedItem.OnEdited) then FOnRenameObject(Self);
end;

procedure TfrmModelObjectInspector.UpdateObjectProperties(AModelElement:
  TModelElement);
begin
  if AModelElement <> nil then
    if AModelElement is TModelDataType then
      UpdateDataTypeProperties(TModelDataType(AModelElement))
    else
    if AModelElement is TModelObject then
      UpdateClassProperties(TModelObject(AModelElement))
    else
    if AModelElement is TModelAttribute then
      UpdateAttributeProperties(TModelAttribute(AModelElement))
    else
    if AModelElement is TModelRelation then
      if Not TModelRelation(AModelElement).IsGeneralization then
        UpdateRelationProperties(TModelRelation(AModelElement))
      else
        UpdateGeneralizationProperties(TModelRelation(AModelElement))
    else
    if AModelElement is TModelObjectIndex then
        UpdateIndexProperties(TModelObjectIndex(AModelElement))
end;

procedure TfrmModelObjectInspector.UpdateDataTypeProperties(AModelDataType:
  TModelDataType);
begin
  with DataTypeProperties.Items do
  begin
    AModelDataType.Name := Items[0].Value;
    AModelDataType.BaseType := Items[1].Value;
    AModelDataType.Size := StrToInt(Items[2].Value);
    AModelDataType.Description := Items[3].Value;
  end;
end;

procedure TfrmModelObjectInspector.UpdateClassProperties(AModelObject:
  TModelObject);
begin
  with ClassProperties.Items do
  begin
    AModelObject.Name := Items[0].Value;
    AModelObject.IsAbstract := Items[1].Value;
    AModelObject.Id := StrToInt(Items[2].Value);
    AModelObject.Description := Items[3].Value;
    AModelObject.TableName := Items[4].Value;
    AModelObject.VisibleForUser := Items[5].Value;
  end;
end;

procedure TfrmModelObjectInspector.UpdateAttributeProperties(AModelAttribute:
  TModelAttribute);
begin
  with AttributeProperties.Items do
  begin
    AModelAttribute.Name := Items[0].Value;
    AModelAttribute.TypeName := Items[1].Value;
    AModelAttribute.Size := StrToInt(Items[2].Value);
    AModelAttribute.Persistent := Items[3].Value;
    AModelAttribute.Calculated := Items[4].Value;
    AModelAttribute.Required := Items[7].Value;
    AModelAttribute.RequiredMsg := Items[8].Value;
    AModelAttribute.Description := Items[9].Value;
    AModelAttribute.FieldName := Items[10].Value;
    AModelAttribute.VisibleForUser := Items[11].Value;
  end;
end;

procedure TfrmModelObjectInspector.UpdateRelationProperties(AModelRelation:
  TModelRelation);
begin
  with RelationProperties.Items do
  begin
    AModelRelation.Name := Items[0].Value;
    AModelRelation.RelationType := TRelationType(Items[2].Value);
    AModelRelation.TargetMinCount := StringToCardinality(Items[3].Value);
    AModelRelation.TargetMaxCount := StringToCardinality(Items[4].Value);
    AModelRelation.DeleteMode := TDeleteMode(Items[5].Value);
    AModelRelation.Id := StrToInt(Items[6].Value);
    AModelRelation.Required := Items[9].Value;
    AModelRelation.RequiredMsg := Items[10].Value;
    AModelRelation.Description := Items[11].Value;
    AModelRelation.TableName := Items[12].Value;
    AModelRelation.FieldName := Items[13].Value;
    AModelRelation.VisibleForUser := Items[14].Value;
    AModelRelation.BiDirectional := Items[15].Value;
  end;
end;

procedure TfrmModelObjectInspector.UpdateGeneralizationProperties(
  AModelRelation: TModelRelation);
begin
  with GeneralizationProperties.Items do
  begin
    AModelRelation.FieldName := Items[0].Value;
  end
end;

procedure TfrmModelObjectInspector.UpdateIndexProperties(AModelIndex:
  TModelObjectIndex);
begin
  with IndexProperties.Items, AModelIndex do
  begin
    Name := Items[0].Value;
    Attributes := Items[1].Value;
    Options := [];
    if Items[2].Value then Options := Options + [ixUnique];
    if Items[3].Value then Options := Options + [ixDescending];
  end;
end;

{***************************************************************************** }
{                                                }
{ **************************************************************************** }

procedure TfrmModelObjectInspector.AttributeTypeNameEnter(Sender: TObject);
begin
  GetAttributeTypes((AttributeProperties.Items.ItemByCaption[
    ' '].Properties as TComboItemProperties).Items);
end;

procedure TfrmModelObjectInspector.GetAttributeTypes(ATypes: TStrings);
var
  I: Integer;
begin
  GetBaseTypes(ATypes);
  if Model <> nil then
  with Model do
    for I := 0 to DataTypeCount - 1 do
      ATypes.Add(DataTypes[I].Name)
end;

end.
