unit ovModelSyncDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, ImgList, ovModelSync, ovComponents,
  ovCheckTreeView, Buttons, Menus;

type
  TfrmModelSynchronizeDlg = class(TForm)
    UpdateTree: TovCheckTreeView;
    ImageList: TImageList;
    Panel: TPanel;
    OkButton: TButton;
    Button2: TButton;
    PopupMenu: TPopupMenu;
    ShowAttributesAndRelations: TMenuItem;
    procedure FormShow(Sender: TObject);
    procedure OkButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ShowAttributesAndRelationsClick(Sender: TObject);
  private
    FModelSynchronizer: TovModelSynchronizer;
    AddedItems, DeletedItems, ExistItems: TStringList;
    procedure BuildTreeItems;
    procedure BuildAttributeItems(RootNode: TTreeNode);
    function BuildClassItems: TTreeNode;
    procedure BuildRelationItems(RootNode: TTreeNode);
    procedure AddClasses(RootNode: TTreeNode);
    procedure DeleteClasses(RootNode: TTreeNode);
    procedure UpdateClasses(RootNode: TTreeNode);
    function AddGroup(RootNode: TTreeNode; GroupTitle: String; GroupItems: TStrings;
      ImageIndex: Integer): TTreeNode;
    { Private declarations }
  public
    procedure SyncModel;
    procedure RefreshObjectTree;
    property ModelSynchronizer: TovModelSynchronizer read FModelSynchronizer;
    { Public declarations }
  end;

  TSyncDirection = (sdImportModel, sdExportModel);

function SynchronizeModels(AModelSyncrhonizer: TovModelSynchronizer; ACaption:
  String = ''): Boolean; overload;

implementation

{$R *.DFM}

function SynchronizeModels(AModelSyncrhonizer: TovModelSynchronizer; ACaption:
  String = ''): Boolean;
begin
  with TfrmModelSynchronizeDlg.Create(Application) do
  try
    if ACaption <> '' then Caption := ACaption;
    FModelSynchronizer := AModelSyncrhonizer;
    Result := ShowModal = mrOk;
  finally
    Free
  end;
end;

{ TfrmModelSynchronizeDlg }

procedure TfrmModelSynchronizeDlg.FormCreate(Sender: TObject);
begin
  AddedItems := TStringList.Create;
  DeletedItems := TStringList.Create;
  ExistItems := TStringList.Create;
end;

procedure TfrmModelSynchronizeDlg.FormDestroy(Sender: TObject);
begin
  AddedItems.Free;
  DeletedItems.Free;
  ExistItems.Free;
end;

procedure TfrmModelSynchronizeDlg.FormShow(Sender: TObject);
begin
  RefreshObjectTree;
end;

procedure TfrmModelSynchronizeDlg.ShowAttributesAndRelationsClick(
  Sender: TObject);
begin
  ShowAttributesAndRelations.Checked := Not ShowAttributesAndRelations.Checked;
  RefreshObjectTree
end;

{
**************************    ************************
}

procedure TfrmModelSynchronizeDlg.RefreshObjectTree;
begin
  Screen.Cursor := crHourGlass;
  try
    with UpdateTree.Items do
    begin
      BeginUpdate;
      try
        Clear;
        BuildTreeItems;
        UpdateTree.AlphaSort;
      finally
        EndUpdate
      end;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmModelSynchronizeDlg.BuildTreeItems;
var
  CommonClasses: TTreeNode;
begin
  CommonClasses := BuildClassItems;
  if CommonClasses <> nil then
  begin
    BuildAttributeItems(CommonClasses);
    BuildRelationItems(CommonClasses);
  end
end;

{{
    ,   
}
function TfrmModelSynchronizeDlg.BuildClassItems: TTreeNode;
begin
  ModelSynchronizer.CompareClasses(AddedItems, DeletedItems, ExistItems);

  if AddedItems.Count > 0 then
    AddGroup(nil, ' ', AddedItems, 1);

  if DeletedItems.Count > 0 then
    AddGroup(nil, ' ', DeletedItems, 1);

  if ExistItems.Count > 0
    then Result := AddGroup(nil, ' ', ExistItems, 1)
    else Result := nil
end;

procedure TfrmModelSynchronizeDlg.BuildAttributeItems(RootNode: TTreeNode);
var
  ClassNode: TTreeNode;
  I: Integer;
begin
  for I := 0 to RootNode.Count - 1 do
  begin
    ClassNode := RootNode.Item[I];
    ModelSynchronizer.CompareAttributes(ClassNode.Text,
      AddedItems, DeletedItems, ExistItems);

    if AddedItems.Count > 0 then
      AddGroup(ClassNode, ' ', AddedItems, 2);

    if DeletedItems.Count > 0 then
       AddGroup(ClassNode, ' ', DeletedItems, 2);

    if ShowAttributesAndRelations.Checked and (ExistItems.Count > 0) then
      AddGroup(ClassNode, ' ', ExistItems, 2);
  end
end;

procedure TfrmModelSynchronizeDlg.BuildRelationItems(RootNode: TTreeNode);
var
  ClassNode: TTreeNode;
  Node: TTreeNode;
  I: Integer;
  AddedGeneralizations, DeletedGeneralizations, ExistGeneralizations: TStringList;
begin
  AddedGeneralizations := TStringList.Create;
  DeletedGeneralizations := TStringList.Create;
  ExistGeneralizations := TStringList.Create;
  try
    for I := 0 to RootNode.Count - 1 do
    begin
      ClassNode := RootNode.Item[I];
      ModelSynchronizer.CompareRelations(ClassNode.Text,
        AddedItems, DeletedItems, ExistItems);
      ModelSynchronizer.CompareGeneralizations(ClassNode.Text,
        AddedGeneralizations, DeletedGeneralizations, ExistGeneralizations);

      if (AddedItems.Count > 0) or (AddedGeneralizations.Count > 0) then begin
        Node := AddGroup(ClassNode, ' ', AddedGeneralizations, 4);
        AddGroup(Node, '', AddedItems, 3);
      end;

      if (DeletedItems.Count > 0) or (DeletedGeneralizations.Count > 0) then begin
        Node := AddGroup(ClassNode, ' ', DeletedGeneralizations, 4);
        AddGroup(Node, '', DeletedItems, 3);
      end;

      if ShowAttributesAndRelations.Checked and (ExistItems.Count > 0) then
          AddGroup(ClassNode, ' ', ExistItems, 3);
    end
  finally
    AddedGeneralizations.Free;
    DeletedGeneralizations.Free;
    ExistGeneralizations.Free;
  end;
end;

function TfrmModelSynchronizeDlg.AddGroup(RootNode: TTreeNode; GroupTitle: String;
  GroupItems: TStrings; ImageIndex: Integer): TTreeNode;
var
  I: Integer;
  GNode, CNode: TTreeNode;
begin
  if GroupTitle = ''
  then GNode := RootNode
  else
  if RootNode = nil
    then GNode := UpdateTree.Items.Add(nil, GroupTitle)
    else GNode := UpdateTree.Items.AddChild(RootNode, GroupTitle);
  for I := 0 to GroupItems.Count - 1 do begin
    CNode := UpdateTree.Items.AddChild(GNode, GroupItems[I]);
    CNode.ImageIndex := ImageIndex;
    CNode.SelectedIndex := ImageIndex;
  end;
  Result := GNode;
end;

{
*****************************   ********************************
}

procedure TfrmModelSynchronizeDlg.OkButtonClick(Sender: TObject);
begin
  SyncModel;
end;

procedure TfrmModelSynchronizeDlg.SyncModel;
var
  I: Integer;
begin
  Screen.Cursor := crHourGlass;
  try
    for I := 0 to UpdateTree.Items.Count - 1 do
      if UpdateTree.Items[I].Text = ' '
        then AddClasses(UpdateTree.Items[I]) //   
      else
      if UpdateTree.Items[I].Text = ' '
        then DeleteClasses(UpdateTree.Items[I]) //   
      else
      if UpdateTree.Items[I].Text = ' '
        then UpdateClasses(UpdateTree.Items[I]) //   
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmModelSynchronizeDlg.AddClasses(RootNode: TTreeNode);
var
  I: Integer;
begin
  //         ( )
  for I := 0 to RootNode.Count - 1 do
    if UpdateTree.ItemState[RootNode.Item[I].AbsoluteIndex] = csChecked
      then ModelSynchronizer.AddClass(RootNode.Item[I].Text);

  //    , ..     
  // ,    
  for I := 0 to RootNode.Count - 1 do
    if UpdateTree.ItemState[RootNode.Item[I].AbsoluteIndex] = csChecked
      then ModelSynchronizer.AddClassRelations(RootNode.Item[I].Text);
end;

procedure TfrmModelSynchronizeDlg.DeleteClasses(RootNode: TTreeNode);
var
  I: Integer;
begin
  for I := 0 to RootNode.Count - 1 do
    if UpdateTree.ItemState[RootNode.Item[I].AbsoluteIndex] = csChecked
      then ModelSynchronizer.DeleteClass(RootNode.Item[I].Text);
end;

procedure TfrmModelSynchronizeDlg.UpdateClasses(RootNode: TTreeNode);
var
  I, J, K: Integer;
  CurrClass: String;
begin
  for I := 0 to RootNode.Count - 1 do //   
  with RootNode.Item[I] do begin
    CurrClass := Text;
    for J := 0 to Count - 1 do //     
    with Item[J] do begin
      if Text = ' ' then
      begin
        for K := 0 to Count - 1 do
          if UpdateTree.ItemState[Item[K].AbsoluteIndex] = csChecked
            then ModelSynchronizer.AddAttribute(CurrClass, Item[K].Text)
      end else
      if Text = ' ' then
      begin
        for K := 0 to Count - 1 do
          if UpdateTree.ItemState[Item[K].AbsoluteIndex] = csChecked
            then ModelSynchronizer.DeleteAttribute(CurrClass, Item[K].Text)
      end else
      if Text = ' ' then
      begin
        for K := 0 to Count - 1 do
          if UpdateTree.ItemState[Item[K].AbsoluteIndex] = csChecked
            then ModelSynchronizer.UpdateAttribute(CurrClass, Item[K].Text)
      end else
      if Text = ' ' then
      begin
        for K := 0 to Count - 1 do
          if UpdateTree.ItemState[Item[K].AbsoluteIndex] = csChecked
            then if Item[K].ImageIndex = 4
                 then ModelSynchronizer.AddGeneralization(CurrClass, Item[K].Text)
                 else ModelSynchronizer.AddRelation(CurrClass, Item[K].Text)
      end else
      if Text = ' ' then
      begin
        for K := 0 to Count - 1 do
          if UpdateTree.ItemState[Item[K].AbsoluteIndex] = csChecked
            then if Item[K].ImageIndex = 4
                 then ModelSynchronizer.DeleteGeneralization(CurrClass, Item[K].Text)
                 else ModelSynchronizer.DeleteRelation(CurrClass, Item[K].Text)
      end else
      if Text = ' ' then
      begin
        for K := 0 to Count - 1 do
          if UpdateTree.ItemState[Item[K].AbsoluteIndex] = csChecked
            then ModelSynchronizer.UpdateRelation(CurrClass, Item[K].Text)
      end
    end
  end
end;


end.
