unit ovModelToolsDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ovModel, StdCtrls, Buttons, ComCtrls, ovModelEditorTypes;

type
  TfmModelToolsDialog = class(TForm)
    CancelBtn: TBitBtn;
    OkBtn: TBitBtn;
    PageControl: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    cbCheckClassIdUniqueness: TCheckBox;
    cbCheckRelationIdUniqueness: TCheckBox;
    cbCheckTableAndFieldNames: TCheckBox;
    cbCheckTableAndFieldNameUniqueness: TCheckBox;
  private
    { Private declarations }
    FModel: TDomainModel;
    FMessageLogger: IMessageLogger;
    procedure CheckRelationIdUniqueness;
    procedure CheckClassIdUniqueness;
    procedure CheckTableAndFieldNames;
    procedure CheckTableAndFieldNameUniqueness;
    procedure GenerateClassIdProc(ModelObject: TModelElement; Data: TObject);
    procedure GenerateRelationIdProc(ModelRelation: TModelElement; Data: TObject);
  protected
    property Model: TDomainModel read FModel;
    property MessageLogger: IMessageLogger read FMessageLogger;
  public
    { Public declarations }
    procedure VerifyModel;
    procedure GenerateModel;
  end;

function ShowGenerateModelDialog(AModel: TDomainModel; AMessageLogger:
  IMessageLogger): Boolean;
function ShowVerifyModelDialog(AModel: TDomainModel; AMessageLogger:
  IMessageLogger): Boolean;

function IsUniqueClassId(Model: TDomainModel; Id: Integer): Boolean;
procedure GenerateClassIds(Model: TDomainModel; NameSpace: String);
procedure GenerateRelationIds(Model: TDomainModel; NameSpace: String);
procedure GenerateFieldNames(Model: TDomainModel; Prefix: String);

procedure GenerateClassId(ModelObject: TModelObject);

implementation

{$R *.dfm}

function ShowVerifyModelDialog(AModel: TDomainModel; AMessageLogger:
  IMessageLogger): Boolean;
begin
  with TfmModelToolsDialog.Create(Application) do
  try
    FModel := AModel;
    FMessageLogger := AMessageLogger;
    PageControl.ActivePageIndex := 0;
    Result := ShowModal = mrOk;
    if Result then VerifyModel;
  finally
    Free
  end;
end;

function ShowGenerateModelDialog(AModel: TDomainModel; AMessageLogger:
  IMessageLogger): Boolean;
begin
  with TfmModelToolsDialog.Create(Application) do
  try
    FModel := AModel;
    FMessageLogger := AMessageLogger;
    PageControl.ActivePageIndex := 1;
    Result := ShowModal = mrOk;
    if Result then GenerateModel;
  finally
    Free
  end;
end;

{***************************************************************************** }
{            ,             }
{ **************************************************************************** }

function IsUniqueClassId(Model: TDomainModel; Id: Integer): Boolean;
var
  I, C: Integer;
begin
  C := 0;
  with Model do
  for I := 0 to ObjectCount - 1 do
    if Objects[I].Id = Id then Inc(C);
  Result := C <= 1;
end;

{{
  -       
.
}
function IsUniqueRelationId(AModelRelation: TModelRelation): Boolean;
var
  I: Integer;
  ARelation, ABackwardRelation: TModelRelation;
begin
  ABackwardRelation := AModelRelation.BackwardRelation;
  with AModelRelation.Model do
  for I := 0 to RelationCount - 1 do
  begin
    ARelation := Relations[I];
    if (ARelation <> AModelRelation) and
      {  ()          }
      (ARelation <> ABackwardRelation) and
      {     (.   CheckRelationIdUniqueness) }
      Not ARelation.IsInherited and
      (ARelation.Id = AModelRelation.Id)
    then begin
      Result := False;
      Exit;
    end
  end;
  Result := True;
end;

{{
   .
}
procedure GenerateClassId(ModelObject: TModelObject);
var
  Id, I: Integer;
begin
  Id := 0;
  with ModelObject.Model do
  for I := 0 to ObjectCount - 1 do
    if Objects[I].Id >= Id then Id := Objects[I].Id;
  ModelObject.Id := Id + 1;
end;

{{
   .
}
procedure GenerateRelationId(ModelRelation: TModelRelation);
var
  Id, I: Integer;
begin
  Id := 0;
  with ModelRelation.Model do
  for I := 0 to RelationCount - 1 do
    if Relations[I].Id >= Id then Id := Relations[I].Id;
  ModelRelation.Id := Id + 1;
end;

{{
        
(    -    ).
}
procedure GenerateClassIds(Model: TDomainModel; NameSpace: String);
var
  I: Integer;
begin
  with Model do
  for I := 0 to ObjectCount - 1 do
    if (Objects[I].NameSpace = NameSpace) and
       Not IsUniqueClassId(Model, Objects[I].Id)
    then GenerateClassId(Objects[I]);
end;

{{
        
  (    -
   ).
}
procedure GenerateRelationIds(Model: TDomainModel; NameSpace: String);
var
  I, J: Integer;
begin
  with Model do
  for I := 0 to ObjectCount - 1 do
    if Objects[I].NameSpace = NameSpace then
      with Objects[I] do
        for J := 0 to RelationCount - 1 do
          if Not IsUniqueRelationId(Relations[J])
            then GenerateRelationId(Relations[J]);
end;

{{
    PrefixNNN      .
}
procedure GenerateFieldNames(Model: TDomainModel; Prefix: String);
var
  I, J, C: Integer;

  function GetNewFieldName: String;
  begin
    Inc(C);
    Result := Prefix + IntToStr(C)
  end;

begin
  C := 0;
  with Model do
  for I := 0 to ObjectCount - 1 do
  with Objects[I] do
  begin
    //    
    for J := 0 to AttributeCount - 1 do
      with Attributes[J] do
        if Not IsInherited then FieldName := GetNewFieldName;

    //       
    for J := 0 to RelationCount - 1 do
      with Relations[J] do
        if Not IsInherited then FieldName := GetNewFieldName;

    //       
    for J := 0 to AncestorCount - 1 do
      AncestorRelations[J].FieldName := GetNewFieldName;
  end
end;

resourcestring

  SModelValidation = ' ';

{ TfmModelToolsDialog }

procedure TfmModelToolsDialog.VerifyModel;
begin
  if cbCheckClassIdUniqueness.Checked then CheckClassIdUniqueness;
  if cbCheckRelationIdUniqueness.Checked then CheckRelationIdUniqueness;
  if cbCheckTableAndFieldNames.Checked then CheckTableAndFieldNames;
  if cbCheckTableAndFieldNameUniqueness.Checked then CheckTableAndFieldNameUniqueness;
end;

procedure TfmModelToolsDialog.GenerateModel;
begin
  // TODO -cMM: TfmModelToolsDialog.GenerateModel default body inserted
end;

{***************************************************************************** }
{                                              }
{ **************************************************************************** }

procedure TfmModelToolsDialog.CheckClassIdUniqueness;
var
  I: Integer;
begin
  with Model do
  for I := 0 to ObjectCount - 1 do
    if Not IsUniqueClassId(Model, Objects[I].Id) then
      MessageLogger.AddMessage(SModelValidation,
        Format('    "%s"', [Objects[I].FullName]),
          Objects[I], GenerateClassIdProc);
end;

procedure TfmModelToolsDialog.GenerateClassIdProc(ModelObject: TModelElement;
  Data: TObject);
begin
  GenerateClassId(TModelObject(ModelObject))
end;

procedure TfmModelToolsDialog.CheckRelationIdUniqueness;
var
  I: Integer;
begin
  with Model do
  for I := 0 to RelationCount - 1 do
    if Not Relations[I].IsGeneralization and
       Not Relations[I].IsInherited and  //    (0.71)   
                                         //       ,    
       Not IsUniqueRelationId(Relations[I])
    then
      MessageLogger.AddMessage(SModelValidation,
        Format('     "%s"',
          [Relations[I].SourceObjectName + '.' + Relations[I].Name]),
            Relations[I], GenerateRelationIdProc);
end;

{{
   .
}
procedure TfmModelToolsDialog.GenerateRelationIdProc(ModelRelation:
  TModelElement; Data: TObject);
begin
  GenerateRelationId(TModelRelation(ModelRelation))
end;

procedure TfmModelToolsDialog.CheckTableAndFieldNames;
begin
  // TODO -cMM: TfmModelToolsDialog.CheckTableAndFieldNames default body inserted
end;

procedure TfmModelToolsDialog.CheckTableAndFieldNameUniqueness;
begin
  // TODO -cMM: TfmModelToolsDialog.CheckTableAndFieldNameUniqueness default body inserted
end;

end.
