unit ovModelViewFilter;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, CheckLst, ovModel, Buttons, ovModelTreeViewBuilder,
  Menus;

type
  TfmModelViewFilter = class(TForm)
    PageControl: TPageControl;
    tsNamespaceFilter: TTabSheet;
    tsObjectFilter: TTabSheet;
    NamespaceList: TCheckListBox;
    ObjectNameEdit: TEdit;
    GroupBox: TGroupBox;
    cbShowRelations: TCheckBox;
    cbShowAttributes: TCheckBox;
    cbShowDataTypes: TCheckBox;
    cbShowIndexes: TCheckBox;
    Label1: TLabel;
    CancelBtn: TBitBtn;
    OkBtn: TBitBtn;
    PopupMenu: TPopupMenu;
    SelectAllItem: TMenuItem;
    UnselectAllItem: TMenuItem;
    procedure SelectAllItemClick(Sender: TObject);
  private
    { Private declarations }
    FModel: TDomainModel;
    FOnFilterChange: TNotifyEvent;
    procedure SelectAllItems(SetChecked: Boolean);
    procedure SetModel(const Value: TDomainModel);
    function GetFilteredNamespaces: String;
    procedure SetFilteredNamespaces(const Value: String);
  protected
    property FilteredNamespaces: String read GetFilteredNamespaces write
      SetFilteredNamespaces;
  public
    { Public declarations }
    function IsNamespaceFiltered(ANamespace: String): Boolean;
    { Public declarations }
    function IsObjectFiltered(AModelObject: TModelObject): Boolean;
    function ShowDialog(var Options: TModelTreeViewBuilderOptions): Boolean;
    procedure SaveToRegistry(RegKey: String);
    procedure LoadFromRegistry(RegKey: String);
    procedure GetFilterOptions(var Options: TModelTreeViewBuilderOptions);
    property Model: TDomainModel read FModel write SetModel;
    property OnFilterChange: TNotifyEvent read FOnFilterChange write
      FOnFilterChange;
  end;

var
  fmModelViewFilter: TfmModelViewFilter;

implementation

uses Registry;

{$R *.dfm}

procedure TfmModelViewFilter.SetModel(const Value: TDomainModel);
begin
  if FModel <> Value then
  begin
    FModel := Value;
    if Assigned(FModel) then
    begin
      NamespaceList.Items.Assign(Model.NameSpaces);
      SelectAllItems(True);
    end else
      NamespaceList.Items.Clear;
  end;
end;

function TfmModelViewFilter.ShowDialog(var Options:
  TModelTreeViewBuilderOptions): Boolean;
begin
  cbShowDataTypes.Checked := tvbShowDataTypes in Options;
  cbShowIndexes.Checked := tvbShowIndexes in Options;
  cbShowAttributes.Checked := tvbShowAttributes in Options;
  cbShowRelations.Checked := tvbShowRelations in Options;

  Result := ShowModal = mrOk;

  if Result then GetFilterOptions(Options);
end;

procedure TfmModelViewFilter.GetFilterOptions(var Options:
  TModelTreeViewBuilderOptions);
begin
  if cbShowDataTypes.Checked then
    Options := Options + [tvbShowDataTypes]
  else
    Options := Options - [tvbShowDataTypes];
  if cbShowIndexes.Checked then
    Options := Options + [tvbShowIndexes]
  else
    Options := Options - [tvbShowIndexes];
  if cbShowAttributes.Checked then
    Options := Options + [tvbShowAttributes]
  else
    Options := Options - [tvbShowAttributes];
  if cbShowRelations.Checked then
    Options := Options + [tvbShowRelations]
  else
    Options := Options - [tvbShowRelations];
end;

{{
 True,    , ..  
.
}
function TfmModelViewFilter.IsNamespaceFiltered(ANamespace: String): Boolean;
var
  I: Integer;
begin
  if ANamespace <> '' then
  begin
    I := NamespaceList.Items.IndexOf(ANamespace);
    Result := Not ((I >= 0) and (NamespaceList.Checked[I]));
  end else
    Result := False
end;

{{
 True,    , ..   .
}
function TfmModelViewFilter.IsObjectFiltered(AModelObject: TModelObject):
  Boolean;
begin
  Result := (ObjectNameEdit.Text <> '') and (Pos(ObjectNameEdit.Text, AModelObject.Name) = 0)
end;

procedure TfmModelViewFilter.SelectAllItemClick(Sender: TObject);
begin
  SelectAllItems(Sender = SelectAllItem);
end;

procedure TfmModelViewFilter.SelectAllItems(SetChecked: Boolean);
var
  I: Integer;
begin
  for I := 0 to NamespaceList.Items.Count - 1 do
    NamespaceList.Checked[I] := SetChecked;
end;

{{
    
}
procedure TfmModelViewFilter.SaveToRegistry(RegKey: String);
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(RegKey, True) then
    begin
      Reg.WriteString('ObjectNameFilter', ObjectNameEdit.Text);
      Reg.WriteString('NamespaceFilter', FilteredNamespaces);
      Reg.WriteString('ShowDataTypesFilter', BoolToStr(cbShowDataTypes.Checked, True));
      Reg.WriteString('ShowIndexesFilter', BoolToStr(cbShowIndexes.Checked, True));
      Reg.WriteString('ShowAttributesFilter', BoolToStr(cbShowAttributes.Checked, True));
      Reg.WriteString('ShowRelationsFilter', BoolToStr(cbShowRelations.Checked, True));
    end;
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

{{
    
}
procedure TfmModelViewFilter.LoadFromRegistry(RegKey: String);
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    if Reg.OpenKey(RegKey, False) then
    begin
      ObjectNameEdit.Text := Reg.ReadString('ObjectNameFilter');
      FilteredNamespaces := Reg.ReadString('NamespaceFilter');
      cbShowDataTypes.Checked := StrToBoolDef(Reg.ReadString('ShowDataTypesFilter'), True);
      cbShowIndexes.Checked := StrToBoolDef(Reg.ReadString('ShowIndexesFilter'), True);
      cbShowAttributes.Checked := StrToBoolDef(Reg.ReadString('ShowAttributesFilter'), True);
      cbShowRelations.Checked := StrToBoolDef(Reg.ReadString('ShowRelationsFilter'), True);
    end;
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

{{
      
}
function TfmModelViewFilter.GetFilteredNamespaces: String;
var
  S: TStringList;
  I: Integer;
begin
  S := TStringList.Create;
  try
    for I := 0 to NamespaceList.Items.Count - 1 do
      if Not NamespaceList.Checked[I] then S.Add(NamespaceList.Items[I]);
    Result := S.Text;
  finally
    S.Free;
  end;
end;

procedure TfmModelViewFilter.SetFilteredNamespaces(const Value: String);
var
  S: TStringList;
  I, J: Integer;
begin
  S := TStringList.Create;
  try
    S.Text := Value;
    for I := 0 to S.Count - 1 do
    begin
      J := NamespaceList.Items.IndexOf(S[I]);
      if J >= 0 then NamespaceList.Checked[J] := False;
    end;
  finally
    S.Free;
  end;
end;

end.
