{$I Core\OView.inc}

unit ovReg;

interface

uses SysUtils, Classes, Forms, Controls, ovComponents,
  {$IFDEF DELPHI6_OR_HIGHER}
  DesignIntf, DesignEditors;
  {$ELSE} DsgnIntf; {$ENDIF}

type
  TModelEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

  TModelSynchronizerEditor = class(TComponentEditor)
  private
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

  TDataObjectsEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

  TModelNameProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TNameSpaceProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TDataObjectNameProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TObjectClassNameProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TObjectAttributesProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  TMasterRelationNameProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TObjectQueryEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

  TControllerClassNameProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TDataLinksEditor = class(TComponentEditor)
  private
    FExplorers: TStrings;
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
    constructor Create(AComponent: TComponent; ADesigner: IDesigner); override;
    destructor Destroy; override;
  end;

  TModelStorageEditor = class(TComponentEditor)
  private
    function ShowModelStorageEditor(AModelStorage: TovCustomModelStorage): Boolean;
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

  TModelEventEventProperty = class(TMethodProperty)
  public
    procedure Edit; override;
  end;

  TModelEventObjectClassNameProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TModelEventObjectAttributeProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TInspectorPropertiesClassProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

procedure Register;

implementation

{$R *.dcr}

uses Dialogs, ActnList, ovCheckTreeView,
     ovConsts, ovUtils, ovModel, ovAttributeSelectionDlg,
     ovModelSync, ovModelTreeViewBuilder,
     {ovVUMLAdapter, }ovEAAdapter,
     ovActions, ovModelStorage, ovDBConnections, ovDataSet, ovXMLSchema,  // 
     ovDataObjects, ovDataLinks, ovXMLImportExport, ovObjectInspector, // 
     ovDBControlLinks,   //      
     ovStdInspectorItems, //        
     ovModelEditor, ovModelEditorEx,
     ovDataEditor, ovDataObjectsEditor, ovDataLinksEditor, ovDataLinksExplorer,
     ovXMLModelStorage, ovDBBackup, ovModelEvents, ovInspector,
     ovDBSynchronizer, ovDBSynchronization, ovDataObjectTreeViewBuilder,
     ovMemObjectManager;

procedure Register;
begin
  //  
  RegisterComponents(cpObjectViewMain,
   [TovModel, TovModelStorage, TovDBModelStorage, TovModelSynchronizer,
    TovExposer, TovQuery, TovDataObjects, TovDataLinks, TovModelEvents,
    TovMemoryObjectManager,
    TovDatabaseConnections, TovModelDatabaseBackup,
    TovXMLImport, TovXMLExport, TovDBSynchronization, TovDBSynchronizer,
    TovObjectInspector]);
  RegisterComponents(cpObjectViewAddOn,
    [{TovVisualUMLAdapter, }TovEAAdapter]);

  RegisterComponents('ObjectView Misc',
  [TovDataObjectsXMLExport, TovXMLSchemaGenerator,
   TovModelTreeViewBuilder, TovDataObjectTreeViewBuilder,
    TovCheckTreeView, TovInspector]);

  RegisterActions('ObjectView', [TovDataObjectsAction, TovDataSetAction], nil);

  //   
  RegisterComponentEditor(TovModel, TModelEditor);
  RegisterComponentEditor(TovModelStorage, TModelStorageEditor);
  RegisterComponentEditor(TovModelSynchronizer, TModelSynchronizerEditor);
  RegisterComponentEditor(TovDataObjects, TDataObjectsEditor);
  RegisterComponentEditor(TovDataLinks, TDataLinksEditor);

  //    
  RegisterPropertyEditor(TypeInfo(String), TovDataSet, 'ModelName', TModelNameProperty);
  RegisterPropertyEditor(TypeInfo(String), TovDataSet, 'NameSpace', TNameSpaceProperty);
  RegisterPropertyEditor(TypeInfo(String), TovDataObjects, 'NameSpace', TNameSpaceProperty);
  RegisterPropertyEditor(TypeInfo(String), TovModelClient, 'ModelName', TModelNameProperty);
  RegisterPropertyEditor(TypeInfo(String), TovDataObjectsAction, 'DataObjectName', TDataObjectNameProperty);
  RegisterPropertyEditor(TypeInfo(String), TovDataSet, 'MasterRelationName', TMasterRelationNameProperty);
  RegisterPropertyEditor(TypeInfo(String), TovExposer, 'ObjectClassName', TObjectClassNameProperty);
  RegisterPropertyEditor(TypeInfo(String), TovExposer, 'ObjectAttributes', TObjectAttributesProperty);
  RegisterPropertyEditor(TypeInfo(String), TovDataLink, 'DataObject', TDataObjectNameProperty);
  RegisterPropertyEditor(TypeInfo(String), TovDataLinkController, 'LookupDataObject', TDataObjectNameProperty);
  RegisterPropertyEditor(TypeInfo(String), TovDataLink, 'ControllerClassName', TControllerClassNameProperty);

  RegisterPropertyEditor(TypeInfo(TCreateObjectEvent), TovModelEvent, 'OnCreateObject', TModelEventEventProperty);
  RegisterPropertyEditor(TypeInfo(TDeleteObjectEvent), TovModelEvent, 'OnDeleteObject', TModelEventEventProperty);
  RegisterPropertyEditor(TypeInfo(TValidateObjectEvent), TovModelEvent, 'OnValidateObject', TModelEventEventProperty);
  RegisterPropertyEditor(TypeInfo(TCalcAttributeEvent), TovModelEvent, 'OnCalcAttribute', TModelEventEventProperty);
  RegisterPropertyEditor(TypeInfo(String), TovModelEvent, 'ObjectClassName', TModelEventObjectClassNameProperty);
  RegisterPropertyEditor(TypeInfo(String), TovModelEvent, 'ObjectAttribute', TModelEventObjectAttributeProperty);

  RegisterPropertyEditor(TypeInfo(String), TInspectorItem, 'PropertiesClass', TInspectorPropertiesClassProperty);
end;

{ TModelEditor }

procedure TModelEditor.ExecuteVerb(Index: Integer);
begin
  inherited;
  case Index of
    0: try
         if ShowModelEditor(Component as TovModel) then Designer.Modified;
       finally
         (Component as TovModel).UnloadModel;
       end;

    1: ShowDataEditor(Component as TovModel);

    2: if MessageDlg('    ?', mtConfirmation, [mbYes,mbNo], 0) = mrYes then
       try
         (Component as TovModel).GenerateDatabase(nil, True);
         MessageDlg('    ', mtInformation, [mbOK], 0);
       except
         on E: Exception do
           MessageDlg('   : ' + E.Message, mtError, [mbOK], 0);
       end;

    3: with TSaveDialog.Create(Application) do
       try
         DefaultExt := '.ovmodel';
         Options := Options + [ofOverwritePrompt];
         if Execute then
         begin
           (Component as TovModel).LoadModel;
           SaveModelToFile(Component as TovModel, FileName, True, True);
         end;
       finally
         Free
       end;
  end
end;

function TModelEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := ' ..';
    1: Result := ' ..';
    2: Result := ' ';
    3: Result := '    ObjectView 2';
    else Result := inherited GetVerb(Index)
  end
end;

function TModelEditor.GetVerbCount: Integer;
begin
  Result := 4
end;

{ TModelSynchronizerEditor }

procedure TModelSynchronizerEditor.ExecuteVerb(Index: Integer);
begin
  inherited;
  case Index of
    0: if (Component as TovModelSynchronizer).ImportModel
         then Designer.Modified;
    1: if (Component as TovModelSynchronizer).ExportModel
         then Designer.Modified;
  end
end;

function TModelSynchronizerEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := '  (Source->Target) ...';
    1: Result := '  (Target->Source) ...';
    else Result := inherited GetVerb(Index)
  end
end;

function TModelSynchronizerEditor.GetVerbCount: Integer;
begin
  Result := 2
end;

{ TObjectsEditor }

procedure TDataObjectsEditor.ExecuteVerb(Index: Integer);
begin
  inherited;
  case Index of
    0: if ShowDataObjectsEditor(Component as TovDataObjects)
       then Designer.Modified
  end
end;

function TDataObjectsEditor.GetVerb(Index: Integer): String;
begin
  if Index = 0
    then Result := ' ..'
    else Result := inherited GetVerb(Index)
end;

function TDataObjectsEditor.GetVerbCount: Integer;
begin
  Result := 1
end;

{ TModelNameProperty }

function TModelNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TModelNameProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Values: TStringList;
begin
  Values := TStringList.Create;
  try
    GetRegisteredModels(Values);
    for I := 0 to Values.Count - 1 do Proc(Values[I]);
  finally
    Values.Free;
  end;
end;

{ TNameSpaceProperty }

function TNameSpaceProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TNameSpaceProperty.GetValues(Proc: TGetStrProc);
var
  Model: TovModel;
  C: TPersistent;
  I: Integer;
begin
  C := GetComponent(0);
  if C is TovDataSet then Model := TovDataSet(C).Model else
  if C is TovDataObjects then Model := TovDataObjects(C).Model else
    Model := nil;

  if Model <> nil then
    for I := 0 to Model.DomainModel.NameSpaces.Count - 1 do
      Proc(Model.DomainModel.NameSpaces[I]);
end;

{ TDataObjectNameProperty }

function TDataObjectNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TDataObjectNameProperty.GetValues(Proc: TGetStrProc);
var
  DataObjects: TovDataObjects;
  I: Integer;
begin
  if GetComponent(0) is TovDataLink then
    DataObjects := TovDataLink(GetComponent(0)).Owner.DataObjects
  else
  if GetComponent(0) is TovDataLinkController then
    DataObjects := TovDataLinkController(GetComponent(0)).Link.Owner.DataObjects
  else
  if GetComponent(0) is TovDataObjectsAction then
    DataObjects := TovDataObjectsAction(GetComponent(0)).DataObjects
  else
    raise Exception.Create('  ');

  if DataObjects <> nil then
    with DataObjects do
      for I := 0 to ObjectCount - 1 do Proc(Objects[I].Name);
end;

{ TObjectQueryEditor }

procedure TObjectQueryEditor.ExecuteVerb(Index: Integer);
begin
  inherited;             {
  case Index of
    0: if ShowQueryEditor(Component as TovObjectQuery)
       then Designer.Modified
  end                     }
end;

function TObjectQueryEditor.GetVerb(Index: Integer): String;
begin
  if Index = 0
    then Result := ' ..'
    else Result := inherited GetVerb(Index)
end;

function TObjectQueryEditor.GetVerbCount: Integer;
begin
  Result := 1
end;

{ TObjectClassNameProperty }

function TObjectClassNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TObjectClassNameProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
begin
  with GetComponent(0) as TovExposer do
    if Assigned(Model) then
      with Model.DomainModel do
        for I := 0 to ObjectCount - 1 do
          if (NameSpace <> '') and (Objects[I].NameSpace = NameSpace)
            then Proc(Objects[I].Name)
            else Proc(Objects[I].FullName);
end;

{ TMasterRelationNameProperty }

function TMasterRelationNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

{{
       .
}
procedure TMasterRelationNameProperty.GetValues(Proc: TGetStrProc);
var
  O: TModelObject;
  I: Integer;
begin
  with GetComponent(0) as TovDataSet do
    if Assigned(Model) and Assigned(MasterDataSet) and
       (MasterDataSet is TovExposer) then
      with Model.DomainModel do begin
        O := FindObject(TovExposer(MasterDataSet).ObjectClassName);
        if Assigned(O) then
          for I := 0 to O.RelationCount - 1 do Proc(O.Relations[I].Name);
      end
end;

{ TObjectAttributesProperty }

function TObjectAttributesProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paMultiSelect];
end;

procedure TObjectAttributesProperty.Edit;
var
  Selection: String;
begin
  with GetComponent(0) as TovExposer do
    if Assigned(Model) then
    begin
      Model.LoadModel;
      Selection := ObjectAttributes;
      if SelectObjectAttributes(Model,
        Model.DomainModel.GetObject(ObjectClassName), Selection, ' ',
          [soShowMultiRelations, soAllowSelectRelations, soExpandRelations])
      then ObjectAttributes := Selection
    end
end;

{ TControllerClassNameProperty }

function TControllerClassNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TControllerClassNameProperty.GetValues(Proc: TGetStrProc);
var
  Controllers: TStrings;
  I: Integer;
begin
  Controllers := TStringList.Create;
  try
    GetDataLinkControllers(Controllers);
    for I := 0 to Controllers.Count - 1 do
      Proc(Controllers[I]);
  finally
    Controllers.Free
  end;
end;

{ TDataLinksEditor }

constructor TDataLinksEditor.Create(AComponent: TComponent; ADesigner:
  IDesigner);
begin
  inherited Create(AComponent, ADesigner);
  FExplorers := TStringList.Create;
  GetDataLinkExplorers(FExplorers);
end;

destructor TDataLinksEditor.Destroy;
begin
  FExplorers.Free;
  inherited;
end;

procedure TDataLinksEditor.ExecuteVerb(Index: Integer);
begin
  inherited;
  ShowDataLinksEditor(Component as TovDataLinks,
    TfrmDataLinksExplorerClass(FExplorers.Objects[Index]), Designer);
end;

function TDataLinksEditor.GetVerb(Index: Integer): String;
begin
  //    ,       
  Result := FExplorers[Index]
end;

function TDataLinksEditor.GetVerbCount: Integer;
begin
  Result := FExplorers.Count
end;

{ TModelStorageEditor }

function TModelStorageEditor.GetVerbCount: Integer;
begin
  Result := 1
end;

function TModelStorageEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := ' ..';
    else Result := inherited GetVerb(Index)
  end
end;

procedure TModelStorageEditor.ExecuteVerb(Index: Integer);
begin
  inherited;
  case Index of
    0: if ShowModelStorageEditor(Component as TovCustomModelStorage)
         then Designer.Modified;
  end
end;

function TModelStorageEditor.ShowModelStorageEditor(AModelStorage:
  TovCustomModelStorage): Boolean;
var
  Model, SaveModel: TovModel;
begin
  Model := TovModel.Create(nil);
  try
    SaveModel := AModelStorage.Model;
    AModelStorage.Model := Model;
    try
      Result := ShowModelEditor(Model)
    finally
      AModelStorage.Model := SaveModel;
    end;
  finally
    Model.Free
  end;
end;

{***************************************************************************** }
{                       TovModelEvent                    }
{ **************************************************************************** }

{ TModelEventEventProperty }

procedure TModelEventEventProperty.Edit;
var
  FormMethodName: string;
begin
  FormMethodName := GetValue;
  if (FormMethodName = '') or
    Designer.MethodFromAncestor(GetMethodValue) then
  begin
    if FormMethodName = '' then
      FormMethodName := 'Event' + IntToStr((GetComponent(0) as TovModelEvent).Index) + GetTrimmedEventName;
    SetValue(FormMethodName);
  end;
  Designer.ShowMethod(FormMethodName);
end;

{ TModelEventObjectClassNameProperty }

function TModelEventObjectClassNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TModelEventObjectClassNameProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
begin
  with (GetComponent(0) as TovModelEvent).Owner do
    if Assigned(Model) then
    begin
//      ModelNeeded;
      Model.LoadModel;
      with Model.DomainModel do
        for I := 0 to ObjectCount - 1 do
          Proc(Objects[I].FullName);
    end;
end;

{ TModelEventObjectAttributeProperty }

function TModelEventObjectAttributeProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TModelEventObjectAttributeProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
begin
  with (GetComponent(0) as TovModelEvent), Owner do
    if Assigned(Model) and (ObjectClassName <> '') then
    begin
//      ModelNeeded;
      Model.LoadModel;
      with Model.DomainModel.GetObject(ObjectClassName) do
        for I := 0 to AttributeCount - 1 do
          Proc(Attributes[I].Name);
    end;
end;

{ TInspectorPropertiesClassProperty }

function TInspectorPropertiesClassProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TInspectorPropertiesClassProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  S: TStrings;
begin
  S := TStringList.Create;
  try
    GetRegisteredProperties(S);
    for I := 0 to S.Count - 1 do Proc(S[I]);
  finally
    S.Free;
  end;
end;

procedure TInspectorPropertiesClassProperty.SetValue(const Value: string);
begin
  inherited SetValue(Value);
  {     .  -  AV
    PropertiesClass   .   -
   - ,       Properties,
     PropertiesClass,    . }
  Designer.NoSelection;
end;

end.
